package com.intuit.payroll.samples;

import com.intuit.dataTypes.EnumTypeIf;
import com.intuit.dataTypes.QBDate;
import com.intuit.payroll.j.CompanyIf;
import com.intuit.payroll.j.EmployeePayrollItemIf;
import com.intuit.payroll.j.PayrollFactory;
import com.intuit.payroll.j.PayrollItemCallbackIf;
import com.intuit.payroll.j.PayrollItemIf;
import com.intuit.payroll.j.PayrollItemListCallbackIf;
import com.intuit.payroll.j.PayrollItemListInfoIf;
import com.intuit.payroll.j.UserTaxabilityIf;
import com.intuit.payroll.j.EffectiveDatedValueIf;
import com.intuit.payroll.j.PayrollDatumIf;
import com.intuit.payroll.j.EffectiveDatedDatumCollectionIf;
import com.intuit.payroll.j.DBPayrollDatumIf;
import com.intuit.payroll.types.PayrollItemDetailType;
import com.intuit.payroll.types.enumtype.DatumID;

import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

/**
 * Ugly sample code to illustrate basic requirements for integration with Payroll Engine.
 * Top level class represents some biz logic for the Payroll Engine client application.
 * Illustrates hourly paycheck. Salary is slightly more complicated.
 * Missing details on employee-specific tax setup info (assumes default values for now).
 */
public class Gemini {

    /**
     * Gemini payroll item IDs.  For convenience in the sample, these happen to match the Payroll Engine IDs.
     */
    public static enum PItemID {

        HOURLY (1000012),
        SALARY (1000013),
        VACATION_ACCRUE (1000018),
        VACATION_DISBURSE (1000019),
        VACATION_PAYOUT (1000024),
        FOUR01K (1000006),
        SEC125HEALTH (1000001),
        OTHER_REIMBURSEMENT (1000045),
        //...
        FIT (1),
        FICE (61),
        FICR (62),  // (twin)
        MEDE (63),
        MEDR (64),  // (twin)
        FUTA (66),
        CASIT (6),
        CASUI (87),
        CASDI (67),
        CAETT (142);

        private int id;
        private PItemID (int id) {
            this.id = id;
        }

        public int toInt () {
            return this.id;
        }
    }

    // Company context
    public static class Context {

        // Payroll Engine company context
        private CompanyIf company;

        // Gemini Cached list of payroll items
        private PItemList pitems;

        /**
         * Constructor
         * @param payrollStartDate Date this company started using Gemini payroll service.
         * No paychecks allowed before this date, only ytd txs.
         */
        public Context(QBDate payrollStartDate) {
            pitems = new PItemList();
            company = PayrollFactory.createCompany(payrollStartDate, null, pitems, pitems);
            pitems.init(company);
        }

        /**
         * Gemini creates a fast/simple payroll item for its cached payroll item list.
         * The full {@link PayrollItemIf} need not be loaded yet.
         * @param geminiID payroll item ID
         * @param name user-specified name for the item (may be null)
         * @param uniqueID company-specific payroll item unique ID from client DB
         */
        public void createPayrollItem (PItemID geminiID, String name, Integer uniqueID) {
            PItem pi = new PItem(company, name, Bridge.getDetailType(geminiID), uniqueID);
            pitems.add(pi);
        }

        /**
         * @return Payroll Engine company context / factory object
         */
        public CompanyIf getCompany () {
            return company;
        }

        /**
         * @param geminiID payroll item unique ID from client DB
         * @return real Payroll Engine object
         */
        public PayrollItemIf getPayrollItem (Integer geminiID) {
            return pitems.getPayrollItem(geminiID);
        }

        /**
         * Use only with singleton pay items.
         * @param type Payroll Engine item type (eg salary, hourly, FIT etc)
         * @return item of passed type (if any), null otherwise
         */
        public PayrollItemIf getPayrollItem (PayrollItemDetailType type) {
            return pitems.getPayrollItem(type);
        }

        /**
         * Use only with singleton pay items.  (Or if you know there is only one
         * item of this type in the test data)
         * @param type Gemini item type (eg salary, hourly, FIT etc)
         * @return item of passed type (if any), null otherwise
         */
        public PayrollItemIf getPayrollItem (PItemID type) {
            return pitems.getPayrollItem(Bridge.getDetailType(type));
        }

        /**
         * @return iterator over the client app's cached simple/fast pitem objects
         */
        public Iterator<PayrollItemListInfoIf> getPayrollItemIterator () {
            return pitems.getFullListIterator();
        }

        /**
         * @return company-specific taxability relationships between tax and non-tax items.
         * For example whether or not SEC125 deduction reduces taxable wages for each tax.
         */
        public UserTaxabilityIf getUserTaxability () {
            return pitems.getUserTaxability();
        }
    }

    // Conversions between Gemini object model and Payroll Engine object model
    private static class Bridge {

        /**
         * Map between Gemini pitem types and Payroll Engine pitem types
         * @param geminiPayrollItemTypeID eg ID for Federal Income Tax, Hourly pay etc
         * @return Payroll Engine pitem detail type
         */
        public static PayrollItemDetailType getDetailType (PItemID geminiPayrollItemTypeID) {
            // TODO your conversion here
            return PayrollItemDetailType.parseInt(Integer.valueOf(geminiPayrollItemTypeID.toInt()));
        }

        public static EnumTypeIf getFilingStatus (EmployeePayrollItemIf employeeTaxInfo, int geminiFilingStatusID) {
            // TODO your conversion here. For now we just take the first value
            return employeeTaxInfo.getDatum(DatumID.FILING_STATUS).getEnumParser().getAllValues()[0];
        }
    }

    /**
     * Gemini cached payroll item info, used by the Payroll Engine to grab company info as needed.
     */
    private static class PItemList implements PayrollItemCallbackIf, PayrollItemListCallbackIf {

        private Map<Integer, PayrollItemListInfoIf> idMap = new TreeMap<Integer, PayrollItemListInfoIf>();
        private Map<PayrollItemDetailType, PayrollItemListInfoIf> typeMap = new TreeMap<PayrollItemDetailType, PayrollItemListInfoIf>();

        // User-editable rules about which non-tax items affect taxable wages for which taxes
        private UserTaxabilityIf taxability;

        public void init (CompanyIf co) {
            taxability = co.createUserTaxability();
        }

        public void add (PItem pi) {
            idMap.put (pi.getUniqueID(), pi);
            typeMap.put (pi.getDetailType(), pi);
        }

        // Implements PayrollItemCallbackIf
        public PayrollItemIf getPayrollItem(PayrollItemDetailType type) {
            // Best results with singleton items
            return typeMap.get(type).getPayrollItem();
        }

        // Implements PayrollItemCallbackIf
        public PayrollItemIf getPayrollItem(Integer id) {
            return idMap.get(id).getPayrollItem();
        }

        // Implements PayrollItemCallbackIf
        public UserTaxabilityIf getUserTaxability() {
            return taxability;
        }

        // Implements PayrollItemListCallbackIf
        public Iterator<PayrollItemListInfoIf> getFullListIterator() {
            return idMap.values().iterator();
        }
    }

    /**
     * Gemini simple, fast client app cached pitem, implements lazy-creation of real
     * {@link PayrollItemIf} object.  Loading data from DB is not shown
     */
    private static class PItem implements PayrollItemListInfoIf {

        // Payroll Engine company context
        private CompanyIf co;

        // Payroll Engine static pitem type info
        private PayrollItemDetailType type;

        // Gemini unique pitem ID
        private Integer id;

        // User-specified name for the item
        private String name;

        // Not used, indicates if user has marked this item 'hidden'
        private boolean hidden = false;

        // Payroll Engine payroll item object
        private PayrollItemIf pi;

        public PItem(CompanyIf co, String name, PayrollItemDetailType type, Integer id) {
            this.co = co;
            this.type = type;
            this.id = id;
            this.name = name;
        }

        // Implements PayrollItemListInfoIf
        public String getName() {
            // Maybe the user's name is cached somewhere?
            // For simplicity here just use the PE default name
            return getPayrollItem().getName();
        }

        // Implements PayrollItemListInfoIf
        public Integer getUniqueID() {
            return id;
        }

        // Implements PayrollItemListInfoIf
        public PayrollItemDetailType getDetailType() {
            return type;
        }

        // Implements PayrollItemListInfoIf
        public boolean isHidden() {
            return hidden;
        }

        // Implements PayrollItemListInfoIf
        public synchronized PayrollItemIf getPayrollItem() {
            // maybe lazy creation for best performance
            if (pi == null) {
                pi = co.createPayrollItem(type);
                DBPayrollDatumIf nameDatum = pi.getDBDatum(DatumID.NAME);
                nameDatum.setFromDB(name);
                pi.finishInit(this.id, false);

                // Here would need to load other data from pitem table
            }
            return pi;
        }
    }

    /**
     * Container for effective-dated values
     */
    static public class EffectiveDatedValue implements EffectiveDatedValueIf {
        // e.g. historical tax rate
        public Object val;

        // date on which this value begins affecting paycheck calculations
        public QBDate beginEffectiveDate;

        /**
         * Constructor
         * @param val Dated value. Type should match {@link PayrollDatumIf#getValueType} for
         * the datum objects returned by {@link EffectiveDatedDatumCollectionIf#getDatumAsOf},
         * {@link EffectiveDatedDatumCollectionIf#getDatumList},
         * {@link EffectiveDatedDatumCollectionIf#getVisibleDatumList}
         * @param beginEffectiveDate Date on which this value starts affecting paycheck calculations.
         * <code>null</code> means the beginning of time (that is, this is the first dated-value.
         */
        public EffectiveDatedValue (Object val, QBDate beginEffectiveDate) {
            this.val = val;
            this.beginEffectiveDate = beginEffectiveDate;
        }

        // Implements EffectiveDatedValueIf
        public Object getValue() {
            return val;
        }

        // Implements EffectiveDatedValueIf
        public QBDate getBeginEffectiveDate() {
            return beginEffectiveDate;
        }
    }

    // In place of 'assert' just so we know for sure that it is on!
    protected static void myAssert (boolean condition)
    {
        if (! condition) throw new IllegalStateException();
    }
}
