package com.intuit.payroll.samples;

import com.intuit.dataTypes.QBDate;
import com.intuit.dataTypes.TimeDuration;
import com.intuit.dataTypes.Money;
import com.intuit.payroll.j.CompanyIf;
import com.intuit.payroll.j.EmployeeIf;
import com.intuit.payroll.j.EmployeePayrollItemIf;
import com.intuit.payroll.j.PaidTimeOffAccrualIf;
import com.intuit.payroll.j.PayrollFactory;
import com.intuit.payroll.j.PayrollItemIf;
import com.intuit.payroll.j.PayrollDatumIf;
import com.intuit.payroll.types.PayrollItemDetailType;
import com.intuit.payroll.types.enumtype.DatumID;
import com.intuit.payroll.types.enumtype.PaidTimeOffAccrualType;
import com.intuit.payroll.types.enumtype.TimeOffType;

/**
 * Paid Time Off
 * Two types: Sick, Vacation.  Other than their names they behave exactly the same.
 * Accruals and disbursements are tracked as PTO transactions {@link PaidTimeOffAccrualIf}.
 * Disbursements are each associated with a paycheck line.  Paycheck accruals are also associated
 * with a paycheck line (eg "per hour", "per paycheck", "per year prorated").
 * Annual accruals are not associated with paychecks (eg "per year on hire date", "per year on Jan 1").
 * <p>
 * Some tricky things about annual accruals and employee {@link DatumID#PAYROLL_START_DATE}:
 * eg no hire date accrual in employee setup year. if hire date anniversary is before employee
 * setup date.
 */
public class GeminiPaidTimeOff {

    /**
     * Load employee PTO setup info from DB.  
     * @param epi Payroll Engine object for employee-specific payroll item info
     * @param salariedEmp true if the employee is salaried
     */
    protected static void loadEmployeePayrollItemInfo(EmployeePayrollItemIf epi, boolean salariedEmp)
    {
        Integer id = epi.getPayrollItem().getUniqueID();
        if (id.equals(GeminiTest.ID_MAP.get(Gemini.PItemID.VACATION_ACCRUE))) {
            epi.setDatumValue(DatumID.HOURS, new TimeDuration("80:00"));
            epi.setDatumValue(DatumID.ACCRUAL_LIMIT, new TimeDuration("80:00"));
            epi.setDatumValue(DatumID.PTO_ACCRUAL_TYPE, PaidTimeOffAccrualType.PER_YEAR_PRORATED);

            // Other per-paycheck options are:
            // hours per hour
            // hours per paycheck
        } else if (id.equals(GeminiTest.ID_MAP.get(Gemini.PItemID.VACATION_DISBURSE))) {
            PayrollDatumIf rate = epi.getDatum(DatumID.RATE);
            if (salariedEmp) {
                // For salaried employee/paychecks the PTO disburse rate comes from the salary
                Gemini.myAssert (! rate.isEditable());
                Gemini.myAssert (! rate.isVisible());
            } else {
                // For non-salary paychecks, we need the user to specify a rate
                Gemini.myAssert (rate.isEditable());
                Gemini.myAssert (rate.isVisible());
                rate.setValue(new Money(50));
            }
        }
    }

    /**
     * Create some of the non-paycheck PTO transactions: "user adjustment" and "per year on hire date".
     * Does not create "per year on Jan 1" but that is very similar to the hire date case.
     * @param context company context
     * @param accrualEpi employee's PTO accrual info
     */
    private static void createPaidTimeOffTxs (Gemini.Context context, EmployeePayrollItemIf accrualEpi)
    {
        EmployeeIf emp = accrualEpi.getEmployee();

        // Create manual user adjustment
        PaidTimeOffAccrualIf userAdjustmentAccrual = accrualEpi.createPaidTimeOffAccrualAdjustment ();
        userAdjustmentAccrual.setDatumValue (DatumID.HOURS, new TimeDuration("2:00"));
        userAdjustmentAccrual.setDataBeforeValidate();
        emp.getTotalsCollection().update (userAdjustmentAccrual);
        printPto("User Adjustment:", accrualEpi, userAdjustmentAccrual);

        // Set date of hire PTO hours awarded higher than annual limit to see clipping
        context.getCompany().setSystemDate(new QBDate (12,1,2007));
        accrualEpi.setDatumValue(DatumID.PTO_ACCRUAL_TYPE, PaidTimeOffAccrualType.PER_YEAR_ON_HIRE_DATE);
//        accrualEpi.setDatumValue(DatumID.HOURS, new TimeDuration("96:00"));
        PaidTimeOffAccrualIf hireDateAccrual = PayrollFactory.getAnnualPaidTimeOffAccrual(null, accrualEpi);
        hireDateAccrual.setDataBeforeValidate();
        emp.getTotalsCollection().update (hireDateAccrual);
        printPto("Date of hire award (should hit limit):", accrualEpi, hireDateAccrual);

        // TODO Can also try with PaidTimeOffAccrualType.PER_YEAR_ON_JAN_1
    }

    private static void printPto (String title, EmployeePayrollItemIf accrualEpi, PaidTimeOffAccrualIf ptoTx)
    {
        EmployeeIf e = accrualEpi.getEmployee();
        System.out.println();
        System.out.println(title);
        System.out.println(ptoTx);
        System.out.println(e.getTotalsCollection().getTotals(accrualEpi.getPayrollItem()));
        System.out.println();
        System.out.println(accrualEpi.getDatumValue(DatumID.PTO_AVAILABLE));
        System.out.println();
    }

    // Illustrates pto accrual by user-creation of manual adj and
    // hire-date based annual accrual
    // (No paychecks involved)
    private static void accruePto ()
    {
        // First day paychecks are allowed in the system (for this company)
        QBDate payrollStartDate = new QBDate (5,1,2007);

        // Create app's context for this company
        Gemini.Context context = new Gemini.Context(payrollStartDate);

        GeminiTest.loadCompanyInfo(context);

        CompanyIf co = context.getCompany();
        EmployeeIf e = co.createEmployee();
        GeminiTest.loadEmployeeInfo(e, context);

        PayrollItemDetailType vacAccrueType = PayrollItemDetailType.getTimeOffAccrue(TimeOffType.VACATION);

        PayrollItemIf vacAccrueItem = context.getPayrollItem(vacAccrueType);

        // Create the employee payroll item object
        EmployeePayrollItemIf epi = e.createEmployeePayrollItem(vacAccrueItem.getPayrollItem());
        loadEmployeePayrollItemInfo(epi, true);

        createPaidTimeOffTxs(context, epi);
    }

    static public void main (String[] args)
    {
        // Illustrates user-created manual adjustment and hire-date based annual accrual
        accruePto();
    }
}
