package com.intuit.sbfinance.json;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.lang.annotation.Annotation;
import java.lang.reflect.Type;

import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.core.UriInfo;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.codehaus.jackson.jaxrs.JacksonJsonProvider;
import org.codehaus.jackson.map.AnnotationIntrospector;
import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.map.SerializationConfig;
import org.codehaus.jackson.map.introspect.JacksonAnnotationIntrospector;

public class SbFinanceJacksonJsonProvider extends JacksonJsonProvider {
    public static final int VERSION = 1;
    public static final int COMPANY = 2;
    public static final int COMPANY_ID = 3;
    public static final int ENTITY_NAME = 4;

	private static final Log LOG = LogFactory
			.getLog(SbFinanceJacksonJsonProvider.class);
    
    @Context
    private UriInfo uriInfo;

    UriInfo getUriInfo() {
        return uriInfo;
    }

    void setUriInfo(UriInfo uriInfo) {
        this.uriInfo = uriInfo;
    }
    
    
    @Override
    public boolean isWriteable(Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType) {
        return inV3Service(type) && super.isWriteable(type, genericType, annotations, mediaType);
    }
    
    @Override
    public boolean isReadable(Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType) {
        return inV3Service(type) && super.isReadable(type, genericType, annotations, mediaType);
    }

    private boolean inV3Service(Class<?> type) {
        return type != null && type.getPackage() != null && type.getPackage().getName() != null
                && type.getPackage().getName().startsWith(JsonUtils.SERVICE_V3_PACKAGE_NAME);
    }


    
    public static void setupObjectMapperSerializationConfig(ObjectMapper mapper) {
        SerializationConfig config = mapper.getSerializationConfig();
        // serialize using JAXB annotations with custom filters
        config = config.withAnnotationIntrospector(new CustomJaxbAnnotationIntrospector());
        // suppress null elements
        config = config.withSerializationInclusion(org.codehaus.jackson.map.annotate.JsonSerialize.Inclusion.NON_NULL);
        config = config.withFilters(new CustomFilterProvider());
        mapper.setSerializationConfig(config);
    }

    public static void setupObjectMapperDeserializationConfig(ObjectMapper mapper) {
        AnnotationIntrospector primary = new CustomJaxbAnnotationIntrospector();
        AnnotationIntrospector secondary = new JacksonAnnotationIntrospector();
        AnnotationIntrospector pair = new AnnotationIntrospector.Pair(primary, secondary);
        // make deserializer use JAXB and Jackson annotations
        mapper.setDeserializationConfig(mapper.getDeserializationConfig().withAnnotationIntrospector(pair));
        
        mapper.getDeserializationConfig().addHandler(new CustomDeserializationProblemHandler());
    }

    @Override    
    public void writeTo(Object value, Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType,
                        MultivaluedMap<String, Object> httpHeaders, OutputStream entityStream) throws IOException, WebApplicationException {

        // get the Object Mapper
        ObjectMapper mapper = locateMapper(type, mediaType);
        setupObjectMapperSerializationConfig(mapper);
        super.writeTo(value, type, genericType, annotations, mediaType, httpHeaders, entityStream);
    }

    private final static String ATTACHABLE_UPLOAD = "upload";
    
    
    @Override
    public Object readFrom(Class<Object> type, Type genericType, Annotation[] annotations, MediaType mediaType, MultivaluedMap<String, String> httpHeaders,
                           InputStream entityStream) throws IOException {
    	
    	  //Convert Stream to String for logging the request body
    	String s = printInputStream(entityStream);
    	if(s==null || s.equals("")) return null;
    	  	           
    	 //Set the stream back from the String
    	 entityStream = new ByteArrayInputStream(s.getBytes());
    	 
    	 LOG.info("requestBody="+ s );

        // get the Object Mapper
        ObjectMapper mapper = locateMapper(type, mediaType);
        setupObjectMapperDeserializationConfig(mapper);
        
        // Attachable Upload needs special handling on the generic type since it comes in as null because it's part of multipart content
        if (uriInfo != null) {
            
        	String[] pathParams = uriInfo.getPath().split("/");
			//If it is a company request, path is going to have fewer than 5. We need to bypass that.
			if(pathParams.length>=5)
			{		
        	
				String entityName = pathParams[ENTITY_NAME];
				if (entityName.equals(ATTACHABLE_UPLOAD)) {
					// let generate type be the same as the object type
					genericType = type;
				}
			}
        }
        return super.readFrom(type, genericType, annotations, mediaType, httpHeaders, entityStream);
    }
    
   
    
    public static String printInputStream(InputStream is) throws IOException
    {
    	if(is!=null)
    	{
	    	StringBuilder sb = new StringBuilder();
	    	BufferedReader in = new BufferedReader(new InputStreamReader(is));
	    	String inputLine;
	    	
			while ((inputLine = in.readLine()) != null)
			 sb.append(inputLine);
		
			in.close();
			
	    	return sb.toString();
    	}
    	
    	return null;
    }

}