package com.intuit.sb.payroll.ca.workerpayment;

import java.util.Date;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.ca.client.dto.PayrollScheduleFrequency;
import com.intuit.sb.payroll.capability.workerpayments.CalculationEngineException;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.Worker;


public interface CanadaWorkerPaymentService 
{	
	public Money getMaxCPPLimit(Date date);
	
	public Money getMaxEILimit(Date date);
	
	/**
     * Calculates the deductions, net & gross pay for a single paycheck from the supplied paycheck input data. The
     * calculation is completely stateless.
     * @param worker The worker, including the following data:
     * <ul> 
     * <li>All wages (CompensationDtl) and taxes (TaxDtl) applicable to this worker
     * (AdditionDtl, BenefitDtl and PaidTimeOffDtl are currently unsupported)</li>
     * <ul> 
     *  <li>All PayrollItemDtl objects must have a valid law id</li>
     *  <li>Year To Date is currently unsupported</li>
     *  <li>All wages must be either Rate or Amount based MONEY types {@link Datum} </li>
     *  <li>All taxes are either:</li>
     *  <ul> 
     *   <li>employer paid and must have EmployerComplianceDocDtl objects with the tax rate.
     *   Only Unemployment and Disability insurance are supported.</li>
     *   <li>worker paid and must have WorkerComplianceDocDtl objects with the appropriate
     *   withholdings. W9 is currently unsupported.</li>
     *   <li>tax table driven so have no compliance data</li>
     *  </ul> 
     * </ul> 
     * <li>payroll frequency</li>
     * <li>The region/state for the worker's Work and Live locations</li>
     * </ul>
     *  
     * @param employer The employer (company), including the following data:
     * <ul> 
     * <li>the date of the first payroll run for this company in our payroll system</li>
     * <li>its tax compliance data</li>
     * </ul>
     *  
     * @param paycheck The paycheck that's being calculated. This is a mutable object that provides the gross wages
     * as inputs to the calculation and is augmented with the calculated deductions. The following data is required
     * as input:
     * <ul> 
     * <li>The pay period start and end dates</li>
     * </ul>
     *  
     * @return The completed paycheck. This is the same object as the {@code paycheck} input parameter. The following
     * data is added to the paycheck as output:
     * <ul> 
     * <li>all line items that appear on the check</li>
     * <li>PTD is unsupported and not returned at this time</li>
     * </ul>
     * @throws PayrollValidationException 
     *  
     * @throws CalculationEngineException Any internal failures that prevent calculation of the paycheck.
     * calculation of the paycheck.
     */
    public Paycheck calculatePaycheck(Worker worker, Employer employer, Paycheck paycheck, PayrollScheduleFrequency frequency) throws ValidationException, ServiceFailureException;

}
