package com.intuit.sb.payroll.capability.ca.common;

import java.io.Serializable;

import org.apache.commons.lang.builder.HashCodeBuilder;

/**
 * Base class for enums.
 * <p>
 * Enums are currently not Java 5 enums because LCDS 2.5 does not support them.
 * They can be changed to Java 5 enums either when we move to Entity Interfaces 
 * or when we move to LCDS 2.6 (supposed to be out in Dec. 2007).
 * </p>
 */
@SuppressWarnings("unchecked")
public abstract class EnumBase implements Comparable, Serializable {
	private static final long serialVersionUID = -8984280375217776259L;

	private String name;
	private int ordinal;
	
	/**
     * Sole constructor.
     * @param name - The name of this enum constant, which is the identifier
     *               used to declare it.
     * @param ordinal - The ordinal of this enumeration constant (its position
     *         in the enum declaration, where the initial constant must be assigned
     *         an ordinal of zero and every subsequent enum has an ordinal of
     *         one plus the ordinal of the prior enum constant).
     */
	public EnumBase(String name, int ordinal)
	{
		this.name = name;
		this.ordinal = ordinal;
	}

	/** 
	 * Public constructor is required by LCDS.  Without it, you get an exception
	 * with the following message: "Types must have a public, no arguments constructor".
	 * This should never be used by hand-written code.
	 * @deprecated
	 */
	public EnumBase()
	{
	}

	/**
	 * Getter should not be used by hand-written code.
	 * @deprecated
	 */
	public String getName()
	{
		return name;
	}
	
	/**
	 * Setter should not be used by hand-written code.
	 * @param name the string name of this enum.
	 * @deprecated
	 */
	public void setName( String name )
	{
		this.name = name;
	}
	
	/**
	 * Getter should not be used by hand-written code.
	 * @return the ordinal
	 * @deprecated
	 */
	public int getOrdinal() {
		return ordinal;
	}

	/**
	 * Setter should not be used by hand-written code.
	 * @param ordinal the ordinal to set.
	 * @deprecated
	 */
	public void setOrdinal(int ordinal) {
		this.ordinal = ordinal;
	}

	/**
	 * Returns the name of this enum constant, exactly as declared in its enum declaration.
	 * <b>Most programmers should use the toString() method in preference to this one, as the toString
	 * method may return a more user-friendly name.</b> This method is designed primarily for use in specialized
	 * situations where correctness depends on getting the exact name, which will not vary from release to release.
	 * @return the name of this enum constant.
	 */
	public String name()
	{
		return name;
	}
	
	/**
	 * Returns the ordinal of this enumeration constant (its position in its enum declaration).
	 * @return the ordinal of this enumeration constant.
	 */
	public int ordinal()
	{
		return ordinal;
	}

    /**
     * Throws CloneNotSupportedException.  This guarantees that enums
     * are never cloned, which is necessary to preserve their "singleton"
     * status.
     *
     * @return (never returns)
     */
    protected final Object clone() throws CloneNotSupportedException
    {
    	throw new CloneNotSupportedException();
    }

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object other)
	{
		if (other == null) return false;
		if (!(other instanceof EnumBase)) return false;
		EnumBase that = (EnumBase)other;
		if (this.getDeclaringClass() != that.getDeclaringClass()) return false;
		if (this.name == null) return (that.name == null);
		return this.ordinal == that.ordinal && this.name.equals(that.name);
	}

	@Override
	public int hashCode()
	{
		return HashCodeBuilder.reflectionHashCode(this);
	}
	
    /**
     * Compares this enum with the specified object for order.  Returns a
     * negative integer, zero, or a positive integer as this object is less
     * than, equal to, or greater than the specified object.
     * 
     * Enum constants are only comparable to other enum constants of the
     * same enum type.  The natural order implemented by this
     * method is the order in which the constants are declared.
     */
	public int compareTo(Object other)
	{
		EnumBase that = (EnumBase)other;
		if (this.getClass() != that.getClass() && // optimization
				this.getDeclaringClass() != that.getDeclaringClass())
		    throw new ClassCastException();
		return this.ordinal - that.ordinal;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString()
	{
		return name();
	}

	/**
     * Returns the Class object corresponding to this enum constant's
     * enum type.  Two enum constants e1 and  e2 are of the
     * same enum type if and only if
     *   e1.getDeclaringClass() == e2.getDeclaringClass().
     * (The value returned by this method may differ from the one returned
     * by the {@link Object#getClass} method for enum constants with
     * constant-specific class bodies.)
     *
     * @return the Class object corresponding to this enum constant's
     *     enum type
     */
    
	public final Class getDeclaringClass() {
    	Class clazz = getClass();
    	Class zuper = clazz.getSuperclass();
    	return (zuper == EnumBase.class) ? clazz : zuper;
    }
}
