package com.intuit.sb.payroll.capability.ca.workerpayment;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Date;

import junit.framework.Assert;

import org.junit.Test;

import com.intuit.sb.payroll.ca.workerpayment.CanadaLawID;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckCalculator;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QBOEPaycheckCalculatorBean;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * This class tests that YTD amounts work properly...
 * 
 * @author gcoulombe
 */
public class YTDAmountTest extends BasePaycheckTest {
	
	@Test
	public void test_OntarioSalaryWithLowYTD() throws Exception {
		BigDecimal amount = new BigDecimal("3269.23");
		
		BigDecimal cppYTD = new BigDecimal("100.00");
		BigDecimal eiYTD = new BigDecimal("55.00");
		
		BigDecimal cppAmt = new BigDecimal("155.16");
		BigDecimal eiAmt = new BigDecimal("56.56");
		
		LineItemAmount lineItems[] = {
				new LineItemAmount(CanadaLawID.Regular_Salary.getLawId(), amount, new BigDecimal("0.00"), new BigDecimal("0.00")),//Hourly
				new LineItemAmount(CanadaLawID.CPP_EE.getLawId(), cppAmt, cppAmt.add(cppYTD), amount, amount),//CPP Employee
				new LineItemAmount(CanadaLawID.EI_EE.getLawId(), eiAmt, eiAmt.add(eiYTD), amount, amount),//EI Employee
				new LineItemAmount(CanadaLawID.FED_TAX.getLawId(), new BigDecimal("545.22"), amount, amount),//Fed Tax
				new LineItemAmount(CanadaLawID.PROV_TAX_ON.getLawId(), new BigDecimal("290.88"), amount, amount)//ON Tax
		};
		
		doTest(DateUtil.createDate(1975, Calendar.JULY, 19), "ON", new BigDecimal("85000.00"), "4%", PayrollFrequencyEnum.BIWEEKLY, 
				DateUtil.createDate(2009, Calendar.JANUARY, 5), CanadaLawID.PROV_TAX_ON, new BigDecimal("10100.00"), 
				new BigDecimal("8881.00"), lineItems, cppYTD, eiYTD); 
	}
	
	@Test
	public void test_OntarioSalaryWithYTDAtLimits() throws Exception {
		BigDecimal amount = new BigDecimal("3269.23");
		
		BigDecimal cppYTD = new BigDecimal("2118.60");
		BigDecimal eiYTD = new BigDecimal("731.79");
		
		BigDecimal cppAmt = new BigDecimal("0.00");
		BigDecimal eiAmt = new BigDecimal("0.00");
		
		// Note about the zeros in this test (on the EI line): since EI was maxed out in the YTD values, the 
		// wage base *for this paycheck* will be zero. 
		
		LineItemAmount lineItems[] = {
				new LineItemAmount(CanadaLawID.Regular_Salary.getLawId(), amount, new BigDecimal("0.00"), new BigDecimal("0.00")),//Hourly
				new LineItemAmount(CanadaLawID.CPP_EE.getLawId(), cppAmt, cppAmt.add(cppYTD), amount, amount),//CPP Employee
				new LineItemAmount(CanadaLawID.EI_EE.getLawId(), eiAmt, eiAmt.add(eiYTD), amount, new BigDecimal("0.00")),//EI Employee
				new LineItemAmount(CanadaLawID.FED_TAX.getLawId(), new BigDecimal("561.67"), amount, amount),//Fed Tax
				new LineItemAmount(CanadaLawID.PROV_TAX_ON.getLawId(), new BigDecimal("301.23"), amount, amount)//ON Tax
		};
		
		doTest(DateUtil.createDate(1975, Calendar.JULY, 19), "ON", new BigDecimal("85000.00"), "4%", PayrollFrequencyEnum.BIWEEKLY, 
				DateUtil.createDate(2009, Calendar.JANUARY, 5), CanadaLawID.PROV_TAX_ON, new BigDecimal("10100.00"), 
				new BigDecimal("8881.00"), lineItems, cppYTD, eiYTD); 
	}
	
	private void doTest(Date birthDate, String province, BigDecimal salary, String vacationRate,
			PayrollFrequencyEnum payFrequency, Date payDate, 
			CanadaLawID provincialTaxLaw, BigDecimal federalTD1Claim, 
			BigDecimal provincialTD1Claim, LineItemAmount[] lineItems, BigDecimal cppYTD, BigDecimal eiYTD) {
		
		Worker worker = createEmployee(birthDate, provincialTaxLaw, province, payDate, cppYTD, eiYTD);
		
		setupSalaryCompensationDetailForWorker(worker, salary);
		
		Employer employer = new Employer();	
	    employer.setFirstPayrollDate(birthDate);
	    
	    int daysPerPeriod = frequencyToDaysPerPeriod.get(payFrequency);
		
	    Date payPeriodStartDate = DateUtil.getNextDate(payDate, -1 * daysPerPeriod);
	    
	    DateRange payPeriod = new DateRange(payPeriodStartDate, payDate);
	    
	    Paycheck paycheck = createPaycheck(payDate, worker, payPeriod,
				provincialTaxLaw, federalTD1Claim, provincialTD1Claim);
        
        PaycheckCalculator paycheckCalculator = new QBOEPaycheckCalculatorBean();
        
	    try {
			Paycheck calculated = paycheckCalculator.calculatePaycheck(worker, employer, paycheck, payFrequency);
			
			// verify paycheck line items
			Assert.assertTrue("Expected lineItem value mismatch!", 
					PaycheckUtils.verifyLineItems(calculated.getPaycheckLineItems(),
                    lineItems, "paycheck line items"));
		} catch (Exception e) {
			e.printStackTrace();
			Assert.fail(e.getMessage());			
		}
	}
}
