package com.intuit.sb.payroll.common.ca;

import java.text.MessageFormat;

import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

import com.intuit.sb.payroll.InfrastructureException;
import com.intuit.sb.payroll.common.ExceptionUtils;

/**
 * Locates objects in the directory. This is an implementation of the standard JEE service locator pattern.
 * 
 * This implementation exlicitly does not cache, we assume the container's JNDI does not suck. As well, it can be
 * dangerous -see {@linkplain http://www.ibm.com/developerworks/websphere/techjournal/0410_woolf/0410_woolf.html}
 * 
 * $Author: thobson $ 
 * $Revision: #3 $ 
 * $DateTime: 2009/01/07 22:53:33 $
 */
public class ServiceLocator {
	static private Logger logger = Logger.getLogger(ServiceLocator.class);

	static private InitialContext initialContext;
	static private final ServiceLocator serviceLocator;

	// FIXME [Impl] This should come from config
	@SuppressWarnings("nls")
	private static final String SERVICE_NAME = "PayrollService-CA";

	/*
	 * Class initialization provides a safe environment for singleton creation.
	 */
	static {
		serviceLocator = new ServiceLocator();
	}


	/*
	 * Private construction for singleton
	 */
	private ServiceLocator() {
		synchronized (this) {
			try {
				// Get the default JNDI context as configured in jndi.properties.
				// If a remote context is required it should be configured in jndi.properties.
				initialContext = new InitialContext();
			} catch (NamingException ne) {
				String mesg = MessageFormat.format("Unable to access JNDI: {0}", new Object[] { ExceptionUtils
						.throwableMessageChain(ne) });
				logger.fatal(mesg);
				throw new InfrastructureException(mesg);
			}
		}
	}

	/**
	 * Gets the singleton {@code ServiceLocator}.
	 * 
	 * @return the service locator.
	 */
	static public ServiceLocator getInstance() {
		return serviceLocator;
	}

	/**
	 * The Contract with this method is that you pass in the Name of the service as well as the scope (/local or
	 * /remote). Bean interfaces in Payroll have a constant that can be used for such purposes.
	 * 
	 * @param <T>
	 * @param serviceName the service name, including the scope (local, remote).
	 * @return a reference to the Service, if found.
	 */
	@SuppressWarnings("unchecked")
	public <T> T getService(String serviceName) {
		try {
			String lookupString = SERVICE_NAME + "/" + serviceName;

			Object o = initialContext.lookup(lookupString);
			T service = (T) o;
			return service;
		} catch (NamingException ne) {
			String mesg = MessageFormat.format("Unable to retrieve service ''{0}'' from JNDI: {1}", new Object[] {
					serviceName, ExceptionUtils.throwableMessageChain(ne) });
			logger.error(mesg);
			throw new InfrastructureException(mesg);
		}
	}


}

