package com.intuit.sb.payroll.capability.workerpayments;

import java.util.Date;
import java.util.List;

import javax.ejb.Local;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * All things PayrollSchedule happen here.  This service exposes functions to manage the payroll schedules for a company,
 * as well as provide date calculations around payroll schedules.
 *
 * $Author$ 
 * $Revision$
 * $DateTime$
 */
@Local
public interface PayrollScheduleComponent {

	@SuppressWarnings("nls")
	public static final String JNDI_NAME =  "PayrollScheduleComponent/local";

	/**
	 * Inserts a new {@link PayrollSchedule} and associates it with the current Employer.
	 * 
	 * @param payrollScheduleToCreate
	 * @return the new copy of the {@link PayrollSchedule}
	 * @throws ValidationException
	 * @throws ServiceFailureException
	 */
	public PayrollSchedule createPayrollSchedule(PayrollSchedule payrollScheduleToCreate) throws ValidationException, ServiceFailureException;

	/**
	 * Saves or updates the payroll schedule
	 * 
	 * @param dePayrollSchedule
	 * @return PayrollSchedule
	 * @throws PayrollScheduleValidationException
	 */
	public PayrollSchedule updatePayrollSchedule(PayrollSchedule payrollScheduleToUpdate) throws ValidationException, ServiceFailureException;
	
	/**
	 * Deletes a {@link PayrollSchedule}.  May not be permitted if there are Workers associated with that Schedule.
	 * 
	 * @param payrollScheduleToDelete
	 * @throws ValidationException
	 * @throws ServiceFailureException
	 */
	public void deletePayrollSchedule(PayrollSchedule payrollScheduleToDelete) throws ValidationException, ServiceFailureException;
	
	/**
	 * Retrieves a PayrollSchedule by it's ID.
	 * @param id
	 * @return
	 * @throws ServiceFailureException
	 * @throws ValidationException 
	 */
	public PayrollSchedule findPayrollSchedule(Long payrollScheduleId) throws ServiceFailureException, ValidationException;
	
	/**
	 * Returns the default {@link PayrollSchedule} for the Employer.
	 * 
	 * @return
	 * @throws ServiceFailureException
	 */
	public PayrollSchedule getDefaultPayrollSchedule() throws ServiceFailureException;
	
	/**
	 * Returns all the {@link PayrollSchedule} for the Employer.
	 * 
	 * @return List of PayrollSchedule
	 * @throws ServiceFailureException
	 */
	public List<PayrollSchedule> getAllPayrollSchedules() throws ServiceFailureException;
	
	

	/**
	 * For a given PayrollSchedule, this function will calculate a number of upcoming scheduled PayrollRuns.
	 * It will start it's calcuations from the last committed {@link PayrollRun}.
	 *  
	 * @param payrollSchedule the {@link PayrollSchedule} on which to base the date calculations
	 * @param numberOfPayrollRuns the number of future scheduled {@link PayrollRun}s to calculate.
	 * @return a List of {@link PayrollRunInfo} objects that encapsulate the date-related details of the {@link PayrollRun}.
	 * @throws ServiceFailureException if there is a database or other unrecoverable error.
	 * @throws ValidationException if the PayrollSchedule is invalid, numberOfPayrollRuns is less than 1 or greater than 52
	 */
	public List<PayrollRunInfo> getUpcomingScheduledPayrollRunInfosByYear(PayrollSchedule payrollSchedule, Date year) throws ValidationException, ServiceFailureException;
	
	/**
	 * Returns the PayrollRunInfo for the next incomplete scheduled PayrollRun.  This will always return a value.
	 * 
	 * @param payrollSchedule
	 * @throws ValidationException
	 * @throws ServiceFailureException
	 */
	public PayrollRunInfo getNextScheduledPayrollRunInfo(PayrollSchedule payrollSchedule) throws ValidationException, ServiceFailureException;
	
	/**
	 * Associates a {@link Worker} to a {@link PayrollSchedule}.
	 *  
	 * @param worker the {@link Worker} to assign.
	 * @param payrollSchedule the {@link PayrollSchedule} to assign the worker to.
	 * @return the latest copy of the {@link PayrollSchedule}.
	 * @throws ValidationException if the {@link Worker} is already associated with the {@link PayrollSchedule}
	 * @throws ServiceFailureException if there is a database or other unrecoverable error.
	 */
	public PayrollSchedule assignWorkerToPayrollSchedule(Worker worker, PayrollSchedule payrollSchedule) throws ValidationException, ServiceFailureException;
	
	
	public PayrollSchedule removeWorkerFromPayrollSchedule(Worker worker, PayrollSchedule payrollSchedule) throws ValidationException, ServiceFailureException;
	
	/**
	 * Associates a {@link PayrollSchedule} to a {@link Employer}.
	 *  
	 * @param payrollSchedule the {@link PayrollSchedule} to assign the Employer to.
	 * @return the latest copy of the {@link PayrollSchedule}.
	 * @throws ServiceFailureException if there is a database or other unrecoverable error.
	 */
	public PayrollSchedule assignPayrollScheduleToEmployer(PayrollSchedule payrollSchedule)	throws ValidationException, ServiceFailureException; 
	
	/**
	 * Removes a {@link PayrollSchedule} from a {@link Employer}.
	 *  
	 * @param payrollSchedule the {@link PayrollSchedule} to remove the Employer from.
	 * @return the latest copy of the {@link PayrollSchedule}.
	 * @throws ValidationException if the {@link PayrollSchedule} is not associated with the {@link Employer}
	 * @throws ServiceFailureException if there is a database or other unrecoverable error.
	 */
	public PayrollSchedule removePayrollScheduleFromEmployer(PayrollSchedule payrollSchedule) throws ValidationException, ServiceFailureException ;

}