package com.intuit.sb.payroll.capability.workerpayments;

import java.util.List;

import javax.ejb.Local;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.common.WorkflowEventType;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;
import com.intuit.sb.payroll.entity.model.Worker;

@Local
public interface WorkerPaymentsComponent {

	@SuppressWarnings("nls")
	public static final String JNDI_NAME =  "WorkerPaymentsComponent/local";

	/**
	 * Creates the next logical PayrollRun based on the current PayrollSchedule.
	 * 
	 * @return a PayrollRun
	 * @throws ServiceFailureException 
	 * @throws ValidationException 
	 */
	public PayrollRun getNextDefaultScheduledPayrollRun() throws ServiceFailureException, ValidationException;
	
	/**
	 * @param payrollRunId
	 * @return the PayrollRun with the corresponding ID.
	 * @throws ServiceFailureException 
	 * @throws ValidationException 
	 */
	public PayrollRun findPayrollRun(Long payrollRunId) throws ValidationException, ServiceFailureException;
	
    /**
     * Retrieves recent committed payroll runs of the specified type. The runs are selected (and ordered) based on
     * paycheck date, newest
     * to oldest. The result set is limited to the most recent N runs as specified by numberOfRuns. If N is greater
     * than the total number of payrollRunType payrolls for the company the results are limited to all the existing
     * runs of payrollRunType. 
     *
     * @param payrollRunType the type of payroll runs to retrieve.
     * @param numberOfRuns the number of payroll runs to retrieve.
     * @return an List of PayrollRun, one for each of the most recent payroll runs. PayrollRun.get(0) is the
     * most recent run and PayrollRun.get(numberOfRuns - 1) is the oldest selected run. The list will be empty of
     * no committed payroll runs exist for the company.
     * @throws ValidationException if numberOfRuns <= 0, or payrollRunType == PayrollRunTypeEnum.NOTSET.
     * @throws ServiceFailureException Something bad happened.
     */
    public List<PayrollRun> findRecentPayrollRuns(PayrollRunTypeEnum payrollRunType, int numberOfRuns) throws ServiceFailureException, ValidationException;
    
    /**
     * Retrieves recent committed payroll runs of the specified type. The runs are selected (and ordered) based on
     * paycheck date, newest to oldest. The result set is limited to the runs in the mentioned year.
     *
     * @param payrollRunType the type of payroll runs to retrieve.
     * @param year the year for which payroll runs to retrieve.
     * @return an List of PayrollRun, one for each of the most recent payroll runs. PayrollRun.get(0) is the
     * most recent run. The list will be empty if no committed payroll runs exist for the company.
     * @throws ValidationException if numberOfRuns <= 0, or payrollRunType == PayrollRunTypeEnum.NOTSET.
     * @throws ServiceFailureException Something bad happened.
     */
    public List<PayrollRun> findRecentPayrollRunsForYear(PayrollRunTypeEnum payrollRunType, int year) throws ServiceFailureException, ValidationException;

	/**
	 * Returns a list of all the payroll runs.
	 * 
	 * @return
	 */
	public List<PayrollRun> findAllPayrollRuns();

	public PayrollRun createPayrollRun(PayrollRunCreateRequest request) throws ServiceFailureException, ValidationException;

	/**
	 * Creates a {@link PayrollRun} entity from a collection of {@link PaycheckCalculationRequest}
	 * instances.  The paychecks will be calculated, and the entity will be persisted in a committed state.
	 * 
	 * @param request
	 * @return a PayrollRun entity with calculated paychecks.
	 * @throws ServiceFailureException 
	 * @throws WorkflowValidationException 
	 */
	public PayrollRun commitPayrollRun(PayrollRunCommitRequest request) throws ValidationException, WorkflowException, ServiceFailureException;
	
	/**
	 * Allows for the update of a select few attributes of a payroll run.
	 * @param updateRequest
	 * @return updated PayrollRun
	 * @throws ServiceFailureException 
	 */
	public PayrollRun updatePayrollRun(PayrollRunUpdateRequest updateRequest) throws ValidationException, ServiceFailureException;

	/**
	 * Void one or more paychecks in a given payroll run.  If the given paychecks are in 
     * different payroll runs, a ValidationException is thrown.
	 * @param voidPaychecksRequest
	 * @return updated PayrollRun
	 * @throws ServiceFailureException
	 */
	public PayrollRun voidPaychecks(List<Long> paycheckIds) throws ValidationException, ServiceFailureException;
	
	/**
	 * @param payrollRun
	 * @throws ServiceFailureException 
	 */
	public void deletePayrollRun(Long payrollRunId) throws ValidationException, ServiceFailureException;

	/**
	 * Sets the PayrollRun to a state of cancelled.  The PayrollRun must have been in Draft mode.
	 * 
	 * @param payrollRunId
	 * @throws ServiceFailureException 
	 * @throws PayrollValidationException if the PayrollRun was already committed.
	 */
	public void cancelPayrollRunById(Long payrollRunId) throws ValidationException, ServiceFailureException;

	/**
	 * @param request the Calculation request
	 * @return a Paycheck
	 * @throws ServiceFailureException 
	 * @throws WorkflowValidationException 
	 * @throws PayrollValidationException 
	 */
	public abstract Paycheck calculatePaycheck(PaycheckCalculationRequest request) throws WorkflowException, ValidationException, ServiceFailureException;

	/**
	 * Retrieves a Paycheck by it's unique identifier.
	 * 
	 * @param paycheckId
	 * @return
	 * @throws ValidationException 
	 */
	public Paycheck getPaycheckById(Long paycheckId) throws ValidationException, ServiceFailureException;

	/**
	 * Finds the most recent paycheck 'issued' to a worker. The paycheck may
	 * not have have been physically issued but it is part of the most recent
	 * committed payroll run that includes this worker.
	 * 
	 * 'Most recent' is based on paycheck transaction date. The presumption is
	 * a worker can't have more than one paycheck created within a millisecond.
	 * 
	 * @param worker the worker whose most recent paycheck we need to find.
	 * @return the paycheck or null if the worker has no paychecks.
	 */
	public Paycheck findMostRecentPaycheck(Worker worker) throws ValidationException, ServiceFailureException;

	/**
	 * WorkerPaymentsComponent is affected by changes in the PayrollSchedule.  Specifically it cares if there is a new
	 * instance of the PayrollSchedule, as it creates Tasks that relate to it.  As well, if an existing PayrollSchedule changes, 
	 * it may invalidate Tasks that relate to it, or pending/draft PayrollRuns in the system.
	 * 
	 * @param workflowEventType
	 * @param payrollSchedule
	 * @throws ServiceFailureException
	 */
	public void handlePayrollScheduleEvent(WorkflowEventType workflowEventType, PayrollSchedule payrollSchedule) throws ServiceFailureException;

	/**
	 * WorkerPaymentsComponent is affected by changes in Workers.  It cares to know when the company has set up it's first
	 * payroll-ready worker (to create payment Tasks) and if there are workers in an pending/scheduled PayrollRun that are no 
	 * longer valid it may have to act on that.
	 * 
	 * @param workflowEventType
	 * @param worker
	 * @throws ServiceFailureException
	 */
	public void handleWorkerEvent(WorkflowEventType workflowEventType, Worker worker) throws ServiceFailureException;
}