package com.intuit.sb.payroll.common;

import java.util.Calendar;
import java.util.Date;

import org.apache.commons.lang.time.DateUtils;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.i18n.LocaleGuru;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;

/**
 * Utilities for calculating Java Calendar Dates. Tests are in the DateUtilTest class.
 * 
 * $Author$ 
 * $Revision$ 
 * $DateTime$
 */
public class DateUtil {

	public static final int SECONDS_PER_MINUTE = 60;
	public static final int MINUTES_PER_HOUR = 60;
	public static final int SECONDS_PER_HOUR = SECONDS_PER_MINUTE * MINUTES_PER_HOUR;

	private DateUtil() {
	}

	/**
	 * Utility method to add or remove number of Dates on/from java.util.Date
	 * 
	 * @param date java.util.Date date value to add days
	 * @param addDays Number of Days to add or remove
	 * @return Next or Previous Date
	 */
	public static Date getNextDate(Date date, int addDays) {
		Calendar cal = LocaleGuru.getInstance().getCalendar();
		cal.setTime(date);
		cal.add(Calendar.DATE, addDays);

		return cal.getTime();
	}

	/**
	 * This method returns Calendar instance that will be the next Day in the week as specified in the parameter The
	 * method will return the same date if the current date is the day of Week requested
	 * 
	 * @param calendar
	 * @param dayofWeek Friday, Saturday
	 * @return Calendar representing the next week Day from the given calendar
	 */

	public static Calendar getNextWeekDayOfWeek(Calendar calendar, int dayofWeek) {
		if (calendar == null) {
			return null;
		}
		Calendar cal = LocaleGuru.getInstance().getCalendar();
		cal.setTime(calendar.getTime());
		while (cal.get(Calendar.DAY_OF_WEEK) != dayofWeek) {
			cal.add(Calendar.DAY_OF_WEEK, 1);
		}
		return cal;
	}

	/**
	 * Returns true if the day of the week is Saturday or Sunday.
	 * 
	 * Note : this may only have meaning in locales where weekends are honored.
	 * 
	 * @param calendar
	 * @return true if the day of hte week is  {@link Calendar#SATURDAY} or {@link Calendar#SUNDAY}
	 */
	public static boolean isWeekend(Calendar calendar) {
		if (calendar.get(Calendar.DAY_OF_WEEK) == Calendar.SUNDAY
				|| calendar.get(Calendar.DAY_OF_WEEK) == Calendar.SATURDAY) {
			return true;
		}
		return false;
	}
	
	/**
	 * Calculate number of days between the Date range specified
	 * 
	 * @param fromDate Calendar Date
	 * @param toDate Calendar Date
	 * @return int Number of days between fromDate and toDate
	 */
	public static int findOffsetDays(Calendar fromDate, Calendar toDate) {
		int days = 0;
		int amount = (fromDate.after(toDate) ? -1 : 1);
		while (!(isSameDay(fromDate, toDate))) {
			fromDate.add(Calendar.DATE, amount);
			days++;
		}
		return days * amount;
	}

	/**
	 * Calculate number of days between the Date range specified
	 * 
	 * @param fromDate java.util.Date from date
	 * @param toDate java.util.Date to Date
	 * @return int Number of days between fromDate and toDate
	 */

	public static int findOffsetDays(Date fromDate, Date toDate) {
		Calendar cal1 = LocaleGuru.getInstance().getCalendar();
		Calendar cal2 = LocaleGuru.getInstance().getCalendar();
		cal1.setTime(fromDate);
		cal2.setTime(toDate);
		return findOffsetDays(cal1, cal2);
	}

	/**
	 * Utility method to compare only Year, Month and Day between 2 Calendars
	 * 
	 * @param cal1 Calendar Date
	 * @param cal2 Calendar Date
	 * @return boolean value true if the dates are same false if the dates are different
	 */
	public static boolean isSameDay(Calendar cal1, Calendar cal2) {
		return DateUtils.isSameDay(cal1, cal2);
	}
	
	/**
	 * Utility method to compare only Year, Month and Day between 2 Dates.
	 * @param date1
	 * @param date2
	 * @return boolean value true if the dates are same false if the dates are different
	 */
	public static boolean isSameDay(Date date1, Date date2) {
		return DateUtils.isSameDay(date1, date2);
	}
	
	/**
	 * Adjusts a date by an offset.
	 * 
	 * FIXME: No Holidays or weekends are currently taken account for until we have clear policy on them.
	 * 
	 * @param date
	 * @param offset assumed to be a positive number.
	 * @return the new Calendar instance with the offset applied.
	 */
	public static Calendar adjustDateForOffset(Calendar date, int offset) {
		if (null == date) {
			throw new IllegalArgumentException("No null dates as inputs.");
		}
		Calendar result = DateUtil.copyYMD(date);
		result.add(Calendar.DAY_OF_MONTH, offset);
		return result;
	}

	/**
	 * Calculates the number of years since the date passed in.
	 * 
	 * @param date the Date to calculate from.
	 * @return the number of years elapsed since the input date.
	 */
	@SuppressWarnings("nls")
	public static int calculateYearsElapsedFromNow(Date date) {
		try {
			PayrollParamValidator.checkIsNotNull(date, "input Date");
		} catch (ValidationException e) {
			throw new IllegalArgumentException(e);
		}
		Date now = new Date();
		int age = calculateYearsElapsedFromDate(date, now);
		return age;
	}

	/**
	 * Calculates the number of years elapsed between two dates.
	 * 
	 * @param fromDate
	 * @param toDate
	 * @return the number or years elapsed between the from and to dates.
	 * @throws PayrollException if the result is negative.
	 */
	@SuppressWarnings("nls")
	public static int calculateYearsElapsedFromDate(Date fromDate, Date toDate) {
		try {
			PayrollParamValidator.checkIsNotNull(fromDate, "fromDate");
			PayrollParamValidator.checkIsNotNull(toDate, "toDate");
		} catch (ValidationException e) {
			throw new IllegalArgumentException(e);
		}

		Calendar fromDateCal = LocaleGuru.getInstance().getCalendar();
		fromDateCal.setTime(fromDate);

		Calendar toDateCal = LocaleGuru.getInstance().getCalendar();
		toDateCal.setTime(toDate);

		int age = toDateCal.get(Calendar.YEAR) - fromDateCal.get(Calendar.YEAR);

		if (age < 0) {
			throw new IllegalArgumentException(PayrollResourceManager.getInstance().getString(
					ResourceKeyConstants.INVALID_AGE_EXCEPTION));
		}
		Calendar newFromDate = (Calendar) fromDateCal.clone();
		newFromDate.add(Calendar.YEAR, age);

		if (toDateCal.before(newFromDate)) {
			age--;
		}
		return age;
	}

	/**
	 * Creates a localized Date based on the three inputs. Remember to use Calendar.JANUARY or other constants for the month, or
	 * remember that months are base 0.
	 * 
	 * @param year
	 * @param month (base 0)
	 * @param day
	 * @return a Date.
	 */
	public static Date createDate(int year, int month, int day) {
		Calendar cal = LocaleGuru.getInstance().getCalendar();
		cal.clear();
		cal.set(year, month, day);
		clearCalendarTime(cal);
		return cal.getTime();
	}

	/**
	 * Creates a localized Date based on the three inputs. Remember to use Calendar.JANUARY or other constants for the month, or
	 * remember that months are base 0.
	 * 
	 * @param year
	 * @param month (base 0)
	 * @param day
	 * @return a Date.
	 */
	public static Calendar createCalendar(int year, int month, int day) {
		Calendar cal = LocaleGuru.getInstance().getCalendar();
		cal.clear();
		cal.set(year, month, day);
		clearCalendarTime(cal);
		return cal;
	}
	
	/**
	 * Copies Year, Month and Day from another Calendar and returns a new localized Calendar instance.
	 * 
	 * @param workingDate the Calendar to copy.
	 * @return a new Copy.
	 */
	public static Calendar copyYMD(Calendar workingDate) {
		Calendar cal = LocaleGuru.getInstance().getCalendar();
		cal.clear();
		cal.set(Calendar.YEAR, workingDate.get(Calendar.YEAR));
		cal.set(Calendar.MONTH, workingDate.get(Calendar.MONTH));
		cal.set(Calendar.DATE, workingDate.get(Calendar.DATE));
		clearCalendarTime(cal);
		return cal;
	}

	/**
	 * Wipe the hours minutes and seconds to be at midnight.
	 * 
	 * @param cal
	 */
	private static void clearCalendarTime(Calendar cal) {
		cal.set(Calendar.HOUR_OF_DAY, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
	}

	/**
	 * Convenience method to return the current localized {@link Date}.
	 * @return the current Date.
	 */
	public static Date getNowDate() {
		Calendar cal = LocaleGuru.getInstance().getCalendar();
		return cal.getTime();
	}
	
	/**
	 * Convenience method to return the current localized {@link Calendar}
	 * @return the current Calendar.
	 */
	public static Calendar getNowCalendar() {
		Calendar cal = LocaleGuru.getInstance().getCalendar();
		return cal;
	}
	
	/**
	 * Returns the last day of the month for the {@link Calendar} provided.
	 * 
	 * @param cal the {@link Calendar} set the month/year you are interested in.
	 * @return the last day of the month for the {@link Calendar} provided.
	 */
	public static int getLastDayOfMonth(Calendar cal) {
		return cal.getActualMaximum(Calendar.DAY_OF_MONTH);
	}

	/**
	 * Creates a Calendar instance from a Date.
	 * 
	 * @param date
	 * @return a Calendar instance.
	 */
	public static Calendar dateToCalendar(Date date) {
		Calendar cal = LocaleGuru.getInstance().getCalendar();
		cal.setTime(date);
		return cal;
	}

	/**
	 * Returns true if the first parameter is between the start and end dates.
	 * 
	 * @param date
	 * @param start
	 * @param end
	 * @return
	 */
	public static boolean isInPeriod(Date date, Date start, Date end) {
		if (date.before(start)) {
			return false;
		}
		if (date.after(end)) {
			return false;
		}
		return true;
	}

	/**
	 * Checks if the startDate is on the same day or before the end date.
	 * 
	 * @param startDate
	 * @param endDate
	 * @return
	 */
	public static boolean isBeforeOrOnSameDay(Date startDate, Date endDate) {
		if (isSameDay(startDate, endDate)) {
			return true;
		}

		if (startDate.before(endDate)) {
			return true;
		}
		return false;
	}
	
	/**
	 * Checks if the startDate is on the same day or after the end date.
	 * 
	 * @param startDate
	 * @param endDate
	 * @return
	 */
	public static boolean isAfterOrOnSameDay(Date startDate, Date endDate) {
		if (isSameDay(startDate, endDate)) {
			return true;
		}

		if (startDate.after(endDate)) {
			return true;
		}
		return false;
	}

	/**
	 * Returns the Year for the given date.  Avoids the deprecation warnings in the java.util.Date.
	 * @param date
	 * @return
	 */
	public static int getYearFromDate(Date date) {
		Calendar cal = dateToCalendar(date);
		return cal.get(Calendar.YEAR);
	}

	/**
	 * 
	 * @param effectiveDate
	 * @param yearFromDate
	 * @return
	 */
	public static boolean isInCalendarYear(Date effectiveDate, int yearFromDate) {
		int theYear = getYearFromDate(effectiveDate);
		if (yearFromDate == theYear) {
			return true;
		}
		return false;
	}

	public static Date getLastDayOfCurrentYear() {
		return createDate(getNowCalendar().get(Calendar.YEAR), Calendar.DECEMBER, 31);
	}
	
	public static Date getLastDayOfYear(int year) {
		return createDate(year, Calendar.DECEMBER, 31);
	}
	
	public static Date getFirstDayOfCurrentYear() {
		return createDate(getNowCalendar().get(Calendar.YEAR), Calendar.JANUARY, 1);
	}
	
	public static Date getFirstDayOfYear(int year) {
		return createDate(year, Calendar.JANUARY, 1);
	}
	
}
