package com.intuit.sb.payroll.common.i18n;

import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.security.Principal;
import java.text.DateFormat;
import java.text.MessageFormat;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.TimeZone;

import org.apache.log4j.Logger;

import com.intuit.sb.payroll.InfrastructureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;

/**
 * 
 * This class serves as a single point of contact for locale-related information. It is implemented as
 * a singleton that is initialized when the class is loaded (i.e. no lazy initialization). This class
 * is a factory for locale-aware objects and depends on three locale configurations (called "profiles" 
 * in this context). The profiles are "user", "offering", and "default". The default profile is simply
 * the profile that is configured on the machine running the application. The offering profile is configured
 * from a property file that contains the properties listed in {@linkplain LocaleProfile}. Finally, the user
 * profile is derived from property values stored in the user's authentication record (IAM). 
 * <p/>
 * A note about the user profile:
 * <p/>
 * The user profile is set (via {@linkplain #setUserProfile(java.security.Principal)}) at the beginning of each EJB business method 
 * call. This is done using an EJB interceptor that pulls the user's Principal from the EJB calling context. The
 * user profile is stored in a ThreadLocal instance so that it does not interfere with other in-flight calls
 * to this class. The same EJB interceptor ensures that the ThreadLocal is cleared after each business method.
 * 
 * @author gcoulombe
 *
 */
@SuppressWarnings("nls")
public class LocaleGuru
{
	private static final Logger sLogger = PayrollLoggerFactory.makeLogger(LocaleGuru.class);
	
	/**
	 * System property used to specify a nonstandard locale profile for the offering.
	 */
	public static final String CONFIG_LOCATION_PROPERTY = "com.intuit.sb.payroll.common.i18n.LocaleGuruConfig";
	
	private static final String DEFAULT_CONFIG_LOCATION = "localeprofile.properties";
	
	/** The singleton instance */
	private static LocaleGuru instance;
	
	/** 
	 * This ThreadLocal holds the user's profile for the duration of a single EJB call, allowing the rest of the
	 * system to remain stateless.
	 */
	private static ThreadLocal<LocaleProfile> userProfile = new ThreadLocal<LocaleProfile>();

	/** The single instance of the default locale profile */
	private static final LocaleProfile defaultProfile = new DefaultLocaleProfile();
	
	/** 
	 * This is the resource bundle for the LocaleGuru itself. It is not used by any other classes.
	 * 
	 *  Note: this code can throw a MissingResourceException. This is not caught and will bring down the app.
	 *  This is by design - the inability to configure the Guru is a fatal error.
	 */
	private static final ResourceBundle guruResources = ResourceBundle.getBundle("com.intuit.sb.payroll.common.i18n.localeguru-resources");
	
	
	private LocaleProfile offeringProfile;

	/** Local cache of resource bundles */
	private HashMap<String, ResourceBundle> resourceBundles = new HashMap<String, ResourceBundle>();
	
	
	static
	{
		instance = new LocaleGuru();
	}
	
	private LocaleGuru()
	{
		try
		{
			configure();
		} 
		catch (IOException e)
		{
			String msg = guruResources.getString("configure.error");
			
			sLogger.error(msg, e);
			
			throw new InfrastructureException(msg, e);
		}
	}

	private synchronized void configure() throws IOException
	{
		String configLocation = System.getProperty(CONFIG_LOCATION_PROPERTY);
		
		if (null == configLocation || configLocation.length() == 0)
		{
			configLocation = DEFAULT_CONFIG_LOCATION;
		}
		
		InputStream stream = getClass().getResourceAsStream(configLocation);
		
		if (null == stream)
		{
			// No offering profile was found: we're going to be using just the default profile.
			// While this is probably not desired (hence the warning), it's not really an error 
			// (hence the lack of an exception).
			
			sLogger.warn(MessageFormat.format(guruResources.getString("config.notfound"), configLocation));
			
			return;
		}
		
		Properties props = new Properties();
		try {
		props.load(stream);
		} finally {
			stream.close();
		}
		
		offeringProfile = new LocaleProfile();
		
		offeringProfile.setCountry(props.getProperty(LocaleProfile.COUNTRY_PROPERTY));
		offeringProfile.setLanguage(props.getProperty(LocaleProfile.LANGUAGE_PROPERTY));
		offeringProfile.setTimeZone(props.getProperty(LocaleProfile.TIME_ZONE_PROPERTY));
		offeringProfile.setCurrency(props.getProperty(LocaleProfile.CURRENCY_PROPERTY));
		offeringProfile.setCurrencyFormat(props.getProperty(LocaleProfile.CURRENCY_FORMAT_PROPERTY));
		offeringProfile.setDateFormat(props.getProperty(LocaleProfile.DATE_FORMAT_PROPERTY));
		
		if (sLogger.isInfoEnabled())
		{
			sLogger.info(MessageFormat.format(guruResources.getString("offering.config.complete"), 
					offeringProfile.getLanguage(), 
					offeringProfile.getCountry(), 
					configLocation));
		}
	}
	
	public static LocaleGuru getInstance()
	{
		// In normal operation, instance will be initialized in the static constructor. In testing, however,
		// it's possible for the instance to be cleared, resulting in a null instance. Here, we will initialize
		// the singleton in that case.
		if (null == instance)
		{
			// Note: not synchronizing - the tests are not multithreaded.
			instance = new LocaleGuru();
		}
		
		return instance;
	}
	
	public void setUserProfile(Principal user)
	{
		// TODO: read from IDManager principal
		
		// get IDManager property name-value pairs
		// 
		
		userProfile.set(new LocaleProfile());
	}
	
	void setUserProfile(LocaleProfile profile)
	{
		userProfile.set(profile);
		
		if (sLogger.isInfoEnabled())
		{
			sLogger.info(MessageFormat.format(guruResources.getString("user.config.complete"), 
					getLanguage(), 
					getCountry()));
		}
	}
	
	/**
	 * Reset the user profile ThreadLocal to its default value.
	 */
	public void clearUserProfile()
	{
		userProfile.remove();
	}
	
	/**
	 * Get a localized string from the named resource bundle. In order to accommodate the multiple levels of
	 * locale specification in this class, the following augmented resource bundle search list is used see 
	 * {@linkplain ResourceBundle} for details.
	 * <p/>
	 * Scenario:
	 * 	bundle name = payroll<br/>
	 *  user locale = fr_CA<br/>
	 *  offering locale = en_CA<br/>
	 *  default locale = en_US<br/>
	 *  
	 * <p/>
	 * The candidate resource bundles in order of preference are:
	 * 
	 * <ol>
	 * <li>payroll_fr_CA.properties</li>
	 * <li>payroll_fr.properties</li>
	 * <li>payroll.properties</li>
	 * <li>payroll_en_CA.properties</li>
	 * <li>payroll_en.properties</li>
	 * <li>payroll_en_US.properties</li>
	 * <li>payroll_en.properties</li>
	 * </ol>
	 * 
	 * TODO: this method doesn't really do the lookup 100% correctly: if the resource bundle exists for a particular
	 * locale (fr_CA, in this case) but does not contain the requested string, the resource manager will look to the
	 * language-only bundle (fr, in this case) and the non-locale-specific bundle (payroll.properties, in this case)
	 * but it won't look to the next locale.
	 * 
	 * @see ResourceBundle
	 * @see MessageFormat
	 * 
	 * @param bundleName the qualified name of the resource bundle to load
	 * @param key the name of the string resource to return
	 * @param args the placeholder values for the given string
	 * 
	 * @return the resource string with the given args substituted for any placeholders in the string
	 * 
	 * @throws IllegalArgumentException if either bundleName or key is null or blank
	 */
	public String getString(String bundleName, String key, String... args)
	{
		try {
			PayrollParamValidator.checkIsNotNullOrBlankString(bundleName, "bundleName");
			PayrollParamValidator.checkIsNotNullOrBlankString(key, "key");
		} catch (ValidationException e) {
			throw new IllegalArgumentException(e);
		}
		
		Locale locale = getLocale();
		
		ResourceBundle bundle;
		
		try
		{	
			bundle = getResourceBundle(bundleName, locale);
		}
		catch(MissingResourceException e)
		{
			String msg = MessageFormat.format(guruResources.getString("bundle.notfound"), bundleName);
			
			sLogger.error(msg);
			
			throw new InfrastructureException(msg, e);
		}
		
		try
		{	
			String s = bundle.getString(key);
			
			String rv = MessageFormat.format(s, (Object[])args);
			
			// UTF-8 fix: since ResourceBundles (well, properties files, actually) work in ISO-8859-1,
			// we need to convert the strings to UTF-8 so the non-latin characters load properly.
			
			return new String(rv.getBytes("ISO-8859-1"), "UTF-8");
		}
		catch(MissingResourceException e)
		{
			String msg = MessageFormat.format(guruResources.getString("string.notfound"), key, bundleName);
			
			sLogger.error(msg);
			
			throw new InfrastructureException(msg, e);
		} 
		catch (UnsupportedEncodingException e)
		{
			String msg = MessageFormat.format(guruResources.getString("string.illegalencoding"), key, bundleName);
			
			sLogger.error(msg, e);
			
			throw new InfrastructureException(msg, e);
		}
	}

	private synchronized ResourceBundle getResourceBundle(String bundleName, Locale locale)
	{
		String bundleKey = bundleName + locale.toString();
		
		ResourceBundle bundle = resourceBundles.get(bundleKey);
		
		if (null == bundle)
		{
			bundle = ResourceBundle.getBundle(bundleName, locale);
					
			// bundle check passed. Save it for later.
			resourceBundles.put(bundleKey, bundle);
		}
		
		return bundle;
	}

	/**
	 * This method returns the effective locale. That is, this method uses the user, offering and default profiles 
	 * (in that order) to construct a locale.
	 * 
	 * @return the effective locale.
	 */
	public Locale getLocale()
	{
		String country = getCountry();
		String language = getLanguage();

        return new Locale(language, country);
	}
	
	/**
	 * This method returns the offering locale. This is different from the locale returned by {@linkplain #getLocale()}
	 * because this locale does not take into account the user's locale preferences. This locale can be used to configure
	 * locale-sensitive business logic, if necessary.
	 * 
	 * @return the offering locale.
	 */
	public Locale getOfferingLocale()
	{
		LocaleProfile languageProfile = offeringProfile;
		
		if (null == languageProfile || null == languageProfile.getLanguage())
		{
			languageProfile = defaultProfile;
		}
		
		String language = languageProfile.getLanguage();
		
		
		LocaleProfile countryProfile = offeringProfile;
		
		if (null == countryProfile || null == countryProfile.getCountry())
		{
			countryProfile = defaultProfile;
		}
		
		String country = countryProfile.getCountry();
		
		return new Locale(language, country);
	}
	
	/**
	 * Get the number formatter that creates formatted current amounts.
	 * Note: for now, this is just using the declared locale rather than the explicit 
	 * currency and currency format properties: it's just easier this way.
	 * 
	 * @return a currency-formatting NumberFormat object.
	 */
	public NumberFormat getCurrencyFormat()
	{
		Locale locale = getLocale();

        return NumberFormat.getCurrencyInstance(locale);
	}
	
	public String getCurrency()
	{
		LocaleProfile profile = findCurrencyProfile();

        return profile.getCurrency();
	}

	private LocaleProfile findCurrencyProfile()
	{
		LocaleProfile profile = userProfile.get();
		
		if (null == profile || null == profile.getCurrency())
		{
			profile = offeringProfile;
			
			if (null == profile || null == profile.getCurrency())
			{
				profile = defaultProfile;
			}
		}
		
		return profile;
	}
	
	public TimeZone getTimeZone()
	{
		LocaleProfile profile = findTimeZoneProfile();
				
		String tz = profile.getTimeZone();

		TimeZone rv = TimeZone.getTimeZone(tz);
		
		if (null == rv || false == tz.equals(rv.getID()))
		{
			String msg = MessageFormat.format(guruResources.getString("timezone.notfound"), tz);
			
			sLogger.error(msg);
			
			throw new InfrastructureException(msg);
		}
		
		return rv;
	}

	private LocaleProfile findTimeZoneProfile()
	{
		LocaleProfile profile = userProfile.get();
		
		if (null == profile || null == profile.getTimeZone())
		{
			profile = offeringProfile;
			
			if (null == profile || null == profile.getTimeZone())
			{
				profile = defaultProfile;
			}
		}
		return profile;
	}
	
	public DateFormat getDateFormat()
	{
		LocaleProfile profile = findDateFormatProfile();
				
		String fmt = profile.getDateFormat();

		DateFormat rv = new SimpleDateFormat(fmt, getLocale());
		
		return rv;
	}

	private LocaleProfile findDateFormatProfile()
	{
		LocaleProfile profile = userProfile.get();
		
		if (null == profile || null == profile.getDateFormat())
		{
			profile = offeringProfile;
			
			if (null == profile || null == profile.getDateFormat())
			{
				profile = defaultProfile;
			}
		}
		return profile;
	}

	public String getLanguage()
	{
		LocaleProfile profile = findLanguageProfile();

        return profile.getLanguage();
	}

	private LocaleProfile findLanguageProfile()
	{
		LocaleProfile profile = userProfile.get();
		
		if (null == profile || null == profile.getLanguage())
		{
			profile = offeringProfile;
			
			if (null == profile || null == profile.getLanguage())
			{
				profile = defaultProfile;
			}
		}
		return profile;
	}

	public String getCountry()
	{
		LocaleProfile profile = findCountryProfile();
		
		String rv = profile.getCountry();
		
		return rv;
	}

	private LocaleProfile findCountryProfile()
	{
		LocaleProfile profile = userProfile.get();
		
		if (null == profile || null == profile.getCountry())
		{
			profile = offeringProfile;
			
			if (null == profile || null == profile.getCountry())
			{
				profile = defaultProfile;
			}
		}
		return profile;
	}
	
	public Calendar getCalendar()
	{
		Locale locale = getLocale();

        return Calendar.getInstance(getTimeZone(), locale);
	}

	/**
	 * Clear the singleton instance. Used for testing purposes only.
	 */
	static void clear()
	{
		instance = null;
		userProfile.remove();
	}
	
	/**
	 * 
	 * This class is a default implementation of LocaleProfile. It provides access to the standard default
	 * Locale and a set of sensible defaults.
	 * 
	 * @author gcoulombe
	 *
	 */
	private static class DefaultLocaleProfile extends LocaleProfile
	{
		@Override
		public String getCountry()
		{
			return getLocale().getCountry();
		}

		@Override
		@SuppressWarnings("nls")
		public String getCurrency()
		{
			return "USD";
		}

		@Override
		@SuppressWarnings("nls")
		public String getCurrencyFormat()
		{
			return "###,###,###,###.##";
		}

		@Override
		public String getDateFormat()
		{
			// TODO return DateFormat.getDateInstance(DateFormat.SHORT, getDefaultLocale()).;
			return null;
		}
		
		@Override
		public String getTimeZone()
		{
			return TimeZone.getDefault().getID();
		}

		@Override
		public String getLanguage()
		{
			return getLocale().getLanguage();
		}

		@Override
		public void setCountry(String country)
		{
			throw new IllegalStateException(guruResources.getString("default.readonly"));
		}

		@Override
		public void setCurrency(String currency)
		{
			throw new IllegalStateException(guruResources.getString("default.readonly"));
		}

		@Override
		public void setCurrencyFormat(String currencyFormat)
		{
			throw new IllegalStateException(guruResources.getString("default.readonly"));
		}

		@Override
		public void setDateFormat(String dateFormat)
		{
			throw new IllegalStateException(guruResources.getString("default.readonly"));
		}

		@Override
		public void setLanguage(String language)
		{
			throw new IllegalStateException(guruResources.getString("default.readonly"));
		}

		private Locale getLocale()
		{
			return Locale.getDefault();
		}		
	}
}