package com.intuit.sb.payroll.common;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.common.i18n.LocaleGuru;

/**
 * Tests the DateUtil class in the same package.
 * 
 * $Author$ 
 * $Revision$ 
 * $DateTime$
 */
public class DateUtilTest {

	@Test
	public void testIsInPeriod() {
		final Date start = DateUtil.createDate(2009, Calendar.JANUARY, 11);
		final Date end = DateUtil.createDate(2009, Calendar.JANUARY, 21);
		
		Date date = DateUtil.createDate(2009, Calendar.JANUARY, 12);
		boolean actual = DateUtil.isInPeriod(date, start, end);
		AssertJUnit.assertTrue("Expected date to be in range", actual);
		
		date = DateUtil.createDate(2009, Calendar.JANUARY, 22);
		actual = DateUtil.isInPeriod(date, start, end);
		AssertJUnit.assertFalse("Expected date to be out of range", actual);
		
		date = DateUtil.createDate(2009, Calendar.JANUARY, 10);
		actual = DateUtil.isInPeriod(date, start, end);
		AssertJUnit.assertFalse("Expected date to be out of range", actual);
		
		date = DateUtil.createDate(2009, Calendar.JANUARY, 11);
		actual = DateUtil.isInPeriod(date, start, end);
		AssertJUnit.assertTrue("Expected date to be in range", actual);
	}
	
	@Test
	public void getLastDayOfMonthJanTest() {
		Calendar fromDate = DateUtil.createCalendar(2009, 0, 1);
		int expected = 31;
		int actual = DateUtil.getLastDayOfMonth(fromDate);
		AssertJUnit.assertEquals(expected, actual);
	}
	
	@Test
	public void getLastDayOfMonthFebTest() {
		Calendar fromDate = DateUtil.createCalendar(2009, 1, 1);
		int expected = 28;
		int actual = DateUtil.getLastDayOfMonth(fromDate);
		AssertJUnit.assertEquals(expected, actual);
	}

	@Test
	public void getLastDayOfMonthFebLeapYearTest() {
		Calendar fromDate = DateUtil.createCalendar(2008, 1, 1);
		int expected = 29;
		int actual = DateUtil.getLastDayOfMonth(fromDate);
		AssertJUnit.assertEquals(expected, actual);
	}


	@Test
	public void testGetNextPaydaySame() {
		// The GregorianCalendar month value starts with 0
		// The following GregorianCalendar date is February 29, 2008
		Calendar nextPayDate = DateUtil.getNextWeekDayOfWeek(DateUtil.createCalendar(2008, 1, 29),
				GregorianCalendar.FRIDAY);
		// Expected date should be same as it falls on Friday
		AssertJUnit.assertEquals("Day mismatch", 29, nextPayDate.get(Calendar.DAY_OF_MONTH));
		// The month value begins with zero
		AssertJUnit.assertEquals("Month mismatch", 2, nextPayDate.get(Calendar.MONTH) + 1);
		AssertJUnit.assertEquals("Year mismatch", 2008, nextPayDate.get(Calendar.YEAR));
	}

	@Test
	public void testGetNextDayOfWeek() {
		// The GregorianCalendar month value starts with 0
		// The following GregorianCalendar date is February 29, 2008
		Calendar nextPayDate = DateUtil.getNextWeekDayOfWeek(DateUtil.createCalendar(2008, 1, 29),
				GregorianCalendar.MONDAY);
		// Expected date should be the following Monday
		AssertJUnit.assertEquals("Day mismatch", 3, nextPayDate.get(Calendar.DAY_OF_MONTH));
		AssertJUnit.assertEquals("Month mismatch", 3, nextPayDate.get(Calendar.MONTH) + 1);
		AssertJUnit.assertEquals("Year mismatch", 2008, nextPayDate.get(Calendar.YEAR));
	}
	
	@Test
	public void testIsWeekend() {
		Calendar weekdayCal = DateUtil.createCalendar(2009, 0, 7);
		boolean actualWeekday = DateUtil.isWeekend(weekdayCal);
		AssertJUnit.assertFalse("Should have resolved to a weekday", actualWeekday);
		Calendar saturdayCal = DateUtil.createCalendar(2009, 0, 10);
		boolean actualSaturday = DateUtil.isWeekend(saturdayCal);
		AssertJUnit.assertTrue("Saturday should have been a weekend", actualSaturday);
		Calendar sundayCal =  DateUtil.createCalendar(2009, 0, 11);
		boolean actualSunday = DateUtil.isWeekend(sundayCal);
		AssertJUnit.assertTrue("Sunday should have been a weekend", actualSunday);
	}

	/**
	 * This test method find the number of days between 2 dates in a month fromDate > toDate
	 */
	@Test
	public void testFindOffsetDaysInMonthReverse() {
		// The GregorianCalendar month value starts with 0
		// The following GregorianCalendar date is May 28, 2008
		Calendar fromDate = DateUtil.createCalendar(2008, 4, 28);

		// The GregorianCalendar month value starts with 0
		// The following GregorianCalendar date is May 13, 2008
		Calendar toDate = DateUtil.createCalendar(2008, 4, 13);

		AssertJUnit.assertEquals("Number of Offset days is not -15 as expected", -15, DateUtil.findOffsetDays(fromDate,
				toDate));

	}

	/**
	 * This test method find the number of days between 2 dates in a month fromDate < toDate
	 */
	@Test
	public void testFindOffsetDaysInMonth() {
		// The GregorianCalendar month value starts with 0
		// The following GregorianCalendar date is May 28, 2008
		Calendar toDate = DateUtil.createCalendar(2008, 4, 28);

		// The GregorianCalendar month value starts with 0
		// The following GregorianCalendar date is May 13, 2008
		Calendar fromDate = DateUtil.createCalendar(2008, 4, 13);

		AssertJUnit.assertEquals("Number of Offset days is not 15 as expected", 15, DateUtil
				.findOffsetDays(fromDate, toDate));

	}

	/**
	 * This test method find the number of days between 2 dates spanning in different months
	 */
	@Test
	public void testFindOffsetDaysInYear() {
		// The GregorianCalendar month value starts with 0
		// The following GregorianCalendar date is March 28, 2008
		Calendar fromDate = DateUtil.createCalendar(2008, 2, 28);

		// The GregorianCalendar month value starts with 0
		// The following GregorianCalendar date is May 13, 2008
		Calendar toDate = DateUtil.createCalendar(2008, 4, 13);

		AssertJUnit.assertEquals("Number of Offset days is not 46 as expected", 46, DateUtil
				.findOffsetDays(fromDate, toDate));
	}

	/**
	 * This test method find the number of days between 2 dates spanning in different years
	 */
	@Test
	public void testFindOffsetDaysAcrossYear() {
		// The GregorianCalendar month value starts with 0
		// The following GregorianCalendar date is March 28, 2008
		Calendar fromDate = DateUtil.createCalendar(2007, 2, 28);

		// The GregorianCalendar month value starts with 0
		// The following GregorianCalendar date is May 13, 2008
		Calendar toDate = DateUtil.createCalendar(2008, 4, 13);

		AssertJUnit.assertEquals("Number of Offset days is not 412 as expected", 412, DateUtil.findOffsetDays(fromDate,
				toDate));
	}

	/**
	 * This test method compares two same dates and expects a true outcome
	 */
	@Test
	public void testCompareSameDates() {
		Calendar today = LocaleGuru.getInstance().getCalendar();

		Calendar date = LocaleGuru.getInstance().getCalendar();

		AssertJUnit.assertTrue("Dates not equal", DateUtil.isSameDay(today, date));
	}

	/**
	 * This Test method compares 2 different dates and expects a false result
	 */
	@Test
	public void testCompareDifferentDates() {
		Calendar today = LocaleGuru.getInstance().getCalendar();

		Calendar date = LocaleGuru.getInstance().getCalendar();

		date.add(Calendar.DAY_OF_WEEK, 1);

		AssertJUnit.assertFalse("Dates should not be equal", DateUtil.isSameDay(today, date));

	}

	/**
	 * This method tests that the calc can deal with partial years elapsing. It creates a scenario where there is a
	 * partial year elapsed.
	 */
	@Test
	public void testCalculatePartialYearsElapsedFromDate() {
		Date myBirthday = DateUtil.createDate(1980, 6, 1);
		Date today = DateUtil.createDate(2007, 5, 1);
		int myAge = DateUtil.calculateYearsElapsedFromDate(myBirthday, today);
		AssertJUnit.assertEquals(26, myAge);
	}

	/**
	 * This method tests that the calc can deal with full years elapsing. It creates a scenario where there is a full
	 * year elapsed.
	 */
	@Test
	public void testCalculateFullYearsElapsedFromDate() {
		Date myBirthday = DateUtil.createDate(1980, 6, 1);
		Date today = DateUtil.createDate(2007, 8, 1);
		int myAge = DateUtil.calculateYearsElapsedFromDate(myBirthday, today);
		AssertJUnit.assertEquals(27, myAge);
	}

	/**
	 * This method tests that the calc can deal with null inputs as expected.
	 */
	@Test(expectedExceptions = IllegalArgumentException.class)
	public void testCalculateNullInputs() {
		DateUtil.calculateYearsElapsedFromDate(null, null);
	}

	/**
	 * This method tests that the calc can deal with bad inputs as expected.
	 */
	@Test(expectedExceptions = IllegalArgumentException.class)
	public void testCalculateYearsElapsedBadDate() {
		Date myBirthday = DateUtil.createDate(1980, 6, 1);
		Date today = DateUtil.createDate(1970, 5, 1);
		DateUtil.calculateYearsElapsedFromDate(myBirthday, today);
	}
	
	@Test
	public void testIsBeforeOrOnSameDayPositive() {
		Date before = DateUtil.createDate(2008, 1, 1);
		Date after = DateUtil.createDate(2008, 1, 2);
		boolean expected = true;
		boolean actual = DateUtil.isBeforeOrOnSameDay(before, after);
		AssertJUnit.assertEquals(expected, actual);
	}
	
	@Test
	public void testIsBeforeOrOnSameDayNegative() {
		Date before = DateUtil.createDate(2008, 1, 1);
		Date after = DateUtil.createDate(2007, 1, 2);
		boolean expected = false;
		boolean actual = DateUtil.isBeforeOrOnSameDay(before, after);
		AssertJUnit.assertEquals(expected, actual);
	}
	
	@Test
	public void testIsBeforeOrOnSameDay() {
		Date before = DateUtil.createDate(2008, 1, 1);
		Date after = DateUtil.createDate(2008, 1, 1);
		boolean expected = true;
		boolean actual = DateUtil.isBeforeOrOnSameDay(before, after);
		AssertJUnit.assertEquals(expected, actual);
	}
	
	@Test
	public void testIsAfterOrOnSameDayPositive() {
		Date before = DateUtil.createDate(2008, 1, 1);
		Date after = DateUtil.createDate(2008, 1, 2);
		boolean expected = true;
		boolean actual = DateUtil.isAfterOrOnSameDay(after, before);
		AssertJUnit.assertEquals(expected, actual);
	}
	
	@Test
	public void testIsAfterOrOnSameDayNegative() {
		Date before = DateUtil.createDate(2008, 1, 1);
		Date after = DateUtil.createDate(2007, 1, 2);
		boolean expected = false;
		boolean actual = DateUtil.isAfterOrOnSameDay(after, before);
		AssertJUnit.assertEquals(expected, actual);
	}
	
	@Test
	public void testIsAfterOrOnSameDay() {
		Date before = DateUtil.createDate(2008, 1, 1);
		Date after = DateUtil.createDate(2008, 1, 1);
		boolean expected = true;
		boolean actual = DateUtil.isAfterOrOnSameDay(after, before);
		AssertJUnit.assertEquals(expected, actual);
	}

}
