package com.intuit.sb.payroll.entity.eao;

//TODO The DAO is for a BaseEntity type, not the more speciailized DomainEntity/GlobalEntity (or NamedEntity). This limits the operations. Is this what we want from the base DAO?

import java.util.List;
import java.util.Map;

import javax.persistence.OptimisticLockException;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.entity.domain.Identifiable;

/**
 * Generic interface for Entity Access Objects (EAOs) that provides an encapsulation
 * of transaction and persistence access concerns in an ORM environment.
 * <p>
 * All CRUD operations are isolated in this interface via state-management oriented persistence
 * (e.g. there is no UDPATE function) that provides automatic transactional dirty checking
 * of persistent entities. All persistence operations (including read) must occur within
 * a transaction.
 * <p>
 * While it's fine to continue to use the tried and tested term "CRUD", in an EJB ORM
 * world it's more correct to think of PRADR (Persist, Retrieve, Attach, Detach, and Remove).
 * This is mentioned purely in order to give the correct mindset and to lead the reader
 * towards the appropriate terminology which manifests itself in some of the operations on
 * this interface.  
 * <p>
 * Entities exist in the following states: 
 * <ul>
 * <li>Transient: an entity with no persistent state. Typically this is an
 * object that has just been constructed and has not yet been passed to a
 * {@link #makePersistent} call. Changes to these entities can
 * never be rolled back and they cease to exist once they are garbage collected
 * by the VM. They can not be recovered.</li>
 * <li>Detached: a persistent entity that is currently NOT being managed by the
 * persistence provider. Changes to this entity will not be persisted and
 * no database fetches will occur on the entity while it is in this state.
 * Unless a member field or associated entity was loaded while the entity was
 * previously managed it will not be accessible while the entity remains detached.
 * Objects for detached entities can be garbage collected by the VM but,
 * unlike transient entities, they can later be restored from the database
 * (as managed entities).</li>
 * <li>Managed: a persistent entity that is currently being managed by the
 * persistence provider. Changes to this entity will be persisted the
 * next time a transaction is committed on the current persistence context.
 * Navigating the object graph from a managed entity results in the associated
 * entity objects being returned. A detached entity can become managed by calling
 * {@link #makeManaged}.</li>
 * <li>Removed: a persistent entity that has been removed from the database
 * via a {@link #makeTransient} call, but the associated
 * transaction has not been committed.</li>
 * </ul>
 * See the EJB3 Persistence specification if more details are needed.
 * <p>
 * All EAOs should be exposed as remote stateless session beans. This allows
 * the persistence tier to be remote from the business logic. They can optional
 * also be exposed as local SLSB and/or web services. The annotations for
 * this must be on the "business logic" interface that extends this class so the
 * specialised bean interface name is registered in JNDI.
 * 
 * @param &lt;E&gt; The class of the entity this EAO accesses.
 */
public interface GenericEAO<E extends Identifiable>
{
    /**
     * Creates an entity that implements interface {@code E}.
     * This isolates the concrete class.
     * 
	 * @return The newly constructed and initialized object. The
	 * object is a transient object.
     */
    public E create();
    
	/**
	 * Find the data object by its unique ID (primary key). Whether this eagerly or lazily
	 * cascades to loading associated entities is determined by the {@code fetch=FetchType.X}
	 * attribute of the association annotations.
	 * <p>
	 * @param Id The Id to query by. 
	 * @return The selected object, or null if there is no match for the specified Id.
	 */
	public E findById(Long Id);

	/**
	 * Find the data object by its identifiable properties. Whether this eagerly or lazily
	 * cascades to loading associated entities is determined by the {@code fetch=FetchType.X}
	 * attribute of the association annotations.
	 * @param identifiable the identifiable properties that uniquely specify an instance of an entity. 
	 * @return The selected object, or null if there is no match.
	 * @throws IllegalArgumentException if the entity does not exist.
	 * @throws OptimisticLockException if the entity is newer then the supplied Identifiable.
	 */
	public E findByIdentifiable(Identifiable identifiable);

	/**
	 * Find all Data objects of a given type.
	 * @return a List of objects or an empty collection if there are no matches.
	 */
	public List<E> findAll();

    /**
     * Obtains the list of entities of type {@code E} which satisfy named query {@code query}.
     * The query must be a standard EJB3 JPQL named query????
     * 
     * @param queryName the name of the named query.
     * 
     * @return The list of selected entities. If no matches are found {@code List.size()}
     * will be zero.
     */
    public List<E> findByNamedQuery(String queryName);
    
    /**
     * Obtains the list of entities of type {@code E} which satisfy named query {@code query}
     * with the provided named parameters.
     * The query must be a standard EJB3 JPQL named query????
     * 
     * @param queryName the name of the named query.
     * @param parameters the map of name/value pairs for the named query parameters.
     * 
     * @return The list of selected entities. If no matches are found {@code List.size()}
     * will be zero.
     */
    public List<E> findByNamedQuery(String queryName, Map<String, Object> parameters);
    
    /**
     * Obtains the paged list of entities of type {@code E} which satisfy named query {@code query}
     * with the provided named parameters.
     * The query must be a standard EJB3 JPQL named query.
     * 
     * @param queryName the name of the named query.
     * @param parameters the map of name/value pairs for the named query parameters.
     * @param firstResult the first results to return. Set to zero to start with the first
     * entity retrieved by the query.
     * @param maxResults the maximum number of entities to return, starting with the
     * firstResult entity. 
     * 
     * @return The list of zero to maxResults entities. If no matches are found {@code List.size()}
     * will be zero. If the number of entities selected by the query is less than maxResults
     * the list will only contain the selected entities.
     */
    public List<E> findByNamedQuery(String queryName, Map<String, Object> parameters, int firstResult, int maxResults);
    
    /**
     * Associates a detached entity with the current persistence context so it is now
     * a managed entity and takes part in the full persistence life cycle.
     * <p>
     * The returned object will not be the same Java object as you pass to this method.
	 * You must update any references to this entity with the newly managed Java object
	 * and discard the old, stale, reference.
     *  
     * @param entity The entity to place under persistence management.
     * @return the reference to the object in the new PC.
     */
    public E makeManaged(E entity);
    
    /**
     * Adds a Java object to the persistent store. More strictly, it makes a transient
     * object a persistent entity. This causes appropriate row(s) to be added to the DB.
     * <p>
     * All objects returned by {@link #create()} must be passed to {@code makePersistent()}
     * in order to have them persisted to the DB. Failure to do so often results in an
     * {@code org.hibernate.TransientObjectException}.
     * <p>
     * Once an object has been persisted there is no need to call {@code makePersistent()}
     * on it again, even when it has been modified in a future transaction; updates will
     * automatically be saved to the DB.
     * <p>
     * Detached entities should not be passed to {@code makePersistent()} as the behaviour of this
     * request on detached objects varies across persistence implementations. Instead,
     * detached objects should be passed to a {@code makeManaged()} request.
     * <p>
     * Once a transient object has been made persistent via a {@code makePersistent()} request
     * it can be updated just like any other managed entity. All updates to an entity between
     * a {@code makePersistent()} call and transaction commit will be persisted during the commit
     * as part of the insert into the persistent store. This occurs with one exception: read-only
     * fields (those marked updatable=false) cannot be modified after the {@code makePersistent()}
     * call. All read-only fields must be set before the {@code makePersistent()} call; any changes 
     * to read-only fields of a managed entity are ignored during transaction commit. 
     * <p>
     * The object will not be permanently written to the persistent store until the current
     * transaction is committed.
     * 
     * @param entity The entity to persist.
     * @return The newly managed entity. The returned object will not be the same Java object as
     * you pass to this method.
     */
    public E makePersistent(E entity);

    /**
     * Returns true if the entity is not in session (is not managed by the EntityManager).
     * 
     * The intent is to inform the caller if they need to make the entity persistent themselves.
     * 
     * @param entity
     * @return true if the entity is not in session.
     */
    public boolean notPersistent(E entity);
    
    /**
     * Removes an entity from the DB. Its row(s) are deleted from the DB. The object is then
     * transient and won't exist once it is garbage collected by the VM.
     * <p>
     * The object will not actually be removed until the current transaction is committed.
     * <p>
     *  
     * @param entity The entity to remove.
     */
    public void makeTransient(E entity);
    
    /**
     * Attempts to flush the current session to the DB.
     */
    public void flush();


    /** Set the TenantContext attribute for the transaction when external calls
     *  require data to be persisted
     * @param companyId
     * @throws ServiceFailureException
     */
    public void setTenantContext(String companyId) throws ServiceFailureException;

    /** Clear the TenantContext after persisting data from external call
     *
     * @throws ServiceFailureException
     */
    public void clearTenantContext();
}
