package com.intuit.sb.payroll.entity.model;

import java.math.BigDecimal;
import java.util.Currency;
import java.util.Date;

import javax.persistence.AttributeOverride;
import javax.persistence.AttributeOverrides;
import javax.persistence.Column;
import javax.persistence.Enumerated;
import javax.persistence.Transient;


/**
 * <p><i>PtdDatumType</i> defines the main part of the container body designed to carry <i>Period_to_Date (PTD)</i> values.
 * <br>For each value the <i>ptdType</i> defines the type of the period, e.g., QTD, YTD, and the actual period (defined by 
 * the <i>start</i> and <i>end</i> dates) for which this <i>datum</i> is defined.
 * <br>The type of the value carried is described by the <i>datumType</i> and may be one of two types:
 * <li>Quantity
 * <li>Money 
 * <p>
 * <br>If the type is <i>Money</i>, then a set of additional (auxiliary) values: Income Subject To Tax (ISTT), Wage Base, and TotalHours 
 * will be defined, when applicable.
 */
@javax.persistence.Embeddable
public class PtdDatumType extends PtdDatumTypeBase {

	private static final long serialVersionUID = -1049087433L;

	private static final java.util.Currency USD = Currency.getInstance("USD");

	/**
	 * Default constructor. 
	 */
	public PtdDatumType() {
		setPtdType(PtdTypeEnum.NOTSET);
		setDatumType(DatumType1Enum.NOTSET);
	}

    /*
     * @see PtdDatumTypeBase#getPtdType
     */

    @Enumerated(javax.persistence.EnumType.STRING)
    public PtdTypeEnum getPtdType()
    {
        return super.getPtdType();
    }

    /*
     * @see PtdDatumTypeBase#setPtdType(PtdTypeEnum)
     */
    public void setPtdType(PtdTypeEnum value)
    {
        super.setPtdType(value);
    }

    /*
     * @see PtdDatumTypeBase#getPeriod
     */

    public DateRange getPeriod()
    {
        return super.getPeriod();
    }

    /*
     * @see PtdDatumTypeBase#setPeriod(DateRange)
     */
    public void setPeriod(DateRange value)
    {
        super.setPeriod(value);
    }

    public void setPeriod(Date startDate, Date endDate)
    {
        super.setPeriod(new DateRange(startDate, endDate));
    }

    /*
     * @see PtdDatumTypeBase#getDatumType
     */

    @Enumerated(javax.persistence.EnumType.STRING)
    public DatumType1Enum getDatumType()
    {
        return super.getDatumType();
    }

    /*
     * @see PtdDatumTypeBase#setDatumType(DatumType1Enum)
     */
    public void setDatumType(DatumType1Enum value)
    {
        super.setDatumType(value);
    }

    /*
     * @see PtdDatumTypeBase#getMoney
     */
	@AttributeOverrides({
	    @AttributeOverride(name="amount", column=@Column(name="moneyAmount")),
	    @AttributeOverride(name="currencyCode", column=@Column(name="moenyCurrencyCode"))
	})
	@Deprecated
    public Money getMoney()
    {
        return super.getMoney();
    }

    /*
     * @see PtdDatumTypeBase#setMoney(Money)
     */
    public void setMoney(Money value)
    {
        super.setMoney(value);
    }

    /*
     * @see PtdDatumTypeBase#getWageBase
     */
	@AttributeOverrides({
	    @AttributeOverride(name="amount", column=@Column(name="wageBaseAmount")),
	    @AttributeOverride(name="currencyCode", column=@Column(name="wageBaseCurrencyCode"))
	})
	@Deprecated
    public Money getWageBase()
    {
        return super.getWageBase();
    }

    /*
     * @see PtdDatumTypeBase#setWageBase(Money)
     */
    public void setWageBase(Money value)
    {
        super.setWageBase(value);
    }

    /*
     * @see PtdDatumTypeBase#getIncomeSubjectToTax
     */
	@AttributeOverrides({
	    @AttributeOverride(name="amount", column=@Column(name="isttAmount")),
	    @AttributeOverride(name="currencyCode", column=@Column(name="isttCurrencyCode"))
	})
    public Money getIncomeSubjectToTax()
    {
        return super.getIncomeSubjectToTax();
    }

    /*
     * @see PtdDatumTypeBase#setIncomeSubjectToTax(Money)
     */
    public void setIncomeSubjectToTax(Money value)
    {
        super.setIncomeSubjectToTax(value);
    }

    /*
     * @see PtdDatumTypeBase#getTotalHours
     */

    public Double getTotalHours()
    {
        return super.getTotalHours();
    }

    /*
     * @see PtdDatumTypeBase#setTotalHours(Double)
     */
    public void setTotalHours(Double value)
    {
        super.setTotalHours(value);
    }

    /*
     * @see PtdDatumTypeBase#getQuantity
     */

    public Double getQuantity()
    {
        return super.getQuantity();
    }

    /*
     * @see PtdDatumTypeBase#setQuantity(Double)
     */
    public void setQuantity(Double value)
    {
        super.setQuantity(value);
    }

	/**
	 * Gets the <i>money</i> value and checks that datum type is <i>money</i>.
	 * @return the money
	 * @throws IllegalStateException
	 */
	@AttributeOverrides({
	    @AttributeOverride(name="amount", column=@Column(name="moneyAmount")),
	    @AttributeOverride(name="currencyCode", column=@Column(name="moneyCurrencyCode"))
	})
	@Transient
	public Money getMoneyValue() throws IllegalStateException {
		if (!this.getDatumType().equals(DatumType1Enum.MONEY)) {
			throw new IllegalStateException("Can't get value as money, it is of type " + getDatumType().name());							
		}
		return getMoney();
	}

	/**
	 * Sets the <i>money</i> value and checks that datum type is <i>money</i>.
	 * @param money the money to set
	 * @throws IllegalStateException
	 */
	@Transient
	public void setMoneyValue(Money money) throws IllegalStateException {
		if (! (this.getDatumType().equals(DatumType1Enum.NOTSET)
				|| this.getDatumType().equals(DatumType1Enum.MONEY))) {
			throw new IllegalStateException("Datum value is not quantity, it is of type " + getDatumType().name());							
		}
		setDatumType(DatumType1Enum.MONEY);
		setMoney(money);
	}

	/**
	 * Sets the <i>money</i> value and checks that datum type is <i>money</i>.
	 * @param money the money to set
	 * @throws IllegalStateException
	 */
	@Transient
	public void setMoneyValue(BigDecimal amount, Currency currencyCode) throws IllegalStateException {
		if (! (this.getDatumType().equals(DatumType1Enum.NOTSET)
				|| this.getDatumType().equals(DatumType1Enum.MONEY))) {
			throw new IllegalStateException("Datum value is not quantity, it is of type " + getDatumType().name());							
		}
		setDatumType(DatumType1Enum.MONEY);
		getMoney().setAmount(amount);
		getMoney().setCurrencyCode(currencyCode);
	}

	/**
	 * Sets the <i>money</i> value and checks that datum type is <i>money</i>.
	 * @param money the money to set
	 * @throws IllegalStateException
	 */
	@Transient
	public void setUSMoneyValue(BigDecimal amount) throws IllegalStateException {
		if (! (this.getDatumType().equals(DatumType1Enum.NOTSET)
				|| this.getDatumType().equals(DatumType1Enum.MONEY))) {
			throw new IllegalStateException("Datum value is not quantity, it is of type " + getDatumType().name());							
		}
		setDatumType(DatumType1Enum.MONEY);
		getMoney().setAmount(amount);
		getMoney().setCurrencyCode(USD);
	}

	/**
	 * Gets the <i>wage base</i> value and checks that datum type is <i>money</i>.
	 * @return the wageBase
	 * @throws IllegalStateException
	 */
	@AttributeOverrides({
	    @AttributeOverride(name="amount", column=@Column(name="wageBaseAmount")),
	    @AttributeOverride(name="currencyCode", column=@Column(name="wageBaseCurrencyCode"))
	})
	@Transient
	public Money getWageBaseValue() throws IllegalStateException {
		if (!this.getDatumType().equals(DatumType1Enum.MONEY)) {
			throw new IllegalStateException("Datum value is not quantity, it is of type " + getDatumType().name());							
		}
		return getWageBase();
	}

	/**
	 * Gets the <i>wage base</i> value and datum type to <i>money</i>.
	 * @param wageBase the wageBase to set
	 * @throws IllegalStateException
	 */
	@Transient
	public void setWageBaseValue(Money wageBase) throws IllegalStateException {
		if (!this.getDatumType().equals(DatumType1Enum.MONEY)) {
			throw new IllegalStateException("Datum value is not quantity, it is of type " + getDatumType().name());							
		}
		setWageBase(wageBase);
	}

	/**
	 * Gets the <i>income subject to tax</i> value and checks that datum type is <i>money</i>.
	 * @return the incomeSubjectToTax
	 * @throws IllegalStateException
	 */
	@AttributeOverrides({
	    @AttributeOverride(name="amount", column=@Column(name="isttAmount")),
	    @AttributeOverride(name="currencyCode", column=@Column(name="isttCurrencyCode"))
	})
	@Transient
	public Money getIncomeSubjectToTaxValue() throws IllegalStateException {
		if (!this.getDatumType().equals(DatumType1Enum.MONEY)) {
			throw new IllegalStateException("Datum value is not quantity, it is of type " + getDatumType().name());							
		}
		return getIncomeSubjectToTax();
	}

	/**
	 * Sets the <i>income subject to tax</i> value and datum type to <i>money</i>.
	 * @param incomeSubjectToTax the incomeSubjectToTax to set
	 * @throws IllegalStateException
	 */
	@Transient
	public void setIncomeSubjectToTaxValue(Money incomeSubjectToTax) throws IllegalStateException {
		if (!this.getDatumType().equals(DatumType1Enum.MONEY)) {
			throw new IllegalStateException("Datum value is not quantity, it is of type " + getDatumType().name());							
		}
		setIncomeSubjectToTax(incomeSubjectToTax);
	}

	/**
	 * Gets the <i>total hours</i> value and checks that datum type is <i>money</i>.
	 * @return the totalHours
	 * @throws IllegalStateException
	 */
	@Transient
	public Double getTotalHoursValue() throws IllegalStateException {
		if (!this.getDatumType().equals(DatumType1Enum.MONEY)) {
			throw new IllegalStateException("Datum value is not quantity, it is of type " + getDatumType().name());							
		}
		return getTotalHours();
	}

	/**
	 * Sets the <i>total hours</i> value and datum type to <i>money</i>.
	 * @param totalHours the totalHours to set
	 */
	@Transient
	public void setTotalHoursValue(Double totalHours) {
		if (!this.getDatumType().equals(DatumType1Enum.MONEY)) {
			throw new IllegalStateException("Datum value is not quantity, it is of type " + getDatumType().name());							
		}
		setTotalHours(totalHours);
	}

	/**
	 * Gets the <i>quantity</i> value and checks that datum type is <i>quantity</i>.
	 * @return the quantity
	 * @throws IllegalStateException
	 */
	@Transient
	public Double getQuantityValue() throws IllegalStateException {
		if (!this.getDatumType().equals(DatumType1Enum.QUANTITY)) {
			throw new IllegalStateException("Datum value is not quantity, it is of type " + getDatumType().name());							
		}
		return getQuantity();
	}
	
	/**
	 * Sets the <i>quantity</i> value and datum type to <i>quantity</i>.
	 * @param quantity the quantity to set
	 * @throws IllegalStateException
	 */
	@Transient
	public void setQuantityValue(Double quantity) throws IllegalStateException {
		if ( !(this.getDatumType().equals(DatumType1Enum.NOTSET)
				|| this.getDatumType().equals(DatumType1Enum.QUANTITY))) {
			throw new IllegalStateException("Datum value is not quantity, it is of type " + getDatumType().name());							
		}
		setDatumType(DatumType1Enum.QUANTITY);
		setQuantity(quantity);
	}



}