package com.intuit.sb.payroll.entity.utils;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.DatumType1Enum;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.PtdDatumType;
import com.intuit.sb.payroll.entity.model.PtdTypeEnum;

/**
 * This class contains some utility methods for dealing with period-to-date. 
 * 
 * @author gcoulombe
 */
public class PtdUtils {
	
	public static PtdDatumType newPtdDatumType(PtdTypeEnum type, Date refDate, 
			Money amount, Money istt, Money wageBase, Double totalHours) {
		
		PtdDatumType datumType = new PtdDatumType();
		
		datumType.setPtdType(type);
		datumType.setDatumType(DatumType1Enum.MONEY);
		
		datumType.setMoneyValue(amount);
		datumType.setIncomeSubjectToTaxValue(istt);
		datumType.setWageBaseValue(wageBase);
		datumType.setTotalHoursValue(totalHours);
		
		setPtdPeriod(type, refDate, datumType);
		
		return datumType;
	}

	/**
	 * Based on the given date and the period type (WEEK, MONTH, etc.), set the period start and
	 * end dates on the ptddatum object.
	 * 
	 * @param type
	 * @param refDate
	 * @param datumType
	 */
	private static void setPtdPeriod(PtdTypeEnum type, Date refDate,
			PtdDatumType datumType) {

		Calendar cal = DateUtil.dateToCalendar(refDate);
		int year = cal.get(Calendar.YEAR);

		DateRange period = getDateRangeForPeriodType(type, year);
		datumType.setPeriod(period);
	}
	
	/**
	 * Returns a {@link DateRange} for a given {@link PtdTypeEnum} in a given year.
	 * @param type
	 * @param year
	 * @return
	 */
	public static DateRange getDateRangeForPeriodType(PtdTypeEnum type, int year) {
		int endYear = year;
		int startMonth, endMonth;
		switch(type) {
			
			case QUARTER1:
				startMonth = Calendar.JANUARY;
				endMonth = Calendar.APRIL;
				break;
			case QUARTER2:
				startMonth = Calendar.APRIL;
				endMonth = Calendar.JULY;
				break;
			case QUARTER3:
				startMonth = Calendar.JULY;
				endMonth = Calendar.OCTOBER;
				break;
			case QUARTER4:
				startMonth = Calendar.OCTOBER;
				endMonth = Calendar.JANUARY;
				endYear = year + 1;
				break;
				
			case YEAR:
				endYear = year + 1;
				startMonth = Calendar.JANUARY;
				endMonth = Calendar.JANUARY;
				break;
				
			default:
				throw new UnsupportedOperationException("Unsupported PTD Type: " + type);
			
			}
		DateRange period = new DateRange(DateUtil.createDate(year, startMonth, 1), DateUtil.createDate(endYear, endMonth, 1));
		return period;
	}
	
	/**
	 * Given a date, this method will return the proper QTR boundaries.
	 * 
	 * @param effectiveDate
	 * @return a {@link DateRange}
	 */
	public static DateRange getQtrRangeForDate(Date effectiveDate) {
		Calendar calDate = DateUtil.dateToCalendar(effectiveDate);

		DateRange qtr4Range = PtdUtils.getDateRangeForPeriodType(PtdTypeEnum.QUARTER4, calDate.get(Calendar.YEAR));
		if (DateUtil.isInPeriod(effectiveDate, qtr4Range.getStartDate(), qtr4Range.getEndDate())) {
			return qtr4Range;
		}
		
		DateRange qtr3Range = PtdUtils.getDateRangeForPeriodType(PtdTypeEnum.QUARTER3, calDate.get(Calendar.YEAR));
		if (DateUtil.isInPeriod(effectiveDate, qtr3Range.getStartDate(), qtr3Range.getEndDate())) {
			return qtr3Range;
		}
	
		DateRange qtr2Range = PtdUtils.getDateRangeForPeriodType(PtdTypeEnum.QUARTER2, calDate.get(Calendar.YEAR));
		if (DateUtil.isInPeriod(effectiveDate, qtr2Range.getStartDate(), qtr2Range.getEndDate())) {
			return qtr2Range;
		}

		DateRange qtr1Range = PtdUtils.getDateRangeForPeriodType(PtdTypeEnum.QUARTER1, calDate.get(Calendar.YEAR));
		if (DateUtil.isInPeriod(effectiveDate, qtr1Range.getStartDate(), qtr1Range.getEndDate())) {
			return qtr1Range;
		}
		
		throw new IllegalStateException("Effective date does not fall into any period");
	}
	
	/**
	 * Returns the QTR that this date falls into as represented by the {@link PtdTypeEnum};
	 * @param effectiveDate
	 * @return
	 */
	public static PtdTypeEnum getPtdTypeForDate(Date effectiveDate) {
		Calendar calDate = DateUtil.dateToCalendar(effectiveDate);
        // Q1 --> Jan 1 - Mar 31
        DateRange q1 = getDateRangeForPeriodType(PtdTypeEnum.QUARTER1, calDate.get(Calendar.YEAR));
        if (DateUtil.isInPeriod(calDate.getTime(), q1.getStartDate(), q1.getEndDate())) {
        	return PtdTypeEnum.QUARTER1;
        }
        // Q2 --> Apr 1 - Jun 30
        DateRange q2 = getDateRangeForPeriodType(PtdTypeEnum.QUARTER2, calDate.get(Calendar.YEAR));
        if (DateUtil.isInPeriod(calDate.getTime(), q2.getStartDate(), q2.getEndDate())) {
        	return PtdTypeEnum.QUARTER2;
        }
        // Q3 --> Jul 1 - Sep 30
        DateRange q3 = getDateRangeForPeriodType(PtdTypeEnum.QUARTER3, calDate.get(Calendar.YEAR));
        if (DateUtil.isInPeriod(calDate.getTime(), q3.getStartDate(), q3.getEndDate())) {
        	return PtdTypeEnum.QUARTER3;
        }
        // Q4 --> Oct 1 - Dec 31
        DateRange q4 = getDateRangeForPeriodType(PtdTypeEnum.QUARTER4, calDate.get(Calendar.YEAR));
        if (DateUtil.isInPeriod(calDate.getTime(), q4.getStartDate(), q4.getEndDate())) {
        	return PtdTypeEnum.QUARTER4;
        }
        
        //Should never get here!
        return PtdTypeEnum.NOTSET;
	}

	/**
	 * Get the calendar quarter today's date belongs to.
	 * Then return a list of {@link DateRange} each corresponding to a quarter before this quarter.
	 * If today is between Jan 1st and March 31st (Q1), the method will return empty list.
	 * @return
	 */
	public static List<DateRange> getQuartersBeforeCurrentQuarter() {
		//FIXME consider using the System Date that can be set for testing.
        Calendar today = DateUtil.getNowCalendar();
        return getQuartersPriorToGivenDate(today);
	}
	
	/**
	 * Get the calendar quarter prior to the calendar quarter the given calendar object belongs to.
	 * Then return a list of {@link DateRange} each corresponding to a quarter before this quarter.
	 * If the date is between Jan 1st and March 31st (Q1), the method will return empty list.
	 * @return
	 */
	public static List<DateRange> getQuartersPriorToGivenDate(Calendar calendar) {
        if (calendar == null) {
        	return null;
        }
        
        List<DateRange> rangeList = new ArrayList<DateRange>();
        DateRange range = PtdUtils.getQtrRangeForDate(calendar.getTime());
        // Q1 --> Jan 1 - Mar 31
        DateRange q1 = getDateRangeForPeriodType(PtdTypeEnum.QUARTER1, calendar.get(Calendar.YEAR));
        // Q2 --> Apr 1 - Jun 30
        DateRange q2 = getDateRangeForPeriodType(PtdTypeEnum.QUARTER2, calendar.get(Calendar.YEAR));
        // Q3 --> Jul 1 - Sep 30
        DateRange q3 = getDateRangeForPeriodType(PtdTypeEnum.QUARTER3, calendar.get(Calendar.YEAR));
        // Q4 --> Oct 1 - Dec 31
        DateRange q4 = getDateRangeForPeriodType(PtdTypeEnum.QUARTER4, calendar.get(Calendar.YEAR));
        
        if (range.equals(q2)){
        	rangeList.add(q1);
        } else if (range.equals(q3)) {
        	rangeList.add(q1);
        	rangeList.add(q2);
        } else if (range.equals(q4)) {
        	rangeList.add(q1);
        	rangeList.add(q2);
        	rangeList.add(q3);
        }
        
        return rangeList;
	}
	
	
}
