package com.intuit.sb.core.rules.remote;

import java.util.List;
import java.util.Map;

import javax.ejb.Remote;

/*
 * @author Seegler Ittyavirah
 */
@Remote
public interface RuleService {
	/*
	 * Property key used to pass the global variable values to the rules runtime.
	 */
	public static final String GLOBAL_VARIABLES_PROPERTY = "GLOBAL_VARIABLES_PROPERTY";

	/**
	 * 
	 * Usage of RuleService is as follows:<br/>
	 * 
	 * 1. Getting remote RuleService instance in the client application<br/>
	 * 
	 * If the client is an ejb3.0 bean, then use ejb3.0 Dependancy Injection
	 * 
	 * @EJB(name="RuleService") RuleService service;
	 * 
	 * Use JNDI lookup from a in-container client application
	 * 
	 * InitialContext ctx = new InitialContext(); 
	 * RuleService service=(RuleService) ctx.lookup("RuleService");
	 * 
	 * Use JNDI lookup from a standalone client application
	 * 
	 * InitialContext ctx = new InitialContext(); 
	 * RuleService service=(RuleService) ctx.lookup("RuleService"); 
	 * 
	 * This would require jndi.properties file to be present in the appilcation classpath.
	 * 
	 * If jndi.properties file is not present, it is possible to set the jndi
	 * properties programatically as follows:
	 * 
	 * Hashtable<String, String> props = new Hashtable<String, String>();
	 * props.put("java.naming.factory.initial",
	 * "org.jnp.interfaces.NamingContextFactory");
	 * props.put("java.naming.provider.url", "jnp://localhost:1099");
	 * props.put("java.naming.factory.url.pkgs",
	 * "org.jboss.naming:org.jnp.interfaces"); 
	 * InitialContext ctx = new InitialContext(props); 
	 * RuleService service=(RuleService) ctx.lookup("RuleService");
	 * 
	 * RuleServiceClient provides a convenient wrapper around JNDI lookup code.
	 * 
	 * From in-container client, 
	 * RuleService service=RuleServiceClient.lookupRuleService("RuleService"); 
	 * 
	 * From standalone client, 
	 * RuleService service=RuleServiceClient.lookupRuleService("RuleService","jnp:localhost:1099");
	 * 
	 * 
	 * 2. Invoke executeRules API
	 * 
	 * String package="com.example.mypackage/LATEST"; 
	 * List factList=new ArrayList(); 
	 * Fact1 fact1=new Fact1(); 
	 * Fact2 fact2=new Fact2();
	 * factList.add(fact1); 
	 * factList.add(fact2); 
	 * try{ 
	 * 	List resultList=service.executeRules(package,factList,null);
	 * }catch(RuleException e){ 
	 * 	//handle exception 
	 * }
	 * 
	 * @param packageName
	 *            name of the ruleset package to execute. packageName has two
	 *            parts. First part is the name of the ruleset package followed
	 *            by '/' and the snapshort version of the package in BRMS.
	 *            RuleService will try to load this snapshot package from BRMS
	 *            and cache it.
	 * 
	 * @param inputs
	 *            list of fact objects to be used in ruleset execution. All fact
	 *            objects should be serializable. Fact classes should be
	 *            available in the RuleService application classpath.
	 * @param properties
	 *            additional properties used in the rule session. Supported
	 *            properties are:
	 *            <ul>
	 *            <li> RuleService.GLOBAL_VARIABLES_PROPERTY: Set this property 
	 *            with the values of global variables to be used in the ruleset 
	 *            execution. Value for this property is of type Map<String,Object>
	 *             </li>
	 *            </ul>
	 * 
	 * @return list of result objects
	 * 
	 * @throws RulesException
	 *             all exceptions raised from rule engine runtime are wrapped in
	 *             RuleException and thrown to the caller
	 * @throws IllegalArgumentException
	 * 				thrown if packageName or inputs arguments are null.
	 */
	@SuppressWarnings("unchecked")
	public List executeRules(String packageName, List inputs, Map properties)
			throws RuleException;
}
