package com.intuit.sb.payroll.capability.compensation.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.common.EntityService;
import com.intuit.sb.payroll.entity.deductions.DeductionEAO;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.Deduction;
import com.intuit.sb.payroll.entity.model.DeductionDtl;
import com.intuit.sb.payroll.entity.model.DeductionTypeEnum;
import com.intuit.sb.payroll.entity.model.PayrollItemDtl;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * Currently handles relationships between compensation and deductions like CashTips and related Offsets.
 * $Author$ 
 * $Revision$
 * $DateTime$
 *
 */
public class LinkedPayrollItemHandler {
	 private static DeductionEAO sDeductionEAO = ServiceLocator.getInstance().getService(DeductionEAO.JNDI_NAME);
   
	static void handleNewCompensationDtl(EntityService entityService, CompensationDtl compensationDtl, Worker worker) throws ValidationException{
		//TipsCash requires an associated deduction to balance out the addition, since
		//this type of compensation does not involve any money moving between employer and employee.
		if (compensationDtl.getCompensationType() == CompensationTypeEnum.TIPSCASH) {
			// Create a related deduction.
	        DeductionDtl cashTipsOffsetDtl = createDeductionDtl(DeductionTypeEnum.CASHTIPSOFFSET);
	        cashTipsOffsetDtl = entityService.makePersistent(cashTipsOffsetDtl);
	        worker.addDeductionDtl(cashTipsOffsetDtl);
	        compensationDtl.setLinkedPayrollItemDtl(cashTipsOffsetDtl);
		} else if (compensationDtl.getCompensationType() == CompensationTypeEnum.HOURLYOVT || 
				compensationDtl.getCompensationType() == CompensationTypeEnum.HOURLYDBL) {
			PayrollItemDtl linkedPayrollItemDtl = compensationDtl.getLinkedPayrollItemDtl();
			boolean hourlyExists = hourlyCompensationExists(worker, linkedPayrollItemDtl);
			if (!hourlyExists) {
				throw new RuntimeException("Adding Overtime or DoubleTime before hourly exists.");
			}
		}
	}

	private static boolean hourlyCompensationExists(Worker worker, PayrollItemDtl linkedPayrollItemDtl) {
		Set<CompensationDtl> compensationDtls = worker.getCompensationDtls();
		long itemId = linkedPayrollItemDtl.getId();
		for (CompensationDtl compensationDtl : compensationDtls) {
			if (compensationDtl.getId() == itemId && compensationDtl.getCompensationType() == CompensationTypeEnum.HOURLYREG) {
				return true;
			}
		}
		return false;
	}

	/**
	 * This utility method will create a DeductionDtl object with the specified DeductionTypeEnum and some
	 *   default values
	 * 
	 * @param deductionTypeEnum The DeductionType to create
	 * @return A DeductionDtl object
	 */
    static DeductionDtl createDeductionDtl(DeductionTypeEnum deductionTypeEnum) {
    	Deduction deduction = sDeductionEAO.findDeduction(deductionTypeEnum);

    	DeductionDtl deductionDtl = new DeductionDtl();

    	deductionDtl.setPayrollItem(deduction);
    	deductionDtl.setDeductionType(deductionTypeEnum);
    	// create an empty container, the actual value depends on the value from the CashTips CompensationDtl item
    	Datum datum = new Datum();
    	deductionDtl.setRate(datum);
    	deductionDtl.setDescription(deduction.getDescription());
    	deductionDtl.setName(deduction.getName());
    	return deductionDtl;
    }

	public static List<CompensationDtl> sortCompensationsToAdd(List<CompensationDtl> compensationsToAdd) {
		//Hourly comes first, then Salary - these are potentially references as linked compensations.
		// Overtime and Doubletime may refer to them for the base rates.
		if (compensationsToAdd.size() > 1) {
			List<CompensationDtl> results = new ArrayList<CompensationDtl>();
			//Consider a while loop for multiple salary and hourly while(findby != null)
			CompensationDtl hourly = findCompensationByType(compensationsToAdd, CompensationTypeEnum.HOURLYREG);
			if (hourly != null) {
				results.add(hourly);
				compensationsToAdd.remove(hourly);
			}
			
			CompensationDtl salary = findCompensationByType(compensationsToAdd, CompensationTypeEnum.SALARYREG);
			if (salary != null) {
				results.add(salary);
				compensationsToAdd.remove(salary);
			}
		
			//Add the rest in whatever order?
			results.addAll(compensationsToAdd);
			return results;
		} 
		
		//nothing to sort
		return compensationsToAdd;
	}

	private static CompensationDtl findCompensationByType(List<CompensationDtl> compensationsToAdd, CompensationTypeEnum type) {
		for (CompensationDtl compensationDtl : compensationsToAdd) {
			if (compensationDtl.getCompensationType() == type) {
				return compensationDtl;
			}
		}
		// nothing found.
		return null;
	}
}
