package com.intuit.sb.payroll.capability.employer.impl;

import java.util.List;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.validation.AbstractEntityDataValidator;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.model.BankAccount;
import com.intuit.sb.payroll.entity.model.BankAccountTypeEnum;

/**
 * Validates fields in the BankAccountValidator entity that have values. Fields with no
 * values are ignored.
 * 
 * This class does not ensure data integrity or completeness.
 * 
 * @see BankAccountValidator
 * 
 */
public class BankAccountValidator extends AbstractEntityDataValidator<BankAccount> {
	private static BankAccountValidator sInstance = new BankAccountValidator();

	private BankAccountValidator() {
	}

	public static BankAccountValidator getInstance() {
		return sInstance;
	}

	@Override
	public void validateEntityCollection(List<BankAccount> employerCollection) throws ValidationException {
		for (BankAccount bankAccount : employerCollection) {
			validateEntityImpl(bankAccount);
		}
	}

	@Override
	public void validateEntity(BankAccount bankAccount) throws ValidationException {
		validateEntityImpl(bankAccount);
	}
	
	/**
	 * 	<ul>Validation logic used for validating Bank Accounts in Gemini:
	 * 		<li>BankAccount object must be NON-NULL
	 * 		<li>Account Type Must be NON-NULL and either Checking or Savings
	 * 		<li>Bank Name, Account Number and Routing Number must be NON-NULL
	 * 		<li>Routing Number must follow the validation logic prescribed by ABA detailed below
	 * 	</ul>
	 * @param bankAccount - Bank Account entity object
	 * @throws ValidationException
	 */
	public void validateEntityImpl(BankAccount bankAccount) throws ValidationException {
		PayrollParamValidator.checkIsNotNull(bankAccount, "Bank Account");
		PayrollParamValidator.checkIsNotNull(bankAccount.getAccountType(), "Account Type");
		validateAccountType(bankAccount);
		PayrollParamValidator.checkIsNotNullOrEmptyString(bankAccount.getBankName(), "Bank Name");
		PayrollParamValidator.checkIsNotNullOrEmptyString(bankAccount.getAccountNo(), "Account Number");
		PayrollParamValidator.checkIsNotNullOrEmptyString(bankAccount.getRoutingNo(), "Routing Number");
		validateRoutingNumberFormat(bankAccount.getRoutingNo());
		PayrollParamValidator.checkIsPositive(Long.parseLong(bankAccount.getAccountNo()), "Account Number");
		PayrollParamValidator.checkIsPositive(Long.parseLong(bankAccount.getRoutingNo()), "Routing Number");
	}
	
	/**
	 * Bank Account Type must be Checking or Savings only
	 * @param type
	 * @throws ValidationException
	 */
	private void validateAccountType(BankAccount account) throws ValidationException {
		if (account.getAccountType() != BankAccountTypeEnum.CHECKING && 
				account.getAccountType() != BankAccountTypeEnum.SAVINGS) {
			throw new ValidationException("Bank Account Type other than CHECKING or SAVINGS not allowed.");
		}
	}
	
	/**
	 * <ul>ABA Routing Number format specifies that:
	 * <li>A valid Routing Number must consist exactly 9 digits
	 * <li>The Routing Number must confirm to the following Algorithm:<br/>
	 * If the number is represented by 9 alphabets "abcdefghi", then we calculate as: <br/>
	 * z = (3*a)+(7*b)+c+(3*d)+(7*e)+f+(3*g)+(7*h)+i
	 * <li>The number z obtained by the above calculation must be<br/>
	 * A non-zero positive number and <br/>
	 * Completely divisible by 10 (i.e. it should end with a zero).
	 * @param routingNumber
	 */
	private void validateRoutingNumberFormat(String routingNumber) throws ValidationException {
		if (routingNumber.length() != 9) {
			throw new ValidationException("Valid ABA Routing Number must be 9 digits long");
		}
		
		try {
			Integer.parseInt(routingNumber);
		} catch (NumberFormatException e) {
			throw new ValidationException("Valid ABA Routing Number must be a positive 9 digit number" +
					" without any non-numeric characters");
		}
		
		int[] digits = new int[routingNumber.length()];
		for (int i=0; i<routingNumber.length(); i++) {
			digits[i] = Integer.parseInt(routingNumber.substring(i, i+1));
		}
		
		// multiplier array for Routing Number Algorithm
		int[] multipliers = {3, 7, 1, 3, 7, 1, 3, 7, 1};
		int total = 0;
		
		for (int i = 0; i < digits.length; i++) {
			int digit = digits[i];
			int multiplier = multipliers[i];
			total = total + (digit*multiplier);
		}
		
		if (total == 0 || (total%10 != 0)) {
			throw new ValidationException("Invalid Routing Number specified");
		}
	}
}
