package com.intuit.sb.payroll.capability.employer.impl;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;
import javax.persistence.OptimisticLockException;

import org.apache.log4j.Logger;
import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.capability.employer.WorkerLocation;
import com.intuit.sb.payroll.capability.task.TaskManager;
import com.intuit.sb.payroll.capability.util.DateRangeUtil;
import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponent;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.PayrollResourceManager;
import com.intuit.sb.payroll.common.ResourceKeyConstants;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.common.WorkflowEventType;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.common.EntityService;
import com.intuit.sb.payroll.entity.employer.EmployerEAO;
import com.intuit.sb.payroll.entity.employer.WorkLocationEAO;
import com.intuit.sb.payroll.entity.model.BankAccount;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.EmployerContactPerson;
import com.intuit.sb.payroll.entity.model.HistPaycheck;
import com.intuit.sb.payroll.entity.model.HistPaycheckLineItem;
import com.intuit.sb.payroll.entity.model.Jurisdiction;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PaycheckLineItem;
import com.intuit.sb.payroll.entity.model.PaycheckLineItemSortSequenceEnum;
import com.intuit.sb.payroll.entity.model.Payment;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.utils.PtdUtils;
import com.intuit.sb.payroll.entity.worker.WorkerEAO;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.message.PayrollMessageService;
import com.intuit.sb.payroll.us.capability.address.impl.USPostalAddressImpl;

/**
 * Implementation of the Employer Management functions
 *
 * $Author$
 * $Revision$
 * $DateTime$
 */
@Stateless(name = "EmployerComponent")
@SecurityDomain("BHPIAM")
@RolesAllowed( { "Authenticated" })
public class EmployerComponentBean implements EmployerComponent {
	@EJB
	EmployerEAO employerEAO;

	@EJB
	WorkerEAO workerEAO;

	@EJB
	WorkLocationEAO workLocationEAO;

	@EJB
	WorkerComponent workerComponent;

	@EJB
	TaskManager taskManager;

	@EJB
	PayrollMessageService msgService;

	@EJB
	EntityService entityService;


	private static final Logger sLogger = PayrollLoggerFactory.makeLogger(EmployerComponentBean.class);
	/**
	 * Provides a list of Jurisdiction for a given WorkLocation. The WorkLocation Address to Jurisdiction Mapping is
	 * currently implemented in Jurisdiction resolver. At a later point in time this will be modified to use some
	 * external service.
	 *
	 * @param workLocation
	 * @return List of Jurisdiction that applies to the work location
	 * @throws PayrollServiceFailureException if something unexpected occurs.
	 */
	public List<Jurisdiction> determineAllJurisdictionsWorkLocation(WorkLocation workLocation) throws ValidationException {
		if (null == workLocation) {
			throw new ValidationException("WorkLocation is <null>");
		}
		USPostalAddress address = new USPostalAddressImpl();
		AddressConvertor.getUSFromGlobal(workLocation.getTaxResidenceAddress(), address);

		List<Jurisdiction> jurisdictions = new ArrayList<Jurisdiction>();

		List<Jurisdiction> jurisdictionsForAddressList = new JurisdictionResolver().resolveJurisdictionsForAddress(address);

		for (Jurisdiction jurisdiction : jurisdictionsForAddressList) {
			jurisdictions.add(jurisdiction);
		}

		return jurisdictions;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.intuit.sb.payroll.capability.employer.EmployerComponent#saveEmployer(java.lang.String)
	 */
	public Employer createEmployer(Employer employer) throws ValidationException, ServiceFailureException {

		// check whether an Employer already exists in the system for this user credentials
		List<Employer> employerList = employerEAO.findAll();
		// If there are no employers in the DB, create one.
		if (employerList == null || employerList.size() == 0) {
			// Validate param
			PayrollParamValidator.checkIsNotNull(employer, "employer");

			try {

				// Perform validation, then save if good else throw ValidationException
				EmployerValidator.getInstance().validateEntity(employer);

				employer = employerEAO.makePersistent(employer);
				// Setup the related system data etc. for the new employer
				// created
				EmployerSeedData.setupEmployerDataInSystem();

				dispatchWorkflowEvent(WorkflowEventType.CREATE_EMPLOYER, employer);
				return employer;
			} catch (Exception e) {
				throw new ServiceFailureException(e);
			}
		} else {
			throw new ValidationException("Employer already exists in the system for the user context");
		}
	}

	/**
     * Sink for workflow events in this component.  Handles side-effects that are germain to this component, then
     * propagates the event to the Messaging infrastructure.
	 *
	 * @param workflowEventType
	 * @param employer
	 * @throws ServiceFailureException
	 */
	private void dispatchWorkflowEvent(WorkflowEventType workflowEventType, Employer employer) throws ServiceFailureException {
		sLogger.debug("Event Dispatched : "+workflowEventType);
		//Add/Update/Close tasks we control
		try {
			switch (workflowEventType) {
				case CREATE_EMPLOYER:
					// Create the initial employer tasks
					EmployerComponentTasks.createCompanySetupTask(taskManager, employer);
					EmployerComponentTasks.createAddWorkLocationTask(taskManager, employer);
					EmployerComponentTasks.createPayrollScheduleTask(taskManager, employer);
					break;
				case UPDATE_EMPLOYER:

					break;
				case UPDATE_EMPLOYERID:

					break;
				default:
					break;
			}
		} catch (Exception e) {
			throw new ServiceFailureException(e);
		}
		//Publish event to interested parties
		msgService.dispatchEmployerEvent(workflowEventType, employer);
	}

	

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.employer.EmployerComponent#handlePayrollScheduleEvent(com.intuit.sb.payroll.message.WorkflowEventType)
	 */
	public void handlePayrollScheduleEvent(WorkflowEventType workflowEventType) throws ServiceFailureException {
		if (WorkflowEventType.ADD_PAYROLL_SCHEDULE == workflowEventType) {
			EmployerComponentTasks.handlePayrollScheduleCreatedEvent(taskManager, getEmployer());
		} else if (WorkflowEventType.DELETE_PAYROLL_SCHEDULE == workflowEventType) {
			EmployerComponentTasks.createPayrollScheduleTask(taskManager, getEmployer());
		}
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.intuit.sb.payroll.capability.employer.EmployerComponent#saveEmployer(java.lang.String)
	 */
	public Employer saveEmployer(Employer employer) throws ValidationException, ServiceFailureException {
		try {
			PayrollParamValidator.checkIsNotNull(employer, "employer");
			EmployerValidator.getInstance().validateEntity(employer);

			if (employerEAO.notPersistent(employer)) {
				employer = employerEAO.makeManaged(employer);
			}
			dispatchWorkflowEvent(WorkflowEventType.UPDATE_EMPLOYER, employer);
		} catch (ValidationException ve) {
			throw ve;
		} catch (ServiceFailureException sfe) {
			throw sfe;
		} catch (Exception e) {
			throw new ServiceFailureException(e);
		}
		return employer;
	}

	

	/*
	 * (non-Javadoc)
	 *
	 * @see com.intuit.sb.payroll.capability.employer.EmployerComponent#getEmployer(java.lang.String)
	 */
	public Employer getEmployer() {
		return employerEAO.findCurrentEmployer();
	}

	// Work Location Functions

	public List<WorkLocation> findAllWorkLocations() {
		List<WorkLocation> locations = workLocationEAO.findAll();
		for (WorkLocation workLocation : locations) {
			int workerCount = workLocationEAO.getWorkerCountForWorkLocation(workLocation);
			workLocation.setWorkerCount(workerCount);
		}
		return locations;
	}

	public WorkLocation findWorkLocation(Long uniqueId) throws ServiceFailureException {
		WorkLocation workLocation = null;
		try {
			workLocation = workLocationEAO.findById(uniqueId);
			if (null != workLocation) {
				int workerCount = workLocationEAO.getWorkerCountForWorkLocation(workLocation);
				workLocation.setWorkerCount(workerCount);
			} else {
				throw new ServiceFailureException(PayrollResourceManager.getInstance().getString(
						ResourceKeyConstants.ID_NOT_FOUND_EXCEPTION)
						+ uniqueId.toString());
			}
		} catch (Exception e) {
			throw new ServiceFailureException(e);
		}

		return workLocation;
	}

	public List<WorkLocation> addWorkLocations(List<WorkLocation> workLocations) throws ValidationException, ServiceFailureException {
		WorkLocationValidator.getInstance().validateEntityCollection(workLocations);

		Employer employer = employerEAO.findCurrentEmployer();
		List<Jurisdiction> existingJurisdictions = determineAllJurisdictions();
		for (WorkLocation workLocation : workLocations) {
			employer.addWorkLocation(workLocation);
		}

		if (workLocations.size() > 0){
			EmployerComponentTasks.handleWorkLocationCreatedEvent(taskManager, employer);
		}

		List<Jurisdiction> currentJurisdictions = determineAllJurisdictions();
		WorkLocationWorkFlow.addNewJurisdictions(employer, existingJurisdictions, currentJurisdictions);

		return findAllWorkLocations();
	}

	public Employer addWorkLocation(WorkLocation workLocation) throws ValidationException, ServiceFailureException {
		WorkLocationValidator.getInstance().validateEntity(workLocation);

		Employer employer = employerEAO.findCurrentEmployer();
		List<Jurisdiction> existingJurisdictions = determineAllJurisdictions();
		employer.addWorkLocation(workLocation);

		EmployerComponentTasks.handleWorkLocationCreatedEvent(taskManager, employer);

		List<Jurisdiction> currentJurisdictions = determineAllJurisdictions();
		WorkLocationWorkFlow.addNewJurisdictions(employer, existingJurisdictions, currentJurisdictions);

		return employer;
	}

	public List<WorkLocation> updateWorkLocations(List<WorkLocation> workLocations) throws ValidationException, ServiceFailureException {
		WorkLocationValidator.getInstance().validateEntityCollection(workLocations);
		List<Jurisdiction> existingJurisdictions = determineAllJurisdictions();

		for (WorkLocation workLocation : workLocations) {
			workLocation = workLocationEAO.makeManaged(workLocation);
		}
		Employer employer = employerEAO.findCurrentEmployer();

		List<Jurisdiction> newJurisdictions = determineAllJurisdictions();
		WorkLocationWorkFlow.updateJurisdictions(employer, existingJurisdictions, newJurisdictions);

		return findAllWorkLocations();
	}

	public List<WorkLocation> deleteWorkLocations(List<WorkLocation> workLocations) throws ValidationException, ServiceFailureException {
		WorkLocationValidator.getInstance().validateEntityCollection(workLocations);

		// The rule is that you can delete all you want except the last one.
		int workLocationCount = workLocationEAO.getWorkLocationCount();
		if (workLocations.size() >= workLocationCount) {
			throw new ValidationException("Cannot delete all Work Locations");
		}

		Employer employer = employerEAO.findCurrentEmployer();
		List<Jurisdiction> existingJurisdictions = determineAllJurisdictions();
		for (WorkLocation workLocation : workLocations) {
			// Can't delete WL if the handle if it's not saved
			if (workLocation.getId() == 0) {
				throw new ValidationException("Cannot delete an unsaved WorkLocation");
			}
			// Cannot delete a WL if there are workers associated with it.
			int workersCount = workLocationEAO.getWorkerCountForWorkLocation(workLocation);
			if (workersCount > 0) {
				throw new ValidationException("Cannot delete a Work Location with Workers associated with it.");
			}
			employer.removeWorkLocation(workLocation);
			workLocationEAO.makeTransient(workLocation);
		}

		List<Jurisdiction> newJurisdictions = determineAllJurisdictions();
		WorkLocationWorkFlow.deleteJurisdictions(employer, existingJurisdictions, newJurisdictions);

		return findAllWorkLocations();
	}

	public List<Worker> assignWorkLocationsToWorkers(WorkerLocation[] workerLocations)
		throws WorkflowException, ValidationException, ServiceFailureException {

	    List<Worker> workerList = new ArrayList<Worker>();

		for (WorkerLocation map : workerLocations) {
			Long workerId = map.workerId;
			Long locationId = map.workLocationId;

			Worker worker = null;
			WorkLocation workLocation = null;
			if (workerId != null) {
				worker = workerEAO.findById(workerId);
				if (null == worker) {
					throw new ServiceFailureException("No Worker found for id.");
				}
			} else {
				throw new ValidationException("WorkerId cannot be null");
			}

			if (locationId != null) {
				workLocation = workLocationEAO.findById(locationId);
				if (null == workLocation) {
					throw new ServiceFailureException("No WorkLocation found for id.");
				}
			} // Location can be empty

			workerList.add(workerComponent.assignWorkLocationToWorker(workLocation, worker));
		}
		return workerList;
	}

	/**
	 * {@inheritDoc}
	 * @throws ServiceFailureException
	 */
	public List<Jurisdiction> determineAllJurisdictions() throws ServiceFailureException {

		List<Jurisdiction> jurisdictions = new ArrayList<Jurisdiction>();

		// find jurisdictions for the employer's worklocations
		Employer employer = employerEAO.findCurrentEmployer();
		Set<WorkLocation> worklocations = employer.getWorkLocations();
		for (WorkLocation workLocation : worklocations) {
			try {
				jurisdictions = getUniqueJurisdictions(workLocation, jurisdictions);
			} catch (ValidationException e) {
				throw new ServiceFailureException("Internal error determining Jurisdictions");
			}
		}

		return jurisdictions;
	}

	/**
	 * Given a WorkLocation and an existing List of Jurisdiction objects, get the Jurisdictions associated with the
	 * WorkLocation given - add the ones not already present in the list to the input list and return the updated list
	 *
	 * @param workLocation
	 * @param jurisdictions
	 * @return
	 * @throws ValidationException
	 */
	private List<Jurisdiction> getUniqueJurisdictions(WorkLocation workLocation, List<Jurisdiction> jurisdictions) throws ValidationException {
		List<Jurisdiction> list = determineAllJurisdictionsWorkLocation(workLocation);
		for (Jurisdiction jurisdiction : list) {
			if (!jurisdictions.contains(jurisdiction)) {
				jurisdictions.add(jurisdiction);
			}
		}
		return jurisdictions;
	}

	// Contact Person Functions

	public Set<EmployerContactPerson> getEmployerContactPersons() {
		Employer employer = employerEAO.findCurrentEmployer();

		Set<EmployerContactPerson> employerContactPersons = employer.getEmployerContactPersons();

		return employerContactPersons;
	}

	public EmployerContactPerson findEmployerContactPerson(Long uniqueId) throws ServiceFailureException {
		EmployerContactPerson employerContactPerson = null;
			Employer employer = employerEAO.findCurrentEmployer();

			Set<EmployerContactPerson> employerContactPersons = employer.getEmployerContactPersons();
			for (EmployerContactPerson contact : employerContactPersons) {
				Long tempId = new Long(contact.getId());
				if (tempId.equals(uniqueId)) {
					employerContactPerson = contact;
					return employerContactPerson;
				}
			}
			throw new ServiceFailureException(PayrollResourceManager.getInstance().getString(
					ResourceKeyConstants.ID_NOT_FOUND_EXCEPTION)
					+ uniqueId.toString());
	}

	public Set<EmployerContactPerson> addEmployerContactPersons(List<EmployerContactPerson> employerContactPersons)
			throws ValidationException, ServiceFailureException {
		EmployerContactPersonValidator.getInstance().validateEntityCollection(employerContactPersons);
		Employer employer = employerEAO.findCurrentEmployer();

		for (EmployerContactPerson contact : employerContactPersons) {
			employer.addEmployerContactPerson(contact);
		}

		// Save employer
		employer = saveEmployer(employer);

		// return EmployerContactPerson set
		Set<EmployerContactPerson> employerContactPersonSet = employer.getEmployerContactPersons();
		return employerContactPersonSet;
	}

	public Set<EmployerContactPerson> updateEmployerContactPersons(
			List<EmployerContactPerson> updateEmployerContactPersons) throws ValidationException, ServiceFailureException {
		EmployerContactPersonValidator.getInstance().validateEntityCollection(updateEmployerContactPersons);
		Employer employer = employerEAO.findCurrentEmployer();

		Set<EmployerContactPerson> currentEmployerContactPersons = employer.getEmployerContactPersons();
		List<EmployerContactPerson> currList = new ArrayList<EmployerContactPerson>(currentEmployerContactPersons
				.size());
		for (EmployerContactPerson employerContactPerson : currentEmployerContactPersons) {
			currList.add(employerContactPerson);
		}

		List<EmployerContactPerson> listToUpdate = new ArrayList<EmployerContactPerson>(updateEmployerContactPersons
				.size());
		for (EmployerContactPerson employerContactPerson : updateEmployerContactPersons) {
			listToUpdate.add(employerContactPerson);
		}

		/*
		 * For each element in the update list find a corresponding element in the current list (flag error, if not
		 * found) and update in place.
		 */
		for (EmployerContactPerson contact : listToUpdate) {
			// assume guilty until proven innocent
			Boolean notFound = true;
			Boolean staleVersion = true;

			for (EmployerContactPerson curContact : currList) {
				// TODO we need a common (and better way) of checking object
				// identity
				if (contact.getId() == curContact.getId()) {
					notFound = false;
					if (contact.getVersion() == curContact.getVersion()) {
						staleVersion = false;
						// to maintain entity identity we should only copy the
						// payload
						contact.contentCopy(curContact);
						break;
					}
				}
			}

			if (notFound) {
				// exception
				throw new OptimisticLockException(
						"Updated element not found in the current element list. Must have been deleted by another user.");
			} else if (staleVersion) {
				// exception
				throw new OptimisticLockException("Updated element is stale. Data refresh is needed.");
			}
		}

		// Save employer
		employer = saveEmployer(employer);

		// return EmployerContactPerson set
		Set<EmployerContactPerson> employerContactPersons = currentEmployerContactPersons;
		return employerContactPersons;
	}

	public void deleteEmployerContactPersons(List<EmployerContactPerson> employerContactPersons)
			throws ValidationException, ServiceFailureException {
		EmployerContactPersonValidator.getInstance().validateEntityCollection(employerContactPersons);

		Employer employer = employerEAO.findCurrentEmployer();

		// The rule is that you can delete all you want except the last one.
		Set<EmployerContactPerson> currentEmployerContactPersons = employer.getEmployerContactPersons();

		if (employerContactPersons.size() > 0
				&& (employerContactPersons.size() == currentEmployerContactPersons.size())) {
			throw new ValidationException("Cannot delete all contact persons");
		}

		else {
			for (EmployerContactPerson employerContactPerson : employerContactPersons) {
				employer.removeEmployerContactPerson(employerContactPerson);
			}
			saveEmployer(employer);
		}
	}

	public void deleteAllContactPersons() throws ValidationException, ServiceFailureException {

		Employer employer = employerEAO.findCurrentEmployer();

		// Creating two lists to avoid concurrent modification errors.
		Set<EmployerContactPerson> employerContactPersons = employer.getEmployerContactPersons();
		List<EmployerContactPerson> listToDelete = new ArrayList<EmployerContactPerson>(employerContactPersons.size());
		for (EmployerContactPerson employerContactPerson : employerContactPersons) {
			listToDelete.add(employerContactPerson);
		}

		for (EmployerContactPerson employerContactPerson : listToDelete) {
			employer.removeEmployerContactPerson(employerContactPerson);
		}
		saveEmployer(employer);
	}

    /**
     * Add the given Bank Accounts to this company.
     * @param bankAccounts
     * @return an array containing all the added BankAccount objects
     */
    public BankAccount[] addBankAccounts(BankAccount[] bankAccounts)
    		throws ServiceFailureException, ValidationException {

    	BankAccountValidator.getInstance().validateEntityCollection(Arrays.asList(bankAccounts));
    	Employer employer = employerEAO.findCurrentEmployer();
    	List<BankAccount> bankAccountList = new ArrayList<BankAccount>();

		for (BankAccount bankAccount : bankAccounts) {
			bankAccount = entityService.makeManaged(bankAccount);
			employer.addBankAccount(bankAccount);
			bankAccountList.add(bankAccount);
		}

		// Save employer
		employer = saveEmployer(employer);

		//Dispatch the message
		msgService.dispatchEmployerBankAccountEvent(WorkflowEventType.ADD_EMPLOYER_BANKACCOUNT, bankAccountList);

		// now get the persisted bank account information and send it back as an array
		Set<BankAccount> accounts = employer.getBankAccounts();
		return accounts.toArray(new BankAccount[accounts.size()]);
    }

	/*
	 * (non-Javadoc)
	 *
	 * @see com.intuit.sb.payroll.capability.employer.EmployerComponent#getBankAccounts
	 */
    public BankAccount[] getBankAccounts()
    		throws ServiceFailureException, ValidationException {
    	Employer employer = employerEAO.findCurrentEmployer();
		Set<BankAccount> accounts = employer.getBankAccounts();
		return accounts.toArray(new BankAccount[accounts.size()]);
    }

    /**
     * Update the given Bank Accounts.  Each Bank Account must have a uniqueId.
     * @param bankAccounts
     * @return an array containing all the saved BankAccount objects
     */
    public BankAccount[] updateBankAccounts(BankAccount[] bankAccounts)
    	throws ServiceFailureException, ValidationException {

    	BankAccountValidator.getInstance().validateEntityCollection(Arrays.asList(bankAccounts));
    	List<BankAccount> bankAccountList = new ArrayList<BankAccount>();
    	for (BankAccount bankAccount : bankAccounts) {
    		bankAccount = entityService.makeManaged(bankAccount);
    		bankAccountList.add(bankAccount);
		}

		//Dispatch the message
    	msgService.dispatchEmployerBankAccountEvent(WorkflowEventType.UPDATE_EMPLOYER_BANKACCOUNT, bankAccountList);

    	// now get the persisted bank account information and send it back as an array
    	Employer employer = employerEAO.findCurrentEmployer();
		Set<BankAccount> accounts = employer.getBankAccounts();
		return accounts.toArray(new BankAccount[accounts.size()]);
    }

	/*
	 * (non-Javadoc)
	 *
	 * @see com.intuit.sb.payroll.capability.employer.EmployerComponent#getPayments
	 */
    public List<Payment> getPayments(DateRange dateRange) throws ValidationException{
    	validateDateRange(dateRange);
    	Employer employer = employerEAO.findCurrentEmployer();
    	Set<Payment> payments = employer.getPayments();
    	List<Payment> dateRangePayments = new ArrayList<Payment>();
    	for (Payment payment:payments){
    		if (DateRangeUtil.isDateWithinDateRange(payment.getPaymentDate(), dateRange)){
    			dateRangePayments.add(payment);
    		}
    	}
    	return dateRangePayments;
    }

	/*
	 * (non-Javadoc)
	 *
	 * @see com.intuit.sb.payroll.capability.employer.EmployerComponent#updatePayments
	 */
    public List<Payment> updatePayments(List<Payment> payments) throws ValidationException{
		PaymentValidator.getInstance().validateEntityCollection(payments);

		Employer employer = employerEAO.findCurrentEmployer();
		Set<Payment> empPayments = employer.getPayments();
		List<Payment> updatedPayments = new ArrayList<Payment>();

		for(Payment payment:payments){
			if (entityService.notPersistent(payment)){
				updatedPayments.add(entityService.makePersistent(payment));
			}else{
				updatedPayments.add(entityService.makeManaged(payment));
			}

			if (!empPayments.contains(payment)){
				employer.addPayment(payment);
			}
		}
		return updatedPayments;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.intuit.sb.payroll.capability.employer.EmployerComponent#removePayments
	 */
	public void removePayments(DateRange dateRange) throws ValidationException, ServiceFailureException{
		List<Payment> paymentsToBeRemoved = getPayments(dateRange);
		Employer employer = employerEAO.findCurrentEmployer();
		for (Payment delPayment:paymentsToBeRemoved){
			employer.removePayment(delPayment);
			entityService.makeTransient(delPayment);
		}
		saveEmployer(employer);
	}
	
	/*
	 * (non-Javadoc)
	 *
	 * @see com.intuit.sb.payroll.capability.employer.EmployerComponent#removePayment
	 */
	public void removePayment(Long uniqueId) throws ServiceFailureException {
		Payment payment = null;
		try {
			payment = entityService.findById(Payment.class, uniqueId);
			if (null != payment) {
				Employer employer = employerEAO.findCurrentEmployer();
				employer.removePayment(payment);
				entityService.makeTransient(payment);
			} else {
				throw new ServiceFailureException(PayrollResourceManager.getInstance().getString(
						ResourceKeyConstants.ID_NOT_FOUND_EXCEPTION)
						+ uniqueId.toString());
			}
		} catch (Exception e) {
			throw new ServiceFailureException(e);
		}
	}

	private void validateDateRange(DateRange dateRange) throws ValidationException {
		PayrollParamValidator.checkIsNotNull(dateRange, "DateRange");
		Date startDate = dateRange.getStartDate();
		Date endDate   = dateRange.getEndDate();
		PayrollParamValidator.checkIsNotNull(startDate, "DateRange startDate");
		PayrollParamValidator.checkIsNotNull(endDate, "DateRange endDate");
	}



    /**
     * Get all the valid prefixes for Federal EIN
     * @return an array of strings containg the valid EIN prefixes
     * @throws IOException
     */
    public String[] getAllValidEinPrefixes()
    	throws ServiceFailureException, IOException{
    	String einPrefixes[] = EmployerTaxIdValidator.getInstance().getValidprefixesforEin();
    	return einPrefixes;
     }

    /**
     * (non-Javadoc)
     * @see com.intuit.sb.payroll.capability.employer.EmployerComponent#getLiabilitiesFromExistingPayrollRuns(com.intuit.sb.payroll.entity.model.DateRange)
     */
	public Map<String, Money> getLiabilitiesFromExistingPayrollRuns(DateRange dateRange)
			throws ValidationException, ServiceFailureException {

		/* -----------------------------------------------------------------------------------------
		 * Using ServiceLocator for WorkerPaymentsComponent instead of @EJB WorkerPaymentsComponent
		 * due to Cyclic Dependency.
		 * ------------------------------------------------------------------------------------------ */
		WorkerPaymentsComponent wpComponent = 
			ServiceLocator.getInstance().getService(WorkerPaymentsComponent.JNDI_NAME);
		// filtered list of PayrollRuns that fall within given date range
		List<PayrollRun> filteredPayrollRuns = new ArrayList<PayrollRun>();
		List<PayrollRun> allRuns = wpComponent.findAllPayrollRuns();

		for (PayrollRun payrollRun : allRuns) {
			Date pRunDate = payrollRun.getPayrollRunDate();
			if (DateRangeUtil.isDateWithinDateRange(pRunDate, dateRange)) {
				filteredPayrollRuns.add(payrollRun);
			}
		}

		// now we have a filtered list of PayrollRun objects - run through them to get
		// the desired elements
		Set<PaycheckLineItem> allLineItems = new HashSet<PaycheckLineItem>();
		for (PayrollRun run : filteredPayrollRuns) {
			Set<Paycheck> paychecks = run.getPaychecks();
			for (Paycheck paycheck : paychecks) {
				Set<PaycheckLineItem> pcLineitems = paycheck.getPaycheckLineItems();
				allLineItems.addAll(pcLineitems);
			}
		}

		// TODO consider voided paychecks - currently WIP

		// now we have all the PaycheckLineItem objects
		// group them -- the key to the map is the law id, the value is the aggregated liability value
		// in case of no PaycheckLineItems, we will return an EMPTY Map
		Map<String, Money> map = new HashMap<String, Money>();
		for (PaycheckLineItem item : allLineItems) {
			String lawid = item.getLawId().getValue();
			Money value = item.getValue();
			if (item.getSortSequence().equals(PaycheckLineItemSortSequenceEnum.EMPLOYEEPAIDTAX) ||
					item.getSortSequence().equals(PaycheckLineItemSortSequenceEnum.EMPLOYERPAIDTAX))
			{	//	get only the TAX items
				if (map.keySet().contains(lawid)) {
					value = value.add(map.get(lawid));
					//	add value to existing value for this law id
				}
				map.put(lawid, value);
				}
			}
		
		return map;
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.employer.EmployerComponent#getFutaPayments()
	 */
	public List<Payment> getFutaPayments(Calendar calendar) throws ValidationException {
		PayrollParamValidator.checkIsNotNull(calendar, "calendar");
		List<DateRange> quarterList = PtdUtils.getQuartersPriorToGivenDate(calendar);
		List<Payment> outputPayments = new ArrayList<Payment>();	//	payments to be sent back
		if (quarterList.isEmpty()) {	//	this is Q1
			return outputPayments;
		}
		Employer emp = employerEAO.findCurrentEmployer();
		Set<Payment> payments = emp.getPayments();	//	get all payments by this Employer
		if (payments != null && payments.size() > 0) {
			for (Payment payment : payments) {	//	 loop through payments
				DateRange paymentQuarter = payment.getPayrollPeriod();
				if (quarterList.contains(paymentQuarter)) {
					outputPayments.add(payment);
				}
			}
		}
		return outputPayments;
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.employer.EmployerComponent#getFutaLiabilities()
	 */
	public List<HistPaycheck> getFutaLiabilities(Calendar calendar) throws ValidationException {
		PayrollParamValidator.checkIsNotNull(calendar, "calendar");
		List<DateRange> quarterList = PtdUtils.getQuartersPriorToGivenDate(calendar);
		List<HistPaycheck> outputLiabilities = new ArrayList<HistPaycheck>();	//	Futa liabilities to be sent back
		if (quarterList.isEmpty()) {	//	this is Q1
			return outputLiabilities;
		}
		Employer emp = employerEAO.findCurrentEmployer();
		Set<HistPaycheck> historicPaychecks = emp.getHistPaychecks();	//	get all Historic Paychecks by this Employer
		if (historicPaychecks != null && historicPaychecks.size() > 0) {
			for (HistPaycheck historicPaycheck : historicPaychecks) {	//	 loop through historicPaychecks
				if (quarterList.contains(historicPaycheck.getPayrollPeriod())) {
					// paycheck is in required quarter - check for law id
					Set<HistPaycheckLineItem> lineItems = historicPaycheck.getHistPaycheckLineItems();
					for (HistPaycheckLineItem item : lineItems) {
						if (item.getLawId().getValue().equals(QboeLawIdConstants.sFUTALawId)) {
							outputLiabilities.add(historicPaycheck);
							break;
						}
					}
				}
			}
		}
		return outputLiabilities;
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.employer.EmployerComponent#removeFutaPaymentsAndLiabilitiesYtd()
	 */
	public void removeFutaPaymentsAndLiabilitiesYtd(Calendar calendar) 
			throws ValidationException, ServiceFailureException {
		PayrollParamValidator.checkIsNotNull(calendar, "calendar");
		Employer employer = employerEAO.findCurrentEmployer();
		List<Payment> payments = getFutaPayments(calendar);

		for (Payment payment : payments) {
            employer.removePayment(payment);
			entityService.makeTransient(payment);
		}
		List<HistPaycheck> liabilities = getFutaLiabilities(calendar);
		for (HistPaycheck histPaycheck : liabilities) {
			employer.removeHistPaycheck(histPaycheck);
			entityService.makeTransient(histPaycheck);
		}
		saveEmployer(employer);
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.employer.EmployerComponent#updateFutaPaymentsAndLiabilitiesYtd(java.util.List, java.util.List)
	 */
	public void updateFutaPaymentsAndLiabilitiesYtd(List<Payment> payments,
			List<HistPaycheck> liabilities) throws ValidationException {
		
		if ((payments == null || payments.size() == 0) && (liabilities == null || liabilities.size() == 0)) {
			throw new ValidationException("Payments AND Liabilities cannot be both null for update");
		}
		
		Employer employer = employerEAO.findCurrentEmployer();
		Set<Payment> employerPayments = employer.getPayments();
		Set<HistPaycheck> employerPaychecks = employer.getHistPaychecks();

		for (Payment payment : payments) {
			if (entityService.notPersistent(payment)) {
				entityService.makePersistent(payment);
			} else {
				entityService.makeManaged(payment);
			}
			
			if (!employerPayments.contains(payment)) {
				employerPayments.add(payment);
			}
		}

		for (HistPaycheck histPaycheck : liabilities) {
			if (entityService.notPersistent(histPaycheck)) {
				entityService.makePersistent(histPaycheck);
			} else {
				entityService.makeManaged(histPaycheck);
			}
			
			if (!employerPaychecks.contains(histPaycheck)) {
				employerPaychecks.add(histPaycheck);
			}
		}
	}	
}
