package com.intuit.sb.payroll.capability.employer.impl;

import java.util.Date;
import java.util.List;


import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.capability.task.TaskManager;
import com.intuit.sb.payroll.capability.task.impl.TaskUtils;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.Task;
import com.intuit.sb.payroll.entity.model.TaskStatusEnum;
import com.intuit.sb.payroll.entity.model.TaskTypeEnum;

/**
 * Encapsulates the management of Tasks that relate to Employers.
 * 
 */
public class EmployerComponentTasks {
	public static final String COMPANY_SETUP_TASKS_CLASS = "com.intuit.sb.payroll.task.employer";
	public static final String COMPANY_SETUP_TASK = "companySetupTask";
	public static final String WORKLOCATION_SETUP_TASK = "worklocationSetupTask";
	public static final String PAYROLLSCHEDULE_SETUP_TASK = "payrollScheduleSetupTask";
	
	/**
	 * Creates a generic "Setup your Company" task.
	 * 
	 * @param taskManager
	 * @param employer
	 */
	public static void createCompanySetupTask(TaskManager taskManager, Employer employer) {
		createCompanySetupTask(taskManager, COMPANY_SETUP_TASK, employer);	
	}

	/**
	 * Creates a task to remind the user to set a WorkLocation on the company.
	 * 
	 * @param taskManager
	 * @param employer
	 */
	public static void createAddWorkLocationTask(TaskManager taskManager, Employer employer) {
		createCompanySetupTask(taskManager, WORKLOCATION_SETUP_TASK, employer);
	}

	/**
	 * Creates a task to remind the user to set up at least one PayrollSchedule.
	 * 
	 * @param taskManager
	 * @param employer
	 */
	public static void createPayrollScheduleTask(TaskManager taskManager, Employer employer) {
		createCompanySetupTask(taskManager, PAYROLLSCHEDULE_SETUP_TASK, employer);
	}

	/**
	 * When a WorkLocation is created, call this method to deal with any task status changes that may result from it.
	 * 
	 * @param taskManager
	 * @param employer
	 * @throws TaskException
	 * @throws ServiceFailureException
	 */
	public static void handleWorkLocationCreatedEvent(TaskManager taskManager, Employer employer) throws ServiceFailureException {
		List<Task> tasks = taskManager.findTasksByClass(COMPANY_SETUP_TASKS_CLASS);
		Task companySetupTask = getCompanySetupTask(tasks, COMPANY_SETUP_TASK, employer);
		if (companySetupTask != null && companySetupTask.getStatus() != TaskStatusEnum.COMPLETED) {
			TaskUtils.closeTask(taskManager, companySetupTask);
		}
		Task workLocationSetupTask = getCompanySetupTask(tasks, WORKLOCATION_SETUP_TASK, employer);
		TaskUtils.closeTask(taskManager, workLocationSetupTask);
	}
	
	/**
	 * When a PayrollSchedule is created, call this method to deal with any task status changes that may result from it.
	 * 
	 * @param taskManager
	 * @param employer
	 * @throws TaskException
	 * @throws ServiceFailureException
	 */
	public static void handlePayrollScheduleCreatedEvent(TaskManager taskManager, Employer employer) throws ServiceFailureException {
		List<Task> tasks = taskManager.findTasksByClass(COMPANY_SETUP_TASKS_CLASS);
		Task companySetupTask = getCompanySetupTask(tasks, COMPANY_SETUP_TASK, employer);
		if (companySetupTask != null && companySetupTask.getStatus() != TaskStatusEnum.COMPLETED) {
			TaskUtils.closeTask(taskManager, companySetupTask);
		}
		Task payrollScheduleSetupTask = getCompanySetupTask(tasks, PAYROLLSCHEDULE_SETUP_TASK, employer);
		TaskUtils.closeTask(taskManager, payrollScheduleSetupTask);
	}

	/**
	 * Creates a CompanySetupTask  with a specified name.
	 * 
	 * @param taskManager
	 * @param taskname
	 * @param employer
	 * @return
	 */
	private static Task createCompanySetupTask(TaskManager taskManager, String taskname, Employer employer ) {
		Task companySetupTask = new Task();
		companySetupTask.setName(taskname);
		companySetupTask.setDescription(null);
		companySetupTask.setTaskClass(COMPANY_SETUP_TASKS_CLASS);
		companySetupTask.setType(TaskTypeEnum.SETUP);
		companySetupTask.setCategory("Setup");
		companySetupTask.setUserManagable(false);
		companySetupTask.setDeadline(new Date());
		companySetupTask.setPayload(new Long(employer.getId()));
		companySetupTask = taskManager.addTask(companySetupTask);
		return companySetupTask;
	}
	
	private static Task getCompanySetupTask(List<Task> alltasks, String companySetupTask, Employer employer) throws ServiceFailureException {
		List<Task> tasks = TaskUtils.getTaskByTaskName(alltasks, companySetupTask, employer);
		Task task = null;
		if (tasks.size() == 1) {
			task = tasks.get(0);
		} else if (tasks.size() > 1) {
			throw new ServiceFailureException("More than one Task found for a company");
		}
		return task;
	}

}