package com.intuit.sb.payroll.capability.ppe;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

import com.intuit.dataTypes.QBDate;
import com.intuit.payroll.j.EffectiveDatedDatumCollectionIf;
import com.intuit.payroll.j.PayrollDatumIf;
import com.intuit.payroll.j.PayrollItemIf;
import com.intuit.payroll.types.DatumType;
import com.intuit.payroll.types.PayrollException;
import com.intuit.payroll.types.enumtype.AccountingType;
import com.intuit.payroll.types.enumtype.DatumID;

public class AgencyView {
	// [rj] TODO change this to a real jurisdiction object
	// with unique jurisdcition id etc.
	private String jurisdiction;

	private String defaultAgencyName;
	private String agencyUrl = null;
	private PayrollDatumIf accountNumber; // PPE representation of TIN, will be usedfor getting the account
											// numberformats
	private String tinNumber; // account number is human readable format

	private Map<String, TaxRowView> rows = new HashMap<String, TaxRowView>();
	private Collection<TaxRowView> orderedTaxRows = new LinkedList<TaxRowView>();

	public AgencyView(String jurisdiction, String defaultAgencyName, PayrollDatumIf accountNumber) {
		this.jurisdiction = jurisdiction;
		this.defaultAgencyName = defaultAgencyName;
		this.accountNumber = accountNumber;
	}

	public String getJusrisdiction() {
		return jurisdiction;
	}

	public String getDefaultAgencyName() {
		return defaultAgencyName;
	}

	public PayrollDatumIf getAccountNumber() {
		return accountNumber;
	}

	public void setAccountNumber(PayrollDatumIf accountNumber) {
		this.accountNumber = accountNumber;
	}

	public String getTinNumber() {
		return tinNumber;
	}

	public void setTinNumber(String tinNumber) {
		this.tinNumber = tinNumber;
	}

	public Collection<TaxRowView> getOrderedTaxRows() {
		return orderedTaxRows;
	}

	public String getAgencyUrl() {
		return agencyUrl;
	}

	public void setAgencyUrl(String agencyUrl) {
		this.agencyUrl = agencyUrl;
	}

	public void addTax(PayrollItemIf tax) {
		String baseName = tax.getDetailType().getBaseName(tax.getName());
		TaxRowView row = rows.get(baseName);
		if (row == null) {
			row = new TaxRowView(baseName);
			rows.put(baseName, row);
			orderedTaxRows.add(row);
		}
		row.addTax(tax);
	}

	// For example "99-9999999"
	public String buildAccountNumberFormats() {
		StringBuilder result = new StringBuilder();
		Collection<String> formats = accountNumber.getValidationInfo().getStringFormats();
		if (formats != null) {
			for (String format : formats) {
				if (result.length() > 0)
					result.append(" or ");
				result.append(format);
			}
		}
		return result.toString();
	}

	// Handles one tax (both Employee and Employer info)
	static public class TaxRowView {
		private String baseName;
		private PayrollItemIf eePaid;
		private PayrollItemIf erPaid;
		private String eeRate = null;
		private String erRate = null;

		private long erTaxGuid; // FIXME - this is a hack to have the GUID from Gemini db for SUI rate so that we could
								// edit in SUI

		public long getErTaxGuid() {
			return erTaxGuid;
		}

		public void setErTaxGuid(long erTaxGuid) {
			this.erTaxGuid = erTaxGuid;
		}

		public TaxRowView(String baseName) {
			this.baseName = baseName;
		}

		public void addTax(PayrollItemIf tax) {
			if (tax.getDetailType().getAccountingType() == AccountingType.LIABILITY_AND_EXPENSE) {
				erPaid = tax;
			} else {
				eePaid = tax;
			}
		}

		public String getTaxName() {
			return baseName;
		}

		// returns the law id for the tax
		public int getLawId() {
			try {
				return erPaid.getDetailType().toInt();
			} catch (Exception e) {

			}
			throw new IllegalStateException("Cannot find lawid for" + baseName);
		}

		public String getEmployeeRate() {
			if (eeRate == null) {
				eeRate = getData(eePaid, DatumID.RATE);
			}
			return eeRate;
		}

		public void setEmployeeRate(String rate) {
			eeRate = rate;
		}

		public String getCompanyRate() {
			if (erRate == null) {
				erRate = getData(erPaid, DatumID.RATE);
			}
			return erRate;
		}

		public void setCompanyRate(String rate) {
			erRate = rate;
		}

		public String getWageLimit() {
			// assumes er and ee-paid taxes have the same wage limit
			return getData(erPaid, DatumID.WAGE_LIMIT);
		}

		/**
		 * determines wther the tax rate is editable
		 * 
		 * @return
		 */
		public boolean isEditable() {
			return isTaxRateEditable(erPaid, DatumID.RATE);
		}

		private static String getEffectiveDatedData(EffectiveDatedDatumCollectionIf datedData) {
			StringBuilder result = new StringBuilder();
			for (PayrollDatumIf datedValue : datedData.getVisibleDatumList()) {
				if (result.length() > 0)
					result.append(", ");
				try {
					result.append(datedValue.getValue());
				} catch (PayrollException e) {
					// blah
					result.append("***err***");
				}
				QBDate asOf = datedValue.getEffectiveBeginDate();
				if (asOf != null) {
					result.append(" As of: ");
					result.append(asOf);
				}
			}
			return result.toString();
		}

		/**
		 * @param tax eg CASUI, FICE, FICR etc
		 * @param field eg RATE or WAGE_LIMIT
		 * @return string description of rate or limit info for the passed tax
		 */
		private static String getData(PayrollItemIf tax, DatumID field) {
			StringBuilder result = new StringBuilder();
			if (tax != null) {
				PayrollDatumIf rate = tax.getDatum(field);
				if (null != rate) {
					if (rate.isEditable()) {
						// need editable field
						result.append("[");
					}
					Object val = null;
					try {
						val = rate.getValue();
					} catch (PayrollException e) {
						// Sorry the PE exception system is screwed up!
						result.append("***err***");
					}
					if (rate.getValueType() == DatumType.EFFECTIVE_DATED_DATUM_LIST) {
						result.append(getEffectiveDatedData((EffectiveDatedDatumCollectionIf) val));
					} else {
						result.append(val);
					}
					if (rate.isEditable()) {
						// need editable field
						result.append("]");
					}
					String desc = rate.getDescription();
					if (desc != null) {
						// Could be message about inexperience rate
						result.append(" ");
						result.append(desc);
					}
				}
			}
			return result.toString();
		}

		/**
		 * checks whether the rate of a tax is editable
		 * 
		 * @param tax
		 * @param field, the DatumID
		 * @return true if the tax rate is editable
		 */
		private static boolean isTaxRateEditable(PayrollItemIf tax, DatumID field) {
			if (tax != null) {
				PayrollDatumIf rate = tax.getDatum(field);
				return rate.isEditable();
			}
			return false;
		}
	}
}