package com.intuit.sb.payroll.capability.pto;

import javax.ejb.Local;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.entity.domain.Identifiable;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.PaidTimeOff;
import com.intuit.sb.payroll.entity.model.PaidTimeOffDtl;
import com.intuit.sb.payroll.entity.model.PaidTimeOffTypeEnum;
import com.intuit.sb.payroll.entity.model.PtoBaseRateTypeEnum;
import com.intuit.sb.payroll.entity.model.Worker;

@Local
public interface PaidTimeOffComponent {

	public static final String JNDI_NAME =  "PaidTimeOffComponent/local";

	/**
	 * Finds a PaidTimeOff.  This is global static data - one per type.
	 * @param paidTimeOffTypeEnum the type of paid time off
	 * @return the PaidTimeOff
	 * @throws ServiceFailureException 
	 */
	public PaidTimeOff findPaidTimeOff(PaidTimeOffTypeEnum paidTimeOffTypeEnum) throws ServiceFailureException;

	/**
	 * Creates a new paid time off detail. All the "immutable" fields in the Dtl are set:
	 * <ul>
	 * <li>The association to its respective PaidTimeOff</li>
	 * <li>any empty Datum</li>
	 * <li>a compensation type</li>
	 * <li>its appropriate law id</li>
	 * </ul>
	 * Additionally, the name and description are set from the values in the corresponding PaidTimeOff.
	 * @param compensationTypeEnum
	 * @return the template paid time off type ready for the "mutable" fields to be set.
	 * @throws ServiceFailureException 
	 */
	public PaidTimeOffDtl createPaidTimeOffDtl(PaidTimeOffTypeEnum paidTimeOffTypeEnum) throws ServiceFailureException;
	
	/**
	 * Associate the given paid time off detail with the given worker.
	 * 
	 * Use {@link #createPaidTimeOffDtl(PaidTimeOffTypeEnum)} to properly create a {@link PaidTimeOffDtl} instance,
	 * set the required values and associate with a {@link Worker} through this API.  Rules:
	 * <ul>
	 * 	<li>There can only be one PTO of a given type (SICK or VACATION).</li>
	 * 	<li>The Worker must have a {@link CompensationDtl} with a corresponding {@link PtoBaseRateTypeEnum} value.</li>
	 * </ul>
	 * The consequence of adding a PTO to the {@link Worker} is that it results in a new {@link CompensationDtl}
	 * being created of type {@link CompensationTypeEnum#SICKTAKEN} or {@link CompensationTypeEnum#VACTAKEN}.  These
	 * items do not have rates - the rates are determined by the Hourly or Salary compensation items (see above).
	 * 
	 * @param worker
	 * @param paidTimeOffDtl the PTO to add.
	 * @return the updated Worker
	 * @throws ValidationException if null inputs or rules are violated.
	 * @throws ServiceFailureException if infrastructure fails.
	 */
	public Worker addPaidTimeOffDtl(Worker worker, PaidTimeOffDtl paidTimeOffDtl)
		throws WorkflowException, ValidationException, ServiceFailureException;

	/**
	 * Finds a PaidTimeOffDtl.
	 * @param identifiable the identifiable of the existing PaidTimeOffDtl.
	 * @return the PaidTimeOffDtl.
	 * @throws ServiceFailureException 
	 * @throws ValidationException 
	 */
	public PaidTimeOffDtl findPaidTimeOffDtl(Identifiable identifiable) throws ServiceFailureException, ValidationException;
	
	/**
	 * Updates a PaidTimeOffDtl
	 * @param workerEntity 
	 * @param identifiable the identifiable of the existing PaidTimeOffDtl
	 * @return the PaidTimeOffDtl
	 * @throws ValidationException 
	 */
	public Worker updatePaidTimeOffDtl(Worker workerEntity, PaidTimeOffDtl paidTimeOffDtl) throws ValidationException;
	
	/**
	 * Deletes a paid time off detail associated with the worker.
	 * @param worker Valid worker object.
	 * @param paidTimeOffDtl the PaidTimeOffDtl that needs to be deleted. 
	 * @throws ServiceFailureException 
	 * @throws ValidationException 
	 */
	public Worker deletePaidTimeOffDtl(Worker worker, PaidTimeOffDtl paidTimeOffDtl) throws ServiceFailureException, ValidationException;
}
