package com.intuit.sb.payroll.capability.pto.impl;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.validation.AbstractEntityDataValidator;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.model.PaidTimeOffAccrualStartTypeEnum;
import com.intuit.sb.payroll.entity.model.PaidTimeOffAccrualTypeEnum;
import com.intuit.sb.payroll.entity.model.PaidTimeOffDtl;

/**
 * Validates PaidTimeOff Entities.
 *
 * $Author$ 
 * $Revision$
 * $DateTime$
 */
public class PaidTimeOffValidator extends AbstractEntityDataValidator<PaidTimeOffDtl> {
	
	private static PaidTimeOffValidator sInstance = new PaidTimeOffValidator();
	
	private PaidTimeOffValidator() {}
	
	/*
	 * Map accrual types to the maximum hours that can be accrued in the corresponding period. 
	 */
	private static Map<PaidTimeOffAccrualTypeEnum, Double> maxAccrualRate =	new HashMap<PaidTimeOffAccrualTypeEnum, Double>();
	static {
		// 999 hours is 25 weeks. This is a reasonable maximum PTO per year!
		maxAccrualRate.put(PaidTimeOffAccrualTypeEnum.HOURLY, 1.0);
		maxAccrualRate.put(PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK, 31 * 24.0); // max pay period is monthly
		maxAccrualRate.put(PaidTimeOffAccrualTypeEnum.PER_YEAR_START_OF_YEAR, 999.0);
		maxAccrualRate.put(PaidTimeOffAccrualTypeEnum.PER_YEAR_PRORATED, 999.0);
		maxAccrualRate.put(PaidTimeOffAccrualTypeEnum.PER_YEAR_FROM_HIRE_DATE, 999.0);
		
	}
	
	public static PaidTimeOffValidator getInstance() {
		return sInstance;
	}
	
	/**
	 * 	<li>if accrual start is deferred, ensure the date is >= today</li>
	 * <li>the accrual rate must be <= the number of hours in the accrual period</li>
	 */
	@Override
	public void validateEntity(PaidTimeOffDtl entity) throws ValidationException {
		PayrollParamValidator.checkIsNotNull(entity, "paidTimeOffDtl");
		PayrollParamValidator.checkIsNotNullOrEmptyString(entity.getName(), "ptoName");
		PayrollParamValidator.checkIsNotNull(entity.getPaidTimeOffType(),"paidTimeOffType");
		
		if (entity.getAccrualStartType().equals(PaidTimeOffAccrualStartTypeEnum.ON_START_DATE)) {
			PayrollParamValidator.checkIsNotNull(entity.getAccrualStartDate(), "accrualStartDate");
		}
		
		PayrollParamValidator.checkIsNotNull(entity.getRate(), "ptoRate");
		
		if (entity.getRate().getValue1AsQuantity() > maxAccrualRate.get(entity.getAccrualType())) {
			//FIXME [I18N] resource translation
			String accrualTypeName = entity.getAccrualType().toString(); // need to look up in a resource bundle
			String mesg = MessageFormat.format("Accrual rate {0} is greater than {1} which is the maximum allowed for ''{2}''",
					entity.getRate().getValue1AsQuantity(), maxAccrualRate.get(entity.getAccrualType()), accrualTypeName);
			throw new ValidationException(mesg);
		}
		
	}

	@Override
	public void validateEntityCollection(List<PaidTimeOffDtl> entityCollection) throws ValidationException {
		for (PaidTimeOffDtl paidTimeOffDtl : entityCollection) {
			validateEntity(paidTimeOffDtl);
		}
	}

	
}