package com.intuit.sb.payroll.capability.pts.adapter;

import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebService;
import javax.jws.soap.SOAPBinding;

import org.apache.log4j.Logger;
import org.jboss.wsf.spi.annotation.WebContext;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.pts.adapter.dtos.AdapterResponse;
import com.intuit.sb.payroll.capability.pts.adapter.dtos.DatetimeDTO;
import com.intuit.sb.payroll.capability.pts.adapter.dtos.MessageDTO;
import com.intuit.sb.payroll.capability.pts.adapter.dtos.NotificationAttribute;
import com.intuit.sb.payroll.capability.pts.adapter.dtos.PTSNotificationDTO;
import com.intuit.sb.payroll.capability.pts.adapter.mapper.CompanyMapper;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationException;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationRequest;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationTypeEnum;
import com.intuit.sb.payroll.capability.pts.notification.mapper.PTSNotificationRequestMapper;
import com.intuit.sb.payroll.capability.pts.statehandler.StatusUpdateHandler;
import com.intuit.sb.payroll.capability.pts.statehandler.StatusUpdateHandlerFactory;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.entity.common.EntityService;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;
import com.intuit.sb.payroll.pts.entity.PTSMessage;
import com.intuit.sb.payroll.security.SecurityHelper;

/**
 * Notification Events exposed by Gemini to update the status of various
 * workflows during enrolling a Company into PSP
 *
 */

@SOAPBinding(style = SOAPBinding.Style.RPC)
@Stateless
@WebContext(contextRoot="/PayrollProcessingService")
@Remote(PTSNotification.class)
@WebService
public class PTSNotificationAdapter implements PTSNotification  {

	private static Logger logger = PayrollLoggerFactory.makeLogger(PTSNotificationAdapter.class);

	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.pts.adapter.PTSNotification#updateCompanyActivationStatus(com.intuit.sb.payroll.capability.pts.adapter.dtos.CompanyActivationNotificationDTO)
	 */

	// TODO: To avoid alerts for company that got removed in dev env accumulate in PSP we will
	// for now send a success for those 
	
	
	@WebMethod
	public AdapterResponse updateCompanyActivationStatus(
			@WebParam(name = "ptsNotificationDTO")
			PTSNotificationDTO ptsNotificationDTO) {
		// Log the request
		logPTSNotificationDTO(ptsNotificationDTO);
		String realmId = null;
		String uuidFromRequest = null;
		try {
			realmId = CompanyMapper.getCompanyRealmIdFromSourceCompanyId(ptsNotificationDTO.getSourceCompanyId());
			uuidFromRequest = CompanyMapper.getCompanyIdentifierFromSourceCompanyId(ptsNotificationDTO.getSourceCompanyId());
		} catch (ValidationException e) {
			return createAdapterResponse(ptsNotificationDTO.getSourceCompanyId(),e.getMessage(), true );
		}
		
		EntityService ptsEmpEao = null;
        try {
        	
        	ptsEmpEao = SecurityHelper.getServiceUsingAuthentication(EntityService.JNDI_NAME);
            ptsEmpEao.setTenantContext(realmId);
            PTSEmployer ptsEmployer = null;
            // Validate the EmployerUUID with the sourceCompanyId rcvd
            List<PTSEmployer> ptsEmployeList = ptsEmpEao.findAll(PTSEmployer.class);
            
            if (ptsEmployeList.size() == 0){
            	logger.error("Not able to locate the Company");
            	return createAdapterResponse(ptsNotificationDTO.getSourceCompanyId(),"Not able to locate the Company", true );
            }else if (ptsEmployeList.size() == 1){
            	ptsEmployer = ptsEmployeList.get(0);
            	Employer employer = ptsEmployer.getEmployer();

            	// Match the UUID for the employer and source company
            	if (!uuidFromRequest.equals(employer.getUuid())){
            		logger.error("CompanyId does not match expected "+ employer.getUuid() + " Received "+ uuidFromRequest);
            		return createAdapterResponse(ptsNotificationDTO.getSourceCompanyId(),"Not able to match the Company Identifier", true );
            	}
            }
			PTSNotificationRequest ptsNotificationRequest = PTSNotificationRequestMapper.mapPTSNotificationRequest(ptsNotificationDTO);
			
			logMappedPTSNotificationRequest(ptsNotificationRequest);
			
			// Get the right status update handler
			PTSNotificationTypeEnum ptsNotificationType = ptsNotificationRequest.getNotificationType();
			if (ptsNotificationType == null){
				logger.error("Not able to Process this request");
            	return createAdapterResponse(ptsNotificationDTO.getSourceCompanyId(),"Not able to Process this request", false );
			}
			StatusUpdateHandler statusUpdateHandler = StatusUpdateHandlerFactory.getStatusUpdateHandler(ptsNotificationType);
			statusUpdateHandler.updateStatus(ptsEmployer, ptsNotificationRequest);
		}
        //TODO catch proper exception and return it to PSP
        catch (ServiceFailureException e) { 
			e.printStackTrace();
			logger.fatal(e.getLocalizedMessage());
		} catch (PTSNotificationException e) {
			logger.fatal(e.getLocalizedMessage());
			e.printStackTrace();
		}
        finally {
			// flush the transaction before clearing tenant context to avoid stale data issues
			ptsEmpEao.flush();
			ptsEmpEao.clearTenantContext();
		}

		// Create a  Success response
		return createAdapterResponse(ptsNotificationDTO.getSourceCompanyId(),null, true );
	}
	
	

	private AdapterResponse createAdapterResponse(String sourceCompanyId, String errorMessage, boolean status){
		AdapterResponse adapterResponse = new AdapterResponse();
		
		adapterResponse.setSourceCompanyId(sourceCompanyId);
		if (errorMessage != null){
			//TODO create Error msg maps
			MessageDTO message = new MessageDTO();
			message.setMessageDescription(errorMessage);
			adapterResponse.setMessage(message);
		}
		adapterResponse.setStatus(status);
		logAdapterResponse(adapterResponse);
		return adapterResponse;
	}


	private void logAdapterResponse(AdapterResponse adapterResponse) {
		if (logger.isDebugEnabled()){
			StringBuffer sb = new StringBuffer();
			sb.append("\n\t Notification Response CompanyId ").append(adapterResponse.getSourceCompanyId());
			MessageDTO message = adapterResponse.getMessage();
			if (message != null){
			sb.append("\n\t Notification Response Message   ").append(message.getMessageDescription());	
			}
			sb.append("\n\t Notification Response Response  ").append(adapterResponse.isStatus());
			logger.debug(sb.toString());
		}
	}


	private void logPTSNotificationDTO(PTSNotificationDTO ptsNotificationDTO) {
		StringBuffer sb = new StringBuffer();
		sb.append("\n\tNotification Received for companyId | "+ ptsNotificationDTO.getSourceCompanyId()).append("\n")
		  .append("\tNotification Type                   | " + ptsNotificationDTO.getNotificationType()).append("\n")
		  .append("\tNew Notification Status             | " + ptsNotificationDTO.getNewNotificationStatus()).append("\n")
		  .append("\tOld Notification Status             | " + ptsNotificationDTO.getOldNotificationStatus()).append("\n")
		  .append("\tNotification TimeStamp              | " + toString(ptsNotificationDTO.getNotificationChangeTimeStamp())).append("\n");

		List<NotificationAttribute> attributes = ptsNotificationDTO.getAttributes();
		StringBuffer notAttrSB = new StringBuffer("\t Notification Attribute ").append("\n");
		for (NotificationAttribute notificationAttribute : attributes) {
			notAttrSB.append("\t\t").append(notificationAttribute.getName())
					.append(" Value  ")
					.append(notificationAttribute.getValue()).append("\n");
		}
		sb.append(notAttrSB);

		List<MessageDTO> messages = ptsNotificationDTO.getActivationMessage();
		StringBuffer sbmess = new StringBuffer("\t Message Identifier ").append("\n");
		for (MessageDTO messageDTO : messages) {
			sbmess.append("\t\t").append(messageDTO.getMessageId())
			.append(" Description  ")
			.append(messageDTO.getMessageDescription()).append("\n");
		}
		sb.append(sbmess);
		logger.debug(sb.toString());
	}
	
	private void logMappedPTSNotificationRequest(
			PTSNotificationRequest ptsNotificationRequest) {
		if (logger.isDebugEnabled()){
			StringBuffer sb = new StringBuffer();
			
			sb.append("\n")
			.append("\tPTS NotificationType       |").append(ptsNotificationRequest.getNotificationType()).append("\n")
			.append("\tPTS NotificationStatus     |").append(ptsNotificationRequest.getNotificationStatus()).append("\n")
			.append("\tPTS Old NotificationStatus |").append(ptsNotificationRequest.getOldNotificationStatus()).append("\n");
			
			sb.append("\tAttributes \n");
			StringBuffer attrSb = new StringBuffer();
			Map<String, String> attribs = ptsNotificationRequest.getNotificationAttributes();
			Set<String> attrKeySet = attribs.keySet();
			
			for (String key : attrKeySet) {
				attrSb.append("\t\t").append(key).append(" value ")
				.append(attribs.get(key)).append("\n");
			}
			sb.append(attrSb.toString());
			
			List<PTSMessage> messages = ptsNotificationRequest.getPTSMessage();
			
			StringBuffer sbmess = new StringBuffer("\tMessage Identifier ").append("\n");
		    for (PTSMessage messageDTO : messages) {
			    sbmess.append("\t\t").append(messageDTO.getMessageCode())
			     .append(" Description  ")
			     .append(messageDTO.getMessageDescription()).append("\n");
		    }
			sb.append(sbmess.toString());
			sb.append("\tTimeStamp                  |").append(ptsNotificationRequest.getTimestamp());
			logger.debug(sb.toString());
		}
	}
	

	private String toString(DatetimeDTO notificationChangeTimeStamp) {
		StringBuffer sb = new StringBuffer();

		sb.append(notificationChangeTimeStamp.getYear()).append("/")
		  .append(notificationChangeTimeStamp.getMonth()).append("/")
		  .append(notificationChangeTimeStamp.getDay()).append("/")
		  .append(notificationChangeTimeStamp.getHour()).append("/")
		  .append(notificationChangeTimeStamp.getMinute()).append("/")
		  .append(notificationChangeTimeStamp.getSecond());

		return sb.toString();
	}
}
