package com.intuit.sb.payroll.capability.pts.adapter;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerPTSWorkflowComponent;
import com.intuit.sb.payroll.capability.employer.impl.EmployerSetupRules;
import com.intuit.sb.payroll.capability.employer.impl.EmployerWorkflowType;
import com.intuit.sb.payroll.capability.pts.LiabilityBalance;
import com.intuit.sb.payroll.capability.pts.adapter.mapper.BankAccountMapper;
import com.intuit.sb.payroll.capability.pts.adapter.mapper.CompanyMapper;
import com.intuit.sb.payroll.capability.pts.adapter.mapper.DateMapper;
import com.intuit.sb.payroll.capability.pts.adapter.mapper.EmployeeMapper;
import com.intuit.sb.payroll.capability.pts.adapter.mapper.JurisdictionMapper;
import com.intuit.sb.payroll.capability.pts.adapter.mapper.PayrollRunMapper;
import com.intuit.sb.payroll.capability.pts.adapter.mapper.TaxPaymentsMapper;
import com.intuit.sb.payroll.capability.pts.adapter.util.PTSAdapterResultUtil;
import com.intuit.sb.payroll.capability.pts.notification.mapper.PTSNotificationRequestMapper;
import com.intuit.sb.payroll.capability.pts.statehandler.StatusUpdateHandlerUtil;
import com.intuit.sb.payroll.capability.worker.WorkerWorkflowType;
import com.intuit.sb.payroll.capability.worker.impl.WorkerSetupRules;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.RequiredData;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.domain.Identifiable;
import com.intuit.sb.payroll.entity.model.AgencyEnum;
import com.intuit.sb.payroll.entity.model.BankAccount;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.EmployerId;
import com.intuit.sb.payroll.entity.model.Jurisdiction;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PaycheckWorkflowStatusEnum;
import com.intuit.sb.payroll.entity.model.Payment;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.pts.entity.InteractionEnum;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;
import com.intuit.sb.payroll.pts.entity.PTSPayrollRun;
import com.intuit.sb.payroll.pts.entity.PTSPayrollRunEAO;
import com.intuit.sb.payroll.pts.entity.PTSVoidPayrollResult;
import com.intuit.sb.payroll.pts.entity.PTSVoidPayrollResultEAO;
import com.intuit.sbd.payroll.psp.adapters.gemini.AdapterResult;
import com.intuit.sbd.payroll.psp.adapters.gemini.CompanyAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.CompanyBankAccountAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.EmployeeAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.LiabilityBalancesResult;
import com.intuit.sbd.payroll.psp.adapters.gemini.PayrollRunAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.PayrollTaxAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.SubmitPayrollResult;
import com.intuit.sbd.payroll.psp.adapters.gemini.TaxPaymentsAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.VoidPayrollResult;

/**
 * This class will initiate calls to PSP to perform add/update Payroll related information
 * This class should NOT have any workflow logic to make these but should only validate that
 * the PSP call is being made at right time and have all required infor
 * 
 *
 */

public class PTSRequestInvoker {

	private static final Logger sLogger = PayrollLoggerFactory.makeLogger(PTSRequestInvoker.class);
	private static final int ADD_WORKER = 101;
	private static final int UPDATE_WORKER = 102;
	private static final int TERMINATE_WORKER = 103;
	private static final int REHIRE_WORKER = 104;
	private static final int DELETE_WORKER = 105;
	
	/**
	 * Add the given {@code PTSEmployer} object to PTS
	 * @param ptsEmployer
	 * @throws ValidationException
	 * @throws ServiceFailureException
	 */
	public static void addEmployerInPTS(PTSEmployer ptsEmployer)
			throws ValidationException, ServiceFailureException {
		sendEmployerinPTS(ptsEmployer, true);		
	}

	/**
	 * Update the given {@code PTSEmployer} object to PTS
	 * @param ptsEmployer
	 * @throws ValidationException
	 * @throws ServiceFailureException
	 */
	public static void updateEmployerInPTS(PTSEmployer ptsEmployer)
			throws ValidationException, ServiceFailureException {
		sendEmployerinPTS(ptsEmployer, false);		
	}

	/**
	 * Contains the core logic to build the PTS Specific Company DTO and then send<br>
	 * to PTS via web-service
	 * @param ptsEmployer
	 * @param addOrUpdateEmployerInPTS
	 * @throws ValidationException
	 * @throws ServiceFailureException
	 */
	private static void sendEmployerinPTS(PTSEmployer ptsEmployer, boolean addOrUpdateEmployerInPTS)
					throws ValidationException, ServiceFailureException {

		// Allow this call only if company has not been added before
		if (StatusUpdateHandlerUtil.isEntityAddedToPTS(ptsEmployer.getEmployer()) && addOrUpdateEmployerInPTS){
			throw new ValidationException("Company already added with PTS");
		}  
		
		// Allow this call only if Employer has enough data to talk to PSP
		List<RequiredData> missingList = EmployerSetupRules.getMissingDataForWorkflow(EmployerWorkflowType.CreateAssistedCompany, ptsEmployer);
		
		if (missingList.size() > 0){
			// Data not sufficient to create an Assisted company in PSP
			StringBuffer sb = new StringBuffer();
			for (RequiredData requiredData : missingList) {
				sb.append(requiredData.getDescription()).append("  ");
			}
			throw new ValidationException("Insufficient data to send information to PTS "+ sb.toString());
		}
		
		// Prepare company DTO from all available data
		CompanyAdapterDTO companyDTO = CompanyMapper.mapToCompanyAdapterDTO(ptsEmployer);
		
		CompanyRequestWrapper ptsCompanyRequestWrapper = new CompanyRequestWrapper();
		AdapterResult adapterResult = null;
		InteractionEnum interactionType = null;
		
		if(addOrUpdateEmployerInPTS) {
			adapterResult = ptsCompanyRequestWrapper.addCompanyinPTS(companyDTO);
			interactionType = InteractionEnum.ADDED;

		} else {
			adapterResult = ptsCompanyRequestWrapper.updateCompanyinPTS(companyDTO);
			interactionType = InteractionEnum.UPDATED;

		}

		/*
		 * Continue processing if the company is added successfully
		 */
		if (adapterResult.isSuccess()) {
			setInteractionStateForEntity(ptsEmployer.getEmployer(), interactionType);
			
			// InActivate all Employer related notifications
			StatusUpdateHandlerUtil.inActivateAllEFTPSNotificationStates();
			
		} else {
			throw new ServiceFailureException("Unable to send information to PTS - addition of company to " +
					"PTS failed: \n REASONS: " + 
					PTSAdapterResultUtil.getAdapterResultMessagesAsDelimitedString(adapterResult));
		}
	}

	/**
	 * @param entity
	 * @param interactionType
	 * @throws ValidationException
	 * @throws ServiceFailureException
	 */
	private static void setInteractionStateForEntity(Identifiable entity, InteractionEnum interactionType) throws ValidationException,
			ServiceFailureException {
		EmployerPTSWorkflowComponent empPTSComp = ServiceLocator.getInstance().getService(EmployerPTSWorkflowComponent.JNDI_NAME);
		empPTSComp.updatePPS_PTS_InteractionState(entity, interactionType);
	}

	/**
	 * This method will get all the existing Jurisdictions and send all of them to PSP
	 * Expect to call this method immediately after company is added
	 * @param ptsEmployer
	 * @throws ValidationException 
	 * @throws ServiceFailureException 
	 */
	public static void addJurisdictionIdToPTS(PTSEmployer ptsEmployer, Jurisdiction jurisdiction) throws ValidationException, ServiceFailureException{

		// check if Company has been already added
		if (!StatusUpdateHandlerUtil.isEntityAddedToPTS(ptsEmployer.getEmployer())){
			throw new ValidationException("Company Not added with PTS, can not process further");
		}
		
		// check if the Jurisdiction we are adding already has been added
		if (StatusUpdateHandlerUtil.isEntityAddedToPTS(jurisdiction )){
			throw new ValidationException("Company Not added with PTS, can not process further");
		}
		

		
		List<String> jurisdictionStrList = new ArrayList<String>();
		List<String> mappedJurisList =  JurisdictionMapper.mapJurisdiction(jurisdiction);
		for (String mapJurisdiction : mappedJurisList) {
			if (!jurisdictionStrList.contains(mapJurisdiction)){
				jurisdictionStrList.add(mapJurisdiction);
			}
		}

		// Now send these jurisdiction to PSP
		String sourceCompanyId = CompanyMapper.namespaceEmployerId(ptsEmployer.getEmployer());
		
		for (String jurisdictionId : jurisdictionStrList) {
			CompanyRequestWrapper ptsCompanyRequestWrapper = new CompanyRequestWrapper();
			AdapterResult adapterResult = ptsCompanyRequestWrapper.addJurisdictionToCompany(sourceCompanyId, jurisdictionId);

			if (adapterResult.isSuccess()){
				setInteractionStateForEntity(jurisdiction, InteractionEnum.ADDED);
			}else{
				throw new ServiceFailureException("Unable to send information to PTS - addition of Jurisdiction to " +
						"PTS failed: \n REASONS: " + 
						PTSAdapterResultUtil.getAdapterResultMessagesAsDelimitedString(adapterResult));
			}
		}
	}

	/**
	 * Utility to send BankAccount info to PTS - account info may be added or updated
	 * @param employer - Organization under which current transaction is operating
	 * @param bankAccount - BankAccount to be sent to PTS
	 * @param addOrUpdateAccount - indicator to flag whether PTS should be send a new account for addition <br/>
	 * or an existing account on PTS should be updated <br/>
	 * <pre>
	 * 		Use {@link}PTSRequestInvoker.ADD_BANK_ACCOUNT_TO_PTS or {@link}PTSRequestInvoker.UPDATE_BANK_ACCOUNT_TO_PTS
	 * 		to indicate add or update respectively.
	 * </pre>
	 * @throws ValidationException
	 * @throws ServiceFailureException
	 */
	private static void sendBankAccountInfotoPTS (
			PTSEmployer ptsEmployer, BankAccount bankAccount, boolean addBankAccount)
			throws ValidationException, ServiceFailureException {

		// TODO check if we have all information avl for adding Bank

		// check if Company has been already added
		Employer employer = ptsEmployer.getEmployer();
		if (!StatusUpdateHandlerUtil.isEntityAddedToPTS(employer)){
			throw new ValidationException("Company Not added with PTS, can not process further");
		}
		
		// check if the BankAccount we are adding already has been added
		if (addBankAccount && StatusUpdateHandlerUtil.isEntityAddedToPTS(bankAccount)){
			throw new ValidationException("Bank Account already added, can not process further");
		}
		
		CompanyBankAccountAdapterDTO companyBankAccountAdapterdto =
			   BankAccountMapper.maptoBankAccountDTO(employer, bankAccount);
		
		CompanyBankAccountRequestWrapper companyBankAcctRequestWrapper = new CompanyBankAccountRequestWrapper();
		AdapterResult adapterResult = null;
		InteractionEnum interactionEnum = null;
		Map<String, String> attributeMap = new HashMap<String, String>();
		attributeMap.put(PTSNotificationRequestMapper.SOURCE_BANK_ACCOUNT_ID, String.valueOf(bankAccount.getId()));
		
		String companySourceId = CompanyMapper.namespaceEmployerId(employer);
		if (addBankAccount) {
			adapterResult  = companyBankAcctRequestWrapper.addCompanyBankAccount(
					companySourceId, companyBankAccountAdapterdto);
			interactionEnum = InteractionEnum.ADDED;
		} else  {
			adapterResult = companyBankAcctRequestWrapper.updateCompanyBankAccount(
					companySourceId, companyBankAccountAdapterdto);
			interactionEnum = InteractionEnum.UPDATED;
		}

		if (adapterResult.isSuccess()){
			setInteractionStateForEntity(bankAccount, interactionEnum);

			StatusUpdateHandlerUtil.inActivateAllBANKNotificationStates(new Long(bankAccount.getId()));
		} else {
			StringBuffer sb = new StringBuffer().append("Unable to send bank information to PTS ")
					.append("PTS failed: \n REASONS: ") 
					.append(PTSAdapterResultUtil.getAdapterResultMessagesAsDelimitedString(adapterResult));
			
			sLogger.error(sb.toString());
			// PSP update failed - log the failure
			throw new ServiceFailureException(sb.toString());
			
		}
	}

	/**
	 * Utility to update BankAccount info in PTS
	 * @param employer - Organization under which current transaction is operating
	 * @param bankAccount - BankAccount to be sent to PTS
	 * @throws ValidationException
	 * @throws ServiceFailureException
	 */
	public static void updateBankAccountInfoInPTS (PTSEmployer ptsEmployer, BankAccount bankAccount)
			throws ValidationException, ServiceFailureException {
		if (!StatusUpdateHandlerUtil.isEntityAddedToPTS(ptsEmployer.getEmployer())){
			throw new ValidationException("Company Not added with PTS, can not process further");
		}
		sendBankAccountInfotoPTS(ptsEmployer, bankAccount, false);
	}

	/**
	 * Utility to add BankAccount info to PTS
	 * @param employer - Organization under which current transaction is operating
	 * @param bankAccount - BankAccount to be sent to PTS
	 * @throws ValidationException
	 * @throws ServiceFailureException
	 */
	public static void addBankAccountInfotoPTS (PTSEmployer ptsEmployer, BankAccount bankAccount)
			throws ValidationException, ServiceFailureException {
		if (!StatusUpdateHandlerUtil.isEntityAddedToPTS(ptsEmployer.getEmployer())){
			throw new ValidationException("Company Not added with PTS, can not process further");
		}
		sendBankAccountInfotoPTS(ptsEmployer, bankAccount, true);
	}
	
	// TODO Move this out from here
	
	public static EmployerId getEmployerIDforAgency(Employer employer, AgencyEnum agencyEnum){
		Set<EmployerId> employerIds = employer.getEmployerIds();

		// loop through the EmployerID's for this Employer to find the required one
		for (EmployerId employerId : employerIds) {
			// for each EmployerId, get the Agency (Gemini type)
			AgencyEnum agency = employerId.getAgencyIdType().getAgencyKey();
			// get the corresponding PTS Agency type
			// if they match, then this is the id
			if (agency.equals(agencyEnum)) {
				return employerId;
			}
		}
		return null;
	}
	
	

	/**
	 * Utility to add Company TaxAgency Id to PTS
	 * @param ptsEmployer PTSEmployer object associated with this company
	 * @param agencyType   AgencyEnum (of the form CA_EDD, TX_TWC, etc)
	 * @param companyAgencyRegistrationId 
	 * @throws ValidationException
	 * @throws ServiceFailureException
	 * @see com.intuit.sb.payroll.entity.model.AgencyEnum
	 */
	public static void addCompanyTaxAgencyId(
			PTSEmployer ptsEmployer, EmployerId employerId)
		throws ValidationException, ServiceFailureException {

		// FIXME add validation check for parameters agencyType to be known/valid one
		// TODO check if this agency has already been added
		// TODO if possible Get AgencyEnum here

		// FIXME Figure out what jurisdiction has been sent and then send the associated aganecyId
		
		if (!StatusUpdateHandlerUtil.isEntityAddedToPTS(ptsEmployer.getEmployer())){
			throw new ValidationException("Company Not added with PTS, can not process further");
		}
		
		if (employerId == null){
			throw new ValidationException("EmployerId provided is <null>");
		}
		Employer employer = ptsEmployer.getEmployer();

		/*---------------------------------------------------------------------------------------------
		 * Three parameters that need to be sent to PTS:
		 * 		sourceCompanyId 			- Identifier for the our company in PTS
		 * 		agencyId					- PTS AgencyEnum (IRS/CAEDD) that is supplied to us by the caller
		 * 						  			  This the agency who's id we need to send to PSP
		 * 		companyAgencyRegistrationId - The Agency Tax Identification Number (EIN/TIN/etc)
		 * 									  for the given Agency, for this Employer/Company
		 ---------------------------------------------------------------------------------------------*/
		String sourceCompanyId = CompanyMapper.namespaceEmployerId(employer);
		// given TX_TWC or CA_EDD (AgencyType form in Gemini), get the PTS
		// format for the same (CAEDD, etc)
		String agencyId = CompanyMapper.mapToPtsAgency(employerId.getAgencyIdType().getAgencyKey()).toString();
		String companyAgencyRegistrationId = employerId.getValue();

		//  matching agency was found - send it PTS
		CompanyRequestWrapper ptsCompanyRequestWrapper = new CompanyRequestWrapper();
		AdapterResult adapterResult = ptsCompanyRequestWrapper.addCompanyTaxAgencyId(
					sourceCompanyId, agencyId, companyAgencyRegistrationId);

		if (adapterResult.isSuccess()){
			setInteractionStateForEntity(employerId, InteractionEnum.ADDED);
		}else{
			throw new ServiceFailureException("Unable to add agencyId to PTS failed: \n REASONS: " + 
					PTSAdapterResultUtil.getAdapterResultMessagesAsDelimitedString(adapterResult));
		}
	}
	
	public static void addWorkerInfotoPTS(PTSEmployer ptsEmployer, Worker worker)
	throws ServiceFailureException, ValidationException {
		interactWorkerInfotoPTS(ptsEmployer, worker, ADD_WORKER);
	}
	

	public static void reHireWorkerInfotoPTS(PTSEmployer ptsEmployer,
			Worker worker) throws ValidationException, ServiceFailureException {
		interactWorkerInfotoPTS(ptsEmployer, worker, REHIRE_WORKER);
	}

	public static void terminateWorkerInfotoPTS(PTSEmployer ptsEmployer,
			Worker worker) throws ValidationException, ServiceFailureException {
		interactWorkerInfotoPTS(ptsEmployer, worker, TERMINATE_WORKER);
		
	}

	public static void updateWorkerInfotoPTS(PTSEmployer ptsEmployer, Worker worker) throws ValidationException, ServiceFailureException {
		interactWorkerInfotoPTS(ptsEmployer, worker, UPDATE_WORKER);
	}
	
	public static void deleteWorkerInfoFromPTS(PTSEmployer ptsEmployer, Worker worker) throws ValidationException, ServiceFailureException{
		interactWorkerInfotoPTS(ptsEmployer, worker, DELETE_WORKER);
	}
	
	private static void interactWorkerInfotoPTS(PTSEmployer ptsEmployer,
			Worker worker, int interaction_type) throws ValidationException, ServiceFailureException{
		if (!StatusUpdateHandlerUtil.isEntityAddedToPTS( ptsEmployer.getEmployer())){
			throw new ValidationException("Company Not added with PTS, can not process further");
		}
		
		// check if this worker has not been already added
		boolean isWorkerAdded = StatusUpdateHandlerUtil.isEntityAddedToPTS(worker);
		
		if (isWorkerAdded){
			if (interaction_type == ADD_WORKER) {
			throw new ValidationException("Worker already added, can not process further");
			}
		}else{
			if ((interaction_type == UPDATE_WORKER)||
				(interaction_type == TERMINATE_WORKER) ||
				interaction_type == REHIRE_WORKER ||
				interaction_type == DELETE_WORKER){
				throw new ValidationException("Worker not added, can not process further");
			}
		}

		// check is all necessary information is avl to add Worker to PSP
		List<RequiredData> missingList = WorkerSetupRules.getMissingDataForWorkflow(WorkerWorkflowType.PTSTaxReady, worker);
		
		if (missingList.size() > 0){
			// Data not sufficient to create an Assisted company in PSP
			StringBuffer sb = new StringBuffer();
			for (RequiredData requiredData : missingList) {
				sb.append(requiredData.getDescription()).append("  ");
			}
			throw new ValidationException("Insufficient data to add Employees to PTS "+ sb.toString());
		}
		
		sLogger.debug("Sending Employee Info to PTS for " + worker.getId());
		
		EmployeeAdapterDTO employeeDTO = EmployeeMapper.mapToEmployeeAdapterDTO(worker);
		
		EmployeeAdapterWrapper employeeAdapterWrapper = new EmployeeAdapterWrapper();
		
		AdapterResult adapterResult = new AdapterResult();
		InteractionEnum interaction = InteractionEnum.ADDED;
		
		if (interaction_type == ADD_WORKER){
			 adapterResult = employeeAdapterWrapper.addEmployee(employeeDTO);
			 interaction = InteractionEnum.ADDED;
		}else if (interaction_type == UPDATE_WORKER){
			 adapterResult = employeeAdapterWrapper.updateEmployee(employeeDTO);
			 interaction = InteractionEnum.UPDATED;
		}else if (interaction_type == TERMINATE_WORKER){
			Date terminatedDate = new Date();  // Default, assuming we are not able to get anydate
			DateRange empPeriod = worker.getEmploymentPeriod();
			if (empPeriod != null){
				terminatedDate = empPeriod.getEndDate();
			}else{
				sLogger.warn("Using current date as Termination Date for worker " + worker.getId());
			}
			adapterResult = employeeAdapterWrapper.terminateEmployee(employeeDTO.getSourceCompanyId(),
					 employeeDTO.getSourceEmployeeId(), DateMapper.mapDateToDTO(terminatedDate));
			interaction = InteractionEnum.DEACTIVATED;
		}else if (interaction_type == REHIRE_WORKER){
			adapterResult = employeeAdapterWrapper.rehireEmployee(employeeDTO.getSourceCompanyId(),
					 employeeDTO.getSourceEmployeeId(), employeeDTO.getHireDate());
			
			if (adapterResult.isSuccess()){
				// Blankly update the worker in case worker was updated while being in terminated state
				sLogger.debug("Updating a re-hired worker " + worker.getId());
				adapterResult = employeeAdapterWrapper.updateEmployee(employeeDTO);
				interaction = InteractionEnum.UPDATED;
			}
		}else if (interaction_type == DELETE_WORKER){
			adapterResult = employeeAdapterWrapper.deleteEmployee(employeeDTO.getSourceCompanyId(),
					 employeeDTO.getSourceEmployeeId());
			interaction = InteractionEnum.REMOVED;
		}
		
		
		if (adapterResult.isSuccess()){
			setInteractionStateForEntity(worker, interaction);
		}else{
			String errMsg = "Unable to send employee to PTS failed: \n REASONS: " + 
			PTSAdapterResultUtil.getAdapterResultMessagesAsDelimitedString(adapterResult);
			sLogger.error(errMsg);
			throw new ServiceFailureException(errMsg);
		}
	}

		

	public static void submitPayrollRunInfoToPTS(PTSEmployer ptsEmployer,
			PayrollRun payrollRun)throws ServiceFailureException, ValidationException {
		
		if (!StatusUpdateHandlerUtil.isEntityAddedToPTS(ptsEmployer.getEmployer())){
			throw new ValidationException("Company Not added with PTS, can not process further");
		}
		// check whether the payrollRun has already submitted to PTS
		if(StatusUpdateHandlerUtil.isEntityAddedToPTS(payrollRun)){
			throw new ValidationException("PayrollRun is already submitted, cannot proceed further");
		}
		
		PayrollRequestWrapper requestWrapper = new PayrollRequestWrapper();
		PayrollRunAdapterDTO payrollRunDTO = PayrollRunMapper.mapToPayrollRunDTO(ptsEmployer, payrollRun);
		
		SubmitPayrollResult adapterResult = requestWrapper.submitPayrollRun(payrollRunDTO);;
		
		if (adapterResult.isSuccess()){
			try {
				PTSPayrollRun ptsPayrollRun = PayrollRunMapper.mapToPTSPayrollRun(adapterResult, payrollRun);
				PTSPayrollRunEAO ptsPayrollRunEAO = ServiceLocator.getInstance().getService(PTSPayrollRunEAO.JNDI_NAME);
				ptsPayrollRunEAO.makePersistent(ptsPayrollRun);
				setInteractionStateForEntity(payrollRun, InteractionEnum.ADDED);
			}catch (Exception e) {
				// Do we need to revert changes made to PSP as our txns will roll back after this
				// exception
				throw new ServiceFailureException(e);
			}
		}else{
			throw new ServiceFailureException("Unable to Submit PayrollRun to PTS: \n REASONS: " + 
					PTSAdapterResultUtil.getAdapterResultMessagesAsDelimitedString(adapterResult));
		}
	}
	
	public static void voidPaychecks(PTSEmployer ptsEmployer, PayrollRun payrollRun)
							throws ServiceFailureException, ValidationException {
		Set<Paycheck> paychecks = payrollRun.getPaychecks();
		List<String> paycheckIdsToVoid = new ArrayList<String>(); 
		for (Paycheck paycheck : paychecks) {
			if(paycheck.getWorkflowStatus() == PaycheckWorkflowStatusEnum.VOIDED){
				if(!StatusUpdateHandlerUtil.isEntityAddedToPTS(paycheck)){
					paycheckIdsToVoid.add(String.valueOf(paycheck.getId()));
				}
			}
		}
		
		PayrollRequestWrapper requestWrapper = new PayrollRequestWrapper();
		
		String sourceCompanyId = CompanyMapper.namespaceEmployerId(ptsEmployer.getEmployer());
		String sourcePayrollRunId = String.valueOf(payrollRun.getId());
		List<PayrollTaxAdapterDTO> payrollTaxes = new ArrayList<PayrollTaxAdapterDTO>();
		VoidPayrollResult voidPayrollResult = requestWrapper.voidPaychecks(sourceCompanyId, sourcePayrollRunId, paycheckIdsToVoid, payrollTaxes);
		
		if(voidPayrollResult.isSuccess()){
			try {
				for (Paycheck paycheck : paychecks) {
					if(paycheck.getWorkflowStatus() == PaycheckWorkflowStatusEnum.VOIDED){
						if(!StatusUpdateHandlerUtil.isEntityAddedToPTS(paycheck)){
							setInteractionStateForEntity(paycheck, InteractionEnum.ADDED);
						}
					}
				}
				PTSVoidPayrollResult ptsVoidPayrollResult = PayrollRunMapper.mapToPTSVoidPayrollResult(voidPayrollResult, payrollRun);
				PTSVoidPayrollResultEAO ptsVoidPayrollResultEAO = ServiceLocator.getInstance().getService(PTSVoidPayrollResultEAO.JNDI_NAME);
				ptsVoidPayrollResultEAO.makePersistent(ptsVoidPayrollResult);
			}
			catch (Exception e) {
				throw new ServiceFailureException(e);
			}
		}
		else {
			throw new ServiceFailureException("Unable to Void Paycheck: \n REASONS: " + 
					PTSAdapterResultUtil.getAdapterResultMessagesAsDelimitedString(voidPayrollResult));
		}
			
	}
	    
	public static List<LiabilityBalance> getHistoricalLiabilityBalances(PTSEmployer ptsEmployer) throws ServiceFailureException {
		PayrollRequestWrapper requestWrapper = new PayrollRequestWrapper();
		String sourceCompanyId = CompanyMapper.namespaceEmployerId(ptsEmployer.getEmployer());
		LiabilityBalancesResult liabilityBalancesResult = requestWrapper.getHistoricalLiabilityBalances(sourceCompanyId);
		List<LiabilityBalance> liabilityBalances = null;
		if(liabilityBalancesResult.isSuccess()){
			liabilityBalances = PayrollRunMapper.mapToLiabilityBalances(liabilityBalancesResult.getBalances());
		}
		return liabilityBalances;
	}
	
	public static void sendPaymentsToPTS(PTSEmployer ptsEmployer, List<Payment> paymentsToAdd) throws ServiceFailureException {
		
			PayrollRequestWrapper requestWrapper = new PayrollRequestWrapper();
			TaxPaymentsAdapterDTO taxPaymentsAdapterDTO = 
				TaxPaymentsMapper.mapToTaxPaymentsAdapterDTO(ptsEmployer, paymentsToAdd);
			AdapterResult adapterResult = requestWrapper.submitHistoricalTaxPayments(taxPaymentsAdapterDTO);
			
			if (adapterResult.isSuccess()){
				try {
					EmployerPTSWorkflowComponent empPTSComp = ServiceLocator.getInstance().getService(EmployerPTSWorkflowComponent.JNDI_NAME);
					for (Payment payment:paymentsToAdd){
						empPTSComp.updatePPS_PTS_InteractionState(payment, InteractionEnum.ADDED);
					}
				}catch (Exception e) {
					// Do we need to revert changes made to PSP as our txns will roll back after this
					// exception
					throw new ServiceFailureException(e);
				}
			}else{
				throw new ServiceFailureException("Unable to Submit Historical Payments to PTS: \n REASONS: " + 
						PTSAdapterResultUtil.getAdapterResultMessagesAsDelimitedString(adapterResult));
			}
		
	}
}
