package com.intuit.sb.payroll.capability.pts.statehandler;

import java.util.Date;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.impl.EmployerPTSWorkflowTasks;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationException;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationRequest;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationTypeEnum;
import com.intuit.sb.payroll.capability.pts.notification.mapper.PTSNotificationRequestMapper;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.common.EntityService;
import com.intuit.sb.payroll.entity.model.BankAccount;
import com.intuit.sb.payroll.pts.entity.BankAccountState;
import com.intuit.sb.payroll.pts.entity.BankAccountStateEAO;
import com.intuit.sb.payroll.pts.entity.BankAccountStatusEnum;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;
import com.intuit.sb.payroll.pts.entity.PTSWorkflowState;
import com.intuit.sb.payroll.pts.entity.TaxServiceState;

public class BankAccountStatusUpdateHandler extends AbstractStatusUpdateHandler  {

	private static BankAccountStatusUpdateHandler sInstance = new BankAccountStatusUpdateHandler();

	private BankAccountStatusUpdateHandler() {
		registerStatusChangeWorkers();
	}

	
	public void validateNotificationRequest(PTSNotificationRequest ptsNotification)
			throws PTSNotificationException {
		
		// Check if this Handler is handling the right notification
		if (!ptsNotification.getNotificationType().equals(PTSNotificationTypeEnum.BANK_ACCOUNT_NOTIFICATION)) {
			throw new PTSNotificationException("Wrong Notification being handler here");
		}
		
		// Ensure that sourceBankAccountId is provided in the request
		if (!ptsNotification.getNotificationAttributes().containsKey(PTSNotificationRequestMapper.SOURCE_BANK_ACCOUNT_ID)){
			throw new PTSNotificationException("BankAccount verification Notification does not have sourceBankId");
		}
	}

	public static StatusUpdateHandler getInstance() {
		return sInstance;
	}
	
	// Inner classes for Individual state handlers

	static class GenericBankAccountStatusChangeWorker extends StatusChangeWorker {

		@Override
		public void updateStatus(
				PTSEmployer ptsEmployer, PTSNotificationRequest ptsNotification) throws PTSNotificationException {
			try {
				updateBankAccountStatus(ptsEmployer, ptsNotification);
			} catch (ValidationException e) {
				throw new PTSNotificationException(e.getLocalizedMessage());
			} catch (ServiceFailureException e) {
				throw new PTSNotificationException(e.getLocalizedMessage());
			}

		}

	}
	
	
	/**
     * This BankAccountRejectionStatusChangeWorker will handle for "BANKACCOUNT_VERIFICATION_FAILED_RECEIVED" Status, when Gemini receives
     * the BANKACCOUNT_VERIFICATION_FAILED_RECEIVED Notification from PTS
     *
     */
    @SuppressWarnings("unused")
    private static class BankAccountRejectionStatusChangeWorker extends GenericBankAccountStatusChangeWorker {
        public BankAccountRejectionStatusChangeWorker() {
            super();
            statusTypeChange.add(BankAccountStatusEnum.BANKACCOUNT_VERIFICATION_FAILED_RECEIVED);
        }
        @Override
        public void updateStatus(PTSEmployer ptsEmployer, PTSNotificationRequest ptsNotification) throws PTSNotificationException {
            super.updateStatus(ptsEmployer,  ptsNotification);
            EmployerPTSWorkflowTasks.handleBankAccountRejectionEvent(ptsNotification);
        }
    }
	
	/**
	 * Generic Post processing event after Bank Account update has been received from PTS
	 */
	@SuppressWarnings("unused")
    private static class CommonStatusChangeWorker extends GenericBankAccountStatusChangeWorker {
		public CommonStatusChangeWorker() {
			super();
			statusTypeChange.add(BankAccountStatusEnum.BANKACCOUNT_UNDER_VERIFICATION);
			statusTypeChange.add(BankAccountStatusEnum.BANKACCOUNT_VERIFIED_RECEIVED);
		}

    	@Override
		public void updateStatus(PTSEmployer ptsEmployer, PTSNotificationRequest ptsNotification) throws PTSNotificationException {
    		super.updateStatus(ptsEmployer, ptsNotification);

		}
    }

	/**
	 * Update the PTS Employer's BankAccount with the BankAccountState - add a new BankAccountState every time
	 * @param ptsEmployer
	 * @param ptsNotification
	 * @param bankAccount
	 * @throws ValidationException
	 * @throws ServiceFailureException
	 */
	public synchronized static void updateBankAccountStatus(PTSEmployer ptsEmployer, PTSNotificationRequest ptsNotification) 
			throws ValidationException, ServiceFailureException {
		ptsEmployer =  getPTSEmployer();
		String bankAccountId = 
			ptsNotification.getNotificationAttributes().get(PTSNotificationRequestMapper.SOURCE_BANK_ACCOUNT_ID);
		EntityService entityService = ServiceLocator.getInstance().getService(EntityService.JNDI_NAME);
		BankAccount bankaccount = entityService.findById(BankAccount.class, Long.valueOf(bankAccountId));
		
		if (bankaccount == null){
			throw new ValidationException("sourceBankAccountId provided in the notification does not exist");
		}
		BankAccountStateEAO bankAccountStateEao = ServiceLocator.getInstance().getService(BankAccountStateEAO.JNDI_NAME);
		BankAccountState baState = new BankAccountState();
		baState.setBankAcctId(new Long(bankaccount.getId()));
		baState.setStatus((BankAccountStatusEnum)ptsNotification.getNotificationStatus());
		if (ptsNotification.getTimestamp() != null){
			baState.setStatusDate(ptsNotification.getTimestamp());
		}else{
			baState.setStatusDate(new Date());
		}
		// FIXME baState.setPTSMessages(ptsNotification.getPTSMessage());
		baState.setActive(Boolean.TRUE);
		PTSWorkflowState ptsWorkflowState = ptsEmployer.getPTSWorkflowState();
		TaxServiceState taxServiceState = ptsWorkflowState.getTaxServiceState();
		baState = bankAccountStateEao.makeManaged(baState);
		taxServiceState.addNotificationStatusEntity(baState);
	}
}
