package com.intuit.sb.payroll.capability.task.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;

import org.apache.log4j.Logger;
import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.capability.task.TaskManager;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.entity.domain.Identifiable;
import com.intuit.sb.payroll.entity.model.Task;
import com.intuit.sb.payroll.entity.model.TaskStatusEnum;
import com.intuit.sb.payroll.entity.task.TaskEAO;

/**
 * Implementation of {@link TaskManager}.
 * 
 */
@Stateless(name = "TaskManager")
@SecurityDomain("BHPIAM")
@RolesAllowed( { "Authenticated" })
public class TaskManagerBean implements TaskManager {

	@EJB
	private TaskEAO taskEAO;

	private static Logger sLogger = PayrollLoggerFactory.makeLogger(TaskManagerBean.class);
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.task.ITaskManager#findCurrentTasks()
	 */
	public List<Task> findCurrentTasksInDateRange(Date fromDate, Date toDate) {
		return taskEAO.findCurrentTasks(fromDate, toDate);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.capability.task.TaskManager#findCurrentActiveTasks()
	 */
	public List<Task> findCurrentActiveTasks() {
		return taskEAO.findCurrentActiveTasks();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.capability.task.TaskManager#findActiveTasksByClassAndPayload(java.lang.String,
	 * java.lang.Long)
	 */
	public List<Task> findActiveTasksByClassAndPayload(String taskClass, Long payload) {
		return taskEAO.findActiveTasksByClassAndPayload(taskClass, payload);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.capability.task.TaskManager#findTaskByTaskNameAndPayLoad(java.lang.String,
	 * java.lang.Long)
	 */
	public List<Task> findTaskByTaskNameAndPayLoad(String taskName, Long workerId) {
		return taskEAO.findTaskByTaskNameAndPayLoad(taskName, workerId);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.task.ITaskManager#findTasksByClass(java.lang.String)
	 */
	public List<Task> findTasksByClass(String taskClass) {
		List<Task> allTasks = new ArrayList<Task>();
		List<Task> tasks = taskEAO.findAll();
		for (Task task : tasks) {
			String taskClassType = task.getTaskClass();
			if (taskClassType != null && taskClassType.equals(taskClass)) {
				allTasks.add(task);
			}
		}
		return allTasks;
	}

	/**
	 * 
	 * Changes the task identified by the passed in Id from New/InProgress to Canceled If the task is already Canceled
	 * or Closed no action is taken If the task corresponding to the id is not found or when the task status cannot be
	 * changed to Canceled a TaskException is thrown
	 * 
	 * @param id
	 * @throws TaskException
	 */
	public Task cancelTask(long id) throws ServiceFailureException {
		Task task = findTaskInSystem(id);
		TaskUtils.setTaskStatus(task, TaskStatusEnum.CANCELED);
		return task;
	}

	/**
	 * 
	 * Changes the task identified by the passed in Id from New/InProgress to Canceled If the task is already Canceled
	 * or Closed no action is taken If the task corresponding to the id is not found or when the task status cannot be
	 * changed to Canceled a TaskException is thrown
	 * 
	 * @param task
	 *            - that Task that needed to be canceled
	 * @throws TaskException
	 */
	public Task cancelTask(Task task) throws ServiceFailureException {
		return cancelTask(task.getId());
	}

	public Task reOpenTask(Task task) throws ServiceFailureException {
		task.setStatus(TaskStatusEnum.STARTED);
		task.setCompletionDate(null);
		task = taskEAO.makePersistent(task);
		return task;
	}

	/**
	 * 
	 * Changes the task identified by Id from New/InProgress to Closed If the task is already Canceled or Closed no
	 * action is taken If the task corresponding to the id is not found a TaskException is thrown
	 * 
	 * @param id
	 * @throws TaskException
	 */
	public Task closeTask(long id) throws ServiceFailureException {
		Task task = findTaskInSystem(id);
		closeTask(task);
		return task;
	}

	public void closeAllTaskByTaskNameAndPayload(String taskName, long payload) throws ServiceFailureException{
		List<Task> tasksToBeClosed = findTaskByTaskNameAndPayLoad(taskName, payload);
		for (Task task : tasksToBeClosed) {
        	if(!task.getStatus().equals(TaskStatusEnum.COMPLETED)){
                closeTask(task);
        	}
        }
	}
	
	/**
	 * 
	 * Changes the task identified by Id from New/InProgress to Closed If the task is already Canceled or Closed no
	 * action is taken If the task corresponding to the id is not found a TaskException is thrown
	 * 
	 * @param task
	 *            - the Task that need to be changed to closed state
	 * @throws TaskException
	 */
	public Task closeTask(Task task) throws ServiceFailureException {
		TaskUtils.setTaskStatus(task, TaskStatusEnum.COMPLETED);
		sLogger.debug("Closing Task "+ TaskUtils.taskToString(task));
		if (taskEAO.notPersistent(task)) {
			task = taskEAO.makeManaged(task);
		}

		return task;
	}

	/**
	 * Checks whether the Task represented by Id is userManaged or not
	 * 
	 * @param id
	 *            - the unique Id of the Task
	 * @return true if user managable, else false
	 * @throws TaskException
	 *             if the Task corresponding to the Id is not found in the system
	 */
	public boolean isUserManagable(long id) throws ServiceFailureException {
		Task task = findTaskInSystem(id);
		return task.isUserManagable();
	}

	/**
	 * Checks whether the Task is userManaged or not
	 * 
	 * @param task
	 *            - the Task that needs to be checked
	 * @return true if user managable, else false
	 * @throws TaskException
	 *             if the Task is not found in the system
	 */
	public boolean isUserManagable(Task task) throws ServiceFailureException {
		if (task == null) {
			throw new ServiceFailureException("Task can not be found in the System");
		}
		return isUserManagable(task.getId());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.capability.task.TaskManager#addTask(com.intuit.sb.payroll.entity.model.Task)
	 */
	public Task addTask(Task task) {
		task = taskEAO.makePersistent(task);
		TaskUtils.logTask(task);
		return task;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.capability.task.TaskManager#updateTask(com.intuit.sb.payroll.entity.model.Task)
	 */
	public Task updateTask(Task task) {
		if (taskEAO.notPersistent(task)) {
			task = taskEAO.makeManaged(task);
		}
		TaskUtils.logTask(task);
		return task;
	}

	/**
	 * returns the overdue tasks for the user
	 * 
	 * @return a list of tasks
	 * @throws TaskException
	 */
	public List<Task> getOverdueTasks() throws ServiceFailureException {
		return taskEAO.getOverdueTasks();
	}

	/**
	 * returns the active tasks in the system for the user Active means Task(s) in New or InProgress state
	 * 
	 * @return a list of Active Task(s)
	 * @throws TaskException
	 */
	public List<Task> getActiveTasks(Date fromDate, Date toDate) throws ServiceFailureException {
		return taskEAO.getActiveTasks(fromDate, toDate);
	}

	/**
	 * returns the tasks in the system for the user context in the Date range specified The Task collection returned
	 * will have even the Canceled and Closed tasks
	 * 
	 * @return a list of Active Task(s)
	 * @throws TaskException
	 */
	public List<Task> getTasks(Date fromDate, Date toDate) throws ServiceFailureException {
		return taskEAO.getTasks(fromDate, toDate);
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.task.TaskManager#deleteTasksByClass(java.lang.String)
	 */
	public void deleteTasksByClass(String className) {
		List<Task> tasks = findTasksByClass(className);
		for (com.intuit.sb.payroll.entity.model.Task task : tasks) {
 		   taskEAO.makeTransient(task);
		}		
	}
	/******************* utility functions ************************************/

	private Task findTaskInSystem(long id) throws ServiceFailureException {
		Task task = taskEAO.findById(id);

		if (task == null) // if existing task is not fond in the system throw exception
		{
			throw new ServiceFailureException("Task could not be found in the system"); // TODO - [rj] move to resource bundle
		}
		return task;
	}

	public List<Task> getTaskByTaskName(List<Task> tasks, String taskName, Identifiable entity) {
		Long payload = new Long(0);
		if (entity != null) {
			payload = new Long(entity.getId());
		}
		List<Task> foundtasks = new ArrayList<Task>();
		for (Task task : tasks) {
			if ((task.getName().equals(taskName)) && task.getPayload().equals(payload)) {
				foundtasks.add(task);
			}
		}
		return foundtasks;
	}
}
