package com.intuit.sb.payroll.capability.taxes;

import java.util.Date;
import java.util.List;

import javax.ejb.Local;

import com.intuit.payroll.types.PayrollException;
import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.ppe.AgencyView;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.Jurisdiction;
import com.intuit.sb.payroll.entity.model.Law;
import com.intuit.sb.payroll.entity.model.Tax;
import com.intuit.sb.payroll.entity.model.TaxDtl;
import com.intuit.sb.payroll.entity.model.UnemploymentInsDocDtl;
import com.intuit.sb.payroll.entity.model.Worker;

@Local
public interface TaxComponent {
	
	@SuppressWarnings("nls")
	public static final String JNDI_NAME = "TaxComponent/local";

	/**
	 * Creates a non-persistent TaxDtl from a Tax.
	 * 
	 * @param tax
	 * @return
	 * @throws ValidationException when null value is passed.
	 */
	public TaxDtl createTaxDtlFromTax(Tax tax) throws ValidationException;
	
	/**
	 * Adds a TaxDtl to a worker.
	 * 
	 * @param taxDtl
	 * @param worker
	 * @return the latest copy of the Worker.
	 * @throws ValidationException when null or invalid params are passed in.
	 */
	public Worker addTaxToWorker(TaxDtl taxDtl, Worker worker) throws ValidationException;
	
	/**
	 * Applies applicable taxes to a Worker and returns the latest copy of the worker.
	 * 
	 * @param worker
	 * @return
	 * @throws ValidationException
	 * @throws WorkflowException
	 * @throws ServiceFailureException 
	 */
	public Worker applyApplicableTaxes(Worker worker) throws ValidationException, WorkflowException, ServiceFailureException;

	public List<Law> determineApplicableTaxes(Worker worker) throws ValidationException, WorkflowException, ServiceFailureException;

	/**
	 * determines whether SSN can be allowed instead of EIN for the Employer if EIN is not available
	 * 
	 * @param employer
	 * @return
	 * @throws ServiceFailureException 
	 */
	public boolean isSSNAllowedForEIN(Employer employer) throws ServiceFailureException;

	/**
	 * determines whether SSN can be allowed instead of EIN for the Employer if EIN is not available
	 * 
	 * @param employer
	 * @return
	 * @throws ValidationException 
	 */
	public List<AgencyView> getEmployerAgencyDetails(Employer employer, List<Jurisdiction> jurisdictions) throws ValidationException;

	/**
	 * Fetches all the Unemployment Insurances for the employer
	 * 
	 * @return List of Unemployment Insurances for the employer
	 */
	public List<UnemploymentInsDocDtl> getEmployerUnemploymentInsurances();

	/**
	 * Returns the UnemploymentInsDocDtl from the repository for the unique Id provided
	 * 
	 * @param uniqueId
	 * @return
	 * @throws ServiceFailureException 
	 * @throws PayrollException
	 */
	public UnemploymentInsDocDtl getEmployerUnemploymentInsurance(Long uniqueId) throws ServiceFailureException;

	
	/**
	 * Update the UnemploymentInsDocDtl entity
	 * 
	 * @param unemploymentInsDocDtl
	 * @return
	 * @throws ValidationException 
	 * @throws PayrollException
	 */
	public UnemploymentInsDocDtl updateEmployerUnemploymentInsurance(UnemploymentInsDocDtl unemploymentInsDocDtl) throws ValidationException;
	
	/**
	 * Get current quarter rate
	 */
	public UnemploymentInsDocDtl getCurrentQuarterRate(Date date) throws ServiceFailureException;
	
	/**
	 * Update a list of Employer UnemploymentInsurances
	 * @param list of UnemploymentInsurances
	 * @return list of UnemploymentInsurances
	 */
	public List<UnemploymentInsDocDtl> updateUnemploymentInsurances(List<UnemploymentInsDocDtl> unList) throws ServiceFailureException,ValidationException;
}
