package com.intuit.sb.payroll.capability.taxes.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;

import com.intuit.payroll.types.PayrollException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.model.Law;
import com.intuit.sb.payroll.entity.model.PayrollItem;
import com.intuit.sb.payroll.entity.model.TaxDtl;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * This class is responsible for providing utility functions for Rules Files. The methods in the class are expected to
 * be thread-safe.
 * 
 */
public class LawUtils {
	private static Logger sLogger = PayrollLoggerFactory.makeLogger(LawUtils.class);

	private LawUtils() {
		// Disable instantiation
	}

	/**
	 * Removes laws from a given list for a specified tax mnemonic. Does not report any errors if the corresponding law
	 * does not exist in the input list. Currently the following Tax Mnemonics exist in the database<br>
	 * 1. US-FED-SST = Social Security Tax <br>
	 * 2. US-FED-MEDT = Medicare Tax <br>
	 * 3. US-FED-FIT = Federal Income Tax<br>
	 * 4. US-FED-FUI = Federal Unemployment Insurance<br>
	 * 5. US-CA-PIT = CA Personal Income Tax<br>
	 * 6. US-CA-SDI = CA State Disability Insurance<br>
	 * 7. US-CA-SUI = CA State Unemployment Insurance<br>
	 * 8. US-CA-ETT = CA Employment Training Tax<br>
	 * 9. US-TX-SUI = TX State Unemployment Insurance<br>
	 * 10 US-CA-SF-PT = CA SF Payroll Tax
	 * 
	 * @param taxMnemonic the tax mnemonic whose laws need to be pruned from the list
	 * @param laws the list of laws
	 * @throws ValidationException
	 * @throws PayrollException if taxMnemonic is null or empty string
	 * @throws PayrollException if laws is null
	 */
	public static void removeTaxMnemonicsLawsFromList(String taxMnemonic, List<Law> laws) throws ValidationException {
		sLogger.trace("Entered removeTaxMnemonicsLawsFromList");

		PayrollParamValidator.checkIsNotNullOrBlankString(taxMnemonic, "taxMnemonic");
		PayrollParamValidator.checkIsNotNull(laws, "inputLaws");
		for (int i = 0; i < laws.size(); i++) {
			Law law = (Law) laws.get(i);
			if (taxMnemonic.equals(law.getName())) {
				laws.remove(law);
				if (sLogger.isDebugEnabled()) {
					sLogger.debug("Removed=" + law.getName());
				}
				break;
			}
		}
		sLogger.trace("Exited removeTaxMnemonicsLawsFromList");
	}

	/**
	 * Removes laws from a given list for a specified Jurisdiction. Does not report any errors if the corresponding law
	 * does not exist in the input list. Currently the following Juridsdiction strings are supported<br>
	 * 1. US-FED for US Federal<br>
	 * 2. US-CA for California State<br>
	 * 3. US-TX for Texas State<br>
	 * 
	 * @param jurisdiction the jurisdiction whose laws need to pruned from the list
	 * @param laws the list of laws
	 * @throws ValidationException 
	 * @throws PayrollException if taxMnemonic is null or empty string
	 * @throws PayrollException if laws is null
	 */
	public static void removeJurisdictionsLawsFromList(String jurisdiction, List<Law> laws) throws ValidationException {
		sLogger.trace("Entered removeJurisdictionsLawsFromList");

		PayrollParamValidator.checkIsNotNull(jurisdiction, "jurisdiction");
		PayrollParamValidator.checkIsNotNull(laws, "inputLaws");

		List<Law> lawsToRemove = new ArrayList<Law>();
		for (Law law: laws) {
			if (law.getName().contains(jurisdiction)) {
				lawsToRemove.add(law);
				sLogger.debug("Removed=" + law.getName());
			}
		}
		laws.removeAll(lawsToRemove); // remove all the laws that need to be removed

		sLogger.trace("Exited removeJurisdictionsLawsFromList");
	}

	public static TaxDtl getMatchingTaxDetail(List<PayrollItem> payrollItemList, String lawId, Worker worker) {
		PayrollItem payrollItemFound = getPayrollItem(payrollItemList, lawId);
		if (payrollItemFound == null) {
			throw new IllegalStateException("Mapping failure, cannot find lawId = " + lawId + " in the system");
		}
		Set<TaxDtl> taxDtls = worker.getTaxDtls();
		for (TaxDtl pid: taxDtls) {
			PayrollItem payrollItem = pid.getPayrollItem();
			if (payrollItem.getLawId().getValue().equals(payrollItemFound.getLawId().getValue())) {
				return pid;
			}
		}
		return null;
	}

	/**
	 * Returns the first PayrollItem Matching the LawId supplied. There is no defined order in iteration for searching
	 * lawId
	 * 
	 * @param lawId
	 * @return PayrollItem with matching lawId, otherwise null
	 * @throws IllegalArgumentException if the lawId input is null
	 */
	public static PayrollItem getPayrollItem(List<PayrollItem> payrollItemList, String lawId) {
		if (lawId == null) {
			throw new IllegalArgumentException("Supplied lawId is null");
		}
		for (PayrollItem item : payrollItemList) {
			if (lawId.equals(item.getLawId().getValue())) {
				return item;
			}
		}
		return null;
	}

}
