package com.intuit.sb.payroll.capability.worker.impl;

import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Set;

import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;

import org.apache.log4j.Logger;
import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.task.TaskManager;
import com.intuit.sb.payroll.capability.task.impl.TaskUtils;
import com.intuit.sb.payroll.capability.taxes.TaxComponent;
import com.intuit.sb.payroll.capability.worker.FindWorkersFilter;
import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.capability.worker.WorkerWorkflowType;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunInfo;
import com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleComponent;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.PayrollResourceManager;
import com.intuit.sb.payroll.common.ResourceKeyConstants;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.common.WorkflowEventType;
import com.intuit.sb.payroll.common.management.PayrollService;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.domain.Identifiable;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.EmploymentPeriod;
import com.intuit.sb.payroll.entity.model.EmploymentStatusEnum;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;
import com.intuit.sb.payroll.entity.model.Task;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.worker.WorkerEAO;
import com.intuit.sb.payroll.message.PayrollMessageService;

/**
 * This class provides basic methods to operate on an Worker entity
 * 
 * $Author$ 
 * $Revision$ 
 * $DateTime$
 */
@Stateless(name = "WorkerComponent")
@SecurityDomain("BHPIAM")
@RolesAllowed( { "Authenticated" })
public class WorkerComponentBean implements WorkerComponent {

	@EJB
	private WorkerEAO workerEAO;

	@EJB
	private TaskManager taskManager;

	@EJB
	private PayrollMessageService messageService;
	
	//@EJB
	private static PayrollScheduleComponent payrollScheduleComponent = ServiceLocator.getInstance().getService(PayrollScheduleComponent.JNDI_NAME);;
	
	// FIXME [Impl] we seem to have a circular dependency somewhere...
	private static TaxComponent taxComponent = ServiceLocator.getInstance().getService(TaxComponent.JNDI_NAME);

	private static Logger sLogger = PayrollLoggerFactory.makeLogger(WorkerComponentBean.class);
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.intuit.sb.payroll.capability.worker.WorkerComponent#findByIdentifiable(com.intuit.sb.payroll.entity.domain
	 * .Identifiable)
	 */
	public Worker findWorkerByIdentifiable(Identifiable identifiable) throws ServiceFailureException {
		Worker worker = null;
		try {
			worker = workerEAO.findByIdentifiable(identifiable);
		}

		catch (Exception e) {
			// TODO For Optimistic lock errors, we probably need something clearer than this.
			throw new ServiceFailureException(e);
		}

		if (worker == null) {
			throw new ServiceFailureException(PayrollResourceManager.getInstance().getString(
					ResourceKeyConstants.ID_NOT_FOUND_EXCEPTION)
					+ identifiable.toString());
		}
		return worker;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.capability.worker.WorkerComponent#getWorker(java.lang.String)
	 */
	public Worker findWorker(Long workerUniqueId) throws ServiceFailureException, ValidationException {
		PayrollParamValidator.checkIsNotNull(workerUniqueId, "workerId");
		Worker worker = null;
		try {
			worker = workerEAO.findById(workerUniqueId);
		} catch (Exception e) {
			throw new ServiceFailureException(e);
		}

		if (worker == null) {
			throw new ServiceFailureException(PayrollResourceManager.getInstance().getString(
					ResourceKeyConstants.ID_NOT_FOUND_EXCEPTION)
					+ workerUniqueId.toString());
		}
		return worker;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.capability.worker.WorkerComponent#findAllWorkers()
	 */
	public List<Worker> findAllWorkers() throws ServiceFailureException {
		return workerEAO.findAll();
	}

    public List<Worker> findWorkersByFilter(FindWorkersFilter filter) throws ServiceFailureException {
		List<Worker> workerList = null;
		if (FindWorkersFilter.ALL.equals(filter)) {
		    workerList = workerEAO.findAll();
		} else if (FindWorkersFilter.READY_FOR_PAYROLL.equals(filter)) {
		    workerList = workerEAO.findWorkersReadyForPayroll();
		} else if (FindWorkersFilter.NOT_READY_FOR_PAYROLL.equals(filter)) {
		    workerList = workerEAO.findWorkersNotReadyForPayroll();
		} else if (FindWorkersFilter.ACTIVE.equals(filter)) {
			workerList = workerEAO.findActiveWorkers();
		} else if (FindWorkersFilter.INACTIVE.equals(filter)) {
			workerList = workerEAO.findInactiveWorkers();
		}
		return workerList;
    }

	public List<Worker> findWorkersByWorkLocation(Long workLocationId) throws ServiceFailureException {
		return workerEAO.findWorkersByWorkLocation(workLocationId);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.capability.worker.WorkerComponent#addWorker(com.intuit.sb.payroll.entity.model.Worker)
	 */
	public Worker addWorker(Worker worker) throws WorkflowException, ValidationException, ServiceFailureException {

		try {
			WorkerValidator.getInstance().validateEntity(worker);
			sLogger.debug("Adding Worker "+ worker.getId());
			// State flags based on completed data.
			WorkerSetupRules.setWorkerStateProperties(worker);

			worker = workerEAO.makePersistent(worker);
			dispatchWorkflowEvent(WorkflowEventType.ADD_WORKER, worker);
		} catch (ValidationException ve) {
			sLogger.debug(ve);
			throw ve;
		} catch (ServiceFailureException sfe) {
			sLogger.debug(sfe);
			throw sfe;
		} catch (Exception e) {
			sLogger.debug(e);
			throw new ServiceFailureException(e);
		}
		return worker;
	}

	/**
     * Sink for workflow events in this component.  Handles side-effects that are germain to this component, then
     * propagates the event to the Messaging infrastructure.
     * 
	 * @param workflowEventType
	 * @param worker
	 * @throws ServiceFailureException
	 * @throws ValidationException 
	 */
	private void dispatchWorkflowEvent(WorkflowEventType workflowEventType, Worker worker) throws ServiceFailureException, ValidationException {
		//Add/Update/Close tasks we control
		//FIXME [th] the list of tasks should be narrowed by worker, and not done here but in the tasks delegate.
		//FIXME furthermore, shouldn't we be only looking at active tasks?  Might depend on context.
		List<Task> tasks = taskManager.findTasksByClass(WorkerComponentTasks.SETUP_EMPLOYEE_CLASS);
		
		Date payrollRunDate;
		PayrollSchedule payrollSchedule = payrollScheduleComponent.getDefaultPayrollSchedule();
		
		if (payrollSchedule == null) {
			payrollRunDate = null;
		}
		else {
			PayrollRunInfo payrollRunInfo = payrollScheduleComponent.getNextScheduledPayrollRunInfo(payrollSchedule);
			payrollRunDate = payrollRunInfo.getPayrollRunDate();
		}
		
		switch (workflowEventType) {
			case ADD_WORKER:
				WorkerComponentTasks.handleAddFirstEmployeeEvent(taskManager, worker);
				WorkerComponentTasks.handleWorkerChangedEvent(taskManager, tasks, worker, payrollRunDate);
				WorkerComponentTasks.handleWorkerWithHoldingEvent(taskManager, tasks, worker);
				break;
			case UPDATE_WORKER:
				WorkerComponentTasks.handleWorkerChangedEvent(taskManager, tasks, worker, payrollRunDate);
				WorkerComponentTasks.handleWorkerWithHoldingEvent(taskManager, tasks, worker);
				break;
			case TERMINATE_WORKER:
				WorkerComponentTasks.removeTasksForWorker(taskManager, worker);
				break;
			case REHIRE_WORKER:
				WorkerComponentTasks.handleAddFirstEmployeeEvent(taskManager, worker);
				WorkerComponentTasks.handleWorkerChangedEvent(taskManager, tasks, worker, payrollRunDate);
				WorkerComponentTasks.handleWorkerWithHoldingEvent(taskManager, tasks, worker);
				break;
			case DELETE_WORKER:
				break;
			default:
				break;
		}
		//Publish event to interested parties 
		messageService.dispatchWorkerEvent(workflowEventType, worker);
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.worker.WorkerComponent#handleEmployerEvent(com.intuit.sb.payroll.message.WorkflowEventType, com.intuit.sb.payroll.entity.model.Employer)
	 */
	public void handleEmployerEvent(WorkflowEventType workflowEventType, Employer employer) throws ServiceFailureException {
		//If the company was just created, create a task to set up the first employee.
		if (WorkflowEventType.CREATE_EMPLOYER == workflowEventType) {
			// Create new Add Employee Task
			try {
				WorkerComponentTasks.createAddEmployeeTask(taskManager, employer);
			} catch (Exception e) {
				throw new ServiceFailureException(e);
			} 
		}
	}
	
	public void handlePayrollScheduleEvent(WorkflowEventType workflowEventType) throws ServiceFailureException, ValidationException {
		PayrollSchedule payrollSchedule = payrollScheduleComponent.getDefaultPayrollSchedule();
		Date payrollRunDate;
		if (payrollSchedule == null) {
			payrollRunDate = null;
		}
		else {
			PayrollRunInfo payrollRunInfo = payrollScheduleComponent.getNextScheduledPayrollRunInfo(payrollSchedule);
			payrollRunDate = payrollRunInfo.getPayrollRunDate();
		}
		
		if (WorkflowEventType.ADD_PAYROLL_SCHEDULE == workflowEventType) {
			WorkerComponentTasks.updateWorkerTasks(taskManager, payrollRunDate);
		} else if (WorkflowEventType.DELETE_PAYROLL_SCHEDULE == workflowEventType) {
			WorkerComponentTasks.updateWorkerTasks(taskManager, null);
		} else if (WorkflowEventType.UPDATE_PAYROLL_SCHEDULE == workflowEventType) {
			WorkerComponentTasks.updateWorkerTasks(taskManager, payrollRunDate);
		}
	}
	
	public void handlePayrollRunEvent(WorkflowEventType workflowEventType) throws ServiceFailureException, ValidationException {
		PayrollSchedule payrollSchedule = payrollScheduleComponent.getDefaultPayrollSchedule();
		Date payrollRunDate;
		if (payrollSchedule == null) {
			payrollRunDate = null;
		}
		else {
			PayrollRunInfo payrollRunInfo = payrollScheduleComponent.getNextScheduledPayrollRunInfo(payrollSchedule);
			payrollRunDate = payrollRunInfo.getPayrollRunDate();
		}
		
		if (WorkflowEventType.COMMIT_SCHEDULED_PAYROLL_RUN == workflowEventType) {
			WorkerComponentTasks.updateWorkerTasks(taskManager, payrollRunDate);
		}
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.intuit.sb.payroll.capability.worker.WorkerComponent#saveWorker(com.intuit.sb.payroll.entity.model.Worker)
	 */
	public Worker updateWorker(Worker worker) throws WorkflowException, ValidationException, ServiceFailureException {

		try {
			WorkerValidator.getInstance().validateEntity(worker);

			// State flags based on completed data.
			WorkerSetupRules.setWorkerStateProperties(worker);
			if (workerEAO.notPersistent(worker)) {
				worker = workerEAO.makeManaged(worker);
			}
			// Closes or adds tasks
			dispatchWorkflowEvent(WorkflowEventType.UPDATE_WORKER, worker);
		} catch (ValidationException ve) {
			sLogger.debug(ve);
			throw ve;
		} catch (Exception e) {
			sLogger.debug(e);
			throw new ServiceFailureException(e);
		}
		return worker;
	}
	
	public void deleteWorker(Worker worker) throws ValidationException, ServiceFailureException {
		WorkerValidator.getInstance().validateEntity(worker);
		//Ensure there are no paychecks against this worker.
		Set<Paycheck> paychecks = worker.getPaychecks();
		if (paychecks != null && paychecks.size() > 0) {
			throw new ValidationException("Cannot delete a worker with Paychecks");
		}
		//Before making the worker transient, remove tasks of this worker that are indirectly created while creation of this worker.
		TaskUtils.removeAllWorkerTasks(worker.getId());
		workerEAO.makeTransient(worker);
		dispatchWorkflowEvent(WorkflowEventType.DELETE_WORKER, worker);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.intuit.sb.payroll.capability.worker.WorkerComponent#terminateWorker(com.intuit.sb.payroll.entity.model.Worker
	 * , java.util.Date)
	 */
	public Worker terminateWorker(Worker worker, Date terminationDate) throws WorkflowException, ValidationException,
			ServiceFailureException {
		WorkerValidator.getInstance().validateEntity(worker);
		PayrollParamValidator.checkIsNotNull(terminationDate, "terminationDate");
		PayrollParamValidator.checkIsNotNull(worker.getEmploymentPeriod(), "worker.getEmploymentPeriod()");
		Calendar todayCal = PayrollService.getInstance().getSystemDate();
		Date today = todayCal.getTime();
		// Validate worker is employed
		if (worker.getEmploymentStatus() == EmploymentStatusEnum.TERMINATED) {
			throw new ValidationException("You cannot terminate an already terminated worker.");
		}

		final Date endDate = worker.getEmploymentPeriod().getEndDate();
		if (endDate != null && DateUtil.isBeforeOrOnSameDay(endDate, today)) {
			// They are already terminated
			// FIXME can you change the date of a terminated employee? For now , no.
			throw new ValidationException("Termination of this worker has already been scheduled");
		} else {
			// ?
		}

		final Date startDate = worker.getEmploymentPeriod().getStartDate();
		if (null == startDate) {
			throw new ValidationException("Worker is not yet employed and cannot be terminated");
		}

		// Validate termination date, then apply it to current period
		if (DateUtil.isAfterOrOnSameDay(terminationDate, startDate)) {
			worker.getEmploymentPeriod().setEndDate(terminationDate);
		} else {
			throw new ValidationException("Termination date cannot be before the hire date.");
		}

		// FIXME - the next two values may have to be set as-of a certain date. Should this be part of the
		// EmploymentPeriod?
		// Set state to inactive
		worker.setActive(false);

		// Set employment status to TERMINATED
		worker.setEmploymentStatus(EmploymentStatusEnum.TERMINATED);

		// Run state evaluation
		WorkerSetupRules.setWorkerStateProperties(worker);

		// Make managed if out of session
		worker = workerEAO.makeManaged(worker);

		// Closes or adds tasks
		dispatchWorkflowEvent(WorkflowEventType.TERMINATE_WORKER, worker);

		return worker;
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.worker.WorkerComponent#hireExistingWorker(com.intuit.sb.payroll.entity.model.Worker, java.util.Date)
	 */
	public Worker hireExistingWorker(Long workerId, Date hireDate) throws WorkflowException, ValidationException,
			ServiceFailureException {
		PayrollParamValidator.checkIsNotNull(hireDate, "hireDate");
		Worker worker = findWorker(workerId);
		PayrollParamValidator.checkIsNotNull(worker.getEmploymentPeriod(), "worker.getEmploymentPeriod()");

		//Ensure they are not already employed
		if (worker.getEmploymentStatus() != EmploymentStatusEnum.TERMINATED) {
			throw new ValidationException("Worker is already employed");
		}
		
		DateRange priorPeriod = worker.getEmploymentPeriod();
		
		//Hire date cannot be before the existing hire date
		if (hireDate.before(priorPeriod.getStartDate())) {
			throw new ValidationException("New hire date precedes prior hire date");
		}
		
		// If the termination date is later than today or after the new hireDate, just remove the end date.
		if (priorPeriod.getEndDate() != null && (DateUtil.isAfterOrOnSameDay(priorPeriod.getEndDate(), hireDate))) {
			worker.getEmploymentPeriod().setEndDate(null);
		} else {
			//Store off the old period
			EmploymentPeriod historicalEmploymentPeriod = new EmploymentPeriod();
			historicalEmploymentPeriod.setEmploymentPeriod(priorPeriod);
			worker.addEmploymentPeriod(historicalEmploymentPeriod);
			// Start a new employment period starting on hire date.
			DateRange employmentPeriod = new DateRange(hireDate, null);
			worker.setEmploymentPeriod(employmentPeriod);
		}
		// Status change
		worker.setEmploymentStatus(EmploymentStatusEnum.EMPLOYED);
		worker.setActive(true);
		
		// Run state evaluation
		WorkerSetupRules.setWorkerStateProperties(worker);

		// Make managed if out of session
		worker = workerEAO.makeManaged(worker);

		// Closes or adds tasks
		dispatchWorkflowEvent(WorkflowEventType.REHIRE_WORKER, worker);
		
		return worker;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.intuit.sb.payroll.capability.worker.WorkerComponent#isWorkerReadyToPay(com.intuit.sb.payroll.entity.model
	 * .Worker)
	 */
	public boolean isWorkerReadyToPay(Worker worker) {
		return WorkerSetupRules.isReadyForWorkflow(WorkerWorkflowType.Payroll, worker);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.intuit.sb.payroll.capability.worker.WorkerComponent#assignWorkLocationToWorker(com.intuit.sb.payroll.entity
	 * .model.WorkLocation, com.intuit.sb.payroll.entity.model.Worker)
	 */
	public Worker assignWorkLocationToWorker(WorkLocation workLocation, Worker worker) throws WorkflowException,
			ValidationException, ServiceFailureException {

		// Validation - Worker cannot be null, Location can.
		if (null == worker) {
			throw new ValidationException("Cannot assign a null worker to a work location");
		}
		// Assign this worker to the given WorkLocation.
		// FIXME [Impl] If the Worker is already assigned to a worklocation end the effectivity
		// of that one and start the new one. See requirements.
		worker.setWorkLocation(workLocation);

		//FIXME: publish workflowevent
		//SideEffect handling - Recalculate the taxes if applicable
		worker = taxComponent.applyApplicableTaxes(worker);

		worker = this.updateWorker(worker);

		return worker;
	}
}
