package com.intuit.sb.payroll.capability.worker.impl;

import java.util.Calendar;
import java.util.List;
import java.util.Set;

import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;

import org.apache.log4j.Logger;
import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerPTSWorkflowComponent;
import com.intuit.sb.payroll.capability.pts.adapter.PTSRequestInvoker;
import com.intuit.sb.payroll.capability.pts.statehandler.StatusUpdateHandlerUtil;
import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.capability.worker.WorkerPTSWorkflowComponent;
import com.intuit.sb.payroll.capability.worker.WorkerWorkflowType;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.WorkflowEventType;
import com.intuit.sb.payroll.common.i18n.LocaleGuru;
import com.intuit.sb.payroll.entity.model.EmploymentStatusEnum;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.workerpayments.PayrollRunEAO;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;

/**
 * This class provides basic Workflow methods to interact with PTS for 
 * Worker entity
 * 
 * $Author: amalik $ 
 * $Revision: #39 $ 
 * $DateTime: 2009/05/14 22:55:31 $
 */
@Stateless(name = "WorkerPTSWorkflowComponent")
@SecurityDomain("BHPIAM")
@RolesAllowed( { "Authenticated" })
public class WorkerPTSWorkflowComponentBean implements
		WorkerPTSWorkflowComponent {
	private static Logger sLogger = PayrollLoggerFactory.makeLogger(WorkerPTSWorkflowComponentBean.class);
	
	@EJB
	EmployerPTSWorkflowComponent employerPTSComponent;
	
	@EJB
	WorkerComponent workerComponent;
	
	@EJB
	PayrollRunEAO payrollRunEAO;
	
	
	public void handleWorkerEvent(WorkflowEventType eventType, Worker worker)
			throws ServiceFailureException {
		
		if (eventType == null){
			sLogger.warn("Null WorflowEvent received");
			return;
		}
		sLogger.debug("Worker Event received "+ eventType + "for Worker "+ worker.getId());
		
		// Act only when current company is in PTS
		PTSEmployer ptsEmployer = employerPTSComponent.getPTSEmployer();
		if (!StatusUpdateHandlerUtil.isEntityAddedToPTS(ptsEmployer.getEmployer())){
			sLogger.debug("Employer "+ ptsEmployer.getEmployer().getId() + " not added to PTS as not yet assisted");
			return;
		}
		
		if (!WorkerSetupRules.isReadyForWorkflow(WorkerWorkflowType.PTSTaxReady, worker)){
			sLogger.debug("Worker "+ worker.getId() + " not added to PTS as its not ready yet");
			return;
		}
		
		try{
			if (WorkflowEventType.ADD_WORKER.equals(eventType)){// A new Worker was created
				PTSRequestInvoker.addWorkerInfotoPTS(ptsEmployer, worker);
			}else if (WorkflowEventType.UPDATE_WORKER.equals(eventType)){
				
				if (EmploymentStatusEnum.TERMINATED.equals(worker.getEmploymentStatus())){
					// Do not update for a terminated worker
					return;
				}
				// Check if this worker is available in PSP and update is required
				if (StatusUpdateHandlerUtil.isEntityAddedToPTS(worker)){
					PTSRequestInvoker.updateWorkerInfotoPTS(ptsEmployer, worker);	
				}else{
					PTSRequestInvoker.addWorkerInfotoPTS(ptsEmployer, worker);
				}
			}else if (WorkflowEventType.REHIRE_WORKER.equals(eventType)){
				// Check if this worker is available in PSP and rehire is required
				if (StatusUpdateHandlerUtil.isEntityAddedToPTS(worker)){
					PTSRequestInvoker.reHireWorkerInfotoPTS(ptsEmployer, worker);	
				}else{
					PTSRequestInvoker.addWorkerInfotoPTS(ptsEmployer, worker);
				}	
			}else if (WorkflowEventType.TERMINATE_WORKER.equals(eventType)){
				// Check if this worker is available in PSP, if not add the worker to PSP
				if(!StatusUpdateHandlerUtil.isEntityAddedToPTS(worker)){
					PTSRequestInvoker.addWorkerInfotoPTS(ptsEmployer, worker);
				}
				
				// Deactivate the worker in PSP
				PTSRequestInvoker.terminateWorkerInfotoPTS(ptsEmployer, worker);
				// Now send the terminated Payroll to PSP 
				// IMP: We need to inactivate the employee first in psp and then
				// send this PayrollRun to pass through
				sendTerminatedPayrollRun(ptsEmployer, worker);
				
			}else if (WorkflowEventType.DELETE_WORKER.equals(eventType)){
				// Check if this worker is available in PSP and delete is required
				if (StatusUpdateHandlerUtil.isEntityAddedToPTS(worker)){
					PTSRequestInvoker.deleteWorkerInfoFromPTS(ptsEmployer, worker);
				}
			}
		}catch(ValidationException e){
			throw new ServiceFailureException(e);
		}
	}
	
	private void sendTerminatedPayrollRun(PTSEmployer ptsEmployer, Worker worker) throws ServiceFailureException {
		if (!StatusUpdateHandlerUtil.isEntityAddedToPTS(ptsEmployer.getEmployer())){
			return; // Act only when current company is in PTS
		}
		
		List<PayrollRun> terminatedPayrollRuns = payrollRunEAO.findRecentPayrollRuns(PayrollRunTypeEnum.TERMINATION, 
				LocaleGuru.getInstance().getCalendar().get(Calendar.YEAR));
		PayrollRun terminatedPayrollRun = null;
		for (PayrollRun termPayrollRun: terminatedPayrollRuns){
			Set<Paycheck> paychecks = termPayrollRun.getPaychecks();
			for(Paycheck paycheck: paychecks){
				if (paycheck.getWorker().getId() == worker.getId()){
					// this is the one
					terminatedPayrollRun = termPayrollRun;
					break;
				}
			}
		}
		
		try {
			PTSRequestInvoker.submitPayrollRunInfoToPTS(ptsEmployer, terminatedPayrollRun);
		} catch (ValidationException e) {
			throw new ServiceFailureException(e);
		}
	}

	public void handleAssistedSetupEvent(WorkflowEventType workflowEventType,
			PTSEmployer ptsEmployer) throws ServiceFailureException {
		 if (WorkflowEventType.EMPLOYER_ADDED_IN_PTS == workflowEventType){
			 try {
				addExistingWorkersToPTS(ptsEmployer);
			} catch (ValidationException e) {
				throw new ServiceFailureException(e);
			}
		 }
	}
	
	
	private void addExistingWorkersToPTS(PTSEmployer ptsEmployer) throws ServiceFailureException, ValidationException{
		// Send Employees
		List<Worker> workers = workerComponent.findAllWorkers();
		for (Worker worker : workers) {
			if(!WorkerSetupRules.isReadyForWorkflow(WorkerWorkflowType.PTSTaxReady, worker)) {
				continue;
			}
			PTSRequestInvoker.addWorkerInfotoPTS(ptsEmployer, worker);
			
			// if the worker is terminated in Gemini, send the info to PTS, to deactivate the worker
			if(worker.getEmploymentStatus().equals(EmploymentStatusEnum.TERMINATED)){
				PTSRequestInvoker.terminateWorkerInfotoPTS(ptsEmployer, worker);
			}
		}
	}
	
	
}
