package com.intuit.sb.payroll.capability.workerpayments.impl;

import java.util.Calendar;
import java.util.Date;
import java.util.List;

import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;

import org.apache.log4j.Logger;
import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunInfo;
import com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleComponent;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.PayrollResourceManager;
import com.intuit.sb.payroll.common.ResourceKeyConstants;
import com.intuit.sb.payroll.common.WorkflowEventType;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.common.EntityService;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.workerpayments.PayrollRunEAO;
import com.intuit.sb.payroll.message.PayrollMessageService;

/**
 * Implementation of the PayrollScheduleComponent interface.  
 *
 * $Author$ 
 * $Revision$
 * $DateTime$
 */
@Stateless(name = "PayrollScheduleComponent")
@SecurityDomain("BHPIAM")
@RolesAllowed({"Authenticated"})
public class PayrollScheduleComponentBean implements PayrollScheduleComponent {
	
	@EJB
	private EntityService entityService;
	
	@EJB
	private PayrollRunEAO prEAO;

	@EJB
	private EmployerComponent employerComponent;
	
	@EJB
	private PayrollMessageService messageService;
	
	private static Logger sLogger = PayrollLoggerFactory.makeLogger(PayrollScheduleComponentBean.class);
	
	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleComponent#createPayrollSchedule(com.intuit.sb.payroll.entity.model.PayrollSchedule)
	 */
	public PayrollSchedule createPayrollSchedule(PayrollSchedule payrollScheduleToCreate) throws ValidationException,
		ServiceFailureException {
        PayrollScheduleValidator.getInstance().validateEntity(payrollScheduleToCreate);

        PayrollScheduleDateUtils.determineCalculatedValues(payrollScheduleToCreate);
		this.prePersist(payrollScheduleToCreate);
		payrollScheduleToCreate = entityService.makePersistent(payrollScheduleToCreate);
		
		//Publish event to interested parties 
		messageService.dispatchPayrollScheduleEvent(WorkflowEventType.ADD_PAYROLL_SCHEDULE, payrollScheduleToCreate);

		return payrollScheduleToCreate;
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleComponent#updatePayrollSchedule(com.intuit.sb.payroll.entity.model.PayrollSchedule)
	 */
	public PayrollSchedule updatePayrollSchedule(PayrollSchedule payrollScheduleToUpdate) throws ValidationException, ServiceFailureException {
        PayrollScheduleValidator.getInstance().validateEntity(payrollScheduleToUpdate);
		
        PayrollScheduleDateUtils.determineCalculatedValues(payrollScheduleToUpdate);
		this.prePersist(payrollScheduleToUpdate);
		payrollScheduleToUpdate = entityService.makeManaged(payrollScheduleToUpdate);

		//Publish event to interested parties 
		messageService.dispatchPayrollScheduleEvent(WorkflowEventType.UPDATE_PAYROLL_SCHEDULE, payrollScheduleToUpdate);
		
		return payrollScheduleToUpdate;
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleComponent#deletePayrollSchedule(com.intuit.sb.payroll.entity.model.PayrollSchedule)
	 */
	public void deletePayrollSchedule(PayrollSchedule payrollScheduleToDelete) throws ValidationException,
		ServiceFailureException {
		payrollScheduleToDelete = entityService.makeManaged(payrollScheduleToDelete);
		if (payrollScheduleToDelete.getWorkers().size() > 0) {
			throw new ValidationException("Cannot delete a PayrollSchedule with Workers associated with it.  Assign the Workers to another Schedule.");
		}
		try {
			entityService.makeTransient(payrollScheduleToDelete);
			// Send event to Task Manager with non managed PayrollSchedule
			//Publish event to interested parties 
			messageService.dispatchPayrollScheduleEvent(WorkflowEventType.DELETE_PAYROLL_SCHEDULE, payrollScheduleToDelete);
		} catch (Exception e) {
			sLogger.error("Error deleting PayrollSchedule", e);
			throw new ServiceFailureException(e);
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleComponent#findPayrollSchedule(java.lang.Long)
	 */
	public PayrollSchedule findPayrollSchedule(Long payrollScheduleId) throws ServiceFailureException, ValidationException {
		PayrollParamValidator.checkIsNotNull(payrollScheduleId, "payrollScheduleId");

		PayrollSchedule payrollSchedule = null;
		try {
			payrollSchedule = entityService.findById(PayrollSchedule.class, payrollScheduleId);
		} catch (Exception e) {
			throw new ServiceFailureException(e);
		}
		if (null == payrollSchedule) {
			throw new ServiceFailureException(PayrollResourceManager.getInstance()
					.getString(ResourceKeyConstants.ID_NOT_FOUND_EXCEPTION)
					+ payrollScheduleId.toString());
		}
		return payrollSchedule;
	}
	

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleComponent#getPayrollSchedule()
	 */
	public PayrollSchedule getDefaultPayrollSchedule() throws ServiceFailureException {
		List<PayrollSchedule> payrollScheduleList = entityService.findAll(PayrollSchedule.class);
		if (payrollScheduleList.size() > 0) {
			for (PayrollSchedule payrollSchedule : payrollScheduleList) {
				if (payrollSchedule.isDefaultt()) {
					PayrollScheduleDateUtils.determineCalculatedValues(payrollSchedule);
					return payrollSchedule;					
				}
			}
		}
		return null;
	}
	
	public List<PayrollSchedule> getAllPayrollSchedules() throws ServiceFailureException{
		try{
			List<PayrollSchedule> payrollScheduleList = entityService.findAll(PayrollSchedule.class);
			return payrollScheduleList;
		}catch (Exception e) {
			throw new ServiceFailureException(e);
		}
		
	}

	/**
	 * Cleans up stuff in case frequencies have changed.
	 * 	FIXME: move to PrePersist on the entity.
	 * 
	 * @param payrollSchedule
	 */
	private void prePersist(PayrollSchedule payrollSchedule) {
		if (payrollSchedule.getFrequency() == PayrollFrequencyEnum.SEMIMONTHLY)
		{
			payrollSchedule.setNextPayPeriodEndDate(null);
		}
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleComponent#getUpcomingScheduledPayrollRunInfos(com.intuit.sb.payroll.entity.model.PayrollSchedule, int)
	 */
	public List<PayrollRunInfo> getUpcomingScheduledPayrollRunInfosByYear(PayrollSchedule payrollSchedule, Date year) throws ValidationException, ServiceFailureException {
		PayrollScheduleValidator.getInstance().validateEntity(payrollSchedule);
		PayrollParamValidator.checkIsNotNull(year, "year");
		final PayrollRun lastRun = prEAO.findLastCompletedPayrollRun();
		Date startDate = null;
		if (null == lastRun) {
			// Never ran a payroll, use the schedule. 
			PayrollScheduleDateUtils.determineCalculatedValues(payrollSchedule);
			startDate = payrollSchedule.getPeriodStartDate();
		} else {
			startDate = DateUtil.getNextDate(lastRun.getPayPeriod().getEndDate(), 1);
		}

		Calendar startCal = DateUtil.dateToCalendar(startDate);
		Calendar inputYear = DateUtil.dateToCalendar(year);
		final List<PayrollRunInfo> allRuns = PayrollScheduleDateUtils.getPayrollRunInfosForSchedule(payrollSchedule, startCal, inputYear);
		return allRuns;
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleComponent#getNextScheduledPayrollRunInfo(com.intuit.sb.payroll.entity.model.PayrollSchedule)
	 */
	public PayrollRunInfo getNextScheduledPayrollRunInfo(PayrollSchedule payrollSchedule) throws ValidationException,
		ServiceFailureException {
		PayrollScheduleValidator.getInstance().validateEntity(payrollSchedule);
		final PayrollRun lastRun = prEAO.findLastCompletedPayrollRun();
		Date startDate = null;
		if (null == lastRun) {
			// Never ran a payroll, use the schedule. 
			PayrollScheduleDateUtils.determineCalculatedValues(payrollSchedule);
			startDate = payrollSchedule.getPeriodStartDate();
		} else {
			startDate = DateUtil.getNextDate(lastRun.getPayPeriod().getEndDate(), 1);
		}

		Calendar startCal = DateUtil.dateToCalendar(startDate);
		final List<PayrollRunInfo> payrollRunInfo = PayrollScheduleDateUtils.getPayrollRunInfosForSchedule(payrollSchedule, startCal, null);
		return payrollRunInfo.get(0);
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleComponent#assignWorkerToPayrollSchedule(com.intuit.sb.payroll.entity.model.Worker, com.intuit.sb.payroll.entity.model.PayrollSchedule)
	 */
	public PayrollSchedule assignWorkerToPayrollSchedule(Worker worker, PayrollSchedule payrollSchedule)
			throws ValidationException, ServiceFailureException {
		PayrollParamValidator.checkIsNotNull(worker, "worker");
		PayrollParamValidator.checkIsNotNull(payrollSchedule, "payrollSchedule");
		if (payrollSchedule.getWorkers().contains(worker)) {
			throw new ValidationException("Worker is already associated with the PayrollSchedule");
		}
		payrollSchedule = entityService.makeManaged(payrollSchedule);
		payrollSchedule.addWorker(worker);
		return payrollSchedule;
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleComponent#removeWorkerFromPayrollSchedule(com.intuit.sb.payroll.entity.model.Worker, com.intuit.sb.payroll.entity.model.PayrollSchedule)
	 */
	public PayrollSchedule removeWorkerFromPayrollSchedule(Worker worker, PayrollSchedule payrollSchedule)
			throws ValidationException, ServiceFailureException {
		PayrollParamValidator.checkIsNotNull(worker, "worker");
		PayrollParamValidator.checkIsNotNull(payrollSchedule, "payrollSchedule");
		if (! payrollSchedule.getWorkers().contains(worker)) {
			throw new ValidationException("Worker is not associated with the PayrollSchedule");
		}
		payrollSchedule = entityService.makeManaged(payrollSchedule);
		payrollSchedule.removeWorker(worker);
		return payrollSchedule;
	}
	
	
	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleComponent#assignPayrollScheduleToEmployer(com.intuit.sb.payroll.entity.model.Worker, com.intuit.sb.payroll.entity.model.PayrollSchedule)
	 */
	public PayrollSchedule assignPayrollScheduleToEmployer(PayrollSchedule payrollSchedule)
			throws ValidationException, ServiceFailureException {

		PayrollParamValidator.checkIsNotNull(payrollSchedule, "payrollSchedule");
		Employer employer = employerComponent.getEmployer();
		
		if (!employer.getPayrollSchedules().contains(payrollSchedule)) {
			payrollSchedule = entityService.makeManaged(payrollSchedule);
			employer.addPayrollSchedule(payrollSchedule);
			employerComponent.saveEmployer(employer);
		}
		return payrollSchedule;
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleComponent#removePayrollScheduleFromEmployer(com.intuit.sb.payroll.entity.model.Worker, com.intuit.sb.payroll.entity.model.PayrollSchedule)
	 */
	public PayrollSchedule removePayrollScheduleFromEmployer(PayrollSchedule payrollSchedule)
			throws ValidationException, ServiceFailureException {
		PayrollParamValidator.checkIsNotNull(payrollSchedule, "payrollSchedule");
		Employer employer = employerComponent.getEmployer();
		if (!employer.getPayrollSchedules().contains(payrollSchedule)) {
			throw new ValidationException("PayrollSchedule is not associated with Employer");
		}
		payrollSchedule = entityService.makeManaged(payrollSchedule);
		employer.removePayrollSchedule(payrollSchedule);
		employerComponent.saveEmployer(employer);
		return payrollSchedule;
	}
}
