package com.intuit.sb.payroll.capability.workerpayments.impl;

import java.util.List;

import org.apache.log4j.Logger;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.capability.task.TaskManager;
import com.intuit.sb.payroll.capability.task.impl.TaskUtils;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunInfo;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;
import com.intuit.sb.payroll.entity.model.Task;
import com.intuit.sb.payroll.entity.model.TaskTypeEnum;

public class WorkerPaymentsComponentTasks {
	
	public static final String EXECUTE_PAYROLL_RUN_CLASS = "com.intuit.sb.payroll.workerpayments"; 
	public static final String RUN_PAYROLLRUN_TASK = "runPayrollRunTask";
	public static final String FINISH_PAYCHECK_DISTRIBUTION_TASK = "finishPaycheckDistributionTask";
	private static Logger sLogger = PayrollLoggerFactory.makeLogger(WorkerPaymentsComponentTasks.class);
	/**
	 * Creates a task to remind the user of the next scheduled payroll Run.
	 * 
	 * @param taskManager
	 * @param payrollRunInfo
	 * @param payrollSchedule
	 * @throws ServiceFailureException
	 * @throws TaskException
	 */
	public static void createNextScheduledPayrollRunTask(TaskManager taskManager, PayrollRunInfo payrollRunInfo, PayrollSchedule payrollSchedule) throws ServiceFailureException {

		Task runPayroll =  new Task();
		runPayroll.setName(RUN_PAYROLLRUN_TASK);
		runPayroll.setDescription("Run your payroll");
		runPayroll.setTaskClass(EXECUTE_PAYROLL_RUN_CLASS);
		runPayroll.setPayload(payrollSchedule.getId());
		runPayroll.setType(TaskTypeEnum.SCHEDULED);
		runPayroll.setCategory("Scheduled");
		runPayroll.setUserManagable(false); 
		runPayroll.setDeadline(payrollRunInfo.getPayrollRunDate());
		runPayroll.setDateRange(new DateRange(payrollRunInfo.getPayPeriod().getStartDate(), payrollRunInfo.getPayPeriod().getEndDate()));
		taskManager.addTask(runPayroll);
	}

	public static void closeScheduledPayrollRunTask(TaskManager taskManager, PayrollRun payrollRun, PayrollSchedule payrollSchedule)
		throws ServiceFailureException {
		List<Task> tasks = taskManager.findActiveTasksByClassAndPayload(EXECUTE_PAYROLL_RUN_CLASS, new Long(payrollSchedule.getId()));
		sLogger.debug("Number of PayrollSchedule to be closed" + tasks.size());
		for (Task task : tasks) {
			sLogger.debug("Attempting to closing Task "+TaskUtils.taskToString(task));
			sLogger.debug("Payroll Period Start Date "+payrollRun.getPayPeriod().getStartDate());
			if (DateUtil.isSameDay(payrollRun.getPayPeriod().getStartDate(), task.getDateRange().getStartDate())) {
				sLogger.debug("Closing Task "+task.getId());
				taskManager.closeTask(task);
			}
		}
	}
	
	/**
	 * Creates a task to finish the Paycheck Distribution workflow.
	 * @param taskManager
	 * @param payrollRun
	 */
	public static void createFinishPaycheckDistributionTask(TaskManager taskManager, PayrollRun payrollRun) {
		Task finishDistributionTask =  new Task();
		finishDistributionTask.setName(FINISH_PAYCHECK_DISTRIBUTION_TASK);
		finishDistributionTask.setDescription("Finish Distributing Paychecks");
		finishDistributionTask.setTaskClass(EXECUTE_PAYROLL_RUN_CLASS);
		finishDistributionTask.setPayload(payrollRun.getId());
		finishDistributionTask.setType(TaskTypeEnum.NONSCHEDULED);
		finishDistributionTask.setCategory("Reminder");
		finishDistributionTask.setUserManagable(false); 
		finishDistributionTask.setDeadline(payrollRun.getPayrollRunDate());
		finishDistributionTask.setDateRange(new DateRange(payrollRun.getPayPeriod().getStartDate(), payrollRun.getPayPeriod().getEndDate()));
		taskManager.addTask(finishDistributionTask);
	}
	
	public static void closeFinishPaycheckDistributionTask(TaskManager taskManager, PayrollRun payrollRun) throws ServiceFailureException {
		List<Task> tasks = taskManager.findActiveTasksByClassAndPayload(EXECUTE_PAYROLL_RUN_CLASS, new Long(payrollRun.getId()));
		for (Task task : tasks) {
			if (task.getName().equals(FINISH_PAYCHECK_DISTRIBUTION_TASK)) {
				taskManager.closeTask(task);
			}
		}
	}
	
	public static boolean payrollRunTaskExists(TaskManager taskManager, PayrollSchedule payrollSchedule) {
		List<Task> tasks = taskManager.findActiveTasksByClassAndPayload(EXECUTE_PAYROLL_RUN_CLASS, new Long(payrollSchedule.getId()));
		if (tasks.size() > 0) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Updates any incomplete tasks related to a schedule.
	 * @param taskManager
	 * @param payrollSchedule
	 * @throws ServiceFailureException 
	 */
	public static void updatePayrollRunTasksForSchedule(TaskManager taskManager, PayrollSchedule payrollSchedule) throws ServiceFailureException {
		List<Task> tasks = taskManager.findCurrentActiveTasks();
		for(Task task : tasks) {
			if(task.getName().equals(RUN_PAYROLLRUN_TASK) && task.getId() != payrollSchedule.getId()) {
				taskManager.cancelTask(task);
			}
		}
	}
	
	/**
	 * Removes any incomplete tasks related to a schedule.  Invoked as a result of a PayrollSchedule being deleted or changed
	 * in such a way that it invalidates the existing tasks.
	 * 
	 * @param taskManager
	 * @param payrollSchedule
	 * @throws TaskException 
	 */
	public static void removePayrollRunTasksForSchedule(TaskManager taskManager, PayrollSchedule payrollSchedule) throws ServiceFailureException {
		List<Task> tasks = taskManager.findActiveTasksByClassAndPayload(EXECUTE_PAYROLL_RUN_CLASS, new Long(payrollSchedule.getId()));
		for (Task task : tasks) {
			taskManager.cancelTask(task);
		}
	}

}
