package com.intuit.sb.payroll.calcengine;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.workerpayments.CalculationInputs;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckInputs;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunInfo;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunInputs;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.DE4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.W4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 *  <P>The <CODE>MiscPaycheckCalcTest</CODE> class contains miscellaneous tests 
 *  for testing the paycheck calculation functionality</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class MiscPaycheckCalcTest extends CalcEngineTestBase {
    /**
     * <P>Test calling CalculatePaycheck 3 times in succession, should have same result as calling it 1 time</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a Weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 40 hours</LI>
     * <LI>Use default withholding of Single 1 for both Federal and CA taxes</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Create a PayrollRun with the dates for a weekly payroll</LI>
     * <LI>Call CalcEngine to calculate the paycheck 3 times in succession</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_CallCalculatePaycheck3times() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        Paycheck paycheck = null;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.SINGLE, 1, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("15.00"), 40.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        // add hourly wages
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();

        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        for (int i = 0; i < 3; i++) {
            final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
    				null);
    		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
    		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
    		paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        }
        
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("600.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("65.00"), new BigDecimal("600.00"), new BigDecimal("600.00")),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("37.20"), new BigDecimal("600.00"), new BigDecimal("600.00")),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("8.70"), new BigDecimal("600.00"), new BigDecimal("600.00")),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("12.77"), new BigDecimal("600.00"), new BigDecimal("600.00")),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.80"), new BigDecimal("600.00"), new BigDecimal("600.00")),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.80"), new BigDecimal("600.00"), new BigDecimal("600.00")),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("37.20"), new BigDecimal("600.00"), new BigDecimal("600.00")),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("8.70"), new BigDecimal("600.00"), new BigDecimal("600.00")),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.40"), new BigDecimal("600.00"), new BigDecimal("600.00")),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.60"), new BigDecimal("600.00"), new BigDecimal("600.00"))
        };
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with no wage and all applicable taxes for California 
     * using withholding status of CA Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a Weekly pay frequency</LI>
     * <LI>Use default withholding of Single 1 for both Federal and CA taxes</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Create a PayrollRun with the dates for a weekly payroll</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_NoWage_WithholdCASingle1_PaidWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.SINGLE, 1, sZeroMoney, 0, NOTEXEMPT);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	null);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00"), new BigDecimal("0.00"), new BigDecimal("0.00")),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00"), new BigDecimal("0.00"), new BigDecimal("0.00")),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00"), new BigDecimal("0.00"), new BigDecimal("0.00")),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00"), new BigDecimal("0.00"), new BigDecimal("0.00")),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00"), new BigDecimal("0.00"), new BigDecimal("0.00")),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00"), new BigDecimal("0.00"), new BigDecimal("0.00")),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00"), new BigDecimal("0.00"), new BigDecimal("0.00")),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00"), new BigDecimal("0.00"), new BigDecimal("0.00")),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00"), new BigDecimal("0.00"), new BigDecimal("0.00")),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00"), new BigDecimal("0.00"), new BigDecimal("0.00"))
        };
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with 1 hourly wage and all applicable taxes for California 
     * with no withholding allowance set</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 40 hours</LI>
     * <LI>Don't set the withholding allowance for both Federal and CA taxes</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Create a PayrollRun with the dates for a weekly payroll</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly_NoWithholdAllowance_PaidWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.NOTSET, 0, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.NOTSET, 0, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("15.00"), 40.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        // add hourly wages
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("600.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("75.00"), new BigDecimal("600.00"), new BigDecimal("600.00")),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("37.20"), new BigDecimal("600.00"), new BigDecimal("600.00")),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("8.70"), new BigDecimal("600.00"), new BigDecimal("600.00")),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("14.58"), new BigDecimal("600.00"), new BigDecimal("600.00")),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.80"), new BigDecimal("600.00"), new BigDecimal("600.00")),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.80"), new BigDecimal("600.00"), new BigDecimal("600.00")),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("37.20"), new BigDecimal("600.00"), new BigDecimal("600.00")),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("8.70"), new BigDecimal("600.00"), new BigDecimal("600.00")),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.40"), new BigDecimal("600.00"), new BigDecimal("600.00")),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.60"), new BigDecimal("600.00"), new BigDecimal("600.00"))
        };
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

}
