package com.intuit.sb.payroll.capability.employer;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.entity.model.BankAccount;
import com.intuit.sb.payroll.entity.model.BankAccountTypeEnum;

/**
 * Tests for Bank Account addition/updation
 */
public class BankAccountTest extends EmployerComponentTestBase {

	@Test(expectedExceptions = ValidationException.class)
	public void testNullBankAccountAddition() throws Exception {
		BankAccount[] bankAccounts = {null};
		sEmployerComponent.addBankAccounts(bankAccounts);
	}

	@Test(expectedExceptions = ValidationException.class)
	public void testBankAccountWithInvalidAccountType() throws Exception {
		BankAccount acc = createBankAccount();
		// only Checking/Savings is allowed - NOTSET should cause validation failure
		acc.setAccountType(BankAccountTypeEnum.NOTSET);

		BankAccount[] accounts = {acc};
		sEmployerComponent.addBankAccounts(accounts);
	}

	@Test(expectedExceptions = ValidationException.class)
	public void testBankAccountWithAlphaNumericRoutingNo() throws Exception {
		BankAccount acc = createBankAccount();
		// routing no should be 9-digit no. with digits following suitable algorithm
		acc.setRoutingNo("abcd12345");

		BankAccount[] accounts = {acc};
		sEmployerComponent.addBankAccounts(accounts);
	}

	@Test(expectedExceptions = ValidationException.class)
	public void testBankAccountWithShortRoutingNo() throws Exception {
		BankAccount acc = createBankAccount();
		// routing no should be 9-digit no. with digits following suitable algorithm
		acc.setRoutingNo("11000000");	//	only 8 digits

		BankAccount[] accounts = {acc};
		sEmployerComponent.addBankAccounts(accounts);
	}

	@Test(expectedExceptions = ValidationException.class)
	public void testBankAccountWithNegativeRoutingNo() throws Exception {
		BankAccount acc = createBankAccount();
		// routing no should be 9-digit +ve no. with digits following suitable algorithm
		acc.setRoutingNo("-110000000");

		BankAccount[] accounts = {acc};
		sEmployerComponent.addBankAccounts(accounts);
	}

	@Test(expectedExceptions = ValidationException.class)
	public void testBankAccountWithNegativeAccountNo() throws Exception {
		BankAccount acc = createBankAccount();
		// account no. should be a +ve no.
		acc.setAccountNo("-123456");
		BankAccount[] accounts = {acc};
		sEmployerComponent.addBankAccounts(accounts);
	}

	@Test(expectedExceptions = ValidationException.class)
	public void testBankAccountWithZeroAccountNo() throws Exception {
		BankAccount acc = createBankAccount();
		// account no. should be non zero
		acc.setAccountNo("0000000");
		BankAccount[] accounts = {acc};
		sEmployerComponent.addBankAccounts(accounts);
	}

	@Test
	public void addAndUpdateValidBankAccount() throws Exception {
		BankAccount acc = createBankAccount();
		String dummyName = "Test Bank Name";
		acc.setName(dummyName);
		BankAccount[] accounts = {acc};
		accounts = sEmployerComponent.addBankAccounts(accounts);

		AssertJUnit.assertEquals(1, accounts.length);	//	persisted account should come back
		AssertJUnit.assertNotNull(accounts);	//	valid array
		AssertJUnit.assertNotNull(accounts[0]);	// valid bankaccount
		AssertJUnit.assertEquals(dummyName, accounts[0].getName());

		// now update the account details
		acc = accounts[0];
		String dummyName2 = "Test Bank Name 2";
		String newRoutingNo = "120003000";
		acc.setRoutingNo(newRoutingNo);
		acc.setName(dummyName2);

		accounts[0] = acc;
		accounts = sEmployerComponent.updateBankAccounts(accounts);

		AssertJUnit.assertEquals(1, accounts.length);	//	persisted account should come back
		AssertJUnit.assertNotNull(accounts);	//	valid array
		AssertJUnit.assertNotNull(accounts[0]);	// valid bankaccount
		AssertJUnit.assertEquals(dummyName2, accounts[0].getName());
		AssertJUnit.assertEquals(newRoutingNo, accounts[0].getRoutingNo());
	}

	/**
	 * Create a random BankAccount object with valid values.
	 * @return BankAccount object
	 */
	private BankAccount createBankAccount() {
		BankAccount acc = new BankAccount();
		acc.setAccountNo("123456");
		acc.setAccountType(BankAccountTypeEnum.SAVINGS);
		acc.setBankName("Bank of Jhumri Thalayya");
		acc.setName("JT Bank");
		acc.setRoutingNo("110000000");

		return acc;
	}
}
