package com.intuit.sb.payroll.capability.employer;

import java.util.List;

import org.testng.AssertJUnit;

import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.common.BasicTestBase;
import com.intuit.sb.payroll.capability.employer.impl.JurisdictionResolver;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.entity.model.Jurisdiction;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;


/**
 *  <P>The <CODE>JurisdictionResolverTest</CODE> class contains test methods for the JurisdictionResolver class</P>
 *
 *  <DL>
 *  <DT><B>Requirements Document:</B></DT>
 *  <DD><A HREF="http://spitfire.corp.intuit.net/sites/merlinpayroll/Shared%20Documents/Forms/AllItems.aspx?RootFolder=%2fsites%2fmerlinpayroll%2fShared%20Documents%2fRequirements&View=%7b146B0034%2d58AF%2d4E76%2d93B1%2dA00364A96B7E%7d">Gemini requirements</A></DD>
 *  </DL>
 *  @author ryasui
 *  @version August 21, 2007
 */

public class JurisdictionResolverTest extends BasicTestBase {

	private JurisdictionResolver mResolver = null;

	@BeforeMethod
	public void setUpJurisdictions() throws Exception {

		mResolver = new JurisdictionResolver();
	}

	/**
     * <P>Tests JurisdictionResolver for address in San Francisco, CA</P>
     *
     * <H3>Requirements Covered:</H3>
     * <TABLE ALIGN="left" BORDER=1 WIDTH="100%">
     *  <TR ALIGN="left">
     *      <TH>Number</TH>
     *      <TH>Description</TH>
     *  </TR>
     *  <TR>
     *      <TD>SN1885</TD>
     *      <TD>Determine tax Jurisdiction for an address</TD>
     *  </TR>
     * </TABLE><BR/><BR/><BR/>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set address to San Francisco, CA</LI>
     * <LI>Pass address to JurisdictionResolver</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that list of Jurisdictions contain San Francisco, CA, US, and no exceptions are thrown</LI>
     * </UL>
     */
	@Test
	public void testResolveJurisdicationsForSFAddress() throws Exception {

		USPostalAddress address = WorkerTestUtils.createUSPhysicalAddress();
		address.setRegionOrState("CA");
		address.setCity("San Francisco");

		List<Jurisdiction> jurisdictions = mResolver.resolveJurisdictionsForAddress(address);

		//Expecting SF, CA, and US Jurisdictions
		AssertJUnit.assertEquals(3, jurisdictions.size());
		AssertJUnit.assertTrue(this.hasJurisdiction(jurisdictions, JurisdictionType.SAN_FRANCISCO.getName()));
		AssertJUnit.assertTrue(this.hasJurisdiction(jurisdictions, JurisdictionType.CALIFORNIA.getName()));
		AssertJUnit.assertTrue(this.hasJurisdiction(jurisdictions, JurisdictionType.UNITED_STATES.getName()));
	}

	/**
     * <P>Tests JurisdictionResolver for address in CA but not in San Francisco</P>
     *
     * <H3>Requirements Covered:</H3>
     * <TABLE ALIGN="left" BORDER=1 WIDTH="100%">
     *  <TR ALIGN="left">
     *      <TH>Number</TH>
     *      <TH>Description</TH>
     *  </TR>
     *  <TR>
     *      <TD>SN1885</TD>
     *      <TD>Determine tax Jurisdiction for an address</TD>
     *  </TR>
     * </TABLE><BR/><BR/><BR/>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set address to Mountain View, CA</LI>
     * <LI>Pass address to JurisdictionResolver</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that list of Jurisdictions contain CA, US, and no exceptions are thrown</LI>
     * </UL>
     */
	@Test
	public void testResolveJurisdicationsForCAAddress() throws Exception {

		USPostalAddress address = WorkerTestUtils.createUSPhysicalAddress();
		address.setCity("Mountain View");
		address.setRegionOrState("CA");

		List<Jurisdiction> jurisdictions = mResolver.resolveJurisdictionsForAddress(address);

		// Expecting CA, and US Jurisdictions
		AssertJUnit.assertEquals(2, jurisdictions.size());
		AssertJUnit.assertTrue(this.hasJurisdiction(jurisdictions, JurisdictionType.CALIFORNIA.getName()));
		AssertJUnit.assertTrue(this.hasJurisdiction(jurisdictions, JurisdictionType.UNITED_STATES.getName()));
	}

	/**
     * <P>Tests JurisdictionResolver for address in TX</P>
     *
     * <H3>Requirements Covered:</H3>
     * <TABLE ALIGN="left" BORDER=1 WIDTH="100%">
     *  <TR ALIGN="left">
     *      <TH>Number</TH>
     *      <TH>Description</TH>
     *  </TR>
     *  <TR>
     *      <TD>SN1885</TD>
     *      <TD>Determine tax Jurisdiction for an address</TD>
     *  </TR>
     * </TABLE><BR/><BR/><BR/>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set address to Austin, TX</LI>
     * <LI>Pass address to JurisdictionResolver</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that list of Jurisdictions contain TX, US, and no exceptions are thrown</LI>
     * </UL>
     */
	@Test
	public void testResolveJurisdicationsForTXAddress() throws Exception {

		USPostalAddress address = WorkerTestUtils.createUSPhysicalAddress();
		address.setCity("Austin");
		address.setRegionOrState("TX");

		List<Jurisdiction> jurisdictions = mResolver.resolveJurisdictionsForAddress(address);

		// Expecting CA, and US Jurisdictions
		AssertJUnit.assertEquals(2, jurisdictions.size());
		AssertJUnit.assertTrue(this.hasJurisdiction(jurisdictions, JurisdictionType.TEXAS.getName()));
		AssertJUnit.assertTrue(this.hasJurisdiction(jurisdictions, JurisdictionType.UNITED_STATES.getName()));
	}

	/**
     * <P>Tests JurisdictionResolver for an unsupported US state</P>
     *
     * <H3>Requirements Covered:</H3>
     * <TABLE ALIGN="left" BORDER=1 WIDTH="100%">
     *  <TR ALIGN="left">
     *      <TH>Number</TH>
     *      <TH>Description</TH>
     *  </TR>
     *  <TR>
     *      <TD>SN2706</TD>
     *      <TD>Check support for tax jurisdiction</TD>
     *  </TR>
     * </TABLE><BR/><BR/><BR/>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set address to New York, NY</LI>
     * <LI>Pass address to JurisdictionResolver</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that exception is thrown.  This is expected for unsupported Jurisdictions</LI>
     * </UL>
     */
	@Test(expectedExceptions=ValidationException.class)
	public void testResolveJurisdicationsForNonSupportedUSAddress() throws Exception {

		USPostalAddress address = WorkerTestUtils.createUSPhysicalAddress();
		address.setCity("New York");
		address.setRegionOrState("NY");

		mResolver.resolveJurisdictionsForAddress(address);
	}

	/**
     * <P>Tests JurisdictionResolver for an unsupported country</P>
     *
     * <H3>Requirements Covered:</H3>
     * <TABLE ALIGN="left" BORDER=1 WIDTH="100%">
     *  <TR ALIGN="left">
     *      <TH>Number</TH>
     *      <TH>Description</TH>
     *  </TR>
     *  <TR>
     *      <TD>SN2706</TD>
     *      <TD>Check support for tax jurisdiction</TD>
     *  </TR>
     * </TABLE><BR/><BR/><BR/>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set address to Edmonton, Canada</LI>
     * <LI>Pass address to JurisdictionResolver</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that exception is thrown.  This is expected for unsupported Jurisdictions</LI>
     * </UL>
     */
	@Test(expectedExceptions=ValidationException.class)
	public void testResolveJurisdicationsForNonSupportedCanadaAddress() throws Exception {

		USPostalAddress address = WorkerTestUtils.createUSPhysicalAddress();
		address.setCity("Edmonton");
		address.setCountryISOCode("CA");

		mResolver.resolveJurisdictionsForAddress(address);
	}

	@Test(expectedExceptions=ValidationException.class)
	public void testResolveJurisdicationsForNull() throws Exception {
		mResolver.resolveJurisdictionsForAddress(null);
	}

	/**
	 * Helper function to determine if a Jurisdiction is in a list.
	 *
	 * @param jurisdictions A list of Jurisdictions to search for a match for jurisdictionName
     * @param jurisdictionName The name of the jurisdiction to search for.
     * @return true if jurisdictions contains a Jurisdiction with the name jurisdictionName. False
     * otherwise.
	 */
	private boolean hasJurisdiction(List<Jurisdiction> jurisdictions, String jurisdictionName)
	{
    	for (Jurisdiction jurisdiction : jurisdictions) {
    		if (jurisdictionName.equals(jurisdiction.getName())) {
    			return true;
    		}
    	}
    	return false;
	}
}
