package com.intuit.sb.payroll.capability.pts.statehandler;

import java.util.List;
import java.util.Set;

import org.testng.Assert;
import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.impl.EmployerPTSWorkflowTasks;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationException;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationRequest;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationTypeEnum;
import com.intuit.sb.payroll.capability.pts.notification.mapper.PTSNotificationRequestMapper;
import com.intuit.sb.payroll.entity.model.BankAccount;
import com.intuit.sb.payroll.entity.model.BankAccountTypeEnum;
import com.intuit.sb.payroll.entity.model.Task;
import com.intuit.sb.payroll.entity.model.TaskStatusEnum;
import com.intuit.sb.payroll.pts.entity.BankAccountStatusEnum;
import com.intuit.sb.payroll.pts.entity.InteractionEnum;
import com.intuit.sb.payroll.pts.entity.PPSPTSInteraction;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;

public class BankAccountStatusUpdateHandlerTest extends
		StatusUpdateHandlerTestBase {

	/**
	 * Test to check that addition of legal data to PPS causes data disabling of
	 * update for Bank Account info. Also, further sending of information to PPS
	 * should be disabled
	 **/
	@Test (enabled=true)
	public void testAddCompanyLocksBankData() throws Exception {
		
		BankAccount ba = new BankAccount();
		ba.setAccountNo("1239089");
		ba.setRoutingNo("110000000");
		ba.setAccountType(BankAccountTypeEnum.SAVINGS);
		ba.setName("Test Account");
		ba.setBankName("Test bank");
		BankAccount[] bankAccounts = sEmployerComponent.addBankAccounts(new BankAccount[]{ba});
		
		BankAccount savedBankAccount = null;
		for (int i = 0; i < bankAccounts.length; i++) {
			if (bankAccounts[i].getAccountNo().equals("1239089")){
				savedBankAccount = bankAccounts[i];
				break;
			}
		}
		
		if (savedBankAccount == null){
			Assert.fail("Not able to test further");
		}

		// simulate "ADD BANK" status update
		employerPTSComponent.updatePPS_PTS_InteractionState(savedBankAccount, InteractionEnum.ADDED);

		// check that the "ADD BANK STATUS" is there for this PTS Employer
		List<PPSPTSInteraction> list = StatusUpdateHandlerUtil.getPPSPTSInteractions(savedBankAccount);
		Assert.assertNotNull(list, "List should NOT be NULL");
		Assert.assertEquals(list.size(), 1,	"List should have ONE and ONLY ONE element");

		// check that Bank data update is disabled
		boolean bankUpdateDisabled = StatusUpdateHandlerUtil.isBankAccountEditAllowed(savedBankAccount);
		Assert.assertEquals(bankUpdateDisabled, false,	"Bank Update should be disabled");
	}
	
	@Test(enabled=true)
	public void testBankAccountStatusUpdateHandler() throws ServiceFailureException, ValidationException{

		BankAccountStatusEnum[] bankAccountStatusEnums = BankAccountStatusEnum.values();
		BankAccount bankaccount = createBankAccount();
		BankAccount[] bankAccountArray = sEmployerComponent.addBankAccounts(new BankAccount[]{bankaccount});
		String sourceBankId = String.valueOf(bankAccountArray[0].getId());
		for (BankAccountStatusEnum bankStatusEnum : bankAccountStatusEnums) {

			PTSNotificationRequest ptsNotificationRequest = createPTSNotificationRequest(
					PTSNotificationTypeEnum.BANK_ACCOUNT_NOTIFICATION, bankStatusEnum);
			ptsNotificationRequest.addNotificationAttributes(PTSNotificationRequestMapper.SOURCE_BANK_ACCOUNT_ID, sourceBankId);
			try {
				updateNotificationRequest(ptsNotificationRequest);
			} catch (PTSNotificationException e) {
				Assert.fail(e.getLocalizedMessage(), e);
			}
		}
	}
	
	@Test(enabled=true)
	public void testBankAccountStatusUpdateHandlerWithoutBankId() throws ServiceFailureException, ValidationException{

		PTSNotificationRequest ptsNotificationRequest = createPTSNotificationRequest(
				PTSNotificationTypeEnum.BANK_ACCOUNT_NOTIFICATION, BankAccountStatusEnum.BANKACCOUNT_VERIFIED_RECEIVED);
		try {
			updateNotificationRequest(ptsNotificationRequest);
		} catch (PTSNotificationException e) {
			Assert.assertEquals(e.getMessage().contains("sourceBankId"), true, "Exception expection not received");
			return;
		}
		Assert.fail("No exception received, expecting PTSNotificationException");
	}
	
	 @Test(enabled=true)
	    public void testBankAccountVerificationRejectionReceived() throws ValidationException , ServiceFailureException, PTSNotificationException {
	        PTSEmployer  ptsEmployer = employerPTSComponent.getPTSEmployer();
	        Set<BankAccount> bankAccounts = ptsEmployer.getEmployer().getBankAccounts();
	        int numberOfBankAccountsBeforeFailureNotice =0;
	        
	        BankAccount testBankAccount=null;
	        // get a bankAccount to update and test for notifications
	        if(bankAccounts.size() > 0){
	            for(BankAccount bankAccount:bankAccounts){
	                testBankAccount = bankAccount;
	                break;
	            }
	        }else{
	            BankAccount[] ba = new BankAccount[1];
	            ba[0] = createBankAccount("testBankName", BankAccountTypeEnum.SAVINGS, generateRandomAccountNumber(), "000067894");
	            BankAccount[] baAccounts = sEmployerComponent.addBankAccounts(ba);
	            testBankAccount = baAccounts[0];
	        }
	        
	        // Retrieve the bank account to check whether it gets deleted while updating
	        long bankAccountId = testBankAccount.getId();
	        
	        //create a PTSNotificationRequest
	        PTSNotificationRequest ptsNotificationRequest  = createPTSNotificationRequest(PTSNotificationTypeEnum.BANK_ACCOUNT_NOTIFICATION,
	                                                         BankAccountStatusEnum.BANKACCOUNT_VERIFICATION_FAILED_RECEIVED);
	        ptsNotificationRequest.addNotificationAttributes("SourceBankAccountId", String.valueOf(testBankAccount.getId()));

	        // send a PTSNotification indicating that the bankAccount verification is rejected
	        updateNotificationRequest(ptsNotificationRequest);

	        //Test if BankAccount Rejection task got created
	        List<Task> rejectedBankAccountVerificationTask = taskEAO.findTaskByTaskNameAndPayLoad(EmployerPTSWorkflowTasks.REJECTED_BANK_ACCOUNT_TASK, testBankAccount.getId());
			@SuppressWarnings("unused")
	        long taskId = rejectedBankAccountVerificationTask.get(0).getId();
	        AssertJUnit.assertEquals("Bank Account Verification failed task is not created",
	                EmployerPTSWorkflowTasks.REJECTED_BANK_ACCOUNT_TASK, rejectedBankAccountVerificationTask.get(0).getName());

	        //Check the status of the task
	        AssertJUnit.assertEquals("Bank Account Verification failed task was already closed", TaskStatusEnum.NOTSTARTED, rejectedBankAccountVerificationTask.get(0).getStatus());

	        numberOfBankAccountsBeforeFailureNotice = entityService.findAll(BankAccount.class).size();
	        
	        //Update the BankAccount
	        BankAccount[] bankAccountToUpdate = new BankAccount[1];
	        bankAccountToUpdate[0]=testBankAccount;
	        sEmployerComponent.updateBankAccounts(bankAccountToUpdate);
	 
	        //Test whether the bankAccount with Id bankAccountId is deleted
	        bankAccounts = ptsEmployer.getEmployer().getBankAccounts();
	        for (BankAccount bankAccount : bankAccountToUpdate) {
				AssertJUnit.assertNotSame("Rejected Bank Account is not deleted from Gemini",bankAccount.getId(), bankAccountId);
			}
	        //Test whether a new entity for bank account is created
	        AssertJUnit.assertEquals("A new Entity for bank Account is not created after update",numberOfBankAccountsBeforeFailureNotice,
	        				entityService.findAll(BankAccount.class).size());
	        
	        //Test if BankAccount Rejection task is deleted when BankAccount is updated
	        rejectedBankAccountVerificationTask = taskEAO.findTaskByTaskNameAndPayLoad(EmployerPTSWorkflowTasks.REJECTED_BANK_ACCOUNT_TASK, testBankAccount.getId());
	        AssertJUnit.assertEquals("Rejected Bank Account Task count is more than one", 0,rejectedBankAccountVerificationTask.size());

	        // Again send a PTSNotification indicating that the bankAccount verification is rejected
	        updateNotificationRequest(ptsNotificationRequest);
	    }
	
	
	/**
	 * Create a random BankAccount object with valid values.
	 * @return BankAccount object 
	 */
	// COPIED from BankAccountTest.java
	private BankAccount createBankAccount() {
		BankAccount acc = new BankAccount();
		acc.setAccountNo("123456");
		acc.setAccountType(BankAccountTypeEnum.SAVINGS);
		acc.setBankName("Bank of Jhumri Thalayya");
		acc.setName("JT Bank");
		acc.setRoutingNo("110000000");
		
		return acc;
	}

}
