package com.intuit.sb.payroll.capability.pts.statehandler;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;

import org.testng.Assert;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationException;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationRequest;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationTypeEnum;
import com.intuit.sb.payroll.pts.entity.EFTPSEnrollmentStatusEnum;
import com.intuit.sb.payroll.pts.entity.InteractionEnum;
import com.intuit.sb.payroll.pts.entity.PPSPTSInteraction;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;



public class EFTPSEnrollmentStatusUpdateHandlerTest extends StatusUpdateHandlerTestBase {

    @Test(enabled=true)
    public void testEFTPSEnrollmentStatusUpdateHandler(){

        EFTPSEnrollmentStatusEnum[] eftpsEnrollementStatus = EFTPSEnrollmentStatusEnum.values();

        for (EFTPSEnrollmentStatusEnum enrollmentStatusEnum : eftpsEnrollementStatus) {
            if (enrollmentStatusEnum.equals(EFTPSEnrollmentStatusEnum.NOTSET)){
                // Skip
                continue;
            }
            PTSNotificationRequest ptsNotificationRequest = createPTSNotificationRequest(
                    PTSNotificationTypeEnum.EFTPS_ENROLLMENT_NOTIFICATION, enrollmentStatusEnum);
            try {
                   updateNotificationRequest(ptsNotificationRequest);
                } catch (PTSNotificationException e) {
                Assert.fail(e.getLocalizedMessage(), e);
            }
        }
    }

    @BeforeMethod
    public void updatePPSPTSInteraction() throws ValidationException, ServiceFailureException{
        PTSEmployer ptsEmployer = employerPTSComponent.getPTSEmployer();
        // simulate "ADD COMPANY" status update
        employerPTSComponent.updatePPS_PTS_InteractionState(ptsEmployer.getEmployer(), InteractionEnum.ADDED);
    }
    /** Test to check that addition of legal data to PTS causes data disabling of update for
     *  EmployerTaxIdInfo.
     *  Also, further sending of information to PTS should be disabled
     * @throws ServiceFailureException
     * @throws ValidationException PTSEmployer ptsEmployer = employerPTSComponent.getPTSEmployer();
     * @throws PTSNotificationException
     **/
    @Test (enabled=true)
    public void testAddCompanyLocksEINData() throws ValidationException, ServiceFailureException, PTSNotificationException {
        PTSEmployer ptsEmployer = employerPTSComponent.getPTSEmployer();

        // check that the "ADD COMPANY STATUS" is there for this PTS Employer
        List<PPSPTSInteraction> list = StatusUpdateHandlerUtil.getPPSPTSInteractions(ptsEmployer.getEmployer());
        Assert.assertNotNull(list, "List should NOT be NULL");
        Assert.assertEquals(list.size(), 1, "List should have ONE and ONLY ONE element");

        // check that ein data update is disabled
        boolean einUpdateDisabled = StatusUpdateHandlerUtil.isEINOrLegalNameUpdateDisabled(ptsEmployer);
        Assert.assertEquals(einUpdateDisabled, true, "EIN Update should be disabled");

        // check that further data submission is disabled
        boolean furtherDataSubmission = StatusUpdateHandlerUtil.allowDataSubmissionToPTS(ptsEmployer);
        Assert.assertEquals(furtherDataSubmission, false, "Further data submission should be disabled");

        // create a PTSNotificationRequest
        PTSNotificationRequest ptsNotificationRequest = createPTSNotificationRequest(
                PTSNotificationTypeEnum.EFTPS_ENROLLMENT_NOTIFICATION, EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_REJECTED_EIN_MISMATCH);

        updateNotificationRequest(ptsNotificationRequest);

        // check that ein data update is enabled because there is an EINMismatch
        einUpdateDisabled = StatusUpdateHandlerUtil.isEINOrLegalNameUpdateDisabled(ptsEmployer);
        Assert.assertEquals(einUpdateDisabled, false, "EIN Update should be enabled");

        // check that further data submission is disabled
        furtherDataSubmission = StatusUpdateHandlerUtil.allowDataSubmissionToPTS(ptsEmployer);
        Assert.assertEquals(furtherDataSubmission, true, "Further data submission should be disabled");

        //simulate "UPDATE COMPANY" status update
        employerPTSComponent.updatePPS_PTS_InteractionState(ptsEmployer.getEmployer(), InteractionEnum.UPDATED);
        StatusUpdateHandlerUtil.inActivateAllEFTPSNotificationStates();

        // check that ein data update is disabled
        einUpdateDisabled = StatusUpdateHandlerUtil.isEINOrLegalNameUpdateDisabled(ptsEmployer);
        Assert.assertEquals(einUpdateDisabled, true, "EIN Update should be disabled");

        // check that further data submission is disabled
        furtherDataSubmission = StatusUpdateHandlerUtil.allowDataSubmissionToPTS(ptsEmployer);
        Assert.assertEquals(furtherDataSubmission, false, "Further data submission should be disabled");

        // create a PTSNotificationRequest
        ptsNotificationRequest = createPTSNotificationRequest(
                PTSNotificationTypeEnum.EFTPS_ENROLLMENT_NOTIFICATION, EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_REJECTED_NAME_MISMATCH);
        updateNotificationRequest(ptsNotificationRequest);

        // check that ein data update is enabled because there is an NameMismatch
        einUpdateDisabled = StatusUpdateHandlerUtil.isEINOrLegalNameUpdateDisabled(ptsEmployer);
        Assert.assertEquals(einUpdateDisabled, false, "EIN Update should be enabled");

        // check that further data submission is disabled
        furtherDataSubmission = StatusUpdateHandlerUtil.allowDataSubmissionToPTS(ptsEmployer);
        Assert.assertEquals(furtherDataSubmission, true, "Further data submission should be disabled");

        //simulate "UPDATE COMPANY" status update
        employerPTSComponent.updatePPS_PTS_InteractionState(ptsEmployer.getEmployer(), InteractionEnum.UPDATED);
        StatusUpdateHandlerUtil.inActivateAllEFTPSNotificationStates();
        
        // check that ein data update is disabled
        einUpdateDisabled = StatusUpdateHandlerUtil.isEINOrLegalNameUpdateDisabled(ptsEmployer);
        Assert.assertEquals(einUpdateDisabled, true, "EIN Update should be disabled");
    }

    @Test(enabled=true)
    public void testLockCompanyDataWithPastStatusDate() throws Exception{
        PTSEmployer ptsEmployer = employerPTSComponent.getPTSEmployer();

        // check that the "ADD COMPANY STATUS" is there for this PTS Employer
        List<PPSPTSInteraction> list = StatusUpdateHandlerUtil.getPPSPTSInteractions(ptsEmployer.getEmployer());
        Assert.assertNotNull(list, "List should NOT be NULL");
        Assert.assertEquals(list.size(), 1, "List should have ONE and ONLY ONE element");

        // check that ein data update is disabled
        boolean einUpdateDisabled = StatusUpdateHandlerUtil.isEINOrLegalNameUpdateDisabled(ptsEmployer);
        Assert.assertEquals(einUpdateDisabled, true, "EIN Update should be disabled");

        // check that further data submission is disabled
        boolean furtherDataSubmission = StatusUpdateHandlerUtil.allowDataSubmissionToPTS(ptsEmployer);
        Assert.assertEquals(furtherDataSubmission, false, "Further data submission should be disabled");

        // create a PTSNotificationRequest
        PTSNotificationRequest ptsNotificationRequest = createPTSNotificationRequest(
                PTSNotificationTypeEnum.EFTPS_ENROLLMENT_NOTIFICATION, EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_REJECTED_EIN_MISMATCH);

        Calendar statusDate = new GregorianCalendar();
        int month = statusDate.get(Calendar.MONTH);
        statusDate.set(Calendar.MONTH, month-1);

        ptsNotificationRequest.setTimestamp(statusDate.getTime());

        updateNotificationRequest(ptsNotificationRequest);

        // check that ein data update is disabled
        // though there is an eftps_enrollment_rejected notification, the status date is in the past,
        // so the update has to be disabled
        einUpdateDisabled = StatusUpdateHandlerUtil.isEINOrLegalNameUpdateDisabled(ptsEmployer);
        Assert.assertEquals(einUpdateDisabled, false, "EIN Update should be enabled");

        // check that further data submission is disabled
        //furtherDataSubmission = StatusUpdateHandlerUtil.allowDataSubmissionToPTS(ptsEmployer);
        //Assert.assertEquals(furtherDataSubmission, false, "Further data submission should be disabled");

    }

}
