package com.intuit.sb.payroll.capability.taxes.withholding;

import java.math.BigDecimal;

import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.DE4Detail;
import com.intuit.sb.payroll.entity.model.DE4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.W4Detail;
import com.intuit.sb.payroll.entity.model.W4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;

/**
 * <P>
 * The <CODE>UpdateWorkerComplianceDocDtlTest</CODE> class contains tests for testing the Exceptions that can be thrown
 * by updateWorkerComplianceDocDtl() in TaxWithholdingsComponent
 * </P>
 * 
 * $Author$ $Revision$ $DateTime$
 * 
 */
public class UpdateWorkerComplianceDocDtlExceptionsTest extends TaxWithholdingsComponentTestBase {
	private static Worker sEmployee;

    @BeforeMethod
    public static void initializeBeforeTest() {
    	sEmployee = WorkerTestUtils.createEmployee();
    }
    
    @AfterMethod
    public static void cleanUpAfterTest() {
    	sEmployee = null;
    }

	/**
	 * <P>Test calling updateWorkerComplianceDocDtl() with a null Worker</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsComponent.updateWorkerComplianceDocDtl() with a null Worker</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown</LI>
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullWorker() throws Exception {
		WorkerComplianceDocDtl workerComplianceDocDtl = sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.W4IRS);
    	// Exercise
		sTaxWithholdingsComponent.updateWorkerComplianceDocDtl(null, workerComplianceDocDtl);
	}

	/**
	 * <P>Test calling updateWorkerComplianceDocDtl() with a null WorkerComplianceDocDtl</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set the worker's WorkLocation</LI>
	 * <LI>Apply the applicable taxes</LI>
	 * <LI>Call TaxWithholdingsComponent.updateWorkerComplianceDocDtl() with a null WorkerComplianceDocDtl</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown</LI>
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullWorkerComplianceDocDtl() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
    	// Exercise
		sTaxWithholdingsComponent.updateWorkerComplianceDocDtl(sEmployee, null);
	}

	/**
	 * <P>Test calling updateWorkerComplianceDocDtl() with no taxes applied</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set the worker's WorkLocation</LI>
	 * <LI>Call TaxWithholdingsComponent.updateWorkerComplianceDocDtl() with no taxes applied</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that WorkflowException is thrown</LI>
	 * </UL>
	 */
	@Test(expectedExceptions = WorkflowException.class)
	public void testTaxesNotApplied() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
		WorkerComplianceDocDtl workerComplianceDocDtl = sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.W4IRS);
    	// Exercise
		sTaxWithholdingsComponent.updateWorkerComplianceDocDtl(sEmployee, workerComplianceDocDtl);
	}

	/**
	 * <P>Test calling updateWorkerComplianceDocDtl() with no WorkLocation set</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsComponent.updateWorkerComplianceDocDtl() with no WorkLocation set</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that WorkflowException is thrown</LI>
	 * </UL>
	 */
	@Test(expectedExceptions = WorkflowException.class)
	public void testNoWorkLocationSet() throws Exception {
		// Setup
		WorkerComplianceDocDtl workerComplianceDocDtl = sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.W4IRS);
    	// Exercise
		sTaxWithholdingsComponent.updateWorkerComplianceDocDtl(sEmployee, workerComplianceDocDtl);
	}

	/**
	 * <P>Test calling updateWorkerComplianceDocDtl() with a negative W4 allowance</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set the worker's WorkLocation</LI>
	 * <LI>Apply the applicable taxes</LI>
	 * <LI>Call TaxWithholdingsComponent.updateWorkerComplianceDocDtl() with a negative W4 allowance</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown</LI>
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNegativeW4Allowance() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
		WorkerComplianceDocDtl workerComplianceDocDtl = sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.W4IRS);
		W4Detail w4Detail = (W4Detail)workerComplianceDocDtl;
		w4Detail.setNoOfAllowances(-1);
    	// Exercise
		sTaxWithholdingsComponent.updateWorkerComplianceDocDtl(sEmployee, w4Detail);
	}

	/**
	 * <P>Test calling updateWorkerComplianceDocDtl() with a negative W4 extra withholding</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set the worker's WorkLocation</LI>
	 * <LI>Apply the applicable taxes</LI>
	 * <LI>Call TaxWithholdingsComponent.updateWorkerComplianceDocDtl() with a negative W4 extra withholding</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown</LI>
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNegativeW4ExtraWithholding() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
		WorkerComplianceDocDtl workerComplianceDocDtl = sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.W4IRS);
		W4Detail w4Detail = (W4Detail)workerComplianceDocDtl;
		w4Detail.setNoOfAllowances(1);
		Money w4AdditionalAmount = new Money();
		w4AdditionalAmount.setAmount(new BigDecimal("-100.00"));
		w4Detail.setAdditionalAmount(w4AdditionalAmount);
    	// Exercise
		sTaxWithholdingsComponent.updateWorkerComplianceDocDtl(sEmployee, w4Detail);
	}

	/**
	 * <P>Test calling updateWorkerComplianceDocDtl() with the WithholdingStatus set to null</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsComponent.updateWorkerComplianceDocDtl() with the WithholdingStatus set to null</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullW4WithholdingStatus() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
    	// Setup a W4Detail
    	W4Detail w4Detail = (W4Detail)sTaxWithholdingsComponent.createWorkerComplianceDocDtl(ComplianceDocIdEnum.W4IRS);
		w4Detail.setDerived(false);
		w4Detail.setTaxExempt(false);
		w4Detail.setWithholdingStatus(W4WithholdingStatusEnum.SINGLE);
		w4Detail.setNoOfAllowances(1);
		// Add the W4Detail
		sEmployee = sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, w4Detail);
		WorkerComponent workerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);
		sEmployee = workerComponent.updateWorker(sEmployee);
		W4Detail savedW4Detail = getW4Detail(sEmployee);
		savedW4Detail.setWithholdingStatus(null);
    	// Exercise
		sTaxWithholdingsComponent.updateWorkerComplianceDocDtl(sEmployee, savedW4Detail);
	}

	/**
	 * <P>Test calling updateWorkerComplianceDocDtl() with the WithholdingStatus not set</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsComponent.updateWorkerComplianceDocDtl() with the WithholdingStatus not set</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testW4WithholdingStatusNotSet() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
    	// Setup a W4Detail
    	W4Detail w4Detail = (W4Detail)sTaxWithholdingsComponent.createWorkerComplianceDocDtl(ComplianceDocIdEnum.W4IRS);
		w4Detail.setDerived(false);
		w4Detail.setTaxExempt(false);
		w4Detail.setWithholdingStatus(W4WithholdingStatusEnum.SINGLE);
		w4Detail.setNoOfAllowances(1);
		// Add the W4Detail
		sEmployee = sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, w4Detail);
		WorkerComponent workerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);
		sEmployee = workerComponent.updateWorker(sEmployee);
		W4Detail savedW4Detail = getW4Detail(sEmployee);
		savedW4Detail.setWithholdingStatus(W4WithholdingStatusEnum.NOTSET);
    	// Exercise
		sTaxWithholdingsComponent.updateWorkerComplianceDocDtl(sEmployee, savedW4Detail);
	}

	/**
	 * <P>Test calling updateWorkerComplianceDocDtl() with a negative DE4 allowance</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set the worker's WorkLocation</LI>
	 * <LI>Apply the applicable taxes</LI>
	 * <LI>Call TaxWithholdingsComponent.updateWorkerComplianceDocDtl() with a negative DE4 allowance</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown</LI>
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNegativeDE4Allowance() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
		WorkerComplianceDocDtl workerComplianceDocDtl = sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.DE4);
		DE4Detail de4Detail = (DE4Detail)workerComplianceDocDtl;
		de4Detail.setNoOfAllowances(-1);
    	// Exercise
		sTaxWithholdingsComponent.updateWorkerComplianceDocDtl(sEmployee, de4Detail);
	}

	/**
	 * <P>Test calling updateWorkerComplianceDocDtl() with a negative DE4 extra withholding</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set the worker's WorkLocation</LI>
	 * <LI>Apply the applicable taxes</LI>
	 * <LI>Call TaxWithholdingsComponent.updateWorkerComplianceDocDtl() with a negative DE4 extra withholding</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown</LI>
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNegativeDE4ExtraWithholding() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
		WorkerComplianceDocDtl workerComplianceDocDtl = sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.DE4);
		DE4Detail de4Detail = (DE4Detail)workerComplianceDocDtl;
		de4Detail.setNoOfAllowances(1);
		Money de4AdditionalAmount = new Money();
		de4AdditionalAmount.setAmount(new BigDecimal("-100.00"));
		de4Detail.setAdditionalAmount(de4AdditionalAmount);
    	// Exercise
		sTaxWithholdingsComponent.updateWorkerComplianceDocDtl(sEmployee, de4Detail);
	}

	/**
	 * <P>Test calling updateWorkerComplianceDocDtl() with the WithholdingStatus set to null</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsComponent.updateWorkerComplianceDocDtl() with the WithholdingStatus set to null</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullDE4WithholdingStatus() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
    	// Setup a DE4Detail
    	DE4Detail de4Detail = (DE4Detail)sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.DE4);
		de4Detail.setDerived(false);
		de4Detail.setTaxExempt(false);
		de4Detail.setWithholdingStatus(DE4WithholdingStatusEnum.SINGLE);
		de4Detail.setNoOfAllowances(1);
		// Add the DE4Detail
		sEmployee = sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, de4Detail);
		WorkerComponent workerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);
		sEmployee = workerComponent.updateWorker(sEmployee);
		DE4Detail savedDe4Detail = getDe4Detail(sEmployee);
		savedDe4Detail.setWithholdingStatus(null);
    	// Exercise
		sTaxWithholdingsComponent.updateWorkerComplianceDocDtl(sEmployee, savedDe4Detail);
	}

	/**
	 * <P>Test calling updateWorkerComplianceDocDtl() with the WithholdingStatus not set</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsComponent.updateWorkerComplianceDocDtl() with the WithholdingStatus not set</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testDE4WithholdingStatusNotSet() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
    	// Setup a DE4Detail
    	DE4Detail de4Detail = (DE4Detail)sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.DE4);
		de4Detail.setDerived(false);
		de4Detail.setTaxExempt(false);
		de4Detail.setWithholdingStatus(DE4WithholdingStatusEnum.SINGLE);
		de4Detail.setNoOfAllowances(1);
		// Add the DE4Detail
		sEmployee = sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, de4Detail);
		WorkerComponent workerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);
		sEmployee = workerComponent.updateWorker(sEmployee);
		DE4Detail savedDe4Detail = getDe4Detail(sEmployee);
		savedDe4Detail.setWithholdingStatus(DE4WithholdingStatusEnum.NOTSET);
    	// Exercise
		sTaxWithholdingsComponent.updateWorkerComplianceDocDtl(sEmployee, savedDe4Detail);
	}

	/**
	 * <P>Test calling updateWorkerComplianceDocDtl() with an unsaved WorkerComplianceDocDtl</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set the worker's WorkLocation</LI>
	 * <LI>Apply the applicable taxes</LI>
	 * <LI>Call TaxWithholdingsComponent.updateWorkerComplianceDocDtl() with an unsaved WorkerComplianceDocDtl</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown</LI>
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testWithUnsavedComplianceDocDtl() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
		WorkerComplianceDocDtl workerComplianceDocDtl = sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.W4IRS);
		W4Detail w4Detail = (W4Detail)workerComplianceDocDtl;
		w4Detail.setNoOfAllowances(1);
    	// Exercise
		sTaxWithholdingsComponent.updateWorkerComplianceDocDtl(sEmployee, w4Detail);
	}

	/**
	 * <P>Test calling updateWorkerComplianceDocDtl() with a WorkerComplianceDocDtl that doesn't exist on the Worker</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set the worker's WorkLocation</LI>
	 * <LI>Apply the applicable taxes</LI>
	 * <LI>Add a W4Detail to the worker</LI>
	 * <LI>Remove the W4Detail from the worker</LI>
	 * <LI>Call TaxWithholdingsComponent.updateWorkerComplianceDocDtl() with a WorkerComplianceDocDtl that doesn't
	 * exists on the Worker</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ServiceFailureException is thrown</LI>
	 * </UL>
	 */
	@Test(expectedExceptions = ServiceFailureException.class)
	public void testWithNonExistentComplianceDocDtl() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
		WorkerComplianceDocDtl workerComplianceDocDtl = sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.W4IRS);
		W4Detail w4Detail = (W4Detail)workerComplianceDocDtl;
		w4Detail.setNoOfAllowances(1);
		w4Detail.setDerived(false);
		w4Detail.setTaxExempt(false);
		// Add the W4Detail
		sEmployee = sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, w4Detail);
		WorkerComponent workerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);
		sEmployee = workerComponent.updateWorker(sEmployee);
		W4Detail savedW4Detail = (W4Detail)sEmployee.getWorkerComplianceDocDtls().iterator().next();
		// Now remove the W4Detail from the Worker
		sEmployee.removeWorkerComplianceDocDtl(savedW4Detail);
    	// Exercise
		savedW4Detail.setTaxExempt(true);
		sTaxWithholdingsComponent.updateWorkerComplianceDocDtl(sEmployee, savedW4Detail);
	}

}
