package com.intuit.sb.payroll.capability.worker;

import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.capability.task.impl.TaskUtils;
import com.intuit.sb.payroll.entity.model.PersonName;
import com.intuit.sb.payroll.entity.model.Task;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 *  <P>The <CODE>DeleteWorkerTest</CODE> class contains tests for testing deleteWorker() in WorkerComponent</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class DeleteWorkerTest extends WorkerComponentTestBase
{

	/**
	 * One-time set up for this class.
	 */
    @BeforeClass
    public static void setUpDeleteWorkerTestClass() {
    	sWorkerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);
        
    	try {
            getUserTransaction().begin();
            saveEmployerInfo();
            saveWorkLocationAddress();
            setUpSanMateoAddress();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in setUpDeleteWorkerTestClass(): " + t.getMessage());
        }
    }
    
    /**
     * One-time tear down for this class.
     */
    @AfterClass
    public static void tearDownDeleteWorkerTestClass() {
        try {
            getUserTransaction().begin();
            restoreWorkLocationAddress();
            restoreEmployerInfo();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in tearDownDeleteWorkerTestClass(): " + t.getMessage());
        }
    }
    
    /**
     * <P>Test calling deleteWorker() with non-persisted Worker object with first name only</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create non-persisted Worker object with first name only</LI>
     * <LI>Call WorkerComponent.deleteWorker() with Worker object</LI>
     * <LI>Verify that no Exception is thrown</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.deleteWorker"})
    public void testDeleteWorkerWithFirstNameOnlyInWorkerObject() throws Exception {
    	// Setup
    	Worker worker = new Worker();
    	PersonName personName = new PersonName();
    	personName.setFirstName("Joe");
    	worker.setPersonName(personName);
    	// Exercise
    	sWorkerComponent.deleteWorker(worker);
    }

    /**
     * <P>Test calling deleteWorker() with a persisted Worker object with no wages</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create persisted Worker with no wages</LI>
     * <LI>Call WorkerComponent.deleteWorker() with Worker object</LI>
     * <LI>Verify that all tasks for this worker are removed and that it is not in the DB any more</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.deleteWorker"})
    public void testDeleteWorkerWithNoWages() throws Exception {
    	// Setup
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Joe", sMountainViewLocation);
    	long workerId = worker.getId();
    	List<Task> tasks = TaskUtils.getAllWorkerTasks(workerId);
    	AssertJUnit.assertTrue("Worker should have existing tasks!", tasks.size() > 0);
    	// Exercise
    	sWorkerComponent.deleteWorker(worker);
    	// Verify
    	tasks = TaskUtils.getAllWorkerTasks(workerId);
    	AssertJUnit.assertTrue("Worker tasks not removed after deleting worker!", tasks.size() == 0);
    	worker = null;
    	try {
    		worker = sWorkerComponent.findWorker(workerId);
    	} catch (ServiceFailureException e) {
    		// This is the expected result if worker is deleted
    	}
    	AssertJUnit.assertNull("Worker is not deleted!", worker);
    }

    /**
     * <P>Test calling deleteWorker() with a persisted Worker object with no withholdings</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create persisted Worker with no withholdings</LI>
     * <LI>Call WorkerComponent.deleteWorker() with Worker object</LI>
     * <LI>Verify that all tasks for this worker are removed and that it is not in the DB any more</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.deleteWorker"})
    public void testDeleteWorkerWithNoWithholdings() throws Exception {
    	// Setup
    	Worker worker = WorkerTestUtils.createAPayrollReadyWorkerWithNoWithHolding("Joe", sMountainViewLocation);
    	long workerId = worker.getId();
    	List<Task> tasks = TaskUtils.getAllWorkerTasks(workerId);
    	AssertJUnit.assertTrue("Worker should have existing tasks!", tasks.size() > 0);
    	// Exercise
    	sWorkerComponent.deleteWorker(worker);
    	// Verify
    	tasks = TaskUtils.getAllWorkerTasks(workerId);
    	AssertJUnit.assertTrue("Worker tasks not removed after deleting worker!", tasks.size() == 0);
    	worker = null;
    	try {
    		worker = sWorkerComponent.findWorker(workerId);
    	} catch (ServiceFailureException e) {
    		// This is the expected result if worker is deleted
    	}
    	AssertJUnit.assertNull("Worker is not deleted!", worker);
    }

    /**
     * <P>Test calling deleteWorker() with a persisted salary Worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create persisted salary Worker</LI>
     * <LI>Call WorkerComponent.deleteWorker() with Worker object</LI>
     * <LI>Verify that all tasks for this worker are removed and that it is not in the DB any more</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.deleteWorker"})
    public void testDeleteSalaryWorker() throws Exception {
    	// Setup
    	Worker worker = WorkerTestUtils.createASalaryPayrollReadyWorker("Joe", sMountainViewLocation);
    	long workerId = worker.getId();
    	List<Task> tasks = TaskUtils.getAllWorkerTasks(workerId);
    	AssertJUnit.assertTrue("Worker should have existing tasks!", tasks.size() > 0);
    	// Exercise
    	sWorkerComponent.deleteWorker(worker);
    	// Verify
    	tasks = TaskUtils.getAllWorkerTasks(workerId);
    	AssertJUnit.assertTrue("Worker tasks not removed after deleting worker!", tasks.size() == 0);
    	worker = null;
    	try {
    		worker = sWorkerComponent.findWorker(workerId);
    	} catch (ServiceFailureException e) {
    		// This is the expected result if worker is deleted
    	}
    	AssertJUnit.assertNull("Worker is not deleted!", worker);
    }

}
