package com.intuit.sb.payroll.capability.worker;

import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.entity.model.Worker;

/**
 *  <P>The <CODE>FindWorkersByFilterTest</CODE> class contains tests for testing the findWorkersByFilter() in
 *   WorkerComponent</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class FindWorkersByFilterTest extends WorkerComponentTestBase
{
	private static Long sReadyForPayWorkerId = 0L;
	private static Long sReadyForPayWorkerId2 = 0L;

	/**
	 * One-time set up for this class.
	 *   Creates 2 payroll ready workers (1 active and 1 inactive)
	 */
    @BeforeClass
    public static void setUpFindWorkersByFilterTestClass() {
    	sWorkerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);
        
    	try {
            getUserTransaction().begin();
            saveEmployerInfo();
            saveWorkLocationAddress();
            setUpSanMateoAddress();
            // Create an active payroll ready worker
			sReadyForPayWorkerId = WorkerTestUtils.createPayrollReadyWorker("Johnny", sMountainViewLocation);
            // Create an inactive payroll ready worker
			Worker worker = WorkerTestUtils.createAPayrollReadyWorker("Paul", sMountainViewLocation);
			sReadyForPayWorkerId2 = worker.getId();
			worker.setActive(false);
			sWorkerComponent.updateWorker(worker);
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in setUpFindWorkersByFilterTestClass(): " + t.getMessage());
        }
    }
    
    /**
     * One-time tear down for this class.
     *   Deletes the 2 workers that were created in this class
     */
    @AfterClass
    public static void tearDownFindWorkersByFilterTestClass() {
        try {
            getUserTransaction().begin();
			if (0 != sReadyForPayWorkerId) {
    			Worker worker = entityService.findById(Worker.class, sReadyForPayWorkerId);
    			WorkerTestUtils.cleanUpWorker(worker);
    			sReadyForPayWorkerId = 0L;
			}
			if (0 != sReadyForPayWorkerId2) {
    			Worker worker = entityService.findById(Worker.class, sReadyForPayWorkerId2);
    			WorkerTestUtils.cleanUpWorker(worker);
    			sReadyForPayWorkerId2 = 0L;
			}
            restoreWorkLocationAddress();
            restoreEmployerInfo();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in tearDownFindWorkersByFilterTestClass(): " + t.getMessage());
        }
    }
    
    /**
     * <P>Test finding all workers using FindWorkersFilter.ALL</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerComponent.findWorkersByFilter() with FindWorkersFilter.ALL to find all workers</LI>
     * <LI>Verify the number of workers returned match the expected value</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.FindWorkersFilter"})
    public void testFindAllWorkers() throws Exception {
    	// There are 3 workers (2 non payroll ready employees and 1 non-employee) that were seeded by SeedData,
    	//   and 2 payroll ready employees (1 active & 1 inactive) are created in setUpFindWorkersByFilterTestClass()  
    	List<Worker> workers = sWorkerComponent.findWorkersByFilter(FindWorkersFilter.ALL);
        AssertJUnit.assertEquals("Expected number of workers mismatch!", 5, workers.size());
    }

    /**
     * <P>Test finding all payroll ready workers using FindWorkersFilter.READY_FOR_PAYROLL</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerComponent.findWorkersByFilter() with FindWorkersFilter.READY_FOR_PAYROLL to find all
     *     payroll ready workers</LI>
     * <LI>Verify the number of workers returned match the expected value</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.FindWorkersFilter"})
    public void testFindAllPayrollReadyWorkers() throws Exception {
    	// There are 3 workers (2 non payroll ready employees and 1 non-employee) that were seeded by SeedData,
    	//   and 2 payroll ready employees (1 active & 1 inactive) are created in setUpFindWorkersByFilterTestClass()  
    	List<Worker> workers = sWorkerComponent.findWorkersByFilter(FindWorkersFilter.READY_FOR_PAYROLL);
        AssertJUnit.assertEquals("Expected number of payroll ready workers mismatch!", 2, workers.size());
    }

    /**
     * <P>Test finding all non payroll ready workers using FindWorkersFilter.NOT_READY_FOR_PAYROLL</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerComponent.findWorkersByFilter() with FindWorkersFilter.NOT_READY_FOR_PAYROLL to find all
     *     non payroll ready workers</LI>
     * <LI>Verify the number of workers returned match the expected value</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.FindWorkersFilter"})
    public void testFindAllNonPayrollReadyWorkers() throws Exception {
    	// There are 3 workers (2 non payroll ready employees and 1 non-employee) that were seeded by SeedData,
    	//   and 2 payroll ready employees (1 active & 1 inactive) are created in setUpFindWorkersByFilterTestClass()  
    	List<Worker> workers = sWorkerComponent.findWorkersByFilter(FindWorkersFilter.NOT_READY_FOR_PAYROLL);
        AssertJUnit.assertEquals("Expected number of non payroll ready workers mismatch!", 3, workers.size());
    }

    /**
     * <P>Test finding all active workers using FindWorkersFilter.ACTIVE</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerComponent.findWorkersByFilter() with FindWorkersFilter.ACTIVE to find all active workers</LI>
     * <LI>Verify the number of workers returned match the expected value</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.FindWorkersFilter"})
    public void testFindAllActiveWorkers() throws Exception {
    	// There are 3 workers (2 non payroll ready employees and 1 non-employee) that were seeded by SeedData,
    	//   and 2 payroll ready employees (1 active & 1 inactive) are created in setUpFindWorkersByFilterTestClass()  
    	List<Worker> workers = sWorkerComponent.findWorkersByFilter(FindWorkersFilter.ACTIVE);
        AssertJUnit.assertEquals("Expected number of active workers mismatch!", 4, workers.size());
    }

    /**
     * <P>Test finding all inactive workers using FindWorkersFilter.INACTIVE</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerComponent.findWorkersByFilter() with FindWorkersFilter.INACTIVE to find all inactive workers</LI>
     * <LI>Verify the number of workers returned match the expected value</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.FindWorkersFilter"})
    public void testFindAllInactiveWorkers() throws Exception {
    	// There are 3 workers (2 non payroll ready employees and 1 non-employee) that were seeded by SeedData,
    	//   and 2 payroll ready employees (1 active & 1 inactive) are created in setUpFindWorkersByFilterTestClass()  
    	List<Worker> workers = sWorkerComponent.findWorkersByFilter(FindWorkersFilter.INACTIVE);
        AssertJUnit.assertEquals("Expected number of inactive workers mismatch!", 1, workers.size());
    }

    /**
     * <P>Test finding all workers using FindWorkersFilter.ALL when there are no workers in the database</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerComponent.findWorkersByFilter() with FindWorkersFilter.ALL to find all workers</LI>
     * <LI>Delete these workers</LI>
     * <LI>Call WorkerComponent.findWorkersByFilter() with FindWorkersFilter.ALL to find all workers</LI>
     * <LI>Verify the number of workers returned is 0</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.FindWorkersFilter"})
    public void testFindAllWorkersWithNoWorkerInDatabase() throws Exception {
    	// There are 3 workers (2 non payroll ready employees and 1 non-employee) that were seeded by SeedData,
    	//   and 2 payroll ready employees (1 active & 1 inactive) are created in setUpFindWorkersByFilterTestClass()  
    	List<Worker> workers = sWorkerComponent.findWorkersByFilter(FindWorkersFilter.ALL);
    	for (Worker worker : workers) {
    		WorkerTestUtils.cleanUpWorker(worker);
    	}
    	workers = sWorkerComponent.findWorkersByFilter(FindWorkersFilter.ALL);
        AssertJUnit.assertEquals("Number of workers returned is not zero!", 0, workers.size());
    }

    /**
     * <P>Test finding all workers with a null FindWorkersFilter</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerComponent.findWorkersByFilter() with null</LI>
     * <LI>Verify that null is returned</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.FindWorkersFilter"})
    public void testFindAllWorkersWithNullFilter() throws Exception {
    	// There are 3 workers (2 non payroll ready employees and 1 non-employee) that were seeded by SeedData,
    	//   and 2 payroll ready employees (1 active & 1 inactive) are created in setUpFindWorkersByFilterTestClass()  
    	List<Worker> workers = sWorkerComponent.findWorkersByFilter(null);
        AssertJUnit.assertNull("Null is not returned!", workers);
    }

}
