package com.intuit.sb.payroll.capability.worker;

import java.util.Calendar;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.EmploymentPeriod;
import com.intuit.sb.payroll.entity.model.EmploymentStatusEnum;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 *  <P>The <CODE>HireExistingWorkerTest</CODE> class contains tests for testing hireExistingWorker() in WorkerComponent</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class HireExistingWorkerTest extends WorkerComponentTestBase
{

	/**
	 * One-time set up for this class.
	 */
    @BeforeClass
    public static void setUpHireExistingWorkerTestClass() {
    	sWorkerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);
        
    	try {
            getUserTransaction().begin();
            saveEmployerInfo();
            saveWorkLocationAddress();
            setUpSanMateoAddress();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in setUpHireExistingWorkerTestClass(): " + t.getMessage());
        }
    }
    
    /**
     * One-time tear down for this class.
     */
    @AfterClass
    public static void tearDownHireExistingWorkerTestClass() {
        try {
            getUserTransaction().begin();
            restoreWorkLocationAddress();
            restoreEmployerInfo();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in tearDownHireExistingWorkerTestClass(): " + t.getMessage());
        }
    }
    
    /**
     * This utility method will only verify the DateRange values of EmployementPeriod and not the other fields
     * 
     * @param expected The expected EmploymentPeriod set
     * @param computed The computed EmploymentPeriod set
     * @return true if the DateRange values in both sets are the same, false otherwise
     */
    private boolean verifyEmploymentPeriods(Set<EmploymentPeriod> expected, Set<EmploymentPeriod> computed) {
        boolean compareResult = (expected.size() == computed.size());
        if (true == compareResult) {
            Set<DateRange> computedSet = new HashSet<DateRange>();
            for (EmploymentPeriod computedItem : computed) {
                computedSet.add(computedItem.getEmploymentPeriod());
            } 
            
            Set<DateRange> expectedSet = new HashSet<DateRange>();
            for(EmploymentPeriod expectedItem: expected) {
                expectedSet.add(expectedItem.getEmploymentPeriod());
            }
            compareResult = computedSet.equals(expectedSet);
        }

        if (false == compareResult) {
            sLogger.error("Computed EmploymentPeriod:");            
            for (EmploymentPeriod employmentPeriod : computed) {
                sLogger.error(employmentPeriod.toString());            
            }            
            sLogger.error("Expected EmploymentPeriod:");            
            for (EmploymentPeriod employmentPeriod : expected) {
                sLogger.error(employmentPeriod.toString());            
            }            
        }
        return compareResult;
    }
    
    /**
     * <P>Test re-hiring a worker on the same day that the worker was terminated</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Terminate the worker</LI>
     * <LI>Call WorkerComponent.hireExistingWorker() to re-hire the worker</LI>
     * <LI>Verify expected results</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.hireExistingWorker"})
    public void testRehireWorkerOnDayOfTermination() throws Exception {
    	// Setup
		Worker worker = WorkerTestUtils.createAPayrollReadyWorker("Johnny", sMountainViewLocation);
		// Set the termination date to yesterday
		Date terminationDate = DateUtil.getNextDate(DateUtil.getNowDate(), -1);
    	worker = sWorkerComponent.terminateWorker(worker, terminationDate);
		// Exercise
    	worker = sWorkerComponent.hireExistingWorker(worker.getId(), terminationDate);
    	// Verify
    	AssertJUnit.assertTrue("Worker should have been marked active!", worker.isActive());
    	AssertJUnit.assertTrue("Current EmploymentPeriod start date mismatch!",
    			DateUtil.isSameDay(WorkerTestUtils.HIRE_DATE, worker.getEmploymentPeriod().getStartDate()));
    	AssertJUnit.assertNull("Current EmploymentPeriod end date is not null!", worker.getEmploymentPeriod().getEndDate());
    	AssertJUnit.assertEquals("Worker's employment status was not set to EMPLOYED!",
    			EmploymentStatusEnum.EMPLOYED, worker.getEmploymentStatus());
    }

    /**
     * <P>Test re-hiring a worker on a date before the worker's scheduled termination date</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Terminate the worker with a termination date 1 week from today</LI>
     * <LI>Call WorkerComponent.hireExistingWorker() to re-hire the worker with a hire date of tomorrow</LI>
     * <LI>Verify expected results</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.hireExistingWorker"})
    public void testRehireWorkerBeforeScheduledTermination() throws Exception {
    	// Setup
		Worker worker = WorkerTestUtils.createAPayrollReadyWorker("Johnny", sMountainViewLocation);
		Date today = DateUtil.getNowDate();
		// Set the termination date to 1 week from today
		Date terminationDate = DateUtil.getNextDate(today, 7);
    	worker = sWorkerComponent.terminateWorker(worker, terminationDate);
		// Exercise
    	Date hireDate = DateUtil.getNextDate(today, 1);
    	worker = sWorkerComponent.hireExistingWorker(worker.getId(), hireDate);
    	// Verify
    	AssertJUnit.assertTrue("Worker should have been marked active!", worker.isActive());
    	AssertJUnit.assertTrue("Current EmploymentPeriod start date mismatch!",
    			DateUtil.isSameDay(WorkerTestUtils.HIRE_DATE, worker.getEmploymentPeriod().getStartDate()));
    	AssertJUnit.assertNull("Current EmploymentPeriod end date is not null!", worker.getEmploymentPeriod().getEndDate());
    	AssertJUnit.assertEquals("Worker's employment status was not set to EMPLOYED!",
    			EmploymentStatusEnum.EMPLOYED, worker.getEmploymentStatus());
    }

    /**
     * <P>Test re-hiring a terminated worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Terminate the worker with a termination date 1 week before today</LI>
     * <LI>Call WorkerComponent.hireExistingWorker() to re-hire the worker with a hire date of today</LI>
     * <LI>Verify expected results</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.hireExistingWorker"})
    public void testRehireWorkerAfterTermination() throws Exception {
    	// Setup
		Worker worker = WorkerTestUtils.createAPayrollReadyWorker("Johnny", sMountainViewLocation);
		Date today = DateUtil.getNowDate();
		// Set the termination date to 1 week before today
		Date terminationDate = DateUtil.getNextDate(today, -7);
    	worker = sWorkerComponent.terminateWorker(worker, terminationDate);
		// Exercise
    	worker = sWorkerComponent.hireExistingWorker(worker.getId(), today);
    	// Verify
    	AssertJUnit.assertTrue("Worker should have been marked active!", worker.isActive());
    	AssertJUnit.assertTrue("Current EmploymentPeriod start date mismatch!",
    			DateUtil.isSameDay(today, worker.getEmploymentPeriod().getStartDate()));
    	AssertJUnit.assertNull("Current EmploymentPeriod end date is not null!", worker.getEmploymentPeriod().getEndDate());
    	AssertJUnit.assertEquals("Worker's employment status was not set to EMPLOYED!",
    			EmploymentStatusEnum.EMPLOYED, worker.getEmploymentStatus());
    	Set<EmploymentPeriod> expectedPeriods = new HashSet<EmploymentPeriod>();
    	DateRange dateRange = new DateRange();
    	dateRange.setStartDate(WorkerTestUtils.HIRE_DATE);
    	dateRange.setEndDate(terminationDate);
    	EmploymentPeriod period = new EmploymentPeriod();
    	period.setEmploymentPeriod(dateRange);
    	expectedPeriods.add(period);
    	AssertJUnit.assertTrue("Historical EmploymentPeriod mismatch!", verifyEmploymentPeriods(expectedPeriods,
    			worker.getEmploymentPeriods()));
    }

    /**
     * <P>Test re-hiring a terminated worker with a future date</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Terminate the worker with a termination date 1 week before today</LI>
     * <LI>Call WorkerComponent.hireExistingWorker() to re-hire the worker with a future date</LI>
     * <LI>Verify expected results</LI>
     * </OL>
     * 
     */
    //TODO  Enable this test when the engine code handle changing the status on the actual hire date
    @Test(groups = {"worker.hireExistingWorker"}, enabled=false)
    public void testRehireWorkerAfterTerminationWithFutureDate() throws Exception {
    	// Setup
		Worker worker = WorkerTestUtils.createAPayrollReadyWorker("Johnny", sMountainViewLocation);
		Date today = DateUtil.getNowDate();
		Date futureDay = DateUtil.getNextDate(today, 7);
		// Set the termination date to 1 week before today
		Date terminationDate = DateUtil.getNextDate(today, -7);
    	worker = sWorkerComponent.terminateWorker(worker, terminationDate);
		// Exercise
    	worker = sWorkerComponent.hireExistingWorker(worker.getId(), futureDay);
    	// Verify
    	AssertJUnit.assertFalse("Worker should have still be inactive!", worker.isActive());
    	AssertJUnit.assertTrue("Current EmploymentPeriod start date mismatch!",
    			DateUtil.isSameDay(futureDay, worker.getEmploymentPeriod().getStartDate()));
    	AssertJUnit.assertNull("Current EmploymentPeriod end date is not null!", worker.getEmploymentPeriod().getEndDate());
    	AssertJUnit.assertEquals("Worker's employment status should still be TERMINATED!",
    			EmploymentStatusEnum.TERMINATED, worker.getEmploymentStatus());
    	Set<EmploymentPeriod> expectedPeriods = new HashSet<EmploymentPeriod>();
    	DateRange dateRange = new DateRange();
    	dateRange.setStartDate(WorkerTestUtils.HIRE_DATE);
    	dateRange.setEndDate(terminationDate);
    	EmploymentPeriod period = new EmploymentPeriod();
    	period.setEmploymentPeriod(dateRange);
    	expectedPeriods.add(period);
    	AssertJUnit.assertTrue("Historical EmploymentPeriod mismatch!", verifyEmploymentPeriods(expectedPeriods,
    			worker.getEmploymentPeriods()));
    }

    /**
     * <P>Test re-hiring a terminated worker multiple times</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Terminate the worker</LI>
     * <LI>Re-hire the worker</LI>
     * <LI>Terminate the worker</LI>
     * <LI>Re-hire the worker</LI>
     * <LI>Terminate the worker</LI>
     * <LI>Re-hire the worker</LI>
     * <LI>Terminate the worker</LI>
     * <LI>Re-hire the worker</LI>
      * <LI>Verify expected results</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.hireExistingWorker"})
    public void testRehireTerminatedWorkerMultipleTimes() throws Exception {
    	// Setup
		Worker worker = WorkerTestUtils.createAPayrollReadyWorker("Johnny", sMountainViewLocation);
		// Exercise
		Date terminationDate1 = DateUtil.createDate(2008, Calendar.MARCH, 31);
    	worker = sWorkerComponent.terminateWorker(worker, terminationDate1);
		Date hireDate2 = DateUtil.createDate(2008, Calendar.MAY, 1);
    	worker = sWorkerComponent.hireExistingWorker(worker.getId(), hireDate2);

		Date terminationDate2 = DateUtil.createDate(2008, Calendar.JUNE, 30);
    	worker = sWorkerComponent.terminateWorker(worker, terminationDate2);
		Date hireDate3 = DateUtil.createDate(2008, Calendar.AUGUST, 1);
    	worker = sWorkerComponent.hireExistingWorker(worker.getId(), hireDate3);

		Date terminationDate3 = DateUtil.createDate(2008, Calendar.SEPTEMBER, 30);
    	worker = sWorkerComponent.terminateWorker(worker, terminationDate3);
		Date hireDate4 = DateUtil.createDate(2008, Calendar.NOVEMBER, 1);
    	worker = sWorkerComponent.hireExistingWorker(worker.getId(), hireDate4);

		Date terminationDate4 = DateUtil.createDate(2008, Calendar.DECEMBER, 31);
    	worker = sWorkerComponent.terminateWorker(worker, terminationDate4);
		Date hireDate5 = DateUtil.createDate(2009, Calendar.FEBRUARY, 1);
    	worker = sWorkerComponent.hireExistingWorker(worker.getId(), hireDate5);
    	
    	// Verify
    	AssertJUnit.assertTrue("Worker should have been marked active!", worker.isActive());
    	AssertJUnit.assertTrue("Current EmploymentPeriod start date mismatch!",
    			DateUtil.isSameDay(hireDate5, worker.getEmploymentPeriod().getStartDate()));
    	AssertJUnit.assertNull("Current EmploymentPeriod end date is not null!", worker.getEmploymentPeriod().getEndDate());
    	AssertJUnit.assertEquals("Worker's employment status was not set to EMPLOYED!",
    			EmploymentStatusEnum.EMPLOYED, worker.getEmploymentStatus());
    	Set<EmploymentPeriod> expectedPeriods = new HashSet<EmploymentPeriod>();
    	EmploymentPeriod period1 = new EmploymentPeriod();
    	period1.setEmploymentPeriod(new DateRange(WorkerTestUtils.HIRE_DATE, terminationDate1));
    	expectedPeriods.add(period1);

    	EmploymentPeriod period2 = new EmploymentPeriod();
    	period2.setEmploymentPeriod(new DateRange(hireDate2, terminationDate2));
    	expectedPeriods.add(period2);

    	EmploymentPeriod period3 = new EmploymentPeriod();
    	period3.setEmploymentPeriod(new DateRange(hireDate3, terminationDate3));
    	expectedPeriods.add(period3);

    	EmploymentPeriod period4 = new EmploymentPeriod();
    	period4.setEmploymentPeriod(new DateRange(hireDate4, terminationDate4));
    	expectedPeriods.add(period4);

    	AssertJUnit.assertTrue("Historical EmploymentPeriod mismatch!", verifyEmploymentPeriods(expectedPeriods,
    			worker.getEmploymentPeriods()));
    }

}
