package com.intuit.sb.payroll.capability.worker;

import java.util.Date;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 *  <P>The <CODE>TerminateWorkerExceptionsTest</CODE> class contains tests for testing exceptions that can be thrown by
 *     terminateWorker() in WorkerComponent</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class TerminateWorkerExceptionsTest extends WorkerComponentTestBase
{

	/**
	 * One-time set up for this class.
	 */
    @BeforeClass
    public static void setUpTerminateWorkerExceptionsTestClass() {
    	sWorkerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);
        
    	try {
            getUserTransaction().begin();
            saveEmployerInfo();
            saveWorkLocationAddress();
            setUpSanMateoAddress();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in setUpTerminateWorkerExceptionsTestClass(): " + t.getMessage());
        }
    }
    
    /**
     * One-time tear down for this class.
     */
    @AfterClass
    public static void tearDownTerminateWorkerExceptionsTestClass() {
        try {
            getUserTransaction().begin();
            restoreWorkLocationAddress();
            restoreEmployerInfo();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in tearDownTerminateWorkerExceptionsTestClass(): " + t.getMessage());
        }
    }
    
    /**
     * <P>Test calling terminateWorker() with a null Worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerComponent.terminateWorker() with a null Worker</LI>
     * <LI>Verify that ValidationException is thrown</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.terminateWorker"}, expectedExceptions=ValidationException.class)
    public void testTerminateWorkerWithNullWorker() throws Exception {
    	@SuppressWarnings("unused")
    	Worker worker = sWorkerComponent.terminateWorker(null, DateUtil.getNowDate());
    }

    /**
     * <P>Test calling terminateWorker() with a null termination date</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerComponent.terminateWorker() with a null terminationDate</LI>
     * <LI>Verify that ValidationException is thrown</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.terminateWorker"}, expectedExceptions=ValidationException.class)
    public void testTerminateWorkerWithNullTerminationDate() throws Exception {
    	@SuppressWarnings("unused")
    	Worker worker = sWorkerComponent.terminateWorker(sWorker, null);
    }

    /**
     * <P>Test terminating a worker when the worker's EmploymentPeriod is null</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Set the EmploymentPeriod to null</LI>
     * <LI>Call WorkerComponent.terminateWorker()</LI>
     * <LI>Verify that ValidationException is thrown</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.terminateWorker"}, expectedExceptions=ValidationException.class)
    public void testTerminateWorkerWithNullEmploymentPeriod() throws Exception {
        // Create an active payroll ready worker
		Worker worker = WorkerTestUtils.createAPayrollReadyWorker("Johnny", sMountainViewLocation);
		Date terminationDate = DateUtil.getNowDate();
		// Set the EmploymentPeriod to null
		worker.setEmploymentPeriod(null);
    	worker = sWorkerComponent.terminateWorker(worker, terminationDate);
    }

    /**
     * <P>Test terminating a worker when the worker's hire date is after the termination date</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Set the hire date to a date 10 days after the termination date</LI>
     * <LI>Call WorkerComponent.terminateWorker()</LI>
     * <LI>Verify that ValidationException is thrown</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.terminateWorker"}, expectedExceptions=ValidationException.class)
    public void testTerminateWorkerBeforeHireDate() throws Exception {
        // Create an active payroll ready worker
		Worker worker = WorkerTestUtils.createAPayrollReadyWorker("Johnny", sMountainViewLocation);
		// Set the hire date to 10 days from the termination date
		Date terminationDate = DateUtil.getNowDate();
		worker.getEmploymentPeriod().setStartDate(DateUtil.getNextDate(terminationDate, 10));
    	worker = sWorkerComponent.terminateWorker(worker, terminationDate);
    }

    /**
     * <P>Test terminating a worker when the worker's hire date is not set</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Set the hire date to null</LI>
     * <LI>Call WorkerComponent.terminateWorker()</LI>
     * <LI>Verify that ValidationException is thrown</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.terminateWorker"}, expectedExceptions=ValidationException.class)
    public void testTerminateWorkerWithNullHireDate() throws Exception {
        // Create an active payroll ready worker
		Worker worker = WorkerTestUtils.createAPayrollReadyWorker("Johnny", sMountainViewLocation);
		Date terminationDate = DateUtil.getNowDate();
		// Set the hire date to null
		worker.getEmploymentPeriod().setStartDate(null);
    	worker = sWorkerComponent.terminateWorker(worker, terminationDate);
    }

    /**
     * <P>Test terminating a worker whom has already been terminated</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Terminate the worker</LI>
     * <LI>Call WorkerComponent.terminateWorker() to terminate the worker again</LI>
     * <LI>Verify that ValidationException is thrown</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.terminateWorker"}, expectedExceptions=ValidationException.class)
    public void testTerminateAlreadyTerminatedWorker() throws Exception {
        // Create an active payroll ready worker
		Worker worker = WorkerTestUtils.createAPayrollReadyWorker("Johnny", sMountainViewLocation);
		Date terminationDate = DateUtil.getNowDate();
    	worker = sWorkerComponent.terminateWorker(worker, terminationDate);
    	// Terminate the worker again
    	worker = sWorkerComponent.terminateWorker(worker, terminationDate);
    }

}
