package com.intuit.sb.payroll.capability.workerpayments;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

import javax.transaction.Status;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.RelationshipToOwnerEnum;
import com.intuit.sb.payroll.entity.model.Worker;



/**
 *  <P>The <CODE>CalculatePartialPeriodPaycheckTest</CODE> class contains tests for calculating partial paychecks using
 *   calculatePaycheck() in WorkerPaymentsComponent</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class CalculatePartialPeriodPaycheckTest extends WorkerPaymentsComponentTestBase {
    private static WorkerPaymentsComponent sWorkerPaymentsComponent;
	private static WorkerComponent sWorkerComponent;

    @BeforeClass
    public static void setUpCalculatePartialPeriodPaycheckTest() {
    	sWorkerPaymentsComponent = 
            qaBootstrap.getEjbFromClass(WorkerPaymentsComponent.class);
    	sWorkerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);
        
    	try {
            getUserTransaction().begin();
            saveEmployerInfo();
            saveWorkLocationAddress();
            setUpSanMateoAddress();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in setUpCalculatePartialPeriodPaycheckTest(): " + t.getMessage());
        }
    }
    
    @AfterClass
    public static void tearDownCalculatePartialPeriodPaycheckTest() {
        try {
            getUserTransaction().begin();
            restoreWorkLocationAddress();
            restoreEmployerInfo();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in tearDownCalculatePartialPeriodPaycheckTest(): " + t.getMessage());
        }
    }
    
    @BeforeMethod
    public void setUp() throws Exception {
        super.setUp();
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
        sEmployer.setNonProfit(false);
        setUpTestData(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED);
    }
    
    @AfterMethod
    public void tearDown() throws Exception
    {
        if (getUserTransaction().getStatus() == Status.STATUS_ACTIVE) {
            // Remove withholdingInfo associated with the employee
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.W4IRS);
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.DE4);
        }
        super.tearDown();
    }
    
    /**
     * <P>Test biweekly paycheck with 1 hourly wage, worked for 4 days in pay period and all applicable taxes for 
     * California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 80 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Set the pay period to 4 work days</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Hourly_WithholdSingle1_PaidBiweekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Frank", sMountainViewLocation);
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                                                               new BigDecimal("15.00"), 32.0);
    	worker.addCompensationDtl(hourlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        // sPeriodStartDate should be 1/1/2008, we want 4 work days so set the endDate to Friday 1/4/2008
        Date endDate = DateUtil.getNextDate(sPeriodStartDate.getTime(), 3);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), endDate);
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        paycheckCalculationRequest.setPartialPayPeriod(true);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("480.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("480.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("24.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("29.76"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("6.96"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.65"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("3.84"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("3.84"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("29.76"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("6.96"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("16.32"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.48"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 salary wage, Worked 2 days in pay period and all applicable taxes for
     * California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000.00/year</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Set the pay period to 2 work days</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_WithholdSingle1_PaidBiweekly_2WorkDays() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Frank", sMountainViewLocation);
    	worker = WorkerTestUtils.setupSalaryWage(worker);
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        // PeriodEndDate should be 1/14/2008, we want 2 work days so set the endDate to Wednesday 1/2/2008
        Date endDate = DateUtil.getNextDate(sPeriodStartDate.getTime(), 1);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), endDate);
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        paycheckCalculationRequest.setPartialPayPeriod(true);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("507.69");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, amount),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("27.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("31.48"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("7.36"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.21"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.06"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.06"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("31.48"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("7.36"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("17.26"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.51"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 salary wage, Worked 9 days in pay period and all applicable taxes for
     * California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000.00/year</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Set the pay period to 9 work days</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_WithholdSingle1_PaidBiweekly_9WorkDays() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Frank", sMountainViewLocation);
    	worker = WorkerTestUtils.setupSalaryWage(worker);
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        // sPeriodStartDate should be 1/1/2008, we want 9 work days so set the endDate to Friday 1/11/2008
        Date endDate = DateUtil.getNextDate(sPeriodStartDate.getTime(), 10);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), endDate);
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        paycheckCalculationRequest.setPartialPayPeriod(true);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("2284.62");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, amount),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("377.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("141.65"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("33.13"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("111.87"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("18.28"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("18.28"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("141.65"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("33.13"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("77.68"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("2.28"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with 1 salary wage, Worked 4 days in pay period and all applicable taxes for
     * California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a weekly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000.00/year</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Set the pay period to 4 work days</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_WithholdSingle1_PaidWeekly_4WorkDays() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Frank", sMountainViewLocation);
    	worker = WorkerTestUtils.setupSalaryWage(worker);
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        setupPaycheckDates(frequency);
        // sPeriodStartDate should be 1/01/2008, we want 4 work days so set the endDate to Wednesday 1/04/2008
        Date endDate = DateUtil.getNextDate(sPeriodStartDate.getTime(), 3);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), endDate);
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        paycheckCalculationRequest.setPartialPayPeriod(true);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("1015.38");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, amount),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("157.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("62.95"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("14.72"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("44.13"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("8.12"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("8.12"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("62.95"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("14.72"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("34.52"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.02"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test monthly paycheck with 1 salary wage, worked 12 days in pay period and all applicable taxes for
     * California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a monthly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000.00/year</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Set the pay period to 12 work days</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_WithholdSingle1_PaidMonthly_12WorkDays() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Frank", sMountainViewLocation);
    	worker = WorkerTestUtils.setupSalaryWage(worker);
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.MONTHLY;
        setupPaycheckDates(frequency);
        // PeriodEndDate should be 1/31/2008, we want 12 work days so set the endDate to Wednesday 1/16/2008
        Date endDate = DateUtil.getNextDate(sPeriodStartDate.getTime(), 15);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), endDate);
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        paycheckCalculationRequest.setPartialPayPeriod(true);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("3046.15");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, amount),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("348.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("188.86"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("44.17"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("82.12"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("24.37"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("24.37"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("188.86"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("44.17"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("103.57"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("3.05"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

}
