package com.intuit.sb.payroll.capability.workerpayments;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

import javax.transaction.Status;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.DatumType2Enum;
import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.RelationshipToOwnerEnum;
import com.intuit.sb.payroll.entity.model.Worker;



/**
 *  <P>The <CODE>CalculatePaycheckTest</CODE> class contains tests for testing the calculatePaycheck() in WorkerPaymentsComponent</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class CalculatePaycheckTest extends WorkerPaymentsComponentTestBase {
    private static WorkerPaymentsComponent sWorkerPaymentsComponent;
	private static WorkerComponent sWorkerComponent;

    @BeforeClass
    public static void setUpCalculatePaycheckTest() {
    	sWorkerPaymentsComponent = 
            qaBootstrap.getEjbFromClass(WorkerPaymentsComponent.class);
    	sWorkerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);
        
    	try {
            getUserTransaction().begin();
            saveEmployerInfo();
            saveWorkLocationAddress();
            setUpSanMateoAddress();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in setUpCalculatePaycheckTest(): " + t.getMessage());
        }
    }
    
    @AfterClass
    public static void tearDownCalculatePaycheckTest() {
        try {
            getUserTransaction().begin();
            restoreWorkLocationAddress();
            restoreEmployerInfo();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in tearDownCalculatePaycheckTest(): " + t.getMessage());
        }
    }
    
    @BeforeMethod
    public void setUp() throws Exception {
        super.setUp();
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
        sEmployer.setNonProfit(false);
        setUpTestData(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED);
    }
    
    @AfterMethod
    public void tearDown() throws Exception
    {
        if (getUserTransaction().getStatus() == Status.STATUS_ACTIVE) {
            // Remove withholdingInfo associated with the employee
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.W4IRS);
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.DE4);
        }
        super.tearDown();
    }
    
    /**
     * <P>Test biweekly paycheck with 1 hourly wage and all applicable taxes for California 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 80 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Hourly_WithholdSingle1_PaidBiweekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Frank", sMountainViewLocation);
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                                                               new BigDecimal("15.00"), 80.0);
    	worker.addCompensationDtl(hourlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("1200.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("1200.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("130.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("74.40"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("17.40"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("25.55"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.60"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.60"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("74.40"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("17.40"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("40.80"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.20"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 hourly wage, 1 fixed commission and all applicable taxes for California 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 80 hours</LI>
     * <LI>Create a commission item with an amount of $500.00</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Hourly_1FixedCommission_WithholdSingle1_PaidBiweekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Frank", sMountainViewLocation);
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                                                               new BigDecimal("15.00"), 80.0);
        CompensationDtl commissionWage = createAmountBasedWageItemDtl(CompensationTypeEnum.COMMISSION, 
                                                               new BigDecimal("500.00"), DatumType2Enum.PERPAYCHECK);
        commissionWage.setFixedRate(true);
        worker.addCompensationDtl(commissionWage);
    	worker.addCompensationDtl(hourlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("1700.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("1200.00")),
            new LineItemAmount(QboeLawIdConstants.sCommissionLawId, new BigDecimal("500.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("231.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("105.40"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("24.65"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("59.56"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("13.60"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("13.60"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("105.40"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("24.65"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("57.80"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.70"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 hourly wage, 1 fixed $0.00 commission and all applicable taxes for California 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 80 hours</LI>
     * <LI>Create a commission item with an amount of $500.00</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Hourly_1FixedZeroCommission_WithholdSingle1_PaidBiweekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Frank", sMountainViewLocation);
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                                                               new BigDecimal("15.00"), 80.0);
        CompensationDtl commissionWage = createAmountBasedWageItemDtl(CompensationTypeEnum.COMMISSION, 
                                                               new BigDecimal("0.00"), DatumType2Enum.PERPAYCHECK);
        commissionWage.setFixedRate(true);
        worker.addCompensationDtl(commissionWage);
    	worker.addCompensationDtl(hourlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("1200.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("1200.00")),
            new LineItemAmount(QboeLawIdConstants.sCommissionLawId, new BigDecimal("0.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("130.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("74.40"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("17.40"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("25.55"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.60"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.60"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("74.40"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("17.40"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("40.80"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.20"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }
    
    /**
     * <P>Test biweekly paycheck with 1 hourly wage, 1 fixed commission and all applicable taxes for California 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 80 hours</LI>
     * <LI>Create a variable commission item with a one-time amount of $500.00</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Hourly_1VariableCommission_WithholdSingle1_PaidBiweekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Frank", sMountainViewLocation);
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                                                               new BigDecimal("15.00"), 80.0);
        CompensationDtl commissionWage = createAmountBasedWageItemDtl(CompensationTypeEnum.COMMISSION, 
                                                               new BigDecimal("0.00"), DatumType2Enum.PERPAYCHECK);
        commissionWage.setFixedRate(false);
        worker.addCompensationDtl(commissionWage);
    	worker.addCompensationDtl(hourlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        java.util.Set<CompensationDtl> compensationSet = worker.getCompensationDtls();
        for (CompensationDtl compensationDtl : compensationSet) {
        	if (compensationDtl.getCompensationType().equals(CompensationTypeEnum.COMMISSION) && 
        			compensationDtl.isFixedRate() == false) {
                compensationDtl.getRate().setValue1AsUSDMoneyAmount(new BigDecimal("500.00"));
        	}
        }
               
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(compensationSet);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("1700.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("1200.00")),
            new LineItemAmount(QboeLawIdConstants.sCommissionLawId, new BigDecimal("500.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("231.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("105.40"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("24.65"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("59.56"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("13.60"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("13.60"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("105.40"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("24.65"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("57.80"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.70"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    
    /**
     * <P>Test biweekly paycheck with 1 hourly wage, a fixed Bonus and all applicable taxes for California 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create a hourly wage item with rate of $15.00/hour</LI>
     * <LI>Create a one time fixed bonus of $1000.00</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Create a PayrollRun with the dates for a weekly payroll</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly_1FixedBonus_WithholdCASingle1_PaidBiWeekly() throws Exception {    	
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Frank", sMountainViewLocation);
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG,                                                          
                                                               new BigDecimal("15.00"), 80.0);
    	worker.addCompensationDtl(hourlyDtl);
    	
    	CompensationDtl bonusWage = createAmountBasedWageItemDtl(CompensationTypeEnum.BONUS, 
    															new BigDecimal("1000.00"), 
    															DatumType2Enum.PERPAYCHECK);
        //The magic setting.
        bonusWage.setFixedRate(true);
        worker.addCompensationDtl(bonusWage);
    	
        worker = sWorkerComponent.updateWorker(worker);
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
    	
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("2200.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("1200.00")),
        	new LineItemAmount(QboeLawIdConstants.sBonusLawId, new BigDecimal("1000.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("356.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("136.40"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("31.90"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("104.00"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("17.60"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("17.60"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("136.40"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("31.90"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("74.80"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("2.20"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }
       

    /**
     * <P>Test biweekly paycheck with 1 hourly wage, a variable bonus and all applicable taxes for California 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with rate $15.00/hour</LI>
     * <LI>Create a variable bonus payroll item</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Create a PayrollRun with the dates for a weekly payroll</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly_1VariableBonus_WithholdCASingle1_PaidBiWeekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Frank", sMountainViewLocation);
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG,                                                          
                                                               new BigDecimal("15.00"), 80.0);
    	worker.addCompensationDtl(hourlyDtl);
    	
    	CompensationDtl bonusWage = createAmountBasedWageItemDtl(CompensationTypeEnum.BONUS, 
        		new BigDecimal(0), DatumType2Enum.PERPAYCHECK);
        bonusWage.setFixedRate(false);
        worker.addCompensationDtl(bonusWage);
    	
        worker = sWorkerComponent.updateWorker(worker);
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        java.util.Set<CompensationDtl> compensationSet = worker.getCompensationDtls();
        for (CompensationDtl compensationDtl : compensationSet) {
        	// Add the variable bonus to the appropriate compensationDtl
        	if (compensationDtl.getCompensationType().equals(CompensationTypeEnum.BONUS) && 
        			compensationDtl.isFixedRate() == false) {
                compensationDtl.getRate().setValue1AsUSDMoneyAmount(new BigDecimal("1000.00"));
        	}
        }
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(compensationSet);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("2200.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("1200.00")),
        	new LineItemAmount(QboeLawIdConstants.sBonusLawId, new BigDecimal("1000.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("356.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("136.40"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("31.90"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("104.00"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("17.60"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("17.60"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("136.40"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("31.90"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("74.80"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("2.20"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));       
    }
    
    /**
     * <P>Test weekly paycheck with 1 hourly wage, 1 piecework and all applicable taxes for California using withholding status of CA Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a Weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 40 hours</LI>
     * <LI>Create a piecework wage item with a rate of 1.00 per unit and 300 units</LI>
     * <LI>Use default withholding of Single 1 for both Federal and CA taxes</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Create a PayrollRun with the dates for a weekly payroll</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly_1Piecework_WithholdCASingle1_PaidWeekly() throws Exception{
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Frank", sMountainViewLocation);
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG,                                                          
                                                               new BigDecimal("15.00"), 40.0);
    	worker.addCompensationDtl(hourlyDtl);
        
        CompensationDtl pieceworkWage = createPieceworkWageItemDtl(new BigDecimal("1.0"), 1.0, 300.0);
        worker.addCompensationDtl(pieceworkWage);
    	
        worker = sWorkerComponent.updateWorker(worker);
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
    	
        BigDecimal amount = new BigDecimal("900.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("600.00")),
            new LineItemAmount(QboeLawIdConstants.sPieceworkLawId, new BigDecimal("300.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("128.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("55.80"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("13.05"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("33.78"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("7.20"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("7.20"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("55.80"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("13.05"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("30.60"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.90"), amount, amount)};
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 salary wage, hired for 2 work days in pay period and all applicable taxes for
     * California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000.00/year</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Set the employment period to 2 work days in the pay period</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_WithholdSingle1_PaidBiweekly_2WorkDays() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Frank", sMountainViewLocation);
    	worker = WorkerTestUtils.setupSalaryWage(worker);
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        // PeriodEndDate should be 1/14/2008, we want 2 work days so set the startDate to Friday 1/11/2008
        Date startDate = DateUtil.getNextDate(mPeriodEndDate.getTime(), -3);
        @SuppressWarnings("unused")
        DateRange origEmploymentPeriod = WorkerTestUtils.setUpEmploymentPeriod(worker, startDate, null);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("507.69");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, amount),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("27.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("31.48"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("7.36"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.21"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.06"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.06"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("31.48"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("7.36"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("17.26"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.51"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 salary wage, hired for 9 work days in pay period and all applicable taxes for
     * California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000.00/year</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Set the employment period to 9 work days in the pay period</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_WithholdSingle1_PaidBiweekly_9WorkDays() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Frank", sMountainViewLocation);
    	worker = WorkerTestUtils.setupSalaryWage(worker);
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        // sPeriodStartDate should be 1/1/2008, we want 9 work days so set the startDate to Wednesday 1/2/2008
        Date startDate = DateUtil.getNextDate(sPeriodStartDate.getTime(), 1);
        @SuppressWarnings("unused")
        DateRange origEmploymentPeriod = WorkerTestUtils.setUpEmploymentPeriod(worker, startDate, null);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("2284.62");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, amount),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("377.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("141.65"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("33.13"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("111.87"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("18.28"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("18.28"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("141.65"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("33.13"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("77.68"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("2.28"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 salary wage, hired for all work days in pay period and all applicable taxes for
     * California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000.00/year</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Set the employment period to all work days in the pay period</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_WithholdSingle1_PaidBiweekly_AllWorkDays() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Frank", sMountainViewLocation);
    	worker = WorkerTestUtils.setupSalaryWage(worker);
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        @SuppressWarnings("unused")
        DateRange origEmploymentPeriod = WorkerTestUtils.setUpEmploymentPeriod(worker, sPeriodStartDate.getTime(), null);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("2538.46");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, amount),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("440.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("135.48"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.31"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.31"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("86.31"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("2.54"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with 1 salary wage, hired for 2 work days in pay period and all applicable taxes for
     * California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a weekly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000.00/year</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Set the employment period to 2 work days in the pay period</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_WithholdSingle1_PaidWeekly_2WorkDays() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Frank", sMountainViewLocation);
    	worker = WorkerTestUtils.setupSalaryWage(worker);
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        setupPaycheckDates(frequency);
        // PeriodEndDate should be 1/07/2008, we want 2 work days so set the startDate to Friday 1/04/2008
        Date startDate = DateUtil.getNextDate(mPeriodEndDate.getTime(), -3);
        @SuppressWarnings("unused")
        DateRange origEmploymentPeriod = WorkerTestUtils.setUpEmploymentPeriod(worker, startDate, null);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("507.69");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, amount),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("51.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("31.48"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("7.36"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("8.26"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.06"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.06"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("31.48"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("7.36"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("17.26"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.51"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with 1 salary wage, hired for 4 work days in pay period and all applicable taxes for
     * California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a weekly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000.00/year</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Set the employment period to 4 work days in the pay period</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_WithholdSingle1_PaidWeekly_4WorkDays() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Frank", sMountainViewLocation);
    	worker = WorkerTestUtils.setupSalaryWage(worker);
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        setupPaycheckDates(frequency);
        // sPeriodStartDate should be 1/01/2008, we want 4 work days so set the startDate to Wednesday 1/02/2008
        Date startDate = DateUtil.getNextDate(sPeriodStartDate.getTime(), 1);
        @SuppressWarnings("unused")
        DateRange origEmploymentPeriod = WorkerTestUtils.setUpEmploymentPeriod(worker, startDate, null);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("1015.38");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, amount),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("157.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("62.95"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("14.72"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("44.13"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("8.12"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("8.12"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("62.95"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("14.72"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("34.52"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.02"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with 1 salary wage, hired for all work days in pay period and all applicable taxes for
     * California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a weekly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000.00/year</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Set the employment period to all work days in the pay period</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_WithholdSingle1_PaidWeekly_AllWorkDays() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Frank", sMountainViewLocation);
    	worker = WorkerTestUtils.setupSalaryWage(worker);
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        setupPaycheckDates(frequency);
        @SuppressWarnings("unused")
        DateRange origEmploymentPeriod = WorkerTestUtils.setUpEmploymentPeriod(worker, sPeriodStartDate.getTime(), null);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("1269.23");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, amount),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("220.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("78.69"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("18.40"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("67.74"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("10.15"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("10.15"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("78.69"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("18.40"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("43.15"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.27"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test monthly paycheck with 1 salary wage, hired for 2 work days in pay period and all applicable taxes for
     * California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a monthly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000.00/year</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Set the employment period to 2 work days in the pay period</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_WithholdSingle1_PaidMonthly_2WorkDays() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Frank", sMountainViewLocation);
    	worker = WorkerTestUtils.setupSalaryWage(worker);
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.MONTHLY;
        setupPaycheckDates(frequency);
        // PeriodEndDate should be 1/31/2008, we want 2 work days so set the startDate to Wednesday 1/30/2008
        Date startDate = DateUtil.getNextDate(mPeriodEndDate.getTime(), -1);
        @SuppressWarnings("unused")
        DateRange origEmploymentPeriod = WorkerTestUtils.setUpEmploymentPeriod(worker, startDate, null);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("507.69");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, amount),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("31.48"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("7.36"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.06"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.06"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("31.48"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("7.36"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("17.26"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.51"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test monthly paycheck with 1 salary wage, hired for 12 work days in pay period and all applicable taxes for
     * California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a monthly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000.00/year</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Set the employment period to 12 work days in the pay period</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_WithholdSingle1_PaidMonthly_12WorkDays() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Frank", sMountainViewLocation);
    	worker = WorkerTestUtils.setupSalaryWage(worker);
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.MONTHLY;
        setupPaycheckDates(frequency);
        // PeriodEndDate should be 1/31/2008, we want 2 work days so set the startDate to Wednesday 1/16/2008
        Date startDate = DateUtil.getNextDate(mPeriodEndDate.getTime(), -15);
        @SuppressWarnings("unused")
        DateRange origEmploymentPeriod = WorkerTestUtils.setUpEmploymentPeriod(worker, startDate, null);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("3046.15");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, amount),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("348.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("188.86"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("44.17"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("82.12"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("24.37"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("24.37"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("188.86"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("44.17"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("103.57"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("3.05"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test monthly paycheck with 1 salary wage, hired for all work days in pay period and all applicable taxes for
     * California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a monthly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000.00/year</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Set the employment period to all work days in the pay period</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_WithholdSingle1_PaidMonthly_AllWorkDays() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Frank", sMountainViewLocation);
    	worker = WorkerTestUtils.setupSalaryWage(worker);
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.MONTHLY;
        setupPaycheckDates(frequency);
        @SuppressWarnings("unused")
        DateRange origEmploymentPeriod = WorkerTestUtils.setUpEmploymentPeriod(worker, sPeriodStartDate.getTime(), null);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("5500.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, amount),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("954.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("341.00"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("79.75"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("293.53"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("44.00"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("44.00"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("341.00"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("79.75"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("187.00"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("5.50"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test monthly paycheck with 1 salary wage, hired for all work days in pay period and all applicable taxes for
     * California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a monthly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000.00/year</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Set the employment period to all work days in the pay period (hireDate is day before pay period start date)</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_WithholdSingle1_PaidMonthly_AllWorkDays_2() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Frank", sMountainViewLocation);
    	worker = WorkerTestUtils.setupSalaryWage(worker);
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.MONTHLY;
        setupPaycheckDates(frequency);
        // PeriodEndDate should be 1/31/2008, we want 1 day before the startDate so set it to Wednesday 12/31/2007
        Date startDate = DateUtil.getNextDate(sPeriodStartDate.getTime(), -1);
        @SuppressWarnings("unused")
        DateRange origEmploymentPeriod = WorkerTestUtils.setUpEmploymentPeriod(worker, startDate, null);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("5500.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, amount),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("954.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("341.00"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("79.75"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("293.53"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("44.00"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("44.00"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("341.00"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("79.75"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("187.00"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("5.50"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

}
