package com.intuit.sb.payroll.capability.workerpayments;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import org.apache.commons.lang.exception.ExceptionUtils;
import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.PayrollDistributionMethodEnum;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollRunWorkflowStatusEnum;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * <P>
 * The <CODE>CommitPayrollRunTest</CODE> class contains tests for testing the
 * commitPayrollRun() in WorkerPaymentsComponent
 * </P>
 * 
 * $Author$ $Revision$ $DateTime$
 * 
 */
public class CommitPayrollRunTest extends WorkerPaymentsComponentTestBase {
	private static Long sReadyForPayWorkerId = 0L;
	private static Long sReadyForPayWorkerId2 = 0L;
	private BigDecimal mHourlyRate = new BigDecimal("10.0");
	private BigDecimal mBonusAmount = new BigDecimal("200.0");

	@BeforeClass
	public static void setUpCommitPayrollRunTest() {
		sLogger.info("Setting Up Commit PayrollRunTest");
		try {
			getUserTransaction().begin();
			saveEmployerInfo();
			saveWorkLocationAddress();
			setUpSanMateoAddress();
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in setUpCommitPayrollRunTest(): " + t.getMessage());
		}
	}

	@AfterClass
	public static void tearDownCommitPayrollRunTest() {
		try {
			sLogger.info("Tearing Down Commit PayrollRunTest");
			getUserTransaction().begin();
			restoreWorkLocationAddress();
			restoreEmployerInfo();
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in tearDownCommitPayrollRunTest(): " + t.getMessage());
		}
	}
	
	@BeforeMethod
	public void initialize() throws Exception {
		sReadyForPayWorkerId = WorkerTestUtils.createPayrollReadyWorker("Johnny", sMountainViewLocation);
		sReadyForPayWorkerId2 = WorkerTestUtils.createPayrollReadyWorker("Paul", sMountainViewLocation);
	}
	
	@AfterMethod
	public void cleanUp() throws Exception {
		getUserTransaction().begin();
		if (0 != sReadyForPayWorkerId) {
			Worker worker = entityService.findById(Worker.class, sReadyForPayWorkerId);
			WorkerTestUtils.cleanUpWorker(worker);
			sReadyForPayWorkerId = 0L;
		}
		if (0 != sReadyForPayWorkerId2) {
			Worker worker = entityService.findById(Worker.class, sReadyForPayWorkerId2);
			WorkerTestUtils.cleanUpWorker(worker);
			sReadyForPayWorkerId2 = 0L;
		}
		getUserTransaction().commit();
		getUserTransaction().begin();
	}

    /**
     * <P>Test commitPayrollRun() API with 1 hourly worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollRunInfo with a biweekly pay frequency using default payroll run values</LI>
     * <LI>Setup a PaycheckCalculationRequest object with some compensation values</LI>
     * <LI>Setup a PayrollRunRequest object</LI>
     * <LI>Call WorkerPaymentsComponent.commitPayrollRun() with a valid PayrollRunRequest</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the payroll run state is committed and the calculated values are correct</LI>
     * </UL>
     */
	@Test
	public void testCommitPayrollRunWith1HourlyWorker() throws Exception {
    	PayrollRun payrollRun = null;
    	try {
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.JANUARY, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
        	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
        	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
        					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
        					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
        	paycheckCalculationRequests.add(calcRequest);
        	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
            payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
            payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);
            getUserTransaction().commit();

            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
        	//Exercise
        	payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();

        	//Verify
        	AssertJUnit.assertTrue(validatePayrollRun(payrollRun, payrollRunRequest));
        	AssertJUnit.assertEquals("Number of paycheck(s) mismatch!", 1, payrollRun.getPaychecks().size());
        	AssertJUnit.assertEquals("workflowStatus mismatch!", PayrollRunWorkflowStatusEnum.COMMITTED,
        					payrollRun.getWorkflowStatus());
        	AssertJUnit.assertEquals("totalNetPay mismatch!", new Money("USD", "908.27"), payrollRun.getTotalNetPay());
        	AssertJUnit.assertEquals("totalWorkerPdTaxes mismatch!", new Money("USD", "241.72"),
        					payrollRun.getTotalWorkerPdTaxes());
        	AssertJUnit.assertEquals("totalEmployerPdTaxes mismatch!", new Money("USD", "137.43"),
        					payrollRun.getTotalEmployerPdTaxes());
        	AssertJUnit.assertEquals("totalWorkerPdBenefits mismatch!", new Money("USD", "0.00"),
        					payrollRun.getTotalWorkerPdBenefits());
        	AssertJUnit.assertEquals("totalEmployerPdBenefits mismatch!", new Money("USD", "0.00"),
        					payrollRun.getTotalEmployerPdBenefits());
    	} catch (Exception e) {
    		AssertJUnit.fail("Unexpected exception: " + e.getMessage());
    	} finally {
    		getUserTransaction().begin();
        	//Tear down
    		if (payrollRun != null) {
            	Long payrollRunId = payrollRun.getId();
            	cleanUpPayrollRun(payrollRunId);
    		}
    		getUserTransaction().commit();
    	}
	}

    /**
     * <P>Test commitPayrollRun() API with 2 hourly workers</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollRunInfo with a biweekly pay frequency using default payroll run values</LI>
     * <LI>Setup a PaycheckCalculationRequest object with some compensation values for worker 1</LI>
     * <LI>Setup a PaycheckCalculationRequest object with some compensation values for worker 2</LI>
     * <LI>Setup a PayrollRunRequest object</LI>
     * <LI>Call WorkerPaymentsComponent.commitPayrollRun() with a valid PayrollRunRequest</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the payroll run state is committed and the calculated values are correct</LI>
     * </UL>
     */
	@Test
	public void testCommitPayrollRunWith2HourlyWorkers() throws Exception {
    	PayrollRun payrollRun = null;
    	try {
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.JANUARY, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
        	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
        	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
        					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
        					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
        	paycheckCalculationRequests.add(calcRequest);
        	PaycheckCalculationRequest calcRequest2 = createPaycheckCalculationRequest(sReadyForPayWorkerId2,
        					payrollRunInfo, mHourlyRate, new Double(80), new Double(5), mBonusAmount,
        					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
        	paycheckCalculationRequests.add(calcRequest2);
        	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
            payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
            payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);
            getUserTransaction().commit();

            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
        	//Exercise
        	payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();

        	//Verify
        	AssertJUnit.assertTrue(validatePayrollRun(payrollRun, payrollRunRequest));
        	AssertJUnit.assertEquals("Number of paycheck(s) mismatch!", 2, payrollRun.getPaychecks().size());
        	AssertJUnit.assertEquals("workflowStatus mismatch!", PayrollRunWorkflowStatusEnum.COMMITTED,
        					payrollRun.getWorkflowStatus());
        	AssertJUnit.assertEquals("totalNetPay mismatch!", new Money("USD", "1762.53"), payrollRun.getTotalNetPay());
        	AssertJUnit.assertEquals("totalWorkerPdTaxes mismatch!", new Money("USD", "462.46"),
        					payrollRun.getTotalWorkerPdTaxes());
        	AssertJUnit.assertEquals("totalEmployerPdTaxes mismatch!", new Money("USD", "265.90"),
        					payrollRun.getTotalEmployerPdTaxes());
        	AssertJUnit.assertEquals("totalWorkerPdBenefits mismatch!", new Money("USD", "0.00"),
        					payrollRun.getTotalWorkerPdBenefits());
        	AssertJUnit.assertEquals("totalEmployerPdBenefits mismatch!", new Money("USD", "0.00"),
        					payrollRun.getTotalEmployerPdBenefits());
    	} catch (Exception e) {
    		e.printStackTrace();
    		AssertJUnit.fail("Unexpected exception: " + e.getMessage());
    	} finally {
    		getUserTransaction().begin();
        	//Tear down
    		if (payrollRun != null) {
            	Long payrollRunId = payrollRun.getId();
            	cleanUpPayrollRun(payrollRunId);
    		}
    		getUserTransaction().commit();
    	}
	}

    /**
     * <P>Test commitPayrollRun() API with 1 hourly worker whom has section 125 Health benefit</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Add the section 125 health benefit to the worker</LI>
     * <LI>Setup a PayrollRunInfo with a biweekly pay frequency using default payroll run values</LI>
     * <LI>Setup a PaycheckCalculationRequest object with some compensation values</LI>
     * <LI>Add the section 125 health benefit to the worker</LI>
     * <LI>Setup a PayrollRunRequest object</LI>
     * <LI>Call WorkerPaymentsComponent.commitPayrollRun() with a valid PayrollRunRequest</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the payroll run state is committed and the calculated values are correct</LI>
     * </UL>
     */
	@Test
	public void testCommitPayrollRunWith1HourlyWorkerAndHealthBenefit() throws Exception {
    	PayrollRun payrollRun = null;
    	Long workerId = null;
    	try {
        	//Setup
    		workerId = WorkerTestUtils.createPayrollReadyWorker("Ben", sMountainViewLocation);
        	Worker worker = entityService.findById(Worker.class, workerId);
        	worker = WorkerTestUtils.addHealthBenefit(worker);
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.JANUARY, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
        	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
        	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(workerId,
        					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
        					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
        	paycheckCalculationRequests.add(calcRequest);
        	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
            payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
            payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);
            getUserTransaction().commit();

            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
        	//Exercise
        	payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();

        	//Verify
        	AssertJUnit.assertTrue(validatePayrollRun(payrollRun, payrollRunRequest));
        	AssertJUnit.assertEquals("Number of paycheck(s) mismatch!", 1, payrollRun.getPaychecks().size());
        	AssertJUnit.assertEquals("workflowStatus mismatch!", PayrollRunWorkflowStatusEnum.COMMITTED,
        					payrollRun.getWorkflowStatus());
        	AssertJUnit.assertEquals("totalNetPay mismatch!", new Money("USD", "861.02"), payrollRun.getTotalNetPay());
        	AssertJUnit.assertEquals("totalWorkerPdTaxes mismatch!", new Money("USD", "223.98"),
        					payrollRun.getTotalWorkerPdTaxes());
        	AssertJUnit.assertEquals("totalEmployerPdTaxes mismatch!", new Money("USD", "129.67"),
        					payrollRun.getTotalEmployerPdTaxes());
        	AssertJUnit.assertEquals("totalWorkerPdBenefits mismatch!", new Money("USD", "65.00"),
        					payrollRun.getTotalWorkerPdBenefits());
        	AssertJUnit.assertEquals("totalEmployerPdBenefits mismatch!", new Money("USD", "0.00"),
        					payrollRun.getTotalEmployerPdBenefits());
    	} catch (Exception e) {
    		AssertJUnit.fail("Unexpected exception: " + e.getMessage());
    	} finally {
    		getUserTransaction().begin();
        	//Tear down
    		if (payrollRun != null) {
            	Long payrollRunId = payrollRun.getId();
            	cleanUpPayrollRun(payrollRunId);
    		}
    		if (null != workerId) {
    			Worker worker = entityService.findById(Worker.class, workerId);
    			WorkerTestUtils.cleanUpWorker(worker);
    		}
    		getUserTransaction().commit();
    	}
	}

    /**
     * <P>Test commitPayrollRun() API with 1 salary worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a worker with a salary wage</LI>
     * <LI>Setup a PayrollRunInfo with a biweekly pay frequency using default payroll run values</LI>
     * <LI>Setup a PaycheckCalculationRequest object with some compensation values</LI>
     * <LI>Setup a PayrollRunRequest object</LI>
     * <LI>Call WorkerPaymentsComponent.commitPayrollRun() with a valid PayrollRunRequest</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the payroll run state is committed and the calculated values are correct</LI>
     * </UL>
     */
	@Test
	public void testCommitPayrollRunWith1SalaryWorker() throws Exception {
    	PayrollRun payrollRun = null;
    	Long salaryWorkerId = null;
    	try {
        	//Setup
			salaryWorkerId = WorkerTestUtils.createSalaryPayrollReadyWorker("Tim", sMountainViewLocation);
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.JANUARY, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
        	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
        	PaycheckCalculationRequest calcRequest = createSalaryPaycheckCalculationRequest(salaryWorkerId,
        					payrollRunInfo, PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
        	paycheckCalculationRequests.add(calcRequest);
        	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
            payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
            payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);
            getUserTransaction().commit();

            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
        	//Exercise
        	payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();

        	//Verify
        	AssertJUnit.assertTrue(validatePayrollRun(payrollRun, payrollRunRequest));
        	AssertJUnit.assertEquals("Number of paycheck(s) mismatch!", 1, payrollRun.getPaychecks().size());
        	AssertJUnit.assertEquals("workflowStatus mismatch!", PayrollRunWorkflowStatusEnum.COMMITTED,
        					payrollRun.getWorkflowStatus());
        	AssertJUnit.assertEquals("totalNetPay mismatch!", new Money("USD", "1748.48"), payrollRun.getTotalNetPay());
        	AssertJUnit.assertEquals("totalWorkerPdTaxes mismatch!", new Money("USD", "789.98"),
        					payrollRun.getTotalWorkerPdTaxes());
        	AssertJUnit.assertEquals("totalEmployerPdTaxes mismatch!", new Money("USD", "303.36"),
        					payrollRun.getTotalEmployerPdTaxes());
        	AssertJUnit.assertEquals("totalWorkerPdBenefits mismatch!", new Money("USD", "0.00"),
        					payrollRun.getTotalWorkerPdBenefits());
        	AssertJUnit.assertEquals("totalEmployerPdBenefits mismatch!", new Money("USD", "0.00"),
        					payrollRun.getTotalEmployerPdBenefits());
    	} catch (Exception e) {
    		e.printStackTrace();
    		ExceptionUtils.printRootCauseStackTrace(e);
    	} finally {
    		getUserTransaction().begin();
        	//Tear down
    		if (null != payrollRun) {
            	Long payrollRunId = payrollRun.getId();
            	cleanUpPayrollRun(payrollRunId);
    		}
    		if (null != salaryWorkerId) {
    			Worker worker = entityService.findById(Worker.class, salaryWorkerId);
    			WorkerTestUtils.cleanUpWorker(worker);
    		}
    		getUserTransaction().commit();
    	}
	}

}
