package com.intuit.sb.payroll.capability.workerpayments;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.PayrollDistributionMethodEnum;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * <P>
 * The <CODE>DeletePayrollRunTest</CODE> class contains tests for testing the deletePayrollRun()
 * in WorkerPaymentsComponent
 * </P>
 * 
 * $Author$ $Revision$ $DateTime$
 * 
 */
public class DeletePayrollRunTest extends WorkerPaymentsComponentTestBase {
	private static Long sReadyForPayWorkerId = 0L;
	private static Long sReadyForPayWorkerId2 = 0L;
	private BigDecimal mHourlyRate = new BigDecimal("10.0");
	private BigDecimal mHourlyRate2 = new BigDecimal("12.0");
	private BigDecimal mBonusAmount = new BigDecimal("200.0");

	@BeforeClass
	public static void setUpDeletePayrollRunTest() {
		try {
			getUserTransaction().begin();
			saveEmployerInfo();
			saveWorkLocationAddress();
			setUpSanMateoAddress();
			sReadyForPayWorkerId = WorkerTestUtils.createPayrollReadyWorker("Johnny", sMountainViewLocation);
			sReadyForPayWorkerId2 = WorkerTestUtils.createPayrollReadyWorker("Paul", sMountainViewLocation);
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in setUpDeletePayrollRunTest(): " + t.getMessage());
		}
	}

	@AfterClass
	public static void tearDownDeletePayrollRunTest() {
		try {
			getUserTransaction().begin();
			if (0 != sReadyForPayWorkerId) {
    			Worker worker = entityService.findById(Worker.class, sReadyForPayWorkerId);
    			WorkerTestUtils.cleanUpWorker(worker);
    			sReadyForPayWorkerId = 0L;
			}
			if (0 != sReadyForPayWorkerId2) {
    			Worker worker = entityService.findById(Worker.class, sReadyForPayWorkerId2);
    			WorkerTestUtils.cleanUpWorker(worker);
    			sReadyForPayWorkerId2 = 0L;
			}
			restoreWorkLocationAddress();
			restoreEmployerInfo();
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in tearDownDeletePayrollRunTest(): " + t.getMessage());
		}
	}

    /**
     * <P>Test deletePayrollRun() API with 2 hourly workers</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollRunInfo with a biweekly pay frequency using default payroll run values</LI>
     * <LI>Setup a PaycheckCalculationRequest object with some compensation values for worker 1</LI>
     * <LI>Setup a PaycheckCalculationRequest object with some compensation values for worker 2</LI>
     * <LI>Setup a PayrollRunRequest object</LI>
     * <LI>Call WorkerPaymentsComponent.commitPayrollRun() with a valid PayrollRunRequest</LI>
     * <LI>Call WorkerPaymentsComponent.deletePayrollRun() with the payrollRun id returned from the commitPayrollRun()</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the payroll run is deleted</LI>
     * </UL>
     */
	@Test
	public void testDeletePayrollRunWith2HourlyWorkers() throws Exception {
    	PayrollRun payrollRun = null;
    	try {
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.getNowDate(), sDEFAULT_PAYROLL_RUN_OFFSET,
        	                                   sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
        	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
        	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
        					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
        					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
        	paycheckCalculationRequests.add(calcRequest);
        	PaycheckCalculationRequest calcRequest2 = createPaycheckCalculationRequest(sReadyForPayWorkerId2,
        					payrollRunInfo, mHourlyRate2, new Double(80), new Double(5), mBonusAmount,
        					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
        	paycheckCalculationRequests.add(calcRequest2);
        	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
            payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
            payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);
            getUserTransaction().commit();

        	payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
        	Long payrollRunId = payrollRun.getId();
        	
        	//Exercise
        	sWorkerPaymentsComponent.deletePayrollRun(payrollRunId);
        	payrollRun = null;

        	//Verify
        	boolean exceptionThrown = false;
        	try {
        		entityService.findById(PayrollRun.class, payrollRunId);
        	} catch (Exception e) {
        		exceptionThrown = true;
        	}
        	if (!exceptionThrown) {
        		AssertJUnit.fail("PayrollRun not deleted!!!");
        	}
    	} catch (Exception e) {
    		AssertJUnit.fail("Unexpected exception: " + e.getMessage());
    	} finally {
        	//Tear down
    		if (payrollRun != null) {
            	Long payrollRunId = payrollRun.getId();
            	cleanUpPayrollRun(payrollRunId);
    		}
    	}
	}

	/**
	 * <P>Test calling deletePayrollRun() with a null PayrollRunId</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.deletePayrollRun() with a null PayrollRunId</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that PayrollServiceFailureException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullPayrollRunId() throws Exception {
    	//Exercise
		sWorkerPaymentsComponent.deletePayrollRun(null);
	}

	/**
	 * <P>Test calling deletePayrollRun() with a non-existent PayrollRunId</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.deletePayrollRun() with a non-existent PayrollRunId</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that PayrollServiceFailureException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ServiceFailureException.class)
	public void testNonExistentPayrollRunId() throws Exception {
    	//Exercise
		sWorkerPaymentsComponent.deletePayrollRun(934893948L);
	}

}
