package com.intuit.sb.payroll.capability.workerpayments;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import javax.transaction.Status;

import org.testng.AssertJUnit;

import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.PayrollDistributionMethodEnum;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollRunWorkflowStatusEnum;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * <P>
 * The <CODE>FindPayrollRunTest</CODE> class contains tests for testing the
 * findPayrollRun() in WorkerPaymentsComponent
 * </P>
 * 
 * $Author$ $Revision$ $DateTime$
 * 
 */
public class FindPayrollRunTest extends WorkerPaymentsComponentTestBase {
	private static Long sReadyForPayWorkerId = 0L;
	private static Long sReadyForPayWorkerId2 = 0L;
	private BigDecimal mHourlyRate = new BigDecimal("10.0");
	private BigDecimal mBonusAmount = new BigDecimal("200.0");

	@BeforeClass
	public static void setUpFindPayrollRunTest() {
		try {
			getUserTransaction().begin();
			saveEmployerInfo();
			saveWorkLocationAddress();
			setUpSanMateoAddress();
			sReadyForPayWorkerId = WorkerTestUtils.createPayrollReadyWorker("Johnny", sMountainViewLocation);
			sReadyForPayWorkerId2 = WorkerTestUtils.createPayrollReadyWorker("Paul", sMountainViewLocation);
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in setUpFindPayrollRunTest(): " + t.getMessage());
		}
	}

	@AfterClass
	public static void tearDownFindPayrollRunTest() {
		try {
			getUserTransaction().begin();
			if (0 != sReadyForPayWorkerId) {
    			Worker worker = entityService.findById(Worker.class, sReadyForPayWorkerId);
    			WorkerTestUtils.cleanUpWorker(worker);
    			sReadyForPayWorkerId = 0L;
			}
			if (0 != sReadyForPayWorkerId2) {
    			Worker worker = entityService.findById(Worker.class, sReadyForPayWorkerId2);
    			WorkerTestUtils.cleanUpWorker(worker);
    			sReadyForPayWorkerId2 = 0L;
			}
			restoreWorkLocationAddress();
			restoreEmployerInfo();
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in tearDownFindPayrollRunTest(): " + t.getMessage());
		}
	}

    /**
     * <P>Test findPayrollRun() API with 2 hourly workers</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollRunInfo with a biweekly pay frequency using default payroll run values</LI>
     * <LI>Setup a PaycheckCalculationRequest object with some compensation values for worker 1</LI>
     * <LI>Setup a PaycheckCalculationRequest object with some compensation values for worker 2</LI>
     * <LI>Setup a PayrollRunRequest object</LI>
     * <LI>Call WorkerPaymentsComponent.commitPayrollRun() with a valid PayrollRunRequest</LI>
     * <LI>Call WorkerPaymentsComponent.findPayrollRun() with the payrollRun id returned from the commitPayrollRun()</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the payroll run state is committed and the calculated values are correct</LI>
     * </UL>
     */
	@Test
	public void testFindPayrollRunWith2HourlyWorkers() throws Exception {
    	PayrollRun payrollRun = null;
    	try {
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
        	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
        	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
        					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
        					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
        	paycheckCalculationRequests.add(calcRequest);
        	PaycheckCalculationRequest calcRequest2 = createPaycheckCalculationRequest(sReadyForPayWorkerId2,
        					payrollRunInfo, mHourlyRate, new Double(80), new Double(5), mBonusAmount,
        					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
        	paycheckCalculationRequests.add(calcRequest2);
        	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
            payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
            payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);
            getUserTransaction().commit();

            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
        	payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();
        	//Exercise
        	PayrollRun savedPayrollRun = sWorkerPaymentsComponent.findPayrollRun(payrollRun.getId());

        	//Verify
        	AssertJUnit.assertTrue(validatePayrollRun(savedPayrollRun, payrollRunRequest));
        	AssertJUnit.assertEquals("Number of paycheck(s) mismatch!", 2, savedPayrollRun.getPaychecks().size());
        	AssertJUnit.assertEquals("workflowStatus mismatch!", PayrollRunWorkflowStatusEnum.COMMITTED,
        					savedPayrollRun.getWorkflowStatus());
        	AssertJUnit.assertEquals("totalNetPay mismatch!", new Money("USD", "1762.53"), savedPayrollRun.getTotalNetPay());
        	AssertJUnit.assertEquals("totalWorkerPdTaxes mismatch!", new Money("USD", "462.46"),
        	                savedPayrollRun.getTotalWorkerPdTaxes());
        	AssertJUnit.assertEquals("totalEmployerPdTaxes mismatch!", new Money("USD", "265.90"),
        	                savedPayrollRun.getTotalEmployerPdTaxes());
        	AssertJUnit.assertEquals("totalWorkerPdBenefits mismatch!", new Money("USD", "0.00"),
        	                savedPayrollRun.getTotalWorkerPdBenefits());
        	AssertJUnit.assertEquals("totalEmployerPdBenefits mismatch!", new Money("USD", "0.00"),
        	                savedPayrollRun.getTotalEmployerPdBenefits());
    	} catch (Exception e) {
    		AssertJUnit.fail("Unexpected exception: " + e.getMessage());
    	} finally {
            if (Status.STATUS_ACTIVE != getUserTransaction().getStatus()) {
            	getUserTransaction().begin();
            }
        	//Tear down
    		if (payrollRun != null) {
            	Long payrollRunId = payrollRun.getId();
            	cleanUpPayrollRun(payrollRunId);
    		}
    		getUserTransaction().commit();
    	}
	}

	/**
	 * <P>Test calling findPayrollRun() with a null PayrollRunId</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.findPayrollRun() with a null PayrollRunId</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that PayrollServiceFailureException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullPayrollRunId() throws Exception {
    	//Exercise
		@SuppressWarnings("unused")
		PayrollRun payollRun = sWorkerPaymentsComponent.findPayrollRun(null);
	}

	/**
	 * <P>Test calling findPayrollRun() with a non-existent PayrollRunId</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.findPayrollRun() with a non-existent PayrollRunId</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that PayrollServiceFailureException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ServiceFailureException.class)
	public void testNonExistentPayrollRunId() throws Exception {
    	//Exercise
		@SuppressWarnings("unused")
		PayrollRun payollRun = sWorkerPaymentsComponent.findPayrollRun(934893943L);
	}

}
