package com.intuit.sb.payroll.capability.workerpayments;

import java.util.Calendar;
import java.util.Date;

import org.testng.annotations.AfterClass;
import org.testng.AssertJUnit;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;
import com.intuit.sb.payroll.entity.model.Worker;


/**
 * The <CODE>GetNextScheduledPayrollRunInfoExistingPayrollRunSemimonthlyTest</CODE> class contains tests for testing the
 * getNextScheduledPayrollRunInfo() in PayrollScheduleComponent using a semimonthly schedule with existing PayrollRuns
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class GetNextScheduledPayrollRunInfoExistingPayrollRunSemimonthlyTest extends WorkerPaymentsComponentTestBase {
	private static Long sReadyForPayWorkerId = 0L;
	private static Calendar periodEndDate = null;
	private static PayrollRun payrollRun = null;
	private static PayrollSchedule aSchedule = null;
	
    /**
     * <P>One time setup for this test class</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Remember some environment setting so that it can be restored later</LI>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Create a semimonthly PayrollSchedule</LI>
     * <LI>Create 1 payrollRun</LI>
     * </OL>
     *
     */
	@BeforeClass
	public static void setUpGetNextScheduledPayrollRunInfoExistingPayrollRunSemimonthlyTest() {
		try {
			getUserTransaction().begin();
			saveEmployerInfo();
			saveWorkLocationAddress();
			setUpSanMateoAddress();
			sReadyForPayWorkerId = WorkerTestUtils.createPayrollReadyWorker("Johnny", sMountainViewLocation);
			periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 15);
	    	aSchedule = createSemimonthlySchedule(periodEndDate.getTime(), 15, PayrollSchedule.LAST_DAY_OF_MONTH,
	    	                                      sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
    		payrollRun = createOnePayrollRun(sReadyForPayWorkerId, aSchedule);
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in setUpGetNextScheduledPayrollRunInfoExistingPayrollRunSemimonthlyTest(): " +
			            t.getMessage());
		}
	}

    /**
     * <P>One time tearDown for this test class</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Delete the payrollRun that was created in setUp</LI>
     * <LI>Delete the payroll ready worker that was created in setUp</LI>
     * <LI>Restore the environment settings</LI>
     * </OL>
     *
     */
	@AfterClass
	public static void tearDownGetNextScheduledPayrollRunInfoExistingPayrollRunSemimonthlyTest() {
		try {
			getUserTransaction().begin();
			if (null != payrollRun) {
            	Long payrollRunId = payrollRun.getId();
            	cleanUpPayrollRun(payrollRunId);
			}
			if (0 != sReadyForPayWorkerId) {
    			Worker worker = entityService.findById(Worker.class, sReadyForPayWorkerId);
    			WorkerTestUtils.cleanUpWorker(worker);
    			sReadyForPayWorkerId = 0L;
			}
			restoreWorkLocationAddress();
			restoreEmployerInfo();
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in tearDownGetNextScheduledPayrollRunInfoExistingPayrollRunSemimonthlyTest(): " +
			            t.getMessage());
		}
	}

    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing PayrollRun</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo() throws Exception {
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 16),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 31)),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 4),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 2),
		                                             PayrollFrequencyEnum.SEMIMONTHLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing PayrollRun, after changing PayrollRunDate</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Setup an updated PayrollSchedule</LI>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testChangingPayRunDate() throws Exception {
		// Setup
		PayrollSchedule schedule = createSemimonthlySchedule(periodEndDate.getTime(), 15,
		                                                     PayrollSchedule.LAST_DAY_OF_MONTH,
		                                                     sDEFAULT_PAYROLL_RUN_OFFSET+1, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 16),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 31)),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 4),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 3),
		                                             PayrollFrequencyEnum.SEMIMONTHLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing PayrollRun, after changing PayDayDate</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup an updated PayrollSchedule</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testChangingPayDayDate() throws Exception {
		// Setup
		PayrollSchedule schedule = createSemimonthlySchedule(periodEndDate.getTime(), 15,
		                                                     PayrollSchedule.LAST_DAY_OF_MONTH,
		                                                     sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET-1);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 16),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 31)),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 3),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 2),
		                                             PayrollFrequencyEnum.SEMIMONTHLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing PayrollRun, NextPayPeriodEndDate in the
     * PayrollSchedule is set before the PayrollRun</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testNextPayPeriodEndDate_Before_PayrollRun() throws Exception {
		// Setup
		Calendar nextPayPeriodEndDate = (Calendar)periodEndDate.clone();
		nextPayPeriodEndDate.add(Calendar.DAY_OF_MONTH, -15);
		PayrollSchedule schedule = createSemimonthlySchedule(periodEndDate.getTime(), 15,
		                                                     PayrollSchedule.LAST_DAY_OF_MONTH,
		                                                     sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 16),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 31)),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 4),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 2),
		                                             PayrollFrequencyEnum.SEMIMONTHLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing PayrollRun, NextPayPeriodEndDate in the
     * PayrollSchedule is set to 2 months after the PayrollRun</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testNextPayPeriodEndDate_After_PayrollRun() throws Exception {
		// Setup
		Calendar nextPayPeriodEndDate = (Calendar)periodEndDate.clone();
		nextPayPeriodEndDate.add(Calendar.MONTH, 2);
		PayrollSchedule schedule = createSemimonthlySchedule(periodEndDate.getTime(), 15,
		                                                     PayrollSchedule.LAST_DAY_OF_MONTH,
		                                                     sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 16),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 31)),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 4),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 2),
		                                             PayrollFrequencyEnum.SEMIMONTHLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing PayrollRun, NextPayPeriodEndDate in the
     * PayrollSchedule is set to 2 months after the PayrollRun, and also changing the PayrollRunDate</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testNextPayPeriodEndDate_After_PayrollRun_And_ChangePayrollRunDate() throws Exception {
		// Setup
		Calendar nextPayPeriodEndDate = (Calendar)periodEndDate.clone();
		nextPayPeriodEndDate.add(Calendar.MONTH, 2);
		PayrollSchedule schedule = createSemimonthlySchedule(periodEndDate.getTime(), 15,
		                                                     PayrollSchedule.LAST_DAY_OF_MONTH,
		                                                     sDEFAULT_PAYROLL_RUN_OFFSET+1, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 16),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 31)),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 4),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 3),
		                                             PayrollFrequencyEnum.SEMIMONTHLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing semimonthly PayrollRun after changing pay
     *  frequency to a biweekly schedule, next pay period end date is exactly 2 weeks from last payroll run</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a biweekly schedule</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Setup an updated PayrollSchedule</LI>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testChangingToBiweekly_with_Exact_PayPeriod() throws Exception {
		// Setup
		Date nextPeriodEndDate = (Date)periodEndDate.getTime().clone();
		nextPeriodEndDate = DateUtil.getNextDate(nextPeriodEndDate, 14);
		PayrollSchedule schedule = createBiweeklySchedule(nextPeriodEndDate, sDEFAULT_PAYROLL_RUN_OFFSET,
		                        	    	              sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 16),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 29)),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 3),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 30),
		                                             PayrollFrequencyEnum.BIWEEKLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing semimonthly PayrollRun after changing pay
     *  frequency to a biweekly schedule, next pay period end date is less than 2 weeks from last payroll run</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a biweekly schedule</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Setup an updated PayrollSchedule</LI>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test(enabled=false, groups={"broken"})
	public void testChangingToBiweekly_with_Shorter_PayPeriod() throws Exception {
		// Setup
		Date nextPeriodEndDate = (Date)periodEndDate.getTime().clone();
		nextPeriodEndDate = DateUtil.getNextDate(nextPeriodEndDate, 12);
		PayrollSchedule schedule = createBiweeklySchedule(nextPeriodEndDate, sDEFAULT_PAYROLL_RUN_OFFSET,
		                        	    	              sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 16),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 27)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 30),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 28),
		                                             PayrollFrequencyEnum.BIWEEKLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing semimonthly PayrollRun after changing pay
     *  frequency to a biweekly schedule, next pay period end date is more than 2 weeks from last payroll run</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a biweekly schedule</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Setup an updated PayrollSchedule</LI>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test(enabled=false, groups={"broken"})
	public void testChangingToBiweekly_with_Longer_PayPeriod() throws Exception {
		// Setup
		Date nextPeriodEndDate = (Date)periodEndDate.getTime().clone();
		nextPeriodEndDate = DateUtil.getNextDate(nextPeriodEndDate, 15);
		PayrollSchedule schedule = createBiweeklySchedule(nextPeriodEndDate, sDEFAULT_PAYROLL_RUN_OFFSET,
		                        	    	              sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 16),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 30)),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 2),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 31),
		                                             PayrollFrequencyEnum.BIWEEKLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing semimonthly PayrollRun after changing pay
     *  frequency to a weekly schedule, next pay period end date is exactly 1 week from last payroll run</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a weekly schedule</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Setup an updated PayrollSchedule</LI>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testChangingToWeekly_with_Exact_PayPeriod() throws Exception {
		// Setup
		Date nextPeriodEndDate = (Date)periodEndDate.getTime().clone();
		nextPeriodEndDate = DateUtil.getNextDate(nextPeriodEndDate, 7);
		PayrollSchedule schedule = createWeeklySchedule(nextPeriodEndDate, sDEFAULT_PAYROLL_RUN_OFFSET,
		                        	    	            sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 16),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 22)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 27),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 23),
		                                             PayrollFrequencyEnum.WEEKLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing semimonthly PayrollRun after changing pay
     *  frequency to a weekly schedule, next pay period end date is less than 1 week from last payroll run</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a weekly schedule</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Setup an updated PayrollSchedule</LI>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test(enabled=false, groups={"broken"})
	public void testChangingToWeekly_with_Shorter_PayPeriod() throws Exception {
		// Setup
		Date nextPeriodEndDate = (Date)periodEndDate.getTime().clone();
		nextPeriodEndDate = DateUtil.getNextDate(nextPeriodEndDate, 5);
		PayrollSchedule schedule = createWeeklySchedule(nextPeriodEndDate, sDEFAULT_PAYROLL_RUN_OFFSET,
		                        	    	            sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 16),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 20)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 23),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 21),
		                                             PayrollFrequencyEnum.WEEKLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing semimonthly PayrollRun after changing pay
     *  frequency to a weekly schedule, next pay period end date is more than 1 week from last payroll run</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a weekly schedule</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Setup an updated PayrollSchedule</LI>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test(enabled=false, groups={"broken"})
	public void testChangingToWeekly_with_Longer_PayPeriod() throws Exception {
		// Setup
		Date nextPeriodEndDate = (Date)periodEndDate.getTime().clone();
		nextPeriodEndDate = DateUtil.getNextDate(nextPeriodEndDate, 8);
		PayrollSchedule schedule = createWeeklySchedule(nextPeriodEndDate, sDEFAULT_PAYROLL_RUN_OFFSET,
		                        	    	            sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 16),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 23)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 26),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 24),
		                                             PayrollFrequencyEnum.WEEKLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing semimonthly PayrollRun after changing pay
     *  frequency to a monthly schedule, next pay period end date is exactly 1 month from last payroll run</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a monthly schedule</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Setup an updated PayrollSchedule</LI>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testChangingToMonthly_with_Exact_PayPeriod() throws Exception {
		// Setup
		Date nextPeriodEndDate = (Date)periodEndDate.getTime().clone();
		nextPeriodEndDate = addDate(nextPeriodEndDate, Calendar.MONTH, 1);
		PayrollSchedule schedule = createMonthlySchedule(nextPeriodEndDate, 15, sDEFAULT_PAYROLL_RUN_OFFSET,
		                        	    	             sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 16),
		                                                           DateUtil.createDate(2009, Calendar.FEBRUARY, 15)),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 19),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 17),
		                                             PayrollFrequencyEnum.MONTHLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing semimonthly PayrollRun after changing pay
     *  frequency to a monthly schedule, next pay period end date is less than 1 month from last payroll run</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a monthly schedule</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Setup an updated PayrollSchedule</LI>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testChangingToMonthly_with_Shorter_PayPeriod() throws Exception {
		// Setup
		Date nextPeriodEndDate = (Date)periodEndDate.getTime().clone();
		nextPeriodEndDate = DateUtil.getNextDate(nextPeriodEndDate, 16);
		PayrollSchedule schedule = createMonthlySchedule(nextPeriodEndDate, PayrollSchedule.LAST_DAY_OF_MONTH,
		                                                 sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 16),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 31)),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 4),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 2),
		                                             PayrollFrequencyEnum.MONTHLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo with 1 existing semimonthly PayrollRun after changing pay
     *  frequency to a monthly schedule, next pay period end date is more than 1 month from last payroll run</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a monthly schedule</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Setup an updated PayrollSchedule</LI>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test(enabled=false, groups={"broken"})
	public void testChangingToMonthly_with_Longer_PayPeriod() throws Exception {
		// Setup
		Date nextPeriodEndDate = (Date)periodEndDate.getTime().clone();
		nextPeriodEndDate = addDate(nextPeriodEndDate, Calendar.MONTH, 1);
		nextPeriodEndDate = DateUtil.getNextDate(nextPeriodEndDate, 1);
		PayrollSchedule schedule = createMonthlySchedule(nextPeriodEndDate, 16, sDEFAULT_PAYROLL_RUN_OFFSET,
		                        	    	             sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(schedule);
		
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 16),
		                                                           DateUtil.createDate(2009, Calendar.FEBRUARY, 16)),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 19),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 17),
		                                             PayrollFrequencyEnum.MONTHLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
}
