package com.intuit.sb.payroll.capability.workerpayments;

import java.util.Calendar;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;


/**
 * The <CODE>GetNextScheduledPayrollRunInfoMonthlyTest</CODE> class contains tests for testing the
 * getNextScheduledPayrollRunInfo() in PayrollScheduleComponent using a monthly schedule with no existing PayrollRuns
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class GetNextScheduledPayrollRunInfoMonthlyTest extends WorkerPaymentsComponentTestBase {
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 1st of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate1() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 1);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 1, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 2),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 1)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 6),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 2),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo in a leap year, Pay period end date is on the 1st of the month
     * </P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate1_LeapYear() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2012, Calendar.MARCH, 1);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 1, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2012, Calendar.FEBRUARY, 2),
		                                                           DateUtil.createDate(2012, Calendar.MARCH, 1)),
		                                             DateUtil.createDate(2012, Calendar.MARCH, 6),
		                                             DateUtil.createDate(2012, Calendar.MARCH, 2),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
 		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 2nd of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate2() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 2);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 2, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 3),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 2)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 7),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 5),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 3rd of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate3() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 3);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 3, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 4),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 3)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 7),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 5),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 4th of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate4() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 4);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 4, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 5),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 4)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 7),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 5),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 5th of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate5() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 5);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 5, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 6),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 5)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 8),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 6),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 6th of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate6() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 6);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 6, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 7),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 6)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 9),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 7),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 7th of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate7() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 7);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 7, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 8),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 7)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 12),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 8),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 8th of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate8() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 8);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 8, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 9),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 8)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 13),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 9),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 9th of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate9() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 9);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 9, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 10),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 9)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 14),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 12),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 10th of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate10() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 10);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 10, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 11),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 10)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 14),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 12),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 11th of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate11() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 11);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 11, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 12),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 11)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 14),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 12),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 12th of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate12() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 12);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 12, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 13),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 12)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 15),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 13),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 13th of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate13() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 13);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 13, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 13)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 16),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 14),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 14th of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate14() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 14);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 14, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 15),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 14)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 20),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 15),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 15th of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate15() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 15);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 15, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 16),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 15)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 21),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 16),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 16th of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate16() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 16);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 16, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 17),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 16)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 22),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 20),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 17th of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate17() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 17);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 17, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 18),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 17)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 22),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 20),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 18th of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate18() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 18);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 18, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 19),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 18)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 22),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 20),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 19th of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate19() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 19);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 19, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 20),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 19)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 22),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 20),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 20th of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate20() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 20);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 20, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 21),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 20)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 23),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 21),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 21st of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate21() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 21);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 21, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 22),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 21)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 26),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 22),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 22nd of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate22() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 22);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 22, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 23),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 22)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 27),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 23),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 23rd of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate23() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 23);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 23, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 24),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 23)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 28),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 26),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 24th of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate24() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 24);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 24, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 25),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 24)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 28),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 26),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 25th of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate25() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 25);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 25, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 26),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 25)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 28),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 26),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 26th of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate26() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 26);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 26, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 27),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 26)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 29),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 27),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the 27th of the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate27() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 27);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), 27, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 28),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 27)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 30),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 28),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the last day of January</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate_LastDay_January() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 31);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                  sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JANUARY, 1),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 31)),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 4),
		                                             DateUtil.createDate(2009, Calendar.FEBRUARY, 2),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the last day of February</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate_LastDay_Febuary() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.FEBRUARY, 28);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                  sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.FEBRUARY, 1),
		                                                           DateUtil.createDate(2009, Calendar.FEBRUARY, 28)),
		                                             DateUtil.createDate(2009, Calendar.MARCH, 4),
		                                             DateUtil.createDate(2009, Calendar.MARCH, 2),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the last day of February in a
     * leap year</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate_LastDay_Febuary_LeapYear() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2012, Calendar.FEBRUARY, 29);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                  sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2012, Calendar.FEBRUARY, 1),
		                                                           DateUtil.createDate(2012, Calendar.FEBRUARY, 29)),
		                                             DateUtil.createDate(2012, Calendar.MARCH, 5),
		                                             DateUtil.createDate(2012, Calendar.MARCH, 1),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the last day of March</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate_LastDay_March() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.MARCH, 31);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                  sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MARCH, 1),
		                                                           DateUtil.createDate(2009, Calendar.MARCH, 31)),
		                                             DateUtil.createDate(2009, Calendar.APRIL, 3),
		                                             DateUtil.createDate(2009, Calendar.APRIL, 1),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the last day of April</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate_LastDay_April() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.APRIL, 30);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                  sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.APRIL, 1),
		                                                           DateUtil.createDate(2009, Calendar.APRIL, 30)),
		                                             DateUtil.createDate(2009, Calendar.MAY, 5),
		                                             DateUtil.createDate(2009, Calendar.MAY, 1),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the last day of May</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate_LastDay_May() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.MAY, 31);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                  sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.MAY, 1),
		                                                           DateUtil.createDate(2009, Calendar.MAY, 31)),
		                                             DateUtil.createDate(2009, Calendar.JUNE, 3),
		                                             DateUtil.createDate(2009, Calendar.JUNE, 1),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the last day of June</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate_LastDay_June() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JUNE, 30);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                  sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JUNE, 1),
		                                                           DateUtil.createDate(2009, Calendar.JUNE, 30)),
		                                             DateUtil.createDate(2009, Calendar.JULY, 3),
		                                             DateUtil.createDate(2009, Calendar.JULY, 1),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the last day of July</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate_LastDay_July() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JULY, 31);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                  sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.JULY, 1),
		                                                           DateUtil.createDate(2009, Calendar.JULY, 31)),
		                                             DateUtil.createDate(2009, Calendar.AUGUST, 5),
		                                             DateUtil.createDate(2009, Calendar.AUGUST, 3),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the last day of August</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate_LastDay_August() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.AUGUST, 31);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                  sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.AUGUST, 1),
		                                                           DateUtil.createDate(2009, Calendar.AUGUST, 31)),
		                                             DateUtil.createDate(2009, Calendar.SEPTEMBER, 3),
		                                             DateUtil.createDate(2009, Calendar.SEPTEMBER, 1),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the last day of September</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate_LastDay_September() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.SEPTEMBER, 30);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                  sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.SEPTEMBER, 1),
		                                                           DateUtil.createDate(2009, Calendar.SEPTEMBER, 30)),
		                                             DateUtil.createDate(2009, Calendar.OCTOBER, 5),
		                                             DateUtil.createDate(2009, Calendar.OCTOBER, 1),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the last day of October</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate_LastDay_October() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.OCTOBER, 31);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                  sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.OCTOBER, 1),
		                                                           DateUtil.createDate(2009, Calendar.OCTOBER, 31)),
		                                             DateUtil.createDate(2009, Calendar.NOVEMBER, 4),
		                                             DateUtil.createDate(2009, Calendar.NOVEMBER, 2),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the last day of November</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate_LastDay_November() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.NOVEMBER, 30);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                  sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.NOVEMBER, 1),
		                                                           DateUtil.createDate(2009, Calendar.NOVEMBER, 30)),
		                                             DateUtil.createDate(2009, Calendar.DECEMBER, 3),
		                                             DateUtil.createDate(2009, Calendar.DECEMBER, 1),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is on the last day of December</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testPayPeriodEndDate_LastDay_December() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.DECEMBER, 31);
    	aSchedule = createMonthlySchedule(periodEndDate.getTime(), PayrollSchedule.LAST_DAY_OF_MONTH,
    	                                  sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2009, Calendar.DECEMBER, 1),
		                                                           DateUtil.createDate(2009, Calendar.DECEMBER, 31)),
		                                             DateUtil.createDate(2010, Calendar.JANUARY, 6),
		                                             DateUtil.createDate(2010, Calendar.JANUARY, 4),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is null with the day on the 1st of the
     *  month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test(enabled=false, groups={"broken"})
	public void testNullPayPeriodEndDate1() throws Exception {
		// Setup
		Calendar cal = DateUtil.getNowCalendar();
		int currentMonth = cal.get(Calendar.MONTH);
		int currentYear = cal.get(Calendar.YEAR);
		cal.add(Calendar.MONTH, -1);
		int lastMonth = cal.get(Calendar.MONTH);
		int year2 = cal.get(Calendar.YEAR);
    	aSchedule = createMonthlySchedule(null, 1, sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(year2, lastMonth, 2),
		                                                           DateUtil.createDate(currentYear, currentMonth, 1)),
		                                             DateUtil.createDate(currentYear, currentMonth, 4),
		                                             DateUtil.createDate(currentYear, currentMonth, 2),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is null with the day on the 2nd of the
     *  month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test(enabled=false, groups={"broken"})
	public void testNullPayPeriodEndDate2() throws Exception {
		// Setup
		Calendar cal = DateUtil.getNowCalendar();
		int currentMonth = cal.get(Calendar.MONTH);
		int currentYear = cal.get(Calendar.YEAR);
		cal.add(Calendar.MONTH, -1);
		int lastMonth = cal.get(Calendar.MONTH);
		int year2 = cal.get(Calendar.YEAR);
    	aSchedule = createMonthlySchedule(null, 2, sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(year2, lastMonth, 3),
		                                                           DateUtil.createDate(currentYear, currentMonth, 2)),
		                                             DateUtil.createDate(currentYear, currentMonth, 5),
		                                             DateUtil.createDate(currentYear, currentMonth, 3),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period end date is null with the day on the last day of
     *  the month</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a monthly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test(enabled=false, groups={"broken"})
	public void testNullPayPeriodEndDate_LastDayOfMonth() throws Exception {
		// Setup
		Calendar cal = DateUtil.getNowCalendar();
		int currentMonth = cal.get(Calendar.MONTH);
		int currentYear = cal.get(Calendar.YEAR);
		int lastDay = DateUtil.getLastDayOfMonth(cal);
		cal.add(Calendar.MONTH, 1);
		int nextMonth = cal.get(Calendar.MONTH);
		int year2 = cal.get(Calendar.YEAR);
    	aSchedule = createMonthlySchedule(null, PayrollSchedule.LAST_DAY_OF_MONTH, sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                  sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(currentYear, currentMonth, 1),
		                                                           DateUtil.createDate(currentYear, currentMonth,
		                                                                               lastDay)),
		                                             DateUtil.createDate(year2, nextMonth, 3),
		                                             DateUtil.createDate(year2, nextMonth, 1),
		                                             PayrollFrequencyEnum.MONTHLY, PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
}
