package com.intuit.sb.payroll.capability.workerpayments;

import java.util.Calendar;
import java.util.Date;

import org.testng.AssertJUnit;

import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;

public class PayrollScheduleComponentTest extends PayrollScheduleTestBase {
	
    @BeforeMethod
    public void testInitialize()
    {
        deleteAllPayrollSchedule();
    }

    /**
     * this method tests PayrollSchedule 
     */
    @Test
    public void testSaveNewPaySchedule() throws Exception {

        Calendar endDate = DateUtil.getNowCalendar();
        endDate.add(Calendar.DAY_OF_YEAR, 15);
        Calendar payrollRunDate = DateUtil.getNowCalendar();
        payrollRunDate.add(Calendar.DAY_OF_YEAR, 16);
        Calendar payDate = DateUtil.getNowCalendar();
        payDate.add(Calendar.DAY_OF_YEAR, 18);

        PayrollSchedule payrollSchedule = new PayrollSchedule();
        payrollSchedule.setFrequency(PayrollFrequencyEnum.WEEKLY);
        payrollSchedule.setNextPayPeriodEndDate(payDate.getTime());
        payrollSchedule.setPayDayOffset(3);
        payrollSchedule.setPayrollRunDayOffset(3);
        payrollSchedule.setMemo("Created Schedule");
        payrollSchedule.setDefaultt(true);

        // Save the new one
        payrollSchedule = payrollScheduleComponent.createPayrollSchedule(payrollSchedule);

        PayrollSchedule savedPs = payrollScheduleComponent.findPayrollSchedule(payrollSchedule.getId());
        
        if (savedPs == null)
        {
          AssertJUnit.assertTrue("Not able to find the PayrollSchedule from DB",false);  
        }
        validatePayrollSchedule(payrollSchedule, savedPs, false);
    }
    
    

    /**
     * Tests Update of a PayrollSchedule.
     */
    @Test
    public void testUpdatePaySchedule() throws Exception {

        Calendar endDate = DateUtil.getNowCalendar();
        endDate.add(Calendar.DAY_OF_YEAR, 15);
        Calendar payrollRunDate = DateUtil.getNowCalendar();
        payrollRunDate.add(Calendar.DAY_OF_YEAR, 16);
        Calendar payDate = DateUtil.getNowCalendar();
        payDate.add(Calendar.DAY_OF_YEAR, 18);

        PayrollSchedule payrollSchedule = new PayrollSchedule();
        payrollSchedule.setFrequency(PayrollFrequencyEnum.WEEKLY);
        payrollSchedule.setNextPayPeriodEndDate(payDate.getTime());
        payrollSchedule.setPayDayOffset(3);
        payrollSchedule.setPayrollRunDayOffset(3);
        payrollSchedule.setMemo("Created Schedule");
        payrollSchedule.setDefaultt(true);

        // Save the new one
        payrollSchedule = payrollScheduleComponent.createPayrollSchedule(payrollSchedule);

        PayrollSchedule savedPs = payrollScheduleComponent.findPayrollSchedule(payrollSchedule.getId());
        
        if (savedPs == null)
        {
          AssertJUnit.assertTrue("Not able to find the PayrollSchedule from DB",false);  
        }
        validatePayrollSchedule(payrollSchedule, savedPs, false);
        
        savedPs.setMemo("NewMemo");
        savedPs.setFrequency(PayrollFrequencyEnum.BIWEEKLY);
        
        savedPs = payrollScheduleComponent.updatePayrollSchedule(savedPs);
        AssertJUnit.assertEquals("NewMemo", savedPs.getMemo());
        AssertJUnit.assertEquals(PayrollFrequencyEnum.BIWEEKLY, savedPs.getFrequency());
    }


    /**
     * This test method will test the update a Validation exception while creating an Invalid Monthly
     * PayrollSchedule
     */
    @Test(expectedExceptions = ValidationException.class)
    public void testInvalidCreatePayrollSchedule()throws Exception
    {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.MONTHLY;
        Calendar cal = DateUtil.createCalendar(2008, 03, 15);
        PayrollSchedule pSchedule = new PayrollSchedule();

        Calendar nextPayDate = DateUtil.getNextWeekDayOfWeek(cal, Calendar.FRIDAY);
        pSchedule.setNextPayPeriodEndDate(nextPayDate.getTime());

        pSchedule.setFrequency(frequency);

        pSchedule.setMemo(frequency.toString());
        pSchedule.setPayDayOffset(3);
        pSchedule.setPayrollRunDayOffset(3);
        payrollScheduleComponent.createPayrollSchedule(pSchedule);
    }

    /**
     * This test method will test the update expecting a Validation exception while updating an Invalid
     * PayrollSchedule
     * 
     * Monthly does not expects an End Date
     */
    @Test(expectedExceptions = ValidationException.class)
    public void testInvalidUpdatePayrollSchedule()throws Exception
    {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.MONTHLY;
        Calendar cal = DateUtil.createCalendar(2008, 03, 15);
        PayrollSchedule pSchedule = new PayrollSchedule();

        pSchedule.setFrequency(frequency);

        pSchedule.setMemo(frequency.toString());
        pSchedule.setPayDayOffset(3);
        pSchedule.setPayrollRunDayOffset(3);
        pSchedule = payrollScheduleComponent.createPayrollSchedule(pSchedule);

        Calendar nextPayDate = DateUtil.getNextWeekDayOfWeek(cal, Calendar.FRIDAY);
        pSchedule.setNextPayPeriodEndDate(nextPayDate.getTime());

        pSchedule = payrollScheduleComponent.updatePayrollSchedule(pSchedule);
    }
    
    @Test (expectedExceptions = ValidationException.class)
    public void testInvalidPayrollSchedule()throws Exception {
        PayrollSchedule pSchedule = new PayrollSchedule();
        payrollScheduleComponent.updatePayrollSchedule(pSchedule);
    }
    
    /**
     * This test will check that we get a sample Payroll Schedule with these parameters payDate : The next upcoming
     * Friday after the current day. payrollPayDateOffset : 2 days before the Friday selected (Wed) frequency :
     * Bi-Weekly (every 2 weeks) periodEndDate : The day before the run payroll date (Tues) periodStartDate : Shown on
     * calendar as Wed 2 weeks before per end date and pay frequency of bi-weekly. and the PayrollSchedule is saved in
     * the repo
     */
    @Test
    public void testGetPayrollScheduleWithDefaultPayrollSchedule() throws Exception
    {
    	PayrollScheduleTestUtils.createDefaultSchedule();
        PayrollSchedule pSchedule = payrollScheduleComponent.getDefaultPayrollSchedule();
        validateDefaultPayrollSchedule(pSchedule);
    }

    /**
     * Save a Weekly PayrollSchedule and then use the API getPayrollSchedule to ensure the API returns the right one
     */

    @Test
    public void testGetPayrollScheduleWithExistingPayrollScheduleWeekly() throws Exception
    {
        Calendar cal = DateUtil.createCalendar(2008, Calendar.OCTOBER, 24);
        PayrollSchedule validPayrollSchedule = createAndSaveNewDefaultPayrollSchedule(PayrollFrequencyEnum.WEEKLY, cal, 1, 2);
        PayrollSchedule pSchedule = payrollScheduleComponent.getDefaultPayrollSchedule();
        validatePayrollSchedule(validPayrollSchedule, pSchedule, true);
    }

    @Test
    public void testGetPayrollScheduleWithExistingPayrollScheduleBiWeekly() throws Exception
    {
        Calendar cal = DateUtil.createCalendar(2008, Calendar.OCTOBER, 24);
        PayrollSchedule validPayrollSchedule = createAndSaveNewDefaultPayrollSchedule(PayrollFrequencyEnum.BIWEEKLY, cal, 1, 2);
        PayrollSchedule pSchedule = payrollScheduleComponent.getDefaultPayrollSchedule();
        validatePayrollSchedule(validPayrollSchedule, pSchedule, true);
    }

    @Test
    public void testGetPayrollScheduleWithExistingPayrollScheduleMonthly() throws Exception
    {
        Calendar cal = DateUtil.createCalendar(2008, Calendar.OCTOBER, 24);
        PayrollSchedule validPayrollSchedule = createAndSaveNewDefaultPayrollSchedule(PayrollFrequencyEnum.MONTHLY, cal, 1, 2);
        PayrollSchedule pSchedule = payrollScheduleComponent.getDefaultPayrollSchedule();
        validatePayrollSchedule(validPayrollSchedule, pSchedule, true);
    }

    /**
     * This test will set payroll Run date before the period End date
     */

    @Test
    public void testGetPayrollScheduleWithNegativeRunOffset() throws Exception
    {
        Calendar cal = DateUtil.createCalendar(2008, Calendar.OCTOBER, 24);
        PayrollSchedule validPayrollSchedule = createAndSaveNewDefaultPayrollSchedule(PayrollFrequencyEnum.BIWEEKLY, cal, -1, 2);
        PayrollSchedule pSchedule = payrollScheduleComponent.getDefaultPayrollSchedule();
        validatePayrollSchedule(validPayrollSchedule, pSchedule, true);
    }

    @Test
    public void testUpdateDefaultPayrollSchedule() throws Exception {
    	PayrollScheduleTestUtils.createDefaultSchedule();
        PayrollSchedule validPayrollSchedule = payrollScheduleComponent.getDefaultPayrollSchedule();

        validPayrollSchedule.setFrequency(PayrollFrequencyEnum.WEEKLY);
        Date periodEndDate = DateUtil.createCalendar(2008, Calendar.APRIL, 4).getTime(); // Apr 4 2008
        validPayrollSchedule.setNextPayPeriodEndDate(periodEndDate);
        validPayrollSchedule.setPayDayOffset(1);

        PayrollSchedule pSchedule = payrollScheduleComponent.updatePayrollSchedule(validPayrollSchedule);
        validatePayrollSchedule(validPayrollSchedule, pSchedule, false);
    }

    @Test
    public void testUpdateMonthlyPayrollSchedule() throws Exception {

        Calendar cal = DateUtil.createCalendar(2008, Calendar.OCTOBER, 24);

        createAndSaveNewDefaultPayrollSchedule(PayrollFrequencyEnum.WEEKLY, cal, 1, 2);

        PayrollSchedule validPayrollSchedule = payrollScheduleComponent.getDefaultPayrollSchedule();

        validPayrollSchedule.setFrequency(PayrollFrequencyEnum.MONTHLY);
        validPayrollSchedule.setPayPeriodEndDayofMonth1(12);
        validPayrollSchedule.setPayDayOffset(2);
        validPayrollSchedule.setPayrollRunDayOffset(1);
        validPayrollSchedule.setMemo("Monthly Schedule");

        PayrollSchedule pSchedule = payrollScheduleComponent.updatePayrollSchedule(validPayrollSchedule);
        validatePayrollSchedule(validPayrollSchedule, pSchedule, true);
    }

    @Test(enabled=false, groups={"broken"})
    public void testUpdateSemiMonthlyPayrollSchedule() throws Exception {

        Calendar cal = DateUtil.createCalendar(2008, Calendar.OCTOBER, 24);

        createAndSaveNewDefaultPayrollSchedule(PayrollFrequencyEnum.WEEKLY, cal, 1, 2);

        PayrollSchedule validPayrollSchedule = payrollScheduleComponent.getDefaultPayrollSchedule();

        validPayrollSchedule.setFrequency(PayrollFrequencyEnum.SEMIMONTHLY);
        validPayrollSchedule.setPayPeriodEndDayofMonth1(12);
        validPayrollSchedule.setPayPeriodEndDayofMonth2(26);
        validPayrollSchedule.setPayDayOffset(2);
        validPayrollSchedule.setPayrollRunDayOffset(1);

        PayrollSchedule pSchedule = payrollScheduleComponent.updatePayrollSchedule(validPayrollSchedule);
        validatePayrollSchedule(validPayrollSchedule, pSchedule, true);
    }

    @Test(expectedExceptions = ValidationException.class)
    public void testUpdateInvalidSemiMonthlyPayrollSchedule() throws Exception {

        Calendar cal = DateUtil.createCalendar(2008, Calendar.OCTOBER, 24);

        createAndSaveNewDefaultPayrollSchedule(PayrollFrequencyEnum.WEEKLY, cal, 1, 2);
        
        PayrollSchedule validPayrollSchedule = payrollScheduleComponent.getDefaultPayrollSchedule();

        validPayrollSchedule.setFrequency(PayrollFrequencyEnum.SEMIMONTHLY);
        validPayrollSchedule.setPayPeriodEndDayofMonth1(12);
        validPayrollSchedule.setPayDayOffset(2);
        validPayrollSchedule.setPayrollRunDayOffset(1);

        PayrollSchedule pSchedule = payrollScheduleComponent.updatePayrollSchedule(validPayrollSchedule);
        validatePayrollSchedule(validPayrollSchedule, pSchedule, true);
    }

    /**
     * Expects InvalidException as PayPeriodEndDayofMonth1 is not set
     * 
     * @throws InvalidPayrollScheduleException
     * @throws IllegalEndDateException
     */
    @Test(expectedExceptions = ValidationException.class)
    public void testUpdateInvalidSemiMonthlyPayrollSchedule2() throws Exception {

        Calendar cal = DateUtil.createCalendar(2008, Calendar.OCTOBER, 24);

        createAndSaveNewDefaultPayrollSchedule(PayrollFrequencyEnum.WEEKLY, cal, 1, 2);

        PayrollSchedule validPayrollSchedule = payrollScheduleComponent.getDefaultPayrollSchedule();

        validPayrollSchedule.setFrequency(PayrollFrequencyEnum.SEMIMONTHLY);
        validPayrollSchedule.setPayPeriodEndDayofMonth2(12);
        validPayrollSchedule.setPayDayOffset(2);
        validPayrollSchedule.setPayrollRunDayOffset(1);

        PayrollSchedule pSchedule = payrollScheduleComponent.updatePayrollSchedule(validPayrollSchedule);
        validatePayrollSchedule(validPayrollSchedule, pSchedule, true);
    }

    /**
     * This test will update PayrollSchedule with invalid frequency
     */

    @Test(expectedExceptions = ValidationException.class)
    public void testUpdatePayrollSchedule_withInvalidPSFrequency() throws Exception {
        Calendar cal = DateUtil.createCalendar(2008, Calendar.OCTOBER, 24);

        createAndSaveNewDefaultPayrollSchedule(PayrollFrequencyEnum.WEEKLY, cal, 1, 2);

        PayrollSchedule pSchedule = payrollScheduleComponent.getDefaultPayrollSchedule();

        pSchedule.setFrequency(PayrollFrequencyEnum.NOTSET);
        pSchedule.setPeriodStartDate(DateUtil.createDate(2008, Calendar.APRIL, 1));

        Date periodEndDate = DateUtil.createDate(2008, Calendar.MAY, 1);
        pSchedule.setNextPayPeriodEndDate(periodEndDate);
        Date payDate = DateUtil.createDate(2008, Calendar.MAY, 7);
        pSchedule.setPayDayOffset(DateUtil.findOffsetDays(periodEndDate, payDate));

        payrollScheduleComponent.updatePayrollSchedule(pSchedule);

    }

    @Test(enabled=false, expectedExceptions = ValidationException.class)
    public void testUpdatePayrollScheduleInValidPayDayOffset() throws Exception {

        Calendar cal = DateUtil.createCalendar(2008, Calendar.OCTOBER, 24);

        PayrollSchedule pSchedule = createAndSaveNewDefaultPayrollSchedule(PayrollFrequencyEnum.BIWEEKLY, cal, 1, 2);

        pSchedule.setFrequency(PayrollFrequencyEnum.WEEKLY);
        pSchedule.setPeriodStartDate(DateUtil.createDate(2008, Calendar.APRIL, 2));

        Date periodEndDate = DateUtil.createDate(2008, Calendar.MAY, 1);
        pSchedule.setNextPayPeriodEndDate(periodEndDate);
        pSchedule.setPayDayOffset(-1);
        pSchedule.setPayrollRunDayOffset(-1);

        payrollScheduleComponent.updatePayrollSchedule(pSchedule);

    }

    @Test(expectedExceptions = ValidationException.class)
    public void testUpdatePayrollSchedule_withInvalidPayDate() throws Exception {

        Calendar cal = DateUtil.createCalendar(2008, Calendar.OCTOBER, 24);

        createAndSaveNewDefaultPayrollSchedule(PayrollFrequencyEnum.WEEKLY, cal, 1, 2);

        PayrollSchedule pSchedule = payrollScheduleComponent.getDefaultPayrollSchedule();

        pSchedule.setFrequency(PayrollFrequencyEnum.MONTHLY);
        pSchedule.setPeriodStartDate(DateUtil.createDate(2008, Calendar.APRIL, 2));

        Date periodEndDate1 = DateUtil.createDate(2008, Calendar.MAY, 1);
        pSchedule.setNextPayPeriodEndDate(periodEndDate1);
        pSchedule.setPayDayOffset(2);
        pSchedule.setPayrollRunDayOffset(-5);

        payrollScheduleComponent.updatePayrollSchedule(pSchedule);

    }

    @Test(enabled=false, groups={"broken"})
    public void testUpdatePayrollSchedule_withSemiMonthlyFrequency() throws Exception {

        Calendar cal = DateUtil.createCalendar(2009, Calendar.MAY, 12);
        createAndSaveNewDefaultPayrollSchedule(PayrollFrequencyEnum.BIWEEKLY, cal, 2, 3);

        PayrollSchedule pSchedule = payrollScheduleComponent.getDefaultPayrollSchedule();

        pSchedule.setFrequency(PayrollFrequencyEnum.SEMIMONTHLY);
        pSchedule.setPayPeriodEndDayofMonth1(8);
        pSchedule.setPayPeriodEndDayofMonth2(22);
        pSchedule.setPayrollRunDayOffset(1);
        pSchedule.setPayDayOffset(2);

        payrollScheduleComponent.updatePayrollSchedule(pSchedule);

        pSchedule = payrollScheduleComponent.getDefaultPayrollSchedule();
    }

    /**
     * payDate : The next upcoming Friday after the current day. payrollPayDateOffset : 3 days after the PeriodEndDate
     * frequency : Bi-Weekly (every 2 weeks) periodEndDate : The day before the run payroll date (Tues)
     * periodStartDate : Shown on calendar as Wed 2 weeks before per end date and pay frequency of bi-weekly.
     */
    private void validateDefaultPayrollSchedule(PayrollSchedule schedule)
    {
        AssertJUnit.assertNotNull(schedule);
        AssertJUnit.assertEquals("Payroll Run offset is not 1 days", new Integer(1), schedule.getPayrollRunDayOffset());
        AssertJUnit.assertEquals("PayDay offset is not 3 days", new Integer(3), schedule.getPayDayOffset());

        // Difference between start and end period date should be 2 weeks
        int noOfDays = DateUtil.findOffsetDays(schedule.getPeriodStartDate(), schedule.getNextPayPeriodEndDate());
        AssertJUnit.assertEquals("Difference between period start and end date is not 2 weeks", noOfDays, 13);
        AssertJUnit.assertEquals(PayrollFrequencyEnum.BIWEEKLY, schedule.getFrequency());
        AssertJUnit.assertEquals("PayPeriodEndDay1 should be zero", 0, schedule.getPayPeriodEndDayofMonth1());
        AssertJUnit.assertEquals("PayPeriodEndDay2 should be zero", 0, schedule.getPayPeriodEndDayofMonth2());
    }
}
