package com.intuit.sb.payroll.capability.workerpayments.impl;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.workerpayments.PayrollRunInfo;
import com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleTestUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;


public class PayrollScheduleDateUtilsTest
{
	/**
	 * Tests that when a weekend is hit, the date is rolled back properly.
	 * 
	 * @throws Exception
	 */
	@Test(enabled=false, groups={"broken"})
	public void testAdjustDateForOffsetWeekend() throws Exception {
		// Sets up a weekend scenario
        Calendar date = DateUtil.createCalendar(2008, Calendar.MAY, 29);
        Calendar expectedDate = DateUtil.createCalendar(2008, Calendar.MAY, 23);
		Calendar actualDate = DateUtil.adjustDateForOffset(date, -4);
		AssertJUnit.assertEquals(expectedDate, actualDate);	
	}
	
	/**
	 * Tests that when there is no weekend, the date stays put.
	 * 
	 * @throws Exception
	 */
	@Test(enabled=false, groups={"broken"})
	public void testAdjustDateForOffsetWeekday() throws Exception {
		// Sets up a weekday scenario
        Calendar date = DateUtil.createCalendar(2008, Calendar.MAY, 30);
        Calendar expectedDate = DateUtil.createCalendar(2008, Calendar.MAY, 26);
		Calendar actualDate = DateUtil.adjustDateForOffset(date, -4);
		AssertJUnit.assertEquals(expectedDate, actualDate);	
	}
	
	/**
	 * Testing input defensive measures.  Null inputs are verboten.
	 * 
	 * @throws Exception
	 */
	@Test(expectedExceptions=IllegalArgumentException.class)
	public void testAdjustDateBackForOffsetNullCalendar() throws Exception {
		DateUtil.adjustDateForOffset(null, 1);
	}
	
	
    /**
     * This test method finds the Previous date from a start date based on the Daily Frequency
     */
    @Test
    public void testGetPreviousDateOnFrequencyDaily()
    {
        Calendar fromDate = DateUtil.createCalendar(2008, Calendar.MAY, 29);
        Calendar expectedDate = DateUtil.createCalendar(2008, Calendar.MAY, 28);

        Calendar returned = PayrollScheduleDateUtils.getPayPeriodStartDate(fromDate, PayrollFrequencyEnum.DAILY);

        AssertJUnit.assertEquals("NextDate on Daily Frequency is not as expected", expectedDate, returned);

    }

    /**
     * This test method finds the Previous date from a start date based on the Weekly Frequency
     */
    @Test
    public void testGetPreviousDateOnFrequencyWeekly()
    {
        Calendar fromDate = DateUtil.createCalendar(2008, Calendar.MAY, 2);
        Calendar expectedDate = DateUtil.createCalendar(2008, Calendar.APRIL, 26);

        Calendar returned = PayrollScheduleDateUtils.getPayPeriodStartDate(fromDate, PayrollFrequencyEnum.WEEKLY);

        AssertJUnit.assertEquals("NextDate on Weekly Frequency is not as expected", expectedDate, returned);
    }

    /**
     * This test method finds the Previous date from a start date based on the BiWeekly Frequency
     */
    @Test
    public void testGetPreviousDateOnFrequencyBiweekly()
    {
        Calendar fromDate = DateUtil.createCalendar(2008, Calendar.MAY, 9);
        Calendar expectedDate = DateUtil.createCalendar(2008, Calendar.APRIL, 26);

        Calendar returned = PayrollScheduleDateUtils.getPayPeriodStartDate(fromDate, PayrollFrequencyEnum.BIWEEKLY);

        AssertJUnit.assertEquals("NextDate on BiWeekly Frequency is not as expected", expectedDate, returned);
    }

    /**
     * This test method finds the Previous date from a start date based on the Monthly Frequency
     */
    @Test
    public void testGetPreviousDateOnFrequencyMonthly()
    {
        Calendar fromDate = DateUtil.createCalendar(2008, Calendar.MAY, 27);
        Calendar expectedDate = DateUtil.createCalendar(2008, Calendar.APRIL, 28);

        Calendar returned = PayrollScheduleDateUtils.getPayPeriodStartDate(fromDate, PayrollFrequencyEnum.MONTHLY);

        AssertJUnit.assertEquals("NextDate on Monthly Frequency is not as expected", expectedDate, returned);
    }

    /**
     * This test method finds the next date from a start date based on the Daily Frequency
     */
    @Test
    public void testGetNextDateOnFrequencyDaily()
    {
        Calendar fromDate = DateUtil.createCalendar(2008, Calendar.MAY, 28);
        Calendar expectedDate = DateUtil.createCalendar(2008, Calendar.MAY, 29);

        Calendar returned = PayrollScheduleDateUtils.getPayPeriodEndDate(fromDate, PayrollFrequencyEnum.DAILY);

        AssertJUnit.assertEquals("NextDate on Daily Frequency is not as expected", expectedDate, returned);

    }

    /**
     * This test method finds the next date from a start date based on the Weekly Frequency
     */
    @Test
    public void testGetNextDateOnFrequencyWeekly()
    {
        Calendar fromDate = DateUtil.createCalendar(2008, Calendar.APRIL, 28);
        Calendar expectedDate = DateUtil.createCalendar(2008, Calendar.MAY, 4);

        Calendar returned = PayrollScheduleDateUtils.getPayPeriodEndDate(fromDate, PayrollFrequencyEnum.WEEKLY);

        AssertJUnit.assertEquals("NextDate on Weekly Frequency is not as expected", expectedDate, returned);
    }

    /**
     * This test method finds the next date from a start date based on the BiWeekly Frequency
     */
    @Test
    public void testGetNextDateOnFrequencyBiweekly()
    {
        Calendar fromDate = DateUtil.createCalendar(2008, Calendar.APRIL, 28);
        Calendar expectedDate = DateUtil.createCalendar(2008, Calendar.MAY, 11);

        Calendar returned = PayrollScheduleDateUtils.getPayPeriodEndDate(fromDate, PayrollFrequencyEnum.BIWEEKLY);

        AssertJUnit.assertEquals("NextDate on BiWeekly Frequency is not as expected", expectedDate, returned);
    }

    /**
     * This test method finds the next date from a start date based on the Monthly Frequency
     */
    @Test
    public void testGetNextDateOnFrequencyMonthly()
    {
        Calendar fromDate = DateUtil.createCalendar(2008, Calendar.APRIL, 28);
        Calendar expectedDate = DateUtil.createCalendar(2008, Calendar.MAY, 27);

        Calendar returned = PayrollScheduleDateUtils.getPayPeriodEndDate(fromDate, PayrollFrequencyEnum.MONTHLY);

        AssertJUnit.assertEquals("NextDate on Monthly Frequency is not as expected", expectedDate, returned);
    }

    /**
     * This test method finds the next date from a start date based on the Quaterly Frequency
     */
    @Test
    public void testGetNextDateOnFrequencyQuaterly()
    {
        Calendar fromDate = DateUtil.createCalendar(2008, Calendar.APRIL, 28);
        Calendar expectedDate = DateUtil.createCalendar(2008, Calendar.JULY, 27); // 26 and 27 are Sat & Sunday

        Calendar returned = PayrollScheduleDateUtils.getPayPeriodEndDate(fromDate, PayrollFrequencyEnum.QUARTERLY);

        AssertJUnit.assertEquals("NextDate on Quaterly Frequency is not as expected", expectedDate, returned);
    }

    /**
     * This test method finds the next date from a start date based on the Yearly Frequency
     */
    @Test
    public void testGetNextDateOnFrequencyYearly()

    {
        Calendar fromDate = DateUtil.createCalendar(2008, Calendar.MAY, 28);
        Calendar expectedDate = DateUtil.createCalendar(2009, Calendar.MAY, 27);

        Calendar returned = PayrollScheduleDateUtils.getPayPeriodEndDate(fromDate, PayrollFrequencyEnum.ANNUALLY);

        AssertJUnit.assertEquals("NextDate on Yearly Frequency is not as expected", expectedDate, returned);
    }
    
    /**
     * Tests that the generation of expected PayrollRunInfos works.
     * @throws Exception
     */
    @Test(enabled=false, groups={"broken"})
    public void testGetPayrollRunInfosForWeeklySchedule() throws Exception {
    	//[FIXME] Refactor test for the refactored api getPayrollRunInfosForSchedule
    	Calendar cal = DateUtil.createCalendar(2008, Calendar.OCTOBER, 24);
    	PayrollSchedule payrollSchedule = PayrollScheduleTestUtils.createPayrollSchedule(cal,0, PayrollFrequencyEnum.WEEKLY,0,2);
		int numberOfPayrollRuns = 3;
		Calendar fromDate = PayrollScheduleDateUtils.getPayPeriodEndDate(cal, PayrollFrequencyEnum.WEEKLY);
		fromDate.add(Calendar.DATE, 1);
		List<PayrollRunInfo> payrollRunInfos = PayrollScheduleDateUtils.getPayrollRunInfosForSchedule(payrollSchedule, fromDate, null);
		AssertJUnit.assertEquals("Number of PayrollRunInfos didn't match.", numberOfPayrollRuns, payrollRunInfos.size());
		List<PayrollRunInfo> expected = createExpectedWeeklyPayrollRunInfos();
		AssertJUnit.assertEquals("PayrollRun Infos do not match", expected, payrollRunInfos);
    }
    
    @Test(enabled=false, groups={"broken"})
    public void testGetPayrollRunInfosForBiWeeklySchedule() throws Exception {
    	Calendar cal = DateUtil.createCalendar(2008, Calendar.OCTOBER, 24);
    	PayrollSchedule payrollSchedule = PayrollScheduleTestUtils.createPayrollSchedule(cal,0, PayrollFrequencyEnum.BIWEEKLY,0,2);
		int numberOfPayrollRuns = 3;
		Calendar fromDate = PayrollScheduleDateUtils.getPayPeriodEndDate(cal, PayrollFrequencyEnum.BIWEEKLY);
		fromDate.add(Calendar.DATE, 1);
		List<PayrollRunInfo> payrollRunInfos = PayrollScheduleDateUtils.getPayrollRunInfosForSchedule(payrollSchedule, fromDate, DateUtil.createCalendar(2008, 0, 1));
		AssertJUnit.assertEquals("Number of PayrollRunInfos didn't match.", numberOfPayrollRuns, payrollRunInfos.size());
		List<PayrollRunInfo> expected = createExpectedBiWeeklyPayrollRunInfos();
		AssertJUnit.assertEquals("PayrollRun Infos do not match", expected, payrollRunInfos);    	
    }

    /**
     * Tests that basic filtering of Existing Payruns vs. PayrollRunInfos works.
     * 
     * @throws Exception
     */
    @Test
    public void testFilterCompletedRuns() throws Exception  {
    	List<PayrollRunInfo> allRuns = createExpectedWeeklyPayrollRunInfos();
    	List<PayrollRun> existingRuns = createExistingPayrollRuns();
    	List<PayrollRunInfo> filteredPayrollRuns = PayrollScheduleDateUtils.filterCompletedPayrollRuns(allRuns, existingRuns);
    	int expectedSize = 1;
    	AssertJUnit.assertEquals("Unexpected Size for filtered Payruns.", expectedSize, filteredPayrollRuns.size());
    	
		DateRange expectedDateRange = new DateRange(DateUtil.createDate(2008, Calendar.NOVEMBER, 14), 
				DateUtil.createDate(2008, Calendar.NOVEMBER, 20));
		AssertJUnit.assertEquals("Unexpected DateRage in result.", expectedDateRange, filteredPayrollRuns.get(0).getPayPeriod());
    }
    
	private List<PayrollRunInfo> createExpectedWeeklyPayrollRunInfos() {
		List<PayrollRunInfo> expected = new ArrayList<PayrollRunInfo>();
		// 1
		DateRange dateRange1 = new DateRange(DateUtil.createDate(2008, Calendar.OCTOBER, 31), 
																DateUtil.createDate(2008, Calendar.NOVEMBER, 6));
		expected.add(new PayrollRunInfo(dateRange1, 
													DateUtil.createDate(2008, Calendar.NOVEMBER, 8), // pay date
													DateUtil.createDate(2008, Calendar.NOVEMBER, 6), // pay run date
													PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		//2
		DateRange dateRange2 = new DateRange(DateUtil.createDate(2008, Calendar.NOVEMBER, 7), 
																DateUtil.createDate(2008, Calendar.NOVEMBER, 13));
		expected.add(new PayrollRunInfo(dateRange2, 
													DateUtil.createDate(2008, Calendar.NOVEMBER, 15), // pay date
													DateUtil.createDate(2008, Calendar.NOVEMBER, 13), // pay run date
													PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		//3
		DateRange dateRange3 = new DateRange(DateUtil.createDate(2008, Calendar.NOVEMBER, 14), 
																DateUtil.createDate(2008, Calendar.NOVEMBER, 20));
		expected.add(new PayrollRunInfo(dateRange3, 
													DateUtil.createDate(2008, Calendar.NOVEMBER, 22), // pay date
													DateUtil.createDate(2008, Calendar.NOVEMBER, 20), // pay run date
													PayrollFrequencyEnum.WEEKLY, PayrollRunTypeEnum.SCHEDULED));
		return expected;
	}
	
	private List<PayrollRunInfo> createExpectedBiWeeklyPayrollRunInfos() {
		List<PayrollRunInfo> expected = new ArrayList<PayrollRunInfo>();
		// 1
		DateRange dateRange1 = new DateRange(DateUtil.createDate(2008, Calendar.NOVEMBER, 7), 
																DateUtil.createDate(2008, Calendar.NOVEMBER, 20));
		expected.add(new PayrollRunInfo(dateRange1, 
													DateUtil.createDate(2008, Calendar.NOVEMBER, 22), // pay date
													DateUtil.createDate(2008, Calendar.NOVEMBER, 20), // pay run date
													PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		//2
		DateRange dateRange2 = new DateRange(DateUtil.createDate(2008, Calendar.NOVEMBER, 21), 
																DateUtil.createDate(2008, Calendar.DECEMBER, 4));
		expected.add(new PayrollRunInfo(dateRange2, 
													DateUtil.createDate(2008, Calendar.DECEMBER, 6), // pay date
													DateUtil.createDate(2008, Calendar.DECEMBER, 4), // pay run date
													PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		//3
		DateRange dateRange3 = new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 5), 
																DateUtil.createDate(2008, Calendar.DECEMBER, 18));
		expected.add(new PayrollRunInfo(dateRange3, 
													DateUtil.createDate(2008, Calendar.DECEMBER, 20), // pay date
													DateUtil.createDate(2008, Calendar.DECEMBER, 18), // pay run date
													PayrollFrequencyEnum.BIWEEKLY, PayrollRunTypeEnum.SCHEDULED));
		return expected;
	}

	private List<PayrollRun> createExistingPayrollRuns() {
		List<PayrollRun> results = new ArrayList<PayrollRun>();
		// 1
		DateRange dateRange1 = new DateRange(DateUtil.createDate(2008, Calendar.OCTOBER, 31), 
																DateUtil.createDate(2008, Calendar.NOVEMBER, 6));
		PayrollRun payRun1 = new PayrollRun();
		payRun1.setPayPeriod(dateRange1);
		payRun1.setPayrollFrequency(PayrollFrequencyEnum.WEEKLY);
		payRun1.setPartialPeriod(false);
		
		//2
		DateRange dateRange2 = new DateRange(DateUtil.createDate(2008, Calendar.NOVEMBER, 7), 
																DateUtil.createDate(2008, Calendar.NOVEMBER, 13));
		PayrollRun payRun2 = new PayrollRun();
		payRun2.setPayPeriod(dateRange2);
		payRun2.setPayrollFrequency(PayrollFrequencyEnum.WEEKLY);
		payRun2.setPartialPeriod(false);
		
		//3 - to try to confuse, simulate an ad hoc.
		DateRange dateRange3 = new DateRange(DateUtil.createDate(2008, Calendar.NOVEMBER, 14), 
				DateUtil.createDate(2008, Calendar.NOVEMBER, 20));
		PayrollRun payRun3 = new PayrollRun();
		payRun3.setPayPeriod(dateRange3);
		payRun3.setPayrollFrequency(PayrollFrequencyEnum.RARELY);
		payRun3.setPartialPeriod(false);
		
		results.add(payRun1);
		results.add(payRun2);
		results.add(payRun3);
		
		return results;
	}
}
