package com.intuit.sb.payroll.services.benefits.impl;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.benefits.BenefitsComponent;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.domain.Identifiable;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.DatumType2Enum;
import com.intuit.sb.payroll.entity.model.HealthBenefitDtl;
import com.intuit.sb.payroll.entity.model.HealthBenefitTypeEnum;
import com.intuit.sb.payroll.services.benefits.HealthBenefit;
import com.intuit.sb.payroll.services.benefits.HealthBenefitTemplate;
import com.intuit.sb.payroll.services.benefits.HealthBenefitType;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.common.impl.ObjectHandleMapper;

public class BenefitMapper {

	private static BenefitsComponent benefitsComponent = 
		ServiceLocator.getInstance().getService(BenefitsComponent.JNDI_NAME);

	/**
	 * Map the HealthBenefitDtls to HealthBenefits.
	 * @param HealthBenefitDtl set
	 * @return HealthBenefit dto array
	 */
	public static HealthBenefit[] mapToHealthBenefits (Set<HealthBenefitDtl> benefitDtls) throws ValidationException {
		
		HealthBenefit[] benefitArray = new HealthBenefit[benefitDtls.size()];
		Set<HealthBenefit> benefitSet = new HashSet<HealthBenefit>();
		
		
		for(HealthBenefitDtl detail : benefitDtls) {
			HealthBenefit benefitItem = BenefitMapper.mapToHealthBenefit(detail);
			benefitSet.add(benefitItem);
		}
		return benefitSet.toArray(benefitArray);
	}
	
	/**
	 * Map the HealthBenefitDtl to HealthBenefit
	 * @param HealthBenefitDtl item
	 * @return HealthBenefit dto 
	 */
	public static HealthBenefit mapToHealthBenefit (HealthBenefitDtl dtl) throws ValidationException {
		HealthBenefit benefit = new HealthBenefit();
		benefit.setObjectHandle(new ObjectHandle(dtl.getId(), dtl.getVersion()));
		benefit.setType(mapToHealthBenefitType(dtl.getBenefitType()));
		benefit.setName(dtl.getDescription());
		Datum datum = dtl.getRate();
		if (null != datum) {
		    benefit.setRate(datum.getValue1AsUSDMoneyAmount());
		}
		
		return benefit;
	}

	/**
	 * Map the HealthBenefits to HealthBenefitDtls.
	 * @param HealthBenefit dto array
	 * @return HealthBenefitDtl set
	 * @throws ServiceFailureException 
	 */
	public static List<HealthBenefitDtl> mapToHealthBenefitDtls (HealthBenefit[] benefits) throws ValidationException, ServiceFailureException {
		List<HealthBenefitDtl> benefitSet = new ArrayList<HealthBenefitDtl>();
		
		for(HealthBenefit benefit : benefits) {
			HealthBenefitDtl benefitDtl = mapToHealthBenefitDtl(benefit);
			benefitSet.add(benefitDtl);
		}
		return benefitSet;
	}
	
	/**
	 * Map the HealthBenefit to HealthBenefitDtl.
	 * @param HealthBenefit item
	 * @return HealthBenefitDtl item
	 * @throws ServiceFailureException 
	 */
	public static HealthBenefitDtl mapToHealthBenefitDtl (HealthBenefit benefit) throws ValidationException, ServiceFailureException {
		HealthBenefitDtl benefitDtl = null;
		if (null == benefit.getObjectHandle()) {
		    HealthBenefitTypeEnum benefitType = mapToHealthBenefitTypeEnum(benefit.getType());
		    benefitDtl = benefitsComponent.createHealthBenefitDtl(benefitType);
		} else {
            Identifiable identifiable = 
                ObjectHandleMapper.mapToIdentifiable(benefit.getObjectHandle());
            benefitDtl = benefitsComponent.findHealthBenefitDtl(identifiable);
		}
		benefitDtl.setBenefitType(mapToHealthBenefitTypeEnum(benefit.getType()));
		String name = benefit.getName();
		if (name != null && name.length() > 0) {
			benefitDtl.setDescription(name);
		}
		Datum datum = benefitDtl.getRate();
		if (null == datum) {
			datum = new Datum();
			benefitDtl.setRate(datum);
		}
		datum.setValue1AsUSDMoneyAmount(benefit.getRate());
		datum.setType2(DatumType2Enum.PERPAYCHECK);
		
		return benefitDtl;
	}

	
	public static HealthBenefitTemplate mapToHealthBenefitTemplate(com.intuit.sb.payroll.entity.model.HealthBenefit benefit) {
		HealthBenefitTemplate template = new HealthBenefitTemplate();
		template.setObjectHandle(new ObjectHandle(benefit.getId(), benefit.getVersion()));
		template.setName(benefit.getDescription());
		template.setType(mapToHealthBenefitType(benefit.getBenefitType()));
		Datum datum = benefit.getRate();
		if (null != datum) {
		    template.setRate(datum.getValue1AsUSDMoneyAmount());
		}
		
		return template;
	}
	
	public static HealthBenefitTypeEnum mapToHealthBenefitTypeEnum(HealthBenefitType type) {
		
		HealthBenefitTypeEnum benefitEnum = null;
		if (HealthBenefitType.HEALTH.equals(type)) {
			benefitEnum = HealthBenefitTypeEnum.SEC125HEALTH;
		}
		else if (HealthBenefitType.DENTAL.equals(type)) {
			benefitEnum = HealthBenefitTypeEnum.SEC125DENTAL;
		}
		else if (HealthBenefitType.VISION.equals(type)) {
			benefitEnum = HealthBenefitTypeEnum.SEC125VISION;
		}
		else if (HealthBenefitType.FLEXIBLE_SPENDING.equals(type)) {
			benefitEnum = HealthBenefitTypeEnum.SEC125FSA;
		}
		else if (HealthBenefitType.DEPENDENT_CARE.equals(type)) {
			benefitEnum = HealthBenefitTypeEnum.SEC125DEPCARE;
		}
		
		return benefitEnum;
	}
	
	public static HealthBenefitType mapToHealthBenefitType(HealthBenefitTypeEnum enumType) {
		
		HealthBenefitType benefitType = null;
		if (enumType == HealthBenefitTypeEnum.SEC125HEALTH) {
			benefitType = HealthBenefitType.HEALTH;
		}
		else if (enumType == HealthBenefitTypeEnum.SEC125DENTAL) {
			benefitType = HealthBenefitType.DENTAL;
		}
		else if (enumType == HealthBenefitTypeEnum.SEC125VISION) {
			benefitType = HealthBenefitType.VISION;
		}
		else if (enumType == HealthBenefitTypeEnum.SEC125FSA) {
			benefitType = HealthBenefitType.FLEXIBLE_SPENDING;
		}
		else if (enumType == HealthBenefitTypeEnum.SEC125DEPCARE) {
			benefitType = HealthBenefitType.DEPENDENT_CARE;
		}
		
		return benefitType;
	}

}
