package com.intuit.sb.payroll.services.common.impl;

import com.intuit.sb.cdm.basetypes.EmailAddress;
import com.intuit.sb.cdm.basetypes.TelephoneNumber;
import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.capability.employer.EmployerPTSWorkflowComponent;
import com.intuit.sb.payroll.capability.pts.statehandler.StatusUpdateHandlerUtil;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.model.EmployerContactPerson;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.PersonName;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.pts.entity.PTSContactPerson;
import com.intuit.sb.payroll.pts.entity.PTSContactPersonTypeEnum;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.employer.EmployerContact;
import com.intuit.sb.payroll.services.employer.EmployerContactType;
import com.intuit.sb.payroll.services.employer.PrimaryPrincipal;
import com.intuit.sb.payroll.us.capability.address.impl.USPostalAddressImpl;

/**
 * Mapper class to handle EmployerContact Mappings between the DTO/Service layer and the Entity Model.
 *
 * $Author$
 * $Revision$
 * $DateTime$
 */
public class EmployerContactMapper {

    /**
     * Maps from an entity EmployerContactPerson to a DTO EmployerContact. Null input yields null output.
     *
     * @param EmployerContact
     * @return EmployerContact dto
     */
    public static EmployerContact mapEmployerContact(PTSContactPerson entityContact) {

        if (null != entityContact) {
            EmployerContactPerson empContactPerson = entityContact.getEmployerContact();

            EmployerContact dtoContact = new EmployerContact();
            
            dtoContact.setObjectHandle(new ObjectHandle(entityContact.getId(), entityContact.getVersion()));

            PersonName personName = empContactPerson.getPersonName();
            if (null != personName) {
                dtoContact.setFirstName(personName.getFirstName());
                dtoContact.setLastName(personName.getLastName());
                dtoContact.setMiddleName(personName.getMiddleName());
            }

            dtoContact.setOwner(empContactPerson.isOwner());

            dtoContact.setEmployerContactType(mapEmployerContactType(entityContact.getContactPersonType()));

            dtoContact.setTitle(empContactPerson.getRoleTitle());

            TelephoneNumber workPhone = empContactPerson.getWorkPhone();
            if (null != workPhone) {
                dtoContact.setPhone(workPhone.getFreeFormNumber());
                dtoContact.setExtn(workPhone.getExtension());
            }

            TelephoneNumber faxPhone = empContactPerson.getFaxPhone();
            if (null != faxPhone) {
                dtoContact.setFax(faxPhone.getFreeFormNumber());
                // no extension for fax phone
            }

            EmailAddress emailAddress = empContactPerson.getEmailAddress();
            if (null != emailAddress) {
                dtoContact.setEmail(emailAddress.getAddress());
            }
            empContactPerson.getEmailAddress().setAddress(dtoContact.getEmail());

            dtoContact.setAddress(PhysicalAddressMapper.mapAddress(AddressConvertor.getUSFromGlobal(empContactPerson
                    .getMailingAddress(), new USPostalAddressImpl())));

            return dtoContact;
        } else {
            return null;
        }
    }



    /**
     * Maps from a DTO EmployerContact to an entity ContacPerson. Null input yields null output.
     *
     * @param dto
     * @return
     * @throws ServiceFailureException
     */
    public static com.intuit.sb.payroll.pts.entity.PTSContactPerson mapEmployerContact(EmployerContact dtoContact)
            throws ServiceFailureException {

        if (null != dtoContact) {

            EmployerPTSWorkflowComponent employerPTSWorkflowComponent = ServiceLocator.getInstance().getService(EmployerPTSWorkflowComponent.JNDI_NAME);

            com.intuit.sb.payroll.entity.model.EmployerContactPerson entityContact = null;
            
            // are we dealing with a new contact or existing?
            if (null == dtoContact.getObjectHandle() || dtoContact.getObjectHandle().getId() == 0) {
                entityContact = new com.intuit.sb.payroll.entity.model.EmployerContactPerson();
            } else {
                PTSContactPerson entityPTSContact = employerPTSWorkflowComponent.findEmployerContactPerson(dtoContact.getObjectHandle().getId());
                entityContact = entityPTSContact.getEmployerContact();
            }

            if (null == entityContact.getPersonName()) {
                entityContact.setPersonName(new PersonName());
            }
            entityContact.getPersonName().setFirstName(dtoContact.getFirstName());
            entityContact.getPersonName().setLastName(dtoContact.getLastName());
            entityContact.getPersonName().setMiddleName(dtoContact.getMiddleName());

            entityContact.setRoleTitle(dtoContact.getTitle());

            entityContact.setOwner(dtoContact.isOwner());

            // [as] the dtoEntity does not specify which phone, I have chosen workPhone
            if (null == entityContact.getWorkPhone()) {
                entityContact.setWorkPhone(new TelephoneNumber());
            }
            entityContact.getWorkPhone().setFreeFormNumber(dtoContact.getPhone());
            entityContact.getWorkPhone().setExtension(dtoContact.getExtn());

            if (null == entityContact.getFaxPhone()) {
                entityContact.setFaxPhone(new TelephoneNumber());
            }
            entityContact.getFaxPhone().setFreeFormNumber(dtoContact.getFax());
            // no extension for fax phones

            if (null == entityContact.getEmailAddress()) {
                entityContact.setEmailAddress(new EmailAddress());
            }
            entityContact.getEmailAddress().setAddress(dtoContact.getEmail());

            if (null != dtoContact.getAddress()) {
                USPostalAddress usAddress = new USPostalAddressImpl();
                PhysicalAddressMapper.mapAddress(dtoContact.getAddress(), usAddress);
                GlobalPostalAddress entityPostalAddress = AddressConvertor.getGlobalFromUS(usAddress,
                        new GlobalPostalAddress());
                entityContact.setMailingAddress(entityPostalAddress);
            }

            PTSContactPerson ptsentityContact;

            if (null == dtoContact.getObjectHandle() || dtoContact.getObjectHandle().getId() == 0) {
                ptsentityContact = new com.intuit.sb.payroll.pts.entity.PTSContactPerson();
            } else {
                ptsentityContact = employerPTSWorkflowComponent.findEmployerContactPerson(dtoContact.getObjectHandle().getId());
            }
            ptsentityContact.setContactPersonType(mapEmployerContactPersonType(dtoContact.getEmployerContactType()));
            ptsentityContact.setEmployerContact(entityContact);
            return ptsentityContact;
        } else {
            return null;
        }
    }

    private static PTSContactPersonTypeEnum mapEmployerContactPersonType(EmployerContactType employerContactType){
        if (employerContactType == null){
            return null;
        }

        if (employerContactType.equals(EmployerContactType.PrimaryContact)){
            return PTSContactPersonTypeEnum.PRIMARY_CONTACT;
        }else if (employerContactType.equals(EmployerContactType.SecondaryContact)){
            return PTSContactPersonTypeEnum.SECONDARY_CONTACT;
        } else if (employerContactType.equals(EmployerContactType.PayrollAdminContact)){
            return PTSContactPersonTypeEnum.PAYROLLADMIN_CONTACT;
        }else if (employerContactType.equals(EmployerContactType.PrimaryPrincipal)){
            return PTSContactPersonTypeEnum.PRIMARY_PRINCIPAL;
        } else{
            return null;
        }

    }

    private static EmployerContactType mapEmployerContactType(
            PTSContactPersonTypeEnum contactPersonType) {
        if (contactPersonType == null){
            return null;
        }
        if (contactPersonType.equals(PTSContactPersonTypeEnum.PRIMARY_CONTACT)){
            return EmployerContactType.PrimaryContact;
        }else if (contactPersonType.equals(PTSContactPersonTypeEnum.SECONDARY_CONTACT)){
            return EmployerContactType.SecondaryContact;
        }else if (contactPersonType.equals(PTSContactPersonTypeEnum.PAYROLLADMIN_CONTACT)){
            return EmployerContactType.PayrollAdminContact;
        }else if (contactPersonType.equals(PTSContactPersonTypeEnum.PRIMARY_PRINCIPAL)){
            return EmployerContactType.PrimaryPrincipal;
        }else {
            return null;
        }
    }
    
    public static PrimaryPrincipal mapToPrimaryPrincipal(PTSContactPerson ptsContactPerson) {
    	
    	if(ptsContactPerson == null) {
    		return null;
    	}
    	EmployerPTSWorkflowComponent employerPTSWorkflowComponent = ServiceLocator.getInstance().getService(EmployerPTSWorkflowComponent.JNDI_NAME);
    	PTSEmployer ptsEmployer = employerPTSWorkflowComponent.getPTSEmployer(); 
    	
    	PrimaryPrincipal primaryPrincipal = new PrimaryPrincipal();
    	primaryPrincipal.setFirstName(ptsContactPerson.getEmployerContact().getPersonName().getFirstName());
    	primaryPrincipal.setLastName(ptsContactPerson.getEmployerContact().getPersonName().getLastName());
    	primaryPrincipal.setMiddleName(ptsContactPerson.getEmployerContact().getPersonName().getMiddleName());
    	primaryPrincipal.setTitle(ptsContactPerson.getEmployerContact().getRoleTitle());
    	primaryPrincipal.setPhoneNumber(ptsContactPerson.getEmployerContact().getWorkPhone().getFreeFormNumber());
    	primaryPrincipal.setPhoneExtn(ptsContactPerson.getEmployerContact().getWorkPhone().getExtension());
    	primaryPrincipal.setEmail(ptsContactPerson.getEmployerContact().getEmailAddress().getAddress());
    	primaryPrincipal.setObjectHandle(new ObjectHandle(ptsContactPerson.getId(), ptsContactPerson.getVersion()));

    	//PrimaryPrincipal is not editable once the company is added to PTS
    	boolean isCompanyAddedToPTS = false;
    	isCompanyAddedToPTS = StatusUpdateHandlerUtil.isEntityAddedToPTS(ptsEmployer.getEmployer());
    	
    	primaryPrincipal.setLocked(isCompanyAddedToPTS);
    	return primaryPrincipal;
    }
    
    public static PTSContactPerson mapToPTSContactPerson(PrimaryPrincipal primaryPrincipal) throws ServiceFailureException {
    	
    	EmployerPTSWorkflowComponent employerPTSWorkflowComponent = ServiceLocator.getInstance().getService(EmployerPTSWorkflowComponent.JNDI_NAME);
    	EmployerComponent employerComponent = ServiceLocator.getInstance().getService(EmployerComponent.JNDI_NAME);
    	
    	PTSContactPerson ptsentityContact = null;
    	EmployerContactPerson entityContact = null;
    	
    	for (PTSContactPerson ptsContactPerson : employerPTSWorkflowComponent.getPTSEmployer().getPTSContactPersons()) {
    		if(ptsContactPerson.getContactPersonType() == PTSContactPersonTypeEnum.PRIMARY_PRINCIPAL) {
    			ptsentityContact = ptsContactPerson;
    			break;
    		}
    	}
    	if(ptsentityContact == null) {
    		ptsentityContact = new com.intuit.sb.payroll.pts.entity.PTSContactPerson();
    		entityContact = new EmployerContactPerson();
    		entityContact.setPersonName(new PersonName());
    		entityContact.setWorkPhone(new TelephoneNumber());
    		entityContact.setEmailAddress(new EmailAddress());
    	}
    	else {
    		entityContact = ptsentityContact.getEmployerContact();
    	}
    	
        entityContact.getPersonName().setFirstName(primaryPrincipal.getFirstName());
        entityContact.getPersonName().setLastName(primaryPrincipal.getLastName());
        entityContact.getPersonName().setMiddleName(primaryPrincipal.getMiddleName());
        entityContact.setRoleTitle(primaryPrincipal.getTitle());
        entityContact.getWorkPhone().setFreeFormNumber(primaryPrincipal.getPhoneNumber());
        entityContact.getWorkPhone().setExtension(primaryPrincipal.getPhoneExtn());        
        entityContact.getEmailAddress().setAddress(primaryPrincipal.getEmail());

        // TODO: Primary principal is also an employerContact, Since Primary principal does not have an address it is set to 
        // legalAddress. Fix by having a separate Primary Principal Entity
        entityContact.setMailingAddress(employerComponent.getEmployer().getLegalAddress());
        //Is owner is required for mapping during retrieving purpose
        entityContact.setOwner(false);
        
        ptsentityContact.setContactPersonType(PTSContactPersonTypeEnum.PRIMARY_PRINCIPAL);
        ptsentityContact.setEmployerContact(entityContact);
        return ptsentityContact;
    
    }
}
