package com.intuit.sb.payroll.services.employer;

import javax.ejb.Remote;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.services.common.BankAccount;
import com.intuit.sb.payroll.services.common.UpdateWorkLocationsResponse;
import com.intuit.sb.payroll.services.common.WorkLocation;
import com.intuit.sb.payroll.services.worker.Worker;

/**
 * This interface contains apis for managing or retrieving Employer specific information
 *  
 * 
 * $Author$ 
 * $Revision$
 * $DateTime$
 * 
 */
@Remote
public interface EmployerService
{
	public static final String JNDI_NAME = "EmployerService/remote";
	
	/**
	 * Provides a listing of all the Work Locations for the current employer.
	 * @return the employer's places of business. Can be empty.
	 * @throws ServiceFailureException if there is a problem with the database etc...
	 */
	public WorkLocation[] getWorkLocations() throws ServiceFailureException, ValidationException;

    /**
     * Add the given WorkLocations to this company.
     * @param workLocations
     * @return an array containing all the WorkLocations
     */
    public UpdateWorkLocationsResponse addWorkLocations(WorkLocation[] workLocations) throws ServiceFailureException, ValidationException;
    
    /**
     * Update the given WorkLocations.  Each work location must have a uniqueId.
     * @param workLocations
     * @return an array containing all the WorkLocations
     */
    public UpdateWorkLocationsResponse updateWorkLocations(WorkLocation[] workLocations) throws ServiceFailureException, ValidationException;
    
    /**
     * Delete the given WorkLocations.  Each work location must have a uniqueId.
     * @param workLocations
     */
    public WorkLocation[] deleteWorkLocations(WorkLocation[] workLocations) throws ServiceFailureException, ValidationException;
    
	/**
	 * Assigns the workLocation passed in for the worker in the db
	 * First key for the HashMap is the workerId and the data is the workLocationId.
	 * If any worker or workLocation is not found the transaction is not committed and a ValidationExceptio is thrown 
	 * @param workerLocations
	 * @return array of updated Workers
     * @throws ValidationException
	 */
	public Worker[] assignWorkLocationsToWorkers(WorkerLocation[] workerLocations)
		throws WorkflowException, ServiceFailureException, ValidationException;
    
	/**
	 * Informs the server that an application session is loading.
	 * The server will create a new Company for the user context
	 * if required and return a basket of initial goodies for the app.
	 * @return loadApplicationResponse 
	 * @throws ValidationException 
	 * @throws ServiceFailureException 
	 */
	public LoadApplicationResponse loadApplication() throws ValidationException, ServiceFailureException;

    /****************** Company Info related ****************************/
		 
	/**
	 * saves the CompanyInfo
	 */
	public void saveCompanyInfo(EmployerInfo  info) throws ServiceFailureException, ValidationException;
	
    /****************** Employer Info related ****************************/
	/**
	 * Returns the company information like the Address and company name etc.
	 * @return the EmployerInfo
	 */
	public EmployerInfo getEmployerInfo() throws ServiceFailureException, ValidationException;
	 
	/**
	 * saves the EmployerInfo
	 */
	public SaveEmployerInfoResponse saveEmployerInfo(SaveEmployerInfoRequest infoRequest) throws ServiceFailureException, ValidationException;
	
	/******************** Employer Contact related ************************/
	
	/**
	 * returns the EmployerContacts for the company.
	 * The array could be empty if no EmployerContacts are setup for the company
	 * @return the EmployerContacts
	 */
	public EmployerContact[] getEmployerContacts() throws ServiceFailureException, ValidationException;
	
	/**
	 * 
	 * @param contacts
	 * @return an array of IDs corresponding to the added contacts
	 */
	 public EmployerContact[] addEmployerContacts(EmployerContact[] contacts) throws ServiceFailureException, ValidationException;
	
	
    /**
     * Update the given EmployerContacts.  Each work location must have a uniqueId.
     * @param EmployerContact
     * @return an array containing the updated EmployerContacts
     */
    public EmployerContact[] updateEmployerContacts(EmployerContact[] workLocations) throws ServiceFailureException, ValidationException;
	    
	/**
	 * Deletes the Employer contacts specified from the company
	 * If any of the contacts specified by the uniqueId collection is found
	 * a Validation Exception is thrown and the whole transaction is rolled back
	 * @param EmployerContact
	 */
	public void deleteEmployerContacts(EmployerContact[] contacts) throws ServiceFailureException, ValidationException;
	
	/**
	 * Deletes every Employer contact from the company
	 * 
	 */
	public void deleteAllEmployerContacts() throws ServiceFailureException, ValidationException;

	/**
	 * returns the Employer contact for the id. If the contact for the Id is not found
	 * throws ValidationException
	 * @param id
	 * @return a EmployerContact
	 * @throws ValidationException
	 */
	public EmployerContact findEmployerContact(Long id) throws ServiceFailureException, ValidationException;

	/****************** Company Legal and Tax Filing Info related ****************************/
	/**
	 * Returns the company legal and tax filing information like the legal address and company name etc.
	 * @return the CompanyLegalTaxFilingInfo
	 */
	public CompanyLegalTaxFilingInfo getCompanyLegalTaxFilingInfo() throws ServiceFailureException, ValidationException;
	 
	/**
	 * saves the CompanyLegalTaxFilingInfo
	 */
	public void saveCompanyLegalTaxFilingInfo(CompanyLegalTaxFilingInfo  info) throws ServiceFailureException, ValidationException;
	
	/**
	 * Returns a collection of industry types
	 * @return 
	 */
	public IndustryType[] getIndustryTypes();
	
	/**
	 * Return a collection of company types
	 * @return
	 */
	public CompanyType[] getCompanyTypes();
	
	/**
	 * Updates the employer with which state of assisted employer is in.
	 * @throws ServiceFailureException
	 * @throws ValidationException
	 */
	public void updateAssistedState(AssistedStateType assistedStateType) throws  ServiceFailureException, ValidationException;
	
	/**
	 * 
	 * @return a list of Items based of the Assisted Setup Workflow 
	 * @throws ServiceFailureException
	 */
	public TaxPaymentsChecklistItem[] getTaxPaymentsChecklist() throws  ServiceFailureException;

    /**
     * Add the given Bank Accounts to this company.
     * @param bankAccounts
     * @return an array containing all the added BankAccount objects
     */
    public BankAccount[] addBankAccounts(BankAccount[] bankAccounts) throws ServiceFailureException, ValidationException;
    
    /**
     * Get Bank Accounts for this company.
     * @return an array containing all the BankAccount objects for this company
     */
    public BankAccount[] getBankAccounts() throws ServiceFailureException, ValidationException;
    
    /**
     * Update the given Bank Accounts.  Each Bank Account must have a uniqueId.
     * @param bankAccounts
     * @return an array containing all the saved BankAccount objects
     */
    public BankAccount[] updateBankAccounts(BankAccount[] bankAccounts) throws ServiceFailureException, ValidationException;

    /**
     * Send Company information to PTS
     * -- Company information, Jurisdiction + Agency information, bank information
     */
    public void sendCompanyInfoToPTS() throws ServiceFailureException, ValidationException;
    
    /**
     * Get the Primary Principal for the company
     */
    public PrimaryPrincipal getPrimaryPrincipal() throws ServiceFailureException;
    
    /**
     * Add a Primary Principal for the company
     */
    public PrimaryPrincipal addPrimaryPrincipal(PrimaryPrincipal primaryPrincipal) throws ServiceFailureException, ValidationException;
    
    /**
     * Update the Primary Principal for the company
     */
    public PrimaryPrincipal updatePrimaryPrincipal(PrimaryPrincipal primaryPrincipal) throws ServiceFailureException, ValidationException;
    
    /** Do a check to figure out if this Employer is allowed to Send Data to PTS */
    public boolean allowDataSubmissionToPTS();
    
    /**
	 * This method will take a given year and return the CalendarEntries for that year that are relevant to the 
	 * Takehome application.
	 * 
	 * @param year The year to get the information for.
	 * @return Returns an array of CalendarEntries with relevant information for the given year. This includes
	 * 		   holidays, paydays, and payroll run days.
	 * @author nsaxena and jboekenoogen
	 */
    public CalendarEntry[] getCalendarEntries(Integer year) throws ServiceFailureException, ValidationException;
}
