package com.intuit.sb.payroll.services.employer.impl;

import com.intuit.sb.cdm.basetypes.TelephoneNumber;
import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.services.common.PhysicalAddress;
import com.intuit.sb.payroll.services.common.impl.PhysicalAddressMapper;
import com.intuit.sb.payroll.services.employer.CompanyLegalTaxFilingInfo;
import com.intuit.sb.payroll.us.capability.address.impl.USPostalAddressImpl;

/**
 * Mapper class to handle CompanyInfo Mappings between the DTO/Service layer and
 * the Entity Model.
 * 
 * $Author$ 
 * $Revision$
 * $DateTime$
 */
public class CompanyLegalTaxFilingInfoMapper {

	/**
	 * Maps legal entity data for the employer to a DTO type. Null input yields null output.
	 * 
	 * @param employer
	 * @return
	 */
	public static CompanyLegalTaxFilingInfo mapCompanyInfo(com.intuit.sb.payroll.entity.model.Employer employer)
			throws ServiceFailureException {
		if (employer != null) {
			CompanyLegalTaxFilingInfo cInfo = new CompanyLegalTaxFilingInfo();
			cInfo.setLegalName(employer.getLegalName());

			cInfo.setLegalAddressSameAsBusinessAddress(employer.isLegalAndBusAddressSame());

			// [as] Legal address always stands on its own
			USPostalAddress entityLegalAddress = AddressConvertor.getUSFromGlobal(employer.getLegalAddress(),
					new USPostalAddressImpl());
			cInfo.setLegalAddress(PhysicalAddressMapper.mapAddress(entityLegalAddress));

			cInfo.setIndustryType(employer.getIndustryType());

			cInfo.setCompanyLegalType(employer.getLegalStructure());

			TelephoneNumber legalPhone = employer.getLegalPhone();
			if (legalPhone != null) {
				cInfo.setLegalPhone(legalPhone.getFreeFormNumber());
				cInfo.setExtn(legalPhone.getExtension());
			}

			return cInfo;
		} else {
			return null;
		}
	}

	/**
	 * Maps legal and tax filing data from a DTO the employer entity. Null input yields null output.
	 * 
	 * @param dto
	 * @return
	 * @throws ServiceFailureException 
	 */
	public static com.intuit.sb.payroll.entity.model.Employer mapCompanyInfo(CompanyLegalTaxFilingInfo cInfo)
			throws ValidationException, ServiceFailureException {
		if (cInfo != null) {
			EmployerComponent employerComponent = ServiceLocator.getInstance().getService(EmployerComponent.JNDI_NAME);
			com.intuit.sb.payroll.entity.model.Employer employer = employerComponent.getEmployer();
			if (employer == null) {
				throw new IllegalStateException("Employer has not been configured.");
			}

			employer.setLegalName(cInfo.getLegalName());

			employer.setLegalStructure(cInfo.getCompanyLegalType());

			employer.setIndustryType(cInfo.getIndustryType());

			TelephoneNumber entityLegalPhone = new TelephoneNumber();
			entityLegalPhone.setFreeFormNumber(cInfo.getLegalPhone());
			entityLegalPhone.setExtension(cInfo.getExtn());
			employer.setLegalPhone(entityLegalPhone);

			employer.setLegalAndBusAddressSame(cInfo.isLegalAddressSameAsBusinessAddress());

			// [as] Legal address always always stands on its own
			PhysicalAddress cInfoAddress = cInfo.getLegalAddress();
			if (cInfoAddress != null) {
				USPostalAddress entityLegalAddress = new USPostalAddressImpl();
				PhysicalAddressMapper.mapAddress(cInfoAddress, entityLegalAddress);
				employer.setLegalAddress(AddressConvertor
						.getGlobalFromUS(entityLegalAddress, new GlobalPostalAddress()));
			}

			return employer;
		} else {
			return null;
		}
	}
}
