package com.intuit.sb.payroll.services.employer.impl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerPTSWorkflowComponent;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.pts.entity.PTSContactPerson;
import com.intuit.sb.payroll.pts.entity.PTSContactPersonTypeEnum;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.common.impl.EmployerContactMapper;
import com.intuit.sb.payroll.services.employer.EmployerContact;
import com.intuit.sb.payroll.services.employer.EmployerContactType;
import com.intuit.sb.payroll.services.employer.EmployerService;
import com.intuit.sb.payroll.services.employer.PrimaryPrincipal;

/**
 * Class that implements the getEmployerContacts service interface. It is expected that this class will be directly
 * invoked by the implementor of EmployerService to realize the getEmployerContacts API.
 */
class EmployerContactsImpl {
    private EmployerContactsImpl() {
    }
    private static EmployerPTSWorkflowComponent erWorkflowComponent = ServiceLocator.getInstance().getService(EmployerPTSWorkflowComponent.JNDI_NAME);
    private static EmployerService employerService = ServiceLocator.getInstance().getService(EmployerService.JNDI_NAME);

    static EmployerContact getEmployerContact(Long uniquePTSId) throws ServiceFailureException, ValidationException {
        PayrollParamValidator.checkIsNotNull(uniquePTSId, "EmployerContact uniqueId");

        PTSContactPerson contact = erWorkflowComponent.findEmployerContactPerson(uniquePTSId);
        EmployerContact dto = EmployerContactMapper.mapEmployerContact(contact);
        return dto;
    }

    static EmployerContact[] getEmployerContacts() throws ServiceFailureException {
        PTSEmployer ptsEmployer = erWorkflowComponent.getPTSEmployer();

        Set<PTSContactPerson> entityEmployerContacts = ptsEmployer.getPTSContactPersons();
        List<EmployerContact> resultEmployerContacts = new ArrayList<EmployerContact>();
        // Map from Entity
        for (PTSContactPerson contact : entityEmployerContacts) {
            EmployerContact dto = EmployerContactMapper.mapEmployerContact(contact);
            resultEmployerContacts.add(dto);
        }

        EmployerContact[] locations = new EmployerContact[resultEmployerContacts.size()];
        return resultEmployerContacts.toArray(locations);
    }

    /**
     * Adds the passed in contacts into the system returns the aggregate of the contacts available in
     * the system after adding the new ones.
     *
     * @param contacts
     * @return
     * @throws ServiceFailureException
     * @throws ValidationException
     */
    static EmployerContact[] addEmployerContacts(EmployerContact[] contacts) throws ServiceFailureException,
            ValidationException {
        validateEmployerContactForAddition(contacts);

        List<PTSContactPerson> mappedEmployerContacts = new ArrayList<PTSContactPerson>();

        // Map to Entity
        for (EmployerContact contact : contacts) {
            PTSContactPerson entityContact = EmployerContactMapper.mapEmployerContact(contact);
            mappedEmployerContacts.add(entityContact);
        }
        erWorkflowComponent.addEmployerContactPersons(mappedEmployerContacts);
        // Return current list of EmployerContacts
        return getEmployerContacts();
    }

    /**
     * Updates the contacts passed in the system. If any of the contacts can not be found an
     * ValidationException exception is thrown and none of the contacts are updated.
     *
     * @param contacts
     * @return
     * @throws ServiceFailureException
     * @throws ValidationException
     */
    static EmployerContact[] updateEmployerContacts(EmployerContact[] contacts) throws ServiceFailureException,
            ValidationException {
    	validateEmployerContactIds(contacts);

        List<PTSContactPerson> mappedEmployerContacts = new ArrayList<PTSContactPerson>();
        // Map to Entity
        for (EmployerContact contact : contacts) {
            PTSContactPerson entityContact = EmployerContactMapper.mapEmployerContact(contact);
            mappedEmployerContacts.add(entityContact);
        }
        erWorkflowComponent.updateEmployerContactPersons(mappedEmployerContacts);
        // Return current list of EmployerContacts
        return getEmployerContacts();
    }

    /**
     * deletes the EmployerContact collection passed in. If any of the contacts are not found a
     * ValidationException is thrown and none of the contacts are deleted from the system
     *
     * @param contact list
     * @return contact list
     * @throws ServiceFailureException
     * @throws ValidationException
     */
    static void deleteEmployerContacts(EmployerContact[] contacts) throws ServiceFailureException,
            ValidationException {
        // Logic to prevent deletion of Primary contact
        validateEmployerContactForDeletion(contacts);

        List<PTSContactPerson> mappedEmployerContacts = new ArrayList<PTSContactPerson>();
        // Map to Entity
        for (EmployerContact contact : contacts) {
            PTSContactPerson entityContact = EmployerContactMapper.mapEmployerContact(contact);
            mappedEmployerContacts.add(entityContact);
        }

        erWorkflowComponent.deleteEmployerContactPersons(mappedEmployerContacts);
    }

    /**
     * deletes all EmployerContacts.
     *
     * @param
     * @return
     * @throws ServiceFailureException
     * @throws ValidationException
     */
    static void deleteAllEmployerContacts() throws ServiceFailureException, ValidationException {
        erWorkflowComponent.deleteAllContactPersons();
    }

    private static void validateEmployerContacts(EmployerContact[] contacts) throws ValidationException {
        if (null == contacts) {
            throw new ValidationException("EmployerContacts array cannot be null");
        }
        if (contacts.length == 0) {
            throw new ValidationException("EmployerContacts array cannot be empty");
        }
        for (EmployerContact contact : contacts) {
            if (null == contact) {
                throw new ValidationException("EmployerContact cannot be null");
            }
            if (contact.getFirstName() == null || contact.getFirstName().length() == 0){
                throw new ValidationException("EmployerContact must have a first name");
            }
            if(contact.getLastName() == null || contact.getLastName().length() == 0){
                throw new ValidationException("Employer Contact must have a last name");
            }
            if(contact.getTitle() == null || contact.getTitle().length() == 0){
                throw new ValidationException("Employer contact must have a title");
            }
            if (! validateEmail(contact.getEmail())){
                throw new ValidationException("Employer contact must have a valid email address");
            }
            if(! validatePhoneNumber(contact.getPhone())){
                throw new ValidationException("Employer contact must have a valid phone number");
            }
        }
    }

    private static boolean validatePhoneNumber(String phoneNumber){
        if (phoneNumber == null){
            return false;
        }
        String  expression="^(\\()?([0-9]{3})(\\)|-)?([0-9]{3})(-)?([0-9]{4}|[0-9]{4})$";
        CharSequence inputStr = phoneNumber;
        Pattern pattern = Pattern.compile(expression,Pattern.CASE_INSENSITIVE);
        Matcher matcher = pattern.matcher(inputStr);
        return matcher.matches();
    }

    private static boolean validateEmail(String email)
    {
        if ( email == null || email.length() == 0){
            return false;
        }
        String  expression="^[A-Z0-9._%+-]+@[A-Z0-9.-]+\\.[A-Z]{2,4}$";//^[\\w\\-]([\\.\\w])+[\\w]+@([\\w\\-]+\\.)+[A-Z]{2,4}$";
        CharSequence inputStr = email;
        Pattern pattern = Pattern.compile(expression,Pattern.CASE_INSENSITIVE);
        Matcher matcher = pattern.matcher(inputStr);
        return matcher.matches();
    }

    private static void validateEmployerContactId(Long contactId) throws ValidationException {
        if (null == contactId || contactId == 0) {
            throw new ValidationException("EmployerContact Id cannot be null");
        }
    }

    private static void validateEmployerContactIds(EmployerContact[] contacts) throws ValidationException {
        if (null == contacts) {
            throw new ValidationException("EmployerContacts Id array cannot be null");
        }
        if (contacts.length == 0) {
            throw new ValidationException("EmployerContacts Id array cannot be empty");
        }

        validateEmployerContacts(contacts);

        for (EmployerContact contact : contacts) {
        	if (contact.getObjectHandle() != null){
        		validateEmployerContactId(contact.getObjectHandle().getId());
        	}
        }
    }

    // validate the EmployerContacts For Deletion
    private static void validateEmployerContactForDeletion(EmployerContact[] employerContacts) throws ValidationException {
        validateEmployerContactIds(employerContacts);
        for ( EmployerContact employerContact : employerContacts){
            if (employerContact.getEmployerContactType().equals(EmployerContactType.PrimaryContact)){
                throw new ValidationException("Primary Contact cannot be deleted");
            }else if ( employerContact.getEmployerContactType().equals(EmployerContactType.PrimaryPrincipal)){
                throw new ValidationException("Primary Principal cannot be deleted");
            }
        }
    }

    //validate the EmployerContacts For Addition
    private static void validateEmployerContactForAddition(EmployerContact[] employerContactsToAdd) throws ValidationException, ServiceFailureException {
        validateEmployerContacts(employerContactsToAdd);

        //sort the employerContatsToAdd based on the EmployerContactType
        Arrays.sort(employerContactsToAdd,
                new Comparator<EmployerContact>(){
                    public int compare(EmployerContact e1, EmployerContact e2)
                    {
                        return e1.getEmployerContactType().compareTo(e2.getEmployerContactType());
                    }
        });

        //check whether the employerContactsToAdd list has more than one EmployerContact of type primaryPrincipal/PrimaryContact
        for(int i=0;i<employerContactsToAdd.length-1 && employerContactsToAdd.length > 1;i++){
            if(!(employerContactsToAdd[i].getEmployerContactType().equals(EmployerContactType.SecondaryContact))
                    && employerContactsToAdd[i].getEmployerContactType().equals(employerContactsToAdd[i+1].getEmployerContactType())){
                throw new ValidationException("Only one PrimaryContact/PrimaryPrincipal can be added");
            }
        }

        EmployerContact[] existingEmployerContacts =employerService.getEmployerContacts();
        // check if primaryContact/PrimaryPrincipal already exist in employerContacts
        // This avoids addition of more than one primaryPrinicpal/PrimaryContact
        for(EmployerContact employerContact:employerContactsToAdd){
            for ( EmployerContact existingContact:existingEmployerContacts){
                    if( employerContact.getEmployerContactType().equals(existingContact.getEmployerContactType())){
                        if(employerContact.getEmployerContactType().equals(EmployerContactType.PrimaryContact)){
                            throw new ValidationException("Primary Contact already Exists");
                        }else if (employerContact.getEmployerContactType().equals(EmployerContactType.PrimaryPrincipal)){
                            throw new ValidationException("Primary Principal already Exists");
                        }
                    }
                }
            	employerContact.setObjectHandle(new ObjectHandle(0L, 0L));
            }
        }

	static PrimaryPrincipal getPrimaryPrincipal() {
		
		PTSEmployer ptsEmployer = erWorkflowComponent.getPTSEmployer();
		for (PTSContactPerson ptsContactPerson : ptsEmployer.getPTSContactPersons()) {
			if(ptsContactPerson.getContactPersonType() == PTSContactPersonTypeEnum.PRIMARY_PRINCIPAL) {
				return EmployerContactMapper.mapToPrimaryPrincipal(ptsContactPerson);
			}
		}
		return null;
	}

	static PrimaryPrincipal addPrimaryPrincipal(PrimaryPrincipal primaryPrincipal) throws ValidationException, ServiceFailureException {
		PTSContactPerson erContact = EmployerContactMapper.mapToPTSContactPerson(primaryPrincipal);
		ArrayList<PTSContactPerson> contactsToAdd = new ArrayList<PTSContactPerson>(1);
		contactsToAdd.add(erContact);
		Set<PTSContactPerson> contactPersons = erWorkflowComponent.addEmployerContactPersons(contactsToAdd);
		PTSContactPerson ptsContactPerson = null;
		for (PTSContactPerson contactPerson : contactPersons) {
			if (contactPerson.getContactPersonType() == PTSContactPersonTypeEnum.PRIMARY_PRINCIPAL) {
				ptsContactPerson = contactPerson;
				break;
			}
		}
		return EmployerContactMapper.mapToPrimaryPrincipal(ptsContactPerson);
	}

	static PrimaryPrincipal updatePrimaryPrincipal(PrimaryPrincipal primaryPrincipal) throws ValidationException, ServiceFailureException {
		PTSContactPerson erContact = EmployerContactMapper.mapToPTSContactPerson(primaryPrincipal);
		ArrayList<PTSContactPerson> contactsToAdd = new ArrayList<PTSContactPerson>(1);
		contactsToAdd.add(erContact);
		Set<PTSContactPerson> contactPersons = erWorkflowComponent.updateEmployerContactPersons(contactsToAdd);
		
		PTSContactPerson ptsContactPerson = null;
		for (PTSContactPerson contactPerson : contactPersons) {
			if (contactPerson.getContactPersonType() == PTSContactPersonTypeEnum.PRIMARY_PRINCIPAL) {
				ptsContactPerson = contactPerson;
				break;
			}
		}
		return EmployerContactMapper.mapToPrimaryPrincipal(ptsContactPerson);
	}
}
