package com.intuit.sb.payroll.services.pto.impl;

import java.util.Set;

import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;

import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.pto.PaidTimeOffComponent;
import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.entity.model.PaidTimeOffDtl;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.services.pto.PTOInfo;
import com.intuit.sb.payroll.services.pto.PTOService;
import com.intuit.sb.payroll.services.pto.PTOType;
import com.intuit.sb.payroll.services.pto.SavePTOResponse;
import com.intuit.sb.payroll.services.worker.impl.WorkerMapper;

@Stateless(name = "PTOService")
@SecurityDomain("BHPIAM")
@RolesAllowed({"Authenticated"})
public class PTOServiceBean implements PTOService {

	@EJB
	private WorkerComponent workerComponent;
	
	@EJB
	private PaidTimeOffComponent paidTimeOffComponent;
	
	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.services.pto.PTOService#getAllPTOInfo(java.lang.Long)
	 */
	public PTOInfo[] getAllPTOInfo(Long workerId) throws ServiceFailureException,
			ValidationException {
		if (workerId == null || workerId.toString().length() == 0) {
			throw new ServiceFailureException("workerId is a required value");
		}
		Worker worker = workerComponent.findWorker(workerId);
		return getWorkerPTOInfos(worker);
	}

	/** Given the worker entity, return sorted array of all PTOInfos */
	private PTOInfo[] getWorkerPTOInfos(Worker worker) throws ServiceFailureException {
	    Set<PaidTimeOffDtl> paidTimeOffDtlSet = worker.getPaidTimeOffDtls();
	    PTOInfo[] ptoInfos = new PTOInfo[paidTimeOffDtlSet.size()];
	    int i = 0;
      for (PaidTimeOffDtl paidTimeOffDtl : paidTimeOffDtlSet) {
	    ptoInfos[i++] = PTOMapper.mapToPTOInfo(worker.getId(), paidTimeOffDtl);
      }
      
      //Modified the service code to return PTOs in a certain order
      //Not having this is pain while testing this code.
      //Now the code returns SICK, VACATION in order consistent with the UI
      
      if(ptoInfos.length > 1) //Only if both Sick and Vacation exists order them
      {
	    if(ptoInfos[0].getPtoType().name().equals(PTOType.VACATION.name()))
	    {
	    	PTOInfo tmpPTOInfo = ptoInfos[0];
	    	ptoInfos[0] = ptoInfos[1];
	    	ptoInfos[1] = tmpPTOInfo;
	    }
      }
	    return ptoInfos;
	}
	
	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.services.worker.IWorkerService#getPTOByNameAndType(java.lang.String, java.lang.String, com.intuit.sb.payroll.services.pto.PTOType)
	 * 
	 * This currently ignores the ptoName value.  May be used in the future 
	 * if the UI differentiates between salary and hourly.
	 */
	public PTOInfo getPTOByNameAndType(Long workerId, String ptoName, PTOType ptoType)
		throws ServiceFailureException, ValidationException {

		validate(workerId, ptoName, ptoType);
		
		PTOInfo ptoInfo = null;
        Worker worker = workerComponent.findWorker(workerId);
        Set<PaidTimeOffDtl> paidTimeOffDtlSet = worker.getPaidTimeOffDtls();
        for (PaidTimeOffDtl paidTimeOffDtl : paidTimeOffDtlSet) {
        	if (ptoType.equals(PTOMapper.mapToPTOType(paidTimeOffDtl.getPaidTimeOffType()))) {
        		ptoInfo = PTOMapper.mapToPTOInfo(workerId, paidTimeOffDtl);
        	}
        }
        // Return null if not found
        return ptoInfo;
	}

	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.services.worker.IWorkerService#savePTO(com.intuit.sb.payroll.services.worker.SavePTORequest)
	 */
	public SavePTOResponse savePTO(PTOInfo ptoInfo) throws WorkflowException, ServiceFailureException, ValidationException {
		
		validate(ptoInfo.getWorkerId(), ptoInfo.getName(), ptoInfo.getPtoType());
		
		PaidTimeOffDtl paidTimeOffDtl = PTOMapper.mapToPaidTimeOffDtl(ptoInfo);
        Worker worker = workerComponent.findWorker(ptoInfo.getWorkerId());
        // If object handle was not passed, create new.  Otherwise update is automatic.
        if (ptoInfo.getObjectHandle() == null) {
           	paidTimeOffComponent.addPaidTimeOffDtl(worker, paidTimeOffDtl);
        }
        	SavePTOResponse response = new SavePTOResponse(
        		WorkerMapper.mapFromEntityToPojo(worker),
        		getWorkerPTOInfos(worker));
        	return response;
	}
	
	private static void validate(Long workerId, String ptoName, PTOType ptoType) throws ServiceFailureException {
		if (workerId == null || workerId.toString().length() == 0) {
			throw new ServiceFailureException("workerId is a required value");
		}
		if (ptoName == null || ptoName.length() == 0) {
			throw new ServiceFailureException("ptoName is a required value");
		}
		if (ptoType == null || ptoType.toString().length() == 0) {
			throw new ServiceFailureException("ptoType is a required value");
		}
	}
}
