package com.intuit.sb.payroll.services.taxes.impl;

import java.util.Set;

import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;

import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.taxes.withholding.TaxWithholdingsComponent;
import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.common.PayrollResourceManager;
import com.intuit.sb.payroll.entity.model.CAWagePlan;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.PersonTaxId;
import com.intuit.sb.payroll.entity.model.PersonTaxIdEnum;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;
import com.intuit.sb.payroll.services.taxes.FormType;
import com.intuit.sb.payroll.services.taxes.SaveWithholdingInfoRequest;
import com.intuit.sb.payroll.services.taxes.SaveWithholdingResponse;
import com.intuit.sb.payroll.services.taxes.TaxWithholdingsService;
import com.intuit.sb.payroll.services.taxes.WithholdingInfoOperation;
import com.intuit.sb.payroll.services.taxes.WithholdingInformation;
import com.intuit.sb.payroll.services.worker.impl.WorkerMapper;

/**
 * Implementation of the TaxWithholdingsService interface. Handles retrieval and updates to WithholdingInformation.
 * 
 * $Author$ $Revision$ $DateTime$
 * 
 */
@Stateless(name = "TaxWithholdingsService")
@SecurityDomain("BHPIAM")
@RolesAllowed( { "Authenticated" })
public class TaxWithholdingsServiceBean implements TaxWithholdingsService {

	protected static final PayrollResourceManager sResourceManager = PayrollResourceManager.getInstance();

	private static final String sDefaultWagePlanCode = "S";

	@EJB
	private TaxWithholdingsComponent taxWithholdingsComponent;

	@EJB
	private WorkerComponent workerComponent;

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.services.taxes.TaxWithholdingsService#getEmployeeWithholdings(java.lang.Long)
	 */
	public WithholdingInformation[] getEmployeeWithholdings(Long workerId) throws ServiceFailureException,
			ValidationException {

		Worker worker = workerComponent.findWorker(workerId);
		WorkerComplianceDocDtl workerCompArray[] = worker.getWorkerComplianceDocDtls().toArray(
				new WorkerComplianceDocDtl[0]);

		return WithholdingInformationEIMapper.mapFromEntitiesToPojos(worker, workerCompArray);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.intuit.sb.payroll.services.taxes.TaxWithholdingsService#saveWithholdingInformation(com.intuit.sb.payroll.
	 * services.taxes.SaveWithholdingInfoRequest)
	 */
	public SaveWithholdingResponse saveWithholdingInformation(SaveWithholdingInfoRequest request)
			throws ValidationException, WorkflowException, ServiceFailureException {
		SaveWithholdingInfoRequestValidator.validate(request);
		Worker worker = workerComponent.findWorker(request.getWorkerId());
		
		// In addition to updating the withholding info, also update worker with tax id number
		String taxId = request.getWorkerTaxIdNumber();
		if (worker.getPersonTaxId() != null)
			worker.getPersonTaxId().setValue(taxId);
		else {
			PersonTaxId pTaxId = new PersonTaxId();
			pTaxId.setValue(taxId);
			pTaxId.setType(PersonTaxIdEnum.SSN);
			worker.setPersonTaxId(pTaxId);
		}
		
		String caWagePlanCode = request.getWagePlanCode();
		// Treat an empty WagePlanCode the same as a null WagePlanCode
		if ((null == caWagePlanCode) || ("".equals(caWagePlanCode))) {
			caWagePlanCode = sDefaultWagePlanCode;
		}
		
		//CA specific code!
		this.addOrUpdateWagePlanCode(worker, caWagePlanCode);
		
		//Withholding info handling
		boolean withholdingInfoChanged = false;
		WithholdingInformation withholdingInfo = request.getWithholdingInfo();

		WithholdingInfoOperation withholdingInfoOperation = request.getWithholdingInfoOperation();
		switch (withholdingInfoOperation) {
			case ADD:
				WorkerComplianceDocDtl workerComplianceDocToAdd = WithholdingInformationEIMapper
						.mapNewPojoToEntity(withholdingInfo);
				worker = taxWithholdingsComponent.addWorkerComplianceDocDtl(worker, workerComplianceDocToAdd);
				withholdingInfoChanged = true;
				break;
			case UPDATE:
				WorkerComplianceDocDtl existingComplianceDocToUpdate = findExistingComplianceDocDtl(worker, withholdingInfo);
				WorkerComplianceDocDtl workerComplianceDocToUpdate = WithholdingInformationEIMapper
						.mapFromPojoToEntity(withholdingInfo, existingComplianceDocToUpdate);
				worker = taxWithholdingsComponent.updateWorkerComplianceDocDtl(worker, workerComplianceDocToUpdate);
				withholdingInfoChanged = true;
				break;

			case DELETE:
				WorkerComplianceDocDtl existingComplianceDocToDelete = findExistingComplianceDocDtl(worker, withholdingInfo);
				worker = taxWithholdingsComponent.deleteWorkerComplianceDocDtl(worker, existingComplianceDocToDelete);
				
				break;

			case NOT_SPECIFIED:
				//WagePlan code update only.
				break;
			default:
				break;
		}

		// Save the worker
		worker = workerComponent.updateWorker(worker);
	
		Set<WorkerComplianceDocDtl> dtlSet = worker.getWorkerComplianceDocDtls();
		SaveWithholdingResponse response = new SaveWithholdingResponse();
		if (withholdingInfoChanged) {
			//The client expects a new version of the withholding
			response.setWithholding(WithholdingInformationEIMapper.mapFromEntityToPojo(worker, getUpdatedDoc(dtlSet, request)));
		}
		response.setWorker(WorkerMapper.mapFromEntityToPojo(worker));

		return response;
	}

	private void addOrUpdateWagePlanCode(Worker worker, String caWagePlanCode) throws ValidationException, WorkflowException, ServiceFailureException {
		Set<WorkerComplianceDocDtl> workerComplianceDocDtls = worker.getWorkerComplianceDocDtls();
		WorkerComplianceDocDtl existingComplianceDoc = null;
		boolean found = false;
		for (WorkerComplianceDocDtl workerComplianceDocDtl : workerComplianceDocDtls) {
			if (workerComplianceDocDtl.getDocId() == ComplianceDocIdEnum.DE545) {
				existingComplianceDoc = workerComplianceDocDtl;
				found = true;
				break;
			}
		}
		CAWagePlan caWagePlanDoc = null;
		if (found) {
			caWagePlanDoc = (CAWagePlan)existingComplianceDoc;
			caWagePlanDoc.setValue(caWagePlanCode);
			worker = taxWithholdingsComponent.updateWorkerComplianceDocDtl(worker, caWagePlanDoc);
		} else {
			caWagePlanDoc = (CAWagePlan)taxWithholdingsComponent.createWorkerComplianceDocDtl(ComplianceDocIdEnum.DE545);
			caWagePlanDoc.setValue(caWagePlanCode);
			worker = taxWithholdingsComponent.addWorkerComplianceDocDtl(worker, caWagePlanDoc);
		}
		
	}

	private WorkerComplianceDocDtl findExistingComplianceDocDtl(Worker worker, WithholdingInformation withholdingInfo) throws ServiceFailureException {

		Set<WorkerComplianceDocDtl> workerComplianceDocDtls = worker.getWorkerComplianceDocDtls();
		WorkerComplianceDocDtl existingComplianceDoc = null;
		// Find the one that exists if at all
		boolean found = false;
		for (WorkerComplianceDocDtl workerComplianceDocDtl : workerComplianceDocDtls) {
			if (workerComplianceDocDtl.getId() == withholdingInfo.getObjectHandle().getId()) {
				existingComplianceDoc = workerComplianceDocDtl;
				found = true;
				break;
			}
		}
		if (!found) {
			throw new ServiceFailureException("Unable to update non-existent Withholding document.");
		}
		return existingComplianceDoc;
	}

	private WorkerComplianceDocDtl getUpdatedDoc(Set<WorkerComplianceDocDtl> dtlSet, SaveWithholdingInfoRequest request) {
		WorkerComplianceDocDtl existingComplianceDoc = null;
		for (WorkerComplianceDocDtl workerComplianceDocDtl : dtlSet) {
			if (workerComplianceDocDtl.getDocId() == ComplianceDocIdEnum.W4IRS) {
				if (request.getWithholdingInfo().getFormType() == FormType.US_W4) {
					existingComplianceDoc = workerComplianceDocDtl;
					break;
				}
			} else if (workerComplianceDocDtl.getDocId() == ComplianceDocIdEnum.DE4) {
				if (request.getWithholdingInfo().getFormType() == FormType.CA_DE4) {
					existingComplianceDoc = workerComplianceDocDtl;
					break;
				}
			}
		}
		return existingComplianceDoc;
	}
}
