package com.intuit.sb.payroll.services.taxpayments.impl;

import java.util.Date;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.util.DateRangeUtil;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.services.taxpayments.PaymentPeriod;

/**
 * Utility class that provides mapping between<br>
 * {@link DateRange DateRange} class in the entity layer and<br>
 * {@link PaymentPeriod PaymentPeriod} class in the composed layer.
 */
public class DateRangeMapper {

	/**
	 * @param payPeriod {@link PaymentPeriod PaymentPeriod} object to be mapped
	 * @return {@link DateRange DateRange} object corresponding to the input payPeriod object
	 * @throws ValidationException if the input payPeriod object is not well formed.
	 */
	public static DateRange mapPaymentPeriodToDateRange(PaymentPeriod payPeriod) 
			throws ValidationException {
		
		PayrollParamValidator.checkIsNotNull(payPeriod, "Pay Period");
		PayrollParamValidator.checkIsNotNull(payPeriod.getPeriodStartDate(), "payPeriod.periodStartDate");
		PayrollParamValidator.checkIsNotNull(payPeriod.getPeriodEndDate(), "payPeriod.periodEndDate");

		DateRange range = new DateRange();

		range.setStartDate(payPeriod.getPeriodStartDate());
		range.setEndDate(payPeriod.getPeriodEndDate());
		
		return range;
	}

	/**
	 * @param dateRange {@link DateRange DateRange} object
	 * @return {@link PaymentPeriod PaymentPeriod} object to be mapped to the input {@link DateRange dateRange}
	 * @throws ValidationException 
	 * @throws ValidationException if the input dateRange object is not well formed.
	 */
	public static PaymentPeriod mapDateRangeToPaymentPeriod(DateRange dateRange) throws ValidationException {
		if (dateRange == null) {
			return null;
		}
		
		PaymentPeriod pPeriod = new PaymentPeriod();
		
		PayrollParamValidator.checkIsNotNull(dateRange.getStartDate(), "dateRange.startDate");
		PayrollParamValidator.checkIsNotNull(dateRange.getEndDate(), "dateRange.endDate()");
		
		pPeriod.setPeriodStartDate(dateRange.getStartDate());
		pPeriod.setPeriodEndDate(dateRange.getEndDate());
		
		return pPeriod;
	}
	
	/**
	 * @param dRange1 first {@link DateRange} object for comparison
	 * @param dRange2 second {@link DateRange} object for comparison
	 * @return true if they both have the same start/end dates, false otherwise
	 * @throws ValidationException if either {@link DateRange} object is invalid
	 */
	public static boolean areDateRangesIdentical(DateRange dRange1, DateRange dRange2) 
			throws ValidationException {
		
		PayrollParamValidator.checkIsNotNull(dRange1, "Input Date Range");
		PayrollParamValidator.checkIsNotNull(dRange2, "Input Date Range");
		PayrollParamValidator.checkIsNotNull(dRange1.getStartDate(), "dateRange.startDate");
		PayrollParamValidator.checkIsNotNull(dRange1.getEndDate(), "dateRange.endDate()");
		PayrollParamValidator.checkIsNotNull(dRange2.getStartDate(), "dateRange.startDate");
		PayrollParamValidator.checkIsNotNull(dRange2.getEndDate(), "dateRange.endDate()");
		
		Date startDate1 = dRange1.getStartDate();
		Date endDate1   = dRange1.getEndDate();
		Date startDate2 = dRange2.getStartDate();
		Date endDate2   = dRange2.getEndDate();
		
		return (DateRangeUtil.dateEquals(startDate1, startDate2) &&
				DateRangeUtil.dateEquals(endDate1, endDate2));
	}
		
	
	/**
	 * @param pPeriod1 first {@link PaymentPeriod} object for comparison
	 * @param pPeriod2 second {@link PaymentPeriod} object for comparison
	 * @return true if they both have the same start/end dates, false otherwise
	 * @throws ValidationException if either {@link PaymentPeriod} object is invalid
	 */
	public static boolean areDateRangesIdentical(PaymentPeriod pPeriod1, PaymentPeriod pPeriod2) 
			throws ValidationException {

		DateRange dRange1 = mapPaymentPeriodToDateRange(pPeriod1);
		DateRange dRange2 = mapPaymentPeriodToDateRange(pPeriod2);
		
		return areDateRangesIdentical(dRange1, dRange2);
		
	}
}
