package com.intuit.sb.payroll.services.taxpayments.impl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;

import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.capability.employer.EmployerPTSWorkflowComponent;
import com.intuit.sb.payroll.capability.pts.adapter.PTSRequestInvoker;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.common.i18n.LocaleGuru;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.HistPaycheck;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.Payment;
import com.intuit.sb.payroll.entity.model.PaymentType;
import com.intuit.sb.payroll.services.taxpayments.FutaPaymentsAndLiabilities;
import com.intuit.sb.payroll.services.taxpayments.LiabilityBalance;
import com.intuit.sb.payroll.services.taxpayments.PaymentPeriod;
import com.intuit.sb.payroll.services.taxpayments.PaymentTypeLiability;
import com.intuit.sb.payroll.services.taxpayments.TaxPaymentService;
import com.intuit.sb.payroll.services.taxpayments.ToDateTaxPayments;

@Stateless(name = "TaxPaymentService")
@SecurityDomain("BHPIAM")
@RolesAllowed( { "Authenticated" })
public class TaxPaymentServiceBean implements TaxPaymentService {

	@EJB 
	EmployerComponent employerComponent;
	
	@EJB
	EmployerPTSWorkflowComponent employerPTSWorkflowComponent;
	
	/**
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.services.taxpayments.TaxPaymentService#getToDateTaxPayments(com.intuit.sb.payroll.services.taxpayments.PaymentPeriod)
	 */
	public ToDateTaxPayments[] getToDateTaxPayments(PaymentPeriod paymentPeriod) 
			throws ValidationException {

		DateRange dateRange = DateRangeMapper.mapPaymentPeriodToDateRange(paymentPeriod);
		ToDateTaxPayments[] todatePayments = null;
		
		if (dateRange != null) {
			List<Payment> payments = employerComponent.getPayments(dateRange);
//			payments may be spread across different quarters/date ranges.
//			group them by their quarter
//			use a Map with key as the guidance payment period/date range
//			the value is a list of Payment objects that fall in that period/range
			Map<DateRange, List<Payment>> map = TaxPaymentsMapper.groupPaymentsByPayrollPeriod(payments);
			todatePayments = TaxPaymentsMapper.mapPaymentEntityToTaxPaymentsDto(map);			
		}
		return todatePayments;
	}

	/**
	 * (non-Javadoc)
	 * @throws ServiceFailureException 
	 * @see com.intuit.sb.payroll.services.taxpayments.TaxPaymentService#removeTaxPayments(com.intuit.sb.payroll.services.taxpayments.PaymentPeriod)
	 */
	public void removeTaxPayments(PaymentPeriod paymentPeriod) 
			throws ValidationException, ServiceFailureException {
		
		DateRange dateRange = DateRangeMapper.mapPaymentPeriodToDateRange(paymentPeriod);
		if (dateRange != null) {
			employerComponent.removePayments(dateRange);
		}
	}
	
	/**
	 * (non-Javadoc)
	 * @throws ServiceFailureException 
	 * @see com.intuit.sb.payroll.services.taxpayments.TaxPaymentService#removeTaxPayment(Long)
	 */
	public void removeTaxPayment(Long paymentId) 
			throws ValidationException, ServiceFailureException {
		
		if (paymentId != null) {
			employerComponent.removePayment(paymentId);
		}
	}
	

	/**
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.services.taxpayments.TaxPaymentService#updateTaxPayments(com.intuit.sb.payroll.services.taxpayments.ToDateTaxPayments[])
	 */
	public ToDateTaxPayments[] updateTaxPayments(ToDateTaxPayments[] taxPayments) 
			throws ValidationException, ServiceFailureException {
		ToDateTaxPayments[] taxPaymentsUpdated = new ToDateTaxPayments[taxPayments.length];
		int i = 0;
		for (ToDateTaxPayments toDateTaxPayments : taxPayments) {
			Payment[] payments = TaxPaymentsMapper.mapTaxPaymentsDtoToPaymentEntity(toDateTaxPayments);
			List<Payment> list = Arrays.asList(payments);
		        list = employerComponent.updatePayments(list);
			taxPaymentsUpdated[i++] = TaxPaymentsMapper.mapPaymentEntityToTaxPaymentsDto(list);
		}
		return taxPaymentsUpdated;
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.services.taxpayments.TaxPaymentService#retrieveLiabilitiesForGivenPayment(com.intuit.sb.payroll.services.taxpayments.PaymentPeriod)
	 */
	public List<PaymentTypeLiability> getLiabilitiesFromExistingPayrollRuns(PaymentPeriod paymentPeriod) 
			throws ValidationException, ServiceFailureException {
		DateRange quarterUnderConsideration = DateRangeMapper.mapPaymentPeriodToDateRange(paymentPeriod);
		// get a map of law-id's and the aggregated liability for each
		Map<String, Money> map = employerComponent.getLiabilitiesFromExistingPayrollRuns(quarterUnderConsideration);
		Map<PaymentType, Money> outputMap = getDefaultLiabilityMap();
		
		if (map != null && map.size() > 0) {
			// now further aggregate the liabilities by PaymentType
			Set<String> lawIds = map.keySet();
			PaymentType type = null;
			// loop through the lawids of the original map and build the new one
			for (String lawId : lawIds) {
				Money liability = (map.get(lawId) == null?new Money("0.0"):map.get(lawId));
				type = PaymentTypeLawMapper.mapLawIdToPaymentType(lawId);
				if (outputMap.keySet().contains(type)) {
					// aggregate the liabilities for that PaymentType
					outputMap.put(type, outputMap.get(type).add(liability));
				} else {
					// add the liability for that PaymentType
					outputMap.put(type, liability);
				}
			}
		}
		
		return getListFromMap(outputMap);
	}
	
	
	/**
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.services.taxpayments.TaxPaymentService#getHistoricalLiabilityBalances()
	 */
	public LiabilityBalance[] getHistoricalLiabilityBalances() throws ServiceFailureException {
		List<com.intuit.sb.payroll.capability.pts.LiabilityBalance> balances = PTSRequestInvoker.getHistoricalLiabilityBalances(employerPTSWorkflowComponent.getPTSEmployer());
		if(balances == null) {
			return new LiabilityBalance[0];
		}
		LiabilityBalance[] newBalances = new LiabilityBalance[balances.size()];
		int index = 0;
		for (com.intuit.sb.payroll.capability.pts.LiabilityBalance liabilityBalance : balances) {
			LiabilityBalance newBalance = new LiabilityBalance();
			newBalance.setLawId(liabilityBalance.getLawId());
			newBalance.setLiabilityAmount(liabilityBalance.getLiabilityAmount().doubleValue());
			newBalance.setPaymentAmount(liabilityBalance.getPaymentAmount().doubleValue());
			newBalance.setBalance(liabilityBalance.getBalance().doubleValue());
			newBalances[index] = newBalance;
			index++;
		}
		return newBalances;
	}
	
	/**
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.services.taxpayments.TaxPaymentService#getHistoricalLiabilityBalances()
	 */
	public LiabilityBalance[] getTaxPaymentBalances() throws ServiceFailureException {
		List<com.intuit.sb.payroll.capability.pts.LiabilityBalance> balances = PTSRequestInvoker.getHistoricalLiabilityBalances(employerPTSWorkflowComponent.getPTSEmployer());
		if(balances == null) {
			return new LiabilityBalance[0];
		}
		List<LiabilityBalance> newBalances = new ArrayList<LiabilityBalance>();
		// TODO: Improve loop by removing futa from the list instead of creatig a new one 
		for (com.intuit.sb.payroll.capability.pts.LiabilityBalance liabilityBalance : balances) {
			if(!liabilityBalance.getLawId().equals(QboeLawIdConstants.sFUTALawId)) {
				LiabilityBalance newBalance = new LiabilityBalance();
				newBalance.setLawId(liabilityBalance.getLawId());
				newBalance.setLiabilityAmount(liabilityBalance.getLiabilityAmount().doubleValue());
				newBalance.setPaymentAmount(liabilityBalance.getPaymentAmount().doubleValue());
				newBalance.setBalance(liabilityBalance.getBalance().doubleValue());
				newBalances.add(newBalance);
			}
		}
		
		LiabilityBalance[] newBalancesArray = new LiabilityBalance[newBalances.size()];
		int index = 0;
		for (LiabilityBalance liabilityBalance : newBalances) {
			newBalancesArray[index] = liabilityBalance;
			index++;
		}
		
		return newBalancesArray;
	}
	
	
	/**
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.services.taxpayments.TaxPaymentService#getHistoricalLiabilityBalances()
	 */
	public LiabilityBalance[] getFUTABalances() throws ServiceFailureException {
		List<com.intuit.sb.payroll.capability.pts.LiabilityBalance> balances = PTSRequestInvoker.getHistoricalLiabilityBalances(employerPTSWorkflowComponent.getPTSEmployer());
		LiabilityBalance[] newBalances = new LiabilityBalance[0];
		
		if(balances == null) {
			return newBalances;
		}
		
		for (com.intuit.sb.payroll.capability.pts.LiabilityBalance liabilityBalance : balances) {
			if(liabilityBalance.getLawId().equals(QboeLawIdConstants.sFUTALawId)) {
				LiabilityBalance newBalance = new LiabilityBalance();
				newBalance.setLawId(liabilityBalance.getLawId());
				newBalance.setLiabilityAmount(liabilityBalance.getLiabilityAmount().doubleValue());
				newBalance.setPaymentAmount(liabilityBalance.getPaymentAmount().doubleValue());
				newBalance.setBalance(liabilityBalance.getBalance().doubleValue());
				newBalances = new LiabilityBalance[1];
				newBalances[0] = newBalance;
				break;
			}
		}
		return newBalances;
	}
	/**
	 * Create a default Map object containing all the payment types with liability value 0.0
	 * The 0.0 will replaced with actual liability value, in case actual liabilities exist.
	 * @return
	 */
	private Map<PaymentType, Money> getDefaultLiabilityMap() {
		Map<PaymentType, Money> defaultMap = new HashMap<PaymentType, Money>();

		for (PaymentType paymentType : PaymentType.values()) {
			defaultMap.put(paymentType, new Money("0.00"));
		}
		return defaultMap;
	}

	
	/**
	 * 
	 * @param mapOfPaymentTypesAndLiabilities - {@link Map} of LawId and corresponding Liability
	 * @return {@link PaymentTypeLiability} {@link List}
	 */
	private List<PaymentTypeLiability> getListFromMap(Map<PaymentType, Money> mapOfPaymentTypesAndLiabilities) {
		List<PaymentTypeLiability> list = new ArrayList<PaymentTypeLiability>();
		Set<PaymentType> paymentTypes = mapOfPaymentTypesAndLiabilities.keySet();
		for (PaymentType paymentType : paymentTypes) {
			list.add(new PaymentTypeLiability(paymentType.name(), mapOfPaymentTypesAndLiabilities.get(paymentType).getAmount()));
		}
		return list;
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.services.taxpayments.TaxPaymentService#getFutaPaymentsAndLiabilitiesYtd()
	 */
	public List<FutaPaymentsAndLiabilities> getFutaPaymentsAndLiabilitiesYtd() 
			throws ValidationException {
		
		/* Get List of Payments and Liabilities based on todays' date
		 * The return would never be null - just a list with zero or more elements
		 */
		Calendar now = LocaleGuru.getInstance().getCalendar();
		List<Payment> payments = employerComponent.getFutaPayments(LocaleGuru.getInstance().getCalendar());
		List<HistPaycheck> liabilities = employerComponent.getFutaLiabilities(now);
		
		List<FutaPaymentsAndLiabilities> fpAndL = 
			FutaPaymentsAndLiabilitiesMapper.mapPaymentAndHistPaycheckEntityToFutaPaymentsAndLiabilities(
					payments, liabilities);

		return fpAndL;
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.services.taxpayments.TaxPaymentService#removeFutaPaymentsAndLiabilitiesYtd()
	 */
	public void removeFutaPaymentsAndLiabilitiesYtd() 
			throws ValidationException, ServiceFailureException {
		Calendar now = LocaleGuru.getInstance().getCalendar();
		employerComponent.removeFutaPaymentsAndLiabilitiesYtd(now);
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.services.taxpayments.TaxPaymentService#updateFutaPaymentsAndLiabilitiesYtd(java.util.List)
	 */
	public List<FutaPaymentsAndLiabilities> updateFutaPaymentsAndLiabilitiesYtd(
			List<FutaPaymentsAndLiabilities> paymentsAndLiabilities) throws ValidationException, ServiceFailureException {
		List<Payment> payments = FutaPaymentsAndLiabilitiesMapper.getPaymentEntityFromFutaPaymentsAndLiabilities(paymentsAndLiabilities);
		List<HistPaycheck> liabilities = FutaPaymentsAndLiabilitiesMapper.getHistPaycheckEntityFromFutaPaymentsAndLiabilities(paymentsAndLiabilities);
		
		employerComponent.updateFutaPaymentsAndLiabilitiesYtd(payments, liabilities);
	
		List<FutaPaymentsAndLiabilities> updatedFuta = getFutaPaymentsAndLiabilitiesYtd();
		
		return updatedFuta;
	}
	
}