package com.intuit.sb.payroll.services.wages.impl;

import java.math.BigDecimal;
import java.text.MessageFormat;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.EnumMap;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.apache.log4j.Logger;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.compensation.CompensationComponent;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.domain.Identifiable;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.DatumType1Enum;
import com.intuit.sb.payroll.entity.model.DatumType2Enum;
import com.intuit.sb.payroll.entity.model.PtoBaseRateTypeEnum;
import com.intuit.sb.payroll.services.common.impl.ObjectHandleMapper;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.wages.PtoBaseRateType;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemRequest;

public class CompensationItemMapper {
	private static final Logger logger = PayrollLoggerFactory.makeLogger(CompensationItemMapper.class);
	private static CompensationComponent wageComponent = 
		ServiceLocator.getInstance().getService(CompensationComponent.JNDI_NAME);

	/**
	 * Map the given CompensationItemRequests to CompensationDtls.
	 * @param compensationItemRequests
	 * @return a HashMap whose keys are the CompensationDtls and values are SaveCompensationItemOperations. 
	 */
	public static Map<CompensationDtl, SaveCompensationItemOperation> mapToCompensationDtls(
			SaveCompensationItemRequest[] compensationItemRequests) throws ValidationException, ServiceFailureException {
		if (null == compensationItemRequests || compensationItemRequests.length == 0) {
			throw new ServiceFailureException("SaveCompensationItemRequests cannot be null or empty...");
		}
		HashMap<CompensationDtl, SaveCompensationItemOperation> map = 
			new HashMap<CompensationDtl, SaveCompensationItemOperation>(
					compensationItemRequests.length);
		
    	HashMap<CompensationItem, CompensationDtl> itemToDetailMap = 
    		new HashMap<CompensationItem, CompensationDtl>(compensationItemRequests.length);
    	
		for (SaveCompensationItemRequest compensationItemRequest : compensationItemRequests) {
			CompensationItem compensationItem = compensationItemRequest.getCompensationItem();
			CompensationItemParamValidator.validateCompensationItem(compensationItem, 
					compensationItemRequest.getCompensationItemOperation());
			CompensationDtl compensationDtl = CompensationItemMapper.mapToCompensationDtl(compensationItem);
			itemToDetailMap.put(compensationItem, compensationDtl);
			map.put(compensationDtl, compensationItemRequest.getCompensationItemOperation());
		}
		
		// Link CompensationDtls based on links between CompensationItems
		for (CompensationItem compensationItem : itemToDetailMap.keySet()) {
			if (compensationItem.getLinkedCompensationItem() != null ) {
				itemToDetailMap.get(compensationItem).setLinkedPayrollItemDtl(
					itemToDetailMap.get(compensationItem.getLinkedCompensationItem()));
			}
		}
		
		return map;
	}
	
	/**
	 * Map the given CompensationItems to CompensationDtls.
	 * @param compensationItems
	 * @return CompensationDtls  
	 */
	public static Collection<CompensationDtl> mapToCompensationDtls(CompensationItem[] compensationItems) throws ValidationException, ServiceFailureException {
		if (null == compensationItems || compensationItems.length == 0) {
			return Collections.emptySet();
		}
    	Map<CompensationItem, CompensationDtl> itemToDetailMap = 
    		new HashMap<CompensationItem, CompensationDtl>(compensationItems.length);
    	
		for (CompensationItem compensationItem : compensationItems) {
			CompensationItemParamValidator.validateCompensationItem(compensationItem);
			CompensationDtl compensationDtl = CompensationItemMapper.mapToCompensationDtl(compensationItem);
			itemToDetailMap.put(compensationItem, compensationDtl);
		}
		// Link CompensationDtls based on links between CompensationItems
		for (CompensationItem compensationItem : itemToDetailMap.keySet()) {
			if (compensationItem.getLinkedCompensationItem() != null ) {
				itemToDetailMap.get(compensationItem).setLinkedPayrollItemDtl(
					itemToDetailMap.get(compensationItem.getLinkedCompensationItem()));
			}
		}			
		
		return itemToDetailMap.values();
	}
	
	/**
	 * Creates a CompensationDtl based on a CompensationItem.
	 * The compensation type is considered immutable and cannot be changed for an existing Dtl.
	 * 
	 * @param CompensationItem
	 * @param compensation
	 * @return
	 * @throws ServiceFailureException 
	 */
	public static CompensationDtl mapToCompensationDtl(CompensationItem compensationItem) throws ValidationException, ServiceFailureException {
		
		CompensationDtl compensationDtl = findOrCreateCompensationDtl(compensationItem);

		// Update the user friendly description
		compensationDtl.setDescription(compensationItem.getName());
		//Update the isFixedRate boolean
		compensationDtl.setFixedRate(compensationItem.isFixedRate());
		compensationDtl.setDeleted(compensationItem.isDeleted());
		
		compensationDtl.setCompensationType(mapToCompensationTypeEnum(compensationItem.getCompensationType()));
		compensationDtl.setPtoBaseRateType(mapPtoBaseRateType(compensationItem));
		
		if (compensationItem.isFixedRate()) {

			Datum compensationRate = compensationDtl.getRate();
			if (null == compensationRate) {
				compensationRate = new Datum();
				compensationDtl.setRate(compensationRate);
			}
			updateCompensationRate(compensationItem, compensationRate);
			
			Datum compensationQuantity = compensationDtl.getQuantity();
			updateCompensationQuantity(compensationItem, compensationQuantity);
		} else {
			// Check for user-supplied inputs
			if (compensationItem.getRate() != null) {
				Datum compensationRate = compensationDtl.getRate();
				if (null == compensationRate) {
					compensationRate = new Datum();
					compensationDtl.setRate(compensationRate);
				}
				updateCompensationRate(compensationItem, compensationRate);
			} else {
    			compensationDtl.setRate(null);
    			compensationDtl.setQuantity(null);
			}
		}

		return compensationDtl;
	}

	private static PtoBaseRateTypeEnum mapPtoBaseRateType(CompensationItem compensationItem) {
		switch (compensationItem.getPtoBaseRateType()) {
			case NOTSET:
				return PtoBaseRateTypeEnum.NOTSET;
			case PTO:
				return PtoBaseRateTypeEnum.PTO;
			case SICK:
				return PtoBaseRateTypeEnum.SICK;
			case VACATION:
				return PtoBaseRateTypeEnum.VACATION;

			default:
				return null;
		}
	}

	private static PtoBaseRateType mapPtoBaseRateType(CompensationDtl compensationDtl) {
		switch (compensationDtl.getPtoBaseRateType()) {
			case NOTSET:
				return PtoBaseRateType.NOTSET;
			case PTO:
				return PtoBaseRateType.PTO;
			case SICK:
				return PtoBaseRateType.SICK;
			case VACATION:
				return PtoBaseRateType.VACATION;

			default:
				return null;
		}
	}
	
	private static CompensationDtl findOrCreateCompensationDtl(CompensationItem compensationItem) throws ValidationException, ServiceFailureException {
		CompensationDtl compensationDtl;
		if (compensationItem.getObjectHandle() == null) {
			CompensationTypeEnum compensationType = 
				CompensationItemMapper.mapToCompensationTypeEnum(compensationItem.getCompensationType());
			compensationDtl = wageComponent.createCompensationDtl(compensationType);
		} else {
			Identifiable identifiable = 
				ObjectHandleMapper.mapToIdentifiable(compensationItem.getObjectHandle());
			compensationDtl = wageComponent.findCompensationDtl(identifiable); 
		}
		return compensationDtl;
	}

	private static void updateCompensationRate(CompensationItem compensationItem, Datum compensationRate) throws ValidationException {
		
		if (compensationItem.getRate() == null) {
			throw new ValidationException("Compensation Item rate cannot be null for fixed type");
		}

		if (compensationItem.getCompensationType().equals(CompensationType.Hourly)) {
			compensationRate.setValue1AsUSDMoneyAmount(compensationItem.getRate());
			// rate is in $ / 1 hour
			compensationRate.setType2(DatumType2Enum.HOUR);
			compensationRate.setValue2(new Double(1));
		} else if (compensationItem.getCompensationType().equals(CompensationType.Overtime) ||
			compensationItem.getCompensationType().equals(CompensationType.Doubletime)) {
			// caller will set linked compensation item
			// quantity is simply multiplier (e.g. 1.5 or 2)
			compensationRate.setValue1AsQuantity(compensationItem.getRate().doubleValue());
		} else {
		// This probably doesn't cover all the cases...
			compensationRate.setValue1AsUSDMoneyAmount(compensationItem.getRate());			
			compensationRate.setType2(DatumType2Enum.NOTAPPLICABLE);
		}
	}
	
	
	private static void updateCompensationQuantity(CompensationItem compensationItem, Datum compensationQuantity) {
		if (compensationQuantity != null) {
			compensationQuantity.setValue1AsQuantity(compensationItem.getUnits());	
		}
	}

	/**
	 * getWageItems returns an array of WageItems for a given worker
	 * It is assumed the worker object is not null and legal worker entity, it is assumed all validations happen
	 * prior to the method call
	 * @param worker Worker entity
	 * @return WageItem array
	 */
	public static CompensationItem[] mapToCompensationItems(Set<CompensationDtl> compensationDtls) {

		Set<CompensationItem> compensationItems = new TreeSet<CompensationItem>(new CompensationItemComparator());		
    	Map<CompensationDtl, CompensationItem> detailToItemMap = 
    		new HashMap<CompensationDtl, CompensationItem>(compensationDtls.size());
    	Map<PtoBaseRateTypeEnum, CompensationDtl> ptoTypetoDetailMap = new HashMap<PtoBaseRateTypeEnum, CompensationDtl>();
		for (CompensationDtl compensationDtl : compensationDtls) {
			if (compensationDtl.getPtoBaseRateType() == PtoBaseRateTypeEnum.PTO ||
					compensationDtl.getPtoBaseRateType() == PtoBaseRateTypeEnum.SICK ||
					compensationDtl.getPtoBaseRateType() == PtoBaseRateTypeEnum.VACATION) {
				ptoTypetoDetailMap.put(compensationDtl.getPtoBaseRateType(), compensationDtl);
			}
			CompensationItem compensationItem = mapToCompensationItem(compensationDtl);
			if (compensationItem != null) {
				compensationItems.add(compensationItem);
				detailToItemMap.put(compensationDtl, compensationItem);
			}
		}
    	//FIXME [LinkedCompensation]
		// Link CompensationItems based on links between CompensationDtls
		for (CompensationDtl compensationDtl : detailToItemMap.keySet()) {
			if (compensationDtl.getLinkedPayrollItemDtl() != null && 
					compensationDtl.getLinkedPayrollItemDtl() instanceof CompensationDtl) {
				CompensationDtl linkedCompensationDtl = (CompensationDtl)compensationDtl.getLinkedPayrollItemDtl();
					detailToItemMap.get(compensationDtl).setLinkedCompensationItem(
						detailToItemMap.get(linkedCompensationDtl));
			}
		}
		//PTO Rate linking
		for (CompensationDtl compensationDtl : detailToItemMap.keySet()) {
			if (compensationDtl.getCompensationType() == CompensationTypeEnum.SICKTAKEN ||
					compensationDtl.getCompensationType() == CompensationTypeEnum.VACTAKEN) {
				
				CompensationDtl baseRate = findBaseRateForPto(compensationDtl.getCompensationType(), ptoTypetoDetailMap);
				CompensationItem item = detailToItemMap.get(compensationDtl);
				item.setRate(baseRate.getRate().getValue1AsUSDMoneyAmount());
			}
		}
		return compensationItems.toArray(new CompensationItem[0]);
	}

	private static CompensationDtl findBaseRateForPto(CompensationTypeEnum compensationType,
			Map<PtoBaseRateTypeEnum, CompensationDtl> ptoTypetoDetailMap) {
		CompensationDtl result = ptoTypetoDetailMap.get(PtoBaseRateTypeEnum.PTO);
		if (result != null) {
			return result;
		}
		if (compensationType == CompensationTypeEnum.SICKTAKEN) {
			result = ptoTypetoDetailMap.get(PtoBaseRateTypeEnum.SICK);
		} else if (compensationType == CompensationTypeEnum.VACTAKEN) {
			result = ptoTypetoDetailMap.get(PtoBaseRateTypeEnum.VACATION);
		}
		return result;
	}

	public static CompensationItem mapToCompensationItem(CompensationDtl compensationDtl) {
		CompensationItem wageItem = new CompensationItem();
		wageItem.setObjectHandle(ObjectHandleMapper.mapToObjectHandle(compensationDtl));
		wageItem.setCompensationType(mapToCompensationType(compensationDtl.getCompensationType()));
		wageItem.setPtoBaseRateType(mapPtoBaseRateType(compensationDtl));
		// Send the user friendly description to the client
		wageItem.setName(compensationDtl.getDescription());
		wageItem.setFixedRate(compensationDtl.isFixedRate());

        Datum payrollDatum = compensationDtl.getRate();

        if (payrollDatum != null) {
            if (DatumType1Enum.MONEY == payrollDatum.getType1()) {
            	wageItem.setRate(payrollDatum.getValue1AsUSDMoneyAmount());
                //TODO [I18n] set the CurrencyCode
                DatumType2Enum datumType2Enum = payrollDatum.getType2();
                if (!datumType2Enum.equals(DatumType2Enum.NOTAPPLICABLE)) {
                    wageItem.setUnits(payrollDatum.getValue2());
                }
            } else if (DatumType1Enum.QUANTITY == payrollDatum.getType1()) {
                //TODO [I18n] set the CurrencyCode
            	if (compensationDtl.getCompensationType().equals(CompensationTypeEnum.HOURLYOVT) ||
            			compensationDtl.getCompensationType().equals(CompensationTypeEnum.HOURLYDBL) ||
            			compensationDtl.getCompensationType().equals(CompensationTypeEnum.SALARYOVT) ||
            			compensationDtl.getCompensationType().equals(CompensationTypeEnum.SALARYDBL) ) {
            		wageItem.setRate(new BigDecimal(payrollDatum.getValue1AsQuantity()));
            	} else {
            		wageItem.setUnits(payrollDatum.getValue1AsQuantity());
            	}
            } else if (DatumType1Enum.PERCENT == payrollDatum.getType1()) {
                //TODO [I18n] set the CurrencyCode
                wageItem.setRate(payrollDatum.getValue1AsUSDMoneyAmount());
                Double percentage = payrollDatum.getValue1AsPercentage();
                wageItem.setUnits(percentage / 100); // convert percentage to multiple 
            } else if (DatumType1Enum.TIME == payrollDatum.getType1()) {
                //TODO [impl] set the UnitOfTime and UnitOfTimeValue
            } else {
                String mesg = MessageFormat.format(
                		"Unsupported DatumType ''{0}'' found in CompensationDtl ''{1}'' with ID={1}",
                		payrollDatum.getType1().name(), compensationDtl.getName(), 
                		compensationDtl.getId());
                logger.error(mesg);
                throw new IllegalArgumentException(mesg);
            }
        }
        
        Datum quantityDatum = compensationDtl.getQuantity();
        
        if (quantityDatum != null && 
        		!DatumType1Enum.NOTSET.equals(quantityDatum.getType1()) && 
        		quantityDatum.getValue1AsQuantity() != null) {
        	wageItem.setUnits(quantityDatum.getValue1AsQuantity().doubleValue());
        }
        
        wageItem.setDeleted(compensationDtl.isDeleted());

		return wageItem;
	}


	private static EnumMap<CompensationTypeEnum, CompensationType> compensationTypeEnumMap =
		new EnumMap<CompensationTypeEnum, CompensationType>(CompensationTypeEnum.class);
	static {
		compensationTypeEnumMap.put(CompensationTypeEnum.BONUS, CompensationType.Bonus);
		compensationTypeEnumMap.put(CompensationTypeEnum.HOURLYREG, CompensationType.Hourly);
		compensationTypeEnumMap.put(CompensationTypeEnum.HOURLYOVT, CompensationType.Overtime);
		compensationTypeEnumMap.put(CompensationTypeEnum.HOURLYDBL, CompensationType.Doubletime);
		compensationTypeEnumMap.put(CompensationTypeEnum.SICKTAKEN, CompensationType.SickTaken);
		compensationTypeEnumMap.put(CompensationTypeEnum.VACTAKEN, CompensationType.VacationTaken);
		compensationTypeEnumMap.put(CompensationTypeEnum.TIPSPAYCHECK, CompensationType.TipsPaycheck);
		compensationTypeEnumMap.put(CompensationTypeEnum.TIPSALLOCATED, CompensationType.TipsAllocated);
		compensationTypeEnumMap.put(CompensationTypeEnum.TIPSCASH, CompensationType.TipsCash);
		compensationTypeEnumMap.put(CompensationTypeEnum.SALARYREG, CompensationType.Salary);
		compensationTypeEnumMap.put(CompensationTypeEnum.SALARYOVT, CompensationType.Overtime);
		compensationTypeEnumMap.put(CompensationTypeEnum.SALARYDBL, CompensationType.Doubletime);
		compensationTypeEnumMap.put(CompensationTypeEnum.COMMISSION, CompensationType.Commission);
		compensationTypeEnumMap.put(CompensationTypeEnum.PIECEWORK, CompensationType.Piecework);
		compensationTypeEnumMap.put(CompensationTypeEnum.REIMBURSEMENT, CompensationType.Reimbursement);
	}
    //Internal mapping method used by WageManagement APIs
    private static CompensationType mapToCompensationType(CompensationTypeEnum compensationType) {
    	return compensationTypeEnumMap.get(compensationType);
    }

    // TODO: currently have no way to map to salary overtime or salary double-time
    private static Map<CompensationType, CompensationTypeEnum> compensationTypeMap =
    	new HashMap<CompensationType, CompensationTypeEnum>();
    static {
    	compensationTypeMap.put(CompensationType.Salary, CompensationTypeEnum.SALARYREG);
    	compensationTypeMap.put(CompensationType.SickTaken, CompensationTypeEnum.SICKTAKEN);
    	compensationTypeMap.put(CompensationType.VacationTaken, CompensationTypeEnum.VACTAKEN);
    	compensationTypeMap.put(CompensationType.Hourly, CompensationTypeEnum.HOURLYREG);
    	compensationTypeMap.put(CompensationType.Overtime, CompensationTypeEnum.HOURLYOVT);
    	compensationTypeMap.put(CompensationType.Doubletime, CompensationTypeEnum.HOURLYDBL);
    	compensationTypeMap.put(CompensationType.Bonus, CompensationTypeEnum.BONUS);
    	compensationTypeMap.put(CompensationType.Commission, CompensationTypeEnum.COMMISSION);
    	compensationTypeMap.put(CompensationType.TipsPaycheck, CompensationTypeEnum.TIPSPAYCHECK);
    	compensationTypeMap.put(CompensationType.TipsAllocated, CompensationTypeEnum.TIPSALLOCATED);
    	compensationTypeMap.put(CompensationType.TipsCash, CompensationTypeEnum.TIPSCASH);
    	compensationTypeMap.put(CompensationType.Piecework, CompensationTypeEnum.PIECEWORK);
    	compensationTypeMap.put(CompensationType.Reimbursement, CompensationTypeEnum.REIMBURSEMENT);    	
    }
    //Internal mapping method used by WageManagement APIs
    public static CompensationTypeEnum mapToCompensationTypeEnum(CompensationType compensationType) {
    	CompensationTypeEnum compensationTypeEnum = compensationTypeMap.get(compensationType);
    	return (compensationTypeEnum == null) ? CompensationTypeEnum.NOTSET : compensationTypeEnum;
    }
    
    /*
     * Comparator for sorting compensation items.
     */
    private static class CompensationItemComparator implements Comparator<CompensationItem> {
        
        public int compare(CompensationItem wageItem1, CompensationItem wageItem2) {
            if (wageItem1.getCompensationType().ordinal() == wageItem2.getCompensationType().ordinal()) {
            	Long tmpId1 = new Long(wageItem1.getObjectHandle().getId());
            	Long tmpId2 = new Long(wageItem2.getObjectHandle().getId());
                return tmpId1.compareTo(tmpId2);
            } else { // handle > and < cases
            	return new Integer(wageItem1.getCompensationType().ordinal()).compareTo(
            			wageItem2.getCompensationType().ordinal());
            }
        }
    }

}
