package com.intuit.sb.payroll.services.wages.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;

import org.apache.log4j.Logger;
import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.compensation.CompensationComponent;
import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.services.common.impl.CommonValidators;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemRequest;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemResponse;
import com.intuit.sb.payroll.services.wages.WageService;
import com.intuit.sb.payroll.services.worker.impl.WorkerMapper;

/**
 *
 */
@Stateless(name = "WageService")
@SecurityDomain("BHPIAM")
@RolesAllowed({"Authenticated"})
public class WageServiceBean implements WageService {
	private static final Logger sLogger = PayrollLoggerFactory.makeLogger(WageServiceBean.class);

	@EJB
	private WorkerComponent workerComponent;
	
	@EJB
	private CompensationComponent wageComponent;
	
	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.services.wages.WageService#getCompensationItems(java.lang.String)
	 */
	public CompensationItem[] getCompensationItems(Long workerId) throws ServiceFailureException, ValidationException {
        Worker worker = workerComponent.findWorker(workerId);
		return CompensationItemMapper.mapToCompensationItems(worker.getCompensationDtls());
	}

	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.services.wages.WageService#saveCompensationItems(java.lang.String, com.intuit.sb.payroll.services.wages.SaveCompensationItemRequest[])
	 */
	public SaveCompensationItemResponse saveCompensationItems(Long workerId, SaveCompensationItemRequest[] compensationItemRequests)
		throws WorkflowException, ValidationException, ServiceFailureException {
		
		CommonValidators.validateId(workerId);
		
		if (compensationItemRequests == null) {
			//FIXME [I18N] resource translation
			throw new ServiceFailureException("The compensationItemRequests is null");
		}

        // We're performing discrete updates of the worker's compensation collection  
		// and hence it's safe to get the latest worker; buried updates are not a concern.
		Worker worker = workerComponent.findWorker(workerId);
		
		Map<CompensationDtl, SaveCompensationItemOperation> detailOperationMap = 
			CompensationItemMapper.mapToCompensationDtls(compensationItemRequests);
		
		List<CompensationDtl> compensationsToAdd = new ArrayList<CompensationDtl>();
		List<CompensationDtl> compensationsToDelete = new ArrayList<CompensationDtl>();
		
		for (CompensationDtl compensationDtl : detailOperationMap.keySet()) {
			SaveCompensationItemOperation operation = detailOperationMap.get(compensationDtl);
			if (operation.equals(SaveCompensationItemOperation.DELETE)) {
				checkLinkedCompensationItems(detailOperationMap);
				compensationsToDelete.add(compensationDtl);
			} 
			else if (operation.equals(SaveCompensationItemOperation.UPDATE)) {
				worker = wageComponent.updateCompensationOnWorker(compensationDtl, worker);
			}
			else if (operation.equals(SaveCompensationItemOperation.ADD)) {
				compensationsToAdd.add(compensationDtl);
			}
			/*
			else { do nothing }
			 * Updating an existing compensation doesn't change the worker hence we don't need to perform
			 * any operation on the worker with the Dtl. The act of having mapped the Item to the Dtl
			 * will have obtained the Dtl from the entity service and updated it. The persistence
			 * system will take care to automatically persist the updated Dtl entity.
			 */
		
		}
		
		if (compensationsToDelete.size() > 0) {
			worker = wageComponent.removeCompensationsFromWorker(compensationsToDelete, worker);
		}
		
		if (compensationsToAdd.size() > 0) {
			worker = wageComponent.addCompensationsToWorker(compensationsToAdd, worker);
		}
		
		SaveCompensationItemResponse saveCompensationItemResponse= new SaveCompensationItemResponse(WorkerMapper.mapFromEntityToPojo(worker), 
				CompensationItemMapper.mapToCompensationItems(worker.getCompensationDtls()));
		
		return saveCompensationItemResponse;
	}
	
	/**
	 * Ensure the request does not orphan any linked compensation items.
	 * @param compensationItemRequests specifies an array of SaveCompensationItemRequests for the worker. 
	 * Each array member specifies the operation and the CompensationItem it applies to.
	 * @throws ValidationException
	 */
	private void checkLinkedCompensationItems(Map<CompensationDtl, SaveCompensationItemOperation> detailOperationMap) 
			throws ServiceFailureException {

		for (CompensationDtl compensationDtl : detailOperationMap.keySet()) {
			SaveCompensationItemOperation operation = detailOperationMap.get(compensationDtl);
			if (operation.equals(SaveCompensationItemOperation.DELETE)) {
				List<CompensationDtl> linkedCompensationDtlList = 
					wageComponent.findLinkedCompensationDtls(compensationDtl);
				// iterate through all CompensationDtls pointing to this one
				for (CompensationDtl sourceDtl : linkedCompensationDtlList) {
					boolean error = false;
					// It's an error if the CompensationDtl pointing to this one is not also being deleted.
					if (!detailOperationMap.containsKey(sourceDtl)) {
						// It's not in the set at all
						error = true;
					} else if (!detailOperationMap.get(sourceDtl).equals(SaveCompensationItemOperation.DELETE)) {
						// It's in the set, but is not being deleted
						error = true;
					}
					if (error) {
						String message = "Cannot delete CompensationDtl when there are still other CompensationDtls linked to it";
						sLogger.error(message);
						throw new ServiceFailureException(message);						
					}
				}
			}
		}
	}
}