package com.intuit.sb.payroll.services.worker.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import com.intuit.sb.cdm.basetypes.EmailAddress;
import com.intuit.sb.cdm.basetypes.TelephoneNumber;
import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.benefits.BenefitsComponent;
import com.intuit.sb.payroll.capability.compensation.CompensationComponent;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.capability.pto.PaidTimeOffComponent;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.model.CAWagePlan;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Employee;
import com.intuit.sb.payroll.entity.model.EmploymentStatusEnum;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.HealthBenefitDtl;
import com.intuit.sb.payroll.entity.model.PaidTimeOffDtl;
import com.intuit.sb.payroll.entity.model.PayrollDistributionMethodEnum;
import com.intuit.sb.payroll.entity.model.PersonName;
import com.intuit.sb.payroll.entity.model.PersonTaxId;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.services.benefits.HealthBenefit;
import com.intuit.sb.payroll.services.benefits.impl.BenefitMapper;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.common.PhysicalAddress;
import com.intuit.sb.payroll.services.common.impl.PhysicalAddressMapper;
import com.intuit.sb.payroll.services.common.impl.WorkLocationMapper;
import com.intuit.sb.payroll.services.employer.AssistedStateType;
import com.intuit.sb.payroll.services.employer.impl.EmployerAssistedImpl;
import com.intuit.sb.payroll.services.pto.PTOInfo;
import com.intuit.sb.payroll.services.pto.impl.PTOMapper;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.impl.CompensationItemMapper;
import com.intuit.sb.payroll.services.worker.WorkerType;
import com.intuit.sb.payroll.services.workerpayments.PaycheckDistributionMethod;
import com.intuit.sb.payroll.services.workerpayments.impl.PayrollDistributionMethodMapper;
import com.intuit.sb.payroll.takehome.entity.TakehomeWorker;
import com.intuit.sb.payroll.takehome.entity.TakehomeWorkerEAO;
import com.intuit.sb.payroll.us.capability.address.impl.USPostalAddressImpl;

/**
 * Maps between Entity and POJO Workers.
 * 
 * $Author$ $Revision$ $DateTime$
 */
public class WorkerMapper {

	/**
	 * Constants that define the properties needed to make a worker ready to pay. This is a combination of minimally
	 * required set of properties required to cut a valid paycheck and those additional properties required by Gemini to
	 * improve the user experience.
	 */
	static private String paycheckDistributionMethodProperty = "paycheckDistributionMethod";
	static private String lastNameProperty = "lastName";
	static private String liveStateProperty = "liveState";
	static private String workStateProperty = "workState";
	static private String hireDateProperty = "hiredate";
	static private String firstNameProperty = "firstName";
	static private String wageItemsProperty = "wageItems";
	static private String withholdingProperty = "withholding";

	private static String trim(String string) {
		return string == null ? string : string.trim();
	}

	private static CompensationComponent wageComponent = ServiceLocator.getInstance().getService(
			CompensationComponent.JNDI_NAME);
	private static BenefitsComponent benefitsComponent = ServiceLocator.getInstance().getService(
			BenefitsComponent.JNDI_NAME);
	private static PaidTimeOffComponent paidTimeOffComponent = ServiceLocator.getInstance().getService(
			PaidTimeOffComponent.JNDI_NAME);
	/**
	 * Maps the Pojo Worker to an eiWorker object and returns it
	 * 
	 * @param worker
	 * @return
	 */
	static Worker mapFromPojoToWorker(com.intuit.sb.payroll.services.worker.Worker worker, 
			com.intuit.sb.payroll.entity.model.Worker workerEntity) throws ValidationException, 	ServiceFailureException, WorkflowException {

		// JobPositionNote
		workerEntity.setJobPositionNote(trim(worker.getJobTitle()));

		// Hire Date
		if (worker.getHireDate() != null) {
			DateRange dt = new DateRange();
			dt.setStartDate(worker.getHireDate());
			workerEntity.setEmploymentPeriod(dt);
			workerEntity.setActive(worker.isActive());
		}

		// Work Location
		com.intuit.sb.payroll.services.common.WorkLocation eiWorkLocation = worker.getWorkLocation();
		boolean workLocationProvided = eiWorkLocation != null && eiWorkLocation.getObjectHandle() != null;
		if (workLocationProvided) {
			EmployerComponent workerWorkLocationFunc = ServiceLocator.getInstance().getService(
					EmployerComponent.JNDI_NAME);
			com.intuit.sb.payroll.entity.model.WorkLocation workLocation = workerWorkLocationFunc
					.findWorkLocation(eiWorkLocation.getObjectHandle().getId());
			workerEntity.setWorkLocation(workLocation);
		}
		// If the worker's work location is null, set the entity's work location
		// to be null
		if (worker.getWorkLocation() == null) {
			workerEntity.setWorkLocation(null);
		}

		// Set Names
		PersonName pname = new PersonName();
		pname.setFirstName(trim(worker.getFirstName()));
		pname.setMiddleName(trim(worker.getMiddleName()));
		pname.setLastName(worker.getLastName() == null ? " " : worker.getLastName().trim());
		workerEntity.setPersonName(pname);

		// set Birthdate
		if (worker.getBirthDate() != null) {
			PayrollParamValidator.checkDateNotAfterToday(worker.getBirthDate());
			workerEntity.setDateOfBirth(worker.getBirthDate());
		}

		// set physical address
		com.intuit.sb.payroll.services.common.PhysicalAddress address = worker.getTaxResidenceAddress();
		if (address != null) {
			if (workerEntity.getTaxResidenceAddress() == null) {
				workerEntity.setTaxResidenceAddress(new GlobalPostalAddress());
			}

			GlobalPostalAddress taxAddressEntity = workerEntity.getTaxResidenceAddress();
			// Get the Corresponding USPostalAddress
			USPostalAddress taxAddressEntityProxy = AddressConvertor.getUSFromGlobal(taxAddressEntity,
					new USPostalAddressImpl());

			// Now Map the PhysicalAddress to the USAddressProxy
			PhysicalAddressMapper.mapAddress(address, taxAddressEntityProxy);

			// Now Modify the Entity Address With the Mapped USAddressProxy
			AddressConvertor.getGlobalFromUS(taxAddressEntityProxy, taxAddressEntity);

		} else {
			workerEntity.setTaxResidenceAddress(null);
		}

		// set phone numbers
		if (hasPhoneNumber(worker)) {
			TelephoneNumber homePhone = new TelephoneNumber();
			homePhone.setFreeFormNumber(trim(worker.getPhoneNumber()));
			workerEntity.setHomePhone(homePhone);
		} else {
			workerEntity.setHomePhone(null);
		}

		// set Email Address
		if (hasEmailAddress(worker)) {
			EmailAddress emailAddress = workerEntity.getEmailAddress();
			if (null == emailAddress) {
				emailAddress = new EmailAddress();
				workerEntity.setEmailAddress(emailAddress);
			}
			emailAddress.setAddress(trim(worker.getEmailAddress()));
		} else {
			workerEntity.setEmailAddress(null);
		}

		// Set paycheck distribution method
		// if the value is null, set the payrollDistributionMethod to not null
		if (worker.getPaycheckDistributionMethod() != null) {
			workerEntity.setPayrollDistributionMethod(PayrollDistributionMethodMapper.mapInterfaceToEntity(worker
					.getPaycheckDistributionMethod()));
		} else {
			workerEntity.setPayrollDistributionMethod(PayrollDistributionMethodMapper
					.mapInterfaceToEntity(PaycheckDistributionMethod.NOT_SET));
		}

		// Set the employment status
		// This overrides the status set by WorkerSetupRules.determineEmploymentStatus
		// as the person can be employed without a hire date since it has not been
		// filled in within Takehome yet. 
		if (worker.getEmploymentStatus() != null) {
			workerEntity.setEmploymentStatus(worker.getEmploymentStatus());
			
			// Also need to set to active as they are now employed.
			if (workerEntity.getEmploymentStatus() == EmploymentStatusEnum.EMPLOYED) {
				workerEntity.setActive(true);
			}
		}
		
		return workerEntity;
	}

	/**
	 * Maps the PayrollItems on a Pojo Worker to an entity Worker object.
	 * 
	 * @param worker
	 *            the Pojo worker with any associated PayrollItems.
	 * @param workerEntity
	 *            the target entity for the mapping.
	 * @return the entity with the payroll items mapped into it.
	 */
	static Worker mapFromPojoPayrollItemsToWorker(com.intuit.sb.payroll.services.worker.Worker worker,
			com.intuit.sb.payroll.entity.model.Worker workerEntity) throws ValidationException,
			ServiceFailureException, WorkflowException {

		// Handle Payroll Items. This functionality will deal with new and
		// existing items. It does not support deletes.
		if (worker.getCompensationItems() != null) {
			Collection<CompensationDtl> compensationDtls = CompensationItemMapper.mapToCompensationDtls(worker
					.getCompensationItems());
			// The compensationDtls returned above is not ordered, i.e. the overtime can be before the hourly item
			// Handle items that are not linked first
			for (CompensationDtl compensationDtl : compensationDtls) {
				if (null == compensationDtl.getLinkedPayrollItemDtl()) {
					if (compensationDtl.getId() == 0) {
						workerEntity = wageComponent.addCompensationToWorker(compensationDtl, workerEntity);
					} else {
						workerEntity = wageComponent.updateCompensationOnWorker(compensationDtl, workerEntity);
					}
				}
			}
			// Now handle the linked items
			for (CompensationDtl compensationDtl : compensationDtls) {
				if (null != compensationDtl.getLinkedPayrollItemDtl()) {
					if (compensationDtl.getId() == 0) {
						workerEntity = wageComponent.addCompensationToWorker(compensationDtl, workerEntity);
					} else {
						workerEntity = wageComponent.updateCompensationOnWorker(compensationDtl, workerEntity);
					}
				}
			}
		}

		if (worker.getHealthBenefits() != null) {
			List<HealthBenefitDtl> dtlSet = BenefitMapper.mapToHealthBenefitDtls(worker.getHealthBenefits());
			boolean foundAdd = false;
			boolean foundUpdate = false;
			for (HealthBenefitDtl benefitDtl : dtlSet) {
				if (benefitDtl.getId() == 0)
					foundAdd = true;
				else
					foundUpdate = true;
			}
			if (foundAdd && foundUpdate) {
				throw new ValidationException("Unable to add and update health benefits in the same request");
			}
			if (foundAdd) {
				workerEntity = benefitsComponent.addHealthBenefitDtls(workerEntity, dtlSet);
			} else if (foundUpdate) {
				workerEntity = benefitsComponent.updateHealthBenefitDtls(workerEntity.getId(), dtlSet);
			}
		}

		if (worker.getPtoInfos() != null) {
			for (PTOInfo ptoInfo : worker.getPtoInfos()) {
				PaidTimeOffDtl paidTimeOffDtl = PTOMapper.mapToPaidTimeOffDtl(workerEntity, ptoInfo);
				if (paidTimeOffDtl.getId() == 0) {
					workerEntity = paidTimeOffComponent.addPaidTimeOffDtl(workerEntity, paidTimeOffDtl);
				} else {
					workerEntity = paidTimeOffComponent.updatePaidTimeOffDtl(workerEntity, paidTimeOffDtl);
				}
			}
		}

		return workerEntity;
	}

	private static boolean hasEmailAddress(com.intuit.sb.payroll.services.worker.Worker eiContact) {
		if (eiContact.getEmailAddress() != null && eiContact.getEmailAddress().trim().length() > 0) {
			return true;
		}
		return false;
	}

	private static boolean hasPhoneNumber(com.intuit.sb.payroll.services.worker.Worker eiContact) {
		String phoneNumber = eiContact.getPhoneNumber();
		if (phoneNumber != null && phoneNumber.trim().length() > 0) {
			return true;
		}
		return false;
	}

	/**
	 * Maps a list of entity Workers to POJO API types.
	 * 
	 * @param workerList
	 * @return an array of Workers.
	 */
	public static com.intuit.sb.payroll.services.worker.Worker[] mapFromEntitiesToPojos(List<Worker> workerList)
			throws ValidationException, ServiceFailureException {
		com.intuit.sb.payroll.services.worker.Worker[] result = new com.intuit.sb.payroll.services.worker.Worker[0];

		if (workerList != null) {
			List<com.intuit.sb.payroll.services.worker.Worker> dtoList = new ArrayList<com.intuit.sb.payroll.services.worker.Worker>();
			for (Worker worker : workerList) {
				com.intuit.sb.payroll.services.worker.Worker mappedDto = WorkerMapper.mapFromEntityToPojo(worker);
				dtoList.add(mappedDto);
			}
			int responseSize = workerList.size();
			if (responseSize > 0) {
				result = dtoList.toArray(new com.intuit.sb.payroll.services.worker.Worker[responseSize]);
			}
		}
		return result;
	}
	
	private static TakehomeWorker getTakeHomeWorker(com.intuit.sb.payroll.services.worker.Worker worker) throws ValidationException, ServiceFailureException {
		TakehomeWorkerEAO workerEAO = ServiceLocator.getInstance().getService(TakehomeWorkerEAO.JNDI_NAME);
		TakehomeWorker	takehomeWorker = workerEAO.findByPayrollWorkerId(worker.getObjectHandle().getId());
		return takehomeWorker;
	}

	/**
	 * returns the eiWorker equivalent to the payroll Worker, including data from TakeHomeWorker
	 * 
	 * @param workerEntity
	 * @return Worker
	 */
	public static com.intuit.sb.payroll.services.worker.Worker mapFromEntityToPojo(Worker workerEntity)
			throws ValidationException, ServiceFailureException {
		if (workerEntity == null) {
			return null;
		}
		if (workerEntity.getId() == 0L) {
			throw new IllegalStateException("You can't map a transient entity");
		}
		com.intuit.sb.payroll.services.worker.Worker workerDto = new com.intuit.sb.payroll.services.worker.Worker();

		// set id
		Long id = workerEntity.getId();
		Long version = workerEntity.getVersion();
		ObjectHandle objectHandle = new ObjectHandle(id, version);

		workerDto.setObjectHandle(objectHandle);

		// Job Title
		workerDto.setJobTitle(workerEntity.getJobPositionNote());
		
		//SSN
		PersonTaxId personTaxId = workerEntity.getPersonTaxId();
		if (personTaxId != null) {
			workerDto.setWorkerTaxIdNumber(personTaxId.getValue());
		}
		
		workerDto.setMissingReadyToPayProperties(findMissingReadyToPayProperties(workerEntity));

		// Setup State
		boolean readyToPay = isReadyToPay(workerEntity);
		// APP override
		if (workerDto.getMissingReadyToPayProperties().length > 0) {
			readyToPay = false;
		}
		workerDto.setReadyToPay(readyToPay);
		workerDto.setActive(workerEntity.isActive());
		workerDto.setAlreadyPaid(workerEntity.getPaychecks().size() != 0);

		// WorkLocation
		com.intuit.sb.payroll.entity.model.WorkLocation workLocation = workerEntity.getWorkLocation();
		com.intuit.sb.payroll.services.common.WorkLocation eiWorkLocation = WorkLocationMapper
				.mapWorkLocation(workLocation);
		workerDto.setWorkLocation(eiWorkLocation);

		if (workerEntity instanceof Employee) {
			workerDto.setWorkerType(WorkerType.EMPLOYEE);
		} else {
			workerDto.setWorkerType(WorkerType.NON_EMPLOYEE);
		}

		// hire date
		DateRange dt = workerEntity.getEmploymentPeriod();
		if (dt != null) {
			workerDto.setHireDate(dt.getStartDate());
			workerDto.setTerminationDate(dt.getEndDate());
		}
		
		workerDto.setEmploymentStatus(workerEntity.getEmploymentStatus());

		// DoB
		workerDto.setBirthDate(workerEntity.getDateOfBirth());

		// name, address, email etc
		populateContactDetails(workerEntity, workerDto);

		// paycheck distribution method
		if (workerEntity.getPayrollDistributionMethod() != null) {
			workerDto.setPaycheckDistributionMethod(PayrollDistributionMethodMapper.mapEntityToInterface(workerEntity
					.getPayrollDistributionMethod()));
		}

		CompensationItem[] compensationItems = CompensationItemMapper.mapToCompensationItems(workerEntity.getCompensationDtls());
		if (compensationItems.length == 0) {
			compensationItems = null;
		}
		workerDto.setCompensationItems(compensationItems);

		PTOInfo[] ptoInfo = PTOMapper.mapToPTOInfos(workerEntity.getId(), workerEntity.getPaidTimeOffDtls());
		if (ptoInfo.length == 0) {
			ptoInfo = null;
		}
		workerDto.setPtoInfos(ptoInfo);

		HealthBenefit[] healthBenefits = BenefitMapper.mapToHealthBenefits(workerEntity.getHealthBenefitDtls());
		if (healthBenefits.length == 0) {
			healthBenefits = null;
		}
		workerDto.setHealthBenefits(healthBenefits);
		
		// Wage Plan code - default to S if not set
		String wagePlanCode = "S";
		for (WorkerComplianceDocDtl complianceDocDtl : workerEntity.getWorkerComplianceDocDtls()) {
			if (complianceDocDtl.getDocId().ordinal() == ComplianceDocIdEnum.DE545.ordinal()) {
				wagePlanCode = ((CAWagePlan)complianceDocDtl).getValue();
			}
		}
		workerDto.setWagePlanCode(wagePlanCode);

		TakehomeWorker takeHomeWorker = getTakeHomeWorker(workerDto);
		if (takeHomeWorker != null) {
			mapTakeHomeEntityToPojo(workerDto, takeHomeWorker);
		}
		return workerDto;
	}

	/*
	 * Determines if the worker is ready to pay or not.
	 */
	static private boolean isReadyToPay(Worker worker) {
		// Start by considering what the rules engine has to say
		boolean readyToPay = worker.isReadyForPay();
		boolean hasDeleted = true;
		if (!readyToPay)
			return false;
		/*
		 * Add in the offering specific properties that we also demand for pay ready
		 */
		// payment method
		if (worker.getPayrollDistributionMethod() == null
				|| worker.getPayrollDistributionMethod().equals(PayrollDistributionMethodEnum.NOTSET)) {
			return false;
		}
		// worker last name
		if (worker.getPersonName().getLastName() == null || worker.getPersonName().getLastName().length() == 0) {
			return false;
		}
		// Add in the requirement for at least one wage item that is not deleted
		if (worker.getCompensationDtls().size() == 0) {
			return false;
		}
		else
		{
			for(CompensationDtl compDtl : worker.getCompensationDtls())
			{
				if(!compDtl.isDeleted())
				{
					hasDeleted = false;
					break;
				}
			}
			return !hasDeleted;
		}
	}

	/*
	 * Determines the properties missing from the worker that are preventing us from being able to pay him/her.
	 */
	static private String[] findMissingReadyToPayProperties(Worker worker) throws ServiceFailureException {
		/*
		 * Start with the properties the rules have marked as missing
		 */
		List<String> missingProperties = new ArrayList<String>();
		// The state of the worker's home address
		GlobalPostalAddress address = worker.getTaxResidenceAddress();
		if (address == null) {
			missingProperties.add(liveStateProperty);
		} else {
			USPostalAddress usAddress = new USPostalAddressImpl();
			AddressConvertor.getUSFromGlobal(address, usAddress);
			if (usAddress.getRegionOrState() == null || usAddress.getRegionOrState().length() == 0) {
				missingProperties.add(liveStateProperty);
			}
		}
		// The state of the worker's work location
		if (worker.getWorkLocation() == null || worker.getWorkLocation().getTaxResidenceAddress() == null) {
			missingProperties.add(workStateProperty);
		} else {
			address = worker.getWorkLocation().getTaxResidenceAddress();
			USPostalAddress usAddress = new USPostalAddressImpl();
			AddressConvertor.getUSFromGlobal(address, usAddress);
			if (usAddress.getRegionOrState() == null || usAddress.getRegionOrState().length() == 0) {
				missingProperties.add(workStateProperty);
			}
		}
		// hire date
		if (worker.getEmploymentPeriod() == null) {
			missingProperties.add(hireDateProperty);
		}
		// first name
		if (worker.getPersonName().getFirstName() == null || worker.getPersonName().getFirstName().length() == 0) {
			missingProperties.add(firstNameProperty);
		}

		/*
		 * Add in the offering specific properties that we also demand for pay ready
		 */
		// Add in payment method
		if (worker.getPayrollDistributionMethod() == null
				|| worker.getPayrollDistributionMethod().equals(PayrollDistributionMethodEnum.NOTSET)) {
			missingProperties.add(paycheckDistributionMethodProperty);
		}
		// Add in worker last name
		if (worker.getPersonName().getLastName() == null || worker.getPersonName().getLastName().length() == 0) {
			missingProperties.add(lastNameProperty);
		}
		// Add in the requirement for at least one wage item that is not deleted
		boolean hasDeleted = true;
		if (worker.getCompensationDtls().size() == 0) {
			missingProperties.add(wageItemsProperty);
		}
		else
		{
			for(CompensationDtl compDtl : worker.getCompensationDtls())
			{
				if(!compDtl.isDeleted())
				{
					hasDeleted = false;
					break;
				}
			}
			if(hasDeleted)
			{
				missingProperties.add(wageItemsProperty);
			}
		}
		//Add in witholding property
		AssistedStateType employerAssistedType = EmployerAssistedImpl.getAssistedState(); 
		PersonTaxId workerTaxIDNumber = worker.getPersonTaxId();
		if(employerAssistedType != AssistedStateType.NOT_ASSISTED && workerTaxIDNumber == null) {
			missingProperties.add(withholdingProperty);
		}
		

		return missingProperties.toArray(new String[0]);
	}

	/**
	 * Populate the platform portion of the eiWorker from a WorkerRole.
	 * 
	 * Assumes we are in a Tx.
	 * 
	 * @param workerRole
	 * @param workerDto
	 */
	static void populateContactDetails(com.intuit.sb.payroll.entity.model.Worker worker,
			com.intuit.sb.payroll.services.worker.Worker workerDto) {
		if (worker == null || workerDto == null) // nothing to populate
		{
			return;
		}

		PersonName person = worker.getPersonName();
		workerDto.setFirstName(person.getFirstName());
		workerDto.setMiddleName(person.getMiddleName());
		workerDto.setLastName(person.getLastName() == null ? "" : person.getLastName().toString().trim());

		GlobalPostalAddress entityAddress = worker.getTaxResidenceAddress();
		USPostalAddress usAddress = null;
		if (entityAddress != null) {
			usAddress = AddressConvertor.getUSFromGlobal(entityAddress, new USPostalAddressImpl());

		}

		PhysicalAddress physicalAddress = PhysicalAddressMapper.mapAddress(usAddress);
		workerDto.setTaxResidenceAddress(physicalAddress);

		if (worker.getHomePhone() != null) {
			workerDto.setPhoneNumber(worker.getHomePhone().getFreeFormNumber());
		}

		EmailAddress email = worker.getEmailAddress();

		if (email != null) {
			workerDto.setEmailAddress(email.getAddress());
		}
	}

	public static void mapTakeHomeEntityToPojo(com.intuit.sb.payroll.services.worker.Worker savedWorker,
			TakehomeWorker takehomeWorker) {
		Long takehomeWorkerId = takehomeWorker.getId();
		Long takehomeWorkerVersion = takehomeWorker.getVersion();
		ObjectHandle takehomeWorkerObjectHandle = new ObjectHandle(takehomeWorkerId, takehomeWorkerVersion);
		savedWorker.setTakehomeWorkerObjectHandle(takehomeWorkerObjectHandle);
		savedWorker.setAvatar(takehomeWorker.getAvatar());
		savedWorker.setWorkerImage(takehomeWorker.getWorkerImage());
	}
}
