package com.intuit.sb.payroll.services.workerpayments;

import java.io.Serializable;
import java.math.BigDecimal;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

import com.intuit.sb.payroll.services.wages.CompensationItem;

/**
 * A single paycheck line item. These can include any kind of line item
 * including wages, taxes, deductions, paid time off, etc.
 * 
 * Each line item comprises a value (the amount the line item corresponds to)
 * and a title (a human readable textual label for the line item).
 */
public class PaycheckLineItem implements Serializable {

	private static final long serialVersionUID = 7348934648143760550L;
	private String title;
	private BigDecimal amount;
	private BigDecimal ytd;
	private BigDecimal rate;
	private double units;
	private PayrollItemCategoryType payrollItemCategory;
	private CompensationItem compensationItem;
	private String lawId = null;
	private long payrollItemId;

	/**
	 * No args constructor.
	 */
	public PaycheckLineItem() {
		super();
	}

	/**
	 * Maximal constructor.
	 * 
	 * @param title
	 *            the human readable title of this paycheck line item
	 * @param amount
	 *            the amount of this paycheck line item
	 * @param category
	 *            the payroll item category type of this paycheck line item
	 * @param compensationItem
	 *            the compensation item for this paycheck line item
	 */
	public PaycheckLineItem(String title, BigDecimal amount, PayrollItemCategoryType category,
	                CompensationItem compensationItem) {
		this();
		this.title = title;
		this.amount = amount;
		// Set to 2 decimal places
		this.amount = this.amount.setScale(2);
		this.compensationItem = compensationItem;
		this.payrollItemCategory = category;
		this.lawId = null;
	}

	/**
	 * Maximal constructor.
	 * 
	 * @param title
	 *            the human readable title of this paycheck line item
	 * @param amount
	 *            the amount of this paycheck line item
	 * @param category
	 *            the payroll item category type of this paycheck line item
	 * @param compensationItem
	 *            the compensation item for this paycheck line item
	 * @param lawId
	 *            the law id for this paycheck line item
	 */
	public PaycheckLineItem(String title, BigDecimal amount, PayrollItemCategoryType category,
	                CompensationItem compensationItem, String lawId) {
		this(title, amount, category, compensationItem, lawId, amount, null, 0);
	}

	/**
	 * Maximal constructor.
	 * 
	 * @param title
	 *            the human readable title of this paycheck line item
	 * @param amount
	 *            the amount of this paycheck line item
	 * @param category
	 *            the payroll item category type of this paycheck line item
	 * @param compensationItem
	 *            the compensation item for this paycheck line item
	 * @param lawId
	 *            the law id for this paycheck line item
	 * @param ytdAmount
	 *            the YTD amount for this paycheck line item
	 */
	public PaycheckLineItem(String title, BigDecimal amount, PayrollItemCategoryType category,
	                CompensationItem compensationItem, String lawId, BigDecimal ytdAmount) {
		this(title, amount, category, compensationItem, lawId, ytdAmount, null, 0);
	}

	/**
	 * Maximal constructor.
	 * 
	 * @param title
	 *            the human readable title of this paycheck line item
	 * @param amount
	 *            the amount of this paycheck line item
	 * @param category
	 *            the payroll item category type of this paycheck line item
	 * @param compensationItem
	 *            the compensation item for this paycheck line item
	 * @param lawId
	 *            the law id for this paycheck line item
	 * @param ytdAmount
	 *            the YTD amount for this paycheck line item
	 * @param rate
	 *            the rate for this paycheck line item
	 * @param units
	 *            the quantity/units for this paycheck line item
	 */
	public PaycheckLineItem(String title, BigDecimal amount, PayrollItemCategoryType category,
	                CompensationItem compensationItem, String lawId, BigDecimal ytdAmount, BigDecimal rate,
	                double units) {
		this();
		this.title = title;
		this.amount = amount;
		// Set to 2 decimal places
		this.amount = this.amount.setScale(2);
		this.compensationItem = compensationItem;
		this.payrollItemCategory = category;
		this.lawId = lawId;
		this.ytd = ytdAmount;
		this.rate = rate;
		this.units = units;
	}

	/**
	 * @return the human readable title of this paycheck line item
	 */
	public String getTitle() {
		return title;
	}

	/**
	 * @param title
	 *            the human readable title of this paycheck line item
	 */
	public void setTitle(String title) {
		this.title = title;
	}

	/**
	 * @return the amount of this paycheck line item
	 */
	public BigDecimal getAmount() {
		return amount;
	}

	/**
	 * @param amount
	 *            the amount of this paycheck line item
	 */
	public void setAmount(BigDecimal amount) {
		this.amount = amount;
		// Set to 2 decimal places
		this.amount = this.amount.setScale(2);
	}

	/**
	 * @return PayrollItemCategoryType of payrollItem.
	 */
	public PayrollItemCategoryType getPayrollItemCategory() {
		return payrollItemCategory;
	}

	/**
	 * Sets the payItemCategory
	 * 
	 * @param category
	 */
	public void setPayrollItemCategory(PayrollItemCategoryType category) {
		this.payrollItemCategory = category;
	}

	/**
	 * @return compensationItem
	 */
	public CompensationItem getCompensationItem() {
		return this.compensationItem;
	}

	/**
	 * Sets the compensationItem
	 * 
	 * @param compensationItem
	 */
	public void setCompensationItem(CompensationItem compensationItem) {
		this.compensationItem = compensationItem;
	}

	/**
	 * 
	 * @return lawId
	 */
	public String getLawId() {
		return lawId;
	}

	/**
	 * Sets the lawId
	 * 
	 * @param lawId
	 */
	public void setLawId(String lawId) {
		this.lawId = lawId;
	}
	
	public BigDecimal getYtd() {
		return ytd;
	}

	public void setYtd(BigDecimal ytd) {
		this.ytd = ytd;
	}

	public BigDecimal getRate() {
		return rate;
	}

	public void setRate(BigDecimal rate) {
		this.rate = rate;
	}

	public double getUnits() {
		return units;
	}

	public void setUnits(double units) {
		this.units = units;
	}

	@Override
	public boolean equals(Object other) {
		return EqualsBuilder.reflectionEquals(this, other);
	}

	@Override
	public int hashCode() {
		return HashCodeBuilder.reflectionHashCode(this);
	}

	@Override
	public String toString() {
		return ToStringBuilder.reflectionToString(this, ToStringStyle.MULTI_LINE_STYLE);
	}

	public void setPayrollItemId(long payrollItemId) {
		this.payrollItemId = payrollItemId;
	}

	public long getPayrollItemId() {
		return payrollItemId;
	}
}