package com.intuit.sb.payroll.services.workerpayments;

import javax.ejb.Remote;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;

/**
 * This interface contains apis for managing or retrieving Company specific information
 * <p>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 */
@Remote
public interface WorkerPaymentService
{
	@SuppressWarnings("nls")
    public static final String JNDI_NAME = "WorkerPaymentService/remote";

	/**
     * This method can be called during the PayrollSchedule setup Task when a user is trying to create a new PayrollSchedule
     *
     * Creates a new PayrollSchedule if one doesn't exists, if one exist it returns the already existing PayrollSchedule. All
     * employees are assigned to this PayrollSchedule irrespective of their status
     * <p>
     * Returns a sample payrollSchedule based on the company type (if no PayrollSchedule exists) with following criteria <br>
     * <pre>
     * payDate              : The next upcoming Friday after the current day.
     * payrollPayDateOffset : 2 days before the Friday selected (Wed).
     * frequency            : Bi-Weekly (every 2 weeks).
     * periodEndDate        : The day before the run payroll date (Tues).
     * periodStartDate      : Shown on calendar as Wed 2 weeks before per end date and pay frequency of bi-weekly.
     * <pre>
     *
     * @return a payrollSchedule
     *
     * @throws ServiceFailureException if it is not able to create a new PayrollSchedule Or
     *      more than one payrollSchedule exist in the system
     *
     */
    public PayrollSchedule getPayrollSchedule() throws ServiceFailureException;

    /**
     * Returns the Pay period information for the next "Uncommitted" PayrollRun.
     * @return
     * @throws ValidationException
     * @throws ServiceFailureException
     */
    public PayrollRunInfo getNextScheduledPayrollRunInfo() throws ValidationException, ServiceFailureException;
    	
    /**
     * This method will create a PayrollSchedule.
     *
     * @param payrollSchedule the new PayrollSchedule
     *
     * @return new PayrollSchedule
     *
     * @throws ServiceFailureException for any infrastructure issues.
     * @throws ValidationException if invalid end date is provided, illegal end date could be a date before endDate
     *
     */
    public PayrollSchedule createPayrollSchedule(PayrollSchedule payrollSchedule) throws ServiceFailureException, ValidationException;
    
    /**
     * This method will update existing PayrollSchedule. The PayrollScheduleId is the unique identifier to identify the
     * PayrollSchedule to update the existing payrollSchedule. <br>
     * This API will not bother about any existing payroll templates, it is up to the application to
     * ensure there are no draft payrolls (there will be a seperate API provided by the PayrollRun to ensure this)
     *
     * @param payrollSchedule the user need to update
     *
     * @return updated PayrollSchedule
     *
     * @throws InvalidPayrollScheduleException if invalid payroll schedule is provide ( parameters missing)
     * @throws IllegalEndDateException if invalid end date is provided, illegal end date could be a date before endDate
     * @throws ValidationException
     *
     */
    public PayrollSchedule updatePayrollSchedule(PayrollSchedule payrollSchedule) throws ServiceFailureException, ValidationException;

    /**
     * Based on request parameters, calculates the full Paycheck details with line items for all the deductions and
     * additions.
     *
     * @param request the CalcRequest
     * @return the Details of the Paycheck.
     * @throws ValidationException when input parameters are missing or invalid.
     * @throws WorkflowException when the given Worker is not ready to pay.
     * @throws ServiceFailureException 
     */
    public PaycheckDetails calcPaycheckDetails(PaycheckRequest request) throws ValidationException, WorkflowException, ServiceFailureException;

    /**
     * 
     * @param paycheckId
     * @return
     * @throws ServiceFailureException
     * @throws ValidationException
     */
    public PaycheckDetails getPaycheckById(Long paycheckId) throws ServiceFailureException, ValidationException;
    
    /**
     * Retrieves a PayrollRunSummary given the PayrollRun unique identifier. This id will typically can be acquired from
     * a Task or other form of reminder.
     *
     * @param payrollRunId the unique identifier for the PayrollRun.
     * @return a PayrollRunSummary
     * @throws ValidationException if payrollRunId is null or empty or invalid format for the ID
     * @throws PayrollRunNotFoundException if the PayrollRunId does not exist
     */
    public PayrollRunResponse getPayrollRunById(Long payrollRunId) throws ServiceFailureException, ValidationException;

    /**
     * Retrieves recent scheduled, committed, payroll runs. The runs are selected (and ordered) based on paycheck date, newest
     * to oldest. The result set is limited to the most recent N runs as specified by numberOfRuns. If N is greater
     * than the total number of scheduled payrolls for the company the results are limited to all the existing runs. 
     *
     * @param numberOfRuns the number of payroll runs to retrieve.
     * @return an array of PayrollRunInfo, one for each of the most recent payroll runs. PayrollRunInfo[0] is the
     * most recent run and PayrollRunInfo[numberOfRuns - 1] is the oldest selected run. The list will be empty of
     * no committed payroll runs exist for the company.
     * @throws ValidationException if numberOfRuns <= 0.
     * @throws ServiceFailureException Something bad happened.
     */
    public PayrollRunInfo[] getRecentScheduledPayrollRuns(int year) throws ServiceFailureException, ValidationException;

    /**
     * Get them all!
     * @return an array of PayrollRunResponses.
     * @throws ServiceFailureException
     */
    public PayrollRunInfo[] getAllPayrollRuns() throws ServiceFailureException;
    
    /**
     * Retrieves a PayrollRunResponce given the PayrollRun unique identifier. This id will typically can be acquired
     * from a Task or other form of reminder.
     *
     * @param payrollRunId the unique identifier for the PayrollRun.
     * @return a PayrollRunResponse
     * @throws ValidationException if payrollRunId is null or empty or invalid format for the ID
     * @throws PayrollRunNotFoundException if the PayrollRunId does not exist
     */
    public PayrollRunSummary getPayrollRunSummaryById(Long payrollRunId) throws ValidationException, ServiceFailureException;
    
    
    /**
     * Creates a PayrollRun for given Employees. The created PayrollRun will be in Draft state
     * until committed. Set the PayrollRunType to "TERMINATION" if it is a Termination check.
     * @param payrollRunCreateRequest
     * @return
     * @throws ValidationException
     * @throws ServiceFailureException
     */
    public PayrollRunResponse createPayrollRun(PayrollRunCreateRequest payrollRunCreateRequest) throws ValidationException, ServiceFailureException;

    /**
     * Commits a PayrollRun and places the PayrollRun in a Completed state. This has the effect of also setting the
     * PayrollRunTask that is associated with this to be marked Complete.
     *
     * @param payrollRun
     * @return a PayrollRunSaveResponse including the PayrollRunSummary that has summary details of this PayrollRun and
     * the updated Workers included in the run.
     * @throws ValidationException
     * @throws WorkflowException 
     */
    public PayrollRunSaveResponse commitPayrollRun(PayrollRunSaveRequest payrollRun) throws ValidationException, ServiceFailureException, WorkflowException;

    /**
     * Allows for the update of selected fields in a PayrollRun such as check number and check status.
     * 
     * @param updateRequest
     * @return a PayrollRunSummary instance with the updates.
     * @throws ValidationException
     * @throws ServiceFailureException
     * @throws WorkflowException
     */
    public PayrollRunSummary updatePayrollRun(PayrollRunUpdateRequest updateRequest) throws ValidationException, ServiceFailureException, WorkflowException;
    
    /**
     * Void one or more paychecks in a single payroll run.  If the given paychecks are in 
     * different payroll runs, a ValidationException is thrown.
     * @param voidPaychecksRequest
     * @return
     * @throws ValidationException
     * @throws ServiceFailureException
     * @throws WorkflowException
     */
    public PayrollRunSummary voidPaychecks(Long[] paycheckIds) throws ValidationException, ServiceFailureException, WorkflowException;

    /**
     * Saves a PayrollRun in Draft State. Sets the Task state to incomplete.
     *
     * @param payrollRun
     * @return a PayrollRunSummary that has summary details of this draft PayrollRun.
     * @throws ValidationException
     */
    public PayrollRunSummary saveDraftPayrollRun(PayrollRunSaveRequest payrollRun) throws ValidationException, ServiceFailureException;

    /**
     * Cancels a PayrollRun given the PayrollRun unique identifier. This id will typically can be acquired from
     * a Task or other form of reminder or from the payroll response created due to saving as draft or commiting a
     * payroll run.
     *
     * @param payrollRunId the unique identifier for the PayrollRun.
     * @throws ValidationException if payrollRunId is null or empty or invalid format for the ID
     * or if the payrollrun is already commited
     * @throws PayrollRunNotFoundException if the PayrollRunId does not exist
     */
    public void cancelPayrollRunById(Long payrollRunId) throws ValidationException, ServiceFailureException;

    /**
     * Creates a CSV file with the PayrollRun information, including full Paycheck Details.
     * 
     * @param payrollRunId
     * @return
     * @throws ValidationException
     * @throws ServiceFailureException
     */
    public String getPayrollRunCSVById(Long payrollRunId) throws ValidationException, ServiceFailureException;
}
