package com.intuit.sb.payroll.task.impl;

import java.util.Date;

import javax.ejb.EJB;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.capability.employer.impl.EmployerComponentTasks;
import com.intuit.sb.payroll.capability.worker.impl.WorkerComponentTasks;
import com.intuit.sb.payroll.capability.workerpayments.impl.WorkerPaymentsComponentTasks;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.TaskStatusEnum;
import com.intuit.sb.payroll.entity.model.TaskTypeEnum;
import com.intuit.sb.payroll.entity.task.TaskEAO;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunPeriod;
import com.intuit.sb.payroll.task.Task;
import com.intuit.sb.payroll.task.TaskStatus;
import com.intuit.sb.payroll.task.TaskType;

/**
 * Utility class to map EntityInterface Task to DataEntity Task and vice versa 
 * 
 * $Author: rjohn2 $
 * $Revision: #1 $ 
 * $DateTime: 2008/11/06 12:49:53 $
 */
public class TaskMapper {

	private final static int ALERT_PRIORITY = 0;
	private final static int OVERDUE_RUN_PAYROLL_PRIORITY = 11;
	private final static int ADD_FIRST_EMPLOYEE_PRIORITY = 21;
	private final static int FINISH_EMPLOYEE_SETUP_PRIORITY = 31;
	private final static int COMPANY_SETUP_PRIORITY = 41;
	private final static int ADD_WORKLOCATION_PRIORITY = 51;
	private final static int ADD_PAYROLLSCHEDULE_PRIORITY = 61;
	private final static int RUN_PAYROLL_PRIORITY = 71;
	private final static int ADD_WITHHOLDING_PRIORITY = 81;
	
	@EJB
	private TaskEAO taskEao;
	
	private TaskMapper() {}

    /**
     * This utility method converts Data Entity Task to EntityInterface Task
     * 
     * @param task Data Entity Task to map from
     * @return EntityInterface Task
     */
    public static com.intuit.sb.payroll.task.Task mapFromDEntityToEI(com.intuit.sb.payroll.entity.model.Task task)
    {
        if (task == null)
            return null;
        
        com.intuit.sb.payroll.task.Task eITask = new com.intuit.sb.payroll.task.Task();

        ObjectHandle objectHandle = new ObjectHandle(task.getId(), task.getVersion());
        eITask.setObjectHandle(objectHandle);
        
        eITask.setDescription(task.getDescription());
        //eITask.setMessageResourceId(messageResourceId); //TODO [rj]- fix this later - where will I get this info?
        //[th] coming soon!
        eITask.setName(task.getName());
        
        TaskStatusEnum status = task.getStatus();

        switch(status)
        {
        	case CANCELED:
                eITask.setStatus(TaskStatus.CANCELLED);
        		break;
        	case COMPLETED:
                eITask.setStatus(TaskStatus.COMPLETE);
        		break;
        	case NOTSET:
                eITask.setStatus(TaskStatus.NEW); //TODO - [rj] is this right, or should we need a not set attribute exposed to the clients
        		break;
        	case NOTSTARTED:
                eITask.setStatus(TaskStatus.NEW);
        		break;
        	case STARTED:
                eITask.setStatus(TaskStatus.IN_PROGRESS);
        		break;
        }
        
        TaskTypeEnum taskType = task.getType();
        
        switch(taskType)
        {
	        case SCHEDULED:
	            eITask.setType(TaskType.SCHEDULED);
	        	break;
	        case NONSCHEDULED:
	            eITask.setType(TaskType.NON_SCHEDULED);
	        	break;
	        case SETUP:
	            eITask.setType(TaskType.SETUP);
	        	break;
	        case DEADLINE:
	            eITask.setType(TaskType.REMINDER);
	        	break;
        }
        
        //set the due date
        eITask.setDueDate(task.getDeadline());
        
        DateRange dateRange = task.getDateRange();
        if (dateRange != null){
            PayrollRunPeriod payrollRunPeriod = new PayrollRunPeriod();
            payrollRunPeriod.setPeriodEndDate(dateRange.getEndDate());
            payrollRunPeriod.setPeriodStartDate(dateRange.getStartDate());
            eITask.setPayrollRunPeriod(payrollRunPeriod);
        }
        
        // Set the priority of the task
        int priority = getPriorityOfTask(task);
        eITask.setPriority(priority);
        
        // For identifying the entity associated with the task
        if (task.getPayload() != null){
        	eITask.setEntityId(task.getPayload().longValue());
        }
        
        
        return eITask;
    }
    
    
    private static int getPriorityOfTask(com.intuit.sb.payroll.entity.model.Task task){
    	if (task == null){
    		return 100;
    	}
    	Date today = new Date();
    	// All setup related tasks without any deadlines
    	if (task.getName().equals(WorkerComponentTasks.SETUP_FIRST_EMPLOYEE_TASK)){
    		return ADD_FIRST_EMPLOYEE_PRIORITY;
    	}else if (task.getName().equals(WorkerComponentTasks.WORKER_PAYROLL_READY_TASK)){
    		return FINISH_EMPLOYEE_SETUP_PRIORITY;
    	}else if (task.getName().equals(WorkerComponentTasks.WORKER_WITHHOLDING_TASK)){
    		return ADD_WITHHOLDING_PRIORITY;
    	}else if (task.getName().equals(EmployerComponentTasks.COMPANY_SETUP_TASK)){
    		return COMPANY_SETUP_PRIORITY;
    	}else if (task.getName().equals(EmployerComponentTasks.PAYROLLSCHEDULE_SETUP_TASK)){
    		return ADD_PAYROLLSCHEDULE_PRIORITY;
    	}else if (task.getName().equals(EmployerComponentTasks.WORKLOCATION_SETUP_TASK)){
    		return ADD_WORKLOCATION_PRIORITY;
    	}else if (task.getName().equals(WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK)){
    		return task.getDeadline().before(today) ? OVERDUE_RUN_PAYROLL_PRIORITY : RUN_PAYROLL_PRIORITY;
    	}else{
    		return ALERT_PRIORITY;
    	}

    }

    /**
     * This utility method converts Entity Interface Task to Data Entity Task
     * 
     * @param eiTask Entity Interface Task to map from
     * @return Data Entity Task
     */
    public static com.intuit.sb.payroll.entity.model.Task mapFromEIToDEntity(com.intuit.sb.payroll.task.Task eiTask) throws ServiceFailureException
    {
    	if (eiTask == null)
            return null;

        ObjectHandle handle = eiTask.getObjectHandle();
        
        com.intuit.sb.payroll.entity.model.Task task = null;
        
        if(handle != null)//fetch the task from the data store
        {
        	TaskMapper mapper = new TaskMapper();
        	task = mapper.getEntity(eiTask);
        }
        else //create a brand new one
        {
            task = new com.intuit.sb.payroll.entity.model.Task();
        }

        //map the task type
        TaskType type = eiTask.getType();
        
        if(type.equals(TaskType.SCHEDULED))
        {
        	task.setType(TaskTypeEnum.SCHEDULED);
        }
        else if(type.equals(TaskType.NON_SCHEDULED))
        {
        	task.setType(TaskTypeEnum.NONSCHEDULED);
        }
        else if(type.equals(TaskType.SETUP))
        {
        	task.setType(TaskTypeEnum.SETUP);
        }
        else if(type.equals(TaskType.REMINDER))
        {
        	task.setType(TaskTypeEnum.DEADLINE);
        }
        
        task.setName(eiTask.getName());
        task.setDescription(eiTask.getDescription());
        
        //map the task status
        TaskStatus status = eiTask.getstatus();
        
        if(status == TaskStatus.CANCELLED)
        {
        	task.setStatus(TaskStatusEnum.CANCELED);
        }
        else if(status == TaskStatus.COMPLETE)
        {
        	task.setStatus(TaskStatusEnum.COMPLETED);
        }
        else if(status == TaskStatus.IN_PROGRESS)
        {
        	task.setStatus(TaskStatusEnum.STARTED);
        }
       
        else if(status == TaskStatus.NEW)
        {
        	task.setStatus(TaskStatusEnum.NOTSTARTED);
        }
        else
        {
        	task.setStatus(TaskStatusEnum.NOTSET);
        }
        
        //every Task created in the system using the Composite APIs are user managable
        task.setUserManagable(true);
    	
        task.setDeadline(eiTask.getDueDate());
        
        PayrollRunPeriod payrunPeriod = eiTask.getPayrollRunPeriod();
        if (payrunPeriod != null){
        	 DateRange dateRange = new DateRange();
             dateRange.setEndDate(payrunPeriod.getPeriodEndDate());
             dateRange.setStartDate(payrunPeriod.getPeriodStartDate());
             task.setDateRange(dateRange);
        }
       
        return task;
    }
    
    private com.intuit.sb.payroll.entity.model.Task getEntity(Task taskEi) throws ServiceFailureException {
    	ObjectHandle taskHandle = taskEi.getObjectHandle();
    	com.intuit.sb.payroll.entity.model.Task task = taskEao.findById(taskHandle.getId());
    	if (task == null) {
    		//TODO [rj] - get this string into a resource bundle
    		throw new ServiceFailureException("Task could not be found in the system");
    	}
    	return task;
    }
    
    
    
}
