package com.intuit.sb.payroll.services.benefits;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import org.testng.annotations.BeforeClass;

import com.intuit.sb.payroll.entity.benefits.HealthBenefitEAO;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.services.benefits.impl.BenefitMapper;
import com.intuit.sb.payroll.services.common.EntityUtilsTestBase;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.worker.Worker;


/**
 *  <P>The <CODE>BenefitsServiceTestBase</CODE> class contains helper methods for testing the Benefits Service</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class BenefitsServiceTestBase extends EntityUtilsTestBase
{
    protected static BenefitsService sBenefitsService;
    protected static HealthBenefitEAO healthBenefitEAO;
    protected static HealthBenefitType sDefaultHealthBenefitType = HealthBenefitType.HEALTH;
    protected static String sDefaultHealthName = "Pre-tax Health";
    protected static BigDecimal sDefaultHealthBenefitAmount = null;
    
    @BeforeClass
    public static void setUpBenefitsServiceTestBase() {
    	sBenefitsService = qaBootstrap.getEjbFromClass(BenefitsService.class);
    	healthBenefitEAO = qaBootstrap.getEjbFromClass(HealthBenefitEAO.class);
        // set to 2 decimal places
    	sDefaultHealthBenefitAmount = new BigDecimal(100.00);
    	sDefaultHealthBenefitAmount = sDefaultHealthBenefitAmount.setScale(2);
    }
    
    /**
     * Creates a HealthBenefit object with a default provider name and amount
     * 
     * @return a HealthBenefit object with default values
     */
    protected HealthBenefit createHealthBenefit() {
        return createHealthBenefit(sDefaultHealthBenefitType, sDefaultHealthName, sDefaultHealthBenefitAmount);
    }

    /**
     * Creates a HealthBenefit object with the specified values
     * 
     * @param providerName the name of the health benefit provider
     * @param amount the premium/deduction of the health benefit
     * @return a HealthBenefit object with the specified values
     */
    protected HealthBenefit createHealthBenefit(HealthBenefitType type, String name, BigDecimal amount) {
        HealthBenefit healthBenefit = new HealthBenefit();
        healthBenefit.setObjectHandle(null);
        healthBenefit.setType(type);
        healthBenefit.setName(name);
        healthBenefit.setRate(amount);
        return healthBenefit;
    }
    
    /**
     * Retrieves the ObjectHandle of the Worker using the specified workerId
     * 
     * @param workerId The id of the worker to retrieve the ObjectHandle
     * @return ObjectHandle the ObjectHandle of the worker
     * @throws Exception
     */
    protected ObjectHandle getWorkerObjectHandle(Long workerId) throws Exception {
        Worker worker = sWorkerService.getWorkerById(workerId);
        return worker.getObjectHandle();
    }
    
    /**
     * Utility method for verifying 2 HealthBenefit arrays for equality
     * @param expected the expected HealthBenefit array
     * @param computed the computed HealthBenefit array
     * @return true if the arrays are equal, false otherwise
     */
    protected boolean verifyHealthBenefits(HealthBenefit[] expected, HealthBenefit[] computed) {
        if ((null == expected) || (null == computed)) {
            throw new RuntimeException("Null parameter");
        }
        boolean compareResult = (expected.length == computed.length);
        HashSet<HealthBenefit> expectedSet = new HashSet<HealthBenefit>();
        for (HealthBenefit healthBenefit : expected) {
            expectedSet.add(healthBenefit);
        }
        HashSet<HealthBenefit> computedSet = new HashSet<HealthBenefit>();
        for (HealthBenefit healthBenefit : computed) {
            computedSet.add(healthBenefit);
        }
        compareResult = expectedSet.equals(computedSet);
        if (false == compareResult) {
            sLogger.error("Expected HealthBenefit:");            
            for (HealthBenefit healthBenefit : expected) {
                sLogger.error(healthBenefit.toString());            
            }            
            sLogger.error("Computed HealthBenefit:");            
            for (HealthBenefit healthBenefit : computed) {
                sLogger.error(healthBenefit.toString());            
            }            
        }
        return compareResult;
    }
    
    /**
     * Utility method for verifying 2 HealthBenefitTemplate arrays for equality
     * @param expected the expected HealthBenefitTemplate array
     * @param computed the computed HealthBenefitTemplate array
     * @return true if the arrays are equal, false otherwise
     */
    protected boolean verifyHealthBenefitTemplates(HealthBenefitTemplate[] expected, HealthBenefitTemplate[] computed) {
        if ((null == expected) || (null == computed)) {
            throw new RuntimeException("Null parameter");
        }
        boolean compareResult = (expected.length == computed.length);
        HashSet<HealthBenefitTemplate> expectedSet = new HashSet<HealthBenefitTemplate>();
        for (HealthBenefitTemplate healthBenefit : expected) {
            expectedSet.add(healthBenefit);
        }
        HashSet<HealthBenefitTemplate> computedSet = new HashSet<HealthBenefitTemplate>();
        for (HealthBenefitTemplate healthBenefit : computed) {
            computedSet.add(healthBenefit);
        }
        compareResult = expectedSet.equals(computedSet);
        if (false == compareResult) {
            sLogger.error("Expected HealthBenefit:");            
            for (HealthBenefitTemplate healthBenefit : expected) {
                sLogger.error(healthBenefit.toString());            
            }            
            sLogger.error("Computed HealthBenefit:");            
            for (HealthBenefitTemplate healthBenefit : computed) {
                sLogger.error(healthBenefit.toString());            
            }            
        }
        return compareResult;
    }
    
    /**
     * Utility method to set up a list of Employer HealthBenefitTemplates retrieved from the DB
     * 
     * @return HealthBenefitTemplate[] an array of HealthBenefitTemplates
     */
    protected HealthBenefitTemplate[] setUpExpectedEmployerHealthBenefitTemplates() {
        List<com.intuit.sb.payroll.entity.model.HealthBenefit>  benefits = entityService.findAll(com.intuit.sb.payroll.entity.model.HealthBenefit.class);
        List<HealthBenefitTemplate> benefitTemplates = new ArrayList<HealthBenefitTemplate>();
        for (com.intuit.sb.payroll.entity.model.HealthBenefit benefit : benefits)
        {
            benefitTemplates.add(mapToHealthBenefitTemplate(benefit));
        }
        
        HealthBenefitTemplate[] benefitArray = new HealthBenefitTemplate[benefits.size()];
        return benefitTemplates.toArray(benefitArray);
    }
    
    /**
     * Utility method to set up a specified HealthBenefitTemplate retrieved from the DB
     * 
     * @return HealthBenefitTemplate a HealthBenefitTemplate
     */
    protected HealthBenefitTemplate setUpExpectedEmployerHealthBenefitTemplate(HealthBenefitType type) {
        com.intuit.sb.payroll.entity.model.HealthBenefit benefit = healthBenefitEAO.findHealthBenefit(BenefitMapper.mapToHealthBenefitTypeEnum(type));
        return mapToHealthBenefitTemplate(benefit);
    }
    
    private HealthBenefitTemplate mapToHealthBenefitTemplate(com.intuit.sb.payroll.entity.model.HealthBenefit benefit) {
        HealthBenefitTemplate template = new HealthBenefitTemplate();
        template.setObjectHandle(new ObjectHandle(benefit.getId(), benefit.getVersion()));
        template.setName(benefit.getDescription());
        template.setType(BenefitMapper.mapToHealthBenefitType(benefit.getBenefitType()));
        Datum datum = benefit.getRate();
        if (null != datum) {
            template.setRate(datum.getValue1AsUSDMoneyAmount());
        }
        return template;
    }
    
}
