package com.intuit.sb.payroll.services.common;

import java.math.BigDecimal;
import java.util.Calendar;

import org.testng.AssertJUnit;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.common.PayrollTestBase;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.services.employer.EmployerService;
import com.intuit.sb.payroll.services.taxes.FormType;
import com.intuit.sb.payroll.services.taxes.TaxWithholdingsService;
import com.intuit.sb.payroll.services.taxes.WithholdingInformation;
import com.intuit.sb.payroll.services.taxes.WithholdingStatus;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.worker.Worker;
import com.intuit.sb.payroll.services.worker.WorkerType;
import com.intuit.sb.payroll.services.workerpayments.PaycheckDistributionMethod;

/**
 * This class contains some helper methods that were in EntityTestBase.
 * EntityTestBase eventually extends EmbeddedContainer. A call to a method in EntityTestBase from integration test
 * tests will fail because some dependencies for embedded container are not present in integration test project.
 * Those embedded container dependencies can not be added to integration test project because that
 * causes dependencies conflicts.
 *
 */
public class EntityTestBaseHelper {

    protected static FormType sUS_W4 = FormType.US_W4;
	protected static FormType sCA_DE4 = FormType.CA_DE4;
    protected static String sUSJurisdiction = "US";
    protected static String sCAJurisdiction = "CA";
    protected static EmployerService sEmployerService = 
    	PayrollTestBase.getQaBootstrap().getEjbFromClass(EmployerService.class);
	protected static TaxWithholdingsService sTaxWithholdingsService = 
		PayrollTestBase.getQaBootstrap().getEjbFromClass(TaxWithholdingsService.class);

    protected static final String HOURLY_RATE_40 = "40.00";
    protected static final String OVERTIME_RATE_60 = "60.00";
    protected static final String SALARY_AMOUNT_100 = "100.00";
    
    protected static final double OVERTIME_MULTIPLIER = 1.5;
    
    /**
     * <p>Utility method to create CompensationItems</p>
     * <p>This method creates and returns 3 CompensationItems with the following attributes:</p>
     * <ol>
     * <li>Hourly CompensationItem with rate set to 40.00 and units set to 80.00</li>
     * <li>Overtime CompensationItem with rate set to 60.00 and units set to 10.00</li>
     * <li>Salary with rate set to 100.00 and units set to 0.0</li>
     * </ol>
     * 
     * @return 3 CompensationItems(Hourly, Overtime and Salary)
     */
    public static CompensationItem[] createCompensationItems() {
        CompensationItem[] compensationItems = new CompensationItem[2];
        CompensationItem hourly = new CompensationItem(null, new BigDecimal(HOURLY_RATE_40), 1, CompensationType.Hourly, "Hourly",true);
        CompensationItem overtime = new CompensationItem(null, new BigDecimal(OVERTIME_MULTIPLIER), 0, CompensationType.Overtime, "Overtime", hourly,true);
        compensationItems[0] = overtime;
        compensationItems[1] = hourly;
        //TODO we get a "unable to delete compensation item when there's still one linked to it" failure if we add the following
//        compensationItems[2] = new CompensationItem(null, new BigDecimal(SALARY_AMOUNT_100), 0, CompensationType.Salary, "Salary");
        return compensationItems;
    }
    
    /**
     * Utility method to create a com.intuit.sb.payroll.services.worker.Worker.  Does not
     * create ContactDetail, or WorkLocation. 
     * 
     * @return com.intuit.sb.payroll.services.worker.Worker
     */
    public static Worker createDtoWorker() {
    	Worker worker = new Worker();
    	worker.setFirstName("Barry");
    	worker.setLastName("Bonds");
    	worker.setJobTitle("Prisoner");
    	worker.setHireDate(DateUtil.createDate(2008, 1, 1));
    	Calendar barryBondsDoB = DateUtil.getNowCalendar();
    	barryBondsDoB.set(1964, 7, 24);
    	worker.setBirthDate(barryBondsDoB.getTime());
    	worker.setWorkerType(WorkerType.EMPLOYEE);
    	worker.setPaycheckDistributionMethod(PaycheckDistributionMethod.HANDWRITTEN);
    	
    	return worker;
    }

    /**
     * Utility method to return a com.intuit.sb.payroll.services.worker.WorkLocation
     *  
     * @return com.intuit.sb.payroll.services.worker.WorkLocation
     */
    protected static WorkLocation getEIWorkLocation() {
        WorkLocation[] workLocations;
        WorkLocation workLocaton = null;
        try {
            workLocations = sEmployerService.getWorkLocations();
            workLocaton = workLocations[0];
        } catch (ServiceFailureException e) {
            e.printStackTrace();
            AssertJUnit.fail("Error in getEIWorkLocation()!");
        } catch (ValidationException e) {
            e.printStackTrace();
            AssertJUnit.fail("Error in getEIWorkLocation()!");
        }
		return workLocaton;
    }
    
    static public class PayrollReadyWorker {
    	private Worker worker;
    	private CompensationItem[] compensationItems;
    	
    	public Worker getWorker() { return worker; }
    	public void setWorker(Worker worker) { this.worker = worker; }

    	public CompensationItem[] getCompensationItems() { return compensationItems; }
    	public void setCompensationItems(CompensationItem[] compensationItems) { this.compensationItems = compensationItems; }
}
    
    /** 
     * Create a payroll ready worker in memory.  Wrap in SaveWithholdingResponse
     * just for convenience, since it contains compensation info as well as the worker.
     */
    public static PayrollReadyWorker createPayrollReadyWorkerInMemory() throws ServiceFailureException, ValidationException {
    	Worker worker = createDtoWorker();
    	worker.setTaxResidenceAddress(PojoUtils.createUSCAPhysicalAddress());
    	worker.setWorkLocation(getEIWorkLocation());
    	    	
		CompensationItem[] compensationItems = createCompensationItems();

		PayrollReadyWorker payrollReadyWorker = new PayrollReadyWorker();
		payrollReadyWorker.setWorker(worker);
		payrollReadyWorker.setCompensationItems(compensationItems);
    	
		return payrollReadyWorker;
    }
    
    /**
     * <p>Creates and returns WithholdingInformation with the following attributes</p>
     * <ol>
     * <li>WithHoldingStatus: Married</li>
     * <li>Number of allowances: 3</li>
     * <li>Extra Withholding: 0.00</li>
     * <li>Exempt: false</li>
     * <li>Derived: false for a new DE4, true for existing DE4 and new or existing W4</li>
     * </ol>
     * @param formType
     * @return WithholdingInformation
     */
    // TODO: use this to test withholdings
    public static WithholdingInformation createEIWithholdingInformation(Worker worker, FormType formType) throws ServiceFailureException, ValidationException {
    	WithholdingInformation wInfo = new WithholdingInformation();

		if (worker.getObjectHandle() != null) {
            for (WithholdingInformation existingWithholdingInformation : sTaxWithholdingsService.getEmployeeWithholdings(worker.getObjectHandle().getId())) {
				// there's an existing document of the same type use it
				if (existingWithholdingInformation.getFormType().ordinal() == formType.ordinal()) {
					wInfo = existingWithholdingInformation;
				}
			}
		}
    	
    	wInfo.setFormType(formType);
    	wInfo.setWithholdingStatus(WithholdingStatus.MARRIED);
    	wInfo.setAllowances(3);
    	wInfo.setExtraWithholding(new BigDecimal("0.00"));
    	wInfo.setExempt(false);    	
      	return wInfo;
    }
}
