package com.intuit.sb.payroll.services.common;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.Set;

import org.testng.AssertJUnit;
import org.testng.annotations.BeforeClass;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.common.BasicTestBase;
import com.intuit.sb.payroll.capability.task.impl.TaskUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.domain.Identifiable;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.services.common.impl.ObjectHandleMapper;
import com.intuit.sb.payroll.services.employer.EmployerService;
import com.intuit.sb.payroll.services.taxes.FormType;
import com.intuit.sb.payroll.services.taxes.SaveWithholdingInfoRequest;
import com.intuit.sb.payroll.services.taxes.SaveWithholdingResponse;
import com.intuit.sb.payroll.services.taxes.TaxesTestBase;
import com.intuit.sb.payroll.services.taxes.WithholdingInfoOperation;
import com.intuit.sb.payroll.services.taxes.WithholdingInformation;
import com.intuit.sb.payroll.services.taxes.WithholdingStatus;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemRequest;
import com.intuit.sb.payroll.services.wages.WageService;
import com.intuit.sb.payroll.services.worker.Worker;
import com.intuit.sb.payroll.services.worker.WorkerType;
import com.intuit.sb.payroll.services.workerpayments.PayrollScheduleFrequency;
import com.intuit.sb.payroll.takehome.entity.TakehomeWorker;
import com.intuit.sb.payroll.us.capability.address.impl.USPostalAddressImpl;


/**
 * <P>The <CODE>EntityUtilsTestBase</CODE> class contains utility methods for testing Entity Interface APIs.
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class EntityUtilsTestBase extends TaxesTestBase
{
    protected static FormType sUS_W4 = FormType.US_W4;
	protected static FormType sCA_DE4 = FormType.CA_DE4;
    protected static String sUSJurisdiction = "US";
    protected static String sCAJurisdiction = "CA";
    protected static EmployerService sEmployerService;
    protected static WageService sWageService;

    protected static final String HOURLY_RATE_40 = "40.00";
    protected static final String OVERTIME_RATE_60 = "60.00";
    protected static final String TIPS_AMOUNT_100 = "100.00";
    
    protected static final String OVERTIME_MULTIPLIER = "1.5";
	
	@BeforeClass
    public static void initializeEntityUtilsTestBaseClass()
    {
    	sEmployerService = qaBootstrap.getEjbFromClass(EmployerService.class);
    	sWageService = qaBootstrap.getEjbFromClass(WageService.class);
    }
    
    /**
     * Utility method to create a com.intuit.sb.payroll.services.worker.Worker.  Does not
     * create ContactDetail, or WorkLocation. 
     * 
     * @return com.intuit.sb.payroll.services.worker.Worker
     */
    public static Worker createDtoWorker()
    {	
    	return EntityTestBaseHelper.createDtoWorker();
    }
    
    /**
     * Utility method to create a com.intuit.sb.payroll.services.worker.Worker with ContactDetail
     * and WorkLocation
     * 
     * @param firstName The worker's first name
     * @param middleName The worker's middle name
     * @param lastName The worker's last name
     * @param jobTitle The worker's job title
     * @param hireDate The worker's hire date
     * @param workerType The worker's type, employee or nonemployee
     * @param taxResidenceAddress The worker's address
     * @param workLocation the worker's work location
     * 
     * @return com.intuit.sb.payroll.services.worker.Worker
     */
    public static Worker createEIWorker(String firstName,String middleName,String lastName,String jobTitle,Date hireDate,WorkerType workerType,PhysicalAddress taxResidenceAddress,WorkLocation workLocation)
    {
    	Worker eiWorker = new Worker();
    	eiWorker.setFirstName(firstName);
    	eiWorker.setMiddleName(middleName);
    	eiWorker.setLastName(lastName);
    	eiWorker.setJobTitle(jobTitle);
    	eiWorker.setHireDate(hireDate);
    	eiWorker.setWorkerType(workerType);
    	eiWorker.setTaxResidenceAddress(taxResidenceAddress);
    	eiWorker.setWorkLocation(workLocation);
    	
    	return eiWorker;
    }
    
    /**
     * Utility method to return a com.intuit.sb.payroll.services.worker.WorkLocation
     *  
     * @return com.intuit.sb.payroll.services.worker.WorkLocation
     */
    protected static WorkLocation getEIWorkLocation()
    {
    	return EntityTestBaseHelper.getEIWorkLocation();
    }
    
    /**
     * Utility method to return a com.intuit.sb.payroll.services.worker.WorkLocation with the specified name
     * 
     * @param workLocationName The name of the WorkLocation to retrieve
     * @return A WorkLocation object if the specified WorkLocation exists, null otherwise
     * @throws Exception
     */
    protected static WorkLocation getNamedWorkLocation(String workLocationName) throws Exception {
		WorkLocation[] workLocations = sEmployerService.getWorkLocations();
		for (WorkLocation workLocation : workLocations) {
			if (workLocationName.equalsIgnoreCase(workLocation.getName())) {
				return workLocation;
			}
		}
		return null;
    }
    
	/**
	 * <P>Utility method to create a PayRoll ready Worker.</P>
	 * <H3>Steps:</H3>
	 * <OL>
     * <LI>Create Worker</LI>
     * <LI>Assign ContactDetail</LI>
     * <LI>Assign WorkLocation</LI>
     * <LI>Set WithHoldingInformation</LI>
	 * </OL>
     * 
     * @param firstName The worker's first name
     * @param middleName The worker's middle name
     * @param lastName The worker's last name
     * @param jobTitle The worker's job title
     * @param hireDate The worker's hire date
     * @param workerType The worker's type, employee or nonemployee
     * @param contactDetail The worker's contact details
     * @param workLocation the worker's work location
     * 
     * @return String Id of the worker
	 */    
    public static Long createPayrollReadyWorker(String firstName, String middleName, String lastName, String jobTitle,
                                                 Date hireDate, WorkerType workerType, PhysicalAddress taxResidenceAddress,
                                                 WorkLocation workLocation) throws Exception {
    	
    	Worker payrollReadyWorker = createEIWorker(firstName, middleName, lastName, jobTitle, hireDate, workerType,
                                                   taxResidenceAddress, workLocation);
    		    	
		// Setup Withholdings
        WithholdingInformation withholding = createEIWithholdingInformation(payrollReadyWorker, sUS_W4);

		// Setup compensation
		CompensationItem[] compensationItems = createCompensationItems();
		SaveCompensationItemRequest[] compensationItemRequests = new SaveCompensationItemRequest[compensationItems.length];
		for (int i = 0; i < compensationItems.length; i++) {
			compensationItemRequests[i] = new SaveCompensationItemRequest(compensationItems[i], SaveCompensationItemOperation.ADD);
		}
		
        Worker savedWorker = sWorkerService.saveWorker(payrollReadyWorker);
        ObjectHandle objectHandle = savedWorker.getObjectHandle();
        SaveWithholdingInfoRequest whRequest = new SaveWithholdingInfoRequest();
        whRequest.setWithholdingInfo(withholding);
        //FIXME [th] this is update because the save operation creates a W4.
        whRequest.setWithholdingInfoOperation(WithholdingInfoOperation.UPDATE);
        whRequest.setWorkerId(objectHandle.getId());
        whRequest.setWorkerTaxIdNumber("");
        sTaxWithholdingsService.saveWithholdingInformation(whRequest);          
		sWageService.saveCompensationItems(objectHandle.getId(), compensationItemRequests);

    	return objectHandle.getId();
    }
    
	/**
	 * <P>Utility method to create a PayRoll ready Worker.</P>
	 * <H3>Steps:</H3>
	 * <OL>
     * <LI>Create Worker</LI>
     * <LI>Assign ContactDetail</LI>
     * <LI>Assign WorkLocation</LI>
     * <LI>Set WageItems</LI>
     * <LI>Set WithHoldingInformation</LI>
	 * </OL>
     * * 
     * @return String Id of the worker
	 */    
    public static ObjectHandle getCreatedPayrollReadyWorker() throws Exception {
    	EntityTestBaseHelper.PayrollReadyWorker payrollReadyWorker = EntityTestBaseHelper.createPayrollReadyWorkerInMemory();

        Worker savedWorker = sWorkerService.saveWorker(payrollReadyWorker.getWorker());
        
		WithholdingInformation withholdingInformation = EntityTestBaseHelper.createEIWithholdingInformation(savedWorker, FormType.US_W4);
		ObjectHandle objectHandle = savedWorker.getObjectHandle();
        SaveWithholdingInfoRequest whRequest = new SaveWithholdingInfoRequest();
        whRequest.setWithholdingInfo(withholdingInformation);
        whRequest.setWorkerId(objectHandle.getId());
        //FIXME [th] this is optional, do we need it?
        whRequest.setWorkerTaxIdNumber("");
        whRequest.setWithholdingInfoOperation(WithholdingInfoOperation.ADD);
        @SuppressWarnings("unused")
        SaveWithholdingResponse whResponse = sTaxWithholdingsService.saveWithholdingInformation(whRequest);          
		
        // Setup compensation
		CompensationItem[] compensationItems = payrollReadyWorker.getCompensationItems();
		SaveCompensationItemRequest[] compensationItemRequests = new SaveCompensationItemRequest[compensationItems.length];
		for (int i = 0; i < compensationItems.length; i++) {
			compensationItemRequests[i] = new SaveCompensationItemRequest(compensationItems[i], SaveCompensationItemOperation.ADD);
		}
        sWageService.saveCompensationItems(objectHandle.getId(), compensationItemRequests);

        return objectHandle;
    }
    
	/**
	 * <P>Utility method to create a PayRoll ready Worker.</P>
	 * <H3>Steps:</H3>
	 * <OL>
     * <LI>Create Worker</LI>
     * <LI>Assign ContactDetail</LI>
     * <LI>Assign WorkLocation</LI>
     * <LI>Set WageItems</LI>
     * <LI>Set WithHoldingInformation</LI>
	 * </OL>
     * * 
     * @return String Id of the worker
	 */    
    public static Long createPayrollReadyWorker() throws Exception {
		ObjectHandle objectHandle = getCreatedPayrollReadyWorker();
        return objectHandle.getId();
    }
    
    public static Worker createNotPayrollReadyWorkerInMemory()
    {
        Worker payrollNotReadyWorker = createDtoWorker();
        payrollNotReadyWorker.setTaxResidenceAddress(PojoUtils.createUSCAPhysicalAddress());
        
        return payrollNotReadyWorker;
    }
    
    /**
     * <P>Utility method to create a worker that's not ready for Payroll.</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create Worker</LI>
     * <LI>Assign ContactDetail</LI>
     * </OL>
     * * 
     * @return Long Id of the worker
     */    
    public static Long createNotPayrollReadyWorker()
    {
        Worker worker = createNotPayrollReadyWorkerInMemory();
        
        ObjectHandle objectHandle = null;
        try
        {
            Worker payrollReadyWorker = sWorkerService.saveWorker(worker);
            objectHandle = payrollReadyWorker.getObjectHandle();
        }
        catch (ValidationException e)
        {
            e.printStackTrace();
            AssertJUnit.fail("Error in createNotPayrollReadyWorker()!");
        }
        catch (WorkflowException e)
        {
            e.printStackTrace();
            AssertJUnit.fail("Error in createNotPayrollReadyWorker()!");
        }
        catch (ServiceFailureException e)
        {
            e.printStackTrace();
            AssertJUnit.fail("Error in createNotPayrollReadyWorker()!");
        }
        return objectHandle.getId();
    }
    
    /**
     * <p>Utility method to create CompensationItems</p>
     * <p>This method creates and returns 3 CompensationItems with the following attributes:</p>
     * <ol>
     * <li>Hourly CompensationItem with rate set to 40.00 and units set to 80.00</li>
     * <li>Overtime CompensationItem with rate set to 60.00 and units set to 10.00</li>
     * <li>Salary with rate set to 100.00 and units set to 0.0</li>
     * </ol>
     * 
     * @return 3 CompensationItems(Hourly, Overtime and Salary)
     */
    public static CompensationItem[] createCompensationItems()
    {
    	return EntityTestBaseHelper.createCompensationItems();
    }
    
    /**
     * Utility method to retrieve the W4 WithholdingInformation from the specified worker id
     * 
     * @param workerId The id of the worker to retrieve the W4 WithholdingInformation
     * @return The W4 WithholdingInformation if it exists, null if not
     * @throws Exception
     */
    protected WithholdingInformation getW4WithholdingInfo(Long workerId) throws Exception {
    	return getSpecifiedWithholdingInfo(workerId, FormType.US_W4);
    }
    
    /**
     * Utility method to retrieve the DE4 WithholdingInformation from the specified worker id
     * 
     * @param workerId The id of the worker to retrieve the DE4 WithholdingInformation
     * @return The DE4 WithholdingInformation if it exists, null if not
     * @throws Exception
     */
    protected WithholdingInformation getDe4WithholdingInfo(Long workerId) throws Exception {
    	return getSpecifiedWithholdingInfo(workerId, FormType.CA_DE4);
    }
    
    /**
     * Utility method to retrieve the specified WithholdingInformation from the specified worker id
     * 
     * @param workerId The id of the worker to retrieve the specified WithholdingInformation
     * @param type The WithholdingInformation type
     * @return The WithholdingInformation if it exists, null if not
     * @throws Exception
     */
    protected WithholdingInformation getSpecifiedWithholdingInfo(Long workerId, FormType type) throws Exception {
    	WithholdingInformation[] withholdings = sTaxWithholdingsService.getEmployeeWithholdings(workerId);
    	for (WithholdingInformation withholdingInformation : withholdings) {
    		if (type == withholdingInformation.getFormType()) {
    			return withholdingInformation;
    		}
    	}
    	return null;
    }
    
    /**
     * Utility method to add a W4 WithholdingInformation to the specified worker
     * 
     * @param workerId The id of the worker to add the W4 WithholdingInformation
     * @return The updated worker
     * @throws Exception
     */
    protected Worker addW4WithholdingInfo(Long workerId) throws Exception {
    	return addWithholdingInfoToWorker(workerId, FormType.US_W4);
    	     }
    
    /**
     * Utility method to add a DE4 WithholdingInformation to the specified worker
     * 
     * @param workerId The id of the worker to add the DE4 WithholdingInformation
     * @return The updated worker
     * @throws Exception
     */
    protected Worker addDe4WithholdingInfo(Long workerId) throws Exception {
    	return addWithholdingInfoToWorker(workerId, FormType.CA_DE4);
    }
    
    /**
     * Utility method to add a specified WithholdingInformation to the specified worker
     * 
     * @param workerId The id of the worker to add the W4 WithholdingInformation
     * @param type The type of WithholdingInformation (of FormType)
     * @return The updated worker
     * @throws Exception
     */
    protected Worker addWithholdingInfoToWorker(Long workerId, FormType type) throws Exception {
		SaveWithholdingInfoRequest saveRequest = new SaveWithholdingInfoRequest();
		saveRequest.setWorkerId(workerId);
		saveRequest.setWorkerTaxIdNumber(null);
		saveRequest.setWagePlanCode(null);
		saveRequest.setWithholdingInfo(newWithholdingInformation(type));
		saveRequest.setWithholdingInfoOperation(WithholdingInfoOperation.ADD);
		SaveWithholdingResponse response = sTaxWithholdingsService.saveWithholdingInformation(saveRequest);
    	return response.getWorker();
    }
    
    /**
     * Utility method to create a WithholdingInformation object with the specified FormType and the following default
     *  values:
     *  
     *  Allowance = 1, Deduction = 0, exempt = false, withholdingStatus = Single, extra withholding of $0.00.
     *  
     * @param type The FormType of this WithholdingInformation
     * @return An initialized WithholdingInformation object with some default values
     */
	protected WithholdingInformation newWithholdingInformation(FormType type) {
		WithholdingInformation withholdingInformation = new WithholdingInformation();
		withholdingInformation.setAllowances(1);
		withholdingInformation.setDeductions(0);
		withholdingInformation.setExempt(false);
		withholdingInformation.setExtraWithholding(new BigDecimal("0.00"));
		withholdingInformation.setFormType(type);
		withholdingInformation.setWithholdingStatus(WithholdingStatus.SINGLE);
		return withholdingInformation;
	}
	
    /**
     * <p>Creates and returns WithholdingInformation with the following attributes</p>
     * <ol>
     * <li>WithHoldingStatus: Married</li>
     * <li>Number of allowances: 3</li>
     * <li>Extra Withholding: 100.00</li>
     * <li>Exempt: false</li>
     * </ol>
     * @param formType
     * @return WithholdingInformation
     */
    protected static WithholdingInformation createEIWithholdingInformation(Worker worker, FormType formType) throws ServiceFailureException, ValidationException {    	
    	return EntityTestBaseHelper.createEIWithholdingInformation(worker, formType);
    }
    
    /**
     * <p>Creates and returns a list of Us_W4 WithholdingInformation with the following attributes</p>
     * <ol>
     * <li>WithHoldingStatus: Married</li>
     * <li>Number of allowances: 3</li>
     * <li>Extra Withholding: 0.00</li>
     * <li>Exempt: false</li>
     * </ol>
     * @return WithholdingInformation
     */
    public static WithholdingInformation createEIWithholdingInformation(Worker worker) throws ServiceFailureException, ValidationException {
    	
    	WithholdingInformation w4 = EntityTestBaseHelper.createEIWithholdingInformation(worker, FormType.US_W4);
    	w4.setExtraWithholding(new BigDecimal("0.00"));
    	return w4;    	
    }

//TODO  Make sure that this routine suits the caller's needs    
    protected WorkerComplianceDocDtl getComplianceDocDtl(Long workerId, ComplianceDocIdEnum docId) {
        WorkerComplianceDocDtl workerComplianceDocDtl = null;
        com.intuit.sb.payroll.entity.model.Worker prWorker = entityService.findById(com.intuit.sb.payroll.entity.model.Worker.class, workerId);
        Set<WorkerComplianceDocDtl> workerComplianceDocDtls = prWorker.getWorkerComplianceDocDtls();
        Iterator<WorkerComplianceDocDtl> iterator = workerComplianceDocDtls.iterator();
        // find the correct one
        while (iterator.hasNext())
        {
            WorkerComplianceDocDtl complianceData = iterator.next();
            ComplianceDocIdEnum ComplianceDocIdEnum = complianceData.getDocId();
            if (null != ComplianceDocIdEnum)
            {
                if (ComplianceDocIdEnum.equals(docId))
                {
                    workerComplianceDocDtl = complianceData;
                    break;
                }
            }
        }
        AssertJUnit.assertNotNull("Couldn't find withholding info that was just set up???", workerComplianceDocDtl);
        return workerComplianceDocDtl;
    }

    protected WorkerComplianceDocDtl getEmployeeWithholdingDataUsingLawId(Long workerId, String lawId)
    {
        WorkerComplianceDocDtl workerComplianceDocDtl = null;
        com.intuit.sb.payroll.entity.model.Worker prWorker = entityService.findById(com.intuit.sb.payroll.entity.model.Worker.class, workerId);
        Set<WorkerComplianceDocDtl> workerComplianceDocDtls = prWorker.getWorkerComplianceDocDtls();
        Iterator<WorkerComplianceDocDtl> iterator = workerComplianceDocDtls.iterator();
        // find the correct one
        while (iterator.hasNext())
        {
            WorkerComplianceDocDtl complianceData = iterator.next();
            if (complianceData.getLawId().getValue().equals(lawId))
            {
                workerComplianceDocDtl = complianceData;
                break;
            }
        }
        AssertJUnit.assertNotNull("Couldn't find withholding info that was just set up???", workerComplianceDocDtl);
        return workerComplianceDocDtl;
    }

/*
    protected PhoneType mapPhoneType(TelephoneNumberType tnType)
    {
        // FIXME  Update the following code with the correct mapping!!!
        PhoneType phoneType = null;
        if (TelephoneNumberType.Fax == tnType)
        {
            phoneType = PhoneType.Work;
        }
        else if (TelephoneNumberType.Land == tnType)
        {
            phoneType = PhoneType.Home;
        }
        else if (TelephoneNumberType.Mobile == tnType)
        {
            phoneType = PhoneType.Mobile;
        }
        else if (TelephoneNumberType.Other == tnType)
        {
            phoneType = PhoneType.Other;
        }
        return phoneType;
    }
*/    
    /**
     * <p>Utility method to create and return a TelePhoneNumber</p>
     * @param number
     * @param tnType
     * @return TelePhoneNumber
     */    
//    protected TelephoneNumber createPhoneContact(String number, TelephoneNumberType tnType)
//    {
//        TelephoneNumber phoneNumber = new TelephoneNumber();
//        phoneNumber.setTNumber(number);
//        phoneNumber.setType(tnType);
//        return phoneNumber;
//    }
    
    /**
     * <Utility method to create and return an EmailAddress</p>
     * @param email
     * @return EmailAddress
     */    
//    protected EmailAddress createEmailContact(String email)
//    {
//        EmailAddress emailAddress = new EmailAddress();
//        emailAddress.setAddress(email);
//        return emailAddress;
//    }
    
    /**
     * <p>Returns a Platform Person from workerId</p>
     * 
     * @param workerId
     * @return Person
     */    
//    protected Person getPersonFromWorker(String workerId)
//    {
//        WorkerEAO wEAO = ServiceLocator.getInstance().getService("WorkerEAO");
//        com.intuit.sb.payroll.component.model.Worker aWorker = wEAO.findById(workerId);
//        Person  person = getPersonFromWorker(aWorker);
//        return person;
//    }
    
    /**
     * <p>Returns a Platform Person from Payroll.Component.Worker</p>
     * 
     * @param aWorker
     * @return Person
     */
//    protected Person getPersonFromWorker(com.intuit.sb.payroll.component.model.Worker aWorker)
//    {
//        WorkerRole workerRole = aWorker.getWorkerRole();
//        Person person = getPersonFromWorkerRole(workerRole);
//        return person;
//    }
    
    /**
     * <P>Utility method for adding a physicalAddress to a Worker</P>
     * 
     * @param workerId The worker's Id
     * @param tag The tag type for the physicalAddress
     * @param address The PhysicalAddress to add
     */
//    protected void addPhysicalAddressToWorker(String workerId, ContactRole tag, PhysicalAddress address)
//    {
//        WorkerEAO wEAO = ServiceLocator.getInstance().getService("WorkerEAO");
//        com.intuit.sb.payroll.component.model.Worker aWorker = wEAO.findById(workerId);
//        Person person = getPersonFromWorker(aWorker);
//        person.addPhysicalAddress(address);
//        person.getPhysicalAddressCollection().addTag(address, tag.name());
//        sEntityService.save(aWorker); 
//    }
    
    /**
     * <P>Utility method for adding an emailAddress to a Worker</P>
     * 
     * @param workerId The worker's Id
     * @param tag The tag type for the emailAddress
     * @param address The EmailAddress to add
     */
//    protected void addEmailAddressToWorker(String workerId, ContactRole tag, EmailAddress address)
//    {
//        WorkerEAO wEAO = ServiceLocator.getInstance().getService("WorkerEAO");
//        com.intuit.sb.payroll.component.model.Worker aWorker = wEAO.findById(workerId);
//        Person person = getPersonFromWorker(aWorker);
//        person.addEmailAddress(address);
//        person.getEmailAddressCollection().addTag(address, tag.name());
//        sEntityService.save(aWorker);
//    }
    
    /**
     * <P>Utility method for adding a webAddress to a Worker</P>
     * 
     * @param workerId The worker's Id
     * @param tag The tag type for the webAddress
     * @param address The WebAddress to add
     */
//    protected void addWebAddressToWorker(String workerId, ContactRole tag, WebAddress address)
//    {
//        WorkerEAO wEAO = ServiceLocator.getInstance().getService("WorkerEAO");
//        com.intuit.sb.payroll.component.model.Worker aWorker = wEAO.findById(workerId);
//        Person person = getPersonFromWorker(aWorker);
//        person.addWebAddress(address);
//        person.getWebAddressCollection().addTag(address, tag.name());
//        sEntityService.save(aWorker); 
//    }
    
    /**
     * <P>Utility method for adding a telephoneNumber to a Worker</P>
     * 
     * @param workerId The worker's Id
     * @param tag The tag type for the telephoneNumber
     * @param number The TelephoneNumber to add
     */
//    protected void addTelephoneNumberToWorker(String workerId, ContactRole tag, TelephoneNumber number)
//    {
//        WorkerEAO wEAO = ServiceLocator.getInstance().getService("WorkerEAO");
//        com.intuit.sb.payroll.component.model.Worker aWorker = wEAO.findById(workerId);
//        Person person = getPersonFromWorker(aWorker);
//        person.addTelephoneNumber(number);
//        person.getTelephoneNumberCollection().addTag(number, tag.name());
//        sEntityService.save(aWorker); 
//    }
    
    /**
     * <P>Utility method for removing a physicalAddress from a Worker</P>
     * 
     * @param workerId The worker's Id
     * @param address The PhysicalAddress to remove
     */
//    protected void removePhysicalAddressFromWorker(String workerId, PhysicalAddress address)
//    {
//        Person person = getPersonFromWorker(workerId);
//        person.removePhysicalAddress(address);
//    }
    
    /**
     * <P>Utility method for removing an emailAddress from a Worker</P>
     * 
     * @param workerId The worker's Id
     * @param address The EmailAddress to remove
     */
//    protected void removeEmailAddressFromWorker(String workerId, EmailAddress address)
//    {
//        Person person = getPersonFromWorker(workerId);
//        person.removeEmailAddress(address);
//    }
    
    /**
     * <P>Utility method for removing a webAddress from a Worker</P>
     * 
     * @param workerId The worker's Id
     * @param address The WebAddress to remove
     */
//    protected void removeWebAddressFromWorker(String workerId, WebAddress address)
//    {
//        Person person = getPersonFromWorker(workerId);
//        person.removeWebAddress(address);
//    }
    
    /**
     * <P>Utility method for removing a telephoneNumber from a Worker</P>
     * 
     * @param workerId The worker's Id
     * @param number The TelephoneNumber to remove
     */
//    protected void removeTelephoneNumberFromWorker(String workerId, TelephoneNumber number)
//    {
//        Person person = getPersonFromWorker(workerId);
//        person.removeTelephoneNumber(number);
//    }
    
 
	/**
	 * <P>Utility method to clean up a worker that was created using the IWorkerService.saveWorker()</P>
	 * <H3>Steps:</H3>
	 * <OL>
     * <LI>Clean up the worker's payroll items</LI>
     * <LI>Clean up the worker's Workflow</LI>
     * <LI>Clean up the worker's Withholding info</LI>
     * <LI>Delete the worker's WorkerRole</LI>
     * <LI>Delete the worker's Party</LI>
	 * </OL>
     * 
     * @param prWorker A Worker object
	 */
	protected static void cleanUpEIWorker(com.intuit.sb.payroll.entity.model.Worker prWorker)
	{
		if (prWorker != null) 
		{
		    try
		    {
                // Refresh
                prWorker = BasicTestBase.getEntityService().findById(com.intuit.sb.payroll.entity.model.Worker.class, prWorker.getId());
    
    	        // Remove withholdingInfo associated with the employee
                prWorker = removeWorkerWithholdingInfoByType(prWorker.getId(), ComplianceDocIdEnum.W4IRS);
                prWorker = removeWorkerWithholdingInfoByType(prWorker.getId(), ComplianceDocIdEnum.DE4);
                
                prWorker.setWorkLocation(null);
    			// Remove the worker's paycheck(s) if they exists
                prWorker.removeAllPaychecks();
                
            	// Remove all payroll items associated with the worker
                prWorker = removeAllWorkerPayrollItems(prWorker.getId()); 
                
                //Remove All Worker related SetUp Tasks that are implicitly created while worker creation
                TaskUtils.removeAllWorkerTasks(prWorker.getId());            	
                
                // Remove the TakeHomeWorker
                removeTakeHomeWorker(prWorker);
                
    			// Delete Worker
                entityService.makeTransient(prWorker);
		    }
	        catch (Exception ex)
	        {
                sLogger.error(ex.getMessage());
	            throw new RuntimeException(ex);
	        }
		}		
	}
	
	private static void removeTakeHomeWorker(com.intuit.sb.payroll.entity.model.Worker prWorker) throws Exception {
		com.intuit.sb.payroll.services.worker.Worker worker = sWorkerService.getWorkerById(prWorker.getId());
		
		ObjectHandle workerUniqueId = worker.getTakehomeWorkerObjectHandle();
		if (workerUniqueId != null && workerUniqueId.getId() != 0L) {
			Identifiable identifiable = ObjectHandleMapper.mapToIdentifiable(workerUniqueId);
			TakehomeWorker takehomeWorker = entityService.findByIdentifiable(TakehomeWorker.class, identifiable);
			takehomeWorker.setPayrollWorkerId(null);
			entityService.makeTransient(takehomeWorker);
		}
	}
	
	/**
	 * <P>Utility method to clean up a worker that was created using the IWorkerService.saveWorker()</P>
	 * <H3>Steps:</H3>
	 * <OL>
     * <LI>Retrieve a Worker object using the specified workerId</LI>
     * <LI>Call cleanUpEIWorker() with this Worker object</LI>
     * 
     * @param workerId A worker's Id
	 * @throws Exception 
	 */
	protected static void cleanUpEIWorker(Long id) throws Exception
	{
        if (null != id)
        {
            com.intuit.sb.payroll.entity.model.Worker worker = BasicTestBase.getEntityService().findById(com.intuit.sb.payroll.entity.model.Worker.class, id);
            cleanUpEIWorker(worker);
        }
	}
	
    private static Date addDate(Date date, int field, int amount)
    {
        Calendar computedDate = DateUtil.getNowCalendar();
        computedDate.setTime(date);
        computedDate.add(field, amount);
        return computedDate.getTime();
    }
    
    /**
     * Calculates a new date given a starting date and the number of days to adjust
     * 
     * @param date The starting date to use for the Date calculation
     * @param amount The number of days to add to or subtract from the starting date
     * @return The calculated date
     */
    protected static Date addDays(Date date, int amount)
    {
        return addDate(date, Calendar.DAY_OF_MONTH, amount);
    }
    
    /**
     * Calculates a new date given a starting date and the number of months to adjust
     * 
     * @param date The starting date to use for the Date calculation
     * @param amount The number of months to add to or subtract from the starting date
     * @return The calculated date
     */
    protected static Date addMonths(Date date, int amount)
    {
        return addDate(date, Calendar.MONTH, amount);
    }
    
    /**
     * Calculates a new date given a starting date and the number of years to adjust
     * 
     * @param date The starting date to use for the Date calculation
     * @param amount The number of years to add to or subtract from the starting date
     * @return The calculated date
     */
    protected static Date addYears(Date date, int amount)
    {
        return addDate(date, Calendar.YEAR, amount);
    }
    
    /**
     * Calculates a period start date given a period end date and pay frequency
     * @param periodEndDate The pay period end date
     * @param frequency The pay frequency (e.g. Weekly, Biweekly, Semimonthly, Monthly.  Unsupported frequenies default to Daily)
     * @return The calculated periodStartDate
     */
    protected static Date calculatePeriodStartDate(Date periodEndDate, PayrollFrequencyEnum frequency)
    {
        Calendar periodStartDate = DateUtil.getNowCalendar();
        // For a Daily frequency, the end date is the same as the start date
        periodStartDate.setTime(periodEndDate);
        if (PayrollFrequencyEnum.WEEKLY == frequency)
        {
            periodStartDate.add(Calendar.DAY_OF_MONTH, -6);
        }
        else if (PayrollFrequencyEnum.BIWEEKLY == frequency)
        {
            periodStartDate.add(Calendar.DAY_OF_MONTH, -13);
        }
        else if (PayrollFrequencyEnum.SEMIMONTHLY == frequency)
        {
            periodStartDate.add(Calendar.DAY_OF_MONTH, -14);
            //TODO  How to handle months that don't have exactly 30 days?
        }
        else if (PayrollFrequencyEnum.MONTHLY == frequency)
        {
            periodStartDate.add(Calendar.MONTH, -1);
            periodStartDate.add(Calendar.DAY_OF_MONTH, 1);
        }

        return periodStartDate.getTime();
    }
    
    /**
     * Maps a PayrollFrequency enum type to a PayrollScheduleFrequency enum type
     * 
     * @param frequency The PayrollFrequency enum type to map to a PayrollScheduleFrequency enum type
     * @return A PayrollScheduleFrequency enum type
     */
    protected static PayrollScheduleFrequency mapPayrollFrequency(PayrollFrequencyEnum frequency)
    {
        PayrollScheduleFrequency payrollScheduleFrequency = PayrollScheduleFrequency.NOT_SET;
        if (PayrollFrequencyEnum.WEEKLY == frequency)
        {
            payrollScheduleFrequency = PayrollScheduleFrequency.WEEKLY;
        }
        else if (PayrollFrequencyEnum.BIWEEKLY == frequency)
        {
            payrollScheduleFrequency = PayrollScheduleFrequency.BIWEEKLY;
        }
        else if (PayrollFrequencyEnum.SEMIMONTHLY == frequency)
        {
            payrollScheduleFrequency = PayrollScheduleFrequency.SEMIMONTHLY;
        }
        else if (PayrollFrequencyEnum.MONTHLY == frequency)
        {
            payrollScheduleFrequency = PayrollScheduleFrequency.MONTHLY;
        }

        return payrollScheduleFrequency;
    }
    
    
    /**
     * <p>Utility method to return PostalAddress from payroll.services.worker.Worker</p>
     * @param eiWorker
     * @return PostalAddress
     */    
    protected USPostalAddress getWorkLocationFromWorker(Worker eiWorker)
    {
        WorkLocation eiWorkLocation = eiWorker.getWorkLocation();
        Long workLocationId = eiWorkLocation.getObjectHandle().getId();
        com.intuit.sb.payroll.entity.model.WorkLocation workLocation = entityService.findById(com.intuit.sb.payroll.entity.model.WorkLocation.class, workLocationId);
        return AddressConvertor.getUSFromGlobal(
        		workLocation.getTaxResidenceAddress(), new USPostalAddressImpl());
    }
    
    protected WithholdingInformation[] setUpExpectedWithholdingInfo(Worker eiWorker) throws ServiceFailureException, ValidationException {
    	
        WithholdingInformation[] expectedWithholdings;
        USPostalAddress workAddress = getWorkLocationFromWorker(eiWorker);
        
        if(workAddress.getRegionOrState().equals(sCAJurisdiction))
        {
            expectedWithholdings = new WithholdingInformation[2];
            expectedWithholdings[0] = createEIWithholdingInformation(eiWorker, sUS_W4);
            expectedWithholdings[1] = createEIWithholdingInformation(eiWorker, sCA_DE4);
        }
        else
        {
            expectedWithholdings = new WithholdingInformation[1];
            expectedWithholdings[0] = createEIWithholdingInformation(eiWorker, sUS_W4);
        }
        return expectedWithholdings;
    }
    
    protected WithholdingInformation[] setUpExpectedWithholdingInfo(Worker eiWorker, WithholdingStatus expectedWHStatus)
    		throws ServiceFailureException, ValidationException {
    	
        WithholdingInformation[] expectedWithholdings;
        USPostalAddress workAddress = getWorkLocationFromWorker(eiWorker);
        
        if(workAddress.getRegionOrState().equals(sCAJurisdiction))
        {
            expectedWithholdings = new WithholdingInformation[2];
            expectedWithholdings[0] = createEIWithholdingInformation(eiWorker, sUS_W4);
            expectedWithholdings[0].setWithholdingStatus(expectedWHStatus);
            expectedWithholdings[1] = createEIWithholdingInformation(eiWorker, sCA_DE4);
            expectedWithholdings[1].setWithholdingStatus(expectedWHStatus);
        }
        else
        {
            expectedWithholdings = new WithholdingInformation[1];
            expectedWithholdings[0] = createEIWithholdingInformation(eiWorker, sUS_W4);
            expectedWithholdings[0].setWithholdingStatus(expectedWHStatus);
        }
        return expectedWithholdings;
    }
}
