package com.intuit.sb.payroll.services.taxes;

import java.util.Calendar;
import java.util.Date;

import javax.transaction.Status;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.taxes.TaxComponent;
import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.RelationshipToOwnerEnum;

/**
 *  <P>The <CODE>GetApplicableTaxesExceptionsTest</CODE> class contains test methods for testing Exceptions in getting applicable taxes</P>
 * 
 *  <DL>
 *  <DT><B>Requirements Document:</B></DT>
 *  <DD><A HREF="http://spitfire.corp.intuit.net/sites/merlinpayroll/Shared%20Documents/Forms/AllItems.aspx?RootFolder=%2fsites%2fmerlinpayroll%2fShared%20Documents%2fRequirements&View=%7b146B0034%2d58AF%2d4E76%2d93B1%2dA00364A96B7E%7d">Gemini requirements</A></DD>
 *  </DL>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class GetApplicableTaxesExceptionsTest extends TaxesTestBase
{

	@BeforeClass
	public static void setUpGetApplicableTaxesExceptions()
	{
        try
        {
            getUserTransaction().begin();
            saveEmployerInfo();
            getUserTransaction().commit();
        }
        catch (Throwable t)
        {
            sLogger.error(t.getMessage());
            AssertJUnit.fail("Error in setUpGetApplicableTaxesExceptions(): " + t.getMessage());
            try
            {
                getUserTransaction().rollback();
            }
            catch (Exception e)
            {
            }
        }
	}
	
	@AfterClass
	public static void tearDownGetApplicableTaxesExceptions()
	{
        try
        {
            getUserTransaction().begin();
            restoreEmployerInfo();
            getUserTransaction().commit();
        }
        catch (Throwable t)
        {
            sLogger.error(t.getMessage());
            AssertJUnit.fail("Error in tearDownGetApplicableTaxesExceptions(): " + t.getMessage());
            try
            {
                getUserTransaction().rollback();
            }
            catch (Exception e)
            {
            }
        }
	}
	
    /**
     * <P>Test Exception caused by invalid age</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to 501c3</LI>
     * <LI>Set work location in San Francisco, CA</LI>
     * <LI>Set up list of expected taxes</LI>
     * <LI>Set relationship to NotRelated</LI>
     * <LI>Set birthdate to a date in the future</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a PayrollException is thrown since the age is invalid
     * </UL>
      */
    @Test
    public void testInvalidAge() throws Exception
    {
        Date origDOB = null;
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
        sEmployer.setNonProfit(true);
        
        // Set up Physical address
        setUpSanFranciscoAddress();

        refreshWorker();
        // Remember the original date of birth
        origDOB = sWorker.getDateOfBirth();

        // Set up list of taxes, although it is not needed to be correct since an exception is expected to be thrown
        String[] expectedTaxes = {};

        Calendar futureBirthDate = DateUtil.getNowCalendar();
        futureBirthDate.add(Calendar.DATE, 365);
        // set up a birth date in the future
        
        try
        {
            performGetApplicableTaxes(sEmployer, sPhysicalAddress, futureBirthDate, RelationshipToOwnerEnum.UNRELATED, expectedTaxes);
            AssertJUnit.fail("Exception was not thrown!");
        }
        catch (Exception ex)
        {
        	ex.printStackTrace();
            AssertJUnit.assertTrue(verifyInvalidAgeException(ex));
        }
        finally
        {
            // Restore the original DOB
            if ((origDOB != null && (getUserTransaction().getStatus() == Status.STATUS_ACTIVE)))
            {
                refreshWorker();
                sWorker.setDateOfBirth(origDOB);
            }
        }
    }
    
    /**
     * <P>Test Exception caused by non-existent EmployeeId</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to 501c3</LI>
     * <LI>Set work location in San Francisco, CA</LI>
     * <LI>Use an invalid EmployeeId</LI>
     * <LI>Perform the test</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a PayrollException is thrown since the worker is null
     * </UL>
      */
    @Test(expectedExceptions=ValidationException.class)
    public void testNullWorker() throws Exception
    {
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
        sEmployer.setNonProfit(true);
        
        // Set up Physical address
        setUpSanFranciscoAddress();

        TaxComponent taxComponent = qaBootstrap.getEjbFromClass(TaxComponent.class);
        taxComponent.determineApplicableTaxes(null);
    }
    
    /**
     * <P>Test Exception caused by null WorkLocation</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to 501c3</LI>
     * <LI>Set work location to null</LI>
     * <LI>Perform the test</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a PayrollException is thrown since the WorkLocation is not set
     * </UL>
      */
    @Test(expectedExceptions=WorkflowException.class)
    public void testNullWorkLocation() throws Exception
    {
        WorkerComponent workerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);
        // Set up Company LegalStructure        
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
        sEmployer.setNonProfit(true);
        
        refreshWorker();
        // Change WorkLocation
        sWorker.setWorkLocation(null);
        //Save the sWorker
        sWorker = workerComponent.updateWorker(sWorker);
        
        TaxComponent taxComponent = qaBootstrap.getEjbFromClass(TaxComponent.class);
        taxComponent.determineApplicableTaxes(sWorker);

    }
    
}
