package com.intuit.sb.payroll.services.taxes;

import org.testng.AssertJUnit;

import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.RelationshipToOwnerEnum;

/**
 * <P>
 * The <CODE>GetApplicableTaxesSanFranciscoTest</CODE> class contains test methods for getting applicable taxes for a
 * San Francisco location
 * </P>
 * 
 * <DL>
 * <DT><B>Requirements Document:</B></DT>
 * <DD><A HREF="http://spitfire.corp.intuit.net/sites/merlinpayroll/Shared%20Documents/Forms/AllItems.aspx?RootFolder=%2fsites%2fmerlinpayroll%2fShared%20Documents%2fRequirements&View=%7b146B0034%2d58AF%2d4E76%2d93B1%2dA00364A96B7E%7d"
 * >Gemini requirements</A></DD>
 * </DL>
 * 
 * $Author$ $Revision$ $DateTime$
 * 
 */
public class GetApplicableTaxesSanFranciscoTest extends TaxesTestBase {

	@BeforeClass
	public static void setUpGetApplicableTaxesSanFrancisco() {
		try {
			getUserTransaction().begin();
			saveEmployerInfo();
			setUpSanFranciscoAddress();
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			AssertJUnit.fail("Error in setUpGetApplicableTaxesSanFrancisco(): " + t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
		}
	}

	@AfterClass
	public static void tearDownGetApplicableTaxesSanFrancisco() {
		try {
			getUserTransaction().begin();
			restoreEmployerInfo();
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			AssertJUnit.fail("Error in tearDownGetApplicableTaxesSanFrancisco(): " + t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
		}
	}

	/**
	 * <P>
	 * Tests applicable taxes for non profit organization in San Francisco, CA; employee is not related to employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to UNRELATED</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3InSanFranciscoAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for church in San Francisco, CA; employee is not related to employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to UNRELATED</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3ChurchInSanFranciscoAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.CHURCH);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-CA-PIT", "US-CA-ETT", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for C-Corp in San Francisco, CA; employee is not related to employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to C-Corp</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to UNRELATED</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testC_CorpInSanFranciscoAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for S-Corp in San Francisco, CA; employee is not related to employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to S-Corp</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to UNRELATED</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testS_CorpInSanFranciscoAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for LLC in San Francisco, CA; employee is not related to employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to LLC</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to UNRELATED</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testLLCInSanFranciscoAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.LLC);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for Partnership in San Francisco, CA; employee is not related to employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to Partnership</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to UNRELATED</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testPartnershipInSanFranciscoAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.PARTNERSHIP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for SoleProprietor in San Francisco, CA; employee is not related to employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to SoleProprietor</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to UNRELATED</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testSoleProprietorInSanFranciscoAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SOLEPROPRIETOR);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
				expectedTaxes);
	}

	/************* Test cases for employee being a partner of the employer *************/

	/**
	 * <P>
	 * Tests applicable taxes for non profit organization in San Francisco, CA; and employee is partner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to partner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3InSanFranciscoForPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for church in San Francisco, CA; and employee is partner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to partner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3ChurchInSanFranciscoForPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.CHURCH);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-CA-PIT", "US-CA-ETT", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for C-Corp in San Francisco, CA; and employee is partner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to C-Corp</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to partner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testC_CorpInSanFranciscoForPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for S-Corp in San Francisco, CA; and employee is partner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to S-Corp</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to partner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testS_CorpInSanFranciscoForPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for LLC in San Francisco, CA; and employee is partner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to LLC</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to partner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testLLCInSanFranciscoForPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.LLC);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = {};

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for Partnership in San Francisco, CA; and employee is partner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to Partnership</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to partner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testPartnershipInSanFranciscoForPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.PARTNERSHIP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = {};

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for SoleProprietor in San Francisco, CA; and employee is partner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to SoleProprietor</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to partner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testSoleProprietorInSanFranciscoForPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SOLEPROPRIETOR);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = {};

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
				expectedTaxes);
	}

	/************* Test cases for employee being a spouse of the employer *************/

	/**
	 * <P>
	 * Tests applicable taxes for non profit organization in San Francisco, CA; and employee is spouse of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to spouse</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3InSanFranciscoForSpouseAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.SPOUSE,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for church in San Francisco, CA; and employee is spouse of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to spouse</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3ChurchInSanFranciscoForSpouseAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.CHURCH);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-CA-PIT", "US-CA-ETT", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.SPOUSE,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for C-Corp in San Francisco, CA; and employee is spouse of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to C-Corp</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to spouse</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testC_CorpInSanFranciscoForSpouseAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.SPOUSE,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for S-Corp in San Francisco, CA; and employee is spouse of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to S-Corp</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to spouse</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testS_CorpInSanFranciscoForSpouseAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.SPOUSE,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for LLC in San Francisco, CA; and employee is spouse of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to LLC</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to spouse</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testLLCInSanFranciscoForSpouseAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.LLC);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.SPOUSE,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for Partnership in San Francisco, CA; and employee is spouse of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to Partnership</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to spouse</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testPartnershipInSanFranciscoForSpouseAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.PARTNERSHIP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.SPOUSE,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for SoleProprietor in San Francisco, CA; and employee is spouse of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to SoleProprietor</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to spouse</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testSoleProprietorInSanFranciscoForSpouseAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SOLEPROPRIETOR);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-CA-PIT", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.SPOUSE,
				expectedTaxes);
	}

	/************* Test cases for employee being a Domestic Partner of the employer *************/

	/**
	 * <P>
	 * Tests applicable taxes for non profit organization in San Francisco, CA; and employee is Domestic Partner of
	 * employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to DomesticPartner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3InSanFranciscoForDomesticPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate,
				RelationshipToOwnerEnum.REGISTERED_DOMESTIC_PARTNER, expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for church in San Francisco, CA; and employee is Domestic Partner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to DomesticPartner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3ChurchInSanFranciscoForDomesticPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.CHURCH);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-CA-PIT", "US-CA-ETT", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate,
				RelationshipToOwnerEnum.REGISTERED_DOMESTIC_PARTNER, expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for C-Corp in San Francisco, CA; and employee is DomesticPartner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to C-Corp</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to DomesticPartner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testC_CorpInSanFranciscoForDomesticPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate,
				RelationshipToOwnerEnum.REGISTERED_DOMESTIC_PARTNER, expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for S-Corp in San Francisco, CA; and employee is DomesticPartner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to S-Corp</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to DomesticPartner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testS_CorpInSanFranciscoForDomesticPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate,
				RelationshipToOwnerEnum.REGISTERED_DOMESTIC_PARTNER, expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for LLC in San Francisco, CA; and employee is DomesticPartner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to LLC</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to DomesticPartner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testLLCInSanFranciscoForDomesticPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.LLC);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate,
				RelationshipToOwnerEnum.REGISTERED_DOMESTIC_PARTNER, expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for Partnership in San Francisco, CA; and employee is DomesticPartner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to Partnership</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to DomesticPartner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testPartnershipInSanFranciscoForDomesticPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.PARTNERSHIP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate,
				RelationshipToOwnerEnum.REGISTERED_DOMESTIC_PARTNER, expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for SoleProprietor in San Francisco, CA; and employee is DomesticPartner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to SoleProprietor</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to DomesticPartner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testSoleProprietorInSanFranciscoForDomesticPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SOLEPROPRIETOR);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate,
				RelationshipToOwnerEnum.REGISTERED_DOMESTIC_PARTNER, expectedTaxes);
	}

	/************* Test cases for employee being a child of the employer *************/

	/**
	 * <P>
	 * Tests applicable taxes for non profit organization in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3InSanFranciscoForChildAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for church in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3ChurchInSanFranciscoForChildAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.CHURCH);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-CA-PIT", "US-CA-ETT", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for C-Corp in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to C-Corp</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testC_CorpInSanFranciscoForChildAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for S-Corp in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to S-Corp</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testS_CorpInSanFranciscoForChildAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for LLC in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to LLC</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testLLCInSanFranciscoForChildAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.LLC);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for Partnership in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to Partnership</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testPartnershipInSanFranciscoForChildAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.PARTNERSHIP);
		sEmployer.setNonProfit(false);
		setUpSanFranciscoAddress();

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for SoleProprietor in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to SoleProprietor</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testSoleProprietorInSanFranciscoForChildAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SOLEPROPRIETOR);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for non profit organization in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 21</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3InSanFranciscoForChildAge21() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge21BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for church in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 21</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3ChurchInSanFranciscoForChildAge21() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.CHURCH);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-CA-PIT", "US-CA-ETT", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge21BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for C-Corp in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to C-Corp</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 21</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testC_CorpInSanFranciscoForChildAge21() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge21BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for S-Corp in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to S-Corp</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 21</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testS_CorpInSanFranciscoForChildAge21() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge21BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for LLC in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to LLC</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 21</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testLLCInSanFranciscoForChildAge21() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.LLC);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge21BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for Partnership in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to Partnership</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 21</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testPartnershipInSanFranciscoForChildAge21() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.PARTNERSHIP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge21BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for SoleProprietor in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to SoleProprietor</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 21</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testSoleProprietorInSanFranciscoForChildAge21() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SOLEPROPRIETOR);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge21BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for non profit organization in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 20</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3InSanFranciscoForChildAge20() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge20BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for church in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 20</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3ChurchInSanFranciscoForChildAge20() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.CHURCH);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-CA-PIT", "US-CA-ETT", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge20BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for C-Corp in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to C-Corp</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 20</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testC_CorpInSanFranciscoForChildAge20() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge20BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for S-Corp in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to S-Corp</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 20</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testS_CorpInSanFranciscoForChildAge20() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge20BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for LLC in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to LLC</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 20</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testLLCInSanFranciscoForChildAge20() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.LLC);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge20BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for Partnership in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to Partnership</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 20</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testPartnershipInSanFranciscoForChildAge20() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.PARTNERSHIP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge20BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for SoleProprietor in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to SoleProprietor</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 20</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testSoleProprietorInSanFranciscoForChildAge20() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SOLEPROPRIETOR);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge20BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	// TODO: Tests for Child, age < 21, legalStructure in (Partnership, LLC), All Partners = Parents of Child

	/**
	 * <P>
	 * Tests applicable taxes for non profit organization in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 18</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3InSanFranciscoForChildAge18() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge18BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for church in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 18</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3ChurchInSanFranciscoForChildAge18() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.CHURCH);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-CA-PIT", "US-CA-ETT", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge18BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for C-Corp in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to C-Corp</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 18</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testC_CorpInSanFranciscoForChildAge18() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge18BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for S-Corp in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to S-Corp</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 18</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testS_CorpInSanFranciscoForChildAge18() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge18BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for LLC in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to LLC</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 18</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testLLCInSanFranciscoForChildAge18() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.LLC);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge18BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for Partnership in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to Partnership</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 18</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testPartnershipInSanFranciscoForChildAge18() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.PARTNERSHIP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge18BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for SoleProprietor in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to SoleProprietor</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 18</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testSoleProprietorInSanFranciscoForChildAge18() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SOLEPROPRIETOR);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge18BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for non profit organization in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 17</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3InSanFranciscoForChildAge17() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge17BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for church in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 17</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3ChurchInSanFranciscoForChildAge17() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.CHURCH);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-CA-PIT", "US-CA-ETT", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge17BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for C-Corp in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to C-Corp</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 17</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testC_CorpInSanFranciscoForChildAge17() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge17BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for S-Corp in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to S-Corp</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 17</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testS_CorpInSanFranciscoForChildAge17() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge17BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for LLC in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to LLC</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 17</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testLLCInSanFranciscoForChildAge17() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.LLC);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge17BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for Partnership in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to Partnership</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 17</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testPartnershipInSanFranciscoForChildAge17() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.PARTNERSHIP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge17BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for SoleProprietor in San Francisco, CA; and employee is child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to SoleProprietor</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to child</LI>
	 * <LI>Set birthdate to age 17</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testSoleProprietorInSanFranciscoForChildAge17() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SOLEPROPRIETOR);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-CA-PIT", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge17BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	// TODO: Tests for Child, age < 18, legalStructure in (Partnership, LLC), All Partners = Parents of Child

	/************* Test cases for employee being a parent of the employer *************/

	/**
	 * <P>
	 * Tests applicable taxes for non profit organization in San Francisco, CA; and employee is parent of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to parent</LI>
	 * <LI>Set birthdate to age 40</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3InSanFranciscoForParentAge40() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge40BirthDate, RelationshipToOwnerEnum.PARENT,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for church in San Francisco, CA; and employee is parent of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to parent</LI>
	 * <LI>Set birthdate to age 40</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3ChurchInSanFranciscoForParentAge40() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.CHURCH);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-CA-PIT", "US-CA-ETT", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge40BirthDate, RelationshipToOwnerEnum.PARENT,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for C-Corp in San Francisco, CA; and employee is parent of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to C-Corp</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to parent</LI>
	 * <LI>Set birthdate to age 40</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testC_CorpInSanFranciscoForParentAge40() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge40BirthDate, RelationshipToOwnerEnum.PARENT,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for S-Corp in San Francisco, CA; and employee is parent of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to S-Corp</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to parent</LI>
	 * <LI>Set birthdate to age 40</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testS_CorpInSanFranciscoForParentAge40() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge40BirthDate, RelationshipToOwnerEnum.PARENT,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for LLC in San Francisco, CA; and employee is parent of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to LLC</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to parent</LI>
	 * <LI>Set birthdate to age 40</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testLLCInSanFranciscoForParentAge40() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.LLC);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge40BirthDate, RelationshipToOwnerEnum.PARENT,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for Partnership in San Francisco, CA; and employee is parent of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to Partnership</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to parent</LI>
	 * <LI>Set birthdate to age 40</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testPartnershipInSanFranciscoForParentAge40() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.PARTNERSHIP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-CA-PIT", "US-CA-SDI", "US-CA-ETT", "US-CA-SUI", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge40BirthDate, RelationshipToOwnerEnum.PARENT,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for SoleProprietor in San Francisco, CA; and employee is parent of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to SoleProprietor</LI>
	 * <LI>Set work location in San Francisco, CA</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to parent</LI>
	 * <LI>Set birthdate to age 40</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testSoleProprietorInSanFranciscoForParentAge40() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SOLEPROPRIETOR);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-CA-PIT", /* "US-CA-SF-PT" */ };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge40BirthDate, RelationshipToOwnerEnum.PARENT,
				expectedTaxes);
	}

	// TODO: Tests for Parent, legalStructure = Partnership, All Partners = Siblings

}
