package com.intuit.sb.payroll.services.taxes;

import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.taxes.withholding.impl.WorkerTaxWithholdingRules;
import com.intuit.sb.payroll.entity.model.ComplianceDoc;
import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.RelationshipToOwnerEnum;
import com.intuit.sb.payroll.entity.model.Worker;


/**
 *  <P>The <CODE>GetWithholdingFormsExceptionsTest</CODE> class contains test methods for testing Exceptions in getting withholding forms</P>
 *
 *  <DL>
 *  <DT><B>Requirements Document:</B></DT>
 *  <DD><A HREF="http://spitfire.corp.intuit.net/sites/merlinpayroll/Shared%20Documents/Forms/AllItems.aspx?RootFolder=%2fsites%2fmerlinpayroll%2fShared%20Documents%2fRequirements&View=%7b146B0034%2d58AF%2d4E76%2d93B1%2dA00364A96B7E%7d">Gemini requirements</A></DD>
 *  </DL>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class GetWithholdingFormsExceptionsTest extends TaxesTestBase
{

	@BeforeClass
	public static void setUpGetWithholdingFormsExceptions()
	{
        try
        {
            getUserTransaction().begin();
            saveEmployerInfo();
    		setUpSanFranciscoAddress();
            getUserTransaction().commit();
        }
        catch (Throwable t)
        {
            sLogger.error(t.getMessage());
            AssertJUnit.fail("Error in setUpGetWithholdingFormsExceptions(): " + t.getMessage());
            try
            {
                getUserTransaction().rollback();
            }
            catch (Exception e)
            {
            }
        }
	}
	
	@AfterClass
	public static void tearDownGetWithholdingFormsExceptions()
	{
        try
        {
            getUserTransaction().begin();
            restoreEmployerInfo();
            getUserTransaction().commit();
        }
        catch (Throwable t)
        {
            sLogger.error(t.getMessage());
            AssertJUnit.fail("Error in tearDownGetWithholdingFormsExceptions(): " + t.getMessage());
            try
            {
                getUserTransaction().rollback();
            }
            catch (Exception e)
            {
            }
        }
	}
	
    /**
     * <P>Test getting withholding forms without applying applicable taxes first</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to 501c3</LI>
     * <LI>Set work location in San Francisco, CA</LI>
     * <LI>Set relationship to NotRelated</LI>
     * <LI>Call utility routine to set up the data</LI>
     * <LI>Remove existing payroll items associated with the employee</LI>
     * <LI>Set up list of expected forms</LI>
     * <LI>Call command to get the withholding forms</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that no forms are returned since this command requires ApplyApplicableTaxes be executed already
     * </UL>
      */
    @Test
    public void testInvalidCallSequence() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
        sEmployer.setNonProfit(true);
        
        setUpTestData(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED);
        
        // Set up expected list of withholding forms
        String[] expectedWithholdingForms = {};
        sWorker = entityService.findById(Worker.class, sWorkerId);
        List<ComplianceDoc> computedWithholdingForms = WorkerTaxWithholdingRules.getWithholdingFormsApplicableToWorker(sWorker);
        
        // Verify the applicable withholding forms
        boolean withholdingFormsMatched = verifyWithholdingForms(computedWithholdingForms, expectedWithholdingForms);
        AssertJUnit.assertTrue("WithholdingForms mismatch!", withholdingFormsMatched);
    }
    
    /**
     * <P>Test Exception caused by non-existent EmployeeId</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to 501c3</LI>
     * <LI>Set work location in San Francisco, CA</LI>
     * <LI>Use an invalid EmployeeId</LI>
     * <LI>Perform the test</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown since the employee id is non-existent
     * </UL>
      */
    @Test(expectedExceptions=ValidationException.class)
    public void testNonExistentEmployeeId() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
        sEmployer.setNonProfit(true);
        
        @SuppressWarnings("unused")
		List<ComplianceDoc> computedWithholdingForms = WorkerTaxWithholdingRules.getWithholdingFormsApplicableToWorker(null);
        
        // An exception should have been thrown!
        AssertJUnit.fail("Exception was not thrown!");
    }
        
}
