package com.intuit.sb.payroll.services.taxes;

import org.testng.AssertJUnit;

import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.RelationshipToOwnerEnum;


/**
 *  <P>The <CODE>GetWithholdingFormsTest</CODE> class contains test methods for getting Withholding forms</P>
 *
 *  <DL>
 *  <DT><B>Requirements Document:</B></DT>
 *  <DD><A HREF="http://spitfire.corp.intuit.net/sites/merlinpayroll/Shared%20Documents/Forms/AllItems.aspx?RootFolder=%2fsites%2fmerlinpayroll%2fShared%20Documents%2fRequirements&View=%7b146B0034%2d58AF%2d4E76%2d93B1%2dA00364A96B7E%7d">Gemini requirements</A></DD>
 *  </DL>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class GetWithholdingFormsTest extends TaxesTestBase
{

	@BeforeClass
	public static void setUpGetWithholdingForms()
	{
        try
        {
            getUserTransaction().begin();
            saveEmployerInfo();
            setUpSanMateoAddress();
            getUserTransaction().commit();
        }
        catch (Throwable t)
        {
            sLogger.error(t.getMessage());
            AssertJUnit.fail("Error in setUpGetWithholdingForms(): " + t.getMessage());
            try
            {
                getUserTransaction().rollback();
            }
            catch (Exception e)
            {
            }
        }
	}
	
	@AfterClass
	public static void tearDownGetWithholdingForms()
	{
        try
        {
            getUserTransaction().begin();
            restoreEmployerInfo();
            getUserTransaction().commit();
        }
        catch (Throwable t)
        {
            sLogger.error(t.getMessage());
            AssertJUnit.fail("Error in tearDownGetWithholdingForms(): " + t.getMessage());
            try
            {
                getUserTransaction().rollback();
            }
            catch (Exception e)
            {
            }
        }
	}
	
    /************* Test cases for employee not related to the employer *************/
    
    /**
     * <P>Tests applicable withholding forms for non profit organization in San Mateo, CA; employee is not related to employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to 501c3</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected withholding forms</LI>
     * <LI>Set relationship to NotRelated</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
	@Test
    public void testNonProfit501c3InSanMateoAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
        sEmployer.setNonProfit(true);
        
        // Set up expected list of withholding forms
        String[] expectedWithholdingForms = {"US-CA-DE-4", "US-FED-W4"};
        
        performGetWithholdingForms(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
                                   expectedWithholdingForms);
    }
    
    /**
     * <P>Tests applicable withholding forms for church in San Mateo, CA; employee is not related to employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to 501c3</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected withholding forms</LI>
     * <LI>Set relationship to NotRelated</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testNonProfit501c3ChurchInSanMateoAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.CHURCH);
        sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
        sEmployer.setNonProfit(true);
        
        // Set up expected list of withholding forms
        String[] expectedWithholdingForms = {"US-CA-DE-4", "US-FED-W4"};
        
        performGetWithholdingForms(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
                                    expectedWithholdingForms);
    }
    
    /**
     * <P>Tests applicable withholding forms for C-Corp in San Mateo, CA; employee is not related to employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to C-Corp</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected withholding forms</LI>
     * <LI>Set relationship to NotRelated</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testC_CorpInSanMateoAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
        sEmployer.setNonProfit(false);
        
        // Set up expected list of withholding forms
        String[] expectedWithholdingForms = {"US-CA-DE-4", "US-FED-W4"};
        
        performGetWithholdingForms(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
                                    expectedWithholdingForms);
    }
    
    /**
     * <P>Tests applicable withholding forms for S-Corp in San Mateo, CA; employee is not related to employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to S-Corp</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected withholding forms</LI>
     * <LI>Set relationship to NotRelated</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testS_CorpInSanMateoAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
        sEmployer.setNonProfit(false);
        
        // Set up expected list of withholding forms
        String[] expectedWithholdingForms = {"US-CA-DE-4", "US-FED-W4"};
        
        performGetWithholdingForms(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
                                    expectedWithholdingForms);
    }
    
    /**
     * <P>Tests applicable withholding forms for LLC in San Mateo, CA; employee is not related to employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to LLC</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected withholding forms</LI>
     * <LI>Set relationship to NotRelated</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testLLCInSanMateoAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.LLC);
        sEmployer.setNonProfit(false);
        
        // Set up expected list of withholding forms
        String[] expectedWithholdingForms = {"US-CA-DE-4", "US-FED-W4"};
        
        performGetWithholdingForms(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
                                    expectedWithholdingForms);
    }
    
    /**
     * <P>Tests applicable withholding forms for Partnership in San Mateo, CA; employee is not related to employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to Partnership</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected withholding forms</LI>
     * <LI>Set relationship to NotRelated</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testPartnershipInSanMateoAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.PARTNERSHIP);
        sEmployer.setNonProfit(false);
        
        // Set up expected list of withholding forms
        String[] expectedWithholdingForms = {"US-CA-DE-4", "US-FED-W4"};
        
        performGetWithholdingForms(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
                                    expectedWithholdingForms);
    }
    
    /**
     * <P>Tests applicable withholding forms for SoleProprietor in San Mateo, CA; employee is not related to employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to SoleProprietor</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected withholding forms</LI>
     * <LI>Set relationship to NotRelated</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testSoleProprietorInSanMateoAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.SOLEPROPRIETOR);
        sEmployer.setNonProfit(false);
        
        // Set up expected list of withholding forms
        String[] expectedWithholdingForms = {"US-CA-DE-4", "US-FED-W4"};
        
        performGetWithholdingForms(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
                                    expectedWithholdingForms);
    }
    
    /************* Test cases for employee being a partner of the employer *************/
    
    /**
     * <P>Tests applicable withholding forms for non profit organization in San Mateo, CA; and employee is partner of employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to 501c3</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected withholding forms</LI>
     * <LI>Set relationship to partner</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testNonProfit501c3InSanMateoForPartnerAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
        sEmployer.setNonProfit(true);
        
        // Set up expected list of withholding forms
        String[] expectedWithholdingForms = {"US-CA-DE-4", "US-FED-W4"};
        
        performGetWithholdingForms(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
                                    expectedWithholdingForms);
    }
    
    /**
     * <P>Tests applicable withholding forms for church in San Mateo, CA; and employee is partner of employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to 501c3</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected withholding forms</LI>
     * <LI>Set relationship to partner</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testNonProfit501c3ChurchInSanMateoForPartnerAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.CHURCH);
        sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
        sEmployer.setNonProfit(true);
        
        // Set up expected list of withholding forms
        String[] expectedWithholdingForms = {"US-CA-DE-4", "US-FED-W4"};
        
        performGetWithholdingForms(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
                                    expectedWithholdingForms);
    }
    
    /**
     * <P>Tests applicable withholding forms for C-Corp in San Mateo, CA; and employee is partner of employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to C-Corp</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected withholding forms</LI>
     * <LI>Set relationship to partner</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testC_CorpInSanMateoForPartnerAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
        sEmployer.setNonProfit(false);
 
        // Set up expected list of withholding forms
        String[] expectedWithholdingForms = {"US-CA-DE-4", "US-FED-W4"};
        
        performGetWithholdingForms(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
                                    expectedWithholdingForms);
    }
    
    /**
     * <P>Tests applicable withholding forms for S-Corp in San Mateo, CA; and employee is partner of employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to S-Corp</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected withholding forms</LI>
     * <LI>Set relationship to partner</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testS_CorpInSanMateoForPartnerAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
        sEmployer.setNonProfit(false);
        
        // Set up expected list of withholding forms
        String[] expectedWithholdingForms = {"US-CA-DE-4", "US-FED-W4"};
        
        performGetWithholdingForms(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
                                    expectedWithholdingForms);
    }
    
    /**
     * <P>Tests applicable withholding forms for LLC in San Mateo, CA; and employee is partner of employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to LLC</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected withholding forms</LI>
     * <LI>Set relationship to partner</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testLLCInSanMateoForPartnerAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.LLC);
        sEmployer.setNonProfit(false);
        
        // Set up expected list of withholding forms
        String[] expectedWithholdingForms = {};
        
        performGetWithholdingForms(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
                                    expectedWithholdingForms);
    }
    
    /**
     * <P>Tests applicable withholding forms for Partnership in San Mateo, CA; and employee is partner of employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to Partnership</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected withholding forms</LI>
     * <LI>Set relationship to partner</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testPartnershipInSanMateoForPartnerAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.PARTNERSHIP);
        sEmployer.setNonProfit(false);
        
        // Set up expected list of withholding forms
        String[] expectedWithholdingForms = {};
        
        performGetWithholdingForms(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
                                    expectedWithholdingForms);
    }
    
    /**
     * <P>Tests applicable withholding forms for SoleProprietor in San Mateo, CA; and employee is partner of employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to SoleProprietor</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected withholding forms</LI>
     * <LI>Set relationship to partner</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testSoleProprietorInSanMateoForPartnerAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.SOLEPROPRIETOR);
        sEmployer.setNonProfit(false);
        
        // Set up expected list of withholding forms
        String[] expectedWithholdingForms = {};
        
        performGetWithholdingForms(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
                                    expectedWithholdingForms);
    }
    
}