package com.intuit.sb.payroll.services.taxes;

import java.math.BigDecimal;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.taxes.TaxComponent;
import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.Employee;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;

public class UpdateWithholdingsTest extends TaxesTestBase {
	static private TaxWithholdingsService taxWithholdingsService;
	static private TaxComponent taxComponent;
	static private WorkerComponent workerComponent;
	static private Long sUniqueId = null;

	@BeforeClass
	public static void setUpUpdateWitholding() {
		taxWithholdingsService = qaBootstrap.getEjbFromClass(TaxWithholdingsService.class);
		taxComponent = qaBootstrap.getEjbFromClass(TaxComponent.class);
		workerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);

		try {
			getUserTransaction().begin();
			saveEmployerInfo();
			setUpSanMateoAddress();

			// Save a new Worker with default withholdings to be used by all tests
			Worker worker = WorkerTestUtils.createEmployee();
			WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sSanFranciscoLocation);
			worker.setWorkLocation(workLocation);
			worker = taxComponent.applyApplicableTaxes(worker);
			worker = workerComponent.addWorker(worker);
			sUniqueId = worker.getId();

			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			AssertJUnit.fail("Error in setUpUpdateWitholding(): " + t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
		}
	}

	@AfterClass
	public static void tearDownUpdateWitholding() {
		try {
			getUserTransaction().begin();
			removeWorkerWithholdingInfoByType(sUniqueId, ComplianceDocIdEnum.W4IRS);
			removeWorkerWithholdingInfoByType(sUniqueId, ComplianceDocIdEnum.DE4);
			Employee worker = entityService.findById(Employee.class, sUniqueId);
			entityService.makeTransient(worker);
			restoreEmployerInfo();
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			AssertJUnit.fail("Error in tearDownUpdateWitholding(): " + t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
		}
	}

	@Test
	public void testAddEmployeeWithholdings() throws Exception {

		WithholdingInformation[] withholdingDataList = null;
		WithholdingInformation w4 = new WithholdingInformation();
		WithholdingInformation de4 = new WithholdingInformation();

		withholdingDataList = taxWithholdingsService.getEmployeeWithholdings(sUniqueId).clone();
		AssertJUnit.assertNotNull("Withholding list returned", withholdingDataList);
		AssertJUnit.assertEquals("The worker should have 0 withholdings to start with", 0, withholdingDataList.length);

		w4.setFormType(FormType.US_W4); // fed income tax
		w4.setWithholdingStatus(WithholdingStatus.SINGLE);

		SaveWithholdingInfoRequest w4request = new SaveWithholdingInfoRequest();
		w4request.setWithholdingInfo(w4);
		w4request.setWorkerId(sUniqueId);
		w4request.setWorkerTaxIdNumber("111-22-3333");
		w4request.setWithholdingInfoOperation(WithholdingInfoOperation.ADD);

		taxWithholdingsService.saveWithholdingInformation(w4request);

		de4.setFormType(FormType.CA_DE4); // CA income tax
		de4.setWithholdingStatus(WithholdingStatus.MARRIED_HEAD_OF_HOUSEHOLD);
		de4.setAllowances(5);
		de4.setExtraWithholding(new BigDecimal("150.00"));

		SaveWithholdingInfoRequest de4request = new SaveWithholdingInfoRequest();
		de4request.setWithholdingInfo(de4);
		de4request.setWorkerId(sUniqueId);
		de4request.setWorkerTaxIdNumber("111-22-3333");
		de4request.setWithholdingInfoOperation(WithholdingInfoOperation.ADD);

		taxWithholdingsService.saveWithholdingInformation(de4request);

		withholdingDataList = taxWithholdingsService.getEmployeeWithholdings(sUniqueId);
		AssertJUnit.assertNotNull("Withholding list returned", withholdingDataList);
		AssertJUnit.assertEquals("All the withholdings were added to the worker", 2, withholdingDataList.length);
	}

	@Test(expectedExceptions = ValidationException.class)
	public void testNegativeWithholdingAllowance() throws Exception {
		WithholdingInformation apiWithholdingInformation = new WithholdingInformation();
		apiWithholdingInformation.setFormType(FormType.US_W4); // fed income tax
		apiWithholdingInformation.setAllowances(-1);
		SaveWithholdingInfoRequest w4Request = new SaveWithholdingInfoRequest();
		w4Request.setWithholdingInfo(apiWithholdingInformation);
		w4Request.setWorkerId(sUniqueId);
		w4Request.setWorkerTaxIdNumber("111-22-3333");
		w4Request.setWithholdingInfoOperation(WithholdingInfoOperation.ADD);

		taxWithholdingsService.saveWithholdingInformation(w4Request);
	}

	@Test(expectedExceptions = ValidationException.class)
	public void testNegativeExtraWithholding() throws Exception {
		WithholdingInformation apiWithholdingInformation = new WithholdingInformation();
		apiWithholdingInformation.setFormType(FormType.US_W4); // fed income tax
		apiWithholdingInformation.setExtraWithholding(new BigDecimal("-0.01"));
		SaveWithholdingInfoRequest w4Request = new SaveWithholdingInfoRequest();
		w4Request.setWithholdingInfo(apiWithholdingInformation);
		w4Request.setWorkerId(sUniqueId);
		w4Request.setWorkerTaxIdNumber("111-22-3333");
		w4Request.setWithholdingInfoOperation(WithholdingInfoOperation.ADD);

		taxWithholdingsService.saveWithholdingInformation(w4Request);
	}

	@Test(expectedExceptions = ValidationException.class)
	public void testNullWithholdingFormType() throws Exception {
		WithholdingInformation apiWithholdingInformation = new WithholdingInformation();
		apiWithholdingInformation.setFormType(null); // fed income tax

		SaveWithholdingInfoRequest w4Request = new SaveWithholdingInfoRequest();
		w4Request.setWithholdingInfo(apiWithholdingInformation);
		w4Request.setWorkerId(sUniqueId);
		w4Request.setWorkerTaxIdNumber("111-22-3333");
		w4Request.setWithholdingInfoOperation(WithholdingInfoOperation.ADD);

		taxWithholdingsService.saveWithholdingInformation(w4Request);
	}

}
