package com.intuit.sb.payroll.services.wages;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import javax.transaction.Status;

import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeClass;

import com.intuit.sb.payroll.capability.common.BasicTestBase;
import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.common.ExceptionUtils;
import com.intuit.sb.payroll.entity.model.Compensation;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.DatumType1Enum;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.wages.CompensationEAO;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.wages.impl.CompensationItemMapper;

/**
 *  <P>The <CODE>WageItemTestBase</CODE> class contains helper methods for setting up wage items</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class CompensationItemTestBase extends BasicTestBase {
    protected static List<CompensationItem> compensationItems = null;
    protected static WageService sWageService;
    protected static WorkerComponent sWorkerComponent;
    protected static CompensationEAO sCompensationEAO;

    @BeforeClass
    public static void initializeCompensationItemTestBaseClass()
    {
    	sWageService = qaBootstrap.getEjbFromClass(WageService.class);
    	sWorkerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);
    	sCompensationEAO = qaBootstrap.getEjbFromClass(CompensationEAO.class);
    }
    
    @AfterMethod
    public void tearDown() throws Exception {
    	try {
            if (getUserTransaction().getStatus() == Status.STATUS_ACTIVE) {
                // Remove all payroll items associated with the worker
                removeAllWorkerPayrollItems(sWorkerId);
            }
        } catch (Throwable e) {
            String mesg = "Error during teardown in CompensationItemTestBase: " + ExceptionUtils.throwableMessage(e);
            sLogger.error(mesg, e);
            e.printStackTrace();
        }
        super.tearDown();
    }

    /**
     * <P>Sets up a Salary and a Bonus wage item for the specified worker.
     * 
     * Note: The Salary wage is $66,000.00 and the Bonus wage is $1,000.00
     * </P>
     * 
     * @param workerId The uniqueId of the worker to set up the wages
     */
    public static void setupSalaryAndBonusWages(Long workerId) throws Exception {
    	Worker worker = entityService.findById(Worker.class, workerId);
    	worker = WorkerTestUtils.setupSalaryWage(worker);
    	// Add a variable bonus wage
    	worker = WorkerTestUtils.addBonus(worker, false);
    }

    /**
     * <P>Sets up an Hourly, Overtime and Bonus wage item for the specified worker.  Also sets up the compensationItems
     *    with these wageItems.</P>
     * 
     * @param workerId The uniqueId of the worker to set up the wages
     */
    public static Worker setupWages(Long workerId) throws Exception {
    	Worker worker = entityService.findById(Worker.class, workerId);
    	return setupWages(worker);
    }
    
    public static Worker setupWages(Worker worker) throws Exception {
    	worker = WorkerTestUtils.setupWages(worker);

    	// Also set up the expected CompensationItem information
    	compensationItems = new ArrayList<CompensationItem>();
    	CompensationItem hourlyItem = createCompensationItem(CompensationType.Hourly, true);
    	compensationItems.add(hourlyItem);
        CompensationItem overtimeItem = createCompensationItem(CompensationType.Overtime, true);
        // Link the overtime to the hourly item
        overtimeItem.setLinkedCompensationItem(hourlyItem);
        overtimeItem.setRate(new BigDecimal(1.5));
    	compensationItems.add(overtimeItem);
    	compensationItems.add(createCompensationItem(CompensationType.Bonus, false));

        // Now fix up the compensationItem uniqueIds and version value
        for (CompensationItem compensationItem : compensationItems) {
            fixUpCompensationItemVersionAndUniqueId(worker, compensationItem);
        }
        return worker;
    }

    private static CompensationItem createCompensationItem(CompensationType compensationType, boolean fixed) {
    	CompensationTypeEnum compensationTypeEnum = CompensationItemMapper.mapToCompensationTypeEnum(compensationType);
		Compensation compensation = sCompensationEAO.findCompensation(compensationTypeEnum);
    	Datum datum = WorkerTestUtils.initDefaultCompensationDatumValue(compensation.getLawId().getValue(), compensationTypeEnum);

    	CompensationItem compensationItem = new CompensationItem();
    	compensationItem.setFixedRate(fixed);
    	if (DatumType1Enum.MONEY.equals(datum.getType1())) {
        	compensationItem.setRate(datum.getValue1AsUSDMoneyAmount());    		
    	} else if (DatumType1Enum.QUANTITY.equals(datum.getType1())) {
    		compensationItem.setRate(new BigDecimal(datum.getValue1AsQuantity()));
    	}
    	compensationItem.setCompensationType(compensationType);
    	compensationItem.setName(compensation.getDescription());
    	compensationItem.setUnits(0.0);
    	
    	return compensationItem;
    }

    private static void fixUpCompensationItemVersionAndUniqueId(Worker worker, CompensationItem compensationItem) {
    	for (CompensationDtl compensationDtl : worker.getCompensationDtls()) {
            if (compensationDtl.getPayrollItem().getLawId().getValue().equals(
            		compensationItem.getCompensationType().getLawId())) {
            	
                compensationItem.setObjectHandle(
                		new ObjectHandle(compensationDtl.getId(), compensationDtl.getVersion()));
                break;
            }
        }
    }
    
    /**
     * <P>Output the CompensationItem information to the log</P>
     * 
     * @param compensationItem The CompensationItem to output
     */
    protected void printCompensationItem(CompensationItem compensationItem) {
        sLogger.error(compensationItem.toString());
    }
    
    /**
     * <P>Returns true if the specified CompensationItem lists are equal</P>
     * 
     * @param computedCompensationItems The computed CompensationItem list to verify
     * @param expectedCompensationItems The expected CompensationItem list to check against
     * @return true if the 2 CompensationItem lists are equal
     */
    protected boolean verifyCompensationItems(List<CompensationItem> computedCompensationItems,
    				List<CompensationItem> expectedCompensationItems) {
        if ( computedCompensationItems == null || expectedCompensationItems == null) {
            throw new RuntimeException("Null parameter");
        }
        
        boolean compareResult = (computedCompensationItems.size() == expectedCompensationItems.size());
        if (true == compareResult) {
            // Only compare if the number of items matches the expected number
            HashSet<CompensationItem> computedSet = new HashSet<CompensationItem>();
            for(int i=0; i < computedCompensationItems.size(); i++) {
                computedSet.add(computedCompensationItems.get(i));
            } 
            HashSet<CompensationItem> expectedSet = new HashSet<CompensationItem>();
            for(int i=0; i < expectedCompensationItems.size(); i++) {
                expectedSet.add(expectedCompensationItems.get(i));
            }
            compareResult = computedSet.equals(expectedSet);
        }
        
        if (false == compareResult) {
            sLogger.error("Computed CompensationItem items:");            
            for(int i=0; i < computedCompensationItems.size(); i++) {
                printCompensationItem((CompensationItem)computedCompensationItems.get(i));            
            }            
            sLogger.error("Expected CompensationItem items:");            
            for(int i=0; i < expectedCompensationItems.size(); i++) {
                printCompensationItem((CompensationItem)expectedCompensationItems.get(i));            
            }            
        }
        
        return compareResult;
    }    
}

