package com.intuit.sb.payroll.entity.common;

import java.util.List;

import javax.ejb.Local;
import javax.persistence.OptimisticLockException;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.entity.domain.Identifiable;

@Local
public interface EntityService {
	@SuppressWarnings("nls")
	public static final String JNDI_NAME =  "EntityService/local";
    
	/**
	 * Find the data object by its unique ID (primary key). Whether this eagerly or lazily
	 * cascades to loading associated entities is determined by the {@code fetch=FetchType.X}
	 * attribute of the association annotations.
	 * <p>
	 * @param Id The Id to query by. 
	 * @return The selected object, or null if there is no match for the specified Id.
	 */
	public <T> T findById(Class<T> type, Long Id);

	/**
	 * Find the data object by its identifiable properties. Whether this eagerly or lazily
	 * cascades to loading associated entities is determined by the {@code fetch=FetchType.X}
	 * attribute of the association annotations.
	 * @param identifiable the identifiable properties that uniquely specify an instance of an entity. 
	 * @return The selected object, or null if there is no match.
	 * @throws IllegalArgumentException if the entity does not exist.
	 * @throws OptimisticLockException if the entity is newer then the supplied Identifiable.
	 */
	public <T> T findByIdentifiable(Class<T> type, Identifiable identifiable);

	/**
	 * Find all Data objects of a given type.
	 * @return a List of objects or an empty collection if there are no matches.
	 */
	public <T> List<T> findAll(Class<T> type);

    /**
     * Associates a detached entity with the current persistence context so it is now
     * a managed entity and takes part in the full persistence life cycle.
     * <p>
     * The returned object will not be the same Java object as you pass to this method.
	 * You must update any references to this entity with the newly managed Java object
	 * and discard the old, stale, reference.
     *  
     * @param entity The entity to place under persistence management.
     * @return the reference to the object in the new PC.
     */
    public <T> T makeManaged(T entity);
    
    /**
     * Adds a Java object to the persistent store. More strictly, it makes a transient
     * object a persistent entity. This causes appropriate row(s) to be added to the DB.
     * <p>
     * All objects returned by {@link #create()} must be passed to {@code makePersistent()}
     * in order to have them persisted to the DB. Failure to do so often results in an
     * {@code org.hibernate.TransientObjectException}.
     * <p>
     * Once an object has been persisted there is no need to call {@code makePersistent()}
     * on it again, even when it has been modified in a future transaction; updates will
     * automatically be saved to the DB.
     * <p>
     * Detached entities should not be passed to {@code makePersistent()} as the behaviour of this
     * request on detached objects varies across persistence implementations. Instead,
     * detached objects should be passed to a {@code makeManaged()} request.
     * <p>
     * Once a transient object has been made persistent via a {@code makePersistent()} request
     * it can be updated just like any other managed entity. All updates to an entity between
     * a {@code makePersistent()} call and transaction commit will be persisted during the commit
     * as part of the insert into the persistent store. This occurs with one exception: read-only
     * fields (those marked updatable=false) cannot be modified after the {@code makePersistent()}
     * call. All read-only fields must be set before the {@code makePersistent()} call; any changes 
     * to read-only fields of a managed entity are ignored during transaction commit. 
     * <p>
     * The object will not be permanently written to the persistent store until the current
     * transaction is committed.
     * 
     * @param entity The entity to persist.
     * @return The newly managed entity. The returned object will not be the same Java object as
     * you pass to this method.
     */
    public <T> T makePersistent(T entity);

    /**
     * Returns true if the entity is not in session (is not managed by the EntityManager).
     * 
     * The intent is to inform the caller if they need to make the entity persistent themselves.
     * 
     * @param entity
     * @return true if the entity is not in session.
     */
    public <T> boolean notPersistent(T entity);
    
    /**
     * Removes an entity from the DB. Its row(s) are deleted from the DB. The object is then
     * transient and won't exist once it is garbage collected by the VM.
     * <p>
     * The object will not actually be removed until the current transaction is committed.
     * <p>
     *  
     * @param entity The entity to remove.
     */
    public <T> void makeTransient(T entity);
    
    /**
     * Attempts to flush the current session to the DB.
     */
    public void flush();


    /** Set the TenantContext attribute for the transaction when external calls
     *  require data to be persisted
     * @param companyId
     * @throws ServiceFailureException
     */
    public void setTenantContext(String companyId) throws ServiceFailureException;

    /** Clear the TenantContext after persisting data from external call
     *
     * @throws ServiceFailureException
     */
    public void clearTenantContext();
}
