package com.intuit.sb.payroll.entity.task;

import java.util.Date;
import java.util.List;

import javax.ejb.Local;

import com.intuit.sb.payroll.entity.eao.GenericEAO;
import com.intuit.sb.payroll.entity.model.Task;
import com.intuit.sb.payroll.entity.model.TaskStatusEnum;

/**
 * EAO operations for {@link Task Task} entities.
 * This class isolates all persistence concerns for Task entities from
 * business logic. It provides all the CRUD operations on {@code Task}
 * entities.
 *
 * @see Task, TaskEOALocal, TaskEAORemote
 */
@Local
public interface TaskEAO extends GenericEAO<Task>
{
	public static final String JNDI_NAME = "TaskEAO/local";

	/**
	 * Get the collection of active Tasks for a given task class. Active means "not started" or "started".
	 * 
	 * @param taskClass The task class to select upon. Logically this is the business logic java Class name that
	 * signals the associated events. In practice it'll be a proxy name for the class in order to avoid exposing
	 * implementation details over the wire.
	 * @return the query results set. 
	 */
	public List<Task> findActiveTasksByClass(String taskClass);

	
	/**
	 * Get the collection of active Tasks for a given task class and payload. Active means "not started" or "started".
	 * 
	 * @param taskClass The task class to select upon. Logically this is the business logic java Class name that
	 * signals the associated events. In practice it'll be a proxy name for the class in order to avoid exposing
	 * implementation details over the wire.
	 * @param payload The task payload. This is a task class specific value used to identify the primary
	 * domain entity associated with a specific event (i.e the Worker for a "pay worker" task).
	 * @return the query results set. 
	 */

	public List<Task> findActiveTasksByClassAndPayload(String taskClass,
			Long payload);

	/**
	 * Get the collection of <i>Current</i> Tasks.
	 * <p>
	 * <i>Current</i> tasks are defined as those that are:
	 * <ul>
	 * <li>inactive (complete/cancelled) "date" tasks for a configured past time period</li>
	 * <li>active (notstarted/started) "date" tasks for a configured future time period</li>
	 * <li>active (notstarted/started) "date" tasks that are overdue</li>
	 * <li>active (notstarted/started) "setup" tasks</li>
	 * </ul>
	 * The collection never includes completed "setup" tasks.
	 *
	 * @param completionStartDate Include inactive tasks that were handled before this date.
	 * @param deadlineEndDate Include active tasks with a deadline before this date.
	 * @return the list of selected tasks (or an empty collection if there are no matches).
	 */
	public List<Task> findCurrentTasks(Date completionStartDate,
			Date deadlineEndDate);

	/**
	 * Get the collection of <i>Current</i> Tasks.
	 * <p>
	 * <i>Current</i> tasks are defined as those that are:
	 * <ul>
	 * <li>inactive (complete/cancelled) "date" tasks for a configured past time period</li>
	 * <li>active (notstarted/started) "date" tasks for a configured future time period</li>
	 * <li>active (notstarted/started) "date" tasks that are overdue</li>
	 * <li>active (notstarted/started) "setup" tasks</li>
	 * </ul>
	 * The collection never includes completed "setup" tasks.
	 *
	 * @return the list of selected tasks (or an empty collection if there are no matches).
	 */
	public List<Task> findCurrentActiveTasks();

	/**
	 * Get the Collection of <i>All</i> Tasks whose payload value is the same as workerId and
	 * name is same as TaskName
	 * 
	 * @param taskName
	 * @param workerId
	 * @return the list of selected tasks (or an empty collection if there are no matches).
	 */
	
	public List<Task> findTaskByTaskNameAndPayLoad(String taskName, Long payLoad);

	/*************************************************** [rj] added from here *******************************/
	/**
	 * Get the collection of <i>Overdue</i> Tasks.
	 * <p>
	 * <i>Overdue Tasks</i> tasks are defined as those that are:
	 * <ul>
	 * <li>active (notstarted/started) "date" tasks that are past overdue date</li>
	 * </ul>
	 * The collection never includes completed/Canceled tasks.
	 *
	 * @return the list of selected tasks (or an empty collection if there are no matches).
	 */
	public List<Task> getOverdueTasks();

	/**
	 * Get the collection of <i>Active</i> Tasks.
	 * <p>
	 * <i>Active Tasks</i> tasks are defined as those that are:
	 * <ul>
	 * <li>are in either New or InProgress state</li>
	 * </ul>
	 * The collection never includes completed/Canceled tasks.
	 *
	 * @return the list of selected tasks (or an empty collection if there are no matches).
	 */
	public List<Task> getActiveTasks(Date fromDate, Date toDate);

	/**
	 * Get the collection of Tasks between the dates specified.
	 *
	 * @return the list of selected tasks (or an empty collection if there are no matches).
	 */
	public List<Task> getTasks(Date fromDate, Date toDate);
	
	public Task updateTaskStatus(Task task, TaskStatusEnum taskStatusEnum);
}
