package com.intuit.sb.payroll.qa.services.employer;



import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.testng.Assert;
import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.common.PTSEmployerServiceTestBase;
import com.intuit.sb.payroll.capability.common.PTSWorkerPaymentServiceTestBase;
import com.intuit.sb.payroll.capability.pts.adapter.dtos.CompanyActivationStatusEnum;
import com.intuit.sb.payroll.capability.pts.adapter.dtos.NotificationTypeEnum;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.pts.entity.PPSPTSInteraction;
import com.intuit.sb.payroll.pts.entity.PTSPayrollRun;
import com.intuit.sb.payroll.services.common.BankAccount;
import com.intuit.sb.payroll.services.employer.AssistedStateType;
import com.intuit.sb.payroll.services.employer.EmployerServiceTestBase;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSaveRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSummary;



public class PTSSubmitRegularPayrollTest extends PTSWorkerPaymentServiceTestBase{
	
	private static Worker worker;	
	
	/**
     * <P>Test submit Payroll run for a Company which is not in PSP</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a New Employer with Bank</LI>
     * <LI>Create an assisted ready employee</LI>
     * <LI>Run a Payroll</LI>
     * </OL>
     * <UL>
     * <LI>Verify that Submit Payroll run call not made to PSP</LI>
     * </UL>
     */
	@Test(enabled=false)// Null pointer exception in tear down of CompensationItemTestBase
	public void testSubmitPayrollOfAssistedReadyEmployerWithAssistedReadyEmployee()throws Exception{
		
		prepareCurrentEmployerForAssistedWithBank();		
		
		worker = new Worker();
		Date hireDate = DateUtil.getNowDate();
		worker = createAssistedReadyEmployee(worker, hireDate);	
		
		Long[] workerIds = new Long[1];
		workerIds[0] = worker.getId();
		Date payPeriodEndDate = DateUtil.getNowDate();
		PayrollRunSaveRequest payrollRunSaveRequest = setUpPayrollRunSaveRequest(workerIds, payPeriodEndDate);
		
		//Exercise
		PayrollRunSummary payrollRunSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
    	
    	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
     	
     	//Assert that no call for Submit Payroll is made by Gemini
     	List<PPSPTSInteraction> ppsPTSInteractions = ppsPTSInteractionEAO.findAll(PayrollRun.class.getName(), payrollRunSummary.getPayrollRunId());
    	AssertJUnit.assertEquals("Submit Payroll should not be called", 0, ppsPTSInteractions.size());
    	
    	//Assert that SubmitPayroll response from PSP is null
     	PTSPayrollRun ptsPayrollRun = ptsPayrollRunEAO.findPTSPayrollRunByPayrollRunId(payrollRunSummary.getPayrollRunId());
     	AssertJUnit.assertNull("Submit Payroll response for DIY Employer should be null",ptsPayrollRun);
     	
	}
		
	/**
     * <P>Test submitting Payroll run for a Company whose Bank Validation is Pending</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a New Employer with Bank</LI>
     * <LI>Create an assisted ready employee</LI>
     * <LI>Add company to PSP</LI>
     * <LI>Run a Payroll</LI>
     * </OL>
     * <UL>
     * <LI>Verify that Submit Payroll run call not made to PSP</LI>
     * </UL>
     */
	@Test(enabled=false)//Expected Validation Exception but Service failure exception is thrown
	public void testSubmitPayrollOfEmployerInProcessOfAssistedWithEmployeeInPSP()throws Exception{
	
		prepareCurrentEmployerForAssistedWithBank();
		Calendar cal = Calendar.getInstance();
		cal.add(Calendar.DAY_OF_YEAR, -4);
		Date hireDate = cal.getTime();
		worker = createAssistedReadyEmployee(worker, hireDate);
		
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
		
		Long[] workerIds = new Long[1];
		workerIds[0] = worker.getId();
		Date payPeriodEndDate = DateUtil.getNowDate();
		PayrollRunSaveRequest payrollRunSaveRequest = setUpPayrollRunSaveRequest(workerIds, payPeriodEndDate);
		
		try{
				//Exercise
				sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
				Assert.fail("Expected Validation Exception");
				
	    	}catch(ValidationException ve){
				//Verify
				//Assert that no call for Submit Payroll is made by Gemini
				//TODO hook to PSP Test Web Service for data validation	
			}		
	}
	
	/**
     * <P>Test submitting Payroll run for a Company whose Bank Validation is failed</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a New Employer with Bank</LI>
     * <LI>Create an assisted ready employee</LI>
     * <LI>Add company to PSP</LI>
     * <LI>Mock Bank Validation failed call</LI>
     * <LI>Run a Payroll</LI>
     * </OL>
     * <UL>
     * <LI>Verify that Submit Payroll run call not made to PSP</LI>
     * </UL>
     */
	@Test(enabled=false)// Expected Validation exception but Service failure exception is thrown
	public void testSubmitPayrollOfInprocessOfAssistedEmployerWithBankFailed() throws Exception{
		prepareCurrentEmployerForAssistedWithBank();	
		Date hireDate = DateUtil.getNowDate();
		worker = createAssistedReadyEmployee(worker, hireDate);
		
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
		
		BankAccount[] bankAccounts = sEmployerService.getBankAccounts();
		
		//Mock Bank Verification Failed Notification
		PTSEmployerServiceTestBase.mockBankAccountInboundCall(NotificationTypeEnum.ACTIVATION_ITEM_BANK_ACCOUNT_VERIFY,
				CompanyActivationStatusEnum.ACTIVATION_CANCELLED, EmployerServiceTestBase.getBankIdFromBankAccount(bankAccounts[0]));
		
		Long[] workerIds = new Long[1];
		workerIds[0] = worker.getId();
		Date payPeriodEndDate = DateUtil.getNowDate();
		PayrollRunSaveRequest payrollRunSaveRequest = setUpPayrollRunSaveRequest(workerIds, payPeriodEndDate);
		
		try{
		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
     	
    	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
		}catch(ValidationException ve){
			//Verify
			//Assert that no call for Submit Payroll is made by Gemini
			//TODO hook to PSP Test Web Service for data validation	
		}
	}
	
	/**
     * <P>Test submitting Payroll run for a Company whose Bank Validation is Success</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a New Employer with Bank</LI>
     * <LI>Create an assisted ready employee</LI>
     * <LI>Add company to PSP</LI>
     * <LI>Mock Bank Validation success call</LI>
     * <LI>Run a Payroll</LI>
     * </OL>
     * <UL>
     * <LI>Verify that Submit Payroll run call not made to PSP</LI>
     * </UL>
     */
	@Test(enabled=true)
	public void testSubmitPayrollOfInprocessOfAssistedEmployerWithActiveBank() throws Exception{
		
		prepareCurrentEmployerForAssistedWithBank();
		Calendar cal = Calendar.getInstance();
		cal.add(Calendar.DAY_OF_YEAR, -4);
		Date hireDate = cal.getTime();
		worker = createAssistedReadyEmployee(worker, hireDate);
		
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
		
		BankAccount[] bankAccounts = sEmployerService.getBankAccounts();
		
		//Mock Bank Verification Completed Notification
		PTSEmployerServiceTestBase.mockBankAccountInboundCall(NotificationTypeEnum.ACTIVATION_ITEM_BANK_ACCOUNT_VERIFY,
				CompanyActivationStatusEnum.ACTIVATION_ACTIVE, EmployerServiceTestBase.getBankIdFromBankAccount(bankAccounts[0]));
		
		Long[] workerIds = new Long[1];
		workerIds[0] = worker.getId();
		Date payPeriodEndDate = DateUtil.getNowDate();
		PayrollRunSaveRequest payrollRunSaveRequest = setUpPayrollRunSaveRequest(workerIds, payPeriodEndDate);
		try{
			//Exercise
			sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
			
			Assert.fail("Expected Service failure exception");     	
			
		}catch(ServiceFailureException sfe){
			//Assert that call for Submit Payroll is made by Gemini
			//TODO hook to PSP Test Web Service for data validation	    			
		}
	}
	
	
	
	
	
	
	
	
}
