package com.intuit.sb.payroll.qa.services.compensation;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.testng.AssertJUnit;

import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemRequest;

/**
 * Tests Commission handling of the CompensationService
 *
 * $Author$ 
 * $Revision$
 * $DateTime$
 */
public class CommissionCompensationTest extends CompensationTestBase {

	/**
	 * <P>
	 * Test Adding Commission Wage Item with fixed rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Commission Wage with fixed rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Commission Wage Item is Saved</LI>
	 */
	@Test
	public void testAddFixedCommission() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		// Setup
		CompensationItem compItem = new CompensationItem();
		compItem.setCompensationType(CompensationType.Commission);
		compItem.setFixedRate(true);
		compItem.setName("Comission");
		compItem.setRate(new BigDecimal("20.22"));
		compensationItemList.add(compItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);

		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(updatedWageItems, compensationItemList));

	}

	/**
	 * <P>
	 * Test Adding Commission Wage Item with fixed rate as null
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Commission Wage with fixed rate as null</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Commission Wage Item is not Saved</LI>
	 */
	@Test
	public void testAddVariedCommission() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		// Setup
		CompensationItem compItem = new CompensationItem();
		compItem.setCompensationType(CompensationType.Commission);
		compItem.setFixedRate(false);
		compItem.setName("Comission");
		compItem.setRate(new BigDecimal("20.22"));
		compensationItemList.add(compItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);

		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		modifyCompensationRate(compensationItemList, true, new BigDecimal("20.22"));

		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(updatedWageItems, compensationItemList));

	}

	/**
	 * <P>
	 * Test Adding Commission Wage Item with fixed rate as null
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Commission Wage with fixed rate as null</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Commission Wage Item is not Saved</LI>
	 */
	@Test
	public void testAddFixedCommissionWithNull() throws Exception {
		getUserTransaction().commit();
		boolean validationExceptionThrown = false;
		CompensationItem[] initialWageItems = sWageService.getCompensationItems(sWorkerId);

		// Setup
		CompensationItem compItem = new CompensationItem();
		compItem.setCompensationType(CompensationType.Commission);
		compItem.setFixedRate(true);
		compItem.setName("Comission");
		compItem.setRate(null);

		CompensationItem[] wageItems = new CompensationItem[1];
		wageItems[0] = compItem;

		try {
			sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
					SaveCompensationItemOperation.ADD, wageItems));
		} catch (ValidationException ve) {

			validationExceptionThrown = true;
		}
		AssertJUnit.assertTrue("Expected ValidationException is not thrown", validationExceptionThrown);
		getUserTransaction().begin();
		validateWageItem(initialWageItems);

	}

	/**
	 * <P>
	 * Test Adding Commission Wage Item with varied rate as null
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Commission Wage with varied rate as null</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Commission Wage Item is not Saved</LI>
	 */
	@Test
	public void testVariedCommissionWithNull() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		// Setup
		CompensationItem compItem = new CompensationItem();
		compItem.setCompensationType(CompensationType.Commission);
		compItem.setFixedRate(false);
		compItem.setName("Comission");
		compItem.setRate(null);

		compensationItemList.add(compItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);

		// Verify
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(updatedWageItems, compensationItemList));

	}

	/**
	 * <P>
	 * Test Adding Commission Wage Item with fixed rate as negative
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Commission Wage with fixed rate as negative</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Commission Wage Item is not Saved</LI>
	 */
	@Test
	public void testAddFixedComissionWithNegative() throws Exception {
		getUserTransaction().commit();
		boolean validationExceptionThrown = false;
		CompensationItem[] compenItem = sWageService.getCompensationItems(sWorkerId);

		// Setup
		CompensationItem compItem = new CompensationItem();
		compItem.setCompensationType(CompensationType.Commission);
		compItem.setFixedRate(true);
		compItem.setName("Comission");
		compItem.setRate(new BigDecimal("-20"));

		CompensationItem[] wageItems = new CompensationItem[1];
		wageItems[0] = compItem;

		try {
			// Exercise
			sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
					SaveCompensationItemOperation.ADD, wageItems));
		} catch (ValidationException ve) {

			validationExceptionThrown = true;
		}
		AssertJUnit.assertTrue("Expected ValidationException is not thrown", validationExceptionThrown);
		getUserTransaction().begin();
		validateWageItem(compenItem);
	}

	/**
	 * <P>
	 * Test Adding Commission Wage Item with varied rate as negative
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Commission Wage with varied rate as negative</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Commission Wage Item is not Saved</LI>
	 */
	@Test
	public void testAddVariedCommissionhNegative() throws Exception {
		getUserTransaction().commit();
		boolean validationExceptionThrown = false;
		CompensationItem[] compenItem = sWageService.getCompensationItems(sWorkerId);

		// Setup
		CompensationItem compItem = new CompensationItem();
		compItem.setCompensationType(CompensationType.Commission);
		compItem.setFixedRate(false);
		compItem.setName("Comission");
		compItem.setRate(new BigDecimal("-20"));

		CompensationItem[] wageItems = new CompensationItem[1];
		wageItems[0] = compItem;

		try {
			// Exercise
			sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
					SaveCompensationItemOperation.ADD, wageItems));
		} catch (ValidationException ve) {

			validationExceptionThrown = true;
		}
		AssertJUnit.assertTrue("Expected ValidationException is not thrown", validationExceptionThrown);
		getUserTransaction().begin();
		validateWageItem(compenItem);

	}

	/**
	 * <P>
	 * Test Adding Commission Wage Item with fixed rate as zero
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Commission Wage with fixed rate as zero</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Commission Wage Item is not Saved</LI>
	 */
	@Test
	public void testAddFixedCommissionWithZero() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpComissionWage(true);
		wageItem.setRate(new BigDecimal("0"));

		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(updatedWageItems, compensationItemList));

	}

	/**
	 * <P>
	 * Test Adding Commission Wage Item with varied rate as zero
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Commission Wage with varied rate as zero</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Commission Wage Item is not Saved</LI>
	 */
	@Test
	public void testAddVariedCommissionWithZero() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpComissionWage(false);
		wageItem.setRate(new BigDecimal("0"));

		compensationItemList.add(wageItem);
		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		List<CompensationItem> complist = modifyCompensationRate(compensationItemList, true, new BigDecimal("0"));

		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(updatedWageItems, complist));

	}

	/**
	 * <P>
	 * Test creating duplicate Fixed Commission Wage Item with unique names
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 Fixed Commission wage items with unique names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Compare the saved and expected compensation items</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify saved and expected compensation items are the same</LI>
	 * 
	 */
	@Test
	public void testAddFixedCommissionWithUniqueName() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem comissionCompItem = new CompensationItem();
		comissionCompItem.setRate(new BigDecimal("122.50"));
		comissionCompItem.setFixedRate(true);
		comissionCompItem.setCompensationType(CompensationType.Commission);
		comissionCompItem.setName("Comission 1");

		CompensationItem comissionCompItem1 = new CompensationItem();
		comissionCompItem1.setRate(new BigDecimal("122.50"));
		comissionCompItem1.setFixedRate(true);
		comissionCompItem1.setCompensationType(CompensationType.Commission);
		comissionCompItem1.setName("Comission 2");

		compensationItemList.add(comissionCompItem);
		compensationItemList.add(comissionCompItem1);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		int count = 0;
		for (CompensationItem compItem : updatedWageItems) {
			if (CompensationType.Commission.equals(compItem.getCompensationType())) {
				if (compItem.getName().equals("Comission 2")) {
					count++;
				} else if (compItem.getName().equals("Comission 1")) {
					count++;
				}
			}
		}
		AssertJUnit.assertEquals(2, count);

	}

	/**
	 * <P>
	 * Test creating duplicate Varied Commission Wage Item with unique names
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 Varied Commission wage items with unique names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Compare the saved and expected compensation items</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify saved and expected compensation items are the same</LI>
	 * 
	 */
	@Test
	public void testAddVariedCommissionWithUniqueName() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem comissionCompItem = new CompensationItem();
		comissionCompItem.setRate(new BigDecimal("122.50"));
		comissionCompItem.setFixedRate(false);
		comissionCompItem.setCompensationType(CompensationType.Commission);
		comissionCompItem.setName("Comission 1");

		CompensationItem comissionCompItem1 = new CompensationItem();
		comissionCompItem1.setRate(new BigDecimal("122.50"));
		comissionCompItem1.setFixedRate(false);
		comissionCompItem1.setCompensationType(CompensationType.Commission);
		comissionCompItem1.setName("Comission 2");

		compensationItemList.add(comissionCompItem);
		compensationItemList.add(comissionCompItem1);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		int count = 0;
		for (CompensationItem compItem : updatedWageItems) {
			if (CompensationType.Commission.equals(compItem.getCompensationType())) {
				if (compItem.getName().equals("Comission 2")) {
					count++;
				} else if (compItem.getName().equals("Comission 1")) {
					count++;
				}
			}
		}
		AssertJUnit.assertEquals(2, count);

	}

	/**
	 * <P>
	 * Test creating duplicate fixed Commission Wage Item with same names
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 fixed Commission wage items with same names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Compare the saved and expected compensation items</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Commission wage item throws validation exception</LI>
	 * 
	 */
	@Test
	public void testAddFixedDuplicateCommissionWithSameName() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem1 = setUpComissionWage(true);
		wageItem1.setRate(new BigDecimal("21"));

		CompensationItem wageItem2 = setUpComissionWage(true);
		wageItem2.setRate(new BigDecimal("21"));

		compensationItemList.add(wageItem1);
		compensationItemList.add(wageItem2);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		int count = 0;
		for (CompensationItem compItem : updatedWageItems) {
			if (CompensationType.Commission.equals(compItem.getCompensationType())) {
				count++;
			}
		}
		AssertJUnit.assertEquals(2, count);

	}

	/**
	 * <P>
	 * Test creating duplicate varied Commission Wage Item with same names
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 varied Commission wage items with same names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Compare the saved and expected compensation items</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Commission wage item throws validation exception</LI>
	 * 
	 */
	@Test
	public void testAddVariedDuplicateCommissionWithSameName() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem1 = setUpComissionWage(false);
		CompensationItem wageItem2 = setUpComissionWage(false);

		compensationItemList.add(wageItem1);
		compensationItemList.add(wageItem2);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		int count = 0;
		for (CompensationItem compItem : updatedWageItems) {
			if (CompensationType.Commission.equals(compItem.getCompensationType())) {
				count++;
			}
		}
		AssertJUnit.assertEquals(2, count);

	}

	/**
	 * <P>
	 * Test Updating FixedComission Wage Item to negative rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up FixedComission Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value i.e -20</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify FixedCommission Wage Item is not Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateFixedComissionWithNegative() throws Exception {
		getUserTransaction().commit();
		boolean validationExceptionThrown = false;
		// CompensationItem[] compenItem =
		// addWageItem(CompensationType.TipsPaycheck, true, "Fixed Tips", new
		// BigDecimal(20));

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem compItem = new CompensationItem();
		compItem.setCompensationType(CompensationType.Commission);
		compItem.setFixedRate(true);
		compItem.setName("Commission");
		compItem.setRate(new BigDecimal("20.00"));

		compensationItemList.add(compItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update the varied commission wage rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		BigDecimal bigDecimal = new BigDecimal("20.00");
		bigDecimal = bigDecimal.setScale(2);

		for (CompensationItem cItem : updatedWageItems) {
			if (cItem.getRate().equals(bigDecimal)) {
				cItem.setRate(new BigDecimal("-20"));
				compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, cItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		try {
			// If the list has values , then call operation to update it
			if (compItemList.size() > 0) {
				SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
				newCompItems = compItemList.toArray(newCompItems);

				sWageService.saveCompensationItems(sWorkerId, newCompItems);
			} else
				AssertJUnit.fail("No Compensation Items to Update");
		} catch (ValidationException ve) {

			validationExceptionThrown = true;
		}

		AssertJUnit.assertTrue("Expected ValidationException is not thrown", validationExceptionThrown);
		getUserTransaction().begin();
		modifyCompensationRate(Arrays.asList(updatedWageItems), false, new BigDecimal("20.00"));
		validateWageItem(updatedWageItems);
		// Cleanup the compensation item(s)
		updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		compItemList.clear();
		for (CompensationItem cItem : updatedWageItems) {
			compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, cItem));
		}
		SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
		newCompItems = compItemList.toArray(newCompItems);
		sWageService.saveCompensationItems(sWorkerId, newCompItems);
		getUserTransaction().commit();
		
	}

	/**
	 * <P>
	 * Test Updating VariedComission Wage Item to negative rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up VariedComission Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value i.e -20</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify VariedCommission Wage Item is not Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateVariedComissionWithNegative() throws Exception {
		getUserTransaction().commit();
		boolean validationExceptionThrown = false;

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem compItem = new CompensationItem();
		compItem.setCompensationType(CompensationType.Commission);
		compItem.setFixedRate(false);
		compItem.setName("Comission123");
		compItem.setRate(new BigDecimal("20.00"));

		compensationItemList.add(compItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update the varied commission wage rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem cItem : updatedWageItems) {
			if (cItem.getName().equals("Comission123")) {
				cItem.setRate(new BigDecimal("-20"));
				compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, cItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		try {
			// If the list has values , then call operation to update it
			if (compItemList.size() > 0) {
				SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
				newCompItems = compItemList.toArray(newCompItems);

				sWageService.saveCompensationItems(sWorkerId, newCompItems);
			} else
				AssertJUnit.fail("No Compensation Items to Update");

		} catch (ValidationException ve) {

			validationExceptionThrown = true;
		}
		AssertJUnit.assertTrue("Expected ValidationException is not thrown", validationExceptionThrown);
		getUserTransaction().begin();
		modifyCompensationRate(Arrays.asList(updatedWageItems), true, new BigDecimal("20.00"));
		validateWageItem(updatedWageItems);
		// Cleanup the compensation item(s)
		updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		compItemList.clear();
		for (CompensationItem cItem : updatedWageItems) {
			compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, cItem));
		}
		SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
		newCompItems = compItemList.toArray(newCompItems);
		sWageService.saveCompensationItems(sWorkerId, newCompItems);
		getUserTransaction().commit();

	}

	/**
	 * <P>
	 * Test Updating FixedComission Wage Item to zero rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up FixedComission Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value i.e zero</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify FixedComission Wage Item is not Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateFixedComissionZero() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem compItem = new CompensationItem();
		compItem.setCompensationType(CompensationType.Commission);
		compItem.setFixedRate(true);
		compItem.setName("Comission123");
		compItem.setRate(new BigDecimal("20.00"));

		compensationItemList.add(compItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		BigDecimal bigDecimal = new BigDecimal("20.00");
		// set to 2 decimal places
		bigDecimal = bigDecimal.setScale(2);

		for (CompensationItem cItem : updatedWageItems) {
			if (cItem.getRate().equals(bigDecimal)) {
				compItem.setRate(new BigDecimal("0.00"));
				compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, cItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		// Verify that Bonus has been updated
		BigDecimal bigDecimalZero = new BigDecimal("0.00");
		// set to 2 decimal places
		bigDecimalZero = bigDecimal.setScale(2);

		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		for (CompensationItem cItem : workerWageItemsNew) {
			if (cItem.getName().endsWith("Comission123")) {
				BigDecimal value = cItem.getRate();
				if (!value.equals(bigDecimalZero)) {
					AssertJUnit.fail("Varied Wage Item not updated to Fixed");
				}
			} else
				AssertJUnit.fail("Tips Comp Item not saved");
		}

	}

	/**
	 * <P>
	 * Test Updating VariedComission Wage Item to zero rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up VariedComission Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value i.e zero</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify VariedComission Wage Item is not Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateVariedComissionhZero() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem compItem = new CompensationItem();
		compItem.setCompensationType(CompensationType.Commission);
		compItem.setFixedRate(false);
		compItem.setName("Comission123");
		compItem.setRate(null);

		compensationItemList.add(compItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		// Update

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem cItem : updatedWageItems) {
			if (cItem.getCompensationType().equals(CompensationType.Commission)) {
				if (cItem.getRate() != null) {
					AssertJUnit.fail("Varied Tips cannot have a rate");
				}
				cItem.setRate(new BigDecimal("0"));
				compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, cItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		modifyCompensationRate(Arrays.asList(updatedWageItems), true, new BigDecimal("0"));
		validateWageItem(updatedWageItems);
	}

	/**
	 * <P>
	 * Test Updating FixedComission Wage Item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up FixedComission Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value i.e 34</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify FixedComission Wage Item is not Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateFixedComission() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem compItem = new CompensationItem();
		compItem.setCompensationType(CompensationType.Commission);
		compItem.setFixedRate(true);
		compItem.setName("Comission");
		compItem.setRate(new BigDecimal("20.00"));

		compensationItemList.add(compItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		BigDecimal bigDecimal = new BigDecimal("20.00");
		bigDecimal = bigDecimal.setScale(2);

		for (CompensationItem cItem : updatedWageItems) {
			if (cItem.getRate().equals(bigDecimal)) {
				cItem.setRate(new BigDecimal("34"));
				compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, cItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}
		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		modifyCompensationRate(Arrays.asList(updatedWageItems), false, new BigDecimal("34"));

		validateWageItem(updatedWageItems);

	}

	/**
	 * <P>
	 * Test Updating varied Comission Wage Item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up FixedComission Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value i.e 34</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify FixedComission Wage Item is not Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateVariedComission() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem compItem = new CompensationItem();
		compItem.setCompensationType(CompensationType.Commission);
		compItem.setFixedRate(false);
		compItem.setName("Comission");
		compItem.setRate(new BigDecimal("20.00"));

		compensationItemList.add(compItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		BigDecimal bigDecimal = new BigDecimal("20.00");
		bigDecimal = bigDecimal.setScale(2);

		for (CompensationItem cItem : updatedWageItems) {
			if (cItem.getRate().equals(bigDecimal)) {
				cItem.setRate(new BigDecimal("34"));
				compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, cItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		modifyCompensationRate(Arrays.asList(updatedWageItems), true, new BigDecimal("34"));
		validateWageItem(updatedWageItems);

	}

	/**
	 * <P>
	 * Test Updating FixedComission To Varied
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up FixedComission Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value i.e 34</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify FixedComission Wage Item is not Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateFixedToVariedComission() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem compItem = new CompensationItem();
		compItem.setCompensationType(CompensationType.Commission);
		compItem.setFixedRate(true);
		compItem.setName("Comission");
		compItem.setRate(new BigDecimal("20.00"));

		compensationItemList.add(compItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		BigDecimal bigDecimal = new BigDecimal("20.00");
		bigDecimal = bigDecimal.setScale(2);

		for (CompensationItem cItem : updatedWageItems) {
			if (cItem.getRate().equals(bigDecimal)) {
				cItem.setFixedRate(false);
				compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, cItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		modifyCompensationRate(Arrays.asList(updatedWageItems), true, new BigDecimal("20.00"));
		validateWageItem(updatedWageItems);

	}

	/**
	 * <P>
	 * Test Updating Varied Comission To Fixed
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up FixedComission Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value i.e 34</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify FixedComission Wage Item is not Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateVariedToFixedComission() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem compItem = new CompensationItem();
		compItem.setCompensationType(CompensationType.Commission);
		compItem.setFixedRate(false);
		compItem.setName("Comission");
		compItem.setRate(null);

		compensationItemList.add(compItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem cItem : updatedWageItems) {
			if (cItem.getRate() == null) {
				cItem.setFixedRate(true);
				cItem.setRate(new BigDecimal("20.00"));
				compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, cItem));
			} else
				AssertJUnit.fail("Tips Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		modifyCompensationRate(Arrays.asList(updatedWageItems), false, new BigDecimal("20.00"));
		validateWageItem(updatedWageItems);

	}

	/**
	 * <P>
	 * Test Deleting FixedCommission Wage Item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up FixedCommission Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Retrieve the saved FixedCommission</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work with operation as Delete</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify FixedCommission Wage Item is Deleted</LI>
	 * 
	 */
	@Test
	public void testDeleteFixedCommission() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpComissionWage(true);
		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.Commission)) {
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, compItem));
			}
		}
		// If the list has values , then call operation to delete it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);
			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		}

		// Verify
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		if (workerWageItemsNew.length > 0) {
			AssertJUnit.fail("Comission Wage Item not deleted");
		}

	}

	/**
	 * <P>
	 * Test Deleting VariedCommission Wage Item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up VariedCommission Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Retrieve the saved VariedCommission</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work with operation as Delete</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify VariedCommission Wage Item is Deleted</LI>
	 * 
	 */
	@Test
	public void testDeleteVariedCommission() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpComissionWage(false);
		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.Commission)) {
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, compItem));
			}
		}
		// If the list has values , then call operation to delete it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);
			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		}

		// Verify
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		if (workerWageItemsNew.length > 0) {
			AssertJUnit.fail("Comission Wage Item not deleted");
		}

	}

	/**
	 * <P>
	 * Test Deleting FixedCommission Wage Item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up FixedCommission Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Retrieve the saved FixedCommission</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work with operation as Delete</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify FixedCommission Wage Item is Deleted</LI>
	 * 
	 */
	@Test
	public void testDeleteFixedCommissionFromMultipleComissions() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpComissionWage(true);
		CompensationItem wageItem2 = setUpComissionWage(true);
		wageItem2.setRate(new BigDecimal("100.1"));
		compensationItemList.add(wageItem);
		compensationItemList.add(wageItem2);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);

		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.Commission)) {
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, compItem));
				break;
			}
		}
		// If the list has values , then call operation to delete it
		if (compItemList.size() > 0) {

			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);
			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		}

		// Verify
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);

		if (workerWageItemsNew.length > 1) {
			AssertJUnit.fail("Comission Wage Item not deleted");
		}

	}

	/**
	 * <P>
	 * Test Deleting FixedCommission Wage Item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up FixedCommission Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Retrieve the saved FixedCommission</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work with operation as Delete</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify FixedCommission Wage Item is Deleted</LI>
	 * 
	 */
	@Test
	public void testDeleteVariedCommissionFromMultipleComissions() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpComissionWage(false);
		CompensationItem wageItem2 = setUpComissionWage(false);

		compensationItemList.add(wageItem);
		compensationItemList.add(wageItem2);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.Commission)) {
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, compItem));
				break;
			}
		}
		// If the list has values , then call operation to delete it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);
			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		}

		// Verify
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		if (workerWageItemsNew.length > 1) {
			AssertJUnit.fail("Comission Wage Item not deleted");
		}

	}

}
