package com.intuit.sb.payroll.qa.services.compensation;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import org.testng.AssertJUnit;

import org.testng.annotations.Test;

import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemRequest;

/**
 * Tests Hourly, OT and DT handling in the CompensationService.
 *
 * $Author$ 
 * $Revision$
 * $DateTime$
 */
public class HourlyCompensationTest extends CompensationTestBase {

	/**
	 * <P>
	 * Test saving Hourly Wage with Doubletime and Overtime
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Hourly Wage, Double Time and Overtime. Link Double Time and
	 * Overtime to hourly</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of wage items returned matches what's expected
	 * </UL>
	 */
	@Test
	public void testAddHourlyWageItemWithDoubleOvertime() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpHourlyWageWithExtraTime("Both");
		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] workerWageItems = sWageService.getCompensationItems(sWorkerId);
		// Verify
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItemsWithLinkedItems(workerWageItems,
				compensationItemList, "Both"));
	}

	/**
	 * <P>
	 * Test saving Hourly Wage without Doubletime and Overtime
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Hourly Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of wage items returned matches what's expected
	 * </UL>
	 */
	@Test
	public void testAddHourlyWageItemWithNoExtraTime() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpHourlyWageWithExtraTime("");
		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] workerWageItems = sWageService.getCompensationItems(sWorkerId);

		// Verify
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItemsWithLinkedItems(workerWageItems,
				compensationItemList, "Both"));
	}

	/**
	 * <P>
	 * Test saving Hourly Wage with Overtime
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Hourly Wage and Overtime. Link Overtime to hourly</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of wage items returned matches what's expected
	 * </UL>
	 */
	@Test
	public void testSaveHourlyWageItemWithOvertime() throws Exception {
		// Setup

		List<CompensationItem> compensationItemList = setUpHourlyWageWithExtraTime("Overtime");
		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] workerWageItems = sWageService.getCompensationItems(sWorkerId);
		// Verify
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItemsWithLinkedItems(workerWageItems,
				compensationItemList, "Overtime"));
	}

	/**
	 * <P>
	 * Test saving Hourly Wage with Doubletime
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Hourly Wage and Double Time . Link Double Time to hourly</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of wage items returned matches what's expected
	 * </UL>
	 */
	public void testSaveHourlyWageItemWithDoubleTime() throws Exception {
		// Setup
		// Create Hourly Compensation Item

		List<CompensationItem> compensationItemList = setUpHourlyWageWithExtraTime("Doubletime");
		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] workerWageItems = sWageService.getCompensationItems(sWorkerId);
		// Verify
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItemsWithLinkedItems(workerWageItems,
				compensationItemList, "Doubletime"));
	}

	/**
	 * <P>
	 * Test Updating Hourly Wage set up with DoubleTime and Overtime to Hourly
	 * Wage with no Double Time and Overtime
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Hourly Wage, Overtime and Double Time . Link Overtime to
	 * Double Time to hourly</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to return the saved
	 * compensations</LI>
	 * <LI>Call WageService's saveCompensationItems command to delete the saved
	 * Overtime and Double time compensations</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of wage items returned matches what's expected
	 * </UL>
	 */
	@Test
	public void testUpdateHourlyWageItemWithNoExtraTime() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpHourlyWageWithExtraTime("Both");
		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] workerWageItems = sWageService.getCompensationItems(sWorkerId);
		// If the wage items match, get the Overtime and Double time wage
		// items,delete it

		List<CompensationItem> delCompItemList = new ArrayList<CompensationItem>();
		for (CompensationItem compItem : workerWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.Overtime)) {
				delCompItemList.add(compItem);
			}
			if (compItem.getCompensationType().equals(CompensationType.Doubletime)) {
				delCompItemList.add(compItem);
			}
		}
		// If the list has values , then call operation to delete it
		if (delCompItemList.size() > 0) {
			CompensationItem[] newWageItems = new CompensationItem[delCompItemList.size()];
			newWageItems = delCompItemList.toArray(newWageItems);
			sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
					SaveCompensationItemOperation.DELETE, newWageItems));
		} else
			AssertJUnit.fail("No Compensation Items to Delete");

		// Verify that Double Time and Overtime have been deleted
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		AssertJUnit.assertFalse("Wage Items Still Match", verifyCompensationItems(workerWageItemsNew, compensationItemList));

	}

	/**
	 * <P>
	 * Test: Updating Hourly Wage set up with DoubleTime and Overtime to Hourly
	 * Wage with no Double Time and Overtime. Updating Hourly Wage Rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Hourly Wage, Overtime and Double Time . Link Overtime to
	 * Double Time to hourly</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to return the saved
	 * compensations</LI>
	 * <LI>Call WageService's saveCompensationItems command to delete the saved
	 * Overtime and Double time compensations</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of wage items returned matches what's expected
	 * </UL>
	 */

	@Test
	public void testUpdateHourlyWageRatewithNoExtraTime() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpHourlyWageWithExtraTime("Both");
		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] workerWageItems = sWageService.getCompensationItems(sWorkerId);
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		BigDecimal bigDecimal = new BigDecimal("18.00");
		// set to 2 decimal places
		bigDecimal = bigDecimal.setScale(2);
		for (CompensationItem compItem : workerWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.Hourly)) {
				BigDecimal rate = compItem.getRate();
				if (rate.equals(bigDecimal)) {
					compItem.setRate(new BigDecimal("23.00"));
					compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE,
							compItem));
				}

				else
					AssertJUnit.fail("Rate of hourly save is not as expected");

			}
			if (compItem.getCompensationType().equals(CompensationType.Overtime)) {
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, compItem));
			}
			if (compItem.getCompensationType().equals(CompensationType.Doubletime)) {
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, compItem));

			}
		}
		// If the list has values , then call operation to delete it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Delete");

		// Verify that Double Time and Overtime have been deleted
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		if (workerWageItemsNew.length > 1)
			AssertJUnit.fail("Double Time and Overtime not deleted");
		else {
			CompensationItem compItemTest = workerWageItemsNew[0];
			if (compItemTest.getCompensationType().equals(CompensationType.Hourly)) {
				BigDecimal value = compItemTest.getRate();
				// Compare rate to 23
				BigDecimal bigDecimal23 = new BigDecimal("23.00");
				// set to 2 decimal places
				bigDecimal23 = bigDecimal23.setScale(2);
				AssertJUnit.assertTrue("Hourly Rate not updated", value.equals(bigDecimal23));
			} else {
				AssertJUnit.fail("Hourly Compensation not found");
			}
		}
	}

	/**
	 * <P>
	 * Test Updating Hourly Wage set up with Overtime to Hourly Wage with no
	 * Overtime and only DoubleTime
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Hourly Wage, Overtime . Link Overtime to hourly</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to return the saved
	 * compensations</LI>
	 * <LI>Call WageService's saveCompensationItems command to delete the saved
	 * Overtime, addDouble time compensations, update the hourly wage rate</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of wage items returned matches what's expected
	 * </UL>
	 */

	@Test
	public void testUpdateHourlyRateWithDoubleTimeFromOvertime() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpHourlyWageWithExtraTime("Overtime");
		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		CompensationItem[] workerWageItems = sWageService.getCompensationItems(sWorkerId);
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		BigDecimal bigDecimal = new BigDecimal("18.00");
		// set to 2 decimal places
		bigDecimal = bigDecimal.setScale(2);

		for (CompensationItem compItem : workerWageItems) {
			if (CompensationType.Hourly.equals(compItem.getCompensationType())) {
				BigDecimal rate = compItem.getRate();
				if (rate.equals(bigDecimal)) {
					compItem.setRate(new BigDecimal(25));
					compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE,
							compItem));
					// Add Doubletime compensation object to the list.
					CompensationItem doubleTime = new CompensationItem();
					doubleTime.setCompensationType(CompensationType.Doubletime);
					doubleTime.setName("DoubleTime");
					doubleTime.setRate(new BigDecimal("2"));
					doubleTime.setFixedRate(true);
					doubleTime.setLinkedCompensationItem(compItem);
					compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.ADD,
							doubleTime));
				} else
					AssertJUnit.fail("Rate of hourly save is not as expected");
			}
			if (compItem.getCompensationType().equals(CompensationType.Overtime)) {
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, compItem));
			}
		}
		// If the list has values , then call operation to delete it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to 	Update");

		// Verify that Overtime has been deleted
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		BigDecimal bigDecimal1 = new BigDecimal("25.00");
		// set to 2 decimal places
		bigDecimal1 = bigDecimal1.setScale(2);

		for (CompensationItem cItem : workerWageItemsNew) {
			if (CompensationType.Hourly.equals(cItem.getCompensationType())) {
				BigDecimal value = cItem.getRate();
				if (!value.equals(bigDecimal1)) {
					AssertJUnit.fail("Hourly Wage Item not updated");
				}

			} else if (CompensationType.Overtime.equals(cItem.getCompensationType())) {
				AssertJUnit.fail("Overtime Compensation not deleted");
			} else if (CompensationType.Doubletime.equals(cItem.getCompensationType())) {
				AssertJUnit.assertEquals(CompensationType.Hourly, cItem.getLinkedCompensationItem().getCompensationType());

			}
		}

	}

	/**
	 * <P>
	 * Test Updating Hourly Wage set up with Double Time to Hourly Wage with no
	 * DoubleTime and only Overtime
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Hourly Wage, DoubleTime . Link DoubleTime to hourly</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to return the saved
	 * compensations</LI>
	 * <LI>Call WageService's saveCompensationItems command to delete the saved
	 * DoubleTime, add Overtime compensations, update the hourly wage rate</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of wage items returned matches what's expected
	 * </UL>
	 */

	@Test
	public void testUpdateHourlyRateWithOverTimeFromDoubleTime() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpHourlyWageWithExtraTime("Doubletime");
		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		CompensationItem[] workerWageItems = sWageService.getCompensationItems(sWorkerId);
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		BigDecimal bigDecimal = new BigDecimal("18.00");
		// set to 2 decimal places
		bigDecimal = bigDecimal.setScale(2);

		for (CompensationItem compItem : workerWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.Hourly)) {
				BigDecimal rate = compItem.getRate();
				if (rate.equals(bigDecimal)) {
					compItem.setRate(new BigDecimal(25));
					compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE,
							compItem));
					// Add Doubletime compensation object to the list.
					CompensationItem overTime = new CompensationItem();
					overTime.setCompensationType(CompensationType.Overtime);
					overTime.setName("OverTime");
					overTime.setFixedRate(true);
					overTime.setRate(new BigDecimal("1.5"));
					overTime.setLinkedCompensationItem(compItem);
					compItemList
							.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.ADD, overTime));
				} else
					AssertJUnit.fail("Rate of hourly save is not as expected");
			}
			if (compItem.getCompensationType().equals(CompensationType.Doubletime)) {
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, compItem));
			}
		}
		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		// Verify that DoubleTime has been deleted, overtime has been added and
		// hourly rate updated
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);

		BigDecimal bigDecimal1 = new BigDecimal("25.00");
		// set to 2 decimal places
		bigDecimal1 = bigDecimal1.setScale(2);

		for (CompensationItem cItem : workerWageItemsNew) {
			if (cItem.getCompensationType().equals(CompensationType.Hourly)) {
				BigDecimal value = cItem.getRate();
				if (!value.equals(bigDecimal1)) {
					AssertJUnit.fail("Hourly Wage Item not updated");
				}

			} else if (cItem.getCompensationType().equals(CompensationType.Doubletime)) {
				AssertJUnit.fail("Doubletime Compensation not deleted");
			} else if (cItem.getCompensationType().equals(CompensationType.Overtime)) {
				AssertJUnit.assertEquals(CompensationType.Hourly, cItem.getLinkedCompensationItem().getCompensationType());

			}
		}
	}

	/**
	 * <P>
	 * Test Deleting Hourly Wage set up with Double Time and Overtime
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Hourly Wage, DoubleTime, Overtime . Link Overtime, DoubleTime
	 * to hourly</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to return the saved
	 * compensations</LI>
	 * <LI>Call WageService's saveCompensationItems command to delete the saved
	 * DoubleTime, Overtime,Hourly Items</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of wage items returned matches what's expected
	 * </UL>
	 */
	@Test
	public void testDeleteHourlyWithExtraTime() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpHourlyWageWithExtraTime("Both");
		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise

		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		// Verify
		CompensationItem[] workerWageItems = sWageService.getCompensationItems(sWorkerId);
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.DELETE, workerWageItems));
		CompensationItem[] newWageItems = sWageService.getCompensationItems(sWorkerId);

		if (newWageItems.length > 0) {
			AssertJUnit.fail("Wage Items not deleted");
		}
	}

	/**
	 * <P>
	 * Test Adding Hourly Wage with Rate equal to Zero
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Hourly Wage . Setup rate as zero</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of wage items returned matches what's expected
	 * </UL>
	 */
	@Test
	public void testAddHourlyRateAsZero() throws Exception {
		// Setup
		CompensationItem hourlyCompensationItem = new CompensationItem();
		hourlyCompensationItem.setCompensationType(CompensationType.Hourly);
		hourlyCompensationItem.setRate(new BigDecimal("0"));
		hourlyCompensationItem.setName("Hourly");
		hourlyCompensationItem.setFixedRate(true);

		List<CompensationItem> compList = new ArrayList<CompensationItem>();
		compList.add(hourlyCompensationItem);

		SaveCompensationItemRequest[] saveRequest = new SaveCompensationItemRequest[1];
		saveRequest[0] = mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.ADD,
				hourlyCompensationItem);

		// Exercise
		sWageService.saveCompensationItems(sWorkerId, saveRequest);

		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(updatedWageItems, compList));
	}

	/**
	 * <P>
	 * Test saving duplicate Hourly Wage with Same Names
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 Hourly Wage with same names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of wage items returned matches what's expected
	 * </UL>
	 */
	@Test
	public void testAddDuplicateHourlyWageItemWithSameNames() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpHourlyWageWithExtraTime("");
		CompensationItem hourlyCompensationItem = new CompensationItem();
		hourlyCompensationItem.setCompensationType(CompensationType.Hourly);
		hourlyCompensationItem.setRate(new BigDecimal("18"));
		hourlyCompensationItem.setName("Hourly");
		hourlyCompensationItem.setFixedRate(true);
		hourlyCompensationItem.setUnits(new Double(10));
		compensationItemList.add(hourlyCompensationItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Verify
		int count = 0;
		for (CompensationItem compItem : updatedWageItems) {
			if (CompensationType.Hourly.equals(compItem.getCompensationType())) {
				count++;
			}
		}
		AssertJUnit.assertEquals(2, count);
	}

	/**
	 * <P>
	 * Test saving duplicate Hourly Wage with unique names
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 Hourly Wage with same names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of wage items returned matches what's expected
	 * </UL>
	 */
	@Test
	public void testAddDuplicateHourlyWageItemWithUniqueNames() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpHourlyWageWithExtraTime("");
		CompensationItem hourlyCompensationItem = new CompensationItem();
		hourlyCompensationItem.setCompensationType(CompensationType.Hourly);
		hourlyCompensationItem.setRate(new BigDecimal("18"));
		hourlyCompensationItem.setName("Hourly 3");
		hourlyCompensationItem.setUnits(new Double(10));
		hourlyCompensationItem.setFixedRate(true);
		compensationItemList.add(hourlyCompensationItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Verify
		int count = 0;
		for (CompensationItem compItem : updatedWageItems) {
			if (CompensationType.Hourly.equals(compItem.getCompensationType())) {
				if (compItem.getName().equals("Hourly 3")) {
					count++;
				} else if (compItem.getName().equals("Hourly")) {
					count++;
				}
			}
		}
		AssertJUnit.assertEquals(2, count);
	}

	/**
	 * <P>
	 * Test updating duplicate Hourly Wage Item with same names
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 Hourly wage items with same names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update one Hourly item with different rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Verify the correct Hourly wage has been updated</LI>
	 * 
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify updated and expected compensation items are the same</LI>
	 * 
	 */
	@Test
	public void testUpdateDuplicateHourlyWageWithSameNames() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpHourlyWageWithExtraTime("");
		// create duplicate Hourly wage with same name and rate
		CompensationItem hourlyCompItem = new CompensationItem();
		hourlyCompItem.setCompensationType(CompensationType.Hourly);
		hourlyCompItem.setRate(new BigDecimal("12.43"));
		hourlyCompItem.setName("Hourly");
		hourlyCompItem.setFixedRate(true);
		compensationItemList.add(hourlyCompItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();
		Long id = null;
		// Update the rate of the first Hourly wage item returned by the Service
		updatedWageItems[0].setRate(new BigDecimal("14.01"));
		id = updatedWageItems[0].getObjectHandle().getId();
		compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE,
				updatedWageItems[0]));

		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else {
			AssertJUnit.fail("No Compensation Items to Update");
		}

		// Verify that correct Hourly has been updated
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		for (CompensationItem cItem : workerWageItemsNew) {
			if (CompensationType.Hourly.equals(cItem.getCompensationType())) {
				if (cItem.getObjectHandle().getId() == id) {
					BigDecimal value = cItem.getRate();
					if (!value.equals(new BigDecimal("14.01"))) {
						AssertJUnit.fail("Hourly Wage Item not updated");
					}
				}
			} else {
				AssertJUnit.fail("Hourly Comp Item not saved");
			}
		}
	}

	/**
	 * <P>
	 * Test updating duplicate Hourly Wage Item with unique names to Hourly wage
	 * items with same names
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 Hourly wage items with unique names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update one Hourly item to the same name as the other</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Verify the correct Hourly wage has been updated</LI>
	 * 
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify updated and expected compensation items are the same</LI>
	 * 
	 */
	@Test
	public void testUpdateDuplicateHourlyWageWithUniqueNames() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpHourlyWageWithExtraTime("");
		// create duplicate hourly wage with same name and rate
		CompensationItem hourlyCompItem = new CompensationItem();
		hourlyCompItem.setCompensationType(CompensationType.Hourly);
		hourlyCompItem.setRate(new BigDecimal("12.43"));
		hourlyCompItem.setName("Hourly 2");
		hourlyCompItem.setFixedRate(true);
		compensationItemList.add(hourlyCompItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Setup for Updates
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();
		Long id = null;
		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getName().equals("Hourly 2")) {
				id = compItem.getObjectHandle().getId();
				compItem.setName("Hourly");
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			}
		}
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else {
			AssertJUnit.fail("No Compensation Items to Update");
		}

		// Verify that correct Hourly has been updated
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		for (CompensationItem cItem : workerWageItemsNew) {
			if (CompensationType.Hourly.equals(cItem.getCompensationType())) {
				if (cItem.getObjectHandle().getId() == id) {
					String name = cItem.getName();
					if (!name.equals("Hourly")) {
						AssertJUnit.fail("Hourly Wage Item not updated");
					}
				}
			} else {
				AssertJUnit.fail("Hourly Comp Item not saved");
			}
		}
	}

	/**
	 * <P>
	 * Test deleting Hourly Wage Item which has the same name as another Hourly
	 * wage item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 Hourly wage items with same names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Delete one Hourly wage item</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Verify the correct Hourly wage has been delete/LI>
	 * 
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify that expected Hourly wage has been deleted</LI>
	 * 
	 */
	@Test
	public void testRemoveDuplicateHourlyWageWithSameNames() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpHourlyWageWithExtraTime("");
		// create duplicate Hourly wage with same name and rate
		CompensationItem hourlyCompItem = new CompensationItem();
		hourlyCompItem.setCompensationType(CompensationType.Hourly);
		hourlyCompItem.setRate(new BigDecimal("1200.43"));
		hourlyCompItem.setName("Hourly");
		hourlyCompItem.setFixedRate(true);
		compensationItemList.add(hourlyCompItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();
		Long id = null;
		// Delete first Hourly wage item returned by the Service
		id = updatedWageItems[0].getObjectHandle().getId();
		compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE,
				updatedWageItems[0]));

		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else {
			AssertJUnit.fail("No Compensation Items to Update");
		}

		// Verify that correct Hourly has been deleted
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		for (CompensationItem cItem : workerWageItemsNew) {
			if (CompensationType.Hourly.equals(cItem.getCompensationType())) {
				if (cItem.getObjectHandle().getId() == id) {
					AssertJUnit.fail("Hourly Wage Item not deleted");
				}
			} else {
				AssertJUnit.fail("Hourly Comp Item not saved");
			}
		}
	}
}
