package com.intuit.sb.payroll.qa.services.employer;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.common.SaveWorkLocationRequest;
import com.intuit.sb.payroll.services.common.WorkLocation;
import com.intuit.sb.payroll.services.common.WorkLocationOperation;
import com.intuit.sb.payroll.services.employer.AssistedStateType;
import com.intuit.sb.payroll.services.employer.EmployerInfo;
import com.intuit.sb.payroll.services.employer.EmployerServiceTestBase;
import com.intuit.sb.payroll.services.employer.SaveEmployerInfoRequest;
import com.intuit.sb.payroll.services.employer.SaveEmployerInfoResponse;

public class SaveEmployerInfoTest extends EmployerServiceTestBase {

	@Test
	public void testSaveEmployerInfoWithNullWorkLocation() throws ServiceFailureException, ValidationException {
		int existingWorkLocationCount = sEmployerService.getWorkLocations().length;
		
		EmployerInfo employerInfo = createBasicEmployerInfo();
		employerInfo.setCompanyName("Bogus");
    	employerInfo.setAssistedStateType(null); // sending in null is returned is NotAssisted

		SaveEmployerInfoRequest employerInfoRequest = new SaveEmployerInfoRequest();
		employerInfoRequest.setEmployerInfo(employerInfo);

		SaveEmployerInfoResponse employerInfoResponse = sEmployerService.saveEmployerInfo(employerInfoRequest);
    	employerInfo.setAssistedStateType(AssistedStateType.NOT_ASSISTED); // check null mapped to NotAssisted
		verifyEmployerInfo(employerInfo, employerInfoResponse.getEmployerInfo());
    	AssertJUnit.assertEquals("work location count", existingWorkLocationCount, employerInfoResponse.getWorkLocations().length);
	}

	@Test
	public void testSaveEmployerInfoWithEmptyWorkLocations() throws ServiceFailureException, ValidationException {
		int existingWorkLocationCount = sEmployerService.getWorkLocations().length;

		EmployerInfo employerInfo = createBasicEmployerInfo();
		employerInfo.setCompanyName("Bogus");

		SaveEmployerInfoRequest employerInfoRequest = new SaveEmployerInfoRequest();
		employerInfoRequest.setEmployerInfo(employerInfo);
		employerInfoRequest.setSaveWorkLocationRequests(new SaveWorkLocationRequest[0]);
		
		SaveEmployerInfoResponse employerInfoResponse = sEmployerService.saveEmployerInfo(employerInfoRequest);
		verifyEmployerInfo(employerInfo, employerInfoResponse.getEmployerInfo());
    	AssertJUnit.assertEquals("work location count", existingWorkLocationCount, employerInfoResponse.getWorkLocations().length);
	}

	@Test
	public void testSaveEmployerInfoWithWorkLocation() throws ServiceFailureException, ValidationException {
		int existingWorkLocationCount = sEmployerService.getWorkLocations().length;
		
		EmployerInfo employerInfo = createBasicEmployerInfo();
		employerInfo.setCompanyName("Bogus");

		SaveEmployerInfoRequest employerInfoRequest = new SaveEmployerInfoRequest();
		employerInfoRequest.setEmployerInfo(employerInfo);
		
		SaveWorkLocationRequest[] workLocationRequests = new SaveWorkLocationRequest[1];
		WorkLocation[] expectedWorkLocations = new WorkLocation[1];
		
		SaveWorkLocationRequest workLocationRequest = new SaveWorkLocationRequest();
		workLocationRequest.setWorkLocationOperation(WorkLocationOperation.ADD);
		WorkLocation workLocation = new WorkLocation();
		workLocation.setName("Main Office");
		workLocation.setObjectHandle(null);
		workLocation.setIsDefault(true);
		workLocation.setAddress(createPhysicalAddress());
		workLocation.setPhoneNumber("111-111-1111");
		workLocation.setExtn("x1111");
		workLocationRequest.setWorkLocation(workLocation);
		expectedWorkLocations[0] = workLocation;
		workLocationRequests[0] = workLocationRequest;
		
		employerInfoRequest.setSaveWorkLocationRequests(workLocationRequests);

		SaveEmployerInfoResponse employerInfoResponse = sEmployerService.saveEmployerInfo(employerInfoRequest);
		verifyEmployerInfo(employerInfo, employerInfoResponse.getEmployerInfo());
    	AssertJUnit.assertEquals("work location count", existingWorkLocationCount + 1, employerInfoResponse.getWorkLocations().length);
    	verifyWorkLocations(expectedWorkLocations, employerInfoResponse.getWorkLocations());
	}

	@Test
	public void testSaveEmployerInfoWithTwoWorkLocations() throws ServiceFailureException, ValidationException {
		int existingWorkLocationCount = sEmployerService.getWorkLocations().length;

		EmployerInfo employerInfo = createBasicEmployerInfo();
		employerInfo.setCompanyName("Bogus");

		SaveEmployerInfoRequest employerInfoRequest = new SaveEmployerInfoRequest();
		employerInfoRequest.setEmployerInfo(employerInfo);
		
		SaveWorkLocationRequest[] workLocationRequests = new SaveWorkLocationRequest[2];
		WorkLocation[] expectedWorkLocations = new WorkLocation[2];
		SaveWorkLocationRequest workLocationRequest = new SaveWorkLocationRequest();
		
		workLocationRequest.setWorkLocationOperation(WorkLocationOperation.ADD);
		WorkLocation workLocation = new WorkLocation();
		workLocation.setName("Main Office");
		workLocation.setObjectHandle(null);
		workLocation.setIsDefault(true);
		workLocation.setAddress(createPhysicalAddress());
		workLocation.setPhoneNumber("111-111-1111");
		workLocation.setExtn("x1111");
		workLocationRequest.setWorkLocation(workLocation);
		expectedWorkLocations[0] = workLocation;
		workLocationRequests[0] = workLocationRequest;
		
		workLocationRequest = new SaveWorkLocationRequest();
		workLocationRequest.setWorkLocationOperation(WorkLocationOperation.ADD);
		workLocation = new WorkLocation();
		workLocation.setName("Annex");
		workLocation.setObjectHandle(null);
		workLocation.setIsDefault(false);
		workLocation.setAddress(createPhysicalAddress());
		workLocation.setPhoneNumber("111-111-2222");
		workLocation.setExtn("x2222");
		workLocationRequest.setWorkLocation(workLocation);
		expectedWorkLocations[1] = workLocation;
		workLocationRequests[1] = workLocationRequest;

		employerInfoRequest.setSaveWorkLocationRequests(workLocationRequests);

		SaveEmployerInfoResponse employerInfoResponse = sEmployerService.saveEmployerInfo(employerInfoRequest);
		verifyEmployerInfo(employerInfo, employerInfoResponse.getEmployerInfo());
    	AssertJUnit.assertEquals("work location count", existingWorkLocationCount + 2, employerInfoResponse.getWorkLocations().length);
    	verifyWorkLocations(expectedWorkLocations, employerInfoResponse.getWorkLocations());
	}

	@Test
	public void testUpdateEmployerInfoWithWorkLocation() throws ServiceFailureException, ValidationException {
		int existingWorkLocationCount = sEmployerService.getWorkLocations().length;

		EmployerInfo employerInfo = createBasicEmployerInfo();
		employerInfo.setCompanyName("Bogus");

		SaveEmployerInfoRequest employerInfoRequest = new SaveEmployerInfoRequest();
		employerInfoRequest.setEmployerInfo(employerInfo);
		
		SaveWorkLocationRequest[] workLocationRequests = new SaveWorkLocationRequest[1];
		WorkLocation[] expectedWorkLocations = new WorkLocation[1];
		
		SaveWorkLocationRequest workLocationRequest = new SaveWorkLocationRequest();
		workLocationRequest.setWorkLocationOperation(WorkLocationOperation.ADD);
		WorkLocation workLocation = new WorkLocation();
		workLocation.setName("Main Office");
		workLocation.setObjectHandle(null);
		workLocation.setIsDefault(true);
		workLocation.setAddress(createPhysicalAddress());
		workLocation.setPhoneNumber("111-111-1111");
		workLocation.setExtn("x1111");
		workLocationRequest.setWorkLocation(workLocation);
		expectedWorkLocations[0] = workLocation;
		workLocationRequests[0] = workLocationRequest;
		
		employerInfoRequest.setSaveWorkLocationRequests(workLocationRequests);

		SaveEmployerInfoResponse employerInfoResponse = sEmployerService.saveEmployerInfo(employerInfoRequest);
		verifyEmployerInfo(employerInfo, employerInfoResponse.getEmployerInfo());
    	AssertJUnit.assertEquals("work location count", existingWorkLocationCount + 1, employerInfoResponse.getWorkLocations().length);
    	verifyWorkLocations(expectedWorkLocations, employerInfoResponse.getWorkLocations());
    	
		employerInfoRequest = new SaveEmployerInfoRequest();
    	employerInfo = employerInfoResponse.getEmployerInfo();
    	employerInfoRequest.setEmployerInfo(employerInfo);
		employerInfo.setCompanyName("Straight Up Guys");
    	
		workLocationRequest = new SaveWorkLocationRequest();
		workLocationRequest.setWorkLocationOperation(WorkLocationOperation.UPDATE);
		for (WorkLocation returnedWorkLocation : employerInfoResponse.getWorkLocations()) {
			if (returnedWorkLocation.getExtn() != null && returnedWorkLocation.getExtn().equals(workLocation.getExtn()) &&
				returnedWorkLocation.getName() != null && returnedWorkLocation.getName().equals(workLocation.getName()) &&
				returnedWorkLocation.getObjectHandle() != null && returnedWorkLocation.getObjectHandle().getVersion() == 0) {
				
				workLocation = returnedWorkLocation;
				break;
			}
		}
		workLocation = employerInfoResponse.getWorkLocations()[0];
    	workLocation.setName("New Digs");
		workLocationRequest.setWorkLocation(workLocation);
		expectedWorkLocations[0] = workLocation;
		workLocationRequests[0] = workLocationRequest;

		employerInfoRequest.setSaveWorkLocationRequests(workLocationRequests);

		employerInfoResponse = sEmployerService.saveEmployerInfo(employerInfoRequest);
		verifyEmployerInfo(employerInfo, employerInfoResponse.getEmployerInfo());
    	AssertJUnit.assertEquals("work location count", existingWorkLocationCount + 1, employerInfoResponse.getWorkLocations().length);
    	verifyWorkLocations(expectedWorkLocations, employerInfoResponse.getWorkLocations());
	}

	@Test
	public void testRemoveWorkLocationFromEmployerInfo() throws ServiceFailureException, ValidationException {
		int existingWorkLocationCount = sEmployerService.getWorkLocations().length;

		EmployerInfo employerInfo = createBasicEmployerInfo();
		employerInfo.setCompanyName("Bogus");

		SaveEmployerInfoRequest employerInfoRequest = new SaveEmployerInfoRequest();
		employerInfoRequest.setEmployerInfo(employerInfo);
		
		SaveWorkLocationRequest[] workLocationRequests = new SaveWorkLocationRequest[1];
		WorkLocation[] expectedWorkLocations = new WorkLocation[1];
		
		SaveWorkLocationRequest workLocationRequest = new SaveWorkLocationRequest();
		workLocationRequest.setWorkLocationOperation(WorkLocationOperation.ADD);
		WorkLocation workLocation = new WorkLocation();
		workLocation.setName("Main Office");
		workLocation.setObjectHandle(null);
		workLocation.setIsDefault(true);
		workLocation.setAddress(createPhysicalAddress());
		workLocation.setPhoneNumber("111-111-1111");
		workLocation.setExtn("x1111");
		workLocationRequest.setWorkLocation(workLocation);
		expectedWorkLocations[0] = workLocation;
		workLocationRequests[0] = workLocationRequest;
		
		employerInfoRequest.setSaveWorkLocationRequests(workLocationRequests);

		SaveEmployerInfoResponse employerInfoResponse = sEmployerService.saveEmployerInfo(employerInfoRequest);
		verifyEmployerInfo(employerInfo, employerInfoResponse.getEmployerInfo());
    	AssertJUnit.assertEquals("work location count", existingWorkLocationCount + 1, employerInfoResponse.getWorkLocations().length);
    	verifyWorkLocations(expectedWorkLocations, employerInfoResponse.getWorkLocations());
    	
		employerInfoRequest = new SaveEmployerInfoRequest();
    	employerInfo = employerInfoResponse.getEmployerInfo();
    	employerInfoRequest.setEmployerInfo(employerInfo);
		employerInfo.setCompanyName("Straight Up Guys");
    	
		workLocationRequest = new SaveWorkLocationRequest();
		workLocationRequest.setWorkLocationOperation(WorkLocationOperation.DELETE);
		for (WorkLocation returnedWorkLocation : employerInfoResponse.getWorkLocations()) {
			if (returnedWorkLocation.getExtn() != null && returnedWorkLocation.getExtn().equals(workLocation.getExtn()) &&
				returnedWorkLocation.getName() != null && returnedWorkLocation.getName().equals(workLocation.getName()) &&
				returnedWorkLocation.getObjectHandle() != null && returnedWorkLocation.getObjectHandle().getVersion() == 0) {
				
				workLocation = returnedWorkLocation;
				break;
			}
		}
		workLocationRequest.setWorkLocation(workLocation);
		workLocationRequests[0] = workLocationRequest;

		employerInfoRequest.setSaveWorkLocationRequests(workLocationRequests);

		employerInfoResponse = sEmployerService.saveEmployerInfo(employerInfoRequest);
		verifyEmployerInfo(employerInfo, employerInfoResponse.getEmployerInfo());
    	AssertJUnit.assertEquals("work location count", existingWorkLocationCount, employerInfoResponse.getWorkLocations().length);
	}

}
