package com.intuit.sb.payroll.qa.services.employer;

import org.testng.AssertJUnit;

import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.common.PhysicalAddress;
import com.intuit.sb.payroll.services.common.WorkLocation;
import com.intuit.sb.payroll.services.employer.EmployerServiceTestBase;

/**
 * Tests the updateWorkLocations operation of the EmployerService.
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class UpdateWorkLocationsTest extends EmployerServiceTestBase {
    /**
     * <P>Test calling updateWorkLocations with an empty array</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call EmployerService's updateWorkLocations() with an empty array</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testUpdateWorkLocationsWithEmptyArray() throws Exception {
        @SuppressWarnings("unused")
        WorkLocation[] workLocations = sEmployerService.updateWorkLocations(new WorkLocation[0]).getWorkLocations();
    }

    /**
     * <P>Test calling updateWorkLocations with a null WorkLocation container</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call EmployerService's updateWorkLocations() with a null WorkLocation container</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testUpdateWorkLocationsWithNullContainer() throws Exception {
        WorkLocation[] newWorkLocation = new WorkLocation[1];
        // Purposely set it to null
        newWorkLocation[0] = null;
        @SuppressWarnings("unused")
        WorkLocation[] workLocations = sEmployerService.updateWorkLocations(newWorkLocation).getWorkLocations();
    }

    /**
     * <P>Test calling updateWorkLocations with null</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call EmployerService's updateWorkLocations() with null</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testUpdateWorkLocationsWithNull() throws Exception {
        @SuppressWarnings("unused")
        WorkLocation[] workLocations = sEmployerService.updateWorkLocations(null).getWorkLocations();
    }

    /**
     * <P>Test calling updateWorkLocations with a non-existent workLocation id</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call EmployerService's updateWorkLocations() with a non-existent workLocation id</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ServiceFailureException is thrown
     * </UL>
     */
    @SuppressWarnings("deprecation")
	@Test (expectedExceptions=ServiceFailureException.class)
    public void testUpdateWorkLocationsWithNonExistentId() throws Exception {
        Long origUniqueId = null;
        WorkLocation[] existingLocations = sEmployerService.getWorkLocations();
        origUniqueId = existingLocations[0].getObjectHandle().getId();
        try {
            WorkLocation[] newWorkLocation = new WorkLocation[1];
            newWorkLocation[0] = existingLocations[0];
            // Now purposely set the uniqueId to a non existent value
            newWorkLocation[0].setObjectHandle(new ObjectHandle(new Long("83491176"), new Long("0")));
            @SuppressWarnings("unused")
            WorkLocation[] workLocations = sEmployerService.updateWorkLocations(newWorkLocation).getWorkLocations();
        } finally {
            // Restore the original uniqueId value
            existingLocations[0].getObjectHandle().setId(origUniqueId);
        }
    }

    /**
     * <P>Test calling updateWorkLocations with a null WorkLocation name</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call EmployerService's updateWorkLocations() with a null WorkLocation name</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testUpdateWorkLocationsWithNullName() throws Exception {
        String origName = null;
        WorkLocation[] existingLocations = sEmployerService.getWorkLocations();
        origName = existingLocations[0].getName();
        try {
            WorkLocation[] newWorkLocation = new WorkLocation[1];
            newWorkLocation[0] = existingLocations[0];
            // Purposely set it to null
            newWorkLocation[0].setName(null);
            @SuppressWarnings("unused")
            WorkLocation[] workLocations = sEmployerService.updateWorkLocations(newWorkLocation).getWorkLocations();
        } finally {
            // Restore the original name value
            existingLocations[0].setName(origName);
        }
    }

    /**
     * <P>Test calling updateWorkLocations with a null address</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call EmployerService's updateWorkLocations() with a null address</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testUpdateWorkLocationsWithNullAddress() throws Exception {
        PhysicalAddress origAddress = null;
        WorkLocation[] existingLocations = sEmployerService.getWorkLocations();
        origAddress = existingLocations[0].getAddress();
        try {
            WorkLocation[] newWorkLocation = new WorkLocation[1];
            newWorkLocation[0] = existingLocations[0];
            // Purposely set it to null
            newWorkLocation[0].setAddress(null);
            @SuppressWarnings("unused")
            WorkLocation[] workLocations = sEmployerService.updateWorkLocations(newWorkLocation).getWorkLocations();
        } finally {
            // Restore the original address value
            existingLocations[0].setAddress(origAddress);
        }
    }

    /**
     * <P>Test calling updateWorkLocations with a null state value</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call EmployerService's updateWorkLocations() with a null state value</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testUpdateWorkLocationsWithNullState() throws Exception {
        String origState = null;
        WorkLocation[] existingLocations = sEmployerService.getWorkLocations();
        origState = existingLocations[0].getAddress().getStateOrRegion();
        try {
            WorkLocation[] newWorkLocation = new WorkLocation[1];
            newWorkLocation[0] = existingLocations[0];
            // Purposely set it to null
            newWorkLocation[0].getAddress().setStateOrRegion(null);
            @SuppressWarnings("unused")
            WorkLocation[] workLocations = sEmployerService.updateWorkLocations(newWorkLocation).getWorkLocations();
        } finally {
            // Restore the original state value
            existingLocations[0].getAddress().setStateOrRegion(origState);
        }
    }

    /**
     * <P>Test updating a workLocation</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerService's updateWorkLocations() to update a workLocation</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that workLocation is updated successfully
     * </UL>
     */
    @Test
    public void testUpdateOneWorkLocation() throws Exception {
        WorkLocation workLocation = null;
        WorkLocation updatedWorkLocation = null;
        WorkLocation[] updatedWorkLocations = null;
        String origName = null;
        String origPhone = null;
        boolean workLocationUpdated = false;
        try {
            WorkLocation[] existingLocations = sEmployerService.getWorkLocations();
            WorkLocation[] updateWorkLocations = new WorkLocation[1];
            workLocation = existingLocations[0];
            // Remember the original values
            origName = workLocation.getName();
            origPhone = workLocation.getPhoneNumber();
            workLocation.setName("Updated Location");
            workLocation.setPhoneNumber("123-456-7890");
            updateWorkLocations[0] = workLocation;
            updatedWorkLocations = sEmployerService.updateWorkLocations(updateWorkLocations).getWorkLocations();
            workLocationUpdated = true;
            // Locate the workLocation that was updated
            for (WorkLocation aWorkLocation : updatedWorkLocations) {
                if (aWorkLocation.getName().equals(workLocation.getName())) {
                    updatedWorkLocation = aWorkLocation;
                    break;
                }
            }
            // Verify
            AssertJUnit.assertTrue("WorkLocation info mismatch", verifyWorkLocation(updateWorkLocations[0], updatedWorkLocation));
        } finally {
            if (workLocationUpdated) {
                // Restore the original value
                workLocation.setName(origName);
                workLocation.setPhoneNumber(origPhone);
            }
        }
    }

    /**
     * <P>Test updating 2 workLocations</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerService's updateWorkLocations() to updating 2 workLocations</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that workLocation are updated successfully
     * </UL>
     */
    @SuppressWarnings("deprecation")
	@Test
    public void testUpdateTwoWorkLocations() throws Exception {
        final int numOfWorkLocations = 2;
        WorkLocation[] updatedWorkLocations = null;
        WorkLocation[] workLocationsUpdated = null;
        WorkLocation origWorkLocation1 = null;
        WorkLocation origWorkLocation2 = null;
        WorkLocation workLocation1 = null;
        WorkLocation workLocation2 = null;
        boolean workLocationUpdated = false;
        try {
            WorkLocation[] existingLocations = sEmployerService.getWorkLocations();
            // Remember the original values
            origWorkLocation1 = existingLocations[0];
            origWorkLocation2 = existingLocations[1];
            // Setup new information
            workLocation1 = createWorkLocation("Updated Location 1", "650-944-5000");
            // Maintain the original uniqueId & workCount values
            workLocation1.setObjectHandle(origWorkLocation1.getObjectHandle());
            workLocation1.setWorkerCount(origWorkLocation1.getWorkerCount());
            workLocation2 = createWorkLocation("Updated Location 2", "650-944-5001");
            // Maintain the original uniqueId & workCount values
            workLocation2.setObjectHandle(origWorkLocation2.getObjectHandle());
            workLocation2.setWorkerCount(origWorkLocation2.getWorkerCount());
            WorkLocation[] newWorkLocations = new WorkLocation[numOfWorkLocations];
            newWorkLocations[0] = workLocation1;
            newWorkLocations[1] = workLocation2;
            updatedWorkLocations = sEmployerService.updateWorkLocations(newWorkLocations).getWorkLocations();
            workLocationUpdated = true;
            workLocationsUpdated = new WorkLocation[numOfWorkLocations];
            int i = 0;
            // Locate the workLocations that were added
            for (WorkLocation aWorkLocation : updatedWorkLocations) {
                if (aWorkLocation.getName().equals(workLocation1.getName()) ||
                    aWorkLocation.getName().equals(workLocation2.getName())) {
                    workLocationsUpdated[i++] = aWorkLocation;
                    if (i >= numOfWorkLocations) {
                        break;
                    }
                }
            }
            // Verify
            AssertJUnit.assertTrue("WorkLocation info mismatch", verifyWorkLocations(newWorkLocations, workLocationsUpdated));
        } finally {
            if (workLocationUpdated) {
                // Fix up the version value
                if (workLocation1.getName().equals(workLocationsUpdated[0].getName())) {
                    origWorkLocation1.getObjectHandle().setVersion(workLocationsUpdated[0].getObjectHandle().getVersion());
                    origWorkLocation2.getObjectHandle().setVersion(workLocationsUpdated[1].getObjectHandle().getVersion());
                } else {
                    origWorkLocation1.getObjectHandle().setVersion(workLocationsUpdated[1].getObjectHandle().getVersion());
                    origWorkLocation2.getObjectHandle().setVersion(workLocationsUpdated[0].getObjectHandle().getVersion());
                }
                // Restore the original value
                WorkLocation[] newWorkLocations = new WorkLocation[numOfWorkLocations];
                newWorkLocations[0] = origWorkLocation1;
                newWorkLocations[1] = origWorkLocation2;
                updatedWorkLocations = sEmployerService.updateWorkLocations(newWorkLocations).getWorkLocations();
            }
        }
    }

}
