package com.intuit.sb.payroll.qa.services.taxes;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.common.EntityUtilsTestBase;
import com.intuit.sb.payroll.services.common.PojoUtils;
import com.intuit.sb.payroll.services.taxes.SaveWithholdingInfoRequest;
import com.intuit.sb.payroll.services.taxes.WithholdingInfoOperation;
import com.intuit.sb.payroll.services.taxes.WithholdingInformation;
import com.intuit.sb.payroll.services.taxes.WithholdingStatus;
import com.intuit.sb.payroll.services.worker.Worker;

/**
 * <P>
 * The <CODE>SaveWithholdingInformationExceptionsTest</CODE> class contains tests for testing the Exceptions that can be
 * thrown by saveWithholdingInformation() in TaxWithholdingsService
 * </P>
 * 
 * $Author$
 * $Revision$
 * $DateTime$
 * 
 */
public class SaveWithholdingInformationExceptionsTest extends EntityUtilsTestBase {

	private static Long sUniqueId = null;

	@BeforeClass
	public static void setUpSaveWithholdingInformationExceptionsTest() {
		try {
			getUserTransaction().begin();
			// Save a new Worker to be used by all tests
			Worker savedWorker = null;
			Worker worker = createDtoWorker();
			worker.setTaxResidenceAddress(PojoUtils.createUSCAPhysicalAddress());
			worker.setWorkLocation(getNamedWorkLocation(sMountainViewLocation));
		    savedWorker = sWorkerService.saveWorker(worker);
			sUniqueId = savedWorker.getObjectHandle().getId();
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in setUpSaveWithholdingInformationExceptionsTest(): " + t.getMessage());
		}
	}

	@AfterClass
	public static void tearDownSaveWithholdingInformationExceptionsTest() {
		try {
			getUserTransaction().begin();
			cleanUpEIWorker(sUniqueId);
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in tearDownSaveWithholdingInformationExceptionsTest(): " + t.getMessage());
		}
	}

	/**
	 * <P>Test calling saveWithholdingInformation() with a null SaveWithholdingInfoRequest</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsService.saveWithholdingInformation() with a null SaveWithholdingInfoRequest</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown</LI>
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullSaveWithholdingInfoRequest() throws Exception {
    	// Exercise
		sTaxWithholdingsService.saveWithholdingInformation(null);
	}

	/**
	 * <P>Test calling saveWithholdingInformation() with a null worker id</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsService.saveWithholdingInformation() with a null worker id</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown</LI>
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullWorkerId() throws Exception {
		// Setup
		SaveWithholdingInfoRequest saveRequest = new SaveWithholdingInfoRequest();
		saveRequest.setWorkerId(null);
		saveRequest.setWagePlanCode("S");
		saveRequest.setWorkerTaxIdNumber("123456789");
		saveRequest.setWithholdingInfoOperation(WithholdingInfoOperation.NOT_SPECIFIED);
		saveRequest.setWithholdingInfo(createAWithholdingInformation());
    	// Exercise
		sTaxWithholdingsService.saveWithholdingInformation(saveRequest);
	}

	/**
	 * <P>Test calling saveWithholdingInformation() with a non-existent worker id</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsService.saveWithholdingInformation() with a non-existent worker id</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ServiceFailureException is thrown</LI>
	 * </UL>
	 */
	@Test(expectedExceptions = ServiceFailureException.class)
	public void testNonExistentWorkerId() throws Exception {
		// Setup
		SaveWithholdingInfoRequest saveRequest = new SaveWithholdingInfoRequest();
		saveRequest.setWorkerId(87654321L);
		saveRequest.setWagePlanCode("S");
		saveRequest.setWorkerTaxIdNumber("123456789");
		saveRequest.setWithholdingInfoOperation(WithholdingInfoOperation.NOT_SPECIFIED);
		saveRequest.setWithholdingInfo(createAWithholdingInformation());
    	// Exercise
		sTaxWithholdingsService.saveWithholdingInformation(saveRequest);
	}

	/**
	 * <P>Test calling saveWithholdingInformation() with a null WithholdingInfoOperation</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsService.saveWithholdingInformation() with a null WithholdingInfoOperation</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown</LI>
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullWithholdingInfoOperation() throws Exception {
		// Setup
		SaveWithholdingInfoRequest saveRequest = new SaveWithholdingInfoRequest();
		saveRequest.setWorkerId(sUniqueId);
		saveRequest.setWagePlanCode("S");
		saveRequest.setWorkerTaxIdNumber("123456789");
		saveRequest.setWithholdingInfoOperation(null);
		saveRequest.setWithholdingInfo(createAWithholdingInformation());
    	// Exercise
		sTaxWithholdingsService.saveWithholdingInformation(saveRequest);
	}

	/**
	 * <P>Test calling saveWithholdingInformation() with a null WithholdingInformation for an ADD operation</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsService.saveWithholdingInformation() with a null WithholdingInformation for an ADD
	 *  operation</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown</LI>
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullWithholdingInformationForAdd() throws Exception {
		// Setup
		SaveWithholdingInfoRequest saveRequest = new SaveWithholdingInfoRequest();
		saveRequest.setWorkerId(sUniqueId);
		saveRequest.setWagePlanCode("S");
		saveRequest.setWorkerTaxIdNumber("123456789");
		saveRequest.setWithholdingInfoOperation(WithholdingInfoOperation.ADD);
		saveRequest.setWithholdingInfo(null);
    	// Exercise
		sTaxWithholdingsService.saveWithholdingInformation(saveRequest);
	}

	/**
	 * <P>Test calling saveWithholdingInformation() with a null WithholdingInformation for an UPDATE operation</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsService.saveWithholdingInformation() with a null WithholdingInformation for an UPDATE
	 *  operation</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown</LI>
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullWithholdingInformationForUpdate() throws Exception {
		// Setup
		SaveWithholdingInfoRequest saveRequest = new SaveWithholdingInfoRequest();
		saveRequest.setWorkerId(sUniqueId);
		saveRequest.setWagePlanCode("S");
		saveRequest.setWorkerTaxIdNumber("123456789");
		saveRequest.setWithholdingInfoOperation(WithholdingInfoOperation.UPDATE);
		saveRequest.setWithholdingInfo(null);
    	// Exercise
		sTaxWithholdingsService.saveWithholdingInformation(saveRequest);
	}

	/**
	 * <P>Test calling saveWithholdingInformation() with a null WithholdingInformation for an DELETE operation</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsService.saveWithholdingInformation() with a null WithholdingInformation for an DELETE
	 *  operation</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown</LI>
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullWithholdingInformationForDelete() throws Exception {
		// Setup
		SaveWithholdingInfoRequest saveRequest = new SaveWithholdingInfoRequest();
		saveRequest.setWorkerId(sUniqueId);
		saveRequest.setWagePlanCode("S");
		saveRequest.setWorkerTaxIdNumber("123456789");
		saveRequest.setWithholdingInfoOperation(WithholdingInfoOperation.DELETE);
		saveRequest.setWithholdingInfo(null);
    	// Exercise
		sTaxWithholdingsService.saveWithholdingInformation(saveRequest);
	}

	/**
	 * <P>Test calling saveWithholdingInformation() to update a DE4 with an invalid CA Wage plan code</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a DE4 WithholdingInformation to the worker</LI>
	 * <LI>Set up the SaveWithholdingInfoRequest object</LI>
	 * <LI>Call TaxWithholdingsService.saveWithholdingInformation() to update a DE4 and add an invalid CA Wage plan code</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown</LI>
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testUpdateDe4_AddInvalidWagePlanCode() throws Exception {
		// Setup
		addDe4WithholdingInfo(sUniqueId);
		WithholdingInformation withholdingInfo = getDe4WithholdingInfo(sUniqueId);
		withholdingInfo.setAllowances(2);
		SaveWithholdingInfoRequest saveRequest = new SaveWithholdingInfoRequest();
		saveRequest.setWorkerId(sUniqueId);
		saveRequest.setWorkerTaxIdNumber(null);
		saveRequest.setWagePlanCode("Z");
		saveRequest.setWithholdingInfo(withholdingInfo);
		saveRequest.setWithholdingInfoOperation(WithholdingInfoOperation.UPDATE);
    	// Exercise
		sTaxWithholdingsService.saveWithholdingInformation(saveRequest);
	}
	
	private WithholdingInformation createAWithholdingInformation() {
		WithholdingInformation withholdingInformation = new WithholdingInformation();
		withholdingInformation.setAllowances(1);
		withholdingInformation.setDeductions(0);
		withholdingInformation.setExempt(false);
		withholdingInformation.setExtraWithholding(null);
		withholdingInformation.setFormType(sUS_W4);
		withholdingInformation.setWithholdingStatus(WithholdingStatus.SINGLE);
		return withholdingInformation;
	}
	
}