package com.intuit.sb.payroll.qa.services.taxes;

import java.math.BigDecimal;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.services.common.EntityUtilsTestBase;
import com.intuit.sb.payroll.services.common.PojoUtils;
import com.intuit.sb.payroll.services.taxes.FormType;
import com.intuit.sb.payroll.services.taxes.SaveWithholdingInfoRequest;
import com.intuit.sb.payroll.services.taxes.SaveWithholdingResponse;
import com.intuit.sb.payroll.services.taxes.WithholdingInfoOperation;
import com.intuit.sb.payroll.services.taxes.WithholdingInformation;
import com.intuit.sb.payroll.services.worker.Worker;

/**
 * <P>
 * The <CODE>SaveWithholdingInformationTest</CODE> class contains tests for testing saveWithholdingInformation() in
 *  TaxWithholdingsService
 * </P>
 * 
 * $Author$
 * $Revision$
 * $DateTime$
 * 
 */
public class SaveWithholdingInformationTest extends EntityUtilsTestBase {

	private static Long sUniqueId = null;

	@BeforeClass
	public static void setUpSaveWithholdingInformationTest() {
		try {
			getUserTransaction().begin();
			// Save a new Worker to be used by all tests
			Worker savedWorker = null;
			Worker worker = createDtoWorker();
			worker.setTaxResidenceAddress(PojoUtils.createUSCAPhysicalAddress());
			worker.setWorkLocation(getNamedWorkLocation(sMountainViewLocation));
		    savedWorker = sWorkerService.saveWorker(worker);
			sUniqueId = savedWorker.getObjectHandle().getId();
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in setUpSaveWithholdingInformationExceptionsTest(): " + t.getMessage());
		}
	}

	@AfterClass
	public static void tearDownSaveWithholdingInformationTest() {
		try {
			getUserTransaction().begin();
			cleanUpEIWorker(sUniqueId);
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in tearDownSaveWithholdingInformationExceptionsTest(): " + t.getMessage());
		}
	}

	/**
	 * <P>Test calling saveWithholdingInformation() to add an exempt W4</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the SaveWithholdingInfoRequest object</LI>
	 * <LI>Call TaxWithholdingsService.saveWithholdingInformation() to add an exempt W4</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the exempt W4 information is saved on the Worker</LI>
	 * </UL>
	 */
	@Test
	public void testAddExemptW4Only() throws Exception {
		// Setup
		WithholdingInformation withholdingInfo = newWithholdingInformation(FormType.US_W4);
		withholdingInfo.setExempt(true);
		SaveWithholdingInfoRequest saveRequest = new SaveWithholdingInfoRequest();
		saveRequest.setWorkerId(sUniqueId);
		saveRequest.setWorkerTaxIdNumber(null);
		saveRequest.setWagePlanCode(null);
		saveRequest.setWithholdingInfo(withholdingInfo);
		saveRequest.setWithholdingInfoOperation(WithholdingInfoOperation.ADD);
    	// Exercise
		SaveWithholdingResponse response = sTaxWithholdingsService.saveWithholdingInformation(saveRequest);
		// Verify
		AssertJUnit.assertNull("WorkerTaxIdNumber not null!", response.getWorker().getWorkerTaxIdNumber());
		WithholdingInformation actual = response.getWithholding();
		// Fix the objectHandle
		withholdingInfo.setObjectHandle(actual.getObjectHandle());
		AssertJUnit.assertEquals("WithholdingInformation mismatch!", withholdingInfo, actual);
		// Verify that the W4 is saved on the worker
		actual = getW4WithholdingInfo(sUniqueId);
		AssertJUnit.assertEquals("WithholdingInformation mismatch!", withholdingInfo, actual);
	}

	/**
	 * <P>Test calling saveWithholdingInformation() to update a W4 and add the Worker's SSN</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a W4 WithholdingInformation to the worker</LI>
	 * <LI>Set up the SaveWithholdingInfoRequest object</LI>
	 * <LI>Call TaxWithholdingsService.saveWithholdingInformation() to update a W4 and add the Worker's SSN</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the W4 information is updated and the SSN is saved on the Worker</LI>
	 * </UL>
	 */
	@Test
	public void testUpdateW4_AddSSN() throws Exception {
		// Setup
		addW4WithholdingInfo(sUniqueId);
		WithholdingInformation withholdingInfo = getW4WithholdingInfo(sUniqueId);
		withholdingInfo.setAllowances(2);
		String expectedSSN = "123456789";
		SaveWithholdingInfoRequest saveRequest = new SaveWithholdingInfoRequest();
		saveRequest.setWorkerId(sUniqueId);
		saveRequest.setWorkerTaxIdNumber(expectedSSN);
		saveRequest.setWagePlanCode(null);
		saveRequest.setWithholdingInfo(withholdingInfo);
		saveRequest.setWithholdingInfoOperation(WithholdingInfoOperation.UPDATE);
    	// Exercise
		SaveWithholdingResponse response = sTaxWithholdingsService.saveWithholdingInformation(saveRequest);
		// Verify
		AssertJUnit.assertEquals("WorkerTaxIdNumber mismatch!", expectedSSN, response.getWorker().getWorkerTaxIdNumber());
		WithholdingInformation actual = response.getWithholding();
		// Fix the objectHandle
		withholdingInfo.setObjectHandle(actual.getObjectHandle());
		AssertJUnit.assertEquals("WithholdingInformation mismatch!", withholdingInfo, actual);
	}

	/**
	 * <P>Test calling saveWithholdingInformation() to update a W4 and update the Worker's SSN</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a W4 WithholdingInformation to the worker</LI>
	 * <LI>Set up the SaveWithholdingInfoRequest object</LI>
	 * <LI>Call TaxWithholdingsService.saveWithholdingInformation() to update a W4 and add the Worker's SSN</LI>
	 * <LI>Change the SSN value</LI>
	 * <LI>Call TaxWithholdingsService.saveWithholdingInformation() to update a W4 and update the Worker's SSN</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the W4 information is updated and the SSN is updated on the Worker</LI>
	 * </UL>
	 */
	@Test
	public void testUpdateW4_UpdateSSN() throws Exception {
		// Setup
		addW4WithholdingInfo(sUniqueId);
		WithholdingInformation withholdingInfo = getW4WithholdingInfo(sUniqueId);
		withholdingInfo.setAllowances(2);
		String expectedSSN = "123456789";
		SaveWithholdingInfoRequest saveRequest = new SaveWithholdingInfoRequest();
		saveRequest.setWorkerId(sUniqueId);
		saveRequest.setWorkerTaxIdNumber(expectedSSN);
		saveRequest.setWagePlanCode(null);
		saveRequest.setWithholdingInfo(withholdingInfo);
		saveRequest.setWithholdingInfoOperation(WithholdingInfoOperation.UPDATE);
		// Save the SSN first
		SaveWithholdingResponse response = sTaxWithholdingsService.saveWithholdingInformation(saveRequest);
		// Now change the SSN value and update with the latest copy of WithholdingInformation
		expectedSSN = "987654321";
		saveRequest.setWorkerTaxIdNumber(expectedSSN);
		withholdingInfo = response.getWithholding();
		withholdingInfo.setAllowances(2);
		saveRequest.setWithholdingInfo(withholdingInfo);
    	// Exercise
		response = sTaxWithholdingsService.saveWithholdingInformation(saveRequest);
		// Verify
		AssertJUnit.assertEquals("WorkerTaxIdNumber mismatch!", expectedSSN, response.getWorker().getWorkerTaxIdNumber());
		WithholdingInformation actual = response.getWithholding();
		// Fix the objectHandle
		withholdingInfo.setObjectHandle(actual.getObjectHandle());
		AssertJUnit.assertEquals("WithholdingInformation mismatch!", withholdingInfo, actual);
	}

	/**
	 * <P>Test calling saveWithholdingInformation() to update a W4 and delete the Worker's SSN</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a W4 WithholdingInformation to the worker</LI>
	 * <LI>Set up the SaveWithholdingInfoRequest object</LI>
	 * <LI>Call TaxWithholdingsService.saveWithholdingInformation() to update a W4 and add the Worker's SSN</LI>
	 * <LI>Change the SSN value</LI>
	 * <LI>Call TaxWithholdingsService.saveWithholdingInformation() to update a W4 and delete the Worker's SSN</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the W4 information is updated and the SSN is deleted on the Worker</LI>
	 * </UL>
	 */
	@Test
	public void testUpdateW4_DeleteSSNByUsingNull() throws Exception {
		// Setup
		addW4WithholdingInfo(sUniqueId);
		WithholdingInformation withholdingInfo = getW4WithholdingInfo(sUniqueId);
		withholdingInfo.setAllowances(2);
		String expectedSSN = "123456789";
		SaveWithholdingInfoRequest saveRequest = new SaveWithholdingInfoRequest();
		saveRequest.setWorkerId(sUniqueId);
		saveRequest.setWorkerTaxIdNumber(expectedSSN);
		saveRequest.setWagePlanCode(null);
		saveRequest.setWithholdingInfo(withholdingInfo);
		saveRequest.setWithholdingInfoOperation(WithholdingInfoOperation.UPDATE);
		// Save the SSN first
		SaveWithholdingResponse response = sTaxWithholdingsService.saveWithholdingInformation(saveRequest);
		// Now set the SSN value to null and update with the latest copy of WithholdingInformation
		saveRequest.setWorkerTaxIdNumber(null);
		withholdingInfo = response.getWithholding();
		withholdingInfo.setAllowances(2);
		saveRequest.setWithholdingInfo(withholdingInfo);
    	// Exercise
		response = sTaxWithholdingsService.saveWithholdingInformation(saveRequest);
		// Verify
		AssertJUnit.assertEquals("WorkerTaxIdNumber not deleted!", null, response.getWorker().getWorkerTaxIdNumber());
		WithholdingInformation actual = response.getWithholding();
		// Fix the objectHandle
		withholdingInfo.setObjectHandle(actual.getObjectHandle());
		AssertJUnit.assertEquals("WithholdingInformation mismatch!", withholdingInfo, actual);
	}

	/**
	 * <P>Test calling saveWithholdingInformation() to update a W4 and set the Worker's SSN to blank</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a W4 WithholdingInformation to the worker</LI>
	 * <LI>Set up the SaveWithholdingInfoRequest object</LI>
	 * <LI>Call TaxWithholdingsService.saveWithholdingInformation() to update a W4 and add the Worker's SSN</LI>
	 * <LI>Change the SSN value</LI>
	 * <LI>Call TaxWithholdingsService.saveWithholdingInformation() to update a W4 and set the Worker's SSN to blank</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the W4 information is updated and the SSN is deleted on the Worker</LI>
	 * </UL>
	 */
	@Test
	public void testUpdateW4_UpdateSSNToBlank() throws Exception {
		// Setup
		addW4WithholdingInfo(sUniqueId);
		WithholdingInformation withholdingInfo = getW4WithholdingInfo(sUniqueId);
		withholdingInfo.setAllowances(2);
		String expectedSSN = "123456789";
		SaveWithholdingInfoRequest saveRequest = new SaveWithholdingInfoRequest();
		saveRequest.setWorkerId(sUniqueId);
		saveRequest.setWorkerTaxIdNumber(expectedSSN);
		saveRequest.setWagePlanCode(null);
		saveRequest.setWithholdingInfo(withholdingInfo);
		saveRequest.setWithholdingInfoOperation(WithholdingInfoOperation.UPDATE);
		// Save the SSN first
		SaveWithholdingResponse response = sTaxWithholdingsService.saveWithholdingInformation(saveRequest);
		// Now set the SSN value to an empty string and update with the latest copy of WithholdingInformation
		expectedSSN = "";
		saveRequest.setWorkerTaxIdNumber(expectedSSN);
		withholdingInfo = response.getWithholding();
		withholdingInfo.setAllowances(2);
		saveRequest.setWithholdingInfo(withholdingInfo);
    	// Exercise
		response = sTaxWithholdingsService.saveWithholdingInformation(saveRequest);
		// Verify
		AssertJUnit.assertEquals("WorkerTaxIdNumber not blank!", expectedSSN, response.getWorker().getWorkerTaxIdNumber());
		WithholdingInformation actual = response.getWithholding();
		// Fix the objectHandle
		withholdingInfo.setObjectHandle(actual.getObjectHandle());
		AssertJUnit.assertEquals("WithholdingInformation mismatch!", withholdingInfo, actual);
	}

	/**
	 * <P>Test calling saveWithholdingInformation() with the Wage plan code set to null, engine will set default value
	 * to 'S'</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a W4 WithholdingInformation to the worker</LI>
	 * <LI>Set up the SaveWithholdingInfoRequest object</LI>
	 * <LI>Call TaxWithholdingsService.saveWithholdingInformation() with the Wage plan code set to null</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that no DE4 information is added and the Wage plan code is set on the Worker</LI>
	 * </UL>
	 */
	@Test
	public void testAddNullWagePlanCode() throws Exception {
		// Setup
		addW4WithholdingInfo(sUniqueId);
		String expectedWagePlanCode = "S";
		SaveWithholdingInfoRequest saveRequest = new SaveWithholdingInfoRequest();
		saveRequest.setWorkerId(sUniqueId);
		saveRequest.setWorkerTaxIdNumber(null);
		// Explicitly setting the wage plan code to null, engine should set a default value of "S"
		saveRequest.setWagePlanCode(null);
		saveRequest.setWithholdingInfo(null);
		saveRequest.setWithholdingInfoOperation(WithholdingInfoOperation.NOT_SPECIFIED);
    	// Exercise
		SaveWithholdingResponse response = sTaxWithholdingsService.saveWithholdingInformation(saveRequest);
		// Verify
		AssertJUnit.assertEquals("Wage plan code mismatch!", expectedWagePlanCode, response.getWorker().getWagePlanCode());
		WithholdingInformation actual = response.getWithholding();
		AssertJUnit.assertNull("WithholdingInformation not null!", actual);
		// Verify that no DE4 is set on the worker
		actual = getDe4WithholdingInfo(sUniqueId);
		AssertJUnit.assertNull("WithholdingInformation not null!", actual);
	}

	/**
	 * <P>Test calling saveWithholdingInformation() to add the Wage plan code only</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a W4 WithholdingInformation to the worker</LI>
	 * <LI>Set up the SaveWithholdingInfoRequest object</LI>
	 * <LI>Call TaxWithholdingsService.saveWithholdingInformation() to add the Wage plan code only</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that no DE4 information is added and the Wage plan code is saved on the Worker</LI>
	 * </UL>
	 */
	@Test
	public void testAddWagePlanCodeOnly() throws Exception {
		// Setup
		addW4WithholdingInfo(sUniqueId);
		String expectedWagePlanCode = "S";
		SaveWithholdingInfoRequest saveRequest = new SaveWithholdingInfoRequest();
		saveRequest.setWorkerId(sUniqueId);
		saveRequest.setWorkerTaxIdNumber(null);
		saveRequest.setWagePlanCode(expectedWagePlanCode);
		saveRequest.setWithholdingInfo(null);
		saveRequest.setWithholdingInfoOperation(WithholdingInfoOperation.NOT_SPECIFIED);
    	// Exercise
		SaveWithholdingResponse response = sTaxWithholdingsService.saveWithholdingInformation(saveRequest);
		// Verify
		AssertJUnit.assertEquals("Wage plan code mismatch!", expectedWagePlanCode, response.getWorker().getWagePlanCode());
		WithholdingInformation actual = response.getWithholding();
		AssertJUnit.assertNull("WithholdingInformation not null!", actual);
		// Verify that no DE4 is set on the worker
		actual = getDe4WithholdingInfo(sUniqueId);
		AssertJUnit.assertNull("WithholdingInformation not null!", actual);
	}

	/**
	 * <P>Test calling saveWithholdingInformation() to add a DE4 WithholdingInformation</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a W4 WithholdingInformation to the worker</LI>
	 * <LI>Set up the SaveWithholdingInfoRequest object</LI>
	 * <LI>Call TaxWithholdingsService.saveWithholdingInformation() to add the DE4</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the DE4 information is added to the Worker</LI>
	 * </UL>
	 */
	@Test
	public void testAddDE4() throws Exception {
		// Setup
		addW4WithholdingInfo(sUniqueId);
		WithholdingInformation withholdingInfo = newWithholdingInformation(FormType.CA_DE4);
		withholdingInfo.setExtraWithholding(new BigDecimal("20.00"));
		String expectedWagePlanCode = "S";
		SaveWithholdingInfoRequest saveRequest = new SaveWithholdingInfoRequest();
		saveRequest.setWorkerId(sUniqueId);
		saveRequest.setWorkerTaxIdNumber(null);
		saveRequest.setWagePlanCode(expectedWagePlanCode);
		saveRequest.setWithholdingInfo(withholdingInfo);
		saveRequest.setWithholdingInfoOperation(WithholdingInfoOperation.ADD);
    	// Exercise
		SaveWithholdingResponse response = sTaxWithholdingsService.saveWithholdingInformation(saveRequest);
		// Verify
		AssertJUnit.assertEquals("Wage plan code mismatch!", expectedWagePlanCode, response.getWorker().getWagePlanCode());
		WithholdingInformation actual = response.getWithholding();
		// Fix the objectHandle
		withholdingInfo.setObjectHandle(actual.getObjectHandle());
		AssertJUnit.assertEquals("WithholdingInformation mismatch!", withholdingInfo, actual);
		// Verify that the DE4 is saved on the worker
		actual = getDe4WithholdingInfo(sUniqueId);
		AssertJUnit.assertEquals("WithholdingInformation mismatch!", withholdingInfo, actual);
	}

	/**
	 * <P>Test calling saveWithholdingInformation() to delete the DE4 WithholdingInformation</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a W4 WithholdingInformation to the worker</LI>
	 * <LI>Add a De4 WithholdingInformation to the worker</LI>
	 * <LI>Set up the SaveWithholdingInfoRequest object</LI>
	 * <LI>Call TaxWithholdingsService.saveWithholdingInformation() to delete the DE4</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the DE4 information is removed from the Worker</LI>
	 * </UL>
	 */
	@Test
	public void testDeleteDE4() throws Exception {
		// Setup
		addW4WithholdingInfo(sUniqueId);
		addDe4WithholdingInfo(sUniqueId);
		WithholdingInformation withholdingInfo = getDe4WithholdingInfo(sUniqueId);
		String expectedWagePlanCode = "S";
		SaveWithholdingInfoRequest saveRequest = new SaveWithholdingInfoRequest();
		saveRequest.setWorkerId(sUniqueId);
		saveRequest.setWorkerTaxIdNumber(null);
		saveRequest.setWagePlanCode(expectedWagePlanCode);
		saveRequest.setWithholdingInfo(withholdingInfo);
		saveRequest.setWithholdingInfoOperation(WithholdingInfoOperation.DELETE);
    	// Exercise
		SaveWithholdingResponse response = sTaxWithholdingsService.saveWithholdingInformation(saveRequest);
		// Verify
		AssertJUnit.assertEquals("Wage plan code mismatch!", expectedWagePlanCode, response.getWorker().getWagePlanCode());
		WithholdingInformation actual = response.getWithholding();
		AssertJUnit.assertNull("WithholdingInformation not null!", actual);
		// Verify that the DE4 is removed from the worker
		actual = getDe4WithholdingInfo(sUniqueId);
		AssertJUnit.assertNull("WithholdingInformation not null!", actual);
	}

	/**
	 * <P>Test calling saveWithholdingInformation() to update a DE4 and add an acceptable CA Wage plan code</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a DE4 WithholdingInformation to the worker</LI>
	 * <LI>Set up the SaveWithholdingInfoRequest object</LI>
	 * <LI>Call TaxWithholdingsService.saveWithholdingInformation() to update a DE4 and add the CA Wage plan code</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the DE4 information is updated and the Wage plan code is saved on the Worker</LI>
	 * </UL>
	 */
	@Test(dataProvider = "wage-plan-code-provider")
	public void testUpdateDe4_AddWagePlanCode(String expectedWagePlanCode) throws Exception {
		// Setup
		addDe4WithholdingInfo(sUniqueId);
		WithholdingInformation withholdingInfo = getDe4WithholdingInfo(sUniqueId);
		withholdingInfo.setAllowances(2);
		SaveWithholdingInfoRequest saveRequest = new SaveWithholdingInfoRequest();
		saveRequest.setWorkerId(sUniqueId);
		saveRequest.setWorkerTaxIdNumber(null);
		saveRequest.setWagePlanCode(expectedWagePlanCode);
		saveRequest.setWithholdingInfo(withholdingInfo);
		saveRequest.setWithholdingInfoOperation(WithholdingInfoOperation.UPDATE);
    	// Exercise
		SaveWithholdingResponse response = sTaxWithholdingsService.saveWithholdingInformation(saveRequest);
		// Verify
		AssertJUnit.assertEquals("Wage plan code mismatch!", expectedWagePlanCode, response.getWorker().getWagePlanCode());
		WithholdingInformation actual = response.getWithholding();
		// Fix the objectHandle
		withholdingInfo.setObjectHandle(actual.getObjectHandle());
		AssertJUnit.assertEquals("WithholdingInformation mismatch!", withholdingInfo, actual);
	}
	
	@DataProvider(name = "wage-plan-code-provider")
	public Object[][] rangeData() {
		return new Object[][] {{"S"}, {"A"}, {"L"}, {"J"}, {"P"}, {"R"}, {"U"}};
	}
	

	/**
	 * <P>Test calling saveWithholdingInformation() to update a DE4 and update the Wage plan code</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a DE4 WithholdingInformation to the worker</LI>
	 * <LI>Set up the SaveWithholdingInfoRequest object</LI>
	 * <LI>Call TaxWithholdingsService.saveWithholdingInformation() to update a DE4 and add the Wage plan code</LI>
	 * <LI>Change the Wage plan code value</LI>
	 * <LI>Call TaxWithholdingsService.saveWithholdingInformation() to update a DE4 and update the Wage plan code</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the DE4 information is updated and the wage plan code is updated on the Worker</LI>
	 * </UL>
	 */
	@Test
	public void testUpdateDe4_UpdateWagePlanCode() throws Exception {
		// Setup
		addDe4WithholdingInfo(sUniqueId);
		WithholdingInformation withholdingInfo = getDe4WithholdingInfo(sUniqueId);
		withholdingInfo.setAllowances(1);
		String expectedWagePlanCode = "S";
		SaveWithholdingInfoRequest saveRequest = new SaveWithholdingInfoRequest();
		saveRequest.setWorkerId(sUniqueId);
		saveRequest.setWorkerTaxIdNumber(null);
		saveRequest.setWagePlanCode(expectedWagePlanCode);
		saveRequest.setWithholdingInfo(withholdingInfo);
		saveRequest.setWithholdingInfoOperation(WithholdingInfoOperation.UPDATE);
		// Save the Wage plan code first
		SaveWithholdingResponse response = sTaxWithholdingsService.saveWithholdingInformation(saveRequest);
		// Now change the wage plan code value and update with the latest copy of WithholdingInformation
		expectedWagePlanCode = "A";
		saveRequest.setWagePlanCode(expectedWagePlanCode);
		withholdingInfo = response.getWithholding();
		withholdingInfo.setAllowances(2);
		saveRequest.setWithholdingInfo(withholdingInfo);
    	// Exercise
		response = sTaxWithholdingsService.saveWithholdingInformation(saveRequest);
		// Verify
		AssertJUnit.assertEquals("Wage plan code mismatch!", expectedWagePlanCode, response.getWorker().getWagePlanCode());
		WithholdingInformation actual = response.getWithholding();
		// Fix the objectHandle
		withholdingInfo.setObjectHandle(actual.getObjectHandle());
		AssertJUnit.assertEquals("WithholdingInformation mismatch!", withholdingInfo, actual);
	}

}