package com.intuit.sb.payroll.qa.services.taxes;

import java.util.HashMap;

import org.testng.AssertJUnit;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.taxes.EmployerTaxIdInfo;
import com.intuit.sb.payroll.services.taxes.EmployerUnemploymentIns;
import com.intuit.sb.payroll.services.taxes.Jurisdiction;
import com.intuit.sb.payroll.services.taxes.TaxDetails;
import com.intuit.sb.payroll.services.taxes.EmployerTaxSettingsServiceTestBase;
import com.intuit.sb.payroll.services.taxes.TaxSummary;

/**
 * The TaxDetailsTest is used to validate the get of tax details and also validating the tax details once the SUI and
 * TIN are updated.
 * 
 * @author vmalik
 * 
 */
public class TaxDetailsTest extends EmployerTaxSettingsServiceTestBase {

	/**
	 * Map of values for the state to the corresponding name.
	 */
	private HashMap<String, String> statesMap;

	@BeforeMethod
	public void setUp() {
		statesMap = new HashMap<String, String>();
		statesMap.put("CA", "California");
		statesMap.put("TX", "Texas");
		try {
			super.setUp();
		} catch (Exception ex) {

		}
	}

	/**
	 * <P>
	 * Test getTaxDetails() - happy path
	 * </P>
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call getTaxDetail()</LI>
	 * <LI>
	 * </OL>
	 * <UL>
	 * <LI>Validate the tax detail array returned</LI>
	 * 
	 * @throws ValidationException
	 */
	@Test
	public void testTaxDetails() throws Exception {
		try {
			TaxDetails[] taxDetails = mEmployerTaxSettingService.getTaxDetails();
			validateTaxDetails(taxDetails);
		} catch (ValidationException ex) {
			AssertJUnit.fail("No validation exception should not be thrown on getting the tax details");
		}
	}

	@Test
	public void testTaxDetailWithSuiUpdated() throws ServiceFailureException, ValidationException {
		EmployerUnemploymentIns[] suisetting = mEmployerTaxSettingService.getEmployerUnemploymentIns();
		AssertJUnit.assertNotNull(suisetting);
		AssertJUnit.assertTrue("Empty SUI setting", suisetting.length > 0);
		for (EmployerUnemploymentIns sui : suisetting) {
			if (sui.getState().equals("California")) {
				sui.setEffectiveRate("11.1");
			} else if (sui.getState().equals("Texas")) {
				sui.setEffectiveRate("12.1");
			}
		}
		mEmployerTaxSettingService.updateEmployerUnemploymentIns(suisetting);
		TaxDetails[] taxDetails = mEmployerTaxSettingService.getTaxDetails();
		AssertJUnit.assertNotNull("TaxDetail is null", taxDetails);
		AssertJUnit.assertTrue("TaxDetail is zero", taxDetails.length > 0);
		for (TaxDetails taxdetail : taxDetails) {
			if (!taxdetail.getJurisdictionName().equals("US")) {
				for (TaxSummary taxsummary : taxdetail.getTaxSummaryCollection()) {
					if (taxsummary.getShortDescription().contains("CA - Unemployment")) {
						AssertJUnit.assertTrue(taxsummary.getCompanyTaxRate().equals("11.1%"));
					} else if (taxsummary.getShortDescription().contains("TX - Unemployment")) {
						AssertJUnit.assertTrue(taxsummary.getCompanyTaxRate().equals("12.1%"));
					}
				}
			}
		}
	}

	@Test
	public void testTaxDetailWithTINUpdated() throws Exception {
		// Update the TIN for states
		EmployerTaxIdInfo[] taxIdInfos = mEmployerTaxSettingService.getEmployerTaxIDs();
		AssertJUnit.assertNotNull(taxIdInfos);
		for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos) {
			AssertJUnit.assertNotNull(employerTaxIdInfo.getTaxAgency());
			if (employerTaxIdInfo.getTaxAgency().name.contains("US_IRS")) {
				employerTaxIdInfo.setTaxId("22-7777777");
			} else if (employerTaxIdInfo.getTaxAgency().name.contains("CA_EDD")) {
				employerTaxIdInfo.setTaxId("333-4444-1");
			} else if (employerTaxIdInfo.getTaxAgency().name.contains("TX_TWC")) {
				employerTaxIdInfo.setTaxId("22-666666-1");
			}
		}
		mEmployerTaxSettingService.updateEmployerTaxIDs(taxIdInfos);
		TaxDetails[] taxDetails = mEmployerTaxSettingService.getTaxDetails();
		AssertJUnit.assertNotNull("TaxDetail is null", taxDetails);
		AssertJUnit.assertTrue("TaxDetail is zero", taxDetails.length > 0);
		for (TaxDetails taxDetail : taxDetails) {
			if (taxDetail.getJurisdictionName() != null && taxDetail.getJurisdictionName().length() > 0) {
				if (taxDetail.getJurisdictionName().equals("US")) {
					AssertJUnit.assertTrue("The TIN for Federal does not match ", taxDetail.getAgencyTIN().equals(
							"227777777"));
				} else if (taxDetail.getJurisdictionName().equals("CA")) {
					AssertJUnit.assertTrue("The TIN for CA does not match ", taxDetail.getAgencyTIN().equals("33344441"));
				} else if (taxDetail.getJurisdictionName().equals("TX")) {
					AssertJUnit.assertTrue("The TIN for CA does not match ", taxDetail.getAgencyTIN().equals("226666661"));
				}
			}
		}
	}

	/**
	 * <P>
	 * Test getTaxDetails() - with on jurisdiction removed
	 * </P>
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call getJurisdictions()</LI>
	 * <LI>remove TX worklocation by calling removeSomeWorkLocations()</LI>
	 * <LI>Verify that the jurisdiction corrosponding to jurisdiction is removed</LI>
	 * <LI>Call getTaxDetail()</LI>
	 * <LI>
	 * </OL>
	 * <UL>
	 * <LI>Validate the tax detail array returned</LI>
	 * 
	 * @throws Exception
	 */
	@Test
	public void testTaxDetailsWithWorklocationWithJursdictionRemoved() throws Exception {
		Jurisdiction[] original_jurisdictions = mEmployerTaxSettingService.getJurisdictions();
		removeSomeWorkLocations(sMesquiteLocation); // retains TX and removes CA worklocations
		Jurisdiction[] after_jurisdictions = mEmployerTaxSettingService.getJurisdictions();
		AssertJUnit.assertTrue(original_jurisdictions.length > after_jurisdictions.length);

		TaxDetails[] taxDetails = mEmployerTaxSettingService.getTaxDetails();
		validateTaxDetails(taxDetails);
	}

	/**
	 * <P>
	 * Test getTaxDetails() - with on jurisdiction added
	 * </P>
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call getJurisdictions()</LI>
	 * <LI>remove TX worklocation by calling removeSomeWorkLocations()</LI>
	 * <LI>Verify that the jurisdiction corrosponding to jurisdiction is removed</LI>
	 * <LI>Add worklocation by calling addWorkLocation()</LI>
	 * <LI>Call getTaxDetail()</LI>
	 * <LI>
	 * </OL>
	 * <UL>
	 * <LI>Validate the tax detail array returned</LI>
	 * 
	 * @throws Exception
	 */
	@Test
	public void testTaxDetailWithWorklocationInNewJurisdictionAdded() throws Exception {
		// Removing jurisdiction and checking if the jurisdiction gets deleted
		Jurisdiction[] original_jurisdictions = mEmployerTaxSettingService.getJurisdictions();
		removeSomeWorkLocations(sMesquiteLocation); // retains TX and removes CA jurisdiction
		Jurisdiction[] after_remove_jurisdictions = mEmployerTaxSettingService.getJurisdictions();
		AssertJUnit.assertTrue(original_jurisdictions.length > after_remove_jurisdictions.length);

		// Add a worklocation
		addWorkLocation("Mountain View", "160001");// add the worklocation
		Jurisdiction[] after_add_jurisdictions = mEmployerTaxSettingService.getJurisdictions();
		AssertJUnit
				.assertTrue("The Jurisdiction after adding is " + after_add_jurisdictions.length + "before adding is "
						+ after_remove_jurisdictions.length,
						after_remove_jurisdictions.length < after_add_jurisdictions.length);

		TaxDetails[] taxDetails = mEmployerTaxSettingService.getTaxDetails();
		// Validate
		validateTaxDetails(taxDetails);
	}

	/**
	 * The Utility method that verifys that : 1) Tax detail exist for federal & each state jurisdiction 2) Agency TIN
	 * exists in tax detail if TIN is already set 3) Tax summary exists for taxdetail 4) Agency url exist for each tax
	 * detail
	 * 
	 * @param taxDetails
	 * @throws ValidationException
	 */
	private void validateTaxDetails(TaxDetails[] taxDetails) throws Exception {
		Jurisdiction[] jurisdictions = mEmployerTaxSettingService.getJurisdictions();
		EmployerTaxIdInfo[] taxIdInfos = mEmployerTaxSettingService.getEmployerTaxIDs();
		boolean allmatch = false;
		// verify that tax detail exist for federal & each state jurisdiction
		for (TaxDetails taxdt : taxDetails) {
			for (Jurisdiction jurisdiction : jurisdictions) {
				allmatch = false;
				if (jurisdiction.getName().equals(statesMap.get(taxdt.getJurisdictionName())))
					;
				{
					allmatch = true;
					break;
				}
			}
		}
		AssertJUnit.assertTrue("Jurisdiction names do not correspond to tax detail jurisdiction", allmatch);
		for (TaxDetails taxdt : taxDetails) {
			for (EmployerTaxIdInfo taxIdInfo : taxIdInfos) {
				if (taxIdInfo.getTaxAgency().name.contains(taxdt.getAgencyName())) {
					AssertJUnit.assertSame(taxIdInfo.getTaxId() == null || taxIdInfo.getTaxId().equals("") ? null
							: taxIdInfo.getTaxId(),
							taxdt.getAgencyTIN() == null || taxdt.getAgencyTIN().equals("") ? null : taxdt
									.getAgencyTIN());
					// verify that agency TIN exist for each tax detail
					if (taxdt.getAgencyTIN().equals(taxIdInfo.getTaxId())) {
						// verify that tax summary exist for taxdetail
						AssertJUnit.assertNotNull("Tax Summary is null", taxdt.getTaxSummaryCollection());
						AssertJUnit.assertTrue("Empty Tax Summary ", taxdt.getTaxSummaryCollection().length > 0);

						// verify that agency url exist for each tax detail
						AssertJUnit.assertTrue("Agency URL does not exist", taxdt.getAgencyUrl().length() > 0);
						AssertJUnit.assertTrue("Agency Name does not exist", taxdt.getAgencyName().length() > 0);
					}
				}
			}

		}
	}
}