package com.intuit.sb.payroll.qa.services.workerpayments;

import java.math.BigDecimal;

import javax.transaction.Status;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.DE4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.W4WithholdingStatusEnum;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.workerpayments.CalcInputs;
import com.intuit.sb.payroll.services.workerpayments.PaycheckDetails;
import com.intuit.sb.payroll.services.workerpayments.PaycheckRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunPeriod;
import com.intuit.sb.payroll.services.workerpayments.PayrollScheduleFrequency;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;


/**
 *  <P>The <CODE>CalcPaycheckDetailsExceptions2Test</CODE> class contains tests for testing exceptions that are thrown
 *     by IPayEmployeeService.calcPaycheckDetails()</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class CalcPaycheckDetailsExceptions2Test extends WorkerPaymentServiceTestBase {
    private String mHourlyRate = "10.00";
    private String mOvertimeRate = "15.00";
    private String mBonusAmount = "1000.00";
    private double mHourlyHours = 80.0;
    private double mOvertimeHours = 8.0;
    private CompensationItem[] wageItems;

    @BeforeClass
    public static void setUpCalcPaycheckDetailsExceptions2Test() {
        setUpSanMateoAddress();
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
        sEmployer.setNonProfit(false);
    }
    
    /**
     * <P>Sets up partial required data for testing IPayEmployeeService.calcPaycheckDetails</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up the withholding infos</LI>
     * <LI>Set up paycheck related dates</LI>
     * <LI>Obtain list of wage items</LI>
     * </OL>
     *
     */
    @BeforeMethod
    public void setUp() throws Exception {
        super.setUp();
        wageItems = null;
        try {
            // Set up wages
            setupWages(sWorkerId);
            // NOTE:  Purposely don't apply the applicable taxes here
            
            // Set up the withholding info
            setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax, W4WithholdingStatusEnum.MARRIED,
                                 1, new BigDecimal("0.00"));
            setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax, 
                                    DE4WithholdingStatusEnum.MARRIEDONEINCOME, 1, 0, new BigDecimal("0.00"));
            
            // Set up paycheck related dates
            setupPaycheckDates(PayrollScheduleFrequency.BIWEEKLY);
            
            // Get the wage items for this worker
            wageItems = sWageService.getCompensationItems(sWorkerId);
        } catch (ValidationException ex) {
            AssertJUnit.fail("Error setting up required data for paycheck calculation!\n" + ex.getMessage());
        } catch (Exception e) {
            AssertJUnit.fail("Unexpected Exception thrown!\n" + e.getMessage());
        }
    }
    
    @AfterMethod
    public void tearDown() throws Exception {
        if (getUserTransaction().getStatus() == Status.STATUS_ACTIVE) {
            // Remove all withholding information associated with the worker
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.W4IRS);
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.DE4);
        }

        super.tearDown();
    }
    
    /**
     * <P>Test calling calcPaycheckDetails() when the worker is not ready for pay (applicable taxes not applied)</P>
     *
     * <H3>Requirements Covered:</H3>
     * <TABLE ALIGN="left" BORDER=1 WIDTH="100%">
     *  <TR ALIGN="left">
     *      <TH>Number</TH>
     *      <TH>Description</TH>
     *  </TR>
     *  <TR>
     *      <TD>?</TD>
     *      <TD>???</TD>
     *  </TR>
     * </TABLE><BR/><BR/><BR/>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up a CalcRequest object</LI>
     * <LI>Call IPayEmployeeService's calcPaycheckDetails method to perform the actual work</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown</LI>
     * </UL>
     */
    @Test(expectedExceptions=WorkflowException.class)
    public void testNotReadyForPay() throws Exception {
        PayrollRunPeriod payrollRunPeriod = createPayrollRunPeriod(PayrollScheduleFrequency.BIWEEKLY);
        PaycheckRequest calcRequest = new PaycheckRequest();
        calcRequest.setWorkerId(sWorkerId);
	    CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, 
	    		mOvertimeRate, mOvertimeHours, mBonusAmount);
	    CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
        calcRequest.setPayrollRunPeriod(payrollRunPeriod);
        
        @SuppressWarnings("unused")
        PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
    }

}
