package com.intuit.sb.payroll.qa.services.workerpayments;

import java.math.BigDecimal;
import java.util.Date;

import javax.transaction.Status;

import org.testng.AssertJUnit;

import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.DE4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.model.RelationshipToOwnerEnum;
import com.intuit.sb.payroll.entity.model.W4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.workerpayments.CalcInputs;
import com.intuit.sb.payroll.services.workerpayments.PaycheckDetails;
import com.intuit.sb.payroll.services.workerpayments.PaycheckDistributionMethod;
import com.intuit.sb.payroll.services.workerpayments.PaycheckLineItem;
import com.intuit.sb.payroll.services.workerpayments.PaycheckRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollItemCategoryType;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunPeriod;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSaveRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSummary;
import com.intuit.sb.payroll.services.workerpayments.PayrollScheduleFrequency;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;


/**
 * <P>
 * The <CODE>CalcPaycheckDetailsYTDSameCompensationTypesTest</CODE> class contains tests for testing
 * the YTD paycheck calculation functionality of WorkerPaymentService.calcPaycheckDetails() using compensation items
 * with the same type.
 * </P>
 * 
 * $Author$ 
 * $Revision$ 
 * $DateTime$
 * 
 */
public class CalcPaycheckDetailsYTDSameCompensationTypesTest extends WorkerPaymentServiceTestBase {
	private static BigDecimal sZeroBigDecimal = new BigDecimal("0.00");
	private static CompensationItem notACompensationItem = null;
	private String mHourlyRate = "10.00";
	private String mOvertimeRate = "1.5";
	private String mBonusAmount = "1000.00";
	private double mPieces = 1000.0;

	@BeforeClass
	public static void setUpCalcPaycheckDetailsTest() {
		setUpSanMateoAddress();
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
		sEmployer.setNonProfit(false);
	}

	@BeforeMethod
	public void setUp() throws Exception {
		super.setUp();
		try {
			refreshWorker();
			// Set up wages
			setupWages(sWorkerId);
			// Apply the applicable taxes
			setUpTestData(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED);
			applyApplicableTaxes();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			AssertJUnit.fail("Error in setUp() in CalcPaycheckDetailsYTDSameCompensationTypesTest(): " + t.getMessage());
		}
	}

	@AfterMethod
	public void tearDown() throws Exception {
		if (getUserTransaction().getStatus() == Status.STATUS_ACTIVE) {
			// Remove all withholding information associated with the worker
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.W4IRS);
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.DE4);
		}

		super.tearDown();
	}
	
	private BigDecimal getCalculatedOvertimeRate() {
		return getCalculatedOvertimeRate(mHourlyRate);
	}

	private BigDecimal getCalculatedOvertimeRate(String rate) {
		BigDecimal calculatedRate = new BigDecimal(rate);
		calculatedRate = calculatedRate.multiply(new BigDecimal(mOvertimeRate));
		// Set to 2 decimal places
		calculatedRate = calculatedRate.setScale(2);
		return calculatedRate;
	}

    private PayrollRunSaveRequest createPayrollRunSaveRequest(Long[] workerIds, Date periodEndDate,
    				PayrollFrequencyEnum frequency, double hourlyHours, double overtimeHours,
    				String bonusAmount) throws Exception {
        PayrollRunSaveRequest saveRequest = new PayrollRunSaveRequest();
        PaycheckRequest[] paycheckRequests = new PaycheckRequest[workerIds.length];
        PayrollRunPeriod payrollRunPeriod = createAPayrollRunPeriod(periodEndDate,
                                                                    frequency, sDefaultPayrollDayOffset,
                                                                    sDefaultPayrollRunDayOffset);

        for(int count = 0;count < workerIds.length;count++) {
            paycheckRequests[count] = new PaycheckRequest();
            CompensationItem[] workerCompensationItems = sWageService.getCompensationItems(workerIds[count]);
            for (CompensationItem compensationItem : workerCompensationItems) {
				if (compensationItem.getCompensationType().equals(CompensationType.Hourly)) {
					compensationItem.setUnits(hourlyHours);
				} else if (compensationItem.getCompensationType().equals(CompensationType.Overtime)) {
					compensationItem.setUnits(overtimeHours);
				} else if (compensationItem.getCompensationType().equals(CompensationType.TipsPaycheck) ||
						   compensationItem.getCompensationType().equals(CompensationType.TipsCash) ||
						   compensationItem.getCompensationType().equals(CompensationType.TipsAllocated)) {
					compensationItem.setRate(new BigDecimal(bonusAmount));
				} else if (compensationItem.getCompensationType().equals(CompensationType.Bonus)) {
					compensationItem.setRate(new BigDecimal(bonusAmount));
				} else if (compensationItem.getCompensationType().equals(CompensationType.Commission)) {
					compensationItem.setRate(new BigDecimal(bonusAmount));
				} else if (compensationItem.getCompensationType().equals(CompensationType.Piecework)) {
			        compensationItem.setUnits(mPieces);
				}
			}
			CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(workerCompensationItems);
			paycheckRequests[count].setCalcInputs(calcInputs);
            paycheckRequests[count].setWorkerId(workerIds[count]);
            paycheckRequests[count].setDistributionMethod(PaycheckDistributionMethod.PRINT);
            paycheckRequests[count].setPayrollRunPeriod(payrollRunPeriod);
        }
        
        saveRequest.setPayrollRunPeriod(payrollRunPeriod);
        saveRequest.setPaycheckRequests(paycheckRequests);
        saveRequest.setPayrollRunType(PayrollRunTypeEnum.SCHEDULED);
        
        return saveRequest;     
    }
    
    private PayrollRunSaveRequest createPayrollRunSaveRequest(Long[] workerIds, Date periodEndDate,
    				PayrollFrequencyEnum frequency, double hourlyHours, double overtimeHours) throws Exception {
    	return createPayrollRunSaveRequest(workerIds, periodEndDate, frequency, hourlyHours, overtimeHours,
    	                                   mBonusAmount);
    }
    
	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker, withholding of Single 1 containing Ytd values with 2
	 * hourly compensation item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a second hourly compensation item</LI>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithholdSingle1WithYtd_2HourlyItems() throws Exception {
		String hourlyWageDescStr2 = "Hourly2";
        Worker worker = entityService.findById(Worker.class, sWorkerId);
        // Add a second hourly item
        worker = WorkerTestUtils.addHourlyWage(worker, hourlyWageDescStr2, false);
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.BIWEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.BIWEEKLY,
    	                                                                          40.0, 4.0);
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, 40.0, mOvertimeRate, 4.0,
		                                                     "1000.00");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("1860.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("1359.45"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("400.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("800.00"),
		                                     new BigDecimal(mHourlyRate), 40.0),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("60.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
		                                     getCalculatedOvertimeRate(), 4.0),
 		                new PaycheckLineItem(hourlyWageDescStr2, new BigDecimal("400.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, hourlyWageDescStr2,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("800.00"),
		                                     new BigDecimal(mHourlyRate), 40.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("2000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("115.32"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("230.64")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("115.32"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("230.64")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("26.97"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("53.94")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("26.97"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("53.94")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("271.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("542.00")),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("14.88"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("29.76")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("72.38"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("144.76")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("14.88"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("29.76")),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("63.24"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("126.48")),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("1.86"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("3.72")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker, withholding of Single 1 containing Ytd values with 2
	 * hourly and 2 overtime item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a second hourly compensation item</LI>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithholdSingle1WithYtd_2Hourly2OvertimeItems() throws Exception {
		String hourlyWageDescStr2 = "Hourly2";
		String overtimeWageDescStr2 = "Hourly2 overtime";
        Worker worker = entityService.findById(Worker.class, sWorkerId);
        // Add a second hourly item
        worker = WorkerTestUtils.addHourlyWage(worker, hourlyWageDescStr2, true);
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.BIWEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.BIWEEKLY,
    	                                                                          40.0, 4.0);
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, 40.0, mOvertimeRate, 4.0,
		                                                     "1000.00");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("1920.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("1393.80"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("400.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("800.00"),
		                                     new BigDecimal(mHourlyRate), 40.0),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("60.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
		                                     getCalculatedOvertimeRate(), 4.0),
 		                new PaycheckLineItem(hourlyWageDescStr2, new BigDecimal("400.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, hourlyWageDescStr2,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("800.00"),
		                                     new BigDecimal(mHourlyRate), 40.0),
 		                new PaycheckLineItem(overtimeWageDescStr2, new BigDecimal("60.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, overtimeWageDescStr2,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
		                                     getCalculatedOvertimeRate(), 4.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("2000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("238.08")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("238.08")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("55.68")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("55.68")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("286.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("572.00")),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("15.36"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("30.72")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("77.96"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("155.92")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("15.36"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("30.72")),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("65.28"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("130.56")),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("1.92"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("3.84")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

}
