package com.intuit.sb.payroll.qa.services.workerpayments;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Date;

import org.testng.AssertJUnit;

import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.services.workerpayments.CalcInputs;
import com.intuit.sb.payroll.services.workerpayments.PaycheckRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunPeriod;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSaveRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSummary;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;

/**
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */

public class CommitPayrollRunExceptionsTest extends WorkerPaymentServiceTestBase {
    private static Long sReadyForPayWorkerId = null;
    private static Long sPayrollRunId = null;
	private static int sNumberOfPayrollReadyWorkers = 1;

    @BeforeClass
    public static void setUpCommitPayrollRunTest() {
        try {
            getUserTransaction().begin();
            sReadyForPayWorkerId = createMVPayrollReadyWorker("Paul");
            //Create a dummy Id for testing purpose
            sPayrollRunId = new Long("932746783");
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in setUpCommitPayrollRunTest(): " + t.getMessage());
        }
    }

    @AfterClass
    public static void tearDownCommitPayrollRunTest() {
        try {
            getUserTransaction().begin();
            if (null != sReadyForPayWorkerId) {
                cleanUpEIWorker(sReadyForPayWorkerId);
                sReadyForPayWorkerId = null;
            }
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in tearDownCommitPayrollRunTest(): " + t.getMessage());
        }
    }

    /**
     * <P>Test ValidationException caused by null PayrollRunSaveRequest object</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's commitPayrollRun() with a null PayrollRunSaveRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown since the PayrollRunSaveRequest object is null</LI>
     * </UL>
     */
    @Test(expectedExceptions=ServiceFailureException.class)
    public void testNullPayrollRunSaveRequest() throws Exception {
        sWorkerPayService.commitPayrollRun(null);
    }

    /**
     * <P>Test ValidationException caused by null PaycheckRequest array inside the PayrollRunSaveRequest object</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's commitPayrollRun() with a null PaycheckRequest array inside the PayrollRunSaveRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown since the PaycheckRequest array inside the PayrollRunSaveRequest object is null</LI>
     * </UL>
     */
    @Test(expectedExceptions=ServiceFailureException.class)
    public void testNullPaycheckRequestInPayrollRunSaveRequest() throws Exception {
        // Setup
    	Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
    	workerIds[0] = sReadyForPayWorkerId;
        PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,sPayrollRunId);
        payrollRunSaveRequest.setPaycheckRequests(null);
        // Exercise
        sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
    }

    /**
     * <P>Test commitPayrollRun() API with a null workerIds in PaycheckRequest in PayrollRunSaveRequest object</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's commitPayrollRun() with a null workerIds in PaycheckRequest in PayrollRunSaveRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown since the workerIds are null in the WorkerWageItems in PayrollRunSaveRequest</LI>
     * </UL>
     */
    @Test(expectedExceptions=ValidationException.class)
    public void testNullWorkerIdInPaycheckRequest() throws Exception {
		//Setup
	    Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
    	workerIds[0] = sReadyForPayWorkerId;
		PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,sPayrollRunId);
		PaycheckRequest[] paycheckRequests = payrollRunSaveRequest.getPaycheckRequests();
		for(int count = 0;count < paycheckRequests.length;count++)
		{
			//Set workerIds to null in all the PaycheckRequest objects
			paycheckRequests[count].setWorkerId(null);
		}
		payrollRunSaveRequest.setPaycheckRequests(paycheckRequests);

		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
    }

    /**
     * <P>Test commitPayrollRun() API with a null WageItems in PaycheckRequest in PayrollRunSaveRequest object</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's commitPayrollRun() with a null WageItems in PaycheckRequest in PayrollRunSaveRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ServiceFailureException is thrown since the WageItems are null in the WorkerWageItems in PayrollRunSaveRequest</LI>
     * </UL>
     */
    @Test(expectedExceptions=ServiceFailureException.class)
    public void testNullCalcInputsInPaycheckRequest() throws Exception {
		//Setup
	    Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
    	workerIds[0] = sReadyForPayWorkerId;
		PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,sPayrollRunId);
		PaycheckRequest[] paycheckRequests = payrollRunSaveRequest.getPaycheckRequests();
		for(int count = 0;count < paycheckRequests.length;count++)
		{
			//Set workerIds to null in all the PaycheckRequest objects
			paycheckRequests[count].setCalcInputs(null);
		}
		payrollRunSaveRequest.setPaycheckRequests(paycheckRequests);

		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
    }

    /**
     * <P>Test commitPayrollRun() API with a null ObjectHandle in CalcInputs in PayrollRunSaveRequest object</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's commitDraftPayrollRun() with a null CompensationType in WageItems in PayrollRunSaveRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ServiceFailureException is thrown since the ObjectHandle in CalcInputs is null in PayrollRunSaveRequest</LI>
     * </UL>
     */
    @Test(expectedExceptions=ServiceFailureException.class)
    public void testNullPayItemObjectHandleInCalcInputs() throws Exception {
		//Setup
	    Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
    	workerIds[0] = sReadyForPayWorkerId;
		PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,null);
		PaycheckRequest[] paycheckRequests = payrollRunSaveRequest.getPaycheckRequests();
		for(int count = 0;count < paycheckRequests.length;count++) {
			CalcInputs[] calcInputs = paycheckRequests[count].getCalcInputs();

			//Set wageItemDefinitionUniqueId in wageItems to null
			for(int wageItemCounter = 0;wageItemCounter < calcInputs.length;wageItemCounter++) {
				calcInputs[wageItemCounter].setPayItemObjectHandle(null);
			}
		}
		payrollRunSaveRequest.setPaycheckRequests(paycheckRequests);

		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
    }

    /**
     * <P>Test commitPayrollRun() API with a null rate in CalcInputs in PayrollRunSaveRequest object</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's commitPayrollRun() with a null rate in CalcInputs in PayrollRunSaveRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that {@link ValidationException} is thrown since the rate in CalcInputs is null in PayrollRunSaveRequest</LI>
     * </UL>
     */
    @Test(expectedExceptions=ValidationException.class)
    public void testNullRateInCalcInputs() throws Exception {
		//Setup
	    Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
    	workerIds[0] = sReadyForPayWorkerId;
		PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,null);
		PaycheckRequest[] paycheckRequests = payrollRunSaveRequest.getPaycheckRequests();
		for(int count = 0;count < paycheckRequests.length;count++)
		{
			CalcInputs[] calcInputs = paycheckRequests[count].getCalcInputs();

			//Set rate in wageItems to null
			for(int wageItemCounter = 0;wageItemCounter < calcInputs.length;wageItemCounter++)
			{
				calcInputs[wageItemCounter].setRate(null);
			}
		}
		payrollRunSaveRequest.setPaycheckRequests(paycheckRequests);

		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
    }

    /**
     * <P>Test commitPayrollRun() API with a negative units in CalcInputs in PayrollRunSaveRequest object</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's commitPayrollRun() with a negative units in CalcInputs in PayrollRunSaveRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown since the units in CalcInputs is negative in PayrollRunSaveRequest</LI>
     * </UL>
     */
    @Test(expectedExceptions=ValidationException.class)
    public void testNegativeUnitsInCalcInputs() throws Exception {
		//Setup
	    Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
    	workerIds[0] = sReadyForPayWorkerId;
		PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,null);
		PaycheckRequest[] paycheckRequests = payrollRunSaveRequest.getPaycheckRequests();
		for(int count = 0;count < paycheckRequests.length;count++) {
			CalcInputs[] calcInputs = paycheckRequests[count].getCalcInputs();

			//Set units to negative
			for(int wageItemCounter = 0;wageItemCounter < calcInputs.length;wageItemCounter++) {
				calcInputs[wageItemCounter].setUnits(-40.00);
			}
		}
		payrollRunSaveRequest.setPaycheckRequests(paycheckRequests);

		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
    }

    /**
     * <P>Test commitPayrollRun() API with a negative rate in CalcInputs in PayrollRunSaveRequest object</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's commitPayrollRun() with a negative rate in CalcInputs in PayrollRunSaveRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown since the rate in CalcInputs is negative in PayrollRunSaveRequest</LI>
     * </UL>
     */
    @Test(expectedExceptions=ValidationException.class)
    public void testNegativeRateInWageItems() throws Exception {
		//Setup
	    Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
    	workerIds[0] = sReadyForPayWorkerId;
		PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,null);
		PaycheckRequest[] paycheckRequests = payrollRunSaveRequest.getPaycheckRequests();
		for(int count = 0;count < paycheckRequests.length;count++) {
			CalcInputs[] calcInputs = paycheckRequests[count].getCalcInputs();

			//Set rate in wageItems to a negative value
			for(int wageItemCounter = 0;wageItemCounter < calcInputs.length;wageItemCounter++) {
				calcInputs[wageItemCounter].setRate(new BigDecimal(-40.00));
			}
		}
		payrollRunSaveRequest.setPaycheckRequests(paycheckRequests);

		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
    }

    /**
     * <P>Test commitPayrollRun() API with payPeriodStartDate after payPeriodEndDate in PayrollRunSaveRequest object</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's commitPayrollRun() with payPeriodStartDate after payPeriodEndDate in PayrollRunSaveRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown since the payPeriodStartDate after payPeriodEndDate in PayrollRunSaveRequest</LI>
     * </UL>
     */
    @Test
    public void testPeriodStartDateAfterPeriodEndDate() throws Exception {
		try {
			//Setup
		    Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
        	workerIds[0] = sReadyForPayWorkerId;
			PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,null);
			PayrollRunPeriod payrollRunPeriod = payrollRunSaveRequest.getPayrollRunPeriod();
			Date periodEndDate = payrollRunPeriod.getPeriodEndDate();
			Calendar periodStartDate = DateUtil.getNowCalendar();
			periodStartDate.setTime(periodEndDate);

			//Set periodStartDate to 1 day after periodEndDate
			periodStartDate.add(Calendar.DAY_OF_MONTH, 1);
			payrollRunPeriod.setPeriodStartDate(periodStartDate.getTime());

			payrollRunSaveRequest.setPayrollRunPeriod(payrollRunPeriod);

			//Exercise
			sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
			AssertJUnit.fail("ValidationException not thrown");
		} catch (ValidationException e) {
			AssertJUnit.assertTrue(e.getMessage().contains("Invalid PayrollRunPeriod"));
		}
    }

    /**
     * <P>Test commitPayrollRun() API with payDate before payPeriodEndDate in PayrollRunSaveRequest object</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's commitPayrollRun() with payDate before payPeriodEndDate in PayrollRunSaveRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown since the payDate before payPeriodEndDate in PayrollRunSaveRequest</LI>
     * </UL>
     */
    @Test
    public void testPayDateBeforePayrollRunPeriodEndDate() throws Exception {
		try {
			//Setup
		    Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
        	workerIds[0] = sReadyForPayWorkerId;
			PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,null);
			PayrollRunPeriod payrollRunPeriod = payrollRunSaveRequest.getPayrollRunPeriod();
			Date periodEndDate = payrollRunPeriod.getPeriodEndDate();
			Calendar payDate = DateUtil.getNowCalendar();
			payDate.setTime(periodEndDate);

			//Set periodStartDate to 1 day before periodEndDate
			payDate.add(Calendar.DAY_OF_MONTH, -1);
			payrollRunPeriod.setPayDate(payDate.getTime());

			payrollRunSaveRequest.setPayrollRunPeriod(payrollRunPeriod);

			//Exercise
			sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
			AssertJUnit.fail("ValidationException not thrown");
		} catch (ValidationException e) {
			AssertJUnit.assertTrue(e.getMessage().contains("Invalid PayrollRunPeriod"));
		}
    }

    /**
     * <P>Test commitPayrollRun() API with payDate before payrollRunDate in PayrollRunSaveRequest object</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's commitPayrollRun() with payDate before payrollRunDate in PayrollRunSaveRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown since the payDate before payrollRunDate in PayrollRunSaveRequest</LI>
     * </UL>
     */
    @Test
    public void testPayDateBeforePayrollRunDate() throws Exception {
		try {
			//Setup
		    Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
        	workerIds[0] = sReadyForPayWorkerId;
			PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,null);
			PayrollRunPeriod payrollRunPeriod = payrollRunSaveRequest.getPayrollRunPeriod();
			Date payrollRunDate = payrollRunPeriod.getPayrollRunDate();
			Calendar payDate = DateUtil.getNowCalendar();
			payDate.setTime(payrollRunDate);

			//Set periodStartDate to 1 day before payrollRunDate
			payDate.add(Calendar.DAY_OF_MONTH, -1);
			payrollRunPeriod.setPayDate(payDate.getTime());

			payrollRunSaveRequest.setPayrollRunPeriod(payrollRunPeriod);

			//Exercise
			sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
			AssertJUnit.fail("ValidationException not thrown");
		} catch (ValidationException e) {
			AssertJUnit.assertTrue(e.getMessage().contains("Invalid PayrollRunPeriod"));
		}
    }

    /**
     * <P>Test commitPayrollRun() API with payDate before periodStartDate in PayrollRunSaveRequest object</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's commitPayrollRun() with payDate before periodStartDate in PayrollRunSaveRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown since the payDate before periodStartDate in PayrollRunSaveRequest</LI>
     * </UL>
     */
    @Test
    public void testPayDateBeforePeriodStartDate() throws Exception {
		try {
			//Setup
		    Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
        	workerIds[0] = sReadyForPayWorkerId;
			PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,null);
			PayrollRunPeriod payrollRunPeriod = payrollRunSaveRequest.getPayrollRunPeriod();
			Date periodStartDate = payrollRunPeriod.getPeriodStartDate();
			Calendar payDate = DateUtil.getNowCalendar();
			payDate.setTime(periodStartDate);

			//Set periodStartDate to 1 day before periodStartDate
			payDate.add(Calendar.DAY_OF_MONTH, -1);
			payrollRunPeriod.setPayDate(payDate.getTime());

			payrollRunSaveRequest.setPayrollRunPeriod(payrollRunPeriod);

			//Exercise
			sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
			AssertJUnit.fail("ValidationException not thrown");
		} catch (ValidationException e) {
			AssertJUnit.assertTrue(e.getMessage().contains("Invalid PayrollRunPeriod"));
		}
    }

    /**
     * <P>Test commitPayrollRun() API with periodStartDate set to null in PayrollRunSaveRequest object</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's commitPayrollRun() with periodStartDate set to null in PayrollRunSaveRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown since the periodStartDate is null in PayrollRunSaveRequest</LI>
     * </UL>
     */
    @Test
    public void testNullPeriodStartDate() throws Exception {
		try {
			//Setup
		    Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
        	workerIds[0] = sReadyForPayWorkerId;
			PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,null);
			PayrollRunPeriod payrollRunPeriod = payrollRunSaveRequest.getPayrollRunPeriod();

			//Set start date to null
			payrollRunPeriod.setPeriodStartDate(null);

			payrollRunSaveRequest.setPayrollRunPeriod(payrollRunPeriod);

			//Exercise
			sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
			AssertJUnit.fail("ValidationException not thrown");
		} catch (ValidationException e) {
			AssertJUnit.assertTrue(e.getMessage().contains("periodStartDate is null"));
		}
    }

    /**
     * <P>Test commitPayrollRun() API with periodEndDate set to null in PayrollRunSaveRequest object</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's commitPayrollRun() with periodEndDate set to null in PayrollRunSaveRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown since the periodEndDate is null in PayrollRunSaveRequest</LI>
     * </UL>
     */
    @Test
    public void testNullPeriodEndDate() throws Exception {
		try {
			//Setup
		    Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
        	workerIds[0] = sReadyForPayWorkerId;
			PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,null);
			PayrollRunPeriod payrollRunPeriod = payrollRunSaveRequest.getPayrollRunPeriod();

			//Set end date to null
			payrollRunPeriod.setPeriodEndDate(null);

			payrollRunSaveRequest.setPayrollRunPeriod(payrollRunPeriod);

			//Exercise
			sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
			AssertJUnit.fail("ValidationException not thrown");
		} catch (ValidationException e) {
			AssertJUnit.assertTrue(e.getMessage().contains("periodEndDate is null"));
		}
    }

    /**
     * <P>Test commitPayrollRun() API with payDate set to null in PayrollRunSaveRequest object</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's commitPayrollRun() with payDate set to null in PayrollRunSaveRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown since the payDate is null in PayrollRunSaveRequest</LI>
     * </UL>
     */
    @Test
    public void testNullPayDate() throws Exception {
		try {
			//Setup
		    Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
        	workerIds[0] = sReadyForPayWorkerId;
			PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,null);
			PayrollRunPeriod payrollRunPeriod = payrollRunSaveRequest.getPayrollRunPeriod();

			//Set payDate to null
			payrollRunPeriod.setPayDate(null);

			payrollRunSaveRequest.setPayrollRunPeriod(payrollRunPeriod);

			//Exercise
			sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
			AssertJUnit.fail("ValidationException not thrown");
		} catch (ValidationException e) {
			AssertJUnit.assertTrue(e.getMessage().contains("paydate is null"));
		}
    }

    /**
     * <P>Test commitPayrollRun() API with payrollRunDate set to null in PayrollRunSaveRequest object</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's commitPayrollRun() with payrollRunDate set to null in PayrollRunSaveRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown since the payrollRunDate is null in PayrollRunSaveRequest</LI>
     * </UL>
     */
    @Test
    public void testNullPeriodRunDate() throws Exception {
		try {
			//Setup
		    Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
        	workerIds[0] = sReadyForPayWorkerId;
			PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,null);
			PayrollRunPeriod payrollRunPeriod = payrollRunSaveRequest.getPayrollRunPeriod();

			//Set payrollRun date to null
			payrollRunPeriod.setPayrollRunDate(null);

			payrollRunSaveRequest.setPayrollRunPeriod(payrollRunPeriod);

			//Exercise
			sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
			AssertJUnit.fail("ValidationException not thrown");
		} catch (ValidationException e) {
			AssertJUnit.assertTrue(e.getMessage().contains("runDate is null"));
		}
    }

    /**
     * <P>Test commitPayrollRun() API with terminationDate set to null in PayrollRunSaveRequest object and
     *  PayrollRunType set to TERMINATION</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's commitPayrollRun() with terminationDate set to null in PayrollRunSaveRequest
     *  object and PayrollRunType set to TERMINATION</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown since the terminationDate is null in PayrollRunSaveRequest</LI>
     * </UL>
     */
    @Test(expectedExceptions=ValidationException.class)
    public void testNullTerminationDate() throws Exception {
		//Setup
	    Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
    	workerIds[0] = sReadyForPayWorkerId;
		PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,null);
		PayrollRunPeriod payrollRunPeriod = payrollRunSaveRequest.getPayrollRunPeriod();

		payrollRunSaveRequest.setPayrollRunPeriod(payrollRunPeriod);
		//Set termination date to null
		payrollRunSaveRequest.setPayrollRunType(PayrollRunTypeEnum.TERMINATION);
		payrollRunSaveRequest.setTerminationDate(null);

		//Exercise
		sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
    }

    /**
     * <P>Test commitDraftPayrollRun() API to update an existing payroll run</P>
     *
     * <BR/><BR/><BR/>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's commitDraftPayrollRun() with a PayrollRunSaveRequest object</LI>
     * <LI>Modify the wage item rate and try to commit again</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the ValidationException is thrown since the payroll run is already committed</LI>
     * </UL>
     */
    @Test(enabled=false, expectedExceptions=ValidationException.class)
    public void testTryUpdatingAlreadyCommitedPayrollRun() throws Exception {
        PayrollRunSummary payrollSummary = null;
    	try {
        	//Setup - Create a new payrollRun
    	    Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
        	workerIds[0] = sReadyForPayWorkerId;
        	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,null);
        	payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();

        	//Update the committed payroll run
        	payrollRunSaveRequest.setPayrollRunId(payrollSummary.getPayrollRunId());
        	PaycheckRequest[] newPaycheckRequests = payrollRunSaveRequest.getPaycheckRequests();

    		for(int count = 0;count < newPaycheckRequests.length;count++) {
    			CalcInputs[] calcInputs = newPaycheckRequests[count].getCalcInputs();

    			//Set rate in wageItems to some new value
    			for(int wageItemCounter = 0;wageItemCounter < calcInputs.length;wageItemCounter++) {
    				calcInputs[wageItemCounter].setRate(new BigDecimal(67.00));
    			}
    		}
        	payrollRunSaveRequest.setPaycheckRequests(newPaycheckRequests);

        	//Exercise
        	sWorkerPayService.commitPayrollRun(payrollRunSaveRequest);
        	AssertJUnit.fail("ValidationException is not thrown");

    	} finally {
            if(payrollSummary != null) {
                cleanUpPayrollRunItems(new Long(payrollSummary.getPayrollRunId()));
            }
        }
    }
}
