package com.intuit.sb.payroll.qa.services.workerpayments;


import java.util.Date;

import org.testng.AssertJUnit;

import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationUtils;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunInfo;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSaveRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSummary;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;

/**
* Tests the getAllPayrollRuns operation of the WorkerPaymentService.
*
* $Author$
* $Revision$
* $DateTime$
*
*/

public class GetAllPayrollRunsTest extends WorkerPaymentServiceTestBase {
	private static Long sReadyForPayWorkerId = null;
	private static Long sReadyForPayWorkerId2 = null;
	private static int sNumberOfPayrollReadyWorkers = 2;
	private String mHourlyRate = "10.00";
	private String mOvertimeRate = "1.5";
	private String mBonusAmount = "1000.00";
	private double mHourlyHours = 80.0;
	private double mOvertimeHours = 8.0;
	
	@BeforeClass
	public static void setUpGetAllPayrollRunsTest() {
        try {
            getUserTransaction().begin();
            sReadyForPayWorkerId = createMVPayrollReadyWorker("Paul");
            sReadyForPayWorkerId2 = createMVPayrollReadyWorker("Pauline");
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in setUpGetAllPayrollRunsTest(): " + t.getMessage());
        }
	}
	
	@AfterClass
	public static void tearDownGetAllPayrollRunsTest() {
        try {
            getUserTransaction().begin();
            if (null != sReadyForPayWorkerId) {
                cleanUpEIWorker(sReadyForPayWorkerId);
                sReadyForPayWorkerId = null;
            }
            if (null != sReadyForPayWorkerId2) {
                cleanUpEIWorker(sReadyForPayWorkerId2);
                sReadyForPayWorkerId2 = null;
            }
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            }  catch (Exception e) {
            }
            AssertJUnit.fail("Error in tearDownGetAllPayrollRunsTest(): " + t.getMessage());
        }
	}
	
	private PayrollRunSummary createAPayrollRun(Date periodEndDate) throws Exception {
		Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
		workerIds[0] = sReadyForPayWorkerId;
		workerIds[1] = sReadyForPayWorkerId2;
		// Setup wage items for the first worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sReadyForPayWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
		                                                     mOvertimeHours, mBonusAmount);
		sWageService.saveCompensationItems(sReadyForPayWorkerId,
		                 CompensationUtils.mapToSaveCompensationItemRequest(SaveCompensationItemOperation.UPDATE,
		                                                                    updatedWageItems));
		// Setup wage items for the second worker
		CompensationItem[] wageItems2 = sWageService.getCompensationItems(sReadyForPayWorkerId2);
		CompensationItem[] updatedWageItems2 = setupWageItems(wageItems2, mHourlyRate, 72.0, mOvertimeRate,
		                                                      7.0, mBonusAmount);
		sWageService.saveCompensationItems(sReadyForPayWorkerId2,
		                 CompensationUtils.mapToSaveCompensationItemRequest(SaveCompensationItemOperation.UPDATE,
		                                                                    updatedWageItems2));
		PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, null, periodEndDate);

        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
		PayrollRunSummary payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
		return payrollSummary;
	}

    /**
     * <P>Test getting all PayrollRuns</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a couple of PayrollRuns</LI>
     * <LI>Call WorkerPaymentService's getAllPayrollRuns() to retrieve all PayrollRuns</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that all PayrollRunResponse objects are correct</LI>
     * </UL>
     */
	@Test
	public void testGetAllPayrollRuns() throws Exception {
		PayrollRunSummary payrollRunSummary1 = null;
		PayrollRunSummary payrollRunSummary2 = null;
		try {
			// Setup
			Date periodEndDate1 = DateUtil.createDate(2008, 5, 1);
			payrollRunSummary1 = createAPayrollRun(periodEndDate1);
			Date periodEndDate2 = DateUtil.getNextDate(periodEndDate1, 14);
			payrollRunSummary2 = createAPayrollRun(periodEndDate2);
			
			//Exercise
			PayrollRunInfo[] payrollRunInfos = sWorkerPayService.getAllPayrollRuns();
			//Verify
			PayrollRunInfo[] expectedInfos = new PayrollRunInfo[2];
			expectedInfos[0] = mapToPayrollRunInfo(payrollRunSummary1);
			expectedInfos[1] = mapToPayrollRunInfo(payrollRunSummary2);
			AssertJUnit.assertTrue("PayrollRunResponses mismatch!", verifyPayrollRunInfos(expectedInfos,
			                                                                         payrollRunInfos));
		} finally {
	        //Tear down - Implicit tear down in tearDownClass()
			if (payrollRunSummary1 != null) {
		    	cleanUpPayrollRunItems(new Long(payrollRunSummary1.getPayrollRunId()));
			}
			if (payrollRunSummary2 != null) {
		    	cleanUpPayrollRunItems(new Long(payrollRunSummary2.getPayrollRunId()));
			}
		}
	}

    /**
     * <P>Test getting all PayrollRuns with no existing PayrollRuns</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's getAllPayrollRuns() to retrieve all PayrollRuns</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo[] is empty</LI>
     * </UL>
     */
	@Test
	public void testGetAllPayrollRunsWithNoPayrollRuns() throws Exception {
		//Exercise
		PayrollRunInfo[] payrollRunInfos = sWorkerPayService.getAllPayrollRuns();
		
		//Verify
		AssertJUnit.assertEquals("PayrollRunInfo[] is not empty!", 0, payrollRunInfos.length);
	}

}
