/**
 * Copyright: 2007, Intuit, Inc. All rights reserved.
 *
 */
package com.intuit.sb.payroll.qa.task;

import java.util.ArrayList;
import java.util.Date;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.task.Task;
import com.intuit.sb.payroll.task.TaskServiceTestBase;
import com.intuit.sb.payroll.task.TaskStatus;
import com.intuit.sb.payroll.task.TaskType;


public class TaskServiceNegativeTest extends TaskServiceTestBase
{
	// COMMENT: amalik
	// None of these tests validates if the task has been persisted properly.
	//  1. We need to save a task and then retrieve it back through the service and validate if all data is same
	// 2. Check if when we complete a task, completion date is there
	// 3. when a task get re-open, completion date should be gone
	// 4. Test DateRange not being the same for runPayrollTask after we have run Payroll for the DateRange



    //Pass a null task to cancelTask() and check
    @Test(expectedExceptions=ServiceFailureException.class)
    public void testCancelNullTask() throws ServiceFailureException {
    	Task task = null;
    	taskService.cancelTask(task);
    }

    //Pass a null task to markTaskComplete() and check
    @Test(expectedExceptions=ServiceFailureException.class)
    public void testCompleteNullTask()throws ServiceFailureException {
    	Task task = null;
    	taskService.markTaskComplete(task);

    }

    //create a task , set a null payrollRunPeriod and then save it as Scheduled task
    @Test
    public void testSaveScheduledTaskWithNullSchedule(){
    	try{
    	Task task = new Task();
    	task.setName("Tast Scheduled Task");
    	task.setType(TaskType.SCHEDULED);
    	task.setPayrollRunPeriod(null);
    	task.setDescription("Sample scheduled task");
    	Date dueDate = DateUtil.createDate(2008, 12, 25);	//25th Dec 2008
		task.setDueDate(dueDate);
    	Task taskFromDb = taskService.createTask(task);

    	AssertJUnit.assertNotNull(taskFromDb);
    	AssertJUnit.assertNotNull(taskFromDb.getObjectHandle());
    	AssertJUnit.assertNotNull(taskFromDb.getDueDate());

    	}
    	catch(ServiceFailureException sfe){
    		AssertJUnit.fail("ServiceFailureException is not expected");
    	}

    }

    //Create a task give it a name, and then create another task with the same name
    public void testCreateDuplicateTask(){
    	try
    	{
    		Date dueDate = DateUtil.createDate(2008, 12, 25);	//25th Dec 2008
    		Task task = createNonScheduledTask(dueDate);
	    	Task taskFromDb = taskService.createTask(task);
	    	AssertJUnit.assertNotNull(taskFromDb);
	    	AssertJUnit.assertNotNull(taskFromDb.getObjectHandle());
	    	AssertJUnit.assertNotNull(taskFromDb.getDueDate());

	    	dueDate = DateUtil.createDate(2008, 12, 25);	//25th Dec 2008
    		task = createNonScheduledTask(dueDate);
	    	Task task2 = taskService.createTask(task);
	    	AssertJUnit.assertNotNull(task2);
	    	AssertJUnit.assertNotNull(task2.getObjectHandle());
	    	AssertJUnit.assertNotNull(task2.getDueDate());

	    	Date today = new Date();
	    	dueDate = DateUtil.createDate(2009, 12, 25);	//25th Dec 2009
	    	Task[] tasks = taskService.getTasks(today, dueDate);
	    	AssertJUnit.assertTrue(tasks.length==2);



    	}
    	catch(ServiceFailureException sfe)
    	{
    		AssertJUnit.fail("ServiceFailureException is not expected");
    	}

    }

    //create tasks mark them complete or cancel then try to getActiveTasks
    @Test(enabled=false, groups={"broken"})
    public void testGetActiveTasksWithNoActiveTasks() throws Exception {
		//first create at least one task in the system
		Date dueDate = DateUtil.createDate(2010, 12, 01);	//25th Dec 2008
		Task task = createNonScheduledTask(dueDate);
		Task task1FromDb = taskService.createTask(task);

    	dueDate = DateUtil.createDate(2009, 5, 25);	//25th Dec 2008
		task = createNonScheduledTask(dueDate);
    	Task task2FromDb = taskService.createTask(task);

    	//now close the first task
    	taskService.markTaskComplete(task2FromDb);
    	taskService.cancelTask(task1FromDb);

    	//now get the active tasks, we should get no active task
    	Date today = new Date();
    	dueDate = DateUtil.createDate(2009, 12, 25);	//25th Dec 2009

    	Task[] activeTasks = taskService.getActiveTasks(today, dueDate);
    	activeTasks = removeSetupTasks(activeTasks);
    	AssertJUnit.assertTrue(activeTasks.length == 0);

    	//the total number of tasks in the system will still be 2
    	Task[] tasks = taskService.getTasks(today, dueDate);

    	AssertJUnit.assertTrue(tasks.length == 2);
    }


    @Test
    public void testGetTasksWithPeriodOutOfActualRange() throws Exception {
		//first create at least one task in the system
		Date dueDate = DateUtil.createDate(2008, 12, 25);	//25th Dec 2008
		Task task = createNonScheduledTask(dueDate);
		Task task1FromDb = taskService.createTask(task);

    	dueDate = DateUtil.createDate(2009, 5, 25);	//25th Dec 2008
		task = createNonScheduledTask(dueDate);
    	Task task2FromDb = taskService.createTask(task);

    	//now close the first task
    	taskService.markTaskComplete(task2FromDb);
    	taskService.cancelTask(task1FromDb);

    	//now get the active tasks, we should get only one active task
    	Date toDate = DateUtil.createDate(2010, 12, 25);
    	Date fromDate = DateUtil.createDate(2012, 12, 25);	//25th Dec 2009

    	//the total number of tasks in the system will be 0 in this period
    	Task[] tasks = taskService.getTasks(toDate, fromDate);

    	AssertJUnit.assertTrue(tasks.length == 0);
    }

    @Test
    public void testGetTasksWithNullPeriodRange() throws Exception {
		//first create at least one task in the system
		Date dueDate = DateUtil.createDate(2008, 12, 25);	//25th Dec 2008
		Task task = createNonScheduledTask(dueDate);
		Task task1FromDb = taskService.createTask(task);

    	dueDate = DateUtil.createDate(2009, 5, 25);	//25th Dec 2008
		task = createNonScheduledTask(dueDate);
    	Task task2FromDb = taskService.createTask(task);

    	//now close the first task
    	taskService.markTaskComplete(task2FromDb);
    	taskService.cancelTask(task1FromDb);

    	//the total number of tasks in the system will be 0 in this period
    	Task[] tasks = taskService.getTasks(null,null);

    	AssertJUnit.assertTrue(tasks.length == 0);
    }

    @Test
    public void testGetActiveTasksWithPeriodOutOfActualRange() throws Exception {
		//first create at least one task in the system
		Date dueDate = DateUtil.createDate(2008, 12, 25);	//25th Dec 2008
		Task task = createNonScheduledTask(dueDate);
		Task task1FromDb = taskService.createTask(task);

    	dueDate = DateUtil.createDate(2009, 5, 25);	//25th Dec 2008
		task = createNonScheduledTask(dueDate);
    	Task task2FromDb = taskService.createTask(task);

    	//now close the first task
    	taskService.markTaskComplete(task2FromDb);
    	taskService.cancelTask(task1FromDb);

    	//now get the active tasks, we should get only one active task
    	Date toDate = DateUtil.createDate(1907, 12, 25);
    	Date fromDate = DateUtil.createDate(1906, 12, 25);	//25th Dec 2009

    	Task[] activeTasks = taskService.getActiveTasks(toDate, fromDate);
    	activeTasks = removeSetupTasks(activeTasks);
    	AssertJUnit.assertTrue(activeTasks.length == 0);
    }

    //With null range of period GetActiveTasks gets the tasks of the current Employer, is this OK?
    //@Test
    public void testGetActiveTasksWithNullPeriodRange() throws Exception {
		//first create at least one task in the system
		Date dueDate = DateUtil.createDate(2008, 12, 25);	//25th Dec 2008
		Task task = createNonScheduledTask(dueDate);
		Task task1FromDb = taskService.createTask(task);

    	dueDate = DateUtil.createDate(2009, 5, 25);	//25th Dec 2008
		task = createNonScheduledTask(dueDate);
    	Task task2FromDb = taskService.createTask(task);

    	//now close the first task
    	taskService.markTaskComplete(task2FromDb);
    	taskService.cancelTask(task1FromDb);

    	//the total number of tasks in the system will be 0 in this period
    	Task[] tasks = taskService.getActiveTasks(null,null);

    	AssertJUnit.assertTrue(tasks.length == 0);
    }


    @Test(enabled=false, groups={"broken"})
    public void testGetOverdueTask() throws Exception {
		//first create at least one task in the system
		Task task = createNonScheduledTask(null);
    	taskService.createTask(task);

    	task = createNonScheduledTask(null);
		taskService.createTask(task);

    	Task[] overDueTasks = taskService.getOverdueTasks();
    	AssertJUnit.assertTrue(overDueTasks.length == 0);
    }

    @Test(enabled=false, groups={"broken"})
    public void testSetDueDateOfCanceledTask() throws Exception {
		//first create at least one task in the system
		Date dueDate = DateUtil.createDate(2008, 12, 25);	//25th Dec 2008
		Task task = createNonScheduledTask(dueDate);
    	Task task1Frondb = taskService.createTask(task);

    	//now cancel the task
    	Task cancelledTask = taskService.cancelTask(task1Frondb);

    	//now get the task back and see the status of the task
    	TaskStatus status = cancelledTask.getstatus();
    	AssertJUnit.assertEquals(TaskStatus.CANCELLED, status); //check the task status as canceled


    	//now get the active tasks, we should get only one active task
    	Date fromDate = DateUtil.createDate(2007, 12, 25);	//25th Dec 2007
    	Date toDate = DateUtil.createDate(2009, 12, 25);	//25th Dec 2009

    	Task[] tasks = taskService.getTasks(fromDate, toDate);
    	tasks = removeSetupTasks(tasks);
    	AssertJUnit.assertTrue(tasks.length == 1);

    	Date newDueDate = DateUtil.createDate(2009,1,3);
    	tasks[0].setDueDate(newDueDate);

    	//Update the Canceled Task
    }

    @Test(enabled=false, groups={"broken"})
    public void testSetDueDateOfCompletedTaskTask(){

    }

    @Test(enabled=false, groups={"broken"})
    public void testUpdateCanceledTaskToCompletedTask(){

    }

    @Test(enabled=false, groups={"broken"})
    public void testUpdateCompletedTaskToCanceledTask(){

    }

    @Test(enabled=false, groups={"broken"})
    public void testUpdateScheduledTaskToNonScheduledTask(){

    }

    @Test(enabled=false, groups={"broken"})
    public void testUpdateNonScheduledTaskToScheduledTask(){

    }

    // Remove SetUp related tasks with no due date
    private Task[] removeSetupTasks(Task[] tasks) {
    	ArrayList<Task> taskList = new ArrayList<Task>();
    	for (Task task : tasks) {
			if ((task.getType().equals(TaskType.SETUP)) &&
				(task.getDueDate() == null)){
				continue;
			}else{
				taskList.add(task);
			}
		}
    	Task[] tmpTasks = new Task[taskList.size()];
    	return taskList.toArray(tmpTasks);
	}



}