/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.SimpleQueryValidator.java
 * 
 * Created: Jul 11, 2011 3:05:10 PM
 * Author: gkesler
 */
package com.intuit.sb.services.query.validator;

import com.intuit.sb.services.query.ASTUtils;
import com.intuit.sb.services.query.Holder;
import com.intuit.sb.services.query.parser.*;

import com.intuit.sb.services.query.validator.context.PathBuilder;
import java.util.Map;

import com.intuit.sb.services.query.optimizer.NotOptimizer;
import com.intuit.sb.services.query.validator.context.ContextFactory;
import com.intuit.sb.services.query.validator.context.DefaultContextFactory;
import com.intuit.sb.services.query.validator.context.Property;
import com.intuit.sb.services.query.validator.context.RootContext;


/**
 *
 * @author gkesler
 */
public class SimpleQueryValidator implements SimpleQueryParserVisitor, SimpleQueryParserTreeConstants {
	
	private ValidatorMemento validatorMemento = null;
	
    /**
     * 
     * @param parser
     * @param properties
     * @param contextFactory 
     */
    public void validate (SimpleQueryParser parser, Map<String, Object> properties, ContextFactory contextFactory) {
    	validatorMemento = new ValidatorMemento(contextFactory, properties);
        parser.getRootNode().jjtAccept(this, validatorMemento);
    }
    
    /**
     * @return the rootcontext of this queryvalidator 
     */
    public RootContext getRootContext() {
    	return validatorMemento.getRootContext();
    }
    
    /**
     * 
     * @param parser
     * @param properties 
     */
    public void validate (SimpleQueryParser parser, Map<String, Object> properties) {
        validate(parser, properties, new DefaultContextFactory());
    }
    
    public Object visit(SimpleNode node, Object data) {
        // no-op
        return data;
    }
    
    public Object visit(ASTSelectStatement node, Object data) {
        // start from resetting root context
        final ValidatorMemento memento = (ValidatorMemento)data;
        
        // depth-first visiting descendants
        node.childrenAccept(this, memento);
        // finish processing deferred constructs
        memento.visitPendedNodes(this);
        
        // set the memento object to indicate the AST
        // has been validated
        node.jjtSetValue(memento.getDefaultContext());
        
        return data;
    }

    public Object visit(ASTSelectClause node, Object data) {
        return node.childrenAccept(this, data);
    }

    public Object visit(ASTFromClause node, Object data) {
        return node.childrenAccept(this, data);
    }

    public Object visit(ASTWhereClause node, Object data) {
        return node.childrenAccept(this, data);
    }

    public Object visit(ASTOrderByClause node, Object data) {      	
    	return node.childrenAccept(this, data);
    }

    public Object visit(ASTPaginationClause node, Object data) {
        return node.childrenAccept(this, data);
    }

    public Object visit(ASTAnyPathElement node, Object data) {
        final PathBuilder pathBuilder = (PathBuilder)data;
        
        pathBuilder.addAnyPathElement();
        
        return pathBuilder;
    }

    public Object visit(ASTIdPathElement node, Object data) {
        final PathBuilder pathBuilder = (PathBuilder)data;
        
        pathBuilder.addIdPathElement((String)node.jjtGetValue());
        
        return pathBuilder;
    }
    
    public Object visit(ASTStringPathElement node, Object data) {
        final PathBuilder pathBuilder = (PathBuilder)data;
        
        pathBuilder.addNamePathElement((String)node.jjtGetValue());
        
        return pathBuilder;
    }
    
    public Object visit(ASTPathElement node, Object data) {
        final PathBuilder pathBuilder = (PathBuilder)data;
        
        pathBuilder.addPathElement((String)node.jjtGetValue());
        
        return pathBuilder;
    }
    
    public Object visit(ASTPathExpression node, Object data) {
        final ValidatorMemento memento = (ValidatorMemento)data;
        // validating path expressions against root context for now
        // FIXME: relative path expressions are not yet supported                
        final PathBuilder pathBuilder = memento.createPathBuilderDecorator();
        // cascade pathBuilder to the descendants of this path expression
        node.childrenAccept(this, pathBuilder);
        
        // the pathBuilder contains the path, finish evaluating
        final Property property = (Property)pathBuilder.evaluate(node);
        node.jjtSetValue(property);
        memento.setValue(property);
        
        return data;
    }

    public Object visit(ASTContextDeclaration node, Object data) {
        try {
            final ValidatorMemento memento = (ValidatorMemento)data;
            // prepare context to use
            node.childrenAccept(this, memento);
            
            // create aliased context out of data collected in children
            final RootContext context = 
                    memento.addAliasedContext(memento.getContextAlias(), memento.getContextName());
            node.jjtSetValue(context);
            
            return data;
        } catch (ValidatorException e) {
            e.setProblemNode(node);
            e.fillInStackTrace();
            throw e;
        }
    }

    public Object visit(ASTContextName node, Object data) {
        final ContextDecl contextDecl = (ContextDecl)data;
        
        contextDecl.setContextName((String)node.jjtGetValue());
        
        return data;
    }

    public Object visit(ASTAliasName node, Object data) {
        final ContextDecl contextDecl = (ContextDecl)data;
        
        contextDecl.setContextAlias((String)node.jjtGetValue());
        
        return data;
    }

    public Object visit(ASTOr node, Object data) {
        return node.childrenAccept(this, data);
    }

    public Object visit(ASTAnd node, Object data) {
        return node.childrenAccept(this, data);
    }

    public Object visit(ASTNot node, Object data) {
        // if our only child node is JJTNOT or relational node,
        // correct the tree.
        NotOptimizer.getInstance().optimize(node, data);
        
        // after that, cascade to the children
        return node.childrenAccept(this, data);
    }

    public Object visit(ASTLike node, Object data) {
        // binary operation: LIKE
        return validateLike(node, data);
    }

    public Object visit(ASTContains node, Object data) {
        // binary operation: CONTAINS
        return validateContains(node, data);
    }

    public Object visit(ASTDistance node, Object data) {
        final Property property = (Property)data;
        final RootContext rootContext = property.getRootContext();
        // visitAllNodes that the value is in the valid range
        node.childrenAccept(this, new Boundaries<Float>(
                (Float)rootContext.getContextProperty(ContextFactory.DISTANCE_MIN_LIMIT),
                (Float)rootContext.getContextProperty(ContextFactory.DISTANCE_MAX_LIMIT),
                distanceDefaults));
        return data;
    }
    
    public Object visit(ASTIn node, Object data) {
        // visitAllNodes that all items in the in set are
        // of the compatible types and they are all together
        // compatible with the path expression
        return validateIn(node, data);
    }

    public Object visit(ASTBetween node, Object data) {
        // ternary operation: BETWEEN
        return validateBetween(node, data);
    }

    public Object visit(ASTEq node, Object data) {
        // binary operation: EQ
        return validateBinary(node, data);
    }

    public Object visit(ASTNEq node, Object data) {
        // binary operation: NEQ
        return validateBinary(node, data);
    }

    public Object visit(ASTLt node, Object data) {
        // binary operation: LT
        return validateBinary(node, data);
    }

    public Object visit(ASTLte node, Object data) {
        // binary operation: LTE
        return validateBinary(node, data);
    }

    public Object visit(ASTGt node, Object data) {
        // binary operation: GT
        return validateBinary(node, data);
    }

    public Object visit(ASTGte node, Object data) {
        // binary operation: GTE
        return validateBinary(node, data);
    }

    public Object visit(ASTAsc node, Object data) {
        node.childrenAccept(this, data);
        return validateSortable(node, (Holder<Property>)data);        
    }

    public Object visit(ASTDesc node, Object data) {
        node.childrenAccept(this, data);
        return validateSortable(node, (Holder<Property>)data);       
    }

    public Object visit(ASTStartPosition node, Object data) {
        // visitAllNodes that the value is in the valid range
        node.childrenAccept(this, new Boundaries<Integer>(limitDefaults));
        return data;
    }

    public Object visit(ASTMaxResults node, Object data) {
        final ValidatorMemento memento = (ValidatorMemento)data;
        // visitAllNodes that the value is in the valid range
        node.childrenAccept(this, new Boundaries<Integer>(
                (Integer)memento.getContextProperty(ContextFactory.PAGE_MIN_LIMIT),
                (Integer)memento.getContextProperty(ContextFactory.PAGE_MAX_LIMIT),
                limitDefaults));
        return data;
    }

    public Object visit(ASTIterator node, Object data) {
        return node.childrenAccept(this, data);
    }

    public Object visit(ASTCurrentDate node, Object data) {
        return validateValue(node, (Holder<Property>)data);
    }

    public Object visit(ASTCurrentTimestamp node, Object data) {
        return validateValue(node, (Holder<Property>)data);
    }

    public Object visit(ASTCurrentTime node, Object data) {
        return validateValue(node, (Holder<Property>)data);
    }

    public Object visit(ASTDateTime node, Object data) {
        return validateValue(node, (Holder<Property>)data);
    }

    public Object visit(ASTString node, Object data) {
        return validateValue(node, (Holder<Property>)data);
    }

    public Object visit(ASTTrue node, Object data) {
        return validateValue(node, (Holder<Property>)data);
    }

    public Object visit(ASTFalse node, Object data) {
        return validateValue(node, (Holder<Property>)data);
    }

    public Object visit(ASTMe node, Object data) {
        final Holder<Property> propertyHolder = (Holder<Property>)data;
        final RootContext rootContext = propertyHolder.getValue().getRootContext();
        
        node.jjtSetValue(rootContext.getContextProperty(ContextFactory.CONTEXT_USER_ID));
        
        return validateValue(node, propertyHolder);
    }

    public Object visit(ASTMyApp node, Object data) {
        final Holder<Property> propertyHolder = (Holder<Property>)data;
        final RootContext rootContext = propertyHolder.getValue().getRootContext();
        
        node.jjtSetValue(rootContext.getContextProperty(ContextFactory.CONTEXT_APP_ID));
        
        return validateValue(node, propertyHolder);
    }

    public Object visit(ASTNull node, Object data) {
        return validateValue(node, (Holder<Property>)data);
    }
    
    public Object visit(ASTId node, Object data) {
        return validateValue(node, (Holder<Property>)data);
    }

    public Object visit(ASTInteger node, Object data) {
        return validateLimits(node, (Boundaries<Integer>)data);
    }

    public Object visit(ASTFloat node, Object data) {
        return validateLimits(node, (Boundaries<Float>)data);
    }

    public Object visit(ASTUUID node, Object data) {
        return node.jjtGetValue();
    }
    
    private final Object validateOperation (SimpleNode opNode, Object data) {
        ASTUtils.trace("validating: " + opNode);
        opNode.childrenAccept(this, data);
        
        return Boolean.TRUE;
    }
    
    private final Object validateBinary (SimpleNode binaryNode, Object data) {
        return validateOperation(binaryNode, data);
    }
    
    private final  Object validateIn (ASTIn inNode, Object data) {
        return validateOperation(inNode, data);
    }
    
    private final Object validateLike (ASTLike likeNode, Object data) {
        return validateOperation(likeNode, data);
    }
    
    private final Object validateContains (ASTContains containsNode, Object data) {
        return validateOperation(containsNode, data);
    }
    
    private final Object validateBetween (ASTBetween betweenNode, Object data) {
        return validateOperation(betweenNode, data);
    }
    
    private final Object validateValue (SimpleNode valueNode, Holder<Property> propertyHolder) {
        final Property property = propertyHolder.getValue();
        final Object value = valueNode.jjtGetValue();
        
        ASTUtils.trace("validating: property=" + property.getName() + ", type=" + property.getType() + ", value=" + value);
        if (!property.isQueryable()) {
            throw new ValidatorException("property '" + property.getName() + "' is not queryable").setProblemNode(valueNode);
        }
        if (!property.validateValue(value)) {
            throw new ValidatorException("value " + value + " is not valid for property '" + property.getName() + "'").setProblemNode(valueNode);
        }
        
        return Boolean.TRUE;
    }
    
    private final Object validateSortable (SimpleNode valueNode, Holder<Property> propertyHolder) {
        final Property property = propertyHolder.getValue();
               
        ASTUtils.trace("validating: property=" + property.getName() + ", type=" + property.getType() );
        if (!property.isSortable()) {
            throw new ValidatorException("property '" + property.getName() + "' is not sortable").setProblemNode(valueNode);
        }
        
        return Boolean.TRUE;
    }
    
    private final <T extends Comparable> Object validateLimits (SimpleNode node, Boundaries<T> limits) {
        final T value = (T)node.jjtGetValue();
        
        if (value.compareTo(limits.lower) < 0) {
            throw new ValidatorException("value " + value + " is too small. Min allowed value is " + limits.lower).setProblemNode(node);
        }
        if (value.compareTo(limits.upper) > 0) {
            throw new ValidatorException("value " + value + " is too large. Max allowed value is " + limits.upper).setProblemNode(node);
        }
        
        return Boolean.TRUE;
    }
    
    private static final Boundaries.Defaults<Integer> limitDefaults = 
            new Boundaries.Defaults<Integer>() {
        public Integer minValue() {
            return Integer.valueOf(0);
        }

        public Integer maxValue() {
            return Integer.valueOf(Integer.MAX_VALUE);
        }
    };
    
    private static final Boundaries.Defaults<Float> distanceDefaults = 
            new Boundaries.Defaults<Float>() {
        public Float minValue() {
            return Float.valueOf(0);
        }

        public Float maxValue() {
            return Float.valueOf(1);
        }
    };
    
    public Object visit(ASTCount node, Object data) {
        return node.childrenAccept(this, data);
    }
    
    
}
