/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.SingleValueProperty.java
 * 
 * Created: Jul 12, 2011 5:51:16 PM
 * Author: gkesler
 */
package com.intuit.sb.services.query.validator.context;

import java.util.Collection;
import java.util.Collections;

import com.intuit.sb.services.query.validator.ValidatorException;
import com.intuit.sb.services.query.validator.ValidatorHelper;

/**
 *
 * @author gkesler
 */
public class SingleValueProperty implements Property {
    protected final Context parent;
    protected final String propertyName;
    protected final Class propertyType;
    protected final boolean queryable;
    protected final boolean sortable;
    
    public SingleValueProperty (Context parent, String propertyName, Class propertyType, boolean queryable, boolean sortable) {
        this.parent = parent;
        this.propertyName = propertyName;
        this.propertyType = propertyType;
        this.queryable = queryable;
        this.sortable = sortable;
    }
    
    public Class getType() {
        return propertyType;
    }

    public boolean validateValue(Object value) {
        // I want NullPointerException to be thrown in case the value is null
        // This way I can catch it on the caller's level and throw ValidatorException
        // since null value is obviously is illegal here
        return value == null 
                || propertyType.isInstance(value)
                || ValidatorHelper.getValueValidator(propertyType).validate(value);
    }

    public String getName() {
        return propertyName;
    }

    public Context[] getFullPath() {
        final Context[] parentPath = parent.getFullPath();
        final Context[] path = new Context[parentPath.length + 1];
        
        System.arraycopy(parentPath, 0, path, 0, parentPath.length);
        path[parentPath.length] = this;
        
        return path;
    }

    public RootContext getRootContext() {
        return parent.getRootContext();
    }

    public Context getParentContext() {
        return parent;
    }

    public PathBuilder createPathBuilder() {
        throw generateException("path");
    }

    public Property lookupProperty(String name) {
        throw generateException(name);
    }

    public Property lookupIdProperty(String idValue) {
        throw generateException(idValue);
    }

    public Property lookupNameProperty(String nameValue) {
        throw generateException(nameValue);
    }

    public Collection<Property> getAllProperties() {
        return Collections.emptyList();
    }

    public Collection<Property> getProperties() {
        return getAllProperties();
    }

    public void setPropertyFilter(PropertyFilter filter) {
        throw generateException("filter");
    }
                
    protected ValidatorException generateException (String expression) {
        return new ValidatorException("not valid expression: " + expression + " for single-value context:" + propertyName);
    }

    public <T> T accept(PropertyVisitor<T> visitor, T data) {
        return visitor.visit(this, data);
    }

    @Override
    public String toString() {
        return getClass().getSimpleName() + "{" + 
                "parent=" + (parent != null ? parent.getName() : null) + 
                ", propertyName=" + propertyName + 
                ", propertyType=" + propertyType + 
                ", properties=" + getAllProperties() +
                '}';
    }

    public boolean isQueryable() {
        return queryable;
    }

    public MetadataProvider getMetadataProvider() {
        return parent.getMetadataProvider();
    }

	public boolean isSortable() {
		return sortable;
	}
}
