/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.datasync.query.ws.rs.uri.UriTemplate.java
 * 
 * Created: Dec 2, 2011 8:47:57 AM
 * Author: gkesler
 */
package com.intuit.sb.datasync.query.ws.rs.utils;

import java.io.StringReader;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Stack;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.apache.log4j.Logger;

/**
 *
 * @author gkesler
 */
public class UriTemplate {
    private static final Logger logger = Logger.getLogger(UriTemplate.class);
    
    private UriTemplate (String s) throws ParseException {
        this.parser = new UriTemplateParser(new StringReader(s));
        
        // parse
        parser.parse();
        // build regex
        parser.getRootNode().jjtAccept(generator, this);
        // and compile
        this.pattern = Pattern.compile(template.toString());
    }
    
    public static UriTemplate compile (String template) throws ParseException {
        return new UriTemplate(template);
    }
    
    UriTemplateParser getParser () {
        return parser;
    }
    
    Pattern getPatern () {
        return pattern;
    }
    
    List<Rule> getRules () {
        return rules;
    }

    public Match matches (CharSequence s) {
        Matcher matcher = pattern.matcher(s);
        if (matcher.matches()) {
            for (Rule rule: rules) {
                if (matcher.group(rule.ruleGroup) != null) {
                    logger.debug("match:" + rule);
                    return new Match(matcher, rule);
                }
            }
        }
        
        logger.debug("match not found:" + s);
        return null;
    }

    @Override
    public String toString() {
        return "UriTemplate{" + "pattern=" + pattern + ", rules=" + rules + '}';
    }
    
    static class Rule {
        final int ruleGroup;
        final Map<String, Integer> groups = new HashMap<String, Integer>();
        
        Rule (int ruleGroup) {
            this.ruleGroup = ruleGroup;
        }

        @Override
        public String toString() {
            return "Rule{" + "ruleGroup=" + ruleGroup + ", groups=" + groups + '}';
        }
    }
    
    public static class Match {
        final Matcher matcher;
        final Rule rule;
        
        Match (Matcher matcher, Rule rule) {
            this.matcher = matcher;
            this.rule = rule;
        }
        
        public String group (String name) {
            Integer group = rule.groups.get(name);
            return group != null ? matcher.group(group.intValue()) : null;
        }
        
        public String ruleGroup () {
            return matcher.group(rule.ruleGroup);
        }
    }
    
    private final Pattern pattern;
    private final Stack<Rule> rules = new Stack<Rule>();
    private final StringBuilder template = new StringBuilder();
    private BuildStrategy buildStrategy = singleRule;
    private int group = -1;
    
    private final UriTemplateParser parser;
    private static final UriTemplateParserVisitor generator = new UriTemplateParserVisitor() {
        public Object visit(SimpleNode node, Object data) {
            throw new UnsupportedOperationException("Not supported yet.");
        }

        public Object visit(ASTTemplate node, Object data) {
            return node.childrenAccept(this, data);
        }

        public Object visit(ASTOr node, Object data) {
            UriTemplate outer = toUriTemplate(data);
            outer.buildStrategy = multipleRules;
            return node.childrenAccept(this, data);
        }

        public Object visit(ASTRule node, Object data) {
            return addRule(node, data);
        }
        
        public Object visit(ASTSegment node, Object data) {
            return addSegment(node, data);
        }

        public Object visit(ASTOptSegment node, Object data) {
            append(data, '(');
            addSegment(node, data);
            append(data, ")?");
            
            return data;
        }

        public Object visit(ASTParam node, Object data) {
            append(data, '(');
            node.childrenAccept(this, data);
            append(data, ')');
            
            return data;
        }

        public Object visit(ASTRegEx node, Object data) {
            append(data, (String)node.jjtGetValue());
            
            return data;
        }

        public Object visit(ASTName node, Object data) {
            addGroup(data, (String)node.jjtGetValue());
            
            return data;
        }

        public Object visit(ASTString node, Object data) {
            append(data, (String)node.jjtGetValue());
            
            return data;
        }
        
        private Object addSegment (SimpleNode node, Object data) {
            append(data, '/');
            return node.childrenAccept(this, data);
        }
        
        private Object addRule (ASTRule node, Object data) {
            UriTemplate outer = toUriTemplate(data);            
            outer.buildStrategy.startRule(outer);
            node.childrenAccept(this, data);
            outer.buildStrategy.endRule(outer);
            
            return data;
        }

        private void addGroup (Object data, String name) {
            UriTemplate outer = toUriTemplate(data);
            Rule rule = outer.rules.peek();
            rule.groups.put(name, Integer.valueOf(++outer.group));
        }

        private void append (Object data, String s) {
            UriTemplate outer = toUriTemplate(data);            
            outer.buildStrategy.append(outer, s);
        }

        private void append (Object data, char c) {
            UriTemplate outer = toUriTemplate(data);            
            outer.buildStrategy.append(outer, c);
        }
        
        private UriTemplate toUriTemplate (Object data) {
            return (UriTemplate)data;
        }
    };
    
    private static abstract class BuildStrategy {
        void startRule (UriTemplate outer) {
            int groupNo = outer.group;
            
            if (groupNo < 0) {
                groupNo = firstGroupNo();
            } else {
                concatRules(outer);
                ++groupNo;
            }
            outer.rules.push(new Rule(groupNo));
            
            outer.group = groupNo;
        }
        abstract void endRule (UriTemplate outer);
        abstract void concatRules (UriTemplate outer);
        abstract int firstGroupNo ();
        void append (UriTemplate outer, String s) {
            outer.template.append(s);
        }
        void append (UriTemplate outer, char c) {
            outer.template.append(c);
        }
    }
    
    private static final BuildStrategy singleRule = new BuildStrategy() {
        public void endRule(UriTemplate outer) {
        }

        public int firstGroupNo() {
            return 0;
        }

        void concatRules(UriTemplate outer) {
        }
    };
    
    private static final BuildStrategy multipleRules = new BuildStrategy() {
        @Override
        public void startRule(UriTemplate outer) {
            super.startRule(outer);
            append(outer, '(');
        }

        public void endRule(UriTemplate outer) {
            append(outer, ')');
        }

        public int firstGroupNo() {
            return 1;
        }

        void concatRules(UriTemplate outer) {
            append(outer, '|');
        }
    };
}
