package com.intuit.sb.payroll.qa.services.workerpayments;

import java.math.BigDecimal;
import java.util.Date;

import javax.transaction.Status;

import org.testng.AssertJUnit;

import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.DE4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.model.RelationshipToOwnerEnum;
import com.intuit.sb.payroll.entity.model.W4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.workerpayments.CalcInputs;
import com.intuit.sb.payroll.services.workerpayments.PaycheckDetails;
import com.intuit.sb.payroll.services.workerpayments.PaycheckDistributionMethod;
import com.intuit.sb.payroll.services.workerpayments.PaycheckLineItem;
import com.intuit.sb.payroll.services.workerpayments.PaycheckRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollItemCategoryType;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunPeriod;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSaveRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSummary;
import com.intuit.sb.payroll.services.workerpayments.PayrollScheduleFrequency;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;


/**
 * <P>
 * The <CODE>CalcPaycheckDetailsYTDTest</CODE> class contains tests for testing
 * the YTD paycheck calculation functionality of WorkerPaymentService.calcPaycheckDetails()
 * </P>
 * 
 * $Author$ 
 * $Revision$ 
 * $DateTime$
 * 
 */
public class CalcPaycheckDetailsYTDTest extends WorkerPaymentServiceTestBase {
	private static BigDecimal sZeroBigDecimal = new BigDecimal("0.00");
	private static CompensationItem notACompensationItem = null;
	private String mHourlyRate = "10.00";
	private String mOvertimeRate = "1.5";
	private String mBonusAmount = "1000.00";
	private double mHourlyHours = 80.0;
	private double mOvertimeHours = 8.0;
	private double mPieces = 1000.0;

	@BeforeClass
	public static void setUpCalcPaycheckDetailsTest() {
		setUpSanMateoAddress();
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
		sEmployer.setNonProfit(false);
	}

	@BeforeMethod
	public void setUp() throws Exception {
		super.setUp();
		try {
			refreshWorker();
			// Set up wages
			setupWages(sWorkerId);
			// Apply the applicable taxes
			setUpTestData(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED);
			applyApplicableTaxes();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			AssertJUnit.fail("Error in setUp() in CalcPaycheckDetailsTest(): " + t.getMessage());
		}
	}

	@AfterMethod
	public void tearDown() throws Exception {
		if (getUserTransaction().getStatus() == Status.STATUS_ACTIVE) {
			// Remove all withholding information associated with the worker
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.W4IRS);
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.DE4);
		}

		super.tearDown();
	}
	
	private BigDecimal getCalculatedOvertimeRate() {
		return getCalculatedOvertimeRate(mHourlyRate);
	}

	private BigDecimal getCalculatedOvertimeRate(String rate) {
		BigDecimal calculatedRate = new BigDecimal(rate);
		calculatedRate = calculatedRate.multiply(new BigDecimal(mOvertimeRate));
		// Set to 2 decimal places
		calculatedRate = calculatedRate.setScale(2);
		return calculatedRate;
	}

    private PayrollRunSaveRequest createPayrollRunSaveRequest(Long[] workerIds, Date periodEndDate,
    				PayrollFrequencyEnum frequency, double hourlyHours, double overtimeHours,
    				String bonusAmount) throws Exception {
        PayrollRunSaveRequest saveRequest = new PayrollRunSaveRequest();
        PaycheckRequest[] paycheckRequests = new PaycheckRequest[workerIds.length];
        PayrollRunPeriod payrollRunPeriod = createAPayrollRunPeriod(periodEndDate,
                                                                    frequency, sDefaultPayrollDayOffset,
                                                                    sDefaultPayrollRunDayOffset);

        for(int count = 0;count < workerIds.length;count++) {
            paycheckRequests[count] = new PaycheckRequest();
            CompensationItem[] workerCompensationItems = sWageService.getCompensationItems(workerIds[count]);
            for (CompensationItem compensationItem : workerCompensationItems) {
				if (compensationItem.getCompensationType().equals(CompensationType.Hourly)) {
					compensationItem.setUnits(hourlyHours);
				} else if (compensationItem.getCompensationType().equals(CompensationType.Overtime)) {
					compensationItem.setUnits(overtimeHours);
				} else if (compensationItem.getCompensationType().equals(CompensationType.TipsPaycheck) ||
						   compensationItem.getCompensationType().equals(CompensationType.TipsCash) ||
						   compensationItem.getCompensationType().equals(CompensationType.TipsAllocated)) {
					compensationItem.setRate(new BigDecimal(bonusAmount));
				} else if (compensationItem.getCompensationType().equals(CompensationType.Bonus)) {
					compensationItem.setRate(new BigDecimal(bonusAmount));
				} else if (compensationItem.getCompensationType().equals(CompensationType.Commission)) {
					compensationItem.setRate(new BigDecimal(bonusAmount));
				} else if (compensationItem.getCompensationType().equals(CompensationType.Piecework)) {
			        compensationItem.setUnits(mPieces);
				}
			}
			CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(workerCompensationItems);
			paycheckRequests[count].setCalcInputs(calcInputs);
            paycheckRequests[count].setWorkerId(workerIds[count]);
            paycheckRequests[count].setDistributionMethod(PaycheckDistributionMethod.PRINT);
            paycheckRequests[count].setPayrollRunPeriod(payrollRunPeriod);
        }
        
        saveRequest.setPayrollRunPeriod(payrollRunPeriod);
        saveRequest.setPaycheckRequests(paycheckRequests);
        saveRequest.setPayrollRunType(PayrollRunTypeEnum.SCHEDULED);
        
        return saveRequest;     
    }
    
    private PayrollRunSaveRequest createPayrollRunSaveRequest(Long[] workerIds, Date periodEndDate,
    				PayrollFrequencyEnum frequency, double hourlyHours, double overtimeHours) throws Exception {
    	return createPayrollRunSaveRequest(workerIds, periodEndDate, frequency, hourlyHours, overtimeHours,
    	                                   mBonusAmount);
    }
    
    private PayrollRunSummary commitAPayrollRun(PayrollRunSaveRequest payrollRunSaveRequest) throws Exception {
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
    	PayrollRunSummary payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
     	return payrollSummary;
    }
    
	/**
	 * <P>
	 * Test calculating a weekly paycheck for a worker, withholding of Single 1 containing Ytd values
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testWeeklyWithholdSingle1WithYtd() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.WEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.WEEKLY,
    	                                                                          40.0, 4.0);
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, 40.0, mOvertimeRate, 4.0,
		                                                     "1000.00");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("1460.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("983.15"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("400.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("800.00"),
		                                     new BigDecimal(mHourlyRate), 40.0),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("60.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
		                                     getCalculatedOvertimeRate(), 4.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("2000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("90.52"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("181.04")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("90.52"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("181.04")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("21.17"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("42.34")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("21.17"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("42.34")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("268.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("536.00")),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("11.68"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("23.36")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("85.48"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("170.96")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("11.68"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("23.36")),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("49.64"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("99.28")),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("1.46"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("2.92")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker, withholding of Married
	 * 1 for Federal and MarriedWithOneIncome 1 for State containing Ytd values
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithholdMarried1WithYtd() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.MARRIED, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.MARRIEDONEINCOME, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.BIWEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.BIWEEKLY,
    	                                                                          mHourlyHours, mOvertimeHours);
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
		                                                     mOvertimeHours, mBonusAmount);
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getGrossTotal(), new BigDecimal("1920.00"), "Gross total"));
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getNetTotal(), new BigDecimal("1528.14"), "Net total"));

		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("1600.00"),
		                                     new BigDecimal(mHourlyRate), mHourlyHours),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("240.00"),
		                                     getCalculatedOvertimeRate(), mOvertimeHours),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("2000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("238.08")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("238.08")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("55.68")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("55.68")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("192.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("384.00")),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("15.36"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("30.72")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("37.62"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("75.24")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("15.36"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("30.72")),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("65.28"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("130.56")),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("1.92"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("3.84")) };
		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker with a fixed commission item, withholding of Married
	 * 1 for Federal and MarriedWithOneIncome 1 for State containing Ytd values
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a fixed commission compensation item to the worker</LI>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithFixedCommissionWithholdMarried1WithYtd() throws Exception {
        Worker worker = entityService.findById(Worker.class, sWorkerId);
        // Add a fixed rate commission item
        worker = WorkerTestUtils.addCommission(worker, true);
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.MARRIED, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.MARRIEDONEINCOME, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.BIWEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.BIWEEKLY,
    	                                                                          mHourlyHours, mOvertimeHours);
    	PaycheckRequest[] paycheckRequests = payrollRunSaveRequest.getPaycheckRequests();
    	for (PaycheckRequest paycheckRequest : paycheckRequests) {
			CalcInputs[] calcInputs = paycheckRequest.getCalcInputs();
			//Remove commission from the inputs or the override will be respected.
			long id = 0;
			for (CompensationItem compensationItem : wageItems) {
				if (compensationItem.getCompensationType().equals(CompensationType.Commission)) {
					id = compensationItem.getObjectHandle().getId();
				}
			}			
			calcInputs = removeInputFromArray(id, calcInputs);
			paycheckRequest.setCalcInputs(calcInputs);
		}
    	
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);

		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
		                                                     mOvertimeHours, mBonusAmount);
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		//Remove commission from the inputs or the override will be respected.
		long id = 0;
		for (CompensationItem compensationItem : updatedWageItems) {
			if (compensationItem.getCompensationType().equals(CompensationType.Commission)) {
				id = compensationItem.getObjectHandle().getId();
			}
		}			
		calcInputs = removeInputFromArray(id, calcInputs);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getGrossTotal(), new BigDecimal("2420.00"), "Gross total"));
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getNetTotal(), new BigDecimal("1884.49"), "Net total"));

		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("1600.00"),
		                                     new BigDecimal(mHourlyRate), mHourlyHours),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("240.00"),
		                                     getCalculatedOvertimeRate(), mOvertimeHours),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("2000.00"),
		                					 new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sCommissionWageDescStr, new BigDecimal("500.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sCommissionWageDescStr,
		                                                          CompensationType.Commission),
		                                     QboeLawIdConstants.sCommissionLawId, new BigDecimal("1000.00"),
		                                     new BigDecimal("500.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("150.04"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("300.08")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("150.04"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("300.08")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("35.09"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("70.18")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("35.09"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("70.18")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("267.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("534.00")),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("19.36"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("38.72")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("64.02"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("128.04")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("19.36"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("38.72")),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("82.28"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("164.56")),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("2.42"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("4.84")) };
		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker with a varies commission item, withholding of Married
	 * 1 for Federal and MarriedWithOneIncome 1 for State containing Ytd values
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a varies commission compensation item to the worker</LI>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithVariesCommissionWithholdMarried1WithYtd() throws Exception {
        Worker worker = entityService.findById(Worker.class, sWorkerId);
        // Add a varied rate commission item
        worker = WorkerTestUtils.addCommission(worker, false);
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.MARRIED, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.MARRIEDONEINCOME, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.BIWEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.BIWEEKLY,
    	                                                                          mHourlyHours, mOvertimeHours);
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
		                                                     mOvertimeHours, mBonusAmount);
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getGrossTotal(), new BigDecimal("2920.00"), "Gross total"));
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getNetTotal(), new BigDecimal("2235.10"), "Net total"));

		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("1600.00"),
		                                     new BigDecimal(mHourlyRate), mHourlyHours),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("240.00"),
		                                     getCalculatedOvertimeRate(), mOvertimeHours),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("2000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sCommissionWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sCommissionWageDescStr,
		                                                          CompensationType.Commission),
		                                     QboeLawIdConstants.sCommissionLawId, new BigDecimal("2000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("181.04"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("362.08")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("181.04"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("362.08")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("42.34"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("84.68")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("42.34"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("84.68")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("343.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("686.00")),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("23.36"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("46.72")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("95.16"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("190.32")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("23.36"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("46.72")),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("99.28"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("198.56")),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("2.92"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("5.84")) };
		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker with a varies PaycheckTips item, withholding of Married
	 * 1 for Federal and MarriedWithOneIncome 1 for State containing Ytd values
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a varies PaycheckTips compensation item to the worker</LI>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithVariesPaycheckTipsWithholdMarried1WithYtd() throws Exception {
        Worker worker = entityService.findById(Worker.class, sWorkerId);
        // Add a varied rate PaycheckTips item
        worker = WorkerTestUtils.addPaycheckTips(worker, false);
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.MARRIED, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.MARRIEDONEINCOME, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.BIWEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.BIWEEKLY,
    	                                                                          mHourlyHours, mOvertimeHours);
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
		                                                     mOvertimeHours, mBonusAmount);
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getGrossTotal(), new BigDecimal("2920.00"), "Gross total"));
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getNetTotal(), new BigDecimal("2235.10"), "Net total"));

		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("1600.00"),
		                                     new BigDecimal(mHourlyRate), mHourlyHours),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("240.00"),
		                                     getCalculatedOvertimeRate(), mOvertimeHours),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("2000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sPaycheckTipsWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sPaycheckTipsWageDescStr,
		                                                          CompensationType.TipsPaycheck),
		                                     QboeLawIdConstants.sPaycheckTipsLawId, new BigDecimal("2000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("181.04"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("362.08")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("181.04"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("362.08")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("42.34"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("84.68")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("42.34"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("84.68")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("343.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("686.00")),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("23.36"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("46.72")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("95.16"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("190.32")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("23.36"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("46.72")),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("99.28"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("198.56")),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("2.92"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("5.84")) };
		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker with a varies CashTips item, withholding of Married
	 * 1 for Federal and MarriedWithOneIncome 1 for State containing Ytd values
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a varies CashTips compensation item to the worker</LI>
	 * <LI>Add a varies CashTipsOffset compensation item to the worker</LI>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithVariesCashTipsWithholdMarried1WithYtd() throws Exception {
        Worker worker = entityService.findById(Worker.class, sWorkerId);
        // Add a varied rate CashTips item
        worker = WorkerTestUtils.addCashTips(worker, false);
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.MARRIED, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.MARRIEDONEINCOME, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.BIWEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.BIWEEKLY,
    	                                                                          mHourlyHours, mOvertimeHours);
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
		                                                     mOvertimeHours, mBonusAmount);
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getGrossTotal(), new BigDecimal("2920.00"), "Gross total"));
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getNetTotal(), new BigDecimal("1235.10"), "Net total"));

		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("1600.00"),
		                                     new BigDecimal(mHourlyRate), mHourlyHours),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("240.00"),
		                                     getCalculatedOvertimeRate(), mOvertimeHours),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("2000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sCashTipsWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sCashTipsWageDescStr,
		                                                          CompensationType.TipsCash),
		                                     QboeLawIdConstants.sCashTipsLawId, new BigDecimal("2000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
 		                new PaycheckLineItem(sCashTipsOffsetDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.POST_TAX_DEDUCTION, notACompensationItem,
		                                     QboeLawIdConstants.sCashTipsOutLawId, new BigDecimal("2000.00")),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("181.04"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("362.08")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("181.04"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("362.08")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("42.34"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("84.68")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("42.34"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("84.68")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("343.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("686.00")),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("23.36"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("46.72")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("95.16"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("190.32")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("23.36"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("46.72")),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("99.28"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("198.56")),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("2.92"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("5.84")) };
		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker with an AllocatedTips item, withholding of Married
	 * 1 for Federal and MarriedWithOneIncome 1 for State containing Ytd values
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a varies AllocatedTips compensation item to the worker</LI>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithAllocatedTipsWithholdMarried1WithYtd() throws Exception {
        Worker worker = entityService.findById(Worker.class, sWorkerId);
        // Add a varied rate AllocatedTips item
        worker = WorkerTestUtils.addAllocatedTips(worker, false);
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.MARRIED, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.MARRIEDONEINCOME, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.BIWEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.BIWEEKLY,
    	                                                                          mHourlyHours, mOvertimeHours);
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
		                                                     mOvertimeHours, mBonusAmount);
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getGrossTotal(), new BigDecimal("1920.00"), "Gross total"));
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getNetTotal(), new BigDecimal("1528.14"), "Net total"));

		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("1600.00"),
		                                     new BigDecimal(mHourlyRate), mHourlyHours),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("240.00"),
		                                     getCalculatedOvertimeRate(), mOvertimeHours),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("2000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sAllocatedTipsWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sAllocatedTipsWageDescStr,
		                                                          CompensationType.TipsAllocated),
		                                     QboeLawIdConstants.sAllocatedTipsLawId, new BigDecimal("2000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("238.08")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("238.08")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("55.68")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("55.68")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("192.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("384.00")),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("15.36"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("30.72")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("37.62"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("75.24")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("15.36"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("30.72")),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("65.28"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("130.56")),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("1.92"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("3.84")) };
		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker with a piecework item, withholding of Married
	 * 1 for Federal and MarriedWithOneIncome 1 for State containing Ytd values
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a piecework compensation item to the worker</LI>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithPieceworkWithholdMarried1WithYtd() throws Exception {
        Worker worker = entityService.findById(Worker.class, sWorkerId);
        // Add a piecework item
        worker = WorkerTestUtils.addPiecework(worker, false);
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.MARRIED, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.MARRIEDONEINCOME, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.BIWEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.BIWEEKLY,
    	                                                                          mHourlyHours, mOvertimeHours);
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
		                                                     mOvertimeHours, mBonusAmount);
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getGrossTotal(), new BigDecimal("2920.00"), "Gross total"));
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getNetTotal(), new BigDecimal("2235.10"), "Net total"));

		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("1600.00"),
		                                     new BigDecimal(mHourlyRate), mHourlyHours),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("240.00"),
		                                     getCalculatedOvertimeRate(), mOvertimeHours),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("2000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sPieceworkWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sPieceworkWageDescStr,
		                                                          CompensationType.Piecework),
		                                     QboeLawIdConstants.sPieceworkLawId, new BigDecimal("2000.00"),
		                                     new BigDecimal("1.00"), 1000.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("181.04"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("362.08")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("181.04"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("362.08")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("42.34"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("84.68")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("42.34"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("84.68")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("343.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("686.00")),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("23.36"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("46.72")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("95.16"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("190.32")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("23.36"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("46.72")),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("99.28"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("198.56")),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("2.92"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("5.84")) };
		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker, withholding of Married
	 * 1 for Federal and MarriedWithTwoIncome 1 for State containing Ytd values
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithholdMarried1CAMarriedTwoIncome1WithYtd() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.MARRIED, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.BIWEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.BIWEEKLY,
    	                                                                          mHourlyHours, mOvertimeHours);
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
		                                                     mOvertimeHours, mBonusAmount);
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getGrossTotal(), new BigDecimal("1920.00"), "Gross total"));
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getNetTotal(), new BigDecimal("1487.80"), "Net total"));

		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("1600.00"),
		                                     new BigDecimal(mHourlyRate), mHourlyHours),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("240.00"),
		                                     getCalculatedOvertimeRate(), mOvertimeHours),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("2000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("238.08")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("238.08")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("55.68")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("55.68")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("192.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("384.00")),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("15.36"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("30.72")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("77.96"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("155.92")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("15.36"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("30.72")),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("65.28"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("130.56")),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("1.92"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("3.84")) };
		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker, withholding of Single
	 * 1 for Federal and HeadOfHousehold 1 for State containing Ytd values
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithholdSingle1CAHeadOfHousehold1WithYtd() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.HEADOFHOUSEHOLD, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.BIWEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.BIWEEKLY,
    	                                                                          mHourlyHours, mOvertimeHours);
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
		                                                     mOvertimeHours, mBonusAmount);
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getGrossTotal(), new BigDecimal("1920.00"), "Gross total"));
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getNetTotal(), new BigDecimal("1438.66"), "Net total"));

		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("1600.00"),
		                                     new BigDecimal(mHourlyRate), mHourlyHours),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("240.00"),
		                                     getCalculatedOvertimeRate(), mOvertimeHours),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("2000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("238.08")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("238.08")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("55.68")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("55.68")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("286.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("572.00")),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("15.36"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("30.72")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("33.10"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("66.20")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("15.36"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("30.72")),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("65.28"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("130.56")),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("1.92"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("3.84")) };
		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker, withholding of Married
	 * using Single 1 for Federal and Single 1 for State containing Ytd values
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithholdMarriedUsingSingle1CASingle1WithYtd() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.MARRIEDSINGLERATE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.BIWEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.BIWEEKLY,
    	                                                                          mHourlyHours, mOvertimeHours);
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
		                                                     mOvertimeHours, mBonusAmount);
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);

		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getGrossTotal(), new BigDecimal("1920.00"), "Gross total"));
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getNetTotal(), new BigDecimal("1393.80"), "Net total"));

		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("1600.00"),
		                                     new BigDecimal(mHourlyRate), mHourlyHours),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("240.00"),
		                                     getCalculatedOvertimeRate(), mOvertimeHours),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("2000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("238.08")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("238.08")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("55.68")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("55.68")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("286.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("572.00")),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("15.36"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("30.72")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("77.96"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("155.92")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("15.36"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("30.72")),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("65.28"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("130.56")),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("1.92"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("3.84")) };
		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a Biweekly paycheck for a worker, withholding exempt for
	 * Federal and exempt for State containing Ytd values
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithholdExemptWithYtd() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.BIWEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.BIWEEKLY,
    	                                                                          mHourlyHours, mOvertimeHours);
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
		                                                     mOvertimeHours, mBonusAmount);
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);

		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getGrossTotal(), new BigDecimal("1920.00"), "Gross total"));
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getNetTotal(), new BigDecimal("1757.76"), "Net total"));

		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("1600.00"),
		                                     new BigDecimal(mHourlyRate), mHourlyHours),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("240.00"),
		                                     getCalculatedOvertimeRate(), mOvertimeHours),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("2000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("238.08")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("238.08")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("55.68")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("55.68")),
		                new PaycheckLineItem(sFICA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, sZeroBigDecimal),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("15.36"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("30.72")),
		                new PaycheckLineItem(sCAPIT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, sZeroBigDecimal),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("15.36"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("30.72")),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("65.28"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("130.56")),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("1.92"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("3.84")) };
		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a semi-monthly paycheck for a worker, withholding of
	 * Single 1 containing Ytd values
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testSemimonthlyWithholdSingle1WithYtd() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.SEMIMONTHLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.SEMIMONTHLY,
    	                                                                          mHourlyHours, mOvertimeHours);
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
		                                                     mOvertimeHours, mBonusAmount);
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);

		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getGrossTotal(), new BigDecimal("1920.00"), "Gross total"));
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getNetTotal(), new BigDecimal("1416.97"), "Net total"));

		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("1600.00"),
		                                     new BigDecimal(mHourlyRate), mHourlyHours),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("240.00"),
		                                     getCalculatedOvertimeRate(), mOvertimeHours),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("2000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("238.08")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("238.08")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("55.68")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("55.68")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("270.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("540.00")),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("15.36"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("30.72")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("70.79"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("141.58")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("15.36"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("30.72")),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("65.28"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("130.56")),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("1.92"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("3.84")) };
		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a monthly paycheck for a worker, withholding of Single 1 containing Ytd values
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testMonthlyWithholdSingle1WithYtd() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.MONTHLY);

		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.MONTHLY,
    	                                                                          mHourlyHours, mOvertimeHours);
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
		                                                  mOvertimeHours, mBonusAmount);
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);

		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		AssertJUnit.assertEquals("Gross total", new BigDecimal("1920.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("1554.18"), paycheckDetails.getNetTotal());

		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("1600.00"),
		                                     new BigDecimal(mHourlyRate), mHourlyHours),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("240.00"),
		                                     getCalculatedOvertimeRate(), mOvertimeHours),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("2000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("238.08")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("238.08")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("55.68")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("55.68")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("179.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("358.00")),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("15.36"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("30.72")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("24.58"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("49.16")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("15.36"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("30.72")),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("65.28"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("130.56")),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("1.92"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("3.84")) };
		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a weekly paycheck for a worker with Bonus wage item
	 * only, withholding of Single 1 containing Ytd values
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBonusOnlyWeeklyWithholdSingle1() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.WEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.WEEKLY,
    	                                                                          0.0, 0.0, "100.00");
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, 0.0, mOvertimeRate, 0.0,
		                                                     "100.00");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);

		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getGrossTotal(), new BigDecimal("100.00"), "Gross total"));
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getNetTotal(), new BigDecimal("91.55"), "Net total"));

		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, sZeroBigDecimal,
		                                     new BigDecimal(mHourlyRate), 0.0),
		                new PaycheckLineItem(sOvertimeWageDescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, sZeroBigDecimal,
		                                     getCalculatedOvertimeRate(), 0.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("100.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("200.00"),
		                                     new BigDecimal("100.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("6.20"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("12.40")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("6.20"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("12.40")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("1.45"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("2.90")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("1.45"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("2.90")),
		                new PaycheckLineItem(sFICA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, sZeroBigDecimal),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("0.80"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("1.60")),
		                new PaycheckLineItem(sCAPIT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, sZeroBigDecimal),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("0.80"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("1.60")),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("3.40"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("6.80")),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("0.10"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("0.20")) };
		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}
	
	/**
	 * <P>
	 * Test calculating a weekly paycheck for a worker, withholding of Single 1 containing Ytd values, limit reached
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testWeeklyWithholdSingle1WithYtd_LimitReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.WEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.WEEKLY,
    	                                                                          0.0, 0.0, "120000");
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, 40.0, mOvertimeRate, 4.0,
		                                                     "1000.00");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("1460.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("1085.35"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("400.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("400.00"),
		                                     new BigDecimal(mHourlyRate), 40.0),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("60.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("60.00"),
		                                     getCalculatedOvertimeRate(), 4.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("121000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("6324.00")),
		                new PaycheckLineItem(sSST_ER_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("6324.00")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("21.17"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("1761.17")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("21.17"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("1761.17")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("268.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("41820.00")),
		                new PaycheckLineItem(sFUTA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("85.48"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("12202.20")),
		                new PaycheckLineItem(sCASDI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("693.58")),
		                new PaycheckLineItem(sCASUI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a weekly paycheck for a worker, withholding of Single 1 containing Ytd values, $7000 limit for
	 * FUTA, CASUI and CAETT already reached.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testWeeklyWithholdSingle1WithYtd_FUTA_CASUI_CAETT_LimitAlreadyReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.WEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.WEEKLY,
    	                                                                          0.0, 0.0, "7000");
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, 40.0, mOvertimeRate, 4.0,
		                                                     "1000.00");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("1460.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("983.15"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("400.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("400.00"),
		                                     new BigDecimal(mHourlyRate), 40.0),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("60.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("60.00"),
		                                     getCalculatedOvertimeRate(), 4.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("8000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("90.52"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("524.52")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("90.52"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("524.52")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("21.17"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("122.67")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("21.17"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("122.67")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("268.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("2270.00")),
		                new PaycheckLineItem(sFUTA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("85.48"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("686.18")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("11.68"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("67.68")),
		                new PaycheckLineItem(sCASUI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a weekly paycheck for a worker, withholding of Single 1 containing Ytd values, $7000 limit for
	 * FUTA, CASUI and CAETT just reached in current paycheck.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testWeeklyWithholdSingle1WithYtd_FUTA_CASUI_CAETT_LimitJustReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.WEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.WEEKLY,
    	                                                                          0.0, 0.0, "6500");
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, 40.0, mOvertimeRate, 4.0,
		                                                     "1000.00");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("1460.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("983.15"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("400.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("400.00"),
		                                     new BigDecimal(mHourlyRate), 40.0),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("60.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("60.00"),
		                                     getCalculatedOvertimeRate(), 4.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("7500.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("90.52"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("493.52")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("90.52"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("493.52")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("21.17"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("115.42")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("21.17"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("115.42")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("268.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("2104.00")),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("4.00"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("85.48"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("639.68")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("11.68"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("63.68")),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("17.00"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("0.50"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a weekly paycheck for a worker, withholding of Single 1 containing Ytd values, $102,000 limit
	 * for Social Security just reached in current paycheck.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testWeeklyWithholdSingle1WithYtd_SocialSecurity_LimitJustReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.WEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.WEEKLY,
    	                                                                          0.0, 0.0, "100000");
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, 40.0, mOvertimeRate, 4.0,
		                                                     "2000.00");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("2460.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("1577.85"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("400.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("400.00"),
		                                     new BigDecimal(mHourlyRate), 40.0),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("60.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("60.00"),
		                                     getCalculatedOvertimeRate(), 4.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("2000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("102000.00"),
		                                     new BigDecimal("2000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("124.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("6324.00")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("124.00"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("6324.00")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("35.67"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("1485.67")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("35.67"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("1485.67")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("544.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("35096.00")),
		                new PaycheckLineItem(sFUTA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("178.48"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("10235.20")),
		                new PaycheckLineItem(sCASDI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("693.58")),
		                new PaycheckLineItem(sCASUI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a weekly paycheck for a worker, withholding of Single 1 containing Ytd values, $86,698 limit
	 * for CASDI just reached in current paycheck.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testWeeklyWithholdSingle1WithYtd_CASDI_LimitJustReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.WEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.WEEKLY,
    	                                                                          0.0, 0.0, "86000");
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, 40.0, mOvertimeRate, 4.0,
		                                                     "1000.00");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("1460.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("989.25"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("400.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("400.00"),
		                                     new BigDecimal(mHourlyRate), 40.0),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("60.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("60.00"),
		                                     getCalculatedOvertimeRate(), 4.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("87000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("90.52"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("5422.52")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("90.52"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("5422.52")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("21.17"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("1268.17")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("21.17"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("1268.17")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("268.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("29920.00")),
		                new PaycheckLineItem(sFUTA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("85.48"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("8700.20")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("5.58"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("693.58")),
		                new PaycheckLineItem(sCASUI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker, withholding of Single 1 containing Ytd values, $7000 limit for
	 * FUTA, CASUI and CAETT already reached.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithholdSingle1WithYtd_FUTA_CASUI_CAETT_LimitAlreadyReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.BIWEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.BIWEEKLY,
    	                                                                          0.0, 0.0, "7000");
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, 80.0, mOvertimeRate, 8.0,
		                                                     "1000.00");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("1920.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("1393.80"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("800.00"),
		                                     new BigDecimal(mHourlyRate), 80.0),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
		                                     getCalculatedOvertimeRate(), 8.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("8000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("553.04")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("553.04")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("129.34")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("129.34")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("286.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("1979.00")),
		                new PaycheckLineItem(sFUTA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("77.96"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("628.36")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("15.36"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("71.36")),
		                new PaycheckLineItem(sCASUI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker, withholding of Single 1 containing Ytd values, $7000 limit for
	 * FUTA, CASUI and CAETT just reached in current paycheck.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithholdSingle1WithYtd_FUTA_CASUI_CAETT_LimitJustReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.BIWEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.BIWEEKLY,
    	                                                                          0.0, 0.0, "6500");
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, 80.0, mOvertimeRate, 8.0,
		                                                     "1000.00");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("1920.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("1393.80"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("800.00"),
		                                     new BigDecimal(mHourlyRate), 80.0),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
		                                     getCalculatedOvertimeRate(), 8.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("7500.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("522.04")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("522.04")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("122.09")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("122.09")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("286.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("1816.00")),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("4.00"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("77.96"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("581.86")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("15.36"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("67.36")),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("17.00"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("0.50"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker, withholding of Single 1 containing Ytd values, $102,000 limit
	 * for Social Security just reached in current paycheck.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithholdSingle1WithYtd_SocialSecurity_LimitJustReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.BIWEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.BIWEEKLY,
    	                                                                          0.0, 0.0, "100000");
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, 80.0, mOvertimeRate, 8.0,
		                                                     "2000.00");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("2920.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("2046.70"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("800.00"),
		                                     new BigDecimal(mHourlyRate), 80.0),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
		                                     getCalculatedOvertimeRate(), 8.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("2000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("102000.00"),
		                                     new BigDecimal("2000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("124.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("6324.00")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("124.00"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("6324.00")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("42.34"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("1492.34")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("42.34"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("1492.34")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("536.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("34639.00")),
		                new PaycheckLineItem(sFUTA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("170.96"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("9984.39")),
		                new PaycheckLineItem(sCASDI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("693.58")),
		                new PaycheckLineItem(sCASUI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker, withholding of Single 1 containing Ytd values, $86,698 limit
	 * for CASDI just reached in current paycheck.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithholdSingle1WithYtd_CASDI_LimitJustReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.BIWEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.BIWEEKLY,
    	                                                                          0.0, 0.0, "86000");
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, 80.0, mOvertimeRate, 8.0,
		                                                     "1000.00");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("1920.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("1403.58"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("800.00"),
		                                     new BigDecimal(mHourlyRate), 80.0),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
		                                     getCalculatedOvertimeRate(), 8.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("87000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("5451.04")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("5451.04")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("1274.84")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("1274.84")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("286.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("29489.00")),
		                new PaycheckLineItem(sFUTA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("77.96"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("8449.39")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("5.58"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("693.58")),
		                new PaycheckLineItem(sCASUI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a semimonthly paycheck for a worker, withholding of Single 1 containing Ytd values, $7000 limit
	 * for FUTA, CASUI and CAETT already reached.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testSemimonthlyWithholdSingle1WithYtd_FUTA_CASUI_CAETT_LimitAlreadyReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.SEMIMONTHLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.SEMIMONTHLY,
    	                                                                          0.0, 0.0, "7000");
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, 80.0, mOvertimeRate, 8.0,
		                                                     "1000.00");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("1920.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("1416.97"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("800.00"),
		                                     new BigDecimal(mHourlyRate), 80.0),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
		                                     getCalculatedOvertimeRate(), 8.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("8000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("553.04")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("553.04")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("129.34")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("129.34")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("270.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("1916.00")),
		                new PaycheckLineItem(sFUTA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("70.79"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("612.81")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("15.36"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("71.36")),
		                new PaycheckLineItem(sCASUI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a semimonthly paycheck for a worker, withholding of Single 1 containing Ytd values, $7000 limit
	 * for FUTA, CASUI and CAETT just reached in current paycheck.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testSemimonthlyWithholdSingle1WithYtd_FUTA_CASUI_CAETT_LimitJustReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.SEMIMONTHLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.SEMIMONTHLY,
    	                                                                          0.0, 0.0, "6500");
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, 80.0, mOvertimeRate, 8.0,
		                                                     "1000.00");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("1920.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("1416.97"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("800.00"),
		                                     new BigDecimal(mHourlyRate), 80.0),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
		                                     getCalculatedOvertimeRate(), 8.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("7500.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("522.04")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("522.04")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("122.09")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("122.09")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("270.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("1776.00")),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("4.00"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("70.79"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("566.31")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("15.36"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("67.36")),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("17.00"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("0.50"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a semimonthly paycheck for a worker, withholding of Single 1 containing Ytd values, $102,000 limit
	 * for Social Security just reached in current paycheck.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testSemimonthlyWithholdSingle1WithYtd_SocialSecurity_LimitJustReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.SEMIMONTHLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.SEMIMONTHLY,
    	                                                                          0.0, 0.0, "100000");
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, 80.0, mOvertimeRate, 8.0,
		                                                     "2000.00");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("2920.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("2071.08"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("800.00"),
		                                     new BigDecimal(mHourlyRate), 80.0),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
		                                     getCalculatedOvertimeRate(), 8.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("2000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("102000.00"),
		                                     new BigDecimal("2000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("124.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("6324.00")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("124.00"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("6324.00")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("42.34"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("1492.34")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("42.34"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("1492.34")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("520.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("34549.00")),
		                new PaycheckLineItem(sFUTA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("162.58"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("9935.46")),
		                new PaycheckLineItem(sCASDI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("693.58")),
		                new PaycheckLineItem(sCASUI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a semimonthly paycheck for a worker, withholding of Single 1 containing Ytd values, $86,698 limit
	 * for CASDI just reached in current paycheck.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testSemimonthlyWithholdSingle1WithYtd_CASDI_LimitJustReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.SEMIMONTHLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.SEMIMONTHLY,
    	                                                                          0.0, 0.0, "86000");
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, 80.0, mOvertimeRate, 8.0,
		                                                     "1000.00");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("1920.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("1426.75"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("800.00"),
		                                     new BigDecimal(mHourlyRate), 80.0),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
		                                     getCalculatedOvertimeRate(), 8.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("87000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("5451.04")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("5451.04")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("1274.84")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("1274.84")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("270.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("29399.00")),
		                new PaycheckLineItem(sFUTA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("70.79"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("8401.67")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("5.58"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("693.58")),
		                new PaycheckLineItem(sCASUI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a monthly paycheck for a worker, withholding of Single 1 containing Ytd values, $7000 limit
	 * for FUTA, CASUI and CAETT already reached.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testMonthlyWithholdSingle1WithYtd_FUTA_CASUI_CAETT_LimitAlreadyReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.MONTHLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.MONTHLY,
    	                                                                          0.0, 0.0, "7000");
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, 160.0, mOvertimeRate, 8.0,
		                                                     "1000.00");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("2720.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("2128.61"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("1600.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("1600.00"),
		                                     new BigDecimal(mHourlyRate), 160.0),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
		                                     getCalculatedOvertimeRate(), 8.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("8000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("168.64"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("602.64")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("168.64"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("602.64")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("39.44"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("140.94")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("39.44"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("140.94")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("299.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("1630.00")),
		                new PaycheckLineItem(sFUTA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("62.55"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("495.58")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("21.76"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("77.76")),
		                new PaycheckLineItem(sCASUI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a monthly paycheck for a worker, withholding of Single 1 containing Ytd values, $7000 limit
	 * for FUTA, CASUI and CAETT just reached in current paycheck.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testMonthlyWithholdSingle1WithYtd_FUTA_CASUI_CAETT_LimitJustReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.MONTHLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.MONTHLY,
    	                                                                          0.0, 0.0, "6500");
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, 160.0, mOvertimeRate, 8.0,
		                                                     "1000.00");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("2720.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("2128.61"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("1600.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("1600.00"),
		                                     new BigDecimal(mHourlyRate), 160.0),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
		                                     getCalculatedOvertimeRate(), 8.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("7500.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("168.64"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("571.64")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("168.64"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("571.64")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("39.44"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("133.69")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("39.44"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("133.69")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("299.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("1503.00")),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("4.00"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("62.55"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("449.08")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("21.76"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("73.76")),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("17.00"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("0.50"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a monthly paycheck for a worker, withholding of Single 1 containing Ytd values, $102,000 limit
	 * for Social Security just reached in current paycheck.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testMonthlyWithholdSingle1WithYtd_SocialSecurity_LimitJustReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.MONTHLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.MONTHLY,
    	                                                                          0.0, 0.0, "100000");
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, 160.0, mOvertimeRate, 8.0,
		                                                     "2000.00");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("3720.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("2901.07"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("1600.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("1600.00"),
		                                     new BigDecimal(mHourlyRate), 160.0),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
		                                     getCalculatedOvertimeRate(), 8.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("2000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("102000.00"),
		                                     new BigDecimal("2000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("124.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("6324.00")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("124.00"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("6324.00")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("53.94"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("1503.94")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("53.94"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("1503.94")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("509.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("33566.00")),
		                new PaycheckLineItem(sFUTA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("131.99"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("9377.76")),
		                new PaycheckLineItem(sCASDI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("693.58")),
		                new PaycheckLineItem(sCASUI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a monthly paycheck for a worker, withholding of Single 1 containing Ytd values, $86,698 limit
	 * for CASDI just reached in current paycheck.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testMonthlyWithholdSingle1WithYtd_CASDI_LimitJustReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.MONTHLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.MONTHLY,
    	                                                                          0.0, 0.0, "86000");
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = commitAPayrollRun(payrollRunSaveRequest);
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, 160.0, mOvertimeRate, 8.0,
		                                                     "1000.00");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("2720.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("2144.79"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("1600.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("1600.00"),
		                                     new BigDecimal(mHourlyRate), 160.0),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
		                                     getCalculatedOvertimeRate(), 8.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("87000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("168.64"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("5500.64")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("168.64"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("5500.64")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("39.44"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("1286.44")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("39.44"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("1286.44")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("299.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("28456.00")),
		                new PaycheckLineItem(sFUTA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("62.55"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("7866.32")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("5.58"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("693.58")),
		                new PaycheckLineItem(sCASUI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

}
